@@ -1,5 +1,5 @@
 #! /bin/sh
-# $Id: install.sh 561966 2018-04-16 13:07:35Z ivanov $
+# $Id: install.sh 577506 2019-01-04 18:39:23Z ivanov $
 # Authors:  Denis Vakatov    (vakatov@ncbi.nlm.nih.gov)
 #           Anton Lavrentiev (lavr@ncbi.nlm.nih.gov)
 #
@@ -34,7 +34,6 @@ echo "[`basename $script`] NCBI C++:  \"$builddir\" to \"$target\"..."
 
 
 # Derive the destination dirs
-docdir="$target"/doc
 scriptdir="$target"/scripts
 incdir="$target"/include
 srcdir="$target"/src
@@ -67,10 +66,6 @@ makedir "$target" -p
 makedir "$tmpdir" -p
 
 
-# Documentation
-echo "[`basename $script`] Installing documentation..."
-install "$builddir/doc" "$docdir"
-
 # Scripts
 echo "[`basename $script`] Installing scripts..."
 install "$builddir/scripts" "$scriptdir"
