@@ -1,5 +1,5 @@
 #!/bin/sh
-# $Id: ptb.sh 507307 2016-07-18 16:33:11Z gouriano $
+# $Id: ptb.sh 574680 2018-11-16 14:33:27Z ivanov $
 # ===========================================================================
 # 
 #                            PUBLIC DOMAIN NOTICE
@@ -166,7 +166,7 @@ test -z "$PTB_PROJECT" && PTB_PROJECT=${PTB_PROJECT_REQ}
 if test ! -x "$PTB_EXE"; then
   echo "=============================================================================="
   echo Building project tree builder locally, please wait
-  cmd="`dirname $0`/xcodebuild.sh -project $PTB_SLN -target $ptbname -configuration ReleaseDLL"
+  cmd="`dirname $0`/xcodebuild.sh -project $PTB_SLN -target $ptbname -configuration ReleaseDLL -arch $PTB_PLATFORM"
   echo "$cmd"
   echo "=============================================================================="
   if test "${NCBICONF_MSVC}" != ""; then
@@ -203,7 +203,10 @@ echo "==========================================================================
 if test "$USE_GUI_CFG" = "yes"; then
   eval java -jar $PTBGUI $PTB_EXE -i $PTB_FLAGS $PTB_EXTRA $PTB_SAVED_CFG -logfile ${SLN_PATH}_configuration_log.txt -conffile $PTB_INI $TREE_ROOT $PTB_PROJECT $SLN_PATH
 else
-  eval $PTB_EXE $PTB_FLAGS $PTB_EXTRA $PTB_SAVED_CFG -logfile ${SLN_PATH}_configuration_log.txt -conffile $PTB_INI $TREE_ROOT $PTB_PROJECT $SLN_PATH
+  eval CHECK_TIMEOUT=900 $TREE_ROOT/scripts/common/check/check_exec.sh \
+    $PTB_EXE $PTB_FLAGS $PTB_EXTRA $PTB_SAVED_CFG \
+    -logfile ${SLN_PATH}_configuration_log.txt -conffile $PTB_INI $TREE_ROOT \
+    $PTB_PROJECT $SLN_PATH
 fi
 if test "$?" -ne 0; then
   PTB_RESULT=1
