@@ -1,4 +1,4 @@
-/*  $Id: blast_args.hpp 565102 2018-06-06 17:25:02Z rackerst $
+/*  $Id: blast_args.hpp 579216 2019-01-31 16:18:17Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -942,6 +942,10 @@ public:
         return m_Subjects;
     }
 
+    void SetIPGFilteringSupport(bool val) {
+        m_SupportIPGFiltering = val;
+    }
+
 protected:
     CRef<CSearchDatabase> m_SearchDb;/**< Description of the BLAST database */
     bool m_RequestMoleculeType;     /**< Determines whether the database's
@@ -958,6 +962,7 @@ protected:
     CRef<objects::CScope> m_Scope;  /**< CScope object in which all subject
                                       sequences read are kept */
     bool m_SupportsDatabaseMasking; /**< true if it's supported */
+    bool m_SupportIPGFiltering;     /**< true if IPG filtering is supported */
 };
 
 /// Argument class to collect options specific to igBLAST
@@ -1066,14 +1071,16 @@ public:
         m_Html(false),
         m_IsIgBlast(isIgblast),
         m_LineLength(align_format::kDfltLineLength),
-        m_FormatFlags(flag)
+        m_FormatFlags(flag),
+        m_HitsSortOption(-1),
+        m_HspsSortOption(-1)
     {
         if (m_IsIgBlast) {
             m_DfltNumAlignments = m_DfltNumDescriptions = 10;
         } else {
             m_DfltNumAlignments = align_format::kDfltArgNumAlignments;
             m_DfltNumDescriptions = align_format::kDfltArgNumDescriptions;
-        }
+        }        
     };
 
     /** Interface method, \sa IBlastCmdLineArgs::SetArgumentDescriptions */
@@ -1093,7 +1100,8 @@ public:
     virtual void
     ParseFormattingString(const CArgs& args,
                           EOutputFormat& fmt_type,
-                          string& custom_fmt_spec) const;
+                          string& custom_fmt_spec,
+                          string& custom_delim) const;
 
     /// Get the choice of formatted output
     EOutputFormat GetFormattedOutputChoice() const {
@@ -1143,6 +1151,14 @@ public:
     size_t GetLineLength() const {
     	return m_LineLength;
     }
+    int GetHitsSortOption() const {
+        return m_HitsSortOption;
+    }
+    int GetHspsSortOption() const {
+        return m_HspsSortOption;
+    }
+    string GetCustomDelimiter(){return m_CustomDelim;}
+    
 protected:
     EOutputFormat m_OutputFormat;   ///< Choice of formatting output
     bool m_ShowGis;                 ///< Display NCBI GIs?
@@ -1157,6 +1173,9 @@ protected:
     string m_CustomOutputFormatSpec;
     size_t m_LineLength;
     EFormatFlags m_FormatFlags;
+    int m_HitsSortOption;
+    int m_HspsSortOption;
+    string m_CustomDelim;
 };
 
 /// Formatting args for magicblast advertising only SAM and fast tabular
@@ -1242,7 +1261,7 @@ public:
 
     /// Get the number of threads to spawn
     size_t GetNumThreads() const { return m_NumThreads; }
-private:
+protected:
     size_t m_NumThreads;        ///< Number of threads to spawn
 };
 
