@@ -1,4 +1,4 @@
-/*  $Id: cmdline_flags.hpp 562577 2018-04-24 15:51:23Z fongah2 $
+/*  $Id: cmdline_flags.hpp 579216 2019-01-31 16:18:17Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -77,6 +77,11 @@ NCBI_BLASTINPUT_EXPORT extern const string kArgNegativeGiList;
 /// argument for gi list to exclude from a BLAST database search
 NCBI_BLASTINPUT_EXPORT extern const string kArgNegativeSeqidList;
 
+/// IPG list file name to restrict BLAST database
+NCBI_BLASTINPUT_EXPORT extern const string kArgIpgList;
+
+/// argument for IPG list to exclude from a BLAST database search
+NCBI_BLASTINPUT_EXPORT extern const string kArgNegativeIpgList;
 
 /// List of filtering algorithms to apply to subjects as soft masking
 extern const string kArgDbSoftMask;
