@@ -1,4 +1,4 @@
-/* $Id: blast_hits.h 532442 2017-04-05 13:47:44Z boratyng $
+/* $Id: blast_hits.h 573792 2018-11-01 15:47:14Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -631,6 +631,13 @@ Blast_HSPListPurgeHSPsWithCommonEndpoints(EBlastProgramType program,
                                           BlastHSPList* hsp_list,
                                           Boolean purge);
 
+NCBI_XBLAST_EXPORT
+Int4
+Blast_HSPListSubjectBestHit(EBlastProgramType program,
+		                    const BlastHSPSubjectBestHitOptions* subject_besthit_opts,
+		                    const BlastQueryInfo *query_info,
+                            BlastHSPList* hsp_list);
+
 /** Reevaluate all ungapped HSPs in an HSP list.  
  * This is only done for an ungapped search, or if traceback is 
  * already available.
@@ -931,6 +938,7 @@ PhiBlastGetEffectiveNumberOfPatterns(const BlastQueryInfo *query_info);
 
 /** Apply Cross_match like masklevel to HSP list.  -RMH-
  */
+NCBI_XBLAST_EXPORT
 Int2 Blast_HSPResultsApplyMasklevel(BlastHSPResults *results,
                                     const BlastQueryInfo *query_info,
                                     Int4 masklevel, Int4 query_length);
