@@ -1,4 +1,4 @@
-/* $Id: blast_options.h 562577 2018-04-24 15:51:23Z fongah2 $
+/* $Id: blast_options.h 577682 2019-01-08 12:49:31Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -167,7 +167,13 @@ extern "C" {
 
 /** Default max frequency for a database word. Words with higher frequency
     will be masked in the lookup table. */
-#define MAX_DB_WORD_COUNT_MAPPER 60
+#define MAX_DB_WORD_COUNT_MAPPER 30
+
+/** Default maximum insert size: distance on the subject between reads that
+    belong to a pair, for spliced and non-spliced alignments */
+#define MAGICBLAST_MAX_INSERT_SIZE_SPLICED 1000000
+#define MAGICBLAST_MAX_INSERT_SIZE_NONSPLICED 100000
+
 
 /** Value used to indicate that no IMPALA-style scaling should be performed
  * when scaling a PSSM */
