@@ -1,4 +1,4 @@
-/*  $Id: win_mask_gen_counts.hpp 244878 2011-02-10 17:03:08Z mozese2 $
+/*  $Id: win_mask_gen_counts.hpp 434184 2014-05-05 14:32:02Z morgulis $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -112,6 +112,7 @@ public:
      **\param exclude_ids set of ids to ignore
      **\param use_ba use bit array optimization for optimized binary
      **              unit counts format
+     **\param metadata the metadata string
      **
      **/
     CWinMaskCountsGenerator( const string & input,
@@ -128,7 +129,8 @@ public:
                              bool use_list,
                              const CWinMaskUtil::CIdSet * ids,
                              const CWinMaskUtil::CIdSet * exclude_ids,
-                             bool use_ba );
+                             bool use_ba,
+                             string const & metadata );
 
     /**
      **\brief Constructor.
@@ -158,6 +160,7 @@ public:
      **\param exclude_ids set of ids to ignore
      **\param use_ba use bit array optimization for optimized binary
      **              unit counts format
+     **\param metadata the metadata string
      **
      **/
     CWinMaskCountsGenerator( const string & input,
@@ -174,7 +177,8 @@ public:
                              bool use_list,
                              const CWinMaskUtil::CIdSet * ids,
                              const CWinMaskUtil::CIdSet * exclude_ids,
-                             bool use_ba );
+                             bool use_ba,
+                             string const & metadata );
 
     /**
      **\brief Object destructor.
@@ -217,7 +221,7 @@ private:
 
     string input;                   /**<\internal input file (or list of input files) */
     CRef< CSeqMaskerOstat > ustat;  /**<\internal object used to output the unit counts statistics */
-    Uint4 max_mem;                  /**<\internal available memory in bytes */
+    Uint8 max_mem;                  /**<\internal available memory in bytes */
     Uint4 unit_size;                /**<\internal n-mer length in base pairs */
     Uint8 genome_size;              /**<\internal genome size in bases */
     Uint4 min_count;                /**<\internal minimal n-mer count to consider */
