@@ -1,7 +1,7 @@
 #ifndef CGI___CGI_SERIAL__HPP
 #define CGI___CGI_SERIAL__HPP
 
-/*  $Id: cgi_serial.hpp 484889 2015-11-16 21:56:43Z lavr $
+/*  $Id: cgi_serial.hpp 535133 2017-05-05 14:01:02Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -96,7 +96,7 @@ public:
 class COStreamHelper
 {
 public:
-    COStreamHelper(CNcbiOstream& os) : m_Ostream(os), m_str(NULL) {}
+    COStreamHelper(CNcbiOstream& os) : m_Ostream(os), m_str(nullptr) {}
     ~COStreamHelper() {  try { flush(); } catch (...) {}   }
 
     operator CNcbiOstream&() { return x_GetStrm(); }
@@ -111,7 +111,7 @@ public:
     void flush(bool write_empty_data = false) 
     {
         if (m_str.get() != NULL) {
-            auto_ptr<CNcbiOstrstream> strm(m_str.release());
+            unique_ptr<CNcbiOstrstream> strm(m_str.release());
             string s = CNcbiOstrstreamToString(*strm);
             // Historically counted, but did not output, a final \0.
             m_Ostream << (s.size() + 1) << ' ' << s;
@@ -127,8 +127,7 @@ private:
         return *m_str;
     }
     CNcbiOstream& m_Ostream;
-    auto_ptr<CNcbiOstrstream> m_str;
-    
+    unique_ptr<CNcbiOstrstream> m_str;
 }; 
 
 
