@@ -1,7 +1,7 @@
 #ifndef CGI___NCBICGI__HPP
 #define CGI___NCBICGI__HPP
 
-/*  $Id: ncbicgi.hpp 512597 2016-09-01 15:49:22Z sadyrovr $
+/*  $Id: ncbicgi.hpp 561634 2018-04-10 16:07:35Z sadyrovr $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -649,6 +649,23 @@ public:
 };
 
 
+class NCBI_XCGI_EXPORT CExtraEntryCollector : public CEntryCollector_Base {
+public:
+    CExtraEntryCollector(void) {}
+    ~CExtraEntryCollector(void) override {}
+    
+    void AddEntry(const string& name,
+                  const string& value,
+                  const string& filename,
+                  bool          is_index) override;
+
+    CDiagContext_Extra::TExtraArgs& GetArgs(void) { return m_Args; }
+
+private:
+    CDiagContext_Extra::TExtraArgs m_Args;
+};
+
+
 ///////////////////////////////////////////////////////
 ///
 ///  CCgiRequest::
@@ -706,7 +723,11 @@ public:
         fSemicolonIsNotArgDelimiter = (1 << 11),
         /// Do not set outgoing tracking cookie. This can also be
         /// done per-request using CCgiResponce::DisableTrackingCookie().
-        fDisableTrackingCookie      = (1 << 12)
+        fDisableTrackingCookie      = (1 << 12),
+        /// When parsing input on demand iterate all existing entries (e.g. those
+        /// read from QUERY_STRING) before parsing POST data.
+        /// @sa fParseInputOnDemand
+        fIncludePreparsedEntries = (1 << 13)
     };
     CCgiRequest(const         CNcbiArguments*   args = 0,
                 const         CNcbiEnvironment* env  = 0,
