@@ -1,7 +1,7 @@
 #ifndef COMMON___NCBI_EXPORT__H
 #define COMMON___NCBI_EXPORT__H
 
-/*  $Id: ncbi_export.h 498832 2016-04-19 21:18:13Z vasilche $
+/*  $Id: ncbi_export.h 534723 2017-05-01 18:22:26Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1463,6 +1463,24 @@
 #endif
 
 
+/* Export specifier for library cdd_access
+ */
+#ifdef NCBI_CDD_ACCESS_EXPORTS
+#  define NCBI_CDD_ACCESS_EXPORT NCBI_DLL_EXPORT
+#else
+#  define NCBI_CDD_ACCESS_EXPORT NCBI_DLL_IMPORT
+#endif
+
+
+/* Export specifier for library ncbi_id2proc_cdd
+ */
+#ifdef NCBI_ID2PROC_CDD_EXPORTS
+#  define NCBI_ID2PROC_CDD_EXPORT NCBI_DLL_EXPORT
+#else
+#  define NCBI_ID2PROC_CDD_EXPORT NCBI_DLL_IMPORT
+#endif
+
+
 /* STATIC LIBRARIES SECTION */
 /* This section is for static-only libraries */
 
