@@ -1,7 +1,7 @@
 #ifndef COMMON___NCBICONF_IMPL__H
 #define COMMON___NCBICONF_IMPL__H
 
-/* $Id: ncbiconf_impl.h 548542 2017-10-16 15:18:59Z ivanov $
+/* $Id: ncbiconf_impl.h 561298 2018-04-05 16:32:11Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -41,6 +41,8 @@
 #  error "The header can be used from <ncbiconf.h> only."
 #endif /*!FORWARDING_NCBICONF_H*/
 
+#include <common/ncbi_build_info.h>
+
 
 /** @addtogroup Portability
  *
@@ -127,9 +129,9 @@
 #  define NCBI_WARN_UNUSED_RESULT
 #endif
 
-#if defined(__SSE_4_2__)  ||  defined(__AVX__)
+#if defined(__SSE4_2__)  ||  defined(__AVX__)
 #  define NCBI_SSE 42
-#elif defined(__SSE_4_1__)
+#elif defined(__SSE4_1__)
 #  define NCBI_SSE 41
 #elif defined(__SSSE3__)
 #  define NCBI_SSE 40
