@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONN_EXCEPTION__HPP
 #define CONNECT___NCBI_CONN_EXCEPTION__HPP
 
-/* $Id: ncbi_conn_exception.hpp 487716 2015-12-21 16:13:26Z ivanov $
+/* $Id: ncbi_conn_exception.hpp 563001 2018-05-01 01:18:53Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -70,11 +70,11 @@ public:
     /// @sa EIO_Status
     enum EErrCode {
         eTimeout      = eIO_Timeout,
-        eClosed       = eIO_Closed,
         eInterrupt    = eIO_Interrupt,
         eInvalidArg   = eIO_InvalidArg,
         eNotSupported = eIO_NotSupported,
-        eUnknown      = eIO_Unknown
+        eUnknown      = eIO_Unknown,
+        eClosed       = eIO_Closed
     };
     virtual const char* GetErrCodeString(void) const;
     NCBI_EXCEPTION_DEFAULT(CIO_Exception, CConnException);
