@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONN_STREAM__HPP
 #define CONNECT___NCBI_CONN_STREAM__HPP
 
-/* $Id: ncbi_conn_stream.hpp 547023 2017-09-25 17:18:32Z ivanov $
+/* $Id: ncbi_conn_stream.hpp 566392 2018-06-28 17:33:58Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -91,24 +91,24 @@ class CConn_Streambuf;  // Forward declaration
 class CSocket;          // Forward declaration
 
 
-const size_t kConn_DefaultBufSize = 4096;
+const size_t kConn_DefaultBufSize = 16384;  ///< I/O buffer size per direction
 
 
 
 /////////////////////////////////////////////////////////////////////////////
 ///
-/// Base class, inherited from "std::iostream", does both input
-/// and output, using the specified CONNECTOR.
-/// "Buf_size" designates the size of internal I/O buffers, which reside in
-/// between the stream and an underlying connector (which in turn may do
-/// further buffering, if needed).
-/// Input operations can be tied to the output ones by setting "tie" to "true"
-/// (default), which means that any input attempt first flushes any pending
-/// output from the internal buffers.
+/// Base class, inherited from "std::iostream", does both input and output,
+/// using the specified CONNECTOR.
+/// The "buf_size" parameter designates the size of internal I/O buffers, which
+/// reside in between the stream and an underlying connector (which in turn may
+/// do further buffering, if needed).
+/// By default, all input operations are tied to the output ones, which means
+/// that any input attempt first flushes any pending output from the internal
+/// buffers.  The fConn_Untie flag can be used to untie the I/O directions.
 ///
-/// @note CConn_IOStream implementation utilizes some connection callbacks
-///       on the underlying CONN object.  Care must be taken when intercepting
-///       the callbacks using the native CONN API.
+/// @note CConn_IOStream implementation utilizes some connection callbacks on
+///       the underlying CONN object.  Care must be taken when intercepting the
+///       callbacks using the native CONN API.
 /// @sa
 ///   CONN_SetCallback
 
@@ -116,29 +116,35 @@ class NCBI_XCONNECT_EXPORT CConn_IOStream : public            CNcbiIostream,
                                             virtual protected CConnIniter
 {
 public:
-    /// Must be compatible by values with TCONN_Flags.
+    /// The values must be compatible with TCONN_Flags.
     enum {
-        fConn_Untie         = 1,  ///< do not flush before reading
-        fConn_ReadBuffered  = 2,  ///< read buffer is to be allocated
-        fConn_WriteBuffered = 4   ///< write buffer is to be allocated
+        fConn_Untie           = fCONN_Untie,///< do not flush before reading
+        fConn_DelayOpen       = 2,          ///< do not force CONN open in ctor
+        fConn_ReadUnbuffered  = 4,          ///< read buffer NOT to be alloc'd
+        fConn_WriteUnbuffered = 8           ///< write buffer NOT 2.b. alloc'd
     } EConn_Flag;
-    typedef unsigned int TConn_Flags;  ///< bitwise OR of EConn_Flag
+    typedef unsigned int TConn_Flags;      ///< bitwise OR of EConn_Flag
 
-    /// Create a stream based on a CONN, which is to be closed upon
-    /// stream dtor only if "close" parameter is passed as "true".
+    /// Create a stream based on a CONN, which is to be closed upon stream
+    /// dtor only if "close" parameter is passed as "true".
     ///
     /// @param conn
-    ///  A C object of type CONN (ncbi_connection.h) on top of which
-    ///  the stream is being constructed.  May not be NULL.
+    ///  A C object of type CONN (ncbi_connection.h) on top of which the stream
+    ///  is being constructed.  May not be NULL.
     /// @param close
     ///  True if to close CONN automatically (otherwise CONN remains open)
     /// @param timeout
     ///  Default I/O timeout
     /// @param buf_size
-    ///  Default size of underlying stream buffer's I/O arena
+    ///  Default size of underlying stream buffer's I/O arena (per direction)
     /// @param flags
     ///  Specifies whether to tie input and output (a tied stream flushes all
-    ///  pending output prior to doing any input) and what to buffer.
+    ///  pending output prior to doing any input) and what to buffer
+    /// @param ptr
+    ///  Specifies data which will be read from the stream prior to extracting
+    ///  from the actual connection
+    /// @param size
+    ///  The size of the area pointed to by the "ptr" argument
     /// @sa
     ///  CONN, ncbi_connection.h
     ///
@@ -147,7 +153,7 @@ public:
      bool            close    = false,
      const STimeout* timeout  = kDefaultTimeout,
      size_t          buf_size = kConn_DefaultBufSize,
-     TConn_Flags     flags    = fConn_ReadBuffered | fConn_WriteBuffered,
+     TConn_Flags     flags    = 0,
      CT_CHAR_TYPE*   ptr      = 0,
      size_t          size     = 0);
 
@@ -157,24 +163,29 @@ protected:
     /// only for internal use in derived classes.
     ///
     /// @param TConn_Pair::connector
-    ///  A C object of type CONNECTOR (ncbi_connector.h) on top of which
-    ///  the stream is being constructed
+    ///  A C object of type CONNECTOR (ncbi_connector.h) on top of which the
+    ///  stream is being constructed.  CONNECTOR may not be NULL.
     /// @param TConn_Pair::status
     ///  I/O status to use in underlying streambuf when CONNECTOR is NULL
     /// @param timeout
     ///  Default I/O timeout
     /// @param buf_size
-    ///  Default size of underlying stream buffer's I/O arena
+    ///  Default size of underlying stream buffer's I/O arena (per direction)
     /// @param flags
     ///  Specifies whether to tie input and output (a tied stream flushes all
-    ///  pending output prior to doing any input) and what to buffer.
+    ///  pending output prior to doing any input) and what to buffer
+    /// @param ptr
+    ///  Specifies data, which will be read from the stream prior to extracting
+    ///  from the actual connection
+    /// @param size
+    ///  The size of the area pointed to by the "ptr" argument
     /// @sa
     ///  CONNECTOR, ncbi_connector.h
     CConn_IOStream
     (const TConn_Pair& connpair,
      const STimeout*   timeout  = kDefaultTimeout,
      size_t            buf_size = kConn_DefaultBufSize,
-     TConn_Flags       flags    = fConn_ReadBuffered | fConn_WriteBuffered,
+     TConn_Flags       flags    = 0,
      CT_CHAR_TYPE*     ptr      = 0,
      size_t            size     = 0);
 
@@ -185,37 +196,51 @@ public:
     ///   Verbal connection type (empty if unknown)
     /// @sa
     ///   CONN_GetType
-    string          GetType(void) const;
+    string             GetType(void) const;
 
     /// @return
     ///   Verbal connection description (empty if unknown)
     /// @sa
     ///   CONN_Description
-    string          GetDescription(void) const;
+    string             GetDescription(void) const;
 
     /// Set connection timeout for "direction"
     /// @param
     ///   Can accept a pointer to a finite timeout, or either of the special
-    ///   values: kInfiniteTimeout, kDefaultTimeout.
+    ///   values: kInfiniteTimeout, kDefaultTimeout
     /// @sa
     ///   CONN_SetTimeout, SetReadTimeout, SetWriteTimeout
-    EIO_Status      SetTimeout(EIO_Event       direction,
-                               const STimeout* timeout) const;
+    EIO_Status         SetTimeout(EIO_Event       direction,
+                                 const STimeout* timeout) const;
 
     /// @return
     ///   Connection timeout for "direction"
     /// @sa
     ///   CONN_GetTimeout
-    const STimeout* GetTimeout(EIO_Event direction) const;
+    const STimeout*    GetTimeout(EIO_Event direction) const;
 
     /// @return
     ///   Status of the last I/O performed by the underlying CONN in the
     ///   specified "direction" (either eIO_Open, IO_Read, or eIO_Write);
-    ///   if "direction" is not specified (eIO_Close), return status
-    ///   of the last CONN I/O performed by the stream.
+    ///   if "direction" is not specified (eIO_Close), return status of the
+    ///   last CONN I/O performed by the stream.
     /// @sa
     ///   CONN_Status
-    EIO_Status      Status(EIO_Event direction = eIO_Close) const;
+    EIO_Status         Status(EIO_Event direction = eIO_Close) const;
+
+    /// Return the specified data "data" of size "size" into the underlying
+    /// connection CONN.
+    /// If there is any non-empty pending input sequence (internal read buffer)
+    /// it will first be attempted to return to CONN.  That excludes any
+    /// initial read area ("ptr") that might have been specified in the ctor.
+    /// Any status different from eIO_Success means that nothing from "data"
+    /// has been pushed back to the connection.
+    /// @note
+    ///   Can be used to push just the pending internal input alone back into
+    ///   the CONN if used with a "size" of 0 ("data" is ignored then).
+    /// @sa
+    ///   CONN_Pushback
+    EIO_Status         Pushback(const CT_CHAR_TYPE* data, streamsize size);
 
     /// Close CONNection, free all internal buffers and underlying structures,
     /// and render the stream unusable for further I/O.
@@ -230,16 +255,16 @@ public:
     /// first superclass.
     /// @sa
     ///   ICanceled
-    EIO_Status      SetCanceledCallback(const ICanceled* canceled);
+    EIO_Status         SetCanceledCallback(const ICanceled* canceled);
 
     /// @return
-    ///   Internal CONNection handle, which is still owned and used by
-    ///   the stream (or NULL if no such handle exists)
+    ///   Internal CONNection handle, which is still owned and used by the
+    ///   stream (or NULL if no such handle exists)
     /// @note
-    ///   Connection can have additional flags set for I/O processing.
+    ///   Connection can have additional flags set for I/O processing
     /// @sa
     ///   CONN, ncbi_connection.h, CONN_GetFlags
-    CONN            GetCONN(void) const;
+    CONN               GetCONN(void) const;
 
 protected:
     void x_Destroy(void);
@@ -295,7 +320,7 @@ inline CConn_IOStreamSetReadTimeout SetReadTimeout(const STimeout* timeout)
 inline CConn_IOStream& operator>> (CConn_IOStream& is,
                                    const CConn_IOStreamSetReadTimeout& s)
 {
-    if (is.good() && is.SetTimeout(eIO_Read, s.GetTimeout()) != eIO_Success)
+    if (is.good()  &&  is.SetTimeout(eIO_Read, s.GetTimeout()) != eIO_Success)
         is.clear(IOS_BASE::badbit);
     return is;
 }
@@ -324,7 +349,7 @@ inline CConn_IOStreamSetWriteTimeout SetWriteTimeout(const STimeout* timeout)
 inline CConn_IOStream& operator<< (CConn_IOStream& os,
                                    const CConn_IOStreamSetWriteTimeout& s)
 {
-    if (os.good() && os.SetTimeout(eIO_Write, s.GetTimeout()) != eIO_Success)
+    if (os.good()  &&  os.SetTimeout(eIO_Write, s.GetTimeout()) != eIO_Success)
         os.clear(IOS_BASE::badbit);
     return os;
 }
@@ -333,9 +358,9 @@ inline CConn_IOStream& operator<< (CConn_IOStream& os,
 
 /////////////////////////////////////////////////////////////////////////////
 ///
-/// This stream exchanges data in a TCP channel, using socket interface.
-/// The endpoint is specified as a "host:port" pair.  The maximal
-/// number of connection attempts is given via "max_try".
+/// This stream exchanges data in a TCP channel, using the SOCK socket API.
+/// The endpoint is specified as a "host:port" pair.  The maximal number of
+/// connection attempts is given via "max_try".
 /// More details on that: <connect/ncbi_socket_connector.h>.
 ///
 /// @sa
@@ -410,6 +435,7 @@ public:
     /// firewall                        -- if true then look at proxy_server
     /// proxy_server                    -- use as "host" if non-empty and FW
     /// debug_printout                  -- how to log socket data by default
+    /// http_push_auth                  -- whether to push credentials at once
     ///
     /// @param net_info
     ///  Connection point and proxy tunnel location
@@ -486,8 +512,8 @@ struct SHTTP_StatusData {
 /// This stream exchanges data with an HTTP server located at the URL:
 /// http[s]://host[:port]/path[?args]
 ///
-/// Note that "path" must include a leading slash, "args" can be empty,
-/// in which case the '?' does not get appended to the path.
+/// Note that "path" must include the leading slash, "args" can be empty, in
+/// which case the '?' does not get appended to the path.
 ///
 /// "User_header" (if not empty) should be a sequence of lines in the form
 /// 'HTTP-tag: Tag value', with each line separated by a CR LF sequence, and
@@ -497,16 +523,15 @@ struct SHTTP_StatusData {
 ///
 /// More elaborate specification(s) of the server can be made via the
 /// SConnNetInfo structure, which otherwise will be created with the use of a
-/// standard registry section to obtain default values from
-/// (details: <connect/ncbi_connutil.h>).  To make sure the actual user header
-/// is empty, remember to delete it from "net_info" with
-/// ConnNetInfo_DeleteUserHeader(net_info).
+/// standard registry section to obtain default values from (details:
+/// <connect/ncbi_connutil.h>).  To make sure the actual user header is empty,
+/// remember to reset it with ConnNetInfo_SetUserHeader(net_info, 0).
 ///
 /// THTTP_Flags and other details: <connect/ncbi_http_connector.h>.
 ///
 /// Provided "timeout" is set at the connection level, and if different from
-/// kDefaultTimeout, it overrides a value supplied by the HTTP connector
-/// (the latter value is kept at SConnNetInfo::timeout).
+/// kDefaultTimeout, it overrides a value supplied by the HTTP connector (the
+/// latter value is kept at SConnNetInfo::timeout).
 ///
 /// @sa
 ///  HTTP_CreateConnector
@@ -570,7 +595,7 @@ public:
     ~CConn_HttpStream();
 
     /// Flush the stream and fetch the response (w/o extracting any user data)
-    EIO_Status    Fetch(const STimeout* timeout = kDefaultTimeout);
+    EIO_Status        Fetch(const STimeout* timeout = kDefaultTimeout);
 
     /// Get the last seen HTTP status code
     int               GetStatusCode(void) const { return m_StatusData.code; }
@@ -644,7 +669,7 @@ public:
     ~CConn_ServiceStream();
 
     /// Flush the stream and fetch the response (w/o extracting any user data)
-    EIO_Status    Fetch(const STimeout* timeout = kDefaultTimeout);
+    EIO_Status        Fetch(const STimeout* timeout = kDefaultTimeout);
 
     /// Get the last seen HTTP status code, if available
     int               GetStatusCode(void) const { return m_CBD.status.code; }
@@ -655,19 +680,19 @@ public:
     /// Get the last seen HTTP status text, if available
     const string&     GetHTTPHeader(void) const { return m_CBD.status.header; }
 
-    /// Get underlying SOCK, if available after Fetch()
-    SOCK          GetSOCK(void);
+    /// Get underlying SOCK, if available (e.g. after Fetch())
+    SOCK              GetSOCK(void);
 
 public:
     /// Helper class
     struct SSERVICE_CBData {
-        SSERVICE_Extra   extra;
-        SHTTP_StatusData status;
+        SSERVICE_Extra       extra;
+        SHTTP_StatusData     status;
     };
 
 protected:
     // Chained callbacks
-    SSERVICE_CBData m_CBD;
+    SSERVICE_CBData          m_CBD;
 
 private:
     // Interceptors
@@ -747,7 +772,7 @@ protected:
 /// being in use as it may cause undefined behavior.
 ///
 /// Provided "timeout" is set at the connection level if different from
-/// kDefaultTimeout (which is infinite for this class be default).
+/// kDefaultTimeout (which is infinite for this class by default).
 ///
 /// @sa
 ///   PIPE_CreateConnector, CPipe
@@ -785,7 +810,7 @@ protected:
 /// CConn_NamedPipeStream for inter-process communication
 ///
 /// Provided "timeout" is set at the connection level if different from
-/// kDefaultTimeout (which is infinite for this class be default).
+/// kDefaultTimeout (which is infinite for this class by default).
 ///
 /// @sa
 ///   NAMEDPIPE_CreateConnector, CNamedPipe
@@ -851,8 +876,7 @@ public:
 
 /// CConn_FtpStream specialization (ctor) for download
 ///
-/// @note
-///   the order of parameters vs generic CConn_FtpStream ctor
+/// @note the order of parameters vs generic CConn_FtpStream ctor
 ///
 class NCBI_XCONNECT_EXPORT CConn_FTPDownloadStream : public CConn_FtpStream
 {
@@ -921,7 +945,8 @@ protected:
 ///
 /// Given the URL, open the data source and make it available for reading.
 /// See <connect/ncbi_connutil.h> for supported schemes.
-/// Writing to the stream is undefined.
+///
+/// @warning Writing to the resultant stream is undefined.
 ///
 extern NCBI_XCONNECT_EXPORT
 CConn_IOStream* NcbiOpenURL(const string& url,
