@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONNECTION__H
 #define CONNECT___NCBI_CONNECTION__H
 
-/* $Id: ncbi_connection.h 471572 2015-06-29 16:05:55Z lavr $
+/* $Id: ncbi_connection.h 566646 2018-07-03 20:18:56Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -74,15 +74,19 @@ typedef unsigned int TCONN_Flags;  /**< bitwise OR of ECONN_Flag */
 
 
 /** Create all data necessary to establish a new connection (merely bind it to
- * the specified connector).
- * Unsuccessful completion sets "*conn" to NULL, and leaves the connector
- * intact (can be used again).
+ * the specified CONNECTOR).
+ * Unsuccessful completion sets "*conn" to NULL, and leaves "connector" intact
+ * (can be used again).
  * @note  Connection is not established right away but at the moment of the
  *        first call to one of "Flush", "Wait", "Write", or "Read" methods.
  * @note  "Connection establishment" at this level of abstraction may differ
- *        from actual link establishment at the underlying connector's level.
+ *        from actual link establishment at the underlying CONNECTOR's level.
  * @note  Initial timeout values are set to kDefaultTimeout, meaning
- *        that connector-specific timeouts are in force for the connection.
+ *        that CONNECTOR-specific timeouts are in force for the connection.
+ * @note  CONN does not buffer any output data, but passes it directly to the
+ *        underlying CONNECTOR.  CONN may buffer input data internally, but
+ *        only in the following circumstances:  CONN_Read() with "peek",
+ *        CONN_ReadLine(), or CONN_Pushback().
  * @sa
  *  CONN_Close
  */
@@ -100,10 +104,10 @@ extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Create
 
 
 /** Reinit using new "connector".
- * If "conn" is already opened, then close the current connection first,
- * even if "connector" is just the same as the current connector.
- * If "connector" is NULL, then close and destroy the incumbent, and leave
- * the connection empty (effective way to destroy connector(s)).
+ * If "conn" is already opened, then close the current connection first, even
+ * if "connector" is just the same as the current CONNECTOR.  If "connector" is
+ * NULL, then close and destroy the incumbent, and leave the connection empty
+ * (effective way to destroy CONNECTOR(s)).
  * @note  Although it closes the previous connection immediately, however it
  *        does not open the new connection right away:  see notes in "Create".
  * @sa
@@ -125,25 +129,24 @@ extern NCBI_XCONNECT_EXPORT const char* CONN_GetType
 
 
 /** Return a human-readable description of the connection as a character
- * '\0'-terminated string.  The string is not guaranteed to have any
- * particular format and is intended solely for something like
- * logging and debugging.  Return NULL if the connection cannot
- * provide any description information (or if it is in a bad state).
- * Application program must call free() to deallocate space occupied
- * by the returned string when the description is no longer needed.
+ * '\0'-terminated string.  The string is not guaranteed to have any particular
+ * format, and is intended solely for something like logging and debugging.
+ * Return NULL if the connection cannot provide any description information (or
+ * if it is in a bad state).  Application program must call free() to
+ * deallocate the space occupied by the returned string when the returned
+ * description is no longer needed.
  */
 extern NCBI_XCONNECT_EXPORT char* CONN_Description
 (CONN conn  /**< [in] connection handle */
  );
 
 
-/** Get read ("event" == eIO_Read) or write ("event" == eIO_Write)
- * position within the connection.
- * Positions are advanced from 0 on, and only concerning I/O that has
- * caused calling to the actual connector's "read" (i.e. pushbacks
- * never considered, and peeks -- not always) and "write" methods.
- * Special case:  eIO_Open as "event" causes to clear both positions
- * with 0, and to return 0.
+/** Get read ("event" == eIO_Read) or write ("event" == eIO_Write) position
+ * within the connection.  Positions are advanced from 0 on, every successive
+ * byte of data, and are only concerning I/O that has caused calling to the
+ * actual CONNECTOR's "read" (i.e. pushbacks never considered, and peeks -- not
+ * always) and "write" methods.  Special case:  eIO_Open as "event" causes to
+ * clear *both* positions with 0, and to return 0.
  */
 extern NCBI_XCONNECT_EXPORT TNCBI_BigCount CONN_GetPosition
 (CONN      conn,  /**< [in] connection handle */ 
@@ -155,7 +158,7 @@ extern NCBI_XCONNECT_EXPORT TNCBI_BigCount CONN_GetPosition
  * and "Close".  May be called at any time during the connection lifetime.
  * @note  If "timeout" is NULL then set the timeout to be infinite.
  * @note  If "timeout" is kDefaultTimeout then an underlying,
- *         connector-specific value is used (this is the default).
+ *        CONNECTOR-specific value is used (this is the default).
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status CONN_SetTimeout
 (CONN            conn,    /**< [in] connection handle */
@@ -179,7 +182,7 @@ extern NCBI_XCONNECT_EXPORT const STimeout* CONN_GetTimeout
  * writing (depending on "event"), until timeout expires, or until any error.
  * @note  "timeout" can also be one of the two special values:
  *        * NULL (for infinite timeout, also known as kInfiniteTimeout);
- *        * kDefaultTimeout (connector-defined).
+ *        * kDefaultTimeout (CONNECTOR-defined).
  * @sa
  *  CONN_Read, CONN_Write
  */
@@ -191,10 +194,9 @@ extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Wait
 
 
 /** Write up to "size" bytes from the buffer "buf" to the connection.
- * Return the number of actually written bytes in "*n_written".
- * May not return eIO_Success if no data at all can be written before
- * the write timeout expires or an error occurs.
- * Parameter "how" modifies the write behavior:
+ * Return the number of actually written bytes in "*n_written".  May not return
+ * eIO_Success if no data at all can be written before the write timeout
+ * expires or an error occurs.  Parameter "how" modifies the write behavior:
  * * @var eIO_WritePlain
  *                        return immediately after having written as little
  *                        as 1 byte of data (return eIO_Success), or if an
@@ -221,26 +223,29 @@ extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Write
  );
 
 
-/** Push "size" bytes from the buffer "buf" back into connection.
- * Return eIO_Success on success, other code on error.
+/** Push "size" bytes from the buffer "data" back into connection.
+ * Return eIO_Success on success (including when pushing back nothing if
+ * "size" is zero -- the "data" pointer is ignored then), other code on error.
  * @note  The data pushed back may not necessarily be the same as previously
  *        obtained from the connection.
  * @note  Upon a following read operation, the pushed back data are taken out
  *        first.
- * @note  Pushback data can desynchronize underlying connector (so use wisely).
+ * @note  As the pushback data do not actually go back into the underlying
+ *        CONNECTOR (but stored internally into a pending input buffer of
+ *        CONN), that can desynchronize the CONNECTOR (so use it wisely).
  * @sa
  *  CONN_Read, CONN_Write
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Pushback
 (CONN        conn,  /**< [in] connection handle                     */
- const void* buf,   /**< [in] pointer to the data being pushed back */
+ const void* data,  /**< [in] pointer to the data being pushed back */
  size_t      size   /**< [in] # of bytes to push back               */
  );
 
 
 /** Explicitly flush connection from any pending data written by CONN_Write().
  * @note  CONN_Flush() effectively opens connection (if it wasn't open yet).
- * @note  Connection considered open if underlying connector's "Open" method
+ * @note  Connection considered open if underlying CONNECTOR's "Open" method
  *        has successfully executed;  an actual data link may not yet exist.
  * @note  CONN_Read() always calls CONN_Flush() before proceeding (unless the
  *        connection was created with fCONN_Untie);  so does CONN_Close() but
@@ -254,17 +259,18 @@ extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Flush
 
 
 /** Read up to "size" bytes from connection to the buffer pointed to by "buf".
- * Return the number of actually read bytes in "*n_read".
- * May not return eIO_Success if no data at all can be read before
- * the read timeout expires or an error occurs.
- *
- * Parameter "how" modifies the read behavior:
+ * Return the number of actually read bytes in "*n_read".  May not return
+ * eIO_Success if no data at all can be read before the read timeout expires or
+ * an error occurs.  Parameter "how" modifies the read behavior:
  * * @var eIO_ReadPlain
  *                         return immediately after having read as many as
  *                         1 byte from connection (return eIO_Success), or
  *                         if an error has occurred (and "*n_read == 0");
  * * @var eIO_ReadPeek
- *                         eIO_ReadPlain but don't discard data read from CONN;
+ *                         eIO_ReadPlain but don't discard data read from CONN,
+ *                         the peek'ed data get extracted from CONNECTOR but
+ *                         stored internally in the CONN buffer, a non-peek
+ *                         read takes data (if any) from that buffer first;
  * * @var eIO_ReadPersist
  *                         return only after having filled full "buf" with data
  *                         (exactly "size" bytes, eIO_Success), or if an error
@@ -287,18 +293,22 @@ extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Read
  );
 
 
-/** Read up to "size" bytes from connection into the string buffer pointed
- * to by "line".  Stop reading if either '\n' or an error is encountered.
- * Replace '\n' with '\0'.  Upon return "*n_read" contains the number
- * of characters written to "line", not including the terminating '\0'.
- * If not enough space provided in "line" to accomodate the '\0'-terminated
- * line, then all "size" bytes are used up and "*n_read" is equal to "size"
- * upon return - this is the only case when "line" will not be '\0'-terminated.
+/** Read up to "size" bytes from connection into a string buffer pointed to by
+ * "line".  Stop reading if either '\n' or an error is encountered.  Replace
+ * '\n' with '\0'.  Upon return, "*n_read" contains the number of characters
+ * stored in "line", not including the terminating '\0'.  If there was not
+ * enough space provided in "line" to accomodate the '\0'-termination, then
+ * all "size" bytes are used up, and "*n_read" is equal to "size" upon return -
+ * this is the _only_ case when "line" will _not_ be '\0'-terminated.
  *
  * Return code advises the caller whether another read can be attempted:
  *   * eIO_Success -- read completed successfully, keep reading;
  *   * other code  -- an error occurred, and further read attempt may fail.
  *
+ * @note
+ *   This call may cause some data read from the underlying CONNECTOR to be
+ *   stored in an internal pending input buffer of CONN.
+ *
  * This call utilizes eIO_Read timeout as set by CONN_SetTimeout().
  * @sa
  *  CONN_Read, CONN_SetTimeout
@@ -311,11 +321,11 @@ extern NCBI_XCONNECT_EXPORT EIO_Status CONN_ReadLine
  );
 
 
-/** Obtain status of the last I/O operation.  This is NOT a completion
- * code of the last CONN call, but rather some status from a lower level
- * connector's layer (if available).
+/** Obtain status of the last I/O operation.  This is NOT a completion code of
+ * the last CONN call, but rather some status from a lower level  CONNECTOR's
+ * layer (if available).
  * @par Special case:  eIO_Open as "dir" checks whether the connection is in
- * an open state (which means the underlying connector has been successfully
+ * an open state (which means the underlying CONNECTOR has been successfully
  * opened, but does not assure/check availability for any data or I/O), and
  * returns eIO_Success if it is, or an error code otherwise.
  * @sa
@@ -339,9 +349,9 @@ extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Close
 
 
 /** Set user callback function to be invoked upon an event specified by the
- * callback type.  Note that the callback function gets always called prior
- * to the event to happen, e.g. the eCONN_OnClose callback is called when
- * the connection is about to close, but has not yet been closed.
+ * callback type.  Note that the callback function gets always called prior to
+ * the event to happen, e.g. the eCONN_OnClose callback is called when the
+ * connection is about to close, but has not yet been closed.
  * @par
  * The callback function is supplied with 3 arguments:  the connection handle,
  * a type of event (except for eCONN_OnTimeout), and a user data (specified
@@ -378,10 +388,10 @@ extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Close
  */
 typedef enum {
     eCONN_OnClose   = 0,  /**< NB: CONN has been flushed prior to the call   */
-    eCONN_OnRead    = 1,  /**< Read from connector is about to occur         */
-    eCONN_OnWrite   = 2,  /**< Write to connector is about to occur          */
+    eCONN_OnRead    = 1,  /**< Read from CONNECTOR is about to occur         */
+    eCONN_OnWrite   = 2,  /**< Write to CONNECTOR is about to occur          */
     eCONN_OnFlush   = 3,  /**< About to be flushed (NB: == eIO_ReadWrite)    */
-    eCONN_OnTimeout = 4,  /**< Connector I/O has timed out                   */
+    eCONN_OnTimeout = 4,  /**< CONNECTOR I/O has timed out                   */
     eCONN_OnOpen    = 8   /**< Call prior to open (NB: "conn" still closed)  */
 } ECONN_Callback;
 #define CONN_N_CALLBACKS  6
@@ -404,9 +414,9 @@ extern NCBI_XCONNECT_EXPORT EIO_Status CONN_SetCallback
 
 
 /** Get an underlying SOCK handle for connection that is implemented as a
- * socket.  Non-eIO_Success return code guarantees "*sock" is NULL.
- * Set "*sock" to NULL when no socket handle can be obtained.
- * @note  The returned SOCK object remains in use by the connection.
+ * socket.  Non-eIO_Success return code guarantees "*sock" is NULL.  Set
+ * "*sock" to NULL when no socket handle can be obtained.
+   @note  The returned SOCK object remains in use by the connection.
  * @sa
  *  SOCK, SOCK_GetOSHandleEx
  */
