@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONNUTIL__H
 #define CONNECT___NCBI_CONNUTIL__H
 
-/* $Id: ncbi_connutil.h 533483 2017-04-17 12:34:50Z mcelhany $
+/* $Id: ncbi_connutil.h 569141 2018-08-16 16:45:11Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -273,16 +273,22 @@ typedef struct {  /* NCBI_FAKE_WARNING: ICC */
 #define DEF_CONN_HTTP_REFERER     0
 
 /* Environment/registry keys that are *not* kept in SConnNetInfo */
-#define REG_CONN_SERVICE_NAME     "SERVICE_NAME"
-#define REG_CONN_LOCAL_ENABLE     "LOCAL_ENABLE"
-#define REG_CONN_LBSMD_DISABLE    "LBSMD_DISABLE"
-#define REG_CONN_LBOS_ENABLE      "LBOS_ENABLE"
-#define REG_CONN_NAMERD_ENABLE    "NAMERD_ENABLE"
-#define REG_CONN_DISPD_DISABLE    "DISPD_DISABLE"
+#define REG_CONN_SERVICE_NAME               "SERVICE_NAME"
+#define REG_CONN_LOCAL_ENABLE               "LOCAL_ENABLE"
+#define REG_CONN_LBSMD_DISABLE              "LBSMD_DISABLE"
+#define REG_CONN_LBOS_ENABLE                "LBOS_ENABLE"
+#define REG_CONN_LINKERD_ENABLE             "LINKERD_ENABLE"
+#define REG_CONN_NAMERD_FOR_LINKERD_ENABLE  "NAMERD_FOR_LINKERD_ENABLE"
+#define REG_CONN_NAMERD_ENABLE              "NAMERD_ENABLE"
+#define REG_CONN_DISPD_DISABLE              "DISPD_DISABLE"
 
 /* Local service dispatcher */
-#define REG_CONN_LOCAL_SERVICES   "LOCAL_SERVICES"
-#define REG_CONN_LOCAL_SERVER     DEF_CONN_REG_SECTION "_" "LOCAL_SERVER"
+#define REG_CONN_LOCAL_SERVICES     "LOCAL_SERVICES"
+#define REG_CONN_LOCAL_SERVER       DEF_CONN_REG_SECTION "_" "LOCAL_SERVER"
+
+/* Local IP table */
+#define REG_CONN_LOCAL_IPS          "LOCAL_IPS"
+#define REG_CONN_LOCAL_IPS_DISABLE  "NONE"
 
 
 /* Lookup "param" in the registry / environment.
@@ -293,8 +299,8 @@ typedef struct {  /* NCBI_FAKE_WARNING: ICC */
  *    then "as-is");
  * 2. Registry key "param" in the section "[service]";
  * 3. Environment setting "param" (in all upper-case);
- * 4. Registry key "param" (with "CONN_", if there was any, stripped)
- *    in the section "[CONN]".
+ * 4. Registry key "param" (with leading "CONN_" stripped) in the default
+ *    section "[CONN]".
  * Steps 1 & 2 skipped for "service" passed as NULL or empty ("").
  * Steps 3 & 4 skipped for non-empty "service" and the "param" that already
  * begins with "CONN_".
@@ -612,10 +618,6 @@ extern NCBI_XCONNECT_EXPORT void ConnNetInfo_Destroy(SConnNetInfo* info);
  * On success, return eIO_Success and non-NULL handle of a socket via the last
  * parameter.
  *
- * ATTENTION:  due to the very essence of the HTTP/1.x connection, you may
- *             perform only one { WRITE, ..., WRITE, READ, ..., READ } cycle,
- *             if using non-CONNECT request methods.
- *
  * The returned socket must be exipicitly closed by "SOCK_Close()" when no
  * longer needed.
  *
