@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CORE__H
 #define CONNECT___NCBI_CORE__H
 
-/* $Id: ncbi_core.h 533755 2017-04-19 16:26:34Z lavr $
+/* $Id: ncbi_core.h 568303 2018-08-02 14:55:14Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -105,10 +105,13 @@ typedef enum {
 
 
 /** I/O event (or direction).
- * @par <b>NOTE:</b>
+ * @note
  *  Internally, these constants are used as bit-values, and therefore must not
  *  be changed in this header.  On the other hand, user code should not rely
  *  on the values of these constants, either.
+ * @warning
+ *  Careful with an unfortunate naming similatiry of eIO_Close from this set
+ *  and eIO_Closed from EIO_Status -- do not mix the two!
  * @sa
  *  SOCK_Wait, SOCK_Poll, CONN_Wait, SOCK_SetTimeout, CONN_SetTimeout
  */
@@ -122,15 +125,20 @@ typedef enum {
 
 
 /** I/O status.
+ * @warning
+ *  Careful with an unfortunate naming similatiry of eIO_Closed from this set
+ *  and eIO_Close from EIO_Event -- do not mix the two!
  */
 typedef enum {
     eIO_Success = 0,    /**< everything is fine, no error occurred           */
     eIO_Timeout,        /**< timeout expired before any I/O succeeded        */
-    eIO_Closed,         /**< peer has closed the connection                  */
-    eIO_Interrupt,      /**< signal received while I/O was in progress       */
-    eIO_InvalidArg,     /**< bad argument value(s)                           */
-    eIO_NotSupported,   /**< the requested operation is not supported        */
-    eIO_Unknown         /**< unknown (most probably -- fatal) error          */
+    eIO_Reserved,       /**< reserved status code -- do not use!             */
+    eIO_Interrupt,      /**< signal arrival prevented any I/O to succeed     */
+    eIO_InvalidArg,     /**< bad argument / parameter value(s) supplied      */
+    eIO_NotSupported,   /**< operation is not supported or is not available  */
+    eIO_Unknown,        /**< unknown I/O error (likely fatal but can retry)  */
+    eIO_Closed          /**< connection is / has been closed, EOF condition  */
+#define EIO_N_STATUS  8
 } EIO_Status;
 
 
@@ -139,6 +147,10 @@ typedef enum {
  *  An enum value to get the text form for
  * @return 
  *  Verbal description of the I/O status
+ * @warning
+ *  Returns NULL on out-of-bound values
+ * @note
+ *  Reserved status code(s) returned as an empty string ("")
  * @sa
  *  EIO_Status
  */
@@ -165,9 +177,9 @@ typedef struct MT_LOCK_tag* MT_LOCK;
 
 
 /** Set the lock/unlock callback function and its data for MT critical section.
- * @par <b>TIP:</b>
- *  If the RW-lock functionality is not provided by the
- *  callback, then: eMT_LockRead <==> eMT_Lock
+ * @note
+ *  If the RW-lock functionality is not provided by the callback, then:
+ *  eMT_LockRead <==> eMT_Lock
  *
  */
 typedef enum {
@@ -186,7 +198,7 @@ typedef enum {
  *  As passed to MT_LOCK_Do()
  * @return
  *  Non-zero value if the requested operation was successful.
- * @par <b>NOTE:</b>
+ * @note
  *  The "-1" value is reserved for unset handler;  you also may want to
  *  return "-1" if your locking function does no locking, and you don't
  *  consider it as an error, but still want the caller to be aware of this
@@ -255,7 +267,7 @@ extern NCBI_XCONNECT_EXPORT MT_LOCK MT_LOCK_Delete(MT_LOCK lk);
  * @return
  *  Value returned by the lock handler ("handler" in MT_LOCK_Create()).
  * If lock handler is not specified then always return "-1".
- * @par <b>NOTE:</b>
+ * @note
  *  Use MT_LOCK_Do to avoid overhead!
  * @sa
  *  MT_LOCK_Create, FMT_LOCK_Handler, EMT_Lock
@@ -368,7 +380,7 @@ typedef void (*FLOG_Cleanup)
 
 
 /** Create a new LOG (with an internal reference count set to 1).
- * @par <b>ATTENTION:</b>
+ * @warning
  *  If non-NULL "lock" is specified then MT_LOCK_AddRef() is called on it here,
  *  and MT_LOCK_Delete() will be called on it when this LOG gets deleted.
  * @param data
@@ -391,7 +403,7 @@ extern NCBI_XCONNECT_EXPORT LOG LOG_Create
 
 
 /** Reset the "lg" to use the new "data", "handler" and "cleanup".
- * @par <b>NOTE:</b>
+ * @note
  *  It does not change the reference count of the log.
  * @param lg
  *  A log handle previously obtained from LOG_Create
@@ -434,11 +446,11 @@ extern NCBI_XCONNECT_EXPORT LOG LOG_AddRef(LOG lg);
 extern NCBI_XCONNECT_EXPORT LOG LOG_Delete(LOG lg);
 
 
-/** Upon having filled SLOG_Handler data from parameters, write a message
+/** Upon having filled SLOG_Message data from parameters, write a message
  * (perhaps with raw data attached) to the log by calling LOG_WriteInternal().
- * @par <b>NOTE:</b>
- *  Do not call this function directly, if possible.
- *  Instead, use LOG_WRITE() and LOG_DATA() macros from <connect/ncbi_util.h>!
+ * @note
+ *  Do not call this function directly, if possible.  Instead, use the
+ *  LOG_WRITE() and LOG_DATA() macros from <connect/ncbi_util.h>!
  * @param code
  *  Error code of the message
  * @param subcode
@@ -479,9 +491,9 @@ extern NCBI_XCONNECT_EXPORT void LOG_Write
 
 /** Write message (perhaps with raw data attached) to the log by calling
  * "lg->handler(lg->data, mess)".
- * @par <b>NOTE:</b>
- *  Do not call this function directly, if possible.
- *  Instead, use the LOG_WRITE() and LOG_DATA() macros from <ncbi_util.h>!
+ * @note
+ *  Do not call this function directly, if possible.  Instead, use the
+ *  LOG_WRITE() and LOG_DATA() macros from <ncbi_util.h>!
  * @param lg
  *  A log handle previously obtained from LOG_Create
  * @sa
@@ -519,9 +531,9 @@ typedef enum {
  * Look for the matching entry first in the transient storage, and then in
  * the persistent storage.  Do not modify the "value" (leave it "as is",
  * i.e. default) if the requested entry is not found in the registry.
- * @par <b>NOTE:</b>
+ * @note
  *  Always terminate value with '\0'.
- * @par <b>NOTE:</b>
+ * @note
  *  Do not put more than "value_size" bytes to "value".
  * @param data
  *  Unspecified data as passed to REG_Create or REG_Reset
@@ -584,7 +596,7 @@ typedef void (*FREG_Cleanup)
 
 
 /** Create a new registry (with an internal reference count set to 1).
- * @par <b>ATTENTION:</b>
+ * @warning
  *  if non-NULL "lock" is specified then MT_LOCK_AddRef() is called on it here,
  *  and MT_LOCK_Delete() will be called on it when this REG gets destroyed.
  *  Passing NULL callbacks below causes limiting the functionality
@@ -613,7 +625,7 @@ extern NCBI_XCONNECT_EXPORT REG REG_Create
 
 /** Reset the registry handle to use the new "data", "set", "get",
  * and "cleanup".
- * @par <b>NOTE:</b>
+ * @note
  *  No change to the internal reference count.
  * @param rg
  *  Registry handle as previously obtained from REG_Create
