@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_PIPE__HPP
 #define CONNECT___NCBI_PIPE__HPP
 
-/* $Id: ncbi_pipe.hpp 482816 2015-10-26 21:18:02Z lavr $
+/* $Id: ncbi_pipe.hpp 567538 2018-07-19 18:30:45Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -104,9 +104,9 @@ public:
                                   ///< fKeepOnClose takes the effect.
         fSigPipe_Restore = 0x040, ///< Restore SIGPIPE processing for child
                                   ///< process to system default.
-        fNewGroup        = 0x100  ///< UNIX: new process group will be
-                                  ///< created and child become the leader
-                                  ///< of the new process group.
+        fNewGroup        = 0x100  ///< UNIX: new process group will be created,
+                                  ///< and the child process will become the
+                                  ///< leader of the new process group.
     };
     typedef unsigned int TCreateFlags;  ///< bitwise OR of "ECreateFlag"
 
@@ -140,8 +140,11 @@ public:
     /// @param current_dir
     ///   Current working directory for the new process if specified.
     /// @param env
-    ///   Pointer to vector with environment variables which will be used
-    ///   instead of current environment.  Last value in an array must be NULL.
+    ///   An optional pointer to a vector with environment variables to use
+    ///   in the child process.  If not speficied, a copy of the parent's
+    ///   process environment is used.  Note that a new environment completely
+    ///   replaces the default environment otherwise inherited from the parent
+    ///   process, and so it does not just add / modify some values.
     /// @sa
     ///   Open
     CPipe(const string&         cmd,
@@ -183,8 +186,11 @@ public:
     ///   also contain a drive letter. If this parameter is empty, the new
     ///   process will have the same current directory as the calling process.
     /// @param env
-    ///   Pointer to vector with environment variables which will be used
-    ///   instead of current environment. Last value in an array must be NULL.
+    ///   An optional pointer to a vector with environment variables to use
+    ///   in the child process.  If not speficied, a copy of the parent's
+    ///   process environment is used.  Note that a new environment completely
+    ///   replaces the default environment otherwise inherited from the parent
+    ///   process, and so it does not just add / modify some values.
     /// @return 
     ///   Completion status.
     /// @sa
@@ -206,7 +212,7 @@ public:
     ///
     /// @sa
     ///   Read, Write, Close
-    void       OpenSelf(void);
+    void OpenSelf(void);
 
     /// Close pipe.
     ///
@@ -449,9 +455,11 @@ public:
     ///   also contain a drive letter.  If this parameter is empty, the new
     ///   process will have the same current directory as the calling process.
     /// @param env
-    ///   Pointer to a vector with environment variables, which will be used
-    ///   instead of the current environment.  Last element in the array must
-    ///   be NULL.
+    ///   An optional pointer to a vector with environment variables to use
+    ///   in the child process.  If not speficied, a copy of the parent's
+    ///   process environment is used.  Note that a new environment completely
+    ///   replaces the default environment otherwise inherited from the parent
+    ///   process, and so it does not just add / modify some values.
     /// @param watcher
     ///   Call back object to monitor the child process execution status
     /// @param kill_timeout
@@ -464,7 +472,7 @@ public:
     ///   eDone if process has finished normally, or eCanceled if a watcher 
     ///   decided to stop it.
     ///
-    /// @sa IProcessWatcher
+    /// @sa IProcessWatcher, Open
     static EFinish ExecWait(const string&         cmd,
                             const vector<string>& args,
                             CNcbiIstream&         in,
