@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_SERVER_INFO__H
 #define CONNECT___NCBI_SERVER_INFO__H
 
-/* $Id: ncbi_server_info.h 494171 2016-03-04 01:46:23Z lavr $
+/* $Id: ncbi_server_info.h 533755 2017-04-19 16:26:34Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -29,17 +29,21 @@
  * Authors:  Anton Lavrentiev, Denis Vakatov
  *
  * File Description:
+ * @file ncbi_server_info.h
  *   NCBI server meta-address info
- *   Note that all server meta-addresses are allocated as
- *   single contiguous pieces of memory, which can be copied in whole
- *   with the use of 'SERV_SizeOfInfo' call. Dynamically allocated
- *   server infos can be freed with a direct call to 'free'.
- *   Assumptions on the fields: all fields in the server info come in
- *   host byte order except 'host', which comes in network byte order.
+ *
+ *   Note that all server meta-addresses are allocated as single contiguous
+ *   pieces of memory, which can be copied in whole with the use of
+ *   'SERV_SizeOfInfo' call.  Dynamically allocated server infos can be freed
+ *   with a direct call to 'free'.
+ *
+ *   Assumptions on the fields: all fields in the server info come in host byte
+ *   order except 'host', which comes in network byte order.
  *
  */
 
 #include <connect/ncbi_connutil.h>
+#include <connect/ncbi_ipv6.h>
 
 
 /** @addtogroup ServiceSupport
@@ -47,7 +51,7 @@
  * @{
  */
 
-#define SERV_DEFAULT_FLAG   eSERV_Regular
+#define SERV_DEFAULT_ALGO   eSERV_Regular
 #define SERV_MINIMAL_RATE   0.001
 #define SERV_MAXIMAL_RATE   100000.0
 #define SERV_MINIMAL_BONUS  0.01
@@ -74,29 +78,29 @@ typedef enum {
 
 /* Flag to specify an algorithm for selecting the most preferred server from
  * a set of available servers (NB: binary properties of the enumerated values
- * may be in use!).  Should have been named ESERV_Algo, but since it's now
- * adding the "Inter" feature (also misplaced, and to be moved away to the part
- * of "locl" area, the mis-name is still retained.
+ * may be in use!).
  */
 typedef enum {
-    eSERV_Regular      = 0,  /* Server either tied up to lavg or static   */
-    eSERV_Blast        = 1,  /* Server is tied up to an instant host load */
-    eSERV_RegularInter = 2,  /* Regular server shared between LBSM zones  */
-    eSERV_BlastInter   = 3,  /* ... same for Blast-type server            */
-} ESERV_Flag;
+    eSERV_Regular = 0,       /* Server either tied up to lavg or static      */
+    eSERV_Blast = 1          /* Server is tied up to an instant host load    */
+} ESERV_Algo;
+typedef unsigned char TSERV_Algo;
 
 
 typedef enum {
-    fSERV_Stateful = 1,
-    fSERV_Secure = 2
+    fSERV_Stateful = 1,      /* Server requires persistent connection        */
+    fSERV_Secure = 2         /* Server requires secure connection            */
 } ESERV_Mode;
+typedef unsigned char TSERV_Mode;
 
 
 typedef enum {
-    fSERV_Local = 1,
-    fSERV_Private = 2,
-    fSERV_SiteMask = 0xF0
+    fSERV_Local = 1,         /* Server accessible only for NCBI local clients*/
+    fSERV_Private = 2,       /* Server accessible only by localhost clients  */
+    fSERV_Interzone = 4,     /* Server record spans LB zones within the site */
+    fSERV_ZoneMask = 0xF0    /* Local zone number for the server             */
 } ESERV_Site;
+typedef unsigned char TSERV_Site;
 
 
 /* Verbal representation of a server type (no internal spaces allowed)
@@ -107,9 +111,8 @@ extern NCBI_XCONNECT_EXPORT const char* SERV_TypeStr
 
 
 /* Read server info type.
- * If successful, assign "type" and return pointer to the position
- * in the "str" immediately following the type tag.
- * On error, return NULL.
+ * If successful, assign "type" and return pointer to the position in the "str"
+ * immediately following the type tag.  On error, return NULL.
  */
 extern NCBI_XCONNECT_EXPORT const char* SERV_ReadType
 (const char* str,
@@ -120,28 +123,27 @@ extern NCBI_XCONNECT_EXPORT const char* SERV_ReadType
 /* Meta-addresses for various types of NCBI servers
  */
 typedef struct {
-    TNCBI_Size   args;
+    TNCBI_Size            args;
 #define SERV_NCBID_ARGS(ui)     ((char*)(ui) + (ui)->args)
 } SSERV_NcbidInfo;
 
 typedef struct {
-    char         dummy;         /* placeholder, not used                     */
+    char                  _pad; /* placeholder, not used                     */
 } SSERV_StandaloneInfo;
 
 typedef struct {
-    TNCBI_Size   path;
-    TNCBI_Size   args;
+    TNCBI_Size            path;
+    TNCBI_Size            args;
 #define SERV_HTTP_PATH(ui)      ((char*)(ui) + (ui)->path)
 #define SERV_HTTP_ARGS(ui)      ((char*)(ui) + (ui)->args)
 } SSERV_HttpInfo;
 
 typedef struct {
-    ESERV_Type   type;          /* type of original server                   */
+    ESERV_Type            type; /* type of the original server               */
 } SSERV_FirewallInfo;
 
 typedef struct {
-    char/*bool*/ name;          /* name presence flag                        */
-    char         pad[7];        /* reserved for the future use, must be zero */
+    unsigned char/*bool*/ name; /* name presence flag                        */
 } SSERV_DnsInfo;
 
 
@@ -159,21 +161,23 @@ typedef struct {
     ESERV_Type            type; /* type of server                            */
     unsigned int          host; /* host the server running on, network b.o.  */
     unsigned short        port; /* port the server running on, host b.o.     */
-    unsigned char         mode; /* connection mode (stateful, secure, etc)   */
-    unsigned char         site; /* site info (LSB: local, private; MSB: @#)  */
+    TSERV_Mode            mode; /* connection mode (stateful, secure, etc)   */
+    TSERV_Site            site; /* site info (LSB: local, private; MSB: @#)  */
     TNCBI_Time            time; /* relaxation period / expiration time       */
     double                coef; /* bonus coefficient for server run locally  */
     double                rate; /* rate of the server                        */
     EMIME_Type          mime_t; /* type,                                     */
     EMIME_SubType       mime_s; /*     subtype,                              */
     EMIME_Encoding      mime_e; /*         and encoding for content-type     */
-    ESERV_Flag            flag; /* rate algorithm for the server (NB: algo!) */
-    unsigned char reserved[16]; /* zeroed reserved area - do not use!        */
+    TSERV_Algo            algo; /* rate algorithm for the server             */
+    TNCBI_IPv6Addr        addr; /* IPv6 address (for host == INADDR_NONE(-1))*/
+    unsigned char        vhost; /* extra (v)host name size if non-zero       */
+    unsigned short       extra; /* extra data size if non-zero               */
     USERV_Info               u; /* server type-specific data/params          */
 } SSERV_Info;
 
 
-/* Constructors for the various types of NCBI server meta-addresses
+/* Constructors for the various types of NCBI server meta-addressesr45
  */
 extern NCBI_XCONNECT_EXPORT SSERV_Info* SERV_CreateNcbidInfo
 (unsigned int   host,           /* network byte order                        */
@@ -201,7 +205,7 @@ extern NCBI_XCONNECT_EXPORT SSERV_Info* SERV_CreateFirewallInfo
  );
 
 extern NCBI_XCONNECT_EXPORT SSERV_Info* SERV_CreateDnsInfo
-(unsigned int   host            /* the only parameter                        */
+(unsigned int   host            /* network byte order, the only parameter    */
  );
 
 
@@ -219,10 +223,11 @@ extern NCBI_XCONNECT_EXPORT char* SERV_WriteInfo
  *
  * TYPE := { STANDALONE | NCBID | HTTP[{_GET|_POST}] | FIREWALL | DNS }
  *
- * Host should be specified as either an IP address (in dotted notation),
- * or as a host name (using domain notation if necessary).
- * Port number must be preceded by a colon.
- * Both host and port get their default values if not specified.
+ * Host must be specified as either an IP address (in dotted notation), as an
+ * IPv6 address (enclosed in square brackets), or as a host name (using domain
+ * notation if necessary).  Port number must be preceded by a colon.  Both host
+ * and port defaults to server-specific values if omitted.  For null
+ * specification ":0" can be used.
  *
  * Server-specific parameters:
  *
@@ -231,32 +236,32 @@ extern NCBI_XCONNECT_EXPORT char* SERV_WriteInfo
  *
  *    NCBID servers: Arguments to CGI in addition to specified by application.
  *                   Empty additional arguments denoted as '' (two single
- *                   quotes, back to back).  Note that the additional
- *                   arguments must not contain space characters.
+ *                   quotes, back to back).  Note that the additional arguments
+ *                   must not contain space characters.
  *
  *    HTTP* servers: Path (required) and args (optional) in the form
  *                   path[?args] (here brackets denote the optional part).
  *                   Note that no spaces are allowed within these parameters.
  *
- *    FIREWALL servers: Servers of this type are converted real servers of
- *                      the above types, when only accessible via FIREWALL
- *                      mode of NCBI dispatcher.  The purpose of this fake
- *                      server type is just to let the client know that
- *                      the service exists.  Additional parameter is optional
- *                      and if present, is the original type of the server
- *                      before conversion.  Note that servers of this type
- *                      cannot be configured in LBSMD.
+ *    FIREWALL servers: Servers of this type are converted real servers of the
+ *                      above types, when only accessible via FIREWALL mode of
+ *                      NCBI dispatcher.  The purpose of this fake server type
+ *                      is just to let the client know that the service exists.
+ *                      Additional parameter is optional and if present, is the
+ *                      original type of the server before conversion.  Note
+ *                      that servers of this type cannot be configured in
+ *                      LBSMD.
  *
- *    DNS servers: Services for DNS and DB load-balancing, 
- *                 and dynamic ProxyPassing at the NCBI Web entry point.
- *                 Never exported to the outside world.
+ *    DNS servers: Services for DNS and DB load-balancing, and dynamic reverse-
+ *                 proxying (the ProxyPass directive of the Apache HTTP daemon)
+ *                 at the NCBI Web entry point.  Never exposed to the outside.
  *
- * Tags may follow in no particular order but no more than one instance
- * of each flag is allowed:
+ * Tags may follow in no particular order but no more than one instance of each
+ * tag is allowed:
  *
- *    Load average calculation for the server:
- *       Regular        (default)
- *       Blast
+ *    Load average calculation algorithm for the server:
+ *       A=R            (Regular=default)
+ *       A=B            (Blast)
  *
  *    Bonus coefficient:
  *       B=double       [0.0 = default]
@@ -285,6 +290,12 @@ extern NCBI_XCONNECT_EXPORT char* SERV_WriteInfo
  *           get proper content type, defined at service configuration.
  *           This tag is not allowed in DNS server specifications.
  *
+ *    (V)Host name:
+ *       H=hostname     (optional)
+ *           specifies the host name to use when accessing the server.  Can be
+ *           implicitly set by server configuration if a hostname (instead of
+ *           an IP) is provided as the connection point (the host:port pair).
+ *
  *    Local server:
  *       L={yes|no}     [default is set by SERV_SetLocalServerDefault()]
  *           Local servers are accessible only by local clients (from within
@@ -302,8 +313,8 @@ extern NCBI_XCONNECT_EXPORT char* SERV_WriteInfo
  *
  *    Reachability base rate:
  *       R=double       [0.0 = default]
- *           specifies availability rate for the server, expressed as
- *           a floating point number with 0.0 meaning the server is down
+ *           specifies availability rate for the server, expressed as a
+ *           floating point number with 0.0 meaning the server is down
  *           (unavailable) and 1000.0 meaning the server is up and running.
  *           Intermediate or higher values can be used to make the server less
  *           or more favorable for choosing by LBSM Daemon, as this coefficient
@@ -314,8 +325,8 @@ extern NCBI_XCONNECT_EXPORT char* SERV_WriteInfo
  *           dynamically switches this rate to be the maximal specified when
  *           the server is up, and to be zero when the server is down.
  *           Note that negative values are reserved for LBSMD private use.
- *           To specify a server as inactive in LBSMD configuration file,
- *           one can use any negative number (note that value "0" in the config
+ *           To specify a server as inactive in LBSMD configuration file, one
+ *           can use any negative number (note that value "0" in the config
  *           file means "default" and gets replaced with the value 1000.0).
  *           Values less than 0.01 define standby server entries, which are
  *           used by the clients only if there are no working entries with a
@@ -338,15 +349,21 @@ extern NCBI_XCONNECT_EXPORT char* SERV_WriteInfo
  *           protocol (HTTPS or SSL).
  *
  *    Validity period:
- *       T=integer      [0 = default]
+ *       T=integer      [0=default]
  *           specifies the time in seconds that this server entry is valid for
- *           without an update. (If equal to 0 then defaulted by
- *           the LBSM Daemon to some reasonable value.)
+ *           without an update.  (If equal to 0 then defaulted to the LBSM
+ *           Daemon to some reasonable value.)
+ *
+ *    Interzone server:
+ *       X=no           (default)
+ *       X=yes
+ *           indicates that the server definition can span zones within the
+ *           current site (by default, a server info is local to a zone).
  *
  *
  * Note that optional arguments can be omitted along with all preceding
- * optional arguments; that is, the following 2 server specifications are
- * both valid:
+ * optional arguments;  that is, the following 2 server specifications are both
+ * valid:
  *
  * NCBID ''
  * and
@@ -354,13 +371,13 @@ extern NCBI_XCONNECT_EXPORT char* SERV_WriteInfo
  *
  * but they are not equal to the following specification:
  *
- * NCBID Regular
+ * NCBID A=R
  *
- * because here 'Regular' is treated as an argument, not as a tag.
- * To make the latter specification equivalent to the former two, one has
- * to use the following form:
+ * because here 'A=R' is treated as an argument, not as a tag.  To make the
+ * latter specification equivalent to the former two (since 'A=R' is the
+ * default tag), one has to use the following form:
  *
- * NCBID '' Regular
+ * NCBID '' A=R
  */
 
 
