@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_SERVICE__H
 #define CONNECT___NCBI_SERVICE__H
 
-/* $Id: ncbi_service.h 533755 2017-04-19 16:26:34Z lavr $
+/* $Id: ncbi_service.h 565384 2018-06-11 17:56:27Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -51,7 +51,7 @@
  * @sa
  *  SERV_OpenEx, SERV_GetInfoEx
  */
-#define SERV_LOCALHOST  ((unsigned int) ~0UL)
+#define SERV_LOCALHOST  ((unsigned int)(~0UL))
 #define SERV_ANYHOST    0
 
 
@@ -77,17 +77,19 @@ typedef const SSERV_Info* SSERV_InfoCPtr;
  */
 enum ESERV_TypeSpecial {
     fSERV_Any               = 0,
-    fSERV_All               = 0x0000FFFF,  /**< Server type mask          */
-    fSERV_Stateless         = 0x00100000,  /**< Stateless servers only    */
-    fSERV_Reserved          = 0x00400000,  /**< Reserved, MBZ             */
-    fSERV_ReverseDns        = 0x00800000,  /**< LB-DNS translation        */
-    /* The following allow to get currently inactive service instances    */
+    fSERV_All               = 0x00007FFF,  /**< Server type mask             */
+    fSERV_Stateless         = 0x00008000,  /**< Stateless servers only       */
+    fSERV_Reserved1         = 0x00100000,  /**< Reserved, MBZ                */
+    fSERV_DelayOpen         = 0x00400000,  /**< Don't open service until use */
+    fSERV_ReverseDns        = 0x00800000,  /**< LB-DNS translation           */
+    /* The following allow to get otherwise excluded service instances       */
     fSERV_IncludeDown       = 0x08000000,
     fSERV_IncludeStandby    = 0x10000000,
-    fSERV_IncludeReserved   = 0x20000000,  /**< @note Not yet implemented */
+    fSERV_IncludeReserved   = 0x20000000,  /**< @note Not yet implemented    */
     fSERV_IncludeSuppressed = 0x40000000,
     fSERV_IncludeInactive   = 0x70000000,
-    fSERV_Promiscuous       = 0x78000000
+    fSERV_IncludePrivate    = 0x80000000,
+    fSERV_Promiscuous       = 0xF8000000   /**< Evrthng and the kitchen sink */
 };
 typedef unsigned int   TSERV_Type;      /**<Bitwise OR of ESERV_Type[Special]*/
 typedef unsigned short TSERV_TypeOnly;  /**<Server type only, w/o specials   */
@@ -104,23 +106,25 @@ typedef unsigned short TSERV_TypeOnly;  /**<Server type only, w/o specials   */
  *  Preferred host to use the service at, nbo.
  * @param net_info
  *  Connection information (NULL prevents use of the network dispatching via
- *  DISPD, still allowing use of LBOS/NAMERD).
- * @note If "net_info" is NULL, only LOCAL/LBSMD/LBOS/NAMERD mappers will be
- *       consulted. If "net_info" is not NULL, LOCAL/LBSMD/LBOS/NAMERD are
- *       consulted first, and then DISPD is consulted last (using the connection
- *       information provided) but only if mapping with LOCAL/LBSMD/LBOS/NAMERD
- *       (if any occurred) has failed.
- * @note The registry section [CONN], keys LOCAL_DISABLE, LBSMD_DISABLE,
- *       LBOS_DISABLE, NAMERD_DISABLE, and DISPD_DISABLE (which can be
- *       overridden by the environment variables CONN_LOCAL_DISABLE,
- *       CONN_LBSMD_DISABLE, CONN_LBOS_DISABLE, CONN_NAMERD_DISABLE, and
- *       CONN_DISPD_DISABLE, respectively) can
- *       be used to skip the corresponding service mappers. This scheme
+ *  DISPD, still allowing use of LBOS/NAMERD/LINKERD).
+ * @note If "net_info" is NULL, only the following mappers will be consulted:
+ *          LOCAL, LBSMD, LBOS, NAMERD, LINKERD
+ *       If "net_info" is not NULL, the above mappers are consulted first,
+ *       and then DISPD is consulted last (using the connection information
+ *       provided) but only if mapping with the above mappers (if any occurred)
+ *       has failed.
+ * @note The registry section [CONN], keys:
+ *          LOCAL_DISABLE, LBSMD_DISABLE, LBOS_DISABLE,
+ *          NAMERD_DISABLE, LINKERD_DISABLED, DISPD_DISABLE
+ *       which can be overridden by the environment variables:
+ *          CONN_LOCAL_DISABLE, CONN_LBSMD_DISABLE, CONN_LBOS_DISABLE,
+ *          CONN_NAMERD_DISABLE, CONN_LINKERD_DISABLE, CONN_DISPD_DISABLE
+ *       can be used to skip the corresponding service mappers. This scheme
  *       permits to use any combination of the service mappers
- *       (local/lbsmd/lbos/namerd/network-based).
+ *       (local/lbsmd/lbos/namerd/linkerd/network-based).
  * @note If "net_info" is not NULL then a non-zero value of
- *       "net_info->stateless" forces "types" to get the "fSERV_StatelessOnly"
- *       bit set implicitly.
+ *       "net_info->stateless" forces "types" to get the "fSERV_Stateless" bit
+ *       set implicitly.
  * @param skip
  *  An array of servers NOT to select: contains server-info elements that are
  *  not to return from the search (whose server-infos match the would-be
@@ -244,7 +248,7 @@ extern NCBI_XCONNECT_EXPORT SSERV_InfoCPtr SERV_GetNextInfo
  *  Preferred host to use the service at, nbo.
  * @param net_info
  *  Connection information (NULL disables network dispatching via
- *  DISPD, still allowing use of LBOS/NAMERD).
+ *  DISPD, still allowing use of LBOS/NAMERD/LINKERD).
  * @param skip[]
  *  An array of servers NOT to select, see SERV_OpenEx() for notes.
  * @param n_skip
