@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_SOCKET__H
 #define CONNECT___NCBI_SOCKET__H
 
-/* $Id: ncbi_socket.h 537331 2017-05-30 19:56:20Z lavr $
+/* $Id: ncbi_socket.h 569145 2018-08-16 16:47:19Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -148,7 +148,8 @@
  *
  * Secure Socket Layer:
  *
- *  SOCK_SetupSSL
+ *  SOCK_SetupSSL[Ex]
+ *  SOCK_SSLName
  *
  */
 
@@ -300,7 +301,7 @@ extern NCBI_XCONNECT_EXPORT const STimeout*SOCK_SetSelectInternalRestartTimeout
  *  SOCK_SetIOWaitSysAPI
  */
 typedef enum {
-    eSOCK_IOWaitSysAPIAuto,   /**< default; use some euristics to choose API */
+    eSOCK_IOWaitSysAPIAuto,   /**< default; use some heuristics to choose API*/
     eSOCK_IOWaitSysAPIPoll,   /**< always use poll()                         */
     eSOCK_IOWaitSysAPISelect  /**< always use select()                       */
 } ESOCK_IOWaitSysAPI;
@@ -308,9 +309,9 @@ typedef enum {
 /** This is a helper call that can improve I/O behavior (ignored for Windows).
  * @param api
  *  [in]  Default behavior is to wait on I/O such a way that accomodates the
- *  requested sockets accordingly.  There is a known limitation of select()
+ *  requested sockets accordingly.  There is a known limitation of the select()
  *  API that requires all sockets to have low-level IO descriptors less than
- *  1024, but works faster than poll() API that does not have limits on the
+ *  1024, but works faster than the poll() API that does not have limits on the
  *  numeric values of the descriptors.  Either API can be enforced here.
  * @return
  *  Previous value of the API selector
@@ -629,25 +630,39 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Create
  );
 
 
-/** [SERVER-side]  Create a socket on top of either an OS-dependent "handle"
+/** [SERVER-side]  Create a SOCKet on top of either an OS-dependent "handle"
  * (file descriptor on Unix, SOCKET on MS-Windows) or an existing SOCK object.
  * Returned socket is not reopenable to its default peer (SOCK_Reconnect() may
  * not specify zeros for the connection point).
  * All timeouts are set to default [infinite] values.
- * The call does *not* destroy either OS handle or SOCK passed in the
+ * The call does *not* destroy either the OS handle or the SOCK passed in the
  * arguments, regardless of the return status code.
- * When a socket gets created on top of a "SOCK" handle, the original SOCK gets
- * always emptied (the underlying OS handle removed from it) upon the call
- * returns:  either the handle gets migrated to the new socket just created,
- * or it gets closed unconditionally by the fSOCK_KeepOnClose flag in the
- * original socket.  In either case, the original SOCK will still need
- * SOCK_Destroy() in the caller's code to free up the memory it occupies.
+ * When a socket is being created on top of a SOCK handle, the original SOCK
+ * gets emptied (and the underlying OS handle removed from it) upon the call
+ * returns (whether successfully or not), yet will still need SOCK_Destroy() in
+ * the caller's code to free up the memory it occupies.
  * Any secure session that may have existed in the original SOCK will have
- * been terminated (and new session may have been initiated in the new SOCK --
- * at this time the old session is not allowed to "migrate").
+ * been migrated to the new socket iff "flags" indicate fSOCK_Secure (if no
+ * session existed, it session may have been initiated in the new SOCK).
+ * Otherwise, the original secure session (if any) will have been closed.
+ * Any pending output will have been flushed (if switching secure / insecure
+ * contexts; otherwise, it will have simply migrated to the created SOCK), and
+ * any pending input still in the original SOCK will migrate to the new socket
+ * object returned.  If this behavior is undesireable, one can use
+ * SOCK_GetOSHandleEx() on the original socket, taking the ownership of the
+ * underlying OS handle, and then create a SOCK on top of the bare "handle",
+ * specifying its (non-zero) "handle_size".
+ * @warning
+ *  It is not recommended to use this call on not fully connected sockets
+ *  (either bare OS handle or SOCK) in native MS-Windows builds (e.g. Cygwin is
+ *  okay).
  * @note
- *  SOCK_Close[Ex]() on the resultant socket will not close the OS handle
- *  if fSOCK_KeepOnClose is set in "flags".
+ *  SOCK_Close[Ex]() on the resultant socket will not close the underlying OS
+ *  handle if fSOCK_KeepOnClose is set in "flags".
+ * @note
+ *  (SOCK_IsClientSide() | SOCK_IsServerSide()) can be used to determine if the
+ *  original SOCK was stripped off the underlying handle:  the expression would
+ *  evaluate to 0(false) iff the SOCK object does not have the OS handle.
  * @param handle
  *  [in]  OS-dependent "handle" or SOCK to be converted
  * @param handle_size
@@ -661,9 +676,10 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Create
  * @param flags
  *  [in]  additional socket requirements
  * @return
- *  Return eIO_Success on success;  otherwise: eIO_Closed if the "handle"
+ *  Return eIO_Success on success;  otherwise: eIO_InvalidArg if the "handle"
  *  does not refer to an open socket [but e.g. to a normal file or a pipe];
- *  other error codes in case of other errors.
+ *  eIO_Closed when the original socket is not connected;  other error codes
+ *  in case of other errors.
  * @sa
  *  SOCK_GetOSHandleEx, SOCK_CreateOnTop, SOCK_Reconnect, SOCK_Close
  */
@@ -746,19 +762,20 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Shutdown
  );
 
 
-/** Close the SOCK handle, destroy relevant internal data.
+/** Close the SOCK handle, and destroy all relevant internal data.
  * The "sock" handle goes invalid after this function call, regardless of
  * whether the call was successful or not.  If eIO_Close timeout was specified
  * (or NULL) then it blocks until either all unsent data are sent, an error
- * flagged, or the timeout expires;  if there is any output pending, that
+ * flagged, or the timeout expires;  if there was any output pending, that
  * output will be flushed.
- * Connection may remain in the system if the socket was created with the
- * fSOCK_KeepOnClose flag set;  otherwise, it gets closed.
- * @note
- *  On MS-Win closing a socket whose OS handle has been used elsewhere (e.g.
- *  in SOCK_CreateOnTop()) may render the OS handle unresponsive, so always
- *  make sure to close the current socket first (assuming fSOCK_KeepOnClose),
- *  and only then use the extracted handle to build another socket.
+ * Connection may remain in the system if SOCK was created with the
+ * fSOCK_KeepOnClose flag set;  otherwise, it gets closed as well.
+ * @warning
+ *  On MS-Windows closing a SOCK whose OS handle has been used elsewhere
+ *  (e.g. in SOCK_CreateOnTop()) may render the OS handle unresponsive, so
+ *  always make sure to close the current SOCK first (assuming
+ *  fSOCK_KeepOnClose), and only then use the extracted handle to build
+ *  another socket.
  * @param sock
  *  [in]  socket handle to close(if not yet closed) and destroy(always)
  * @sa
@@ -773,26 +790,27 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Close(SOCK sock);
  * unsent data are sent, an error flagged, or the timeout expires;  if there is
  * any output pending, that output will be flushed.
  * Connection may remain in the system if the socket was created with the
- * fSOCK_KeepOnClose flag set;  otherwise, it gets closed.
+ * fSOCK_KeepOnClose flag set;  otherwise, it gets closed as well.
  * @note
- *  On MS-Win closing a socket whose OS handle has been used elsewhere (e.g.
- *  in SOCK_CreateOnTop()) may render the OS handle unresponsive, so always
- *  make sure to close the current socket first (assuming fSOCK_KeepOnClose),
- *  and only then use the extracted handle to build another socket.
+ *  On MS-Windows closing a SOCK whose OS handle has been used elsewhere
+ *  (e.g. in SOCK_CreateOnTop()) may render the OS handle unresponsive, so
+ *  always make sure to close the current SOCK first (assuming
+ *  fSOCK_KeepOnClose), and only then use the extracted handle to buil
+ *  another socket.
  * @param sock
  *  [in]  handle of the socket to close
  * @param destroy
  *  [in]  =1 to destroy the SOCK handle; =0 to keep the handle
  * @note
- *  A kept SOCK handle can be freed/destroyed by the SOCK_Close() call.
+ *  The kept SOCK handle can be freed/destroyed by SOCK_Destroy().
  * @note
- *  SOCK_CloseEx(sock, 1) is equivalent to SOCK_Close(sock).
+ *  SOCK_CloseEx(sock, 1) is equivalent to SOCK_Destroy(sock).
  * @sa
  *  SOCK_Close, SOCK_Create, SOCK_CreateOnTop, DSOCK_Create, SOCK_SetTimeout
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_CloseEx
-(SOCK         sock,
- int/**bool*/ destroy
+(SOCK        sock,
+ int/*bool*/ destroy
  );
 
 
@@ -800,10 +818,11 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_CloseEx
  * The call retries repeatedly if interrupted by a singal (so no eIO_Interrupt
  * should be expected).  Return eIO_Success when the handle has been closed
  * successfully, eIO_Closed if the handle has been passed already closed,
- * eIO_InvalidArg if passed arguments are not valid, eIO_Unknow if the
- * handle cannot be closed (per an error returned by the system).
- * @note  Using this call on a handle that belongs to an active [LD]SOCK object
- *        is undefined.
+ * eIO_InvalidArg if passed arguments are not valid, eIO_Unknown if the
+ * handle cannot be closed (per an error returned by the system, see errno).
+ * @warning
+ *  Using this call on a handle that belongs to an active [LD]SOCK object is
+ *  undefined.
  * @sa
  *  SOCK_GetOSHandleEx
  */
@@ -841,6 +860,8 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_CloseOSHandle
  * @note  It is allowed to use a non-NULL zeroed STimeout to poll
  *        on the socket for the immediately available event and
  *        return it (or eIO_Timeout, otherwise) without blocking.
+ * @sa
+ *  SOCK_Poll, POLLABLE_Poll
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Wait
 (SOCK            sock,
@@ -902,7 +923,7 @@ typedef struct {
  * @param n_ready
  *  [out] # of ready sockets  (may be NULL)
  * @sa
- *  POLLABLE_Poll
+ *  SOCK_Wait, POLLABLE_Poll
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Poll
 (size_t          n,
@@ -921,6 +942,8 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Poll
  *  [in]  one of:  eIO_[Read/Write/ReadWrite/Close]
  * @param timeout
  *  [in]  new timeout value to set
+ * @sa
+ *  SOCK_Read, SOCK_Write, SOCK_Close
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_SetTimeout
 (SOCK            sock,
@@ -938,6 +961,8 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_SetTimeout
  *  [in]  socket handle
  * @param event
  *  [in]  one of:  eIO_[Read/Write/Close]
+ * @sa
+ *  SOCK_Read, SOCK_Write, SOCK_Close
  */
 extern NCBI_XCONNECT_EXPORT const STimeout* SOCK_GetTimeout
 (SOCK      sock,
@@ -1000,6 +1025,8 @@ extern NCBI_XCONNECT_EXPORT const STimeout* SOCK_GetTimeout
  *  [out] # of bytes read  (can be NULL)
  * @param how
  *  [in]  how to read the data
+ * @sa
+ *  SOCK_SetTimeout
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Read
 (SOCK           sock,
@@ -1032,6 +1059,8 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Read
  *  an error.  Note that *n_read must be analyzed prior to return code,
  *  because the buffer could have received some contents before
  *  the indicated error occurred (especially when connection closed).
+ * @sa
+ *  SOCK_SetTimeout
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_ReadLine
 (SOCK    sock,
@@ -1128,6 +1157,8 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Status
  * @note  If "size"==0, return value can be eIO_Success if no
  *        pending data left in the socket, or eIO_Timeout if there are still
  *        data pending.  In either case, "*n_written" is set to 0 on return.
+ * @sa
+ *  SOCK_SetTimeout
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Write
 (SOCK            sock,
@@ -1252,6 +1283,8 @@ extern NCBI_XCONNECT_EXPORT unsigned short SOCK_GetRemotePort
  *  [in]  what parts of address to include
  * @return
  *  On success, return its "buf" argument; return 0 on error.
+ * @sa
+ *  SOCK_HostPortToString
  */
 typedef enum {
     eSAF_Full = 0,  /**< address in full, native form                      */
@@ -1334,6 +1367,8 @@ extern NCBI_XCONNECT_EXPORT ESwitch SOCK_SetReadOnWriteAPI
  *
  * @return
  *  Prior setting
+ * @sa
+ *  SOCK_Read
  */
 extern NCBI_XCONNECT_EXPORT ESwitch SOCK_SetReadOnWrite
 (SOCK    sock,
@@ -1341,44 +1376,46 @@ extern NCBI_XCONNECT_EXPORT ESwitch SOCK_SetReadOnWrite
  );
 
 
-/** Control OS-defined send strategy by disabling/enabling TCP
- * layer to send incomplete network frames (packets).
- * With the "cork" set on, data gets always buffered until a complete
- * hardware packet is full (or connection is about to close), and only
- * then is sent out to the medium.
- * @note The setting cancels any effects of SOCK_DisableOSSendDelay().
+/** Control OS-defined send strategy by disabling/enabling the TCP layer to
+ * send incomplete network frames (packets).  With the "cork" set on, data gets
+ * always buffered until a complete hardware packet is full (or connection is
+ * about to close), and only then is sent out to the medium.
+ * @note The setting cancels the effects of SOCK_DisableOSSendDelay().
  * @param sock
  *  [in]  socket handle [stream socket only]
  * @param on_off
  *  [in]  1 to set the cork; 0 to remove the cork
  * @sa
- *  SOCK_DisableOSSendDelay
+ *  SOCK_Write, SOCK_DisableOSSendDelay
  */
 extern NCBI_XCONNECT_EXPORT void SOCK_SetCork
-(SOCK         sock,
- int/**bool*/ on_off
+(SOCK        sock,
+ int/*bool*/ on_off
  );
 
 
-/** Control OS-defined send strategy by disabling/enabling TCP
- * Nagle algorithm that packs multiple requests into a single
- * packet and thus transferring data in fewer transactions,
- * miminizing the network traffic and bursting the throughput.
- * Some applications, however, may find it useful to disable this
- * default behavior for the sake of their performance increase
- * (like in case of short transactions otherwise held by the system
- * to be possibly coalesced into larger chunks).
- * @note The setting cancels any effects of SOCK_SetCork().
+/** Control OS-defined send strategy by disabling/enabling the TCP Nagle
+ * algorithm (which is on by default) that packs multiple requests into a
+ * single packet and thus transferring data in fewer transactions, miminizing
+ * the network traffic and generally bursting the throughput.  However, some
+ * applications may find it useful to disable this default behavior for the
+ * sake of their performance increase (like in case of short transactions
+ * otherwise held off by the system to be possibly coalesced into larger
+ * chunks -- a typical example is an interactive transmission of keystrokes).
+ * Disabling the Nagle algorithm causes all internally pending yet
+ * untransmitted data to flush down to the hardware.
+ * @note The setting is overridden by SOCK_SetCork() but it still performs the
+ * flush, if set to disable.
  * @param sock
  *  [in]  socket handle [stream socket only]
  * @param on_off
  *  [in]  1 to disable the send delay; 0 to enable the send delay
  * @sa
- *  SOCK_SetCork
+ *  SOCK_Write, SOCK_SetCork
  */
 extern NCBI_XCONNECT_EXPORT void SOCK_DisableOSSendDelay
-(SOCK         sock,
- int/**bool*/ on_off
+(SOCK        sock,
+ int/*bool*/ on_off
  );
 
 
@@ -1505,6 +1542,8 @@ extern NCBI_XCONNECT_EXPORT EIO_Status DSOCK_WaitMsg
  *  [in]  additional data to send
  * @param datalen
  *  [in]  size of additional data (bytes)
+ * @sa
+ *  SOCK_Write, SOCK_SetTimeout
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status DSOCK_SendMsg
 (SOCK           sock,
@@ -1530,6 +1569,8 @@ extern NCBI_XCONNECT_EXPORT EIO_Status DSOCK_SendMsg
  *  [out] net byte order, may be NULL
  * @param sender_port
  *  [out] host byte order, may be NULL
+ * @sa
+ *  SOCK_Read, SOCK_SetTimeout
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status DSOCK_RecvMsg
 (SOCK            sock,
@@ -1561,8 +1602,8 @@ extern NCBI_XCONNECT_EXPORT EIO_Status DSOCK_WipeMsg
  *  [in]  set(1)/unset(0) broadcast capability
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status DSOCK_SetBroadcast
-(SOCK         sock,
- int/**bool*/ broadcast
+(SOCK        sock,
+ int/*bool*/ broadcast
  );
 
 
@@ -1593,7 +1634,7 @@ extern NCBI_XCONNECT_EXPORT TNCBI_BigCount DSOCK_GetMessageCount
  *  Non-zero value if socket "sock" was created by DSOCK_Create[Ex]().
  *  Return zero otherwise.
  */
-extern NCBI_XCONNECT_EXPORT int/**bool*/ SOCK_IsDatagram(SOCK sock);
+extern NCBI_XCONNECT_EXPORT int/*bool*/ SOCK_IsDatagram(SOCK sock);
 
 
 /** Check whether a socket is client-side.
@@ -1603,7 +1644,7 @@ extern NCBI_XCONNECT_EXPORT int/**bool*/ SOCK_IsDatagram(SOCK sock);
  *  Non-zero value if socket "sock" was created by SOCK_Create[Ex]().
  *  Return zero otherwise.
  */
-extern NCBI_XCONNECT_EXPORT int/**bool*/ SOCK_IsClientSide(SOCK sock);
+extern NCBI_XCONNECT_EXPORT int/*bool*/ SOCK_IsClientSide(SOCK sock);
 
 
 /** Check whether a socket is server-side.
@@ -1613,7 +1654,7 @@ extern NCBI_XCONNECT_EXPORT int/**bool*/ SOCK_IsClientSide(SOCK sock);
  *  Non-zero value if socket "sock" was created by LSOCK_Accept().
  *  Return zero otherwise.
  */
-extern NCBI_XCONNECT_EXPORT int/**bool*/ SOCK_IsServerSide(SOCK sock);
+extern NCBI_XCONNECT_EXPORT int/*bool*/ SOCK_IsServerSide(SOCK sock);
 
 
 /** Check whether a socket is UNIX type.
@@ -1623,7 +1664,7 @@ extern NCBI_XCONNECT_EXPORT int/**bool*/ SOCK_IsServerSide(SOCK sock);
  *  Non-zero value if socket "sock" is a UNIX family named socket
  *  Return zero otherwise.
  */
-extern NCBI_XCONNECT_EXPORT int/**bool*/ SOCK_IsUNIX(SOCK sock);
+extern NCBI_XCONNECT_EXPORT int/*bool*/ SOCK_IsUNIX(SOCK sock);
 
 
 /** Check whether a socket is using SSL (Secure Socket Layer).
@@ -1633,7 +1674,7 @@ extern NCBI_XCONNECT_EXPORT int/**bool*/ SOCK_IsUNIX(SOCK sock);
  *  Non-zero value if socket "sock" is using Secure Socket Layer (SSL).
  *  Return zero otherwise.
  */
-extern NCBI_XCONNECT_EXPORT int/**bool*/ SOCK_IsSecure(SOCK sock);
+extern NCBI_XCONNECT_EXPORT int/*bool*/ SOCK_IsSecure(SOCK sock);
 
 
 /** Get current read or write position within a socket.
@@ -1747,8 +1788,8 @@ extern NCBI_XCONNECT_EXPORT ESwitch SOCK_SetReuseAddressAPI
  *  SOCK_SetReuseAddressAPI, SOCK_Create, DSOCK_Create
  */
 extern NCBI_XCONNECT_EXPORT void SOCK_SetReuseAddress
-(SOCK         sock,
- int/**bool*/ on_off
+(SOCK        sock,
+ int/*bool*/ on_off
  );
 
 
@@ -1849,6 +1890,8 @@ typedef struct {
  *  none were and the specified time interval had elapsed, other error code
  *  for some other error condition (in which case the "revent" fields in the
  *  array may not have been updated with valid values).
+ * @sa
+ *   SOCK_Wait, SOCK_Poll
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status POLLABLE_Poll
 (size_t          n,
@@ -2005,7 +2048,7 @@ extern NCBI_XCONNECT_EXPORT int SOCK_gethostname
  *  (or local host, if hostname is passed as NULL), which could have been
  *  specified as either domain name or an IP address in the dotted notation
  *  (e.g. "123.45.67.89\0").  Return 0 on error.
- *  @note  "0.0.0.0" and "255.255.255.255" are both considered invalid.
+ * @note "0.0.0.0" and "255.255.255.255" are both considered invalid.
  * @sa
  *  SOCK_gethostbyname, SOCK_gethostname
  */
@@ -2024,19 +2067,19 @@ extern NCBI_XCONNECT_EXPORT unsigned int SOCK_gethostbyname
  );
 
 
-/** Take INET host address (in network byte order) and fill out the
- * the provided buffer with the name, which the address corresponds to
+/** Take INET host address (in network byte order) or 0 for current host, and
+ * fill out the provided buffer with the name, which the address corresponds to
  * (in case of multiple names the primary name is used).
  * @param addr
- *  [in]  host address in network byte order
+ *  [in]  host address (0 means current host) in network byte order
  * @param name
- *  [out] buffer to put the name to
+ *  [out] buffer to store the name to
  * @param namelen
  *  [in]  size (bytes) of the buffer above
  * @param log
  *  [in]  whether to log failures
  * @return
- *  Value 0 means error; success is denoted by the 'name' argument returned.
+ *  Value 0 means error;  success is denoted by the 'name' argument returned.
  *  Note that on error the name returned emptied (name[0] == '\0').
  * @sa
  *  SOCK_gethostbyaddr
@@ -2074,6 +2117,8 @@ extern NCBI_XCONNECT_EXPORT unsigned int SOCK_GetLoopbackAddress(void);
  *  eOff     not to recache even if unknown, return whatever is available.
  * @return
  *  Local address (in network byte order).
+ * @sa
+ *  SOCK_gethostname
  */
 extern NCBI_XCONNECT_EXPORT unsigned int SOCK_GetLocalHostAddress
 (ESwitch reget
@@ -2081,7 +2126,7 @@ extern NCBI_XCONNECT_EXPORT unsigned int SOCK_GetLocalHostAddress
 
 
 /** Read (skipping leading blanks) "[host][:port]" from a string stopping
- * at EOL or a blank character.
+ * at either EOL or a blank character.
  * @param str
  *  must not be NULL
  * @param host
@@ -2091,10 +2136,13 @@ extern NCBI_XCONNECT_EXPORT unsigned int SOCK_GetLocalHostAddress
  * @return
  *  On success, return the advanced pointer past the host/port read.
  *  If no host/port detected, return 'str'.  On format error, return 0.
- *  If host and/or port fragments are missing, then the corresponding 'host'/
+ *  If either host or port fragment is missing, then the corresponding 'host'/
  *  'port' parameters get a value of 0.
  * @note  'host' gets returned in network byte order, unlike 'port', which
  *        always comes out in host (native) byte order.
+ * @note  ":0" is accepted to denote no-host:zero-port.  
+ * @sa
+ *   SOCK_HostPortToString, SOCK_isip, SOCK_gethostbyname
  */
 extern NCBI_XCONNECT_EXPORT const char* SOCK_StringToHostPort
 (const char*     str,
@@ -2103,9 +2151,11 @@ extern NCBI_XCONNECT_EXPORT const char* SOCK_StringToHostPort
  );
 
 
-/** Print host:port into provided buffer string, not to exceed 'bufsize' bytes
- * (including the teminating '\0' character).
- * Suppress printing host if parameter 'host' is zero.
+/** Print numeric string "host:port" into a buffer provided, not to exceed
+ * 'bufsize' bytes (including the teminating '\0' character).  Suppress
+ * printing the "host" if the 'host' parameter is zero.  Suppress printing the
+ * ":port" part if 'port' passed as zero.  If both the 'host' and the 'port'
+ * parameters are zero, output is the literal ":0".
  * @param host
  *  in network byte order
  * @param port
@@ -2115,7 +2165,9 @@ extern NCBI_XCONNECT_EXPORT const char* SOCK_StringToHostPort
  * @param bufsize
  *  must be large enough
  * @return
- *  Number of bytes printed, or 0 on error (e.g. buffer too short).
+ *  Number of bytes printed, or 0 on error (e.g. buffer is too small).
+ * @sa
+ *  SOCK_StringToHostPort, SOCK_ntoa, SOCK_gethostbyaddr
  */
 extern NCBI_XCONNECT_EXPORT size_t SOCK_HostPortToString
 (unsigned int   host,
@@ -2147,16 +2199,26 @@ typedef SOCKSSL (*FSSLSetup)(void);
 
 /** Store SSL setup callback until actual initialization.
  * @param setup
- *  non-NULL SSL setup routine
+ *  non-NULL SSL setup routine, or NULL to shut the SSL down
+ * @warning
+ *  Do not use this function unless you know what you're doing.
+ *  Use other means of initialization such as CONNECT_Init() or CConnIniter.
+ * @sa
+ *  SOCK_SetupSSLEx, CONNECT_Init, CConnIniter
  */
 extern NCBI_XCONNECT_EXPORT void SOCK_SetupSSL(FSSLSetup setup);
 
 
-/** Take SSL setup callback and then init the entire API (SOCK + SSL).
+/** Take SSL setup callback, and then init the entire API (SOCK + SSL).
  * @param setup
- *  non-NULL SSL setup routine
+ *  non-NULL SSL setup routine, or NULL to shut the SSL down
  * @return
  *  eIO_Success if successful, other code on error.
+ * @warning
+ *  Do not use this function unless you know what you're doing.
+ *  Use other means of initialization such as CONNECT_Init() or CConnIniter.
+ * @sa
+ *  SOCK_SetupSSL, CONNECT_Init, CConnIniter
  */ 
 extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_SetupSSLEx(FSSLSetup setup);
 
@@ -2166,6 +2228,8 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_SetupSSLEx(FSSLSetup setup);
  *  NULL if SSL has not been properly set up.
  * @note
  *  "NONE" is returned as a name if SSL has been explicitly disabled.
+ * @sa
+ *   SOCK_SetupSSL, SOCK_SetupSSLEx
  */
 extern NCBI_XCONNECT_EXPORT const char* SOCK_SSLName(void);
 
