@@ -1,7 +1,7 @@
 #ifndef CONNECT___SERVER__HPP
 #define CONNECT___SERVER__HPP
 
-/* $Id: server.hpp 506707 2016-07-11 15:26:50Z satskyse $
+/* $Id: server.hpp 565441 2018-06-12 15:57:02Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -66,7 +66,8 @@ enum EServIO_Event {
     eServIO_ClientClose = 0x04,
     eServIO_OurClose    = 0x08,
     eServIO_Inactivity  = 0x10,
-    eServIO_Delete      = 0x20
+    eServIO_Delete      = 0x20,
+    eServIO_Alarm       = 0x40
 };
 
 
