@@ -1,7 +1,7 @@
 #ifndef CONNECT_SERVICES__COMPOUND_ID__HPP
 #define CONNECT_SERVICES__COMPOUND_ID__HPP
 
-/*  $Id: compound_id.hpp 431243 2014-04-02 15:31:07Z kazimird $
+/*  $Id: compound_id.hpp 458824 2015-02-10 17:42:48Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -51,6 +51,7 @@ enum ECompoundIDClass {
     eCIC_GenericID,
     eCIC_NetCacheBlobKey,
     eCIC_NetScheduleJobKey,
+    eCIC_NetStorageObjectLocV1, ///< Old version of NetStorageObjectLoc
     eCIC_NetStorageObjectLoc,
 
     eCIC_NumberOfClasses
@@ -303,7 +304,7 @@ class NCBI_XCONNECT_EXPORT CCompoundID
 ///
 class NCBI_XCONNECT_EXPORT CCompoundIDPool
 {
-    NCBI_NET_COMPONENT_WITH_DEFAULT_CTOR(CompoundIDPool);
+    NCBI_NET_COMPONENT_IMPL(CompoundIDPool);
 
     /// Construct a new pool of CompoundID objects.
     CCompoundIDPool();
