@@ -1,7 +1,7 @@
 #ifndef CONNECT_SERVICES_GRID__GRID_CLIENT__HPP
 #define CONNECT_SERVICES_GRID__GRID_CLIENT__HPP
 
-/*  $Id: grid_client.hpp 537172 2017-05-26 15:53:31Z sadyrovr $
+/*  $Id: grid_client.hpp 541328 2017-07-17 15:43:10Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -229,7 +229,7 @@ public:
 
     /// Submit a job to the queue
     ///
-    CNetScheduleAPI::EJobStatus SubmitAndWait(unsigned wait_time, pair<bool*, CNcbiOstream*> = {});
+    CNetScheduleAPI::EJobStatus SubmitAndWait(unsigned wait_time);
 
     /// Wait for job to finish its execution (done/failed/canceled).
     /// @sa CNetScheduleAPI
