@@ -1,7 +1,7 @@
 #ifndef CONN___NETCACHE_API__HPP
 #define CONN___NETCACHE_API__HPP
 
-/*  $Id: netcache_api.hpp 525579 2017-01-25 17:33:11Z sadyrovr $
+/*  $Id: netcache_api.hpp 541436 2017-07-18 15:58:57Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -590,16 +590,6 @@ class NCBI_XCONNECT_EXPORT CNetCacheAPI
     /// @see nc_cache_name
     typedef CNamedParameter<string, eNPT_CacheName> TCacheName;
 
-    /// Extract one of the servers comprising this service
-    /// as a separate CNetCacheAPI object.
-    /// This method is for use by the grid_cli utility only.
-    /// @internal
-    CNetCacheAPI GetServer(CNetServer::TInstance server);
-
-    /// This method is for use by the grid_cli utility only.
-    /// @internal
-    void SetEventHandler(INetEventHandler* event_handler);
-
     /// @internal
     CCompoundIDPool GetCompoundIDPool();
     /// @internal
