@@ -1,7 +1,7 @@
 #ifndef CONNECT_SERVICES___NETSERVICE_API__HPP
 #define CONNECT_SERVICES___NETSERVICE_API__HPP
 
-/*  $Id: netservice_api.hpp 514890 2016-09-26 19:05:36Z sadyrovr $
+/*  $Id: netservice_api.hpp 555600 2018-01-19 16:46:14Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,6 +37,8 @@
 #include "srv_connections.hpp"
 #include "json_over_uttp.hpp"
 
+#include <functional>
+
 BEGIN_NCBI_SCOPE
 
 struct SNetServiceIteratorImpl;
@@ -86,12 +88,6 @@ class NCBI_XCONNECT_EXPORT CNetService
 
     CNetServerPool GetServerPool();
 
-    enum EServiceType {
-        eServiceNotDefined,
-        eLoadBalancedService,
-        eSingleServerService
-    };
-
     enum EIterationMode {
         eSortByLoad,
         eRandomize,
@@ -112,7 +108,6 @@ class NCBI_XCONNECT_EXPORT CNetService
     CNetServiceIterator FindServer(INetServerFinder* finder,
         EIterationMode mode = eSortByLoad);
 
-    EServiceType GetServiceType() const;
     bool IsLoadBalanced() const;
 
     CNetServer::SExecResult FindServerAndExec(const string& cmd,
@@ -135,21 +130,17 @@ class NCBI_XCONNECT_EXPORT CNetService
 
     CNetService Clone(const string& name);
 
+    using TEventHandler = function<bool(const string&, CNetServer)>;
+
+    void SetErrorHandler(TEventHandler error_handler);
+    void SetWarningHandler(TEventHandler warning_handler);
+
 #ifdef NCBI_GRID_XSITE_CONN_SUPPORT
     static void AllowXSiteConnections();
     static bool IsUsingXSiteProxy();
 #endif
 };
 
-/// This class is for use by the grid_cli utility only.
-/// @internal
-class NCBI_XCONNECT_EXPORT INetEventHandler : public CObject
-{
-public:
-    virtual bool OnError(CException::TErrCode) { return false; }
-    virtual void OnWarning(const string&, CNetServer) {}
-};
-
 /// This class is for use by the grid_cli utility only.
 /// @internal
 class NCBI_XCONNECT_EXPORT IExecToJson
@@ -165,7 +156,6 @@ public:
 extern NCBI_XCONNECT_EXPORT
 CJsonNode g_ExecToJson(IExecToJson& exec_to_json,
         CNetService service,
-        CNetService::EServiceType service_type,
         CNetService::EIterationMode iteration_mode = CNetService::eSortByLoad);
 
 /// This function is for use by the grid_cli utility only.
@@ -174,6 +164,10 @@ extern NCBI_XCONNECT_EXPORT
 CNetService g_DiscoverService(const string& service_name,
         const string& client_name);
 
+/// @internal
+extern NCBI_XCONNECT_EXPORT
+void g_AppendClientIPSessionIDHitID(string& cmd, bool = false);
+
 END_NCBI_SCOPE
 
 #endif  /* CONNECT_SERVICES___NETSERVICE_API__HPP */
