@@ -1,7 +1,7 @@
 #ifndef CONNECT_SERVICES__NETSTORAGE__HPP
 #define CONNECT_SERVICES__NETSTORAGE__HPP
 
-/*  $Id: netstorage.hpp 505855 2016-06-29 16:17:14Z sadyrovr $
+/*  $Id: netstorage.hpp 530038 2017-03-09 20:41:02Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -155,13 +155,20 @@ typedef unsigned TNetStorageFlags;  ///< Bitwise OR of ENetStorageFlags
 /// Sequential I/O only
 /// Can switch between reading and writing but only explicitly, using Close()
 ///
+/// @warning Different I/O intefaces cannot be used at the same time.
+/// E.g. calls to Write(buf) and Write(string) cannot be mixed.
+/// Different IO can be used after calling Close().
+///
+/// @warning Once I/O started non-I/O methods cannot be called until Close() is called.
+/// E.g. calls to Write() and GetSize() cannot be mixed.
+///
 
 class NCBI_XCONNECT_EXPORT CNetStorageObject
 {
     NCBI_NET_COMPONENT(NetStorageObject);
 
     /// Return object locator
-    string GetLoc(void);
+    string GetLoc(void) const;
 
     /// Read no more than 'buf_size' bytes of the object contents
     /// (starting at the current position)
@@ -248,6 +255,8 @@ class NCBI_XCONNECT_EXPORT CNetStorageObject
     /// NetStorage object as well as storing data into this object.
     /// The returned iostream object must be deleted by the caller.
     ///
+    /// @warning Empty writes are ignored by the returned iostream instance!
+    ///
     CNcbiIostream* GetRWStream();
 
     /// Write string to the object (starting at the current position)
@@ -449,10 +458,6 @@ class NCBI_XCONNECT_EXPORT CNetStorage
     ///  File to remove
     ///
     ENetStorageRemoveResult Remove(const string& object_loc);
-
-    /// @deprecated To change flags use Relocate(object_loc, flags) instead.
-    NCBI_DEPRECATED
-    CNetStorageObject Open(const string& object_loc, TNetStorageFlags flags);
 };
 
 
