@@ -1,7 +1,7 @@
 #ifndef CONNECT_SERVICES__REMOTE_APP_MB_HPP
 #define CONNECT_SERVICES__REMOTE_APP_MB_HPP
 
-/*  $Id: remote_app.hpp 478906 2015-09-14 20:38:33Z sadyrovr $
+/*  $Id: remote_app.hpp 537183 2017-05-26 17:52:46Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,6 +33,7 @@
  */
 
 #include <connect/services/netcache_api.hpp>
+#include <connect/services/grid_rw_impl.hpp>
 
 #include <connect/connect_export.h>
 
@@ -75,9 +76,8 @@ protected:
 
 private:
     CNetCacheAPI m_Storage;
-    auto_ptr<CNcbiIstream> m_IStream;
-    auto_ptr<IEmbeddedStreamWriter> m_Writer;
-    auto_ptr<CNcbiOstream> m_OStream;
+    SGridRead m_GridRead;
+    SGridWrite m_GridWrite;
     string* m_Data;
     size_t* m_DataSize;
 };
