@@ -2,7 +2,7 @@
 #define CORELIB___EXPR__HPP
 
 
-/*  $Id: expr.hpp 395085 2013-04-09 16:23:01Z zakharov $
+/*  $Id: expr.hpp 455123 2014-12-23 13:38:22Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -242,7 +242,7 @@ public:
         const CException* prev_exception, EErrCode err_code,
         const string& message, int pos, 
         EDiagSev severity = eDiag_Error)
-    : CException(info, prev_exception, CException::eInvalid, message)
+    : CException(info, prev_exception, message, severity)
     , m_Pos(pos)
     NCBI_EXCEPTION_DEFAULT_IMPLEMENTATION(CExprParserException, CException);
 
