@@ -1,7 +1,7 @@
 #ifndef CORELIB___RWSTREAMBUF__HPP
 #define CORELIB___RWSTREAMBUF__HPP
 
-/*  $Id: rwstreambuf.hpp 530261 2017-03-13 17:05:56Z lavr $
+/*  $Id: rwstreambuf.hpp 564105 2018-05-21 16:48:08Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -29,12 +29,12 @@
  * Authors:  Anton Lavrentiev
  *
  * File Description:
- *   Reader-writer based stream buffer
+ *   Reader-writer-based stream buffer
  *
  */
 
 /// @file rwstreambuf.hpp
-/// Reader-writer based stream buffer
+/// Reader-writer-based stream buffer.
 /// @sa IReader, IWriter, IReaderWriter, CRStream, CWStream, CRWStream
 
 #include <corelib/ncbistre.hpp>
@@ -54,21 +54,22 @@
 BEGIN_NCBI_SCOPE
 
 
-/// Reader-writer based stream buffer
+/// Reader-writer-based stream buffer.
 
 class NCBI_XNCBI_EXPORT CRWStreambuf : public CRWStreambufBase
 {
 public:
-    /// Which of the objects (passed in the constructor) should be
-    /// deleted on this object's destruction, whether to tie I/O,
-    /// and how to process exceptions thrown at lower levels...
+    /// Which of the objects (passed in the constructor) must be deleted on
+    /// this object's destruction, whether to tie I/O, and how to process
+    /// exceptions thrown at lower levels...
     enum EFlags {
-        fOwnReader      = 1 << 1,    ///< Own the underlying reader
-        fOwnWriter      = 1 << 2,    ///< Own the underlying writer
+        fOwnReader      = 1 << 0,    ///< Own the underlying reader
+        fOwnWriter      = 1 << 1,    ///< Own the underlying writer
         fOwnAll         = fOwnReader + fOwnWriter,
-        fUntie          = 1 << 5,    ///< Do not flush before reading
-        fLogExceptions  = 1 << 8,    ///< Exceptions logged only
-        fLeakExceptions = 1 << 9     ///< Exceptions leaked out
+        fUntie          = 1 << 2,    ///< Do not flush before reading
+        fNoStatusLog    = 1 << 3,    ///< Do not log unsuccessful I/O
+        fLogExceptions  = 1 << 4,    ///< Exceptions logged only
+        fLeakExceptions = 1 << 5     ///< Exceptions leaked out
     };
     typedef int TFlags;              ///< Bitwise OR of EFlags
 
@@ -98,14 +99,27 @@ protected:
 
     virtual int         sync(void);
 
-    /// Note: setbuf(0, 0) has no effect
+    /// Per the standard, setbuf(0, 0) makes I/O unbuffered.  Other behavior is
+    /// implementation-dependent:
+    /// "buf_size" == 1 makes I/O unbuffered ("buf", if provided, may still be
+    /// used internally as a one-char un-get location).
+    /// Special case: setbuf(non-NULL, 0) creates an internal buffer of some
+    /// predefined size, which will be automatically deallocated in dtor;  the
+    /// value of the first argument is ignored (can be any non-NULL pointer).
+    /// Otherwise, setbuf() sets I/O arena of size "buf_size" located at "buf",
+    /// and halved between the I/O directions, if both are used.  If "buf" is
+    /// provided as NULL, the buffer of the requested size gets allocated
+    /// internally, and gets automatically freed upon CRWStreambuf destruction.
+    /// Before replacing the buffer, this call first attempts to flush any
+    /// pending output (sync) to the output device, and return any pending
+    /// input sequence (internal read buffer) to the input device (pushback).
     virtual CNcbiStreambuf* setbuf(CT_CHAR_TYPE* buf, streamsize buf_size);
 
     /// Only seekoff(0, IOS_BASE::cur, *) to obtain current position, and input
     /// skip-forward are permitted:
     /// seekoff(off, IOS_BASE::cur or IOS_BASE::beg, IOS_BASE::in) when the
     /// requested stream position is past the current input position (so the
-    //  stream can read forward internally to reach that position).
+    /// stream can read forward internally to reach that position).
     virtual CT_POS_TYPE seekoff(CT_OFF_TYPE off, IOS_BASE::seekdir whence,
                                 IOS_BASE::openmode which =
                                 IOS_BASE::in | IOS_BASE::out);
@@ -146,6 +160,7 @@ protected:
     CT_POS_TYPE       x_GPos;    ///< get position [for istream.tellg()]
     CT_POS_TYPE       x_PPos;    ///< put position [for ostream.tellp()]
 
+    bool              x_Eof;     ///< whether at EOF
     bool              x_Err;     ///< whether there was a _write_ error
     CT_POS_TYPE       x_ErrPos;  ///< position of the _write_ error (if x_Err)
 
