@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBI_PARAM__HPP
 #define CORELIB___NCBI_PARAM__HPP
 
-/*  $Id: ncbi_param.hpp 448556 2014-10-07 16:40:16Z ivanov $
+/*  $Id: ncbi_param.hpp 550022 2017-10-30 16:18:25Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -235,7 +235,7 @@ double NCBI_XNCBI_EXPORT g_GetConfigDouble(const char* section,
                   const TParamDesc& descr)                                  \
     { return CEnumParser< type, ptype >::EnumToString(val, descr); }
 
-// Defenition of SNcbiParamDesc_XXXX static members common for normal
+// Definition of SNcbiParamDesc_XXXX static members common for normal
 // and enum parameters.
 #define X_NCBI_PARAM_STATIC_DEF(descname, defval)                           \
     descname::TStaticValue descname::sm_Default = defval;                   \
@@ -268,6 +268,11 @@ double NCBI_XNCBI_EXPORT g_GetConfigDouble(const char* section,
 /// Enum parameter declaration. In addition to NCBI_PARAM_DECL also
 /// specializes CParamParser<type, ptype> to convert between strings and
 /// enum values.
+/// @attention 
+/// Only use this macro in the global or "ncbi" namespace, otherwise you will
+/// get compilation errors. Using this macro in the said namespaces will not 
+/// affect usability of a created CParamParser<type, ptype> specialization
+/// from other namespaces.
 /// @sa NCBI_PARAM_ENUM_ARRAY
 /// @sa NCBI_PARAM_ENUM_DEF
 #define NCBI_PARAM_ENUM_DECL(type, section, name)                             \
@@ -279,6 +284,11 @@ double NCBI_XNCBI_EXPORT g_GetConfigDouble(const char* section,
 
 
 /// Same as NCBI_PARAM_ENUM_DECL but with export specifier (e.g. NCBI_XNCBI_EXPORT)
+/// @attention 
+/// Only use this macro in the global or "ncbi" namespace, otherwise you will
+/// get compilation errors. Using this macro in the said namespaces will not 
+/// affect usability of a created CParamParser<type, ptype> specialization
+/// from other namespaces.
 /// @sa NCBI_PARAM_ENUM_DECL
 #define NCBI_PARAM_ENUM_DECL_EXPORT(expname, type, section, name)             \
     struct expname X_NCBI_PARAM_DECLNAME(section, name);                      \
@@ -288,8 +298,9 @@ double NCBI_XNCBI_EXPORT g_GetConfigDouble(const char* section,
         X_NCBI_PARAM_DECLNAME(section, name))
 
 
-/// Parameter definition. "value" is used to set the initial parameter
-/// value, which may be overriden by registry or environment.
+/// Parameter definition. "default_value" is used to set the initial parameter
+/// value, which may be overriden by registry or environment. The default value
+/// must be a scalar literal for the default value to be initialized statically.
 /// @sa NCBI_PARAM_DECL
 #define NCBI_PARAM_DEF(type, section, name, default_value)                  \
     SParamDescription< type >                                               \
@@ -376,6 +387,31 @@ double NCBI_XNCBI_EXPORT g_GetConfigDouble(const char* section,
         default_value)
 
 
+/// Define CSafeStatic_Proxy for a non-POD type using the provided POD type
+/// to store the default value.
+/// NOTE: The proxy specialization must be defined in 'ncbi' namespace.
+#define NCBI_PARAM_STATIC_PROXY(user_type, pod_type)                    \
+    template<> class CSafeStatic_Proxy<user_type> {                     \
+    public:                                                             \
+        typedef user_type TValue;                                       \
+        typedef pod_type TStaticInit;                                   \
+        CSafeStatic_Proxy(void) {}                                      \
+        CSafeStatic_Proxy(TStaticInit init) : m_Value(init) {}          \
+        CSafeStatic_Proxy(TValue value) : m_Value(value) {}             \
+        CSafeStatic_Proxy& operator=(TStaticInit init) {                \
+            m_Value = init;                                             \
+            return *this;                                               \
+        }                                                               \
+        CSafeStatic_Proxy& operator=(TValue value) {                    \
+            m_Value = value;                                            \
+            return *this;                                               \
+        }                                                               \
+        operator TValue& (void) { return m_Value; }                     \
+    private:                                                            \
+        mutable TValue m_Value;                                         \
+    }
+
+
 /////////////////////////////////////////////////////////////////////////////
 ///
 /// CParamException
@@ -448,7 +484,8 @@ public:
         eState_Func   = 2, ///< Initialized using FParamInit function
         eState_User   = 3, ///< Value has been set by user
         eState_EnvVar = 4, ///< The environment variable has been checked
-        eState_Config = 5  ///< The app. config file has been checked
+        eState_Config = 5, ///< The app. config file has been checked
+        eState_Error = 99  ///< Error reading param value, do not try to re-read
     };
 };
 
@@ -559,7 +596,7 @@ private:
     static bool sx_IsSetFlag(ENcbiParamFlags flag);
     static bool sx_CanGetDefault(void);
 
-    mutable bool       m_ValueSet;
+    mutable bool       m_ValueSet = false;
     mutable TValueType m_Value;
 };
 
