@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBI_URL__HPP
 #define CORELIB___NCBI_URL__HPP
 
-/*  $Id: ncbi_url.hpp 538305 2017-06-09 12:27:12Z ivanov $
+/*  $Id: ncbi_url.hpp 566197 2018-06-26 18:22:05Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -376,7 +376,7 @@ public:
     // Access parts of the URL
 
     string GetScheme(void) const            { return m_Scheme; }
-    void   SetScheme(const string& value)   { m_Scheme = value; }
+    void   SetScheme(const string& value);
 
     /// Generic schemes use '//' prefix (after optional scheme).
     bool GetIsGeneric(void) const           { return m_IsGeneric; }
@@ -441,8 +441,9 @@ public:
         fFragment_ReplaceIfEmpty = 0x0080, ///< Replace fragment only if not yet set
         fArgs_Replace            = 0x0100, ///< Discard all args, replace with args from 'other'
         fArgs_Append             = 0x0200, ///< Append args, allow duplicate names and values
-        fArgs_Merge              = 0x0400  ///< Append new args; replace values of existing args,
+        fArgs_Merge              = 0x0400, ///< Append new args; replace values of existing args,
                                            ///< do not allow to set multiple values with the same name
+        fScheme_Replace          = 0x0800  ///< Replace scheme if set in 'other'
     };
     typedef int TAdjustFlags;
 
@@ -555,6 +556,7 @@ void CUrl::SetService(const string& service)
 {
     m_Host.clear();
     m_Service = service;
+    m_IsGeneric = true; // services are always generic
 }
 
 inline
