@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBIDBG__HPP
 #define CORELIB___NCBIDBG__HPP
 
-/*  $Id: ncbidbg.hpp 491415 2016-02-04 17:54:53Z grichenk $
+/*  $Id: ncbidbg.hpp 572606 2018-10-17 12:14:35Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -320,12 +320,19 @@ private:
 
 #undef CHECKME_VALIDATE
 
-
-
 /////////////////////////////////////////////////////////////////////////////
 
 END_NCBI_SCOPE
 
+#ifdef HAVE_NCBI_SKEW_GUARD
+#include <common/ncbi_source_ver.cmake_nodep>
+#define     NAME_NCBI_LIBRARY_VERSION_MISMATCH(a)           Ncbi_library_version_mismatch__Full_rebuild_to_fix_##a
+#define MAKENAME_NCBI_LIBRARY_VERSION_MISMATCH(a)      NAME_NCBI_LIBRARY_VERSION_MISMATCH(a)
+#define    CHECK_NCBI_LIBRARY_VERSION_MISMATCH     MAKENAME_NCBI_LIBRARY_VERSION_MISMATCH(NCBI_DEVELOPMENT_VER)
+
+NCBI_XNCBI_EXPORT int CHECK_NCBI_LIBRARY_VERSION_MISMATCH(void);
+static int s__check_ncbi_library_version_mismatch = CHECK_NCBI_LIBRARY_VERSION_MISMATCH();
+#endif
 
 /* @} */
 
