@@ -1,7 +1,7 @@
 #ifndef NCBIEXPT__HPP
 #define NCBIEXPT__HPP
 
-/*  $Id: ncbiexpt.hpp 534857 2017-05-03 12:26:07Z ivanov $
+/*  $Id: ncbiexpt.hpp 568343 2018-08-02 18:37:52Z vakatov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -39,8 +39,7 @@
 /// the C++ exception specification.
 
 
-#include <corelib/ncbidiag.hpp>
-#include <corelib/ncbi_stack.hpp>
+#include <corelib/ncbimisc.hpp>
 #include <errno.h>
 #include <string.h>
 #include <typeinfo>
@@ -57,8 +56,10 @@
 
 BEGIN_NCBI_SCOPE
 
-#if (_MSC_VER >= 1200)
-#undef NCBI_USE_THROW_SPEC
+
+/// Do not use 'throw' dynamic exception specification for C++11 compilers 
+#if defined(NCBI_USE_THROW_SPEC)  &&  defined(NCBI_HAVE_CXX11)
+#  undef NCBI_USE_THROW_SPEC
 #endif
 
 /// Define THROWS macros for C++ exception specification.
@@ -628,7 +629,10 @@ public:
     typedef int      TFlags;
     typedef EDiagSev TSeverity;
 
-    CExceptionArgs_Base(void) : m_ErrCode(0), m_Flags(0), m_Severity(eDiag_Error) {}
+    CExceptionArgs_Base(void) :
+        m_ErrCode(0), m_Flags(0),
+        m_Severity(eDiag_Error), m_Retriable(eRetriable_Unknown)
+    {}
 
     void SetErrCodeVal(TErrCodeVal err_code) { m_ErrCode = err_code; }
     TErrCodeVal GetErrCodeVal(void) const { return m_ErrCode; }
@@ -643,11 +647,15 @@ public:
     const string& GetModule(void) const { return m_Module; }
     bool IsSetModule(void) const { return !m_Module.empty(); }
 
+    void SetRetriable(ERetriable  retriable) { m_Retriable = retriable; }
+    ERetriable GetRetriable(void) const { return m_Retriable; }
+
 private:
     TErrCodeVal m_ErrCode;
     TFlags      m_Flags;
     TSeverity   m_Severity;
     string      m_Module;
+    ERetriable  m_Retriable;
 };
 
 
@@ -706,6 +714,9 @@ public:
 /// defines additional generic error codes for applications, and error
 /// reporting capabilities.
 
+class CRequestContextRef;
+class CRequestContext;
+
 class NCBI_XNCBI_EXPORT CException : public std::exception
 {
 public:
@@ -888,6 +899,15 @@ public:
     /// Unset flag (other flags are left as is)
     CException& UnsetFlag(EFlags flag) { m_Flags &= ~flag; return *this; }
 
+    /// Get the request context in which the exception was thrown.
+    CRequestContext& GetRequestContext(void) const;
+
+    /// Set the info about ability to retry an action caused the exception
+    void       SetRetriable(ERetriable retriable)  { m_Retriable = retriable; }
+
+    /// Retrieve info about ability to retry an action caused the exception
+    virtual ERetriable GetRetriable(void)  const   { return m_Retriable; }
+
 protected:
     /// Constructor for derived classes
     CException(const CDiagCompileInfo& info,
@@ -957,6 +977,13 @@ private:
 
     TFlags       m_Flags;            ///< Flags, hints, attributes
 
+    ERetriable   m_Retriable;        ///< In some cases it is known for sure if
+                                     ///< an action caused the exception can be
+                                     ///< tried again or not. This member holds
+                                     ///< this information.
+
+    unique_ptr<CRequestContextRef> m_RequestContext;
+
     /// Private assignment operator to prohibit assignment.
     CException& operator= (const CException&);
 };
@@ -1048,6 +1075,24 @@ private:
 };
 
 
+/// Retriable manipulator
+
+class Retriable : public CExceptionArgsManip
+{
+public:
+    Retriable(ERetriable  retriable) : m_Retriable(retriable) {}
+    virtual ~Retriable(void) {}
+
+    virtual void operator()(CExceptionArgs_Base& args) const
+    {
+        args.SetRetriable(m_Retriable);
+    }
+
+private:
+    ERetriable  m_Retriable;
+};
+
+
 END_SCOPE(ncbi_ex_manip)
 
 
@@ -1419,7 +1464,7 @@ public:
     /// Copy constructor.
     CErrnoTemplExceptionEx(
         const CErrnoTemplExceptionEx<TBase, PErrCode, PErrStr>& other)
-        : TBase( other)
+        : TBase(other)
     {
         m_Errno = other.m_Errno;
         this->x_Assign(other);
@@ -1454,19 +1499,9 @@ public:
     int GetErrno(void) const throw() { return m_Errno; }
 
 protected:
-    /// Constructor.
-    CErrnoTemplExceptionEx(const CDiagCompileInfo& info,
-                           const CException* prev_exception,
-                           const string& message, 
-                           EDiagSev severity = eDiag_Error,
-                           CException::TFlags flags = 0)
-          : TBase(info, prev_exception, message, severity, flags)
-     {
-        m_Errno = PErrCode();
-        this->x_Init(info, message, prev_exception, severity);
-    }
-
-    /// Constructor.
+    /// Constructor (with errno only).
+    /// It is unable to get correct error code after all base constructors initialization,
+    /// that can reset it, so error code should be provided as parameter.
     CErrnoTemplExceptionEx(const CDiagCompileInfo& info,
                            const CException* prev_exception,
                            const string& message,
@@ -1477,9 +1512,10 @@ protected:
     {
         this->x_Init(info, message, prev_exception, severity);
     }
-    /// Constructor.
+    /// Default constructor.
     CErrnoTemplExceptionEx(void) { m_Errno = PErrCode(); }
 
+
     /// Helper clone method.
     virtual const CException* x_Clone(void) const
     {
@@ -1513,7 +1549,7 @@ public:
                                 typename CParent::EErrCode err_code,
                                 const string&              message,
                                 EDiagSev                   severity = eDiag_Error)
-        : CParent(info, prev_exception, message, severity, 0)
+        : CParent(info, prev_exception, message, NCBI_ERRNO_CODE_WRAPPER(), severity, 0)
     NCBI_EXCEPTION_DEFAULT_IMPLEMENTATION_TEMPL_ERRNO(CErrnoTemplException<TBase>, CParent)
 
 protected:
@@ -1522,7 +1558,7 @@ protected:
                                 const string&              message,
                                 EDiagSev                   severity,
                                 CException::TFlags         flags)
-        : CParent(info, prev_exception, message, severity, flags) {
+        : CParent(info, prev_exception, message, NCBI_ERRNO_CODE_WRAPPER(), severity, flags) {
     }
 };
 
@@ -1543,7 +1579,7 @@ public:
                                     typename CParent::EErrCode err_code,
                                     const string&              message,
                                     EDiagSev                   severity = eDiag_Error)
-        : CParent(info, prev_exception, message, severity, 0)
+        : CParent(info, prev_exception, message, NCBI_LASTERROR_CODE_WRAPPER(), severity, 0)
     NCBI_EXCEPTION_DEFAULT_IMPLEMENTATION_TEMPL_ERRNO(CErrnoTemplException_Win<TBase>, CParent)
 
 protected:
@@ -1552,7 +1588,7 @@ protected:
                                     const string&              message,
                                     EDiagSev                   severity,
                                     CException::TFlags         flags)
-        : CParent(info, prev_exception, message, severity, flags) {
+        : CParent(info, prev_exception, message, NCBI_LASTERROR_CODE_WRAPPER(), severity, flags) {
     }
 };
 #endif
