@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBIMISC__HPP
 #define CORELIB___NCBIMISC__HPP
 
-/*  $Id: ncbimisc.hpp 563123 2018-05-02 17:08:47Z lavr $
+/*  $Id: ncbimisc.hpp 580817 2019-02-21 14:01:24Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1025,6 +1025,10 @@ public:
 #define GI_TO(T, gi) (static_cast<T>(TIntId(gi)))
 #define GI_FROM(T, value) (TGi(static_cast<TIntId>(value)))
 
+/// Temporary macros to convert TEntrezId to other types (int, unsigned etc.).
+#define ENTREZ_ID_TO(T, entrez_id) (static_cast<T>(TIntId(entrez_id)))
+#define ENTREZ_ID_FROM(T, value) (TGi(static_cast<TIntId>(value)))
+
 /// Convert gi-compatible int to/from other types.
 #define INT_ID_TO(T, id) (static_cast<T>(id))
 #define INT_ID_FROM(T, value) (static_cast<TIntId>(value))
