@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBIMTX__HPP
 #define CORELIB___NCBIMTX__HPP
 
-/* $Id: ncbimtx.hpp 534857 2017-05-03 12:26:07Z ivanov $
+/* $Id: ncbimtx.hpp 567287 2018-07-17 13:02:49Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -70,6 +70,17 @@
 # pragma intrinsic(_ReadWriteBarrier)
 #endif
 
+#if defined(NCBI_WIN32_THREADS)
+#define NCBI_SRWLOCK_USE_NEW     1
+#define NCBI_SEMAPHORE_USE_NEW   1
+#endif
+
+#if NCBI_SRWLOCK_USE_NEW || NCBI_SEMAPHORE_USE_NEW
+# include <atomic>
+# include <mutex>
+# include <condition_variable>
+#endif
+
 
 /** @addtogroup Threads
  *
@@ -111,9 +122,13 @@ typedef pthread_mutex_t TSystemMutex;
 #elif defined(NCBI_WIN32_THREADS)
 
 #  define NCBI_USE_CRITICAL_SECTION
+#  define NCBI_FASTMUTEX_USE_NEW
+#  define NCBI_FASTRWLOCK_USE_NEW
 
+#  if defined(NCBI_FASTMUTEX_USE_NEW)
+typedef SRWLOCK TSystemMutex;
 /// Define a platform independent system mutex.
-#  if defined(NCBI_USE_CRITICAL_SECTION)
+#  elif defined(NCBI_USE_CRITICAL_SECTION)
 typedef CRITICAL_SECTION TSystemMutex;
 #  else
 typedef HANDLE TSystemMutex;
@@ -875,7 +890,9 @@ private:
 
 
 // Forward declaration of internal (platform-dependent) RW-lock representation
+#if !NCBI_SRWLOCK_USE_NEW
 class CInternalRWLock;
+#endif
 class CRWLock;
 //class CReadLockGuard;
 //class CWriteLockGuard;
@@ -1003,19 +1020,35 @@ public:
     void Unlock(void);
 
 private:
+#if NCBI_SRWLOCK_USE_NEW
+    mutex                       m_Mtx;
+    condition_variable          m_Cv;
+    TThreadSystemID             m_Owner;   ///< Writer ID
+    atomic<long>                m_Count;   ///< Number of readers (if >0) or writers (if <0)
+    long                        m_WaitingWriters; ///< Number of writers waiting; zero if not keeping track
+    vector<TThreadSystemID>     m_Readers; ///< List of all readers to detect recursion
+    bool                        m_TrackReaders;
+    bool                        m_FavorWriters;
+
+    bool x_HasReader(TThreadSystemID self_id);
+    vector<TThreadSystemID>::const_iterator x_FindReader(TThreadSystemID self_id);
+    bool x_MayAcquireForReading(TThreadSystemID self_id);
+    bool x_TryReadLock();
+    bool x_TryWriteLock();
+#else
     enum EInternalFlags {
         /// Keep track of which threads have read locks
         fTrackReaders = 0x40000000
     };
-
     TFlags                      m_Flags;   ///< Configuration flags
     unique_ptr<CInternalRWLock> m_RW;      ///< Platform-dependent RW-lock data
     volatile TThreadSystemID    m_Owner;   ///< Writer ID, one of the readers ID
     volatile long               m_Count;   ///< Number of readers (if >0) or writers (if <0)
     volatile unsigned int       m_WaitingWriters; ///< Number of writers waiting; zero if not keeping track
-    vector<TThreadSystemID>     m_Readers; ///< List of all readers or writers for debugging
+    vector<TThreadSystemID>     m_Readers; ///< List of all readers or writers
 
     bool x_MayAcquireForReading(TThreadSystemID self_id);
+#endif
 
     /// Private copy constructor to disallow initialization.
     CRWLock(const CRWLock&);
@@ -1126,11 +1159,15 @@ private:
         kWriteLockValue = 0x100000
     };
 
+#if defined(NCBI_WIN32_THREADS) && defined(NCBI_FASTRWLOCK_USE_NEW)
+    SRWLOCK m_Lock;
+#else
     /// Number of read locks acquired or value of kWriteLockValue if write
     /// lock was acquired
     CAtomicCounter  m_LockCount;
     /// Mutex implementing write lock
     CFastMutex      m_WriteLock;
+#endif
 };
 
 
@@ -1339,9 +1376,9 @@ class NCBI_XNCBI_EXPORT CSemaphore
 public:
     /// Constructor.
     ///
-    /// @param
-    ///   int_count   The initial value of the semaphore.
-    ///   max_count   Maximum value that semaphore value can be incremented to.
+    /// @param int_count   The initial value of the semaphore.
+    /// @param max_count   Maximum value that semaphore value can be incremented to.
+    ///                    NOTE: On Windows max_count can not be greater than LONG_MAX.
     CSemaphore(unsigned int init_count, unsigned int max_count);
 
     /// Destructor.
@@ -1371,7 +1408,15 @@ public:
     void Post(unsigned int count = 1);
 
 private:
+#if NCBI_SEMAPHORE_USE_NEW
+    mutex                 m_Mtx;
+    condition_variable    m_Cv;
+    const unsigned int    m_Max;
+    atomic<unsigned int>  m_Count;
+    bool x_TryAcquire(void);
+#else
     struct SSemaphore* m_Sem;  ///< System-specific semaphore data.
+#endif
 
     /// Private copy constructor to disallow initialization.
     CSemaphore(const CSemaphore&);
@@ -1388,7 +1433,7 @@ private:
 ///
 ///   Condition variable.
 
-#if defined(NCBI_POSIX_THREADS) || (defined(NCBI_WIN32_THREADS) && defined(NCBI_USE_CRITICAL_SECTION))
+#if defined(NCBI_POSIX_THREADS) || (defined(NCBI_WIN32_THREADS) && (defined(NCBI_USE_CRITICAL_SECTION) || defined(NCBI_FASTMUTEX_USE_NEW)))
 #  define NCBI_HAVE_CONDITIONAL_VARIABLE
 #endif
 
@@ -1417,7 +1462,7 @@ public:
     ///   SignalSome, SignalAll, CConditionVariableException
     bool WaitForSignal
     (CMutex&           mutex,
-     const CDeadline&  deadline = CDeadline(CTimeout::eInfinite));
+     const CDeadline&  deadline = CDeadline::eInfinite);
 
     /// Release mutex and lock the calling thread until the condition
     /// variable is signalled.
@@ -1435,7 +1480,7 @@ public:
     ///   SignalSome, SignalAll, CConditionVariableException
     bool WaitForSignal
     (CFastMutex&       mutex,
-     const CDeadline&  deadline = CDeadline(CTimeout::eInfinite));
+     const CDeadline&  deadline = CDeadline::eInfinite);
 
     /// Wake at least one of the threads that are currently waiting on this
     /// condition variable (if any threads are waiting on it).
