@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBISTRE__HPP
 #define CORELIB___NCBISTRE__HPP
 
-/*  $Id: ncbistre.hpp 535284 2017-05-08 13:25:16Z ucko $
+/*  $Id: ncbistre.hpp 562846 2018-04-28 17:39:52Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -90,10 +90,12 @@
 
 #  ifdef NO_PUBSYNC
 #    define PUBSYNC     sync
+#    define PUBSETBUF   setbuf
 #    define PUBSEEKOFF  seekoff
 #    define PUBSEEKPOS  seekpos
 #  else
 #    define PUBSYNC     pubsync
+#    define PUBSETBUF   pubsetbuf
 #    define PUBSEEKOFF  pubseekoff
 #    define PUBSEEKPOS  pubseekpos
 #  endif
