@@ -1,7 +1,7 @@
 #ifndef CORELIB__NCBITIME__HPP
 #define CORELIB__NCBITIME__HPP
 
-/*  $Id: ncbitime.hpp 481663 2015-10-14 17:54:55Z ivanov $
+/*  $Id: ncbitime.hpp 559006 2018-03-05 20:22:21Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1804,6 +1804,14 @@ public:
     /// Initialize deadline by adding relative timeout to the current time.
     CDeadline(const CTimeout& timeout);
 
+    /// Type of special deadlines.
+    enum EType {
+        eInfinite, ///< Infinite deadline.
+    };
+
+    /// Initialize deadline of specified type.
+    CDeadline(EType);
+
     /// Check if the deadline is infinite.
     bool IsInfinite(void) const { return m_Infinite; }
 
@@ -1822,6 +1830,8 @@ public:
     bool operator < (const CDeadline& right_hand_operand) const;
 
 private:
+    CDeadline(CTimeout::EType);
+
     void x_Now(void);
     void x_Add(unsigned int seconds, unsigned int nanoseconds);
 
