@@ -1,7 +1,7 @@
 #ifndef CORELIB___REQUEST_CTX__HPP
 #define CORELIB___REQUEST_CTX__HPP
 
-/*  $Id: request_ctx.hpp 511451 2016-08-22 16:11:29Z grichenk $
+/*  $Id: request_ctx.hpp 563431 2018-05-09 14:33:19Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -115,6 +115,10 @@ private:
 };
 
 
+class CMask;
+class CMaskFileName;
+
+
 class NCBI_XNCBI_EXPORT CRequestContext : public CObject
 {
 public:
@@ -315,6 +319,24 @@ public:
     /// commas and optional spaces, ignore UNK_SESSION value.
     static string SelectLastSessionID(const string& session_ids);
 
+    /// Add pass-through value if it matches a pattern from NCBI_CONTEXT_FIELDS.
+    void AddPassThroughProperty(const string& name, const string& value);
+
+    /// Switch request context to read-only mode. A read-only context
+    /// can be attached to multiple threads. The mode must be disabled
+    /// before making any modifications (e.g. printing request-stop).
+    /// To avoid overhead CRequestContext does not check if there are
+    /// any threads currently using the same context in a different mode.
+    /// Any attempts to modify request context while in read-only mode
+    /// are ignored and reported as an error (once per process).
+    /// Note that some const methods may still need to modify the context
+    /// (e.g. GetHitID() generates a new hit id if it's not yet assigned).
+    /// To prevent the above error from being logged, the method should be
+    /// called at least once before switching to read-only mode.
+    void SetReadOnly(bool read_only) { m_IsReadOnly = read_only; }
+    /// Get current read-only flag.
+    bool GetReadOnly(void) const { return m_IsReadOnly; }
+
 private:
     // Prohibit copying
     CRequestContext(const CRequestContext&);
@@ -372,13 +394,29 @@ private:
     // Copy std properties from pass-through data to CRequestContext.
     void x_UpdateStdContextProp(CTempString name) const;
 
+    static const CMask& sx_GetContextFieldsMask(void);
+    static string sx_NormalizeContextPropertyName(const string& name);
+
+    // Load environment values matching NCBI_CONTEXT_FIELDS.
+    void x_LoadEnvContextProperties(void);
+
+    friend class CDiagBuffer;
+    bool x_LogHitIDOnError(void) const;
+
+    bool x_CanModify(void) const;
+        
+    enum FLoggedHitIDFlag {
+        fLoggedOnRequest = 1, // Logged on creation or request start
+        fLoggedOnError = 2    // Logged on ERR_POST when applog messages are disabled
+    };
+
     TCount         m_RequestID;
     EDiagAppState  m_AppState;
     string         m_ClientIP;
     CEncodedString m_SessionID;
     CSharedHitId   m_HitID;
     string         m_Dtab;
-    mutable bool   m_LoggedHitID;
+    mutable int    m_HitIDLoggedFlag;
     int            m_ReqStatus;
     CStopWatch     m_ReqTimer;
     Int8           m_BytesRd;
@@ -394,11 +432,21 @@ private:
     friend class CDiagContextThreadData;
     // TID of the thread currently using this context or -1.
     Uint8          m_OwnerTID;
+    bool           m_IsReadOnly;
+
+    // Name/value map for properties to be passed between requests.
+    // @sa CRequestContext_PassThrough
+    typedef map<string, string, PNocase> TPassThroughProperties;
 
     // Access to passable properties.
     friend class CRequestContext_PassThrough;
-    typedef map<string, string, PNocase> TPassThroughProperties;
     mutable TPassThroughProperties m_PassThroughProperties;
+
+    // Patterns from NCBI_CONTEXT_FIELDS variable.
+    static unique_ptr<CMaskFileName> sm_ContextFields;
+
+    // Context values loaded from the global environment.
+    static unique_ptr<TPassThroughProperties> sm_EnvContextProperties;
 };
 
 
@@ -457,6 +505,52 @@ private:
 };
 
 
+/// Take guard of the current CRequestContext, handle app-state, start/stop
+/// logging and request status in the dtor.
+class NCBI_XNCBI_EXPORT CRequestContextGuard_Base
+{
+public:
+    enum EFlags {
+        fPrintRequestStart = 1 << 0 ///< Print request-start automatically in the
+                                    ///< constructor. By default request-start is
+                                    ///< not printed to allow the caller log request
+                                    ///< arguments.
+    };
+    typedef int TFlags;
+
+    /// Initialize guard.
+    /// @param context
+    ///  Request context to be used. If null, re-use current context.
+    /// @param flags
+    ///  Optional flags, @sa FFlags.
+    CRequestContextGuard_Base(CRequestContext* context, TFlags flags = 0);
+
+    /// Destroy guard. If released do nothing. On exception set error status.
+    /// Print request-stop. Restore previous context.
+    ~CRequestContextGuard_Base(void);
+
+    /// Set request context status.
+    void SetStatus(int status) { m_RequestContext->SetRequestStatus(status); }
+
+    /// Set default error status, which will be used if an uncaught exception
+    /// is detected.
+    void SetDefaultErrorStatus(int status);
+
+    /// Get the guarded request context.
+    CRequestContext& GetRequestContext() const { return *m_RequestContext; }
+
+    /// Release the guarded context, restore the saved context if any, do not
+    /// perform any other actions (logging, setting status).
+    void Release(void);
+
+private:
+    TFlags                        m_Flags = 0;
+    int                           m_ErrorStatus = 500;
+    CRef<CRequestContext>         m_SavedContext;
+    mutable CRef<CRequestContext> m_RequestContext;
+};
+
+
 class NCBI_XNCBI_EXPORT CRequestContextException :
     EXCEPTION_VIRTUAL_BASE public CException
 {
@@ -486,6 +580,7 @@ CRequestContext::TCount CRequestContext::GetRequestID(void) const
 inline
 void CRequestContext::SetRequestID(TCount rid)
 {
+    if (!x_CanModify()) return;
     x_SetProp(eProp_RequestID);
     m_RequestID = rid;
 }
@@ -493,6 +588,7 @@ void CRequestContext::SetRequestID(TCount rid)
 inline
 CRequestContext::TCount CRequestContext::SetRequestID(void)
 {
+    if (!x_CanModify()) return m_RequestID;
     SetRequestID(GetNextRequestID());
     return m_RequestID;
 }
@@ -506,6 +602,7 @@ bool CRequestContext::IsSetRequestID(void) const
 inline
 void CRequestContext::UnsetRequestID(void)
 {
+    if (!x_CanModify()) return;
     x_UnsetProp(eProp_RequestID);
     m_RequestID = 0;
 }
@@ -526,6 +623,7 @@ bool CRequestContext::IsSetClientIP(void) const
 inline
 void CRequestContext::UnsetClientIP(void)
 {
+    if (!x_CanModify()) return;
     x_UnsetProp(eProp_ClientIP);
     m_ClientIP.clear();
 }
@@ -534,9 +632,12 @@ void CRequestContext::UnsetClientIP(void)
 inline
 string CRequestContext::GetSessionID(void) const
 {
-    return x_IsSetProp(eProp_SessionID) ?
-        m_SessionID.GetOriginalString()
-        : GetDiagContext().GetDefaultSessionID();
+    if ( x_IsSetProp(eProp_SessionID) ) {
+        return m_SessionID.GetOriginalString();
+    }
+    string def_sid = GetDiagContext().GetDefaultSessionID();
+    if ( !def_sid.empty() ) return def_sid;
+    return const_cast<CRequestContext*>(this)->SetSessionID();
 }
 
 inline
@@ -563,6 +664,7 @@ bool CRequestContext::IsSetExplicitSessionID(void) const
 inline
 void CRequestContext::UnsetSessionID(void)
 {
+    if (!x_CanModify()) return;
     x_UnsetProp(eProp_SessionID);
     m_SessionID.SetString(kEmptyStr);
 }
@@ -588,9 +690,10 @@ bool CRequestContext::IsSetHitID(EHitIDSource src) const
 inline
 void CRequestContext::UnsetHitID(void)
 {
+    if (!x_CanModify()) return;
     x_UnsetProp(eProp_HitID);
     m_HitID.SetHitId(kEmptyStr);
-    m_LoggedHitID = false;
+    m_HitIDLoggedFlag = 0;
     m_SubHitIDCache.clear();
 }
 
@@ -598,6 +701,7 @@ void CRequestContext::UnsetHitID(void)
 inline
 const string& CRequestContext::GetNextSubHitID(CTempString prefix)
 {
+    if (!x_CanModify()) return m_SubHitIDCache;
     x_UpdateSubHitID(true, prefix);
     return m_SubHitIDCache;
 }
@@ -628,6 +732,7 @@ const string& CRequestContext::GetDtab(void) const
 inline
 void CRequestContext::SetDtab(const string& dtab)
 {
+    if (!x_CanModify()) return;
     x_SetProp(eProp_Dtab);
     m_Dtab = dtab;
 }
@@ -636,6 +741,7 @@ void CRequestContext::SetDtab(const string& dtab)
 inline
 void CRequestContext::UnsetDtab(void)
 {
+    if (!x_CanModify()) return;
     x_UnsetProp(eProp_Dtab);
 }
 
@@ -649,6 +755,7 @@ int CRequestContext::GetRequestStatus(void) const
 inline
 void CRequestContext::SetRequestStatus(int status)
 {
+    if (!x_CanModify()) return;
     x_SetProp(eProp_ReqStatus);
     m_ReqStatus = status;
 }
@@ -656,6 +763,7 @@ void CRequestContext::SetRequestStatus(int status)
 inline
 void CRequestContext::SetRequestStatus(CRequestStatus::ECode code)
 {
+    if (!x_CanModify()) return;
     SetRequestStatus((int)code);
 }
 
@@ -668,6 +776,7 @@ bool CRequestContext::IsSetRequestStatus(void) const
 inline
 void CRequestContext::UnsetRequestStatus(void)
 {
+    if (!x_CanModify()) return;
     x_UnsetProp(eProp_ReqStatus);
     m_ReqStatus = 0;
 }
@@ -682,6 +791,7 @@ Int8 CRequestContext::GetBytesRd(void) const
 inline
 void CRequestContext::SetBytesRd(Int8 bytes)
 {
+    if (!x_CanModify()) return;
     x_SetProp(eProp_BytesRd);
     m_BytesRd = bytes;
 }
@@ -695,6 +805,7 @@ bool CRequestContext::IsSetBytesRd(void) const
 inline
 void CRequestContext::UnsetBytesRd(void)
 {
+    if (!x_CanModify()) return;
     x_UnsetProp(eProp_BytesRd);
     m_BytesRd = 0;
 }
@@ -709,6 +820,7 @@ Int8 CRequestContext::GetBytesWr(void) const
 inline
 void CRequestContext::SetBytesWr(Int8 bytes)
 {
+    if (!x_CanModify()) return;
     x_SetProp(eProp_BytesWr);
     m_BytesWr = bytes;
 }
@@ -722,6 +834,7 @@ bool CRequestContext::IsSetBytesWr(void) const
 inline
 void CRequestContext::UnsetBytesWr(void)
 {
+    if (!x_CanModify()) return;
     x_UnsetProp(eProp_BytesWr);
     m_BytesWr = 0;
 }
@@ -747,6 +860,28 @@ void CRequestContext::x_UnsetProp(EProperty prop)
 }
 
 
+inline
+bool CRequestContext::x_LogHitIDOnError(void) const
+{
+    if (m_HitIDLoggedFlag & fLoggedOnError || m_HitID.Empty()) {
+        return false;
+    }
+    m_HitIDLoggedFlag |= fLoggedOnError;
+    return true;
+}
+
+
+inline
+bool CRequestContext::x_CanModify(void) const
+{
+    if ( m_IsReadOnly ) {
+        ERR_POST_ONCE("Attempt to modify a read-only request context.");
+        return false;
+    }
+    return true;
+}
+
+
 inline
 CRequestContext_PassThrough::CRequestContext_PassThrough(void)
 {
