@@ -1,7 +1,7 @@
 #ifndef CORELIB___TEST_BOOST__HPP
 #define CORELIB___TEST_BOOST__HPP
 
-/*  $Id: test_boost.hpp 534440 2017-04-27 12:05:00Z ivanov $
+/*  $Id: test_boost.hpp 570863 2018-09-17 14:00:14Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -45,6 +45,8 @@
 #include <corelib/expr.hpp>
 #include <corelib/ncbiargs.hpp>
 #include <corelib/ncbiapp.hpp>
+#include <corelib/request_ctx.hpp>
+#include <list>
 
 
 // Keep Boost's inclusion of <limits> from breaking under old WorkShop versions.
@@ -215,6 +217,11 @@ struct test_name : public F { void test_method(); };                    \
                                                                         \
 static void BOOST_AUTO_TC_INVOKER( test_name )()                        \
 {                                                                       \
+    NCBI_NS_NCBI::CDiagContext& dctx = NCBI_NS_NCBI::GetDiagContext();  \
+    NCBI_NS_NCBI::CRequestContext& rctx = dctx.GetRequestContext();     \
+    rctx.SetRequestID();                                                \
+    NCBI_NS_NCBI::CRequestContextGuard_Base rg(&rctx);                  \
+    dctx.PrintRequestStart().Print("test_name", #test_name);            \
     BOOST_TEST_CHECKPOINT('"' << #test_name << "\" fixture entry.");    \
     test_name t;                                                        \
     BOOST_TEST_CHECKPOINT('"' << #test_name << "\" entry.");            \
@@ -226,8 +233,11 @@ static void BOOST_AUTO_TC_INVOKER( test_name )()                        \
                  << boost::unit_test                                    \
                          ::framework::current_test_case().p_name        \
                  << "\"" << ex);                                        \
+        char* msg = strdup(ex.what());                                  \
+        NCBI_NS_NCBI::CNcbiTestMemoryCleanupList::GetInstance()->Add(msg); \
         throw boost::execution_exception(                               \
-                boost::execution_exception::cpp_exception_error, ""     \
+                boost::execution_exception::cpp_exception_error,        \
+                msg                                                     \
                 NCBI_BOOST_LOCATION() );                                \
     }                                                                   \
     BOOST_TEST_CHECKPOINT('"' << #test_name << "\" exit.");             \
@@ -252,6 +262,11 @@ struct test_name : public F { void test_method(); };                    \
                                                                         \
 static void BOOST_AUTO_TC_INVOKER( test_name )()                        \
 {                                                                       \
+    NCBI_NS_NCBI::CDiagContext& dctx = NCBI_NS_NCBI::GetDiagContext();  \
+    NCBI_NS_NCBI::CRequestContext& rctx = dctx.GetRequestContext();     \
+    rctx.SetRequestID();                                                \
+    NCBI_NS_NCBI::CRequestContextGuard_Base rg(&rctx);                  \
+    dctx.PrintRequestStart().Print("test_name", #test_name);            \
     test_name t;                                                        \
     try {                                                               \
         t.test_method();                                                \
@@ -261,8 +276,11 @@ static void BOOST_AUTO_TC_INVOKER( test_name )()                        \
                  << boost::unit_test                                    \
                          ::framework::current_test_case().p_name        \
                  << "\"" << ex);                                        \
+        char* msg = strdup(ex.what());                                  \
+        NCBI_NS_NCBI::CNcbiTestMemoryCleanupList::GetInstance()->Add(msg); \
         throw boost::execution_exception(                               \
-                boost::execution_exception::cpp_exception_error, ""     \
+                boost::execution_exception::cpp_exception_error,        \
+                msg                                                     \
                 NCBI_BOOST_LOCATION() );                                \
     }                                                                   \
 }                                                                       \
@@ -920,6 +938,23 @@ NcbiTestGenTestCases(void (*test_func)(ParamType),
 }
 
 
+/////////////////////////////////////////////////////////////////////////////
+///
+/// CNcbiTestMemoryCleanupList -- Define a list of pointers to free at exit.
+///
+/// Not really necessary for tests, but used to avoid memory leaks and 
+/// corresponding reports by memory checkers like Valgrind.
+
+class CNcbiTestMemoryCleanupList
+{
+public:
+    static CNcbiTestMemoryCleanupList* GetInstance();
+    ~CNcbiTestMemoryCleanupList();
+    void Add(void* ptr);
+private:
+    std::list<void*> m_List;
+};
+
 
 END_NCBI_SCOPE
 
