@@ -1,6 +1,6 @@
 #ifndef DB_SQLITE_SQLITEWRAPP__HPP
 #define DB_SQLITE_SQLITEWRAPP__HPP
-/*  $Id: sqlitewrapp.hpp 532370 2017-04-04 17:33:28Z grichenk $
+/*  $Id: sqlitewrapp.hpp 566371 2018-06-28 14:31:38Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -70,11 +70,6 @@ public:
     /// Method can be called only before SQLite is initialized with
     /// Initialize() method.
     static void SetCustomPageCache(sqlite3_pcache_methods* methods);
-    /// Install non-default memory management for SQLite. For more details on
-    /// sqlite3_mem_methods look into comments before it in sqlite3.h.
-    /// Method can be called only before SQLite is initialized with
-    /// Initialize() method.
-    static void SetCustomMallocFuncs(sqlite3_mem_methods* methods);
 
     /// Initialization of SQLite and tuning some default parameters.
     /// For these SQLite wrappers to work properly this method should be
@@ -841,7 +836,7 @@ CSQLITE_Connection::SetPageSize(unsigned int size)
 inline unsigned int
 CSQLITE_Connection::GetCacheSize(void)
 {
-    return m_CacheSize;
+    return m_CacheSize == (unsigned int)(-1) ? 0 : m_CacheSize;
 }
 
 inline void
