@@ -1,7 +1,7 @@
 #ifndef DBAPI_CONN_FACTORY_HPP
 #define DBAPI_CONN_FACTORY_HPP
 
-/*  $Id: dbapi_conn_factory.hpp 548536 2017-10-16 15:16:58Z ivanov $
+/*  $Id: dbapi_conn_factory.hpp 553345 2017-12-15 17:11:22Z satskyse $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -171,9 +171,20 @@ private:
         impl::CDBHandlerStack       handlers;
         list<string>                tried;
         TSvrRef                     last_tried;
+        CStopWatch                  make_valid_connection_sw;
+        CStopWatch                  dispatch_server_name_sw;
         // params deliberately left out (potentially call-specific)
     };
 
+    struct CStopWatchScope {
+        CStopWatchScope(CStopWatch&  sw_) : sw(sw_)
+        { sw.Restart(); }
+        ~CStopWatchScope()
+        { sw.Stop(); }
+
+        CStopWatch& sw;
+    };
+
     // Methods
     CDB_Connection* DispatchServerName(SOpeningContext& ctx,
                                        const CDBConnParams& params);
