@@ -1,7 +1,7 @@
 #ifndef DBAPI_DRIVER___DRIVERS__HPP
 #define DBAPI_DRIVER___DRIVERS__HPP
 
-/* $Id: drivers.hpp 487476 2015-12-17 19:48:39Z ucko $
+/* $Id: drivers.hpp 565395 2018-06-11 18:48:16Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -43,15 +43,17 @@ BEGIN_NCBI_SCOPE
 
 // New and recommended.
 extern void DBAPI_RegisterDriver_CTLIB      (void);
-extern void DBAPI_RegisterDriver_DBLIB      (void);
 extern void DBAPI_RegisterDriver_FTDS       (void);
-extern void DBAPI_RegisterDriver_FTDS64     (void);
 extern void DBAPI_RegisterDriver_FTDS95     (void);
+extern void DBAPI_RegisterDriver_FTDS100    (void);
 extern void DBAPI_RegisterDriver_ODBC       (void);
 extern void DBAPI_RegisterDriver_MYSQL      (void);
 // Development ...
 // extern void DBAPI_RegisterDriver_GATEWAY (void);
 
+NCBI_DEPRECATED
+extern void DBAPI_RegisterDriver_FTDS64     (void);
+
 END_NCBI_SCOPE
 
 
