@@ -1,7 +1,7 @@
 #ifndef DBAPI_DRIVER___PUBLIC__HPP
 #define DBAPI_DRIVER___PUBLIC__HPP
 
-/* $Id: public.hpp 532493 2017-04-05 17:37:13Z ucko $
+/* $Id: public.hpp 565697 2018-06-15 14:21:31Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,6 +35,14 @@
 #include <corelib/plugin_manager.hpp>
 #include <dbapi/driver/interfaces.hpp>
 
+
+// Visual Studio 2013 and 2015 overreact on deprecated methods
+#if defined(_MSC_VER) && _MSC_VER < 1910
+    #pragma warning(push)
+    #pragma warning(disable : 4996)
+#endif
+
+
 /** @addtogroup DbPubInterfaces
  *
  * @{
@@ -291,19 +299,34 @@ public:
 
     /// @brief 
     ///   Set connection timeout.    
-    /// 
+    ///
+    /// This timeout limits how long each query or the like can take.
     /// @param nof_secs 
     ///   Number of seconds.  If "nof_secs" is zero or is "too big" 
     ///   (depends on the underlying DB API), then set the timeout to infinite.
+    /// @sa GetTimeout, SetCancelTimeout, I_DriverContext::SetTimeout, I_DriverContext::SetLoginTimeout
     virtual void SetTimeout(size_t nof_secs);
 
     /// Set timeout for command cancellation and connection closing
+    ///
+    /// This timeout limits how long to wait for the server to acknowledge
+    /// requests to cancel a query gracefully or close a connection gracefully
+    /// before pulling the plug.
+    /// @sa GetCancelTimeout, SetTimeout, I_DriverContext::SetTimeout, I_DriverContext::SetLoginTimeout
     virtual void SetCancelTimeout(size_t nof_secs);
 
     /// Get connection timeout.    
+    ///
+    /// This timeout limits how long each query or the like can take.
+    /// @sa SetTimeout, GetCancelTimeout, I_DriverContext::GetTimeout, I_DriverContext::GetLoginTimeout
     size_t GetTimeout(void) const;
 
     /// Get timeout for command cancellation and connection closing
+    ///
+    /// This timeout limits how long to wait for the server to acknowledge
+    /// requests to cancel a query gracefully or close a connection gracefully
+    /// before pulling the plug.
+    /// @sa SetCancelTimeout, GetTimeout, I_DriverContext::GetTimeout, I_DriverContext::GetLoginTimeout
     size_t GetCancelTimeout(void) const;
     
     /// Get interface for extra features that could be implemented in the driver.
@@ -501,15 +524,20 @@ public:
     virtual size_t ReadItem(void* buffer, size_t buffer_size,
                             bool* is_null = 0);
 
-    /// @brief 
+    /// @brief
     ///   Get a descriptor for a BLOB column (for SendData).
-    /// 
-    /// @return 
+    ///
+    /// @return
     ///   Return NULL if this result doesn't (or can't) have a BLOB descriptor.
     ///
     /// @note
     ///   You need to call ReadItem (maybe even with buffer_size == 0)
     ///   before calling this method!
+    /// @deprecated
+    ///   Please use CDB_BlobDescriptor instead.
+    /// @sa
+    ///   CDB_BlobDescriptor
+    NCBI_DEPRECATED
     virtual I_BlobDescriptor* GetBlobDescriptor();
 
     /// @brief 
@@ -811,7 +839,7 @@ private:
 class NCBI_DBAPIDRIVER_EXPORT CDB_CursorCmd : public I_CursorCmd
 {
 public:
-    /// Get meta-information about parameters. 
+    /// Get meta-information about parameters.
     virtual CDBParams& GetBindParams(void);
     virtual CDBParams& GetDefineParams(void);
 
@@ -821,7 +849,7 @@ public:
         GetBindParams().Bind(name, value);
         return true;
     }
-        
+
     /// Open the cursor.
     /// Return NULL if cursor resulted in no data.
     /// Throw exception on error.
@@ -830,24 +858,35 @@ public:
     /// Update the last fetched row.
     /// NOTE: the cursor must be declared for update in CDB_Connection::Cursor()
     virtual bool Update(const string& table_name, const string& upd_query);
+
+    /// @deprecated
+    ///   Please use CDB_Connection::SendData instead.
+    /// @sa
+    ///   CDB_Connection::SendData
+    NCBI_DEPRECATED
     virtual bool UpdateBlob(unsigned int item_num, CDB_Stream& data,
                             bool log_it = true);
     /// Make "send-data" command
-    /// @brief 
+    /// @brief
     ///   Create send-data command.
-    /// 
-    /// @param item_num 
+    ///
+    /// @param item_num
     ///   Column number to rewrite.
-    /// @param size 
+    /// @param size
     ///   Maximal data size.
-    /// @param log_it 
+    /// @param log_it
     ///   Log LOB operation if this value is set to true.
     /// @param discard_results
     ///   Discard all resultsets that might be returned from server
     ///   if this value is set to true.
-    /// 
-    /// @return 
+    ///
+    /// @return
     ///   Newly created send-data object.
+    /// @deprecated
+    ///   Please use CDB_Connection::SendDataCmd instead.
+    /// @sa
+    ///   CDB_Connection::SendDataCmd
+    NCBI_DEPRECATED
     virtual CDB_SendDataCmd* SendDataCmd(unsigned int item_num, size_t size,
                                          bool log_it = true,
                                          bool discard_results = true);
@@ -1075,5 +1114,9 @@ END_NCBI_SCOPE
 
 /* @} */
 
+#if defined(_MSC_VER) && _MSC_VER < 1910
+    #pragma warning(pop)
+#endif
+
 
 #endif  /* DBAPI_DRIVER___PUBLIC__HPP */
