@@ -1,7 +1,7 @@
 #ifndef NCBI_TAXON1_HPP
 #define NCBI_TAXON1_HPP
 
-/* $Id: taxon1.hpp 532449 2017-04-05 14:12:46Z domrach $
+/* $Id: taxon1.hpp 558464 2018-02-27 16:49:23Z domrach $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -109,6 +109,7 @@ public:
     ///
     bool GetScientificName(TTaxId tax_id, string& name_out);
 
+    typedef unsigned TOrgRefStatus;
     //----------------------------------------------
     // Get organism data by OrgRef
     // Returns: pointer to Taxon2Data if organism exists
@@ -116,13 +117,14 @@ public:
     //
     // NOTE:
     // 1. These functions uses the following data from inp_orgRef to find
-    //    organism in taxonomy database. It uses taxname first. If no organism
-    //    was found (or multiple nodes found) then it tries to find organism
-    //    using common name. If nothing found, then it tries to find organism
-    //    using synonyms. Lookup never uses tax_id to find organism.
+    //    organism in taxonomy database. It uses old-name modifier from orgname.
+    //    If no organism was found (or multiple nodes found) then it tries to find organism
+    //    using taxname. If nothing found, then it tries to find organism
+    //    using synonyms. Lookup never uses tax_id to find organism (except when multiple
+    //    nodes found by name then resulting node is chosen by tax_id).
     // 2. LookupMerge function modifies given OrgRef to correspond to the
     //    found one and returns constant pointer to the Taxon2Data structure
-    //    stored internally.
+    //    stored internally. It also fills the org-ref status output parameter when provided.
     // 3. If non-null pointer psLog specified, then lookup log string is returned.
     //    The latter has following format: each operation record begins with <
     //    and ends with >, fields inside record are delimited with |, first field
@@ -132,7 +134,7 @@ public:
     //    fifth field is comment text.
     ///
     CRef< CTaxon2_data > Lookup(const COrg_ref& inp_orgRef, string* psLog = 0);
-    CConstRef< CTaxon2_data > LookupMerge(COrg_ref& inp_orgRef, string* psLog = 0);
+    CConstRef< CTaxon2_data > LookupMerge(COrg_ref& inp_orgRef, string* psLog = 0, TOrgRefStatus* pStatusOut = 0);
 
     //-----------------------------------------------
     // Get tax_id by OrgRef
@@ -147,28 +149,33 @@ public:
     TTaxId GetTaxIdByOrgRef(const COrg_ref& inp_orgRef);
 
     enum EOrgRefStatus {
-        eStatus_Ok = 0,
-        eStatus_WrongTaxId      = 0x0001,
-        eStatus_WrongGC         = 0x0002,
-        eStatus_WrongMGC        = 0x0004,
-        eStatus_NoOrgname       = 0x0008,
-        eStatus_WrongTaxname    = 0x0010,
-        eStatus_WrongLineage    = 0x0020,
-        eStatus_WrongCommonName = 0x0040,
-        eStatus_WrongOrgname    = 0x0080,
-        eStatus_WrongDivision   = 0x0100,
-        eStatus_WrongOrgmod     = 0x0200,
-        eStatus_WrongPGC        = 0x0400,
-        eStatus_WrongOrgrefMod  = 0x0800,
-        eStatus_WrongOrgnameAttr= 0x1000
+        eStatus_Ok                   = COrg_ref::eOrgref_nothing,
+        eStatus_WrongTaxId           = COrg_ref::eOrgref_db_taxid,
+        eStatus_NoOrgname            = COrg_ref::eOrgref_orgname,
+        eStatus_WrongTaxname         = COrg_ref::eOrgref_taxname,
+        eStatus_WrongOrgrefMod       = COrg_ref::eOrgref_mod,
+        eStatus_WrongCommonName      = COrg_ref::eOrgref_common,
+        eStatus_WrongOrgname         = COrg_ref::eOrgref_on_name,
+        eStatus_WrongOrgmod          = COrg_ref::eOrgref_on_mod,
+        eStatus_WrongLineage         = COrg_ref::eOrgref_on_lin,
+        eStatus_WrongDivision        = COrg_ref::eOrgref_on_div,
+        eStatus_WrongGC              = COrg_ref::eOrgref_on_gc,
+        eStatus_WrongMGC             = COrg_ref::eOrgref_on_mgc,
+        eStatus_WrongPGC             = COrg_ref::eOrgref_on_pgc,
+        eStatus_WrongOrgnameAttr     = COrg_ref::eOrgref_on_attr,
+	eStatus_WrongONASpecified    = COrg_ref::eOrgref_on_attr_spec,
+	eStatus_WrongONANoModFwd     = COrg_ref::eOrgref_on_attr_nofwd,
+	eStatus_WrongONAUncultured   = COrg_ref::eOrgref_on_attr_uncult,
+	eStatus_WrongNomenclature    = COrg_ref::eOrgref_on_mod_nom,
+	eStatus_WrongModOldname      = COrg_ref::eOrgref_on_mod_oldname,
+	eStatus_WrongModTypeMaterial = COrg_ref::eOrgref_on_mod_tm
     };
-    typedef unsigned TOrgRefStatus;
     //-----------------------------------------------
     // Checks whether OrgRef is current
     // Returns: false on any error, stat_out filled with status flags
     // (see above)
     ///
-    bool CheckOrgRef( const COrg_ref& orgRef, TOrgRefStatus& stat_out );
+    bool CheckOrgRef( const COrg_ref& orgRef, TOrgRefStatus& stat_out, string* psLog = 0 );
 
     enum ESearch {
         eSearch_Exact,
@@ -341,11 +348,25 @@ public:
     ///
     bool GetRankName(TTaxRank rank_id, string& rank_name_out );
 
+    //---------------------------------------------
+    // Get taxonomic rank id by rank name
+    // Returns: rank id
+    //          -2 - in case of error
+    ///
+    TTaxRank GetRankIdByName(const string& rank_name);
+
     //---------------------------------------------
     // Get taxonomic division name by division id
     ///
     bool GetDivisionName(TTaxDivision div_id, string& div_name_out, string* div_code_out = NULL );
 
+    //---------------------------------------------
+    // Get taxonomic division id by division name (or code)
+    // Returns: rank id
+    //          -1 - in case of error
+    ///
+    TTaxDivision GetDivisionIdByName(const string& div_name);
+
     //---------------------------------------------
     // Get taxonomic name class name by name class id
     ///
