@@ -1,7 +1,7 @@
 #ifndef OBJECTS_MISC_SERIAL___GRID_RPC_CLIENT__HPP
 #define OBJECTS_MISC_SERIAL___GRID_RPC_CLIENT__HPP
 
-/* $Id: gridrpcclient.hpp 514371 2016-09-21 15:22:54Z ivanov $
+/* $Id: gridrpcclient.hpp 526845 2017-02-07 18:20:56Z meric $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -316,6 +316,7 @@ public:
 
     pair<CNetScheduleJob, bool> AskStream(CNcbiIstream& request, CNcbiOstream& reply) const
     {
+        CPerfLogGuard pl("CGridRPCBaseClient::AskStream");
         CGridClient grid_cli(m_NS_api.GetSubmitter(),
                              m_NC_api,
                              CGridClient::eManualCleanup,
@@ -370,12 +371,14 @@ public:
             }
             throw e;
         }
+        pl.Post(CRequestStatus::e200_Ok);
         return make_pair(job, timed_out);
     }
 
     template <class TRequest, class TReply>
     pair<CNetScheduleJob, bool> Ask(const TRequest& request, TReply& reply) const
     {
+        CPerfLogGuard pl("CGridRPCBaseClient::Ask");
         CGridClient grid_cli(m_NS_api.GetSubmitter(),
                              m_NC_api,
                              CGridClient::eManualCleanup,
@@ -432,6 +435,7 @@ public:
             }
             throw e;
         }
+        pl.Post(CRequestStatus::e200_Ok);
         return make_pair(job, timed_out);
     }
 
@@ -450,6 +454,7 @@ protected:
     template <class TReply>
     CNetScheduleJob x_GetJobById(const string job_id, TReply& reply) const
     {
+        CPerfLogGuard pl("CGridRPCBaseClient::x_GetJobById");
         CNetScheduleJob job;
         job.job_id = job_id;
 
@@ -478,6 +483,7 @@ protected:
                        "Unexpected status: " + CNetScheduleAPI::StatusToString(evt)
                       );
         }
+        pl.Post(CRequestStatus::e200_Ok);
         return job;
     }
 
