@@ -1,7 +1,7 @@
 #ifndef BIOSEQ_HANDLE__HPP
 #define BIOSEQ_HANDLE__HPP
 
-/*  $Id: bioseq_handle.hpp 496211 2016-03-24 15:33:11Z vasilche $
+/*  $Id: bioseq_handle.hpp 576406 2018-12-14 15:28:32Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -518,6 +518,7 @@ public:
     TDescr& SetDescr(void) const;
     bool AddSeqdesc(CSeqdesc& d) const;
     CRef<CSeqdesc> RemoveSeqdesc(const CSeqdesc& d) const;
+    CRef<CSeqdesc> ReplaceSeqdesc(const CSeqdesc& old_desc, CSeqdesc& new_desc) const;
     void AddSeq_descr(TDescr& v) const;
     // inst
     void SetInst(TInst& v) const;
@@ -648,6 +649,7 @@ public:
     void x_RealSetDescr(TDescr& v) const;
     bool x_RealAddSeqdesc(CSeqdesc& d) const;
     CRef<CSeqdesc> x_RealRemoveSeqdesc(const CSeqdesc& d) const;
+    CRef<CSeqdesc> x_RealReplaceSeqdesc(const CSeqdesc& old_desc, CSeqdesc& new_desc) const;
     void x_RealAddSeq_descr(TDescr& v) const;
 
     void x_RealResetId(void) const;
