@@ -1,7 +1,7 @@
 #ifndef OBJMGR_GC_ASSEMBLY_PARSER__HPP
 #define OBJMGR_GC_ASSEMBLY_PARSER__HPP
 
-/*  $Id: gc_assembly_parser.hpp 479439 2015-09-21 13:04:11Z grichenk $
+/*  $Id: gc_assembly_parser.hpp 578929 2019-01-28 18:18:01Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -127,7 +127,7 @@ public:
         eUnsupported,    ///< Unsupported type/flag.
         eOtherError
     };
-    virtual const char* GetErrCodeString(void) const;
+    virtual const char* GetErrCodeString(void) const override;
     NCBI_EXCEPTION_DEFAULT(CAssemblyParserException, CException);
 };
 
