@@ -1,7 +1,7 @@
 #ifndef ANNOT_COLLECTOR__HPP
 #define ANNOT_COLLECTOR__HPP
 
-/*  $Id: annot_collector.hpp 538958 2017-06-15 17:46:01Z ivanov $
+/*  $Id: annot_collector.hpp 569392 2018-08-21 15:46:08Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -50,6 +50,7 @@
 #include <set>
 #include <vector>
 #include <memory>
+#include <utility>
 
 BEGIN_NCBI_SCOPE
 
@@ -72,6 +73,7 @@ class CMappedFeat;
 class CAnnot_CI;
 class CSeqMap_CI;
 class CGraphRanges;
+class CIdRangeMap;
 
 class NCBI_XOBJMGR_EXPORT CAnnotMapping_Info
 {
@@ -92,7 +94,8 @@ public:
         eMappedObjType_Seq_feat,
         eMappedObjType_Seq_align,
         eMappedObjType_Seq_loc_Conv,
-        eMappedObjType_Seq_loc_Conv_Set
+        eMappedObjType_Seq_loc_Conv_Set,
+        eMappedObjType_IdRangeMap // id/range map used for sorting
     };
 
     CAnnotMapping_Info(void);
@@ -162,6 +165,9 @@ public:
 
     void SetAnnotObjectRange(const TRange& range, bool product);
     
+    void SetIdRangeMap(CIdRangeMap& id_range_map);
+    const CIdRangeMap& GetIdRangeMap(void) const;
+
     void Swap(CAnnotMapping_Info& info);
 
 private:
@@ -473,6 +479,8 @@ private:
     bool x_MatchLocIndex(const SAnnotObject_Index& index) const;
 
     bool x_NoMoreObjects(void) const;
+    
+    bool x_FoundAllNamedAnnotAccessions(unique_ptr<SAnnotSelector>& local_sel);
 
     void x_AddPostMappings(void);
     void x_AddPostMappingsCvt(CSeq_loc_Conversion_Set& cvt);
@@ -637,7 +645,8 @@ bool CAnnotMapping_Info::IsMapped(void) const
 {
     _ASSERT((GetMappedObjectType() == eMappedObjType_not_set) ==
             !m_MappedObject);
-    return GetMappedObjectType() != eMappedObjType_not_set;
+    return GetMappedObjectType() != eMappedObjType_not_set  &&
+        GetMappedObjectType() != eMappedObjType_IdRangeMap;
 }
 
 
