@@ -1,7 +1,7 @@
 #ifndef OBJECTS_OBJMGR_IMPL___BIOSEQ_SET_INFO__HPP
 #define OBJECTS_OBJMGR_IMPL___BIOSEQ_SET_INFO__HPP
 
-/*  $Id: bioseq_set_info.hpp 521281 2016-12-07 16:40:59Z vasilche $
+/*  $Id: bioseq_set_info.hpp 569712 2018-08-28 11:45:26Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -201,6 +201,9 @@ protected:
 
     static CRef<TObject> sx_ShallowCopy(const TObject& obj);
 
+    typedef pair<int, TChunkId> TChunkSeqSetsKey; // nucleotides first
+    void x_SetChunkBioseqs2(const list< CRef<CBioseq> >& bioseqs, TChunkSeqSetsKey key);
+    
 private:
     // core object
     CRef<TObject>       m_Object;
@@ -214,7 +217,7 @@ private:
         size_t count;
         CBioseq_set::TSeq_set::iterator first_iter;
     };
-    typedef map<TChunkId, SChunkSeqSet> TChunkSeqSets;
+    typedef map<TChunkSeqSetsKey, SChunkSeqSet> TChunkSeqSets;
     TChunkSeqSets m_ChunkSeqSets;
 
     //
