@@ -1,7 +1,7 @@
 #ifndef SCOPE_INFO__HPP
 #define SCOPE_INFO__HPP
 
-/*  $Id: scope_info.hpp 534487 2017-04-27 17:21:02Z vasilche $
+/*  $Id: scope_info.hpp 569391 2018-08-21 15:45:28Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -231,8 +231,8 @@ public:
     typedef CMutex TTSE_LockSetMutex;
     const TTSE_LockSet& GetTSE_LockSet(void) const;
     TTSE_LockSetMutex& GetTSE_LockSetMutex(void) const;
-    void UpdateTSELock(CTSE_ScopeInfo& tse, CTSE_Lock lock);
-    void ReleaseTSELock(CTSE_ScopeInfo& tse); // into queue
+    void ReleaseTSEUserLock(CTSE_ScopeInfo& tse); // into queue
+    void AcquireTSEUserLock(CTSE_ScopeInfo& tse); // from queue
     void ForgetTSELock(CTSE_ScopeInfo& tse); // completely
     void RemoveFromHistory(CTSE_ScopeInfo& tse, bool drop_from_ds = false);
     bool TSEIsInQueue(const CTSE_ScopeInfo& tse) const;
@@ -343,7 +343,7 @@ private:
 };
 
     
-class NCBI_XOBJMGR_EXPORT CTSE_ScopeInfo : public CTSE_ScopeInfo_Base
+class NCBI_XOBJMGR_EXPORT CTSE_ScopeInfo : public CObject
 {
 public:
     typedef CBlobIdKey                                    TBlobId;
@@ -371,9 +371,9 @@ public:
     bool CanBeEdited(void) const;
 
     // True if the TSE is referenced
-    bool IsLocked(void) const;
+    bool IsUserLocked(void) const;
     // True if the TSE is referenced by more than one handle
-    bool LockedMoreThanOnce(void) const;
+    bool IsUserLockedMoreThanOnce(void) const;
 
     bool ContainsBioseq(const CSeq_id_Handle& id) const;
     // returns matching Seq-id handle
@@ -446,9 +446,17 @@ protected:
     void x_ResetTSE_Lock(void);
     void x_DetachDS(void);
 
+    friend class CTSE_ScopeInternalLocker;
+    friend class CTSE_ScopeUserLocker;
+    
     // Number of internal locks, not related to handles
     int x_GetDSLocksCount(void) const;
 
+    void x_InternalLockTSE(void);
+    void x_InternalUnlockTSE(void);
+    void x_UserLockTSE(void);
+    void x_UserUnlockTSE(void);
+    
     CRef<CBioseq_ScopeInfo> x_FindBioseqInfo(const TSeqIds& ids) const;
     CRef<CBioseq_ScopeInfo> x_CreateBioseqInfo(const TSeqIds& ids);
     void x_IndexBioseq(const CSeq_id_Handle& id,
@@ -482,6 +490,8 @@ private: // members
     TBioseqById                 m_BioseqById;
     // TSE locking support
     mutable CMutex              m_TSE_LockMutex;
+    mutable CAtomicCounter_WithAutoInit m_TSE_LockCounter;
+    mutable CAtomicCounter_WithAutoInit m_UserLockCounter;
     mutable CTSE_Lock           m_TSE_Lock;
     // Used by TSE support
     mutable const CTSE_ScopeInfo*   m_UsedByTSE;
