@@ -1,7 +1,7 @@
 #ifndef PREFETCH_MANAGER__HPP
 #define PREFETCH_MANAGER__HPP
 
-/*  $Id: prefetch_manager.hpp 347369 2011-12-16 14:16:32Z vasilche $
+/*  $Id: prefetch_manager.hpp 578929 2019-01-28 18:18:01Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -140,7 +140,7 @@ public:
     enum EErrCode {
         eFailed
     };
-    virtual const char* GetErrCodeString(void) const;
+    virtual const char* GetErrCodeString(void) const override;
     NCBI_EXCEPTION_DEFAULT(CPrefetchFailed,CException);
 };
 
@@ -152,7 +152,7 @@ public:
     enum EErrCode {
         eCanceled
     };
-    virtual const char* GetErrCodeString(void) const;
+    virtual const char* GetErrCodeString(void) const override;
     NCBI_EXCEPTION_DEFAULT(CPrefetchCanceled,CException);
 };
 
