@@ -1,7 +1,7 @@
 #ifndef OBJMGR_SCOPE__HPP
 #define OBJMGR_SCOPE__HPP
 
-/*  $Id: scope.hpp 554030 2017-12-27 16:11:11Z gouriano $
+/*  $Id: scope.hpp 576250 2018-12-12 13:23:09Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -68,6 +68,7 @@ class CSeq_entry;
 class CBioseq_set;
 class CBioseq;
 class CSeq_annot;
+class CSeq_submit;
 class CSeq_id;
 class CSeq_loc;
 
@@ -316,6 +317,11 @@ public:
                                    TPriority pri = kPriority_Default,
                                    EExist action = eExist_Throw);
 
+    /// Add Seq-submit, return its CSeq_entry_Handle.
+    /// Add object to the score with possibility to edit it directly.
+    CSeq_entry_Handle AddSeq_submit(CSeq_submit& submit,
+                                    TPriority pri = kPriority_Default);
+
     /// Get editable Biosec handle by regular one
     CBioseq_EditHandle     GetEditHandle(const CBioseq_Handle&     seq);
 
