@@ -1,7 +1,7 @@
 #ifndef SEQ_ENTRY_HANDLE__HPP
 #define SEQ_ENTRY_HANDLE__HPP
 
-/*  $Id: seq_entry_handle.hpp 194592 2010-06-15 18:54:05Z vasilche $
+/*  $Id: seq_entry_handle.hpp 576406 2018-12-14 15:28:32Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -70,6 +70,8 @@ class CTSE_Info;
 
 class CSeqdesc;
 class CBioObjectId;
+class CSeq_submit;
+class CSubmit_block;
 
 
 class CSeq_entry_ScopeInfo : public CScopeInfo_Base
@@ -129,9 +131,17 @@ public:
     /// has more than one subentry exception is thrown
     CSeq_entry_Handle GetSingleSubEntry(void) const;
 
+    /// Check if this handle is top-level entry
+    bool IsTopLevelEntry(void) const;
+    
     /// Get top level Seq-entry handle
     CSeq_entry_Handle GetTopLevelEntry(void) const;
 
+    /// Seq-submit access functions
+    bool IsTopLevelSeq_submit(void) const;
+    const CSeq_submit& GetTopLevelSeq_submit(void) const;
+    const CSubmit_block& GetTopLevelSubmit_block(void) const;
+
     /// Get Bioseq handle from the TSE of this Seq-entry
     CBioseq_Handle GetBioseqHandle(const CSeq_id& id) const;
     CBioseq_Handle GetBioseqHandle(const CSeq_id_Handle& id) const;
@@ -255,6 +265,10 @@ public:
     /// Get parent seq-entry edit handle
     CSeq_entry_EditHandle GetParentEntry(void) const;
 
+    /// Seq-submit support
+    CSubmit_block& SetTopLevelSubmit_block(void) const;
+    void SetTopLevelSubmit_block(CSubmit_block& sub) const;
+
     /// Get edit handle of the sub seq-entry
     /// If current seq-entry is not seq-set or 
     /// has more than one subentry exception is thrown
@@ -266,6 +280,7 @@ public:
     void ResetDescr(void) const;
     bool AddSeqdesc(CSeqdesc& v) const;
     CRef<CSeqdesc> RemoveSeqdesc(const CSeqdesc& v) const;
+    CRef<CSeqdesc> ReplaceSeqdesc(const CSeqdesc& old_desc, CSeqdesc& new_desc) const;
 
     void AddDescr(TDescr& v) const;
 
@@ -566,6 +581,7 @@ public:
     void x_RealResetDescr(void) const;
     bool x_RealAddSeqdesc(CSeqdesc& v) const;
     CRef<CSeqdesc> x_RealRemoveSeqdesc(const CSeqdesc& v) const;
+    CRef<CSeqdesc> x_RealReplaceSeqdesc(const CSeqdesc& old_desc, CSeqdesc& new_desc) const;
     void x_RealAddSeq_descr(TDescr& v) const;
 
 };
