@@ -1,7 +1,7 @@
 #ifndef OBJMGR_UTIL___AUTODEF_MOD_COMBO__HPP
 #define OBJMGR_UTIL___AUTODEF_MOD_COMBO__HPP
 
-/*  $Id: autodef_mod_combo.hpp 572646 2018-10-17 16:58:07Z ivanov $
+/*  $Id: autodef_mod_combo.hpp 575407 2018-11-28 17:52:52Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -77,7 +77,7 @@ public:
     bool HasSubSource(CSubSource::ESubtype st);
     bool HasOrgMod(COrgMod::ESubtype st);
     
-    void AddSource(const CBioSource& bs, string feature_clauses = "");
+    void AddSource(const CBioSource& bs, const string& feature_clauses = "");
     
     void AddSubsource(CSubSource::ESubtype st, bool even_if_not_uniquifying = false);
     void AddOrgMod(COrgMod::ESubtype st, bool even_if_not_uniquifying = false);
@@ -114,7 +114,7 @@ public:
 
     typedef vector<CSubSource::ESubtype> TSubSourceTypeVector;
     typedef vector<COrgMod::ESubtype> TOrgModTypeVector;
-    typedef vector<CAutoDefSourceGroup *> TGroupListVector;
+    typedef vector<CRef<CAutoDefSourceGroup> > TGroupListVector;
 
     const TGroupListVector& GetGroupList() const { return m_GroupList; }
     const CAutoDefSourceDescription::TModifierVector& GetModifiers() const { return m_Modifiers; }
