@@ -105,7 +105,8 @@ public:
         fHideIntronFeats =  32,
         fHideMiscFeats =    64,
         fShowSNPFeats =    128,
-        fShowCDDFeats =    256
+        fShowCDDFeats =    256,
+        fGeneRNACDSOnly =  512
     };
     typedef int TFlags; // Binary "OR" of EFlags
 
