@@ -1,7 +1,7 @@
 #ifndef SEQUENCE__HPP
 #define SEQUENCE__HPP
 
-/*  $Id: sequence.hpp 567127 2018-07-13 17:55:39Z kans $
+/*  $Id: sequence.hpp 578929 2019-01-28 18:18:01Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -729,7 +729,7 @@ public:
 
     // Translate the specific error code into a string representations of
     // that error code.
-    virtual const char* GetErrCodeString(void) const;
+    virtual const char* GetErrCodeString(void) const override;
 
     NCBI_EXCEPTION_DEFAULT(CSeqIdFromHandleException, CException);
 };
