@@ -1,4 +1,4 @@
-/*  $Id: align_format_util.hpp 568473 2018-08-06 16:28:49Z zaretska $
+/*  $Id: align_format_util.hpp 577747 2019-01-08 18:06:34Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1070,6 +1070,18 @@ public:
                                ILinkoutDB* linkoutdb,
                                const string& mv_build_name);
 
+    static CRef<objects::CSeq_align_set> 
+        SortSeqalignForSortableFormat(objects::CSeq_align_set& aln_set,
+                               bool nuc_to_nuc_translation,
+                               int hit_order,
+                               int hsp_order);
+
+    static list< CRef<objects::CSeq_align_set> >
+        SortOneSeqalignForSortableFormat(const objects::CSeq_align_set& source,                                                                                                 
+                                      bool nuc_to_nuc_translation,                                             
+                                      int hit_sort,
+                                      int hsp_sort);
+
 	/// function for calculating  percent match for an alignment.	
 	///@param numerator
 	/// int numerator in percent identity calculation.
