@@ -1,4 +1,4 @@
-/*  $Id: showdefline.hpp 553793 2017-12-21 16:51:53Z zaretska $
+/*  $Id: showdefline.hpp 577751 2019-01-08 18:07:37Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -95,7 +95,9 @@ public:
         eShowNewSeqGif = (1 << 8),      //show new sequence gif image
         eShowPercentIdent = (1 << 9),   //show percent identity column
         eLongSeqId = (1 << 10),         //print long sequences id (with bars)
-        eRealtedInfoLinks = (1 << 11)           //Related information links - Linkout links with descriptions + others
+        eRealtedInfoLinks = (1 << 11),  //Related information links - Linkout links with descriptions + others
+        eShowTotalScore = (1 << 12),    //show total score column
+        eShowQueryCoverage = (1 << 13)  //show query coverage column
     };
 
     ///Data Representing each defline
@@ -127,6 +129,30 @@ public:
         bool   advancedView;
     };
 
+    //Defline info for formatting 
+    struct SDeflineFormattingInfo {
+        string dfln_url;            
+        string dfln_rid;
+        string dfln_gi;            
+        string dfln_seqid;
+        string full_dfln_defline;
+        string dfln_defline;
+        string dfln_id;
+        string dflnFrm_id;
+        string dflnFASTA_id;
+        string dflnAccs;
+            
+        string score_info;            
+        string dfln_hspnum; 
+        string dfln_alnLen;
+        string dfln_blast_rank;     
+	    string total_bit_string;      
+        string percent_coverage;
+        string evalue_string;
+        string percent_identity;    
+    };
+
+
     //data represnting info to record in applog amd metadata
     struct SAppLogInfo {
         int     topMatchesNum;
@@ -346,6 +372,10 @@ public:
     ///Initialize defline params
     void Init(void);
 
+    ///Get  deflines formatting info
+    ///@return vector of SDeflineFormattingInfo structs for defilens
+    vector <CShowBlastDefline::SDeflineFormattingInfo *> GetFormattingInfo(void);
+    
     ///Display defline
     ///@param out: stream to output
     ///
@@ -487,6 +517,7 @@ protected:
     //info to record in applog amd metadata
     SAppLogInfo *m_AppLogInfo;
     
+    vector <SDeflineFormattingInfo *> m_SdlFormatInfoVec;
 
     ///Internal function to return score info
     ///@param aln seq-align we are working with [in]
@@ -564,6 +595,7 @@ protected:
     ///
     void x_DisplayDeflineTableTemplate(CNcbiOstream & out);
 
+    void x_InitFormattingInfo(SScoreInfo* sci);    
     //For internal test
     friend struct ::CShowBlastDeflineTest;    
 };
