@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS_SEQDB__SEQDBFILE_HPP
 #define OBJTOOLS_READERS_SEQDB__SEQDBFILE_HPP
 
-/*  $Id: seqdbfile.hpp 530763 2017-03-17 14:00:40Z zaretska $
+/*  $Id: seqdbfile.hpp 553487 2017-12-18 14:23:38Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -574,6 +574,8 @@ public:
         x_ClrAmb();
     }
     
+    string GetLMDBFileName()const {return m_LMDBFile;}
+
     /// Verify the integrity of this object and subobjects.
     /*
     void Verify()
@@ -681,6 +683,11 @@ private:
     
     /// Offset of the end of the ambiguity section.
     TIndx m_EndAmb;
+
+    /// Name of matching SQLite file (empty if version 4 DB)
+    string m_LMDBFile;
+    /// Volume number (only set in version 5 DBs)
+    Uint4 m_Volume;
 };
 
 bool
