@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS_SEQDB__SEQDBISAM_HPP
 #define OBJTOOLS_READERS_SEQDB__SEQDBISAM_HPP
 
-/*  $Id: seqdbisam.hpp 536658 2017-05-22 15:48:20Z zaretska $
+/*  $Id: seqdbisam.hpp 579226 2019-01-31 16:32:08Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -146,7 +146,7 @@ public:
     typedef int TOid;
 
     /// PIG identifiers for numeric indices over protein volumes.
-    typedef int TPig;
+    //typedef int TPig;
 
     /// Genomic IDs, the most common numerical identifier.
 //    typedef int TGi;
