@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS_SEQDB__SEQDBVOL_HPP
 #define OBJTOOLS_READERS_SEQDB__SEQDBVOL_HPP
 
-/*  $Id: seqdbvol.hpp 553487 2017-12-18 14:23:38Z fongah2 $
+/*  $Id: seqdbvol.hpp 579218 2019-01-31 16:18:47Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -934,9 +934,9 @@ private:
                        bool          & have_vol) const
     {
         if (! have_user) {
-            if (m_UserGiList.NotEmpty() && m_UserGiList->GetNumTaxIds() == 0) {
+            if (m_UserGiList.NotEmpty() && m_UserGiList->GetNumTaxIds() == 0 && m_UserGiList->GetNumPigs() == 0 ) {
                 have_user |= x_ListIncludesId(*m_UserGiList, id);
-            } else if (m_NegativeList.NotEmpty() && m_NegativeList->GetNumTaxIds() == 0) {
+            } else if (m_NegativeList.NotEmpty() && m_NegativeList->GetNumTaxIds() == 0 && m_NegativeList->GetNumPigs() == 0 ) {
                 have_user |= x_ListIncludesId(*m_NegativeList, id);
             } else {
                 have_user = true;
