@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_BLAST_SEQDB_READER___SEQDB__HPP
 #define OBJTOOLS_BLAST_SEQDB_READER___SEQDB__HPP
 
-/*  $Id: seqdb.hpp 553487 2017-12-18 14:23:38Z fongah2 $
+/*  $Id: seqdb.hpp 579716 2019-02-05 16:53:21Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -326,7 +326,15 @@ public:
     ///   Specify eProtein, eNucleotide, or eUnknown.
     /// @param gilist
     ///   The database will be filtered by this GI list if non-null.
-    CSeqDB(const string & dbname, ESeqType seqtype, CSeqDBGiList * gilist = 0);
+    /// @param use_atlas_lock
+    ///   Enable/diable thread synchronization. If true single Atlas mutex
+    ///   will be used to protect most of critical parts of the code.
+    ///   If false, CSeqdDBAtlas::Lock and CSeqDBAtlas::Unlock functions
+    ///   will be noops. If each thread accesses
+    ///   a different database vloume, then setting this parameter to false
+    ///   will reduce contention. Otherwise it should be set to true.
+    CSeqDB(const string & dbname, ESeqType seqtype, CSeqDBGiList * gilist = 0,
+           bool use_atlas_lock = true);
 
     /// Short Constructor with Negative ID list.
     ///
@@ -345,6 +353,44 @@ public:
            ESeqType             seqtype,
            CSeqDBNegativeList * nlist);
 
+    /// Short Constructor with Positive and Negative ID list.
+    ///
+    /// This version of the constructor assumes the entire OID range
+    /// will be included, and applies filtering by a negative ID list.
+    /// Please use quotes ("") around database names that contains
+    /// space characters.
+    ///
+    /// @param dbname
+    ///   A list of database or alias names, seperated by spaces
+    /// @param seqtype
+    ///   Specify eProtein, eNucleotide, or eUnknown.
+    /// @param nlist
+    ///   The database will be filtered to not include these GIs or TIs.
+    CSeqDB(const string       & dbname,
+           ESeqType             seqtype,
+           CSeqDBGiList *       gilist,
+           CSeqDBNegativeList * nlist);
+
+    /// Short Constructor with Positive and Negative ID list with oid range.
+    ///
+    /// This version of the constructor assumes the entire OID range
+    /// will be included, and applies filtering by a negative ID list.
+    /// Please use quotes ("") around database names that contains
+    /// space characters.
+    ///
+    /// @param dbname
+    ///   A list of database or alias names, seperated by spaces
+    /// @param seqtype
+    ///   Specify eProtein, eNucleotide, or eUnknown.
+    /// @param nlist
+    ///   The database will be filtered to not include these GIs or TIs.
+    CSeqDB(const string       & dbname,
+           ESeqType             seqtype,
+           int                  oid_begin,
+           int                  oid_end,
+           CSeqDBGiList *       gilist,
+           CSeqDBNegativeList * nlist);
+
     /// Short Constructor with Computed ID list.
     ///
     /// This version of the constructor takes a computed CSeqDBIdSet
