@@ -2,7 +2,7 @@
 #define BLOB_ID__HPP_INCLUDED
 /* */
 
-/*  $Id: blob_id.hpp 440703 2014-07-16 15:38:41Z vasilche $
+/*  $Id: blob_id.hpp 578552 2019-01-22 15:38:07Z ivanov $
 * ===========================================================================
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
@@ -181,6 +181,13 @@ enum EBlobContentsMask
 typedef int TBlobContentsMask;
 
 
+enum EGBErrorAction {
+    eGBErrorAction_ignore,
+    eGBErrorAction_report,
+    eGBErrorAction_throw
+};
+
+
 END_SCOPE(objects)
 END_NCBI_SCOPE
 
