@@ -1,7 +1,7 @@
 #ifndef GBLOADER__HPP_INCLUDED
 #define GBLOADER__HPP_INCLUDED
 
-/*  $Id: gbloader.hpp 572532 2018-10-16 12:00:58Z ivanov $
+/*  $Id: gbloader.hpp 578552 2019-01-22 15:38:07Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -454,6 +454,17 @@ public:
             return m_HasHUPIncluded;
         }
 
+    
+    EGBErrorAction GetPTISErrorAction(void) const
+        {
+            return m_PTISErrorAction;
+        }
+
+    void SetPTISErrorAction(EGBErrorAction action)
+        {
+            m_PTISErrorAction = action;
+        }
+
     virtual CObjectManager::TPriority GetDefaultPriority(void) const override;
 
 protected:
@@ -493,6 +504,7 @@ private:
     bool                    m_AlwaysLoadNamedAcc;
     bool                    m_AddWGSMasterDescr;
     bool                    m_HasHUPIncluded;
+    EGBErrorAction          m_PTISErrorAction;
     string                  m_WebCookie;
 
     //
