@@ -1,7 +1,7 @@
 #ifndef GBLOADER_PARAMS__HPP_INCLUDED
 #define GBLOADER_PARAMS__HPP_INCLUDED
 
-/*  $Id: gbloader_params.h 475861 2015-08-12 15:33:37Z vasilche $
+/*  $Id: gbloader_params.h 578552 2019-01-22 15:38:07Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -67,5 +67,10 @@
 #define NCBI_GBLOADER_PARAM_ALWAYS_LOAD_NAMED_ACC "ALWAYS_LOAD_NAMED_ACC"
 /* Add WGS master descriptors to all WGS sequences */
 #define NCBI_GBLOADER_PARAM_ADD_WGS_MASTER "ADD_WGS_MASTER"
+/* How GBLoader should react on PTIS failure */
+#define NCBI_GBLOADER_PARAM_PTIS_ERROR_ACTION "PTIS_ERROR_ACTION"
+#define NCBI_GBLOADER_PARAM_PTIS_ERROR_ACTION_IGNORE "ignore"
+#define NCBI_GBLOADER_PARAM_PTIS_ERROR_ACTION_REPORT "report"
+#define NCBI_GBLOADER_PARAM_PTIS_ERROR_ACTION_THROW "throw"
 
 #endif
