@@ -1,6 +1,6 @@
 #ifndef PROCESSORS__HPP_INCLUDED
 #define PROCESSORS__HPP_INCLUDED
-/*  $Id: processors.hpp 487082 2015-12-15 00:25:55Z vasilche $
+/*  $Id: processors.hpp 576534 2018-12-17 19:38:30Z ivanov $
 * ===========================================================================
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
@@ -337,7 +337,8 @@ public:
     enum {
         eSat_ANNOT_CDD      = 10,
         eSat_ANNOT          = 26,
-        eSat_VDB_WGS        = 1000
+        eSat_VDB_WGS_MIN    = 1000,
+        eSat_VDB_WGS_MAX    = 1999
     };
     enum {
         eSubSat_SNP         = 1<<0,
