@@ -1,7 +1,7 @@
 #ifndef READER_ID2_BASE__HPP_INCLUDED
 #define READER_ID2_BASE__HPP_INCLUDED
 
-/*  $Id: reader_id2_base.hpp 529981 2017-03-09 16:30:29Z vasilche $
+/*  $Id: reader_id2_base.hpp 568299 2018-08-02 14:49:59Z vasilche $
 * ===========================================================================
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
@@ -215,16 +215,23 @@ protected:
 
     void x_ProcessReply(CReaderRequestResult& result,
                         SId2LoadedSet& loaded_set,
-                        const CID2_Reply& reply);
+                        const CID2_Reply& main_reply,
+                        const CID2_Request& main_request);
+    void x_ProcessEmptyReply(CReaderRequestResult& result,
+                             SId2LoadedSet& loaded_set,
+                             const CID2_Reply& main_reply,
+                             const CID2_Request& main_request);
     void x_ProcessGetSeqId(CReaderRequestResult& result,
                            SId2LoadedSet& loaded_set,
                            const CID2_Reply& main_reply,
-                           const CID2_Reply_Get_Seq_id& reply);
+                           const CID2_Request_Get_Seq_id& request,
+                           const CID2_Reply_Get_Seq_id* reply);
     void x_ProcessGetSeqIdSeqId(CReaderRequestResult& result,
                                 SId2LoadedSet& loaded_set,
                                 const CID2_Reply& main_reply,
                                 const CSeq_id_Handle& seq_id,
-                                const CID2_Reply_Get_Seq_id& reply);
+                                const CID2_Request_Get_Seq_id& request,
+                                const CID2_Reply_Get_Seq_id* reply);
     void x_ProcessGetBlobId(CReaderRequestResult& result,
                             SId2LoadedSet& loaded_set,
                             const CID2_Reply& main_reply,
@@ -275,6 +282,8 @@ protected:
                      int num,
                      const CID2_Reply& reply);
 
+    void x_DisableProcessors();
+
 private:
     CAtomicCounter_WithAutoInit m_RequestSerialNumber;
 
