@@ -1,6 +1,6 @@
 #ifndef READER__HPP_INCLUDED
 #define READER__HPP_INCLUDED
-/*  $Id: reader.hpp 568378 2018-08-03 14:54:41Z dondosha $
+/*  $Id: reader.hpp 579016 2019-01-29 16:46:05Z ivanov $
 * ===========================================================================
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
@@ -296,6 +296,9 @@ protected:
     virtual void x_DisconnectAtSlot(TConn conn, bool failed);
     // force connection at connection slot with key 'conn'
     virtual void x_ConnectAtSlot(TConn conn) = 0;
+    // report failed or stale connection
+    void x_ReportDisconnect(const char* reader, const char* server,
+                            TConn conn, bool failed) const;
 
 private:
     friend class CReaderAllocatedConnection;
