@@ -1,6 +1,8 @@
 #ifndef __REMOTE_UPDATER_HPP_INCLUDED__
 #define __REMOTE_UPDATER_HPP_INCLUDED__
 
+#include<functional>
+
 BEGIN_NCBI_SCOPE
 
 class CSerialObject;
@@ -8,7 +10,6 @@ class CSerialObject;
 BEGIN_SCOPE(objects)
 
 class CSeq_entry_EditHandle;
-class ILineErrorListener;
 class CSeq_entry;
 class CSeqdesc;
 class CSeq_descr;
@@ -23,15 +24,18 @@ class CCachedTaxon3_impl;
 class NCBI_XOBJEDIT_EXPORT CRemoteUpdater
 {
 public:
+
+   using FLogger = function<void(const string&)>;
+
    CRemoteUpdater(bool enable_caching = true);
    ~CRemoteUpdater();
 
    void UpdatePubReferences(CSerialObject& obj);
    void UpdatePubReferences(CSeq_entry_EditHandle& obj);
 
-   void UpdateOrgFromTaxon(ILineErrorListener* logger, CSeq_entry& entry);
-   void UpdateOrgFromTaxon(ILineErrorListener* logger, CSeq_entry_EditHandle& obj);
-   void UpdateOrgFromTaxon(ILineErrorListener* logger, CSeqdesc& obj);
+   void UpdateOrgFromTaxon(FLogger f_logger, CSeq_entry& entry);
+   void UpdateOrgFromTaxon(FLogger f_logger, CSeq_entry_EditHandle& obj);
+   void UpdateOrgFromTaxon(FLogger f_logger, CSeqdesc& obj);
    void ClearCache();
    static void ConvertToStandardAuthors(CAuth_list& auth_list);
    static void PostProcessPubs(CSeq_entry_EditHandle& obj);
@@ -44,7 +48,7 @@ public:
 private:
    void xUpdatePubReferences(CSeq_entry& entry);
    void xUpdatePubReferences(CSeq_descr& descr);
-   void xUpdateOrgTaxname(ILineErrorListener* logger, COrg_ref& org);
+   void xUpdateOrgTaxname(FLogger f_logger, COrg_ref& org);
 
 
    CRef<CMLAClient>  m_mlaClient;
