@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___FORMATTER_HPP
 #define OBJTOOLS_FORMAT___FORMATTER_HPP
 
-/*  $Id: formatter.hpp 294826 2011-05-27 11:19:20Z kornbluh $
+/*  $Id: formatter.hpp 360035 2012-04-19 13:43:48Z kornbluh $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -60,6 +60,7 @@ class CSegmentItem;
 class CFeatureItemBase;
 class CContigItem;
 class CWGSItem;
+class CTSAItem;
 class CGenomeItem;
 class CEndSectionItem;
 class CFlatTextOStream;
@@ -104,6 +105,7 @@ public:
     virtual void FormatPrimary(const CPrimaryItem& prim, IFlatTextOStream& text_os) = 0;
     virtual void FormatContig(const CContigItem& contig, IFlatTextOStream& text_os) = 0;
     virtual void FormatWGS(const CWGSItem& wgs, IFlatTextOStream& text_os) = 0;
+    virtual void FormatTSA(const CTSAItem& wgs, IFlatTextOStream& text_os) = 0;
     virtual void FormatGenome(const CGenomeItem& genome, IFlatTextOStream& text_os) = 0;
     virtual void FormatOrigin(const COriginItem& origin, IFlatTextOStream& text_os) = 0;
     virtual void FormatGap(const CGapItem& gap, IFlatTextOStream& text_os) = 0;
