@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___GATHER_ITEMS__HPP
 #define OBJTOOLS_FORMAT___GATHER_ITEMS__HPP
 
-/*  $Id: gather_items.hpp 530957 2017-03-20 14:27:19Z bollin $
+/*  $Id: gather_items.hpp 568182 2018-07-31 23:53:48Z kans $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -63,6 +63,8 @@ public:
     static CFlatGatherer* New(CFlatFileConfig::TFormat format);
 
     virtual void Gather(CFlatFileContext& ctx, CFlatItemOStream& os) const;
+    virtual void Gather(CFlatFileContext& ctx, CFlatItemOStream& os,
+                        const CSeq_entry_Handle& entry, bool useSeqEntryIndexing) const;
 
     virtual void SetCanceledCallback(const ICanceled* pCanceledCallback) {
         m_pCanceledCallback = pCanceledCallback;
@@ -81,6 +83,10 @@ protected:
 
     virtual void x_GatherSeqEntry(CFlatFileContext& ctx,
         CRef<CTopLevelSeqEntryContext> topLevelSeqEntryContext = CRef<CTopLevelSeqEntryContext>() ) const;
+    virtual void x_GatherSeqEntry(CFlatFileContext& ctx,
+        const CSeq_entry_Handle& entry, bool useSeqEntryIndexing,
+        CRef<CTopLevelSeqEntryContext> topLevelSeqEntryContext = CRef<CTopLevelSeqEntryContext>() ) const;
+
     virtual void x_GatherBioseq(
         const CBioseq_Handle& prev_seq, const CBioseq_Handle& this_seq, const CBioseq_Handle& next_seq, 
         CRef<CTopLevelSeqEntryContext> topLevelSeqEntryContext = CRef<CTopLevelSeqEntryContext>() ) const;
@@ -113,14 +119,21 @@ protected:
     void x_GetFeatsOnCdsProduct(const CSeq_feat& feat, CBioseqContext& ctx,
         CRef<CSeq_loc_Mapper> slice_mapper,
         CConstRef<CFeatureItem> cdsFeatureItem = CConstRef<CFeatureItem>() ) const;
+    void x_GetFeatsOnCdsProductIdx(CMappedFeat mf,const CSeq_feat& feat, CBioseqContext& ctx,
+        CRef<CSeq_loc_Mapper> slice_mapper,
+        CConstRef<CFeatureItem> cdsFeatureItem = CConstRef<CFeatureItem>() ) const;
     static void x_GiveOneResidueIntervalsBogusFuzz(CSeq_loc & loc);
     static void x_RemoveBogusFuzzFromIntervals(CSeq_loc & loc);
     void x_CopyCDSFromCDNA(const CSeq_feat& feat, CBioseqContext& ctx) const;
     bool x_SkipFeature(const CSeq_feat& feat, const CBioseqContext& ctx) const;
     virtual void x_GatherFeaturesOnLocation(const CSeq_loc& loc, SAnnotSelector& sel,
         CBioseqContext& ctx) const;
+    void x_GatherFeaturesOnWholeLocationIdx(const CSeq_loc& loc, SAnnotSelector& sel,
+        CBioseqContext& ctx) const;
     void x_GatherFeaturesOnWholeLocation(const CSeq_loc& loc, SAnnotSelector& sel,
         CBioseqContext& ctx) const;
+    void x_GatherFeaturesOnRangeIdx(const CSeq_loc& loc, SAnnotSelector& sel,
+        CBioseqContext& ctx) const;
     void x_GatherFeaturesOnRange(const CSeq_loc& loc, SAnnotSelector& sel,
         CBioseqContext& ctx) const;
 
