@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___GBSEQ_FORMATTER__HPP
 #define OBJTOOLS_FORMAT___GBSEQ_FORMATTER__HPP
 
-/*  $Id: gbseq_formatter.hpp 440037 2014-07-08 19:26:28Z kans $
+/*  $Id: gbseq_formatter.hpp 560973 2018-03-29 18:36:43Z dondosha $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -50,6 +50,8 @@ class CKeywordsItem;
 class CSourceItem;
 class CReferenceItem;
 class CCommentItem;
+class CPrimaryItem;
+class CDBSourceItem;
 class CFeatureItemBase;
 class CSequenceItem;
 class CSegmentItem;
@@ -60,7 +62,7 @@ class CGenomeProjectItem;
 class NCBI_FORMAT_EXPORT CGBSeqFormatter : public CFlatItemFormatter
 {
 public:
-    CGBSeqFormatter(void);
+    CGBSeqFormatter(bool isInsd=false);
     ~CGBSeqFormatter(void);
 
     virtual void Start       (IFlatTextOStream&);
@@ -76,35 +78,50 @@ public:
     virtual void FormatSource(const CSourceItem& source, IFlatTextOStream& text_os);
     virtual void FormatReference(const CReferenceItem& keys, IFlatTextOStream& text_os);
     virtual void FormatComment(const CCommentItem& keys, IFlatTextOStream& text_os);
+    virtual void FormatPrimary(const CPrimaryItem& primary, IFlatTextOStream& text_os);
+    virtual void FormatDBSource(const CDBSourceItem& dbs, IFlatTextOStream& text_os);
     virtual void FormatFeature(const CFeatureItemBase& feat, IFlatTextOStream& text_os);
     virtual void FormatSequence(const CSequenceItem& seq, IFlatTextOStream& text_os);
     virtual void FormatSegment(const CSegmentItem& seg, IFlatTextOStream& text_os);
     virtual void FormatContig(const CContigItem& contig, IFlatTextOStream& text_os);
     virtual void FormatGenomeProject(const CGenomeProjectItem&, IFlatTextOStream&);
+    virtual void FormatGap(const CGapItem& gap, IFlatTextOStream& text_os);
+    virtual void FormatWGS(const CWGSItem& wgs, IFlatTextOStream& text_os);
+    virtual void FormatTSA(const CTSAItem& tsa, IFlatTextOStream& text_os);
 
 private:
     void x_WriteFileHeader(IFlatTextOStream& text_os);
-    void x_StartWriteGBSet(IFlatTextOStream& text_os);
     void x_WriteGBSeq(IFlatTextOStream& text_os);
-    void x_EndWriteGBSet(IFlatTextOStream& text_os);
     void x_StrOStreamToTextOStream(IFlatTextOStream& text_os);
 
-    struct SOStreamContainer
-    {
-        SOStreamContainer(CObjectOStream& out, 
-            const CObjectTypeInfo& containerType) :
-            m_Cont(out, containerType)
-        {}
-        void WriteElement(const CConstObjectInfo& element) {
-            m_Cont.WriteElement(element);
-        }
-    private:
-        COStreamContainer  m_Cont;
-    };
+    // ID-4631 : common code for formatting accession ranges for WGS/TSA/TLS masters
+    template <typename T> void
+    x_FormatAltSeq(const T& item, const string& name, IFlatTextOStream& text_os);
+
     CRef<CGBSeq> m_GBSeq;
     auto_ptr<CObjectOStream> m_Out;
     CNcbiOstrstream m_StrStream;
-    auto_ptr<SOStreamContainer>  m_Cont;
+    bool m_IsInsd;
+    bool m_DidFeatStart;
+    bool m_DidJourStart;
+    bool m_DidKeysStart;
+    bool m_DidRefsStart;
+    bool m_DidWgsStart;
+    bool m_DidSequenceStart;
+    bool m_NeedFeatEnd;
+    bool m_NeedJourEnd;
+    bool m_NeedRefsEnd;
+    bool m_NeedWgsEnd;
+    bool m_NeedComment;
+    bool m_NeedPrimary;
+    bool m_NeedDbsource;
+    bool m_NeedXrefs;
+    string m_OtherSeqIDs;
+    string m_SecondaryAccns;
+    list<string> m_Comments;
+    string m_Primary;
+    list<string> m_Dbsource;
+    list<string> m_Xrefs;
 };
 
 
