@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS___FASTA__HPP
 #define OBJTOOLS_READERS___FASTA__HPP
 
-/*  $Id: fasta.hpp 538706 2017-06-13 16:56:56Z ivanov $
+/*  $Id: fasta.hpp 567787 2018-07-24 16:38:05Z gotvyans $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -66,6 +66,7 @@ class CSeq_loc;
 //class CSeqIdGenerator;
 
 class ILineErrorListener;
+class CSourceModParser;
 
 /// Base class for reading FASTA sequences.
 ///
@@ -107,14 +108,22 @@ public:
         fQuickIDCheck         = 1<<23, ///< Just check local IDs' first characters
         fUseIupacaa           = 1<<24, ///< If Prot, use iupacaa instead of the default ncbieaa.
         fHyphensIgnoreAndWarn = 1<<25, ///< When a hyphen is encountered in seq data, ignore it but warn.
-        fDisableNoResidues    = 1<<26  ///< If no residues found do not raise an error
+        fDisableNoResidues    = 1<<26, ///< If no residues found do not raise an error
+        fDisableParseRange    = 1<<27  ///< No ranges in seq-ids.  Ranges part of seq-id instead.
     };
-    typedef int TFlags; ///< binary OR of EFlags
+    using TFlags = long; ///< binary OR of EFlags
 
-    CFastaReader(ILineReader& reader, TFlags flags = 0);
-    CFastaReader(CNcbiIstream& in,    TFlags flags = 0);
-    CFastaReader(const string& path,  TFlags flags = 0);
-    CFastaReader(CReaderBase::TReaderFlags fBaseFlags,  TFlags flags = 0);
+    using SDefLineParseInfo = CFastaDeflineReader::SDeflineParseInfo;
+    using FIdCheck = CFastaDeflineReader::FIdCheck;
+
+
+    CFastaReader(ILineReader& reader, TFlags flags = 0, FIdCheck f_idcheck = CSeqIdCheck());
+    CFastaReader(CNcbiIstream& in,    TFlags flags = 0, FIdCheck f_idcheck = CSeqIdCheck());
+    CFastaReader(const string& path,  TFlags flags = 0, FIdCheck f_idcheck = CSeqIdCheck());
+    CFastaReader(CReaderBase::TReaderFlags fBaseFlags,  
+                 TFlags flags = 0, 
+                 FIdCheck f_idcheck = CSeqIdCheck()
+                 );
     virtual ~CFastaReader(void);
 
     /// CReaderBase overrides
@@ -171,11 +180,11 @@ public:
     }
 
     /// If this is set, an exception will be thrown if a Sequence ID exceeds the
-    /// given length.
+    /// given length. Overrides the id lengths specified in class CSeq_id.
     /// @param max_len
     ///   The new maximum to set.  Of course, you can set it to kMax_UI4
     ///   to effectively have no limit.
-    void SetMaxIDLength(Uint4 max_len) { m_MaxIDLength = max_len; }
+    void SetMaxIDLength(Uint4 max_len);
 
     /// Get the maximum ID allowed, which will be kMax_UI4
     /// unless someone has manually lowered it.
@@ -198,8 +207,6 @@ public:
     using TSeqTitles = vector<SLineTextAndLoc>;
     typedef CTempString TStr;
 
-    using SDefLineParseInfo = CFastaDeflineReader::SDeflineParseInfo;
-
     static void ParseDefLine(const TStr& defLine, 
         const SDefLineParseInfo& info,
         const TIgnoredProblems& ignoredErrors,
@@ -217,11 +224,11 @@ protected:
     };
 
 
-    virtual CRef<CSeq_entry> x_ReadSegSet(ILineErrorListener * pMessageListener);
+    NCBI_DEPRECATED virtual CRef<CSeq_entry> x_ReadSegSet(ILineErrorListener * pMessageListener);
 
-    virtual void   ParseDefLine  (const TStr& s, ILineErrorListener * pMessageListener);
+    virtual void ParseDefLine (const TStr& s, ILineErrorListener * pMessageListener);
 
-    virtual bool   ParseIDs(const TStr& s, ILineErrorListener * pMessageListener);
+    virtual bool ParseIDs(const TStr& s, ILineErrorListener * pMessageListener);
 
     virtual void PostProcessIDs(const CBioseq::TId& defline_ids, 
         const string& defline,
@@ -252,6 +259,7 @@ protected:
         const TStr& sLineText, 
         TSeqPos iLineNum,
         ILineErrorListener * pMessageListener) const;
+    virtual void   ApplySourceMods(CBioseq & bioseq, CSourceModParser& smp);
     static bool ExcessiveSeqDataInTitle(const string& title, 
                                         TFlags fFastaFlags);
     virtual void   PostWarning(ILineErrorListener * pMessageListener,
@@ -260,6 +268,7 @@ protected:
             ILineError::EProblem _eProblem, 
             CTempString _sFeature, CTempString _sQualName, CTempString _sQualValue) const;
 
+    CSourceModParser* xCreateSourceModeParser(ILineErrorListener* pErrorListener);
 
     typedef int                         TRowNum;
     typedef map<TRowNum, TSignedSeqPos> TSubMap;
@@ -397,6 +406,7 @@ protected:
     TSeqTitles m_CurrentSeqTitles;
     CRef<CSourceModParser::CModFilter> m_pModFilter;
     std::vector<ILineError::EProblem> m_ignorable;
+    FIdCheck m_fIdCheck;
 };
 
 
