@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS___FASTA_EXCEPTION__HPP
 #define OBJTOOLS_READERS___FASTA_EXCEPTION__HPP
 
-/*  $Id: fasta_exception.hpp 437343 2014-06-05 13:58:46Z ucko $
+/*  $Id: fasta_exception.hpp 578929 2019-01-28 18:18:01Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -45,7 +45,7 @@ public:
         eBadResidues
     };
 
-    virtual const char* GetErrCodeString(void) const
+    virtual const char* GetErrCodeString(void) const override
     {
         switch (GetErrCode()) {
             case eBadResidues:    return "eBadResidues";
@@ -89,7 +89,7 @@ public:
         TBadIndexMap m_BadIndexMap;
     };
 
-    virtual void ReportExtra(ostream& out) const;
+    virtual void ReportExtra(ostream& out) const override;
 
     CBadResiduesException(const CDiagCompileInfo& info,
         const CException* prev_exception,
