@@ -1,4 +1,4 @@
- /*  $Id: gff2_reader.hpp 560230 2018-03-20 17:29:23Z foleyjp $
+ /*  $Id: gff2_reader.hpp 575511 2018-11-29 19:38:28Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -127,14 +127,6 @@ public:
     //
     //  new stuff: 
     //
-    bool x_ParseBrowserLineGff(
-        const string&,
-        CRef< CAnnotdesc >& );
-        
-    virtual bool x_ParseTrackLineGff(
-        const string&,
-        CRef< CAnnotdesc >& );
-                                
     virtual bool xParseStructuredComment(
         const string&);
     
@@ -159,11 +151,6 @@ public:
         CRef<CSeq_annot>&,
         const string& = "");
 
-    virtual bool x_ParseFeatureGff(
-        const string&,
-        TAnnots&,
-        ILineErrorListener*);
-
     virtual bool x_ParseAlignmentGff(
         const string& strLine, 
         list<string>& id_list,
@@ -208,7 +195,7 @@ public:
         CRef< CSeq_annot >,
         ILineErrorListener* =0);
 
-    virtual bool x_UpdateAnnotFeature(
+    virtual bool xUpdateAnnotFeature(
         const CGff2Record&,
         CRef< CSeq_annot >,
         ILineErrorListener* =0);
@@ -226,10 +213,6 @@ public:
         const CGff2Record&,
         CRef< CSeq_feat > );
 
-    bool x_FeatureTrimQualifiers(
-        const CGff2Record&,
-        CRef< CSeq_feat > );
-
     bool xFeatureSetQualifier(
         const string&,
         const string&,
