@@ -1,4 +1,4 @@
- /*  $Id: gff3_reader.hpp 547755 2017-10-03 18:03:15Z ludwigf $
+ /*  $Id: gff3_reader.hpp 575510 2018-11-29 19:38:03Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -40,7 +40,7 @@
 
 #include <objtools/readers/reader_base.hpp>
 #include <objtools/readers/gff2_reader.hpp>
-//#include <objtools/readers/gff3_data.hpp>
+#include <objtools/readers/gff2_data.hpp>
 
 BEGIN_NCBI_SCOPE
 
@@ -96,15 +96,15 @@ public:
 protected:
     virtual CGff2Record* x_CreateRecord() { return new CGff3ReadRecord(); };    
 
-    virtual bool x_UpdateAnnotFeature(
+    virtual bool xInitializeFeature(
+        const CGff2Record&,
+        CRef<CSeq_feat> );
+
+    virtual bool xUpdateAnnotFeature(
         const CGff2Record&,
         CRef< CSeq_annot >,
         ILineErrorListener*);
 
-    virtual bool x_UpdateFeatureCds(
-        const CGff2Record&,
-        CRef<CSeq_feat>);
-
     virtual bool xAddFeatureToAnnot(
         CRef< CSeq_feat >,
         CRef< CSeq_annot > );
@@ -139,6 +139,10 @@ protected:
         CRef<CSeq_annot>,
         ILineErrorListener*);
 
+    virtual bool xFindFeatureUnderConstruction(
+        const CGff2Record&,
+        CRef<CSeq_feat>&);
+
     virtual bool xVerifyCdsParents(
         const CGff2Record&);
 
@@ -162,9 +166,25 @@ protected:
     virtual bool xIsIgnoredFeatureType(
         const string&);
 
+    virtual void xAddPendingExon(
+        const string&,
+        const CGff2Record&);
+    virtual void xGetPendingExons(
+        const string&,
+        list<CGff2Record>&);
+    virtual void xPostProcessAnnot(
+        CRef<CSeq_annot>&,
+        ILineErrorListener*);
+
+
     // Data:
     map<string, string> mCdsParentMap;
     map<string, CRef<CSeq_interval> > mMrnaLocs;
+    map<string, string> mIdToSeqIdMap;
+
+    using PENDING_EXONS = map<string, list<CGff2Record> >;
+    PENDING_EXONS mPendingExons;
+
     static unsigned int msGenericIdCounter;
 };
 
