@@ -1,4 +1,4 @@
- /*  $Id: gtf_reader.hpp 547755 2017-10-03 18:03:15Z ludwigf $
+ /*  $Id: gtf_reader.hpp 575512 2018-11-29 19:38:56Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -40,6 +40,83 @@ BEGIN_NCBI_SCOPE
 
 BEGIN_SCOPE(objects) // namespace ncbi::objects::
 
+//  ============================================================================
+class CGtfAttributes
+//  ============================================================================
+{
+public:
+    using MultiValue = vector<string>;
+    using MultiAttributes = map<string, MultiValue>;
+
+    const MultiAttributes&
+    Get() const
+    {
+        return mAttributes;
+    };
+
+    string
+    ValueOf(
+        const string& key) const
+    {
+        MultiValue values;
+        GetValues(key, values);
+        if (values.size() == 1) {
+            return values.front();
+        }
+        return "";
+    }
+
+    bool
+    HasValue(
+        const string& key,
+        const string& value = "") const
+    {
+        auto it = mAttributes.find(key);
+        if (it == mAttributes.end()) {
+            return false;
+        }
+        const auto& values = it->second;
+        if (values.empty()) {
+            return false;
+        }
+        if (value.empty()) {
+            return true;
+        }
+        return (find(values.begin(), values.end(), value) != values.end());
+    };
+
+    void
+    GetValues(
+        const string& key,
+        MultiValue& values) const
+    {
+        const MultiValue empty;
+        values = empty;
+        auto it = mAttributes.find(key);
+        if (it != mAttributes.end()) {
+            values = it->second;
+        }
+    };
+
+    void
+    AddValue(
+        const string& key,
+        const string& value)
+    {
+        auto kit = mAttributes.find(key);
+        if (kit == mAttributes.end()) {
+            kit = mAttributes.insert(make_pair(key, MultiValue())).first;
+        }
+        auto vit = find(kit->second.begin(), kit->second.end(), value);
+        if (vit == kit->second.end()) {
+            kit->second.push_back(value);
+        }
+    };
+
+protected:
+    MultiAttributes mAttributes;
+};
+
 //  ============================================================================
 class CGtfReadRecord
 //  ============================================================================
@@ -49,10 +126,43 @@ public:
     CGtfReadRecord(): CGff2Record() {};
     ~CGtfReadRecord() {};
 
+    const CGtfAttributes&
+    GtfAttributes() const 
+    {
+        return mAttributes;
+    };
+
+    string
+    GeneKey() const
+    {
+        string geneId = mAttributes.ValueOf("gene_id");
+        if (geneId.empty()) {
+            cerr << "Unexpected: GTF feature without a gene_id." << endl;
+        }
+        return geneId;
+    };
+
+    string
+    FeatureKey() const
+    {
+        static unsigned int tidCounter(1);
+        if (Type() == "gene") {
+            return GeneKey();
+        }
+
+        string transcriptId = mAttributes.ValueOf("transcript_id");
+        if (transcriptId.empty()) {
+            transcriptId = "t" + NStr::IntToString(tidCounter++);
+        }
+        return GeneKey() + "_" + transcriptId;
+    }
+
 protected:
-    bool x_AssignAttributesFromGff(
+    bool xAssignAttributesFromGff(
         const string&,
         const string& );
+
+    CGtfAttributes mAttributes;
 };
 
 //  ----------------------------------------------------------------------------
@@ -76,116 +186,132 @@ public:
 protected:
     virtual CGff2Record* x_CreateRecord() { return new CGtfReadRecord(); };    
 
-    virtual bool x_UpdateAnnotFeature(
+    virtual bool xUpdateAnnotFeature(
         const CGff2Record&,
         CRef< CSeq_annot >,
         ILineErrorListener* =0);
 
     virtual bool x_UpdateAnnotCds(
-        const CGff2Record&,
+        const CGtfReadRecord&,
         CRef< CSeq_annot > );
 
     virtual bool x_UpdateAnnotTranscript(
-        const CGff2Record&,
+        const CGtfReadRecord&,
         CRef< CSeq_annot > );
 
     bool x_CreateFeatureId(
-        const CGff2Record&,
+        const CGtfReadRecord&,
         const string&,
         CRef< CSeq_feat > );
 
     bool x_CreateFeatureLocation(
-        const CGff2Record&,
+        const CGtfReadRecord&,
         CRef< CSeq_feat > );
     
     bool x_CreateGeneXrefs(
-        const CGff2Record&,
+        const CGtfReadRecord&,
         CRef< CSeq_feat > );
     
     bool x_CreateMrnaXrefs(
-        const CGff2Record&,
+        const CGtfReadRecord&,
         CRef< CSeq_feat > );
     
     bool x_CreateCdsXrefs(
-        const CGff2Record&,
+        const CGtfReadRecord&,
         CRef< CSeq_feat > );
     
     bool x_MergeFeatureLocationSingleInterval(
-        const CGff2Record&,
+        const CGtfReadRecord&,
         CRef< CSeq_feat > );
     
     bool x_MergeFeatureLocationMultiInterval(
-        const CGff2Record&,
+        const CGtfReadRecord&,
         CRef< CSeq_feat > );
 
     bool x_CreateParentGene(
-        const CGff2Record&,
+        const CGtfReadRecord&,
         CRef< CSeq_annot > );
         
     bool x_MergeParentGene(
-        const CGff2Record&,
+        const CGtfReadRecord&,
         CRef< CSeq_feat > );
             
     bool xFeatureSetQualifiersGene(
-        const CGff2Record& record,
+        const CGtfReadRecord& record,
         CRef<CSeq_feat>);
 
     bool xFeatureSetQualifiersRna(
-        const CGff2Record& record,
+        const CGtfReadRecord& record,
         CRef<CSeq_feat>);
 
     bool xFeatureSetQualifiersCds(
-        const CGff2Record& record,
+        const CGtfReadRecord& record,
         CRef<CSeq_feat>);
 
     bool x_CreateParentCds(
-        const CGff2Record&,
+        const CGtfReadRecord&,
         CRef< CSeq_annot > );
         
     bool x_CreateParentMrna(
-        const CGff2Record&,
+        const CGtfReadRecord&,
         CRef< CSeq_annot > );
         
     bool x_MergeParentCds(
-        const CGff2Record&,
+        const CGtfReadRecord&,
         CRef< CSeq_feat > );
             
     bool x_FeatureSetDataGene(
-        const CGff2Record&,
+        const CGtfReadRecord&,
         CRef< CSeq_feat > );
 
     virtual bool x_FeatureSetDataRna(
-        const CGff2Record&,
+        const CGtfReadRecord&,
         CRef< CSeq_feat >,
         CSeqFeatData::ESubtype );
 
     bool x_FeatureSetDataMRNA(
-        const CGff2Record&,
+        const CGtfReadRecord&,
         CRef< CSeq_feat > );
 
     bool x_FeatureSetDataCDS(
-        const CGff2Record&,
+        const CGtfReadRecord&,
+        CRef< CSeq_feat > );
+
+    bool x_FeatureTrimQualifiers(
+        const CGtfReadRecord&,
         CRef< CSeq_feat > );
 
 protected:
     bool x_FindParentGene(
-        const CGff2Record&,
+        const CGtfReadRecord&,
         CRef< CSeq_feat >& );
 
     bool x_FindParentCds(
-        const CGff2Record&,
+        const CGtfReadRecord&,
         CRef< CSeq_feat >& );
 
     bool x_FindParentMrna(
-        const CGff2Record&,
+        const CGtfReadRecord&,
         CRef< CSeq_feat >& );
 
-    virtual bool x_ProcessQualifierSpecialCase(
-        CGff2Record::TAttrCit,
+    virtual bool xProcessQualifierSpecialCase(
+        const string&,
+        const CGtfAttributes::MultiValue&,
         CRef< CSeq_feat > );
-  
+
+    virtual void
+        xFeatureAddQualifiers(
+            const string& key,
+            const CGtfAttributes::MultiValue& values,
+            CRef<CSeq_feat> pFeature)
+    {
+        for (auto value: values) {
+            pFeature->AddQualifier(key, value);
+        }
+    };
+
     bool x_CdsIsPartial(
-        const CGff2Record& );
+        const CGtfReadRecord& );
 
     typedef map< string, CRef< CSeq_feat > > TIdToFeature;
     TIdToFeature m_GeneMap;
