@@ -1,4 +1,4 @@
- /*  $Id: gvf_reader.hpp 513672 2016-09-14 14:10:57Z ludwigf $
+ /*  $Id: gvf_reader.hpp 575504 2018-11-29 19:35:15Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -70,7 +70,7 @@ public:
     bool SanityCheck() const;
 
 protected:
-    virtual bool x_AssignAttributesFromGff(
+    virtual bool xAssignAttributesFromGff(
         const string&,
         const string& );
     void xTraceError(
