@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS___READER_EXCEPTION__HPP
 #define OBJTOOLS_READERS___READER_EXCEPTION__HPP
 
-/*  $Id: reader_exception.hpp 543406 2017-08-09 18:06:39Z foleyjp $
+/*  $Id: reader_exception.hpp 578929 2019-01-28 18:18:01Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -61,7 +61,7 @@ public:
         eWrongGap,
         eInvalidID
     };
-    virtual const char* GetErrCodeString(void) const
+    virtual const char* GetErrCodeString(void) const override
     {
         switch (GetErrCode()) {
         case eFormat:      return "eFormat";
