@@ -1,7 +1,7 @@
 #ifndef OBJSTACK__HPP
 #define OBJSTACK__HPP
 
-/*  $Id: objstack.hpp 534119 2017-04-24 17:07:34Z gouriano $
+/*  $Id: objstack.hpp 570663 2018-09-12 18:28:27Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -157,6 +157,7 @@ public:
     const TFrame& FetchFrameFromTop(size_t index) const;
     const TFrame& FetchFrameFromBottom(size_t index) const;
     TTypeInfo GetRecentTypeInfo(void) const;
+    bool IsKnownElement(const CTempString& name) const;
 
     static TTypeInfo GetRealTypeInfo(TTypeInfo typeInfo);
     static ETypeFamily GetRealTypeFamily(TTypeInfo typeInfo);
@@ -167,6 +168,11 @@ public:
     const string& GetStackPath(void) const;
 
     void WatchPathHooks(bool set=true);
+
+    void RegisterPathHook(CPathHook* h) {
+        m_PathHooks.insert(h);
+    }
+    void ResetPathHooks(void);
 protected:
     virtual void x_SetPathHooks(bool set) = 0;
     virtual void ResetState(void);
@@ -183,6 +189,7 @@ private:
     string  m_MemberPath;
     bool    m_WatchPathHooks;
     bool    m_PathValid;
+    set<CPathHook*> m_PathHooks;
 };
 
 #include <serial/impl/objstack.inl>
