@@ -1,7 +1,7 @@
 #ifndef STDTYPES__HPP
 #define STDTYPES__HPP
 
-/*  $Id: stdtypes.hpp 507795 2016-07-21 17:24:14Z gouriano $
+/*  $Id: stdtypes.hpp 546693 2017-09-20 17:04:38Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -64,16 +64,16 @@ public:
     CPrimitiveTypeInfo(size_t size, const string& name,
                        EPrimitiveValueType valueType, bool isSigned = true);
 
-    virtual bool IsDefault(TConstObjectPtr object) const;
+    virtual bool IsDefault(TConstObjectPtr object) const override;
     virtual bool Equals(TConstObjectPtr , TConstObjectPtr,
-                        ESerialRecursionMode how = eRecursive) const;
-    virtual void SetDefault(TObjectPtr dst) const;
+                        ESerialRecursionMode how = eRecursive) const override;
+    virtual void SetDefault(TObjectPtr dst) const override;
     virtual void Assign(TObjectPtr dst, TConstObjectPtr src,
-                        ESerialRecursionMode how = eRecursive) const;
+                        ESerialRecursionMode how = eRecursive) const override;
 
     EPrimitiveValueType GetPrimitiveValueType(void) const;
 
-    bool IsSigned(void) const;
+    virtual bool IsSigned(void) const;
 
     virtual bool GetValueBool(TConstObjectPtr objectPtr) const;
     virtual void SetValueBool(TObjectPtr objectPtr, bool value) const;
@@ -124,13 +124,6 @@ public:
     virtual void SetValueBitString(TObjectPtr objectPtr,
                                    const CBitString& value) const;
 
-#ifdef NCBI_STRICT_GI
-    virtual void GetValueGi(TConstObjectPtr objectPtr,
-                            TGi& value) const;
-    virtual void SetValueGi(TObjectPtr objectPtr,
-                            const TGi& value) const;
-#endif
-
     virtual void GetValueAnyContent(TConstObjectPtr objectPtr,
                                     CAnyContentObject& value) const;
     virtual void SetValueAnyContent(TObjectPtr objectPtr,
@@ -392,17 +385,6 @@ public:
 };
 
 
-#ifdef NCBI_STRICT_GI
-EMPTY_TEMPLATE
-class NCBI_XSERIAL_EXPORT CStdTypeInfo<TGi>
-{
-public:
-    static TTypeInfo GetTypeInfo(void);
-    static CTypeInfo* CreateTypeInfo(void);
-};
-#endif
-
-
 /* @} */
 
 
