@@ -1,7 +1,7 @@
 #ifndef OBJECTINFO__HPP
 #define OBJECTINFO__HPP
 
-/*  $Id: objectinfo.hpp 363528 2012-05-17 12:57:20Z gouriano $
+/*  $Id: objectinfo.hpp 380188 2012-11-08 18:49:33Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -41,7 +41,6 @@
 #include <serial/impl/classinfo.hpp>
 #include <serial/impl/choice.hpp>
 #include <serial/impl/enumerated.hpp>
-#include <serial/impl/objstrasnb.hpp>
 #include <vector>
 #include <memory>
 
@@ -389,7 +388,8 @@ public:
     CMemberIterator GetMemberIterator(TMemberIndex index) const;
     CVariantIterator GetVariantIterator(TMemberIndex index) const;
 
-    CAsnBinaryDefs::ETagValue GetASNTag() const;
+    typedef int TASNTag;
+    TASNTag GetASNTag() const;
 
 protected:
     void ResetTypeInfo(void);
