@@ -1,7 +1,7 @@
 #ifndef OBJOSTR__HPP
 #define OBJOSTR__HPP
 
-/*  $Id: objostr.hpp 507795 2016-07-21 17:24:14Z gouriano $
+/*  $Id: objostr.hpp 546693 2017-09-20 17:04:38Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -394,14 +394,14 @@ public:
     ///
     /// @return
     ///   string
-    virtual string GetStackTrace(void) const;
+    virtual string GetStackTrace(void) const override;
 
     /// Get current stream position as string.
     /// Useful for diagnostic and information messages.
     ///
     /// @return
     ///   string
-    virtual string GetPosition(void) const;
+    virtual string GetPosition(void) const override;
 
 //---------------------------------------------------------------------------
 // Local write hooks
@@ -578,7 +578,7 @@ public:
     // report error about unended block
     void Unended(const string& msg);
     // report error about unended object stack frame
-    virtual void UnendedFrame(void);
+    virtual void UnendedFrame(void) override;
 
     enum EFlags {
         fFlagNone                = 0,
@@ -781,7 +781,7 @@ protected:
     void RegisterObject(TTypeInfo typeInfo);
     void RegisterObject(TConstObjectPtr object, TTypeInfo typeInfo);
 
-    void x_SetPathHooks(bool set);
+    virtual void x_SetPathHooks(bool set) override;
     EFixNonPrint x_GetFixCharsMethodDefault(void) const;
     EFixNonPrint x_FixCharsMethod(void) const {
         return m_FixMethod;
@@ -801,6 +801,7 @@ protected:
     bool  m_WriteNamedIntegersByValue;
     bool  m_FastWriteDouble;
     bool  m_EnforceWritingDefaults;
+    TTypeInfo m_TypeAlias;
 
 private:
     static CObjectOStream* OpenObjectOStreamAsn(CNcbiOstream& out,
