@@ -1,7 +1,7 @@
 #ifndef SERIAL___RPCBASE__HPP
 #define SERIAL___RPCBASE__HPP
 
-/*  $Id: rpcbase.hpp 541584 2017-07-19 16:03:23Z ivanov $
+/*  $Id: rpcbase.hpp 554977 2018-01-11 14:18:53Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -94,22 +94,22 @@ protected:
         return string();
     }
 
-    virtual void x_WriteRequest(CObjectOStream& out, const CSerialObject& request)
+    virtual void x_WriteRequest(CObjectOStream& out, const CSerialObject& request) override
     {
         WriteRequest(out, dynamic_cast<const TRequest&>(request));
     }
 
-    virtual void x_ReadReply(CObjectIStream& in, CSerialObject& reply)
+    virtual void x_ReadReply(CObjectIStream& in, CSerialObject& reply) override
     {
         ReadReply(in, dynamic_cast<TReply&>(reply));
     }
 
-    virtual string x_GetAffinity(const CSerialObject& request) const
+    virtual string x_GetAffinity(const CSerialObject& request) const override
     {
         return GetAffinity(dynamic_cast<const TRequest&>(request));
     }
 
-    virtual void x_Connect(void);
+    virtual void x_Connect(void) override;
 
     /// Connect to a URL.  (Discouraged; please establish and use a
     /// suitable named service if possible.)
@@ -166,8 +166,8 @@ void CRPCClient<TRequest, TReply>::x_Connect(void)
     x_extra.parse_header = sx_ParseHeader;
     x_extra.flags = fHTTP_NoAutoRetry;
 
-    auto_ptr<CConn_ServiceStream> stream(new CConn_ServiceStream(
-        m_Service, fSERV_Any, net_info, &x_extra, m_Timeout));
+    unique_ptr<CConn_ServiceStream> stream(new CConn_ServiceStream(
+        m_Service, fSERV_Any | fSERV_DelayOpen, net_info, &x_extra, m_Timeout));
     if ( m_Canceler.NotNull() ) {
         stream->SetCanceledCallback(m_Canceler.GetNonNullPointer());
     }
@@ -194,7 +194,7 @@ void CRPCClient<TRequest, TReply>::x_ConnectURL(const string& url)
                 "Error sending retry context arguments");
         }
     }
-    auto_ptr<CConn_HttpStream> stream(new CConn_HttpStream(net_info,
+    unique_ptr<CConn_HttpStream> stream(new CConn_HttpStream(net_info,
         kEmptyStr, // user_header
         sx_ParseHeader, // callback
         &m_RetryCtx,    // user data for the callback
