@@ -1,7 +1,7 @@
 #ifndef SERIALIMPL__HPP
 #define SERIALIMPL__HPP
 
-/*  $Id: serialimpl.hpp 507795 2016-07-21 17:24:14Z gouriano $
+/*  $Id: serialimpl.hpp 556123 2018-01-26 13:43:52Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -497,6 +497,7 @@ const NCBI_NS_NCBI::CTypeInfo* Method(void) \
 
 #define END_CHOICE_INFO                                                 \
     NCBI_NS_NCBI::CClassInfoHelper<CClass>::SetReadWriteVariantMethods(info); \
+    info->SetSelectorInfo(MEMBER_PTR(m_choice), sizeof(BASE_OBJECT()->m_choice)); \
     END_STRUCT_INFO
 
 // sub class definition
@@ -585,6 +586,11 @@ const NCBI_NS_NCBI::CTypeInfo* BaseClassName::GetTypeInfo(void) \
     NCBI_NAME2(ClassName,_Base), \
     SERIAL_ALIAS(RefType), RefCode)
 
+#define BEGIN_NESTED_ALIAS_INFO(AliasName,ClassName,RefType,RefCode) \
+    BEGIN_ALIAS_INFO_METHOD(AliasName, ClassName, \
+    ClassName, \
+    SERIAL_ALIAS(RefType), RefCode)
+
 #define BEGIN_ENUM_ALIAS_INFO_METHOD(AliasName,ClassName,BaseClassName,SerialRef,Code) \
 const NCBI_NS_NCBI::CTypeInfo* BaseClassName::GetTypeInfo(void) \
 { \
@@ -604,6 +610,11 @@ const NCBI_NS_NCBI::CTypeInfo* BaseClassName::GetTypeInfo(void) \
     NCBI_NAME2(ClassName,_Base), \
     SERIAL_ALIAS(RefType), RefCode)
 
+#define BEGIN_NESTED_ENUM_ALIAS_INFO(AliasName,ClassName,RefType,RefCode) \
+    BEGIN_ENUM_ALIAS_INFO_METHOD(AliasName, ClassName, \
+    ClassName, \
+    SERIAL_ALIAS(RefType), RefCode)
+
 #define SET_STD_ALIAS_DATA_PTR \
     info->SetDataOffset(NCBI_NS_NCBI::TPointerOffsetType(GetDataPtr(BASE_OBJECT())))
 #define SET_CLASS_ALIAS_DATA_PTR \
