@@ -1,7 +1,7 @@
 #ifndef UTIL___ALIGN_RANGE_COLL__HPP
 #define UTIL___ALIGN_RANGE_COLL__HPP
 
-/*  $Id: align_range_coll.hpp 500279 2016-05-03 17:12:04Z ivanov $
+/*  $Id: align_range_coll.hpp 578926 2019-01-28 18:17:50Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -51,7 +51,7 @@ public:
     {
     }
 
-    virtual const char* GetErrCodeString(void) const
+    virtual const char* GetErrCodeString(void) const override
     {
         return "CAlignRangeCollection - operation resulted in invalid state."; 
     }
