@@ -1,29 +1,25 @@
 #ifndef BMALGO_IMPL__H__INCLUDED__
 #define BMALGO_IMPL__H__INCLUDED__
 /*
-Copyright(c) 2002-2010 Anatoliy Kuznetsov(anatoliy_kuznetsov at yahoo.com)
+Copyright(c) 2002-2017 Anatoliy Kuznetsov(anatoliy_kuznetsov at yahoo.com)
 
-Permission is hereby granted, free of charge, to any person 
-obtaining a copy of this software and associated documentation 
-files (the "Software"), to deal in the Software without restriction, 
-including without limitation the rights to use, copy, modify, merge, 
-publish, distribute, sublicense, and/or sell copies of the Software, 
-and to permit persons to whom the Software is furnished to do so, 
-subject to the following conditions:
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
 
-The above copyright notice and this permission notice shall be included 
-in all copies or substantial portions of the Software.
+    http://www.apache.org/licenses/LICENSE-2.0
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
-EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
-OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
-DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
-ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
-OTHER DEALINGS IN THE SOFTWARE.
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
 
-For more information please visit:  http://bmagic.sourceforge.net
+For more information please visit:  http://bitmagic.io
+*/
 
+/*! \file bmalgo_impl.h
+    \brief Algorithms for bvector<>
 */
 
 #ifdef _MSC_VER
@@ -38,9 +34,9 @@ namespace bm
 {
 
 /*! 
-   @defgroup setalgo Bitset algorithms
+   @defgroup setalgo bvector<> algorithms 
  
-   Set algebra algorithms
+   Set algebra algorithms using bit-vectors, arrays. Binary metrics and distances. Random sub-sets.
    @ingroup bvector
  */
 
@@ -110,6 +106,7 @@ struct distance_metric_descriptor
 };
 
 
+
 /*!
     \brief Internal function computes different distance metrics.
     \internal 
@@ -155,11 +152,13 @@ void combine_count_operation_with_block(const bm::word_t*           blk,
                      dmd.result += gap_count_xor(g1, g2);
                     break;
                  case bm::COUNT_A:
-                    dmd.result += gap_bit_count(g1);
+                    dmd.result += gap_bit_count_unr(g1);
                     break;
                  case bm::COUNT_B:
-                    dmd.result += gap_bit_count(g2);
+                    dmd.result += gap_bit_count_unr(g2);
                     break;
+                 default:
+                     BM_ASSERT(0);
                  } // switch
                                      
              } // for it
@@ -181,19 +180,17 @@ void combine_count_operation_with_block(const bm::word_t*           blk,
                      break;
                  case bm::COUNT_OR:
                      if (!arg_blk)
-                        dmd.result += gap_bit_count(g1);
+                        dmd.result += gap_bit_count_unr(g1);
                      else
                         dmd.result += gap_bitset_or_count(arg_blk, g1); 
                      break;
                  case bm::COUNT_SUB_AB:
                      {
-                     bm::word_t  BM_ALIGN16 temp_bit_blk[bm::set_block_size] BM_ALIGN16ATTR;
+                     bm::word_t  BM_VECT_ALIGN temp_bit_blk[bm::set_block_size] BM_VECT_ALIGN_ATTR;
 
                      gap_convert_to_bitset((bm::word_t*) temp_bit_blk, g1);
                      dmd.result += 
-                       bit_operation_sub_count((bm::word_t*)temp_bit_blk, 
-                          ((bm::word_t*)temp_bit_blk) + bm::set_block_size,
-                           arg_blk);
+                       bit_operation_sub_count((bm::word_t*)temp_bit_blk, arg_blk);
                      }
                      break;
                  case bm::COUNT_SUB_BA:
@@ -205,22 +202,23 @@ void combine_count_operation_with_block(const bm::word_t*           blk,
                      break;
                  case bm::COUNT_XOR:
                      if (!arg_blk)
-                        dmd.result += gap_bit_count(g1);
+                        dmd.result += gap_bit_count_unr(g1);
                      else
                         dmd.result += gap_bitset_xor_count(arg_blk, g1);
                      break;
                  case bm::COUNT_A:
                     if (g1)
-                        dmd.result += gap_bit_count(g1);
+                        dmd.result += gap_bit_count_unr(g1);
                     break;
                  case bm::COUNT_B:
                     if (arg_blk)
                     {
                         dmd.result += 
-                          bit_block_calc_count(arg_blk, 
-                                               arg_blk + bm::set_block_size);
+                          bit_block_count(arg_blk);
                     }
                     break;
+                 default:
+                     BM_ASSERT(0);
                  } // switch
                                      
              } // for it
@@ -245,7 +243,7 @@ void combine_count_operation_with_block(const bm::word_t*           blk,
                      break;
                  case bm::COUNT_OR:
                      if (!blk)
-                        dmd.result += gap_bit_count(g2);
+                        dmd.result += gap_bit_count_unr(g2);
                      else
                         dmd.result += gap_bitset_or_count(blk, g2);
                      break;
@@ -264,7 +262,7 @@ void combine_count_operation_with_block(const bm::word_t*           blk,
                      break;
                  case bm::COUNT_XOR:
                      if (!blk)
-                        dmd.result += gap_bit_count(g2);
+                        dmd.result += gap_bit_count_unr(g2);
                      else
                         dmd.result += gap_bitset_xor_count(blk, g2); 
                     break;
@@ -272,14 +270,15 @@ void combine_count_operation_with_block(const bm::word_t*           blk,
                     if (blk)
                     {
                         dmd.result += 
-                            bit_block_calc_count(blk, 
-                                                 blk + bm::set_block_size);
+                            bit_block_count(blk);
                     }
                     break;
                  case bm::COUNT_B:
                     if (g2)
-                        dmd.result += gap_bit_count(g2);
+                        dmd.result += gap_bit_count_unr(g2);
                     break;
+                 default:
+                     BM_ASSERT(0);
                  } // switch
                                      
              } // for it
@@ -292,13 +291,6 @@ void combine_count_operation_with_block(const bm::word_t*           blk,
      //
      // Here we combine two plain bitblocks 
 
-     //const bm::word_t* blk_end;
-     //const bm::word_t* arg_end;
-
-     //blk_end = blk + (bm::set_block_size);
-     //arg_end = arg_blk + (bm::set_block_size);
-
-
      for (distance_metric_descriptor* it = dmit; it < dmit_end; ++it)
      {
          distance_metric_descriptor& dmd = *it;
@@ -306,7 +298,7 @@ void combine_count_operation_with_block(const bm::word_t*           blk,
             operation_functions<true>::bit_operation_count(dmd.metric);
         if (gfunc)
         {
-            dmd.result += (*gfunc)(blk, blk + bm::set_block_size, arg_blk);
+            dmd.result += (*gfunc)(blk, arg_blk);
         }
         else
         {
@@ -314,14 +306,17 @@ void combine_count_operation_with_block(const bm::word_t*           blk,
             {
             case bm::COUNT_A:
                 if (blk)
-                    dmd.result += bit_block_calc_count(blk, blk + bm::set_block_size);
+                    dmd.result += bm::bit_block_count(blk);
                 break;
             case bm::COUNT_B:
                 if (arg_blk)
-                    dmd.result += 
-                        bit_block_calc_count(arg_blk, 
-                                             arg_blk + bm::set_block_size);
+                    dmd.result += bm::bit_block_count(arg_blk);
                 break;
+            case bm::COUNT_AND:
+            case bm::COUNT_XOR:
+            case bm::COUNT_OR:
+            case bm::COUNT_SUB_AB:
+            case bm::COUNT_SUB_BA:
             default:
                 BM_ASSERT(0);
             } // switch
@@ -363,7 +358,7 @@ unsigned combine_count_and_operation_with_block(const bm::word_t* blk,
     // --------------------------------------------
     // Here we combine two plain bitblocks 
 
-    return bit_operation_and_count(blk, blk + (bm::set_block_size), arg_blk);
+    return bit_operation_and_count(blk, arg_blk);
 }
 
 
@@ -376,7 +371,7 @@ inline
 void combine_any_operation_with_block(const bm::word_t* blk,
                                       unsigned gap,
                                       const bm::word_t* arg_blk,
-                                      int arg_gap,
+                                      unsigned arg_gap,
                                       distance_metric_descriptor* dmit,
                                       distance_metric_descriptor* dmit_end)
                                             
@@ -424,9 +419,11 @@ void combine_any_operation_with_block(const bm::word_t* blk,
                  case bm::COUNT_B:
                     res = g2;
                     break;
+                 default:
+                     BM_ASSERT(0);
                  } // switch
                 if (res)
-                    dmd.result += !gap_is_all_zero(res, bm::gap_max_bits);
+                    dmd.result += !gap_is_all_zero(res);
                                      
              } // for it
              
@@ -451,18 +448,16 @@ void combine_any_operation_with_block(const bm::word_t* blk,
                      break;
                  case bm::COUNT_OR:
                      if (!arg_blk)
-                        dmd.result += !gap_is_all_zero(g1, bm::gap_max_bits);
+                        dmd.result += !gap_is_all_zero(g1);
                      else
                         dmd.result += gap_bitset_or_any(arg_blk, g1); 
                      break;
                  case bm::COUNT_SUB_AB:
                      {
-                     bm::word_t  BM_ALIGN16 temp_blk[bm::set_block_size] BM_ALIGN16ATTR;
+                     bm::word_t  BM_VECT_ALIGN temp_blk[bm::set_block_size] BM_VECT_ALIGN_ATTR;
                      gap_convert_to_bitset((bm::word_t*) temp_blk, g1);
                      dmd.result += 
-                       bit_operation_sub_any((bm::word_t*)temp_blk, 
-                          ((bm::word_t*)temp_blk) + bm::set_block_size,
-                           arg_blk);
+                       bit_operation_sub_any((bm::word_t*)temp_blk, arg_blk);
                      }
                      break;
                  case bm::COUNT_SUB_BA:
@@ -476,22 +471,23 @@ void combine_any_operation_with_block(const bm::word_t* blk,
                      break;
                  case bm::COUNT_XOR:
                      if (!arg_blk)
-                        dmd.result += !gap_is_all_zero(g1, bm::gap_max_bits);
+                        dmd.result += !gap_is_all_zero(g1);
                      else
                         dmd.result += gap_bitset_xor_any(arg_blk, g1);
                      break;
                  case bm::COUNT_A:
                     if (g1)
-                        dmd.result += !gap_is_all_zero(g1, bm::gap_max_bits);
+                        dmd.result += !gap_is_all_zero(g1);
                     break;
                  case bm::COUNT_B:
                     if (arg_blk)
                     {
                         dmd.result += 
-                          !bit_is_all_zero((bm::wordop_t*)arg_blk, 
-                                           (bm::wordop_t*)(arg_blk + bm::set_block_size));
+                          !bit_is_all_zero(arg_blk);
                     }
                     break;
+                 default:
+                     BM_ASSERT(0);
                  } // switch
                                      
              } // for it
@@ -520,7 +516,7 @@ void combine_any_operation_with_block(const bm::word_t* blk,
                      break;
                  case bm::COUNT_OR:
                      if (!blk)
-                        dmd.result += !gap_is_all_zero(g2, bm::gap_max_bits);
+                        dmd.result += !gap_is_all_zero(g2);
                      else
                         dmd.result += gap_bitset_or_any(blk, g2);
                      break;
@@ -539,7 +535,7 @@ void combine_any_operation_with_block(const bm::word_t* blk,
                      break;
                  case bm::COUNT_XOR:
                      if (!blk)
-                        dmd.result += !gap_is_all_zero(g2, bm::gap_max_bits);
+                        dmd.result += !gap_is_all_zero(g2);
                      else
                         dmd.result += gap_bitset_xor_any(blk, g2); 
                     break;
@@ -547,14 +543,15 @@ void combine_any_operation_with_block(const bm::word_t* blk,
                     if (blk)
                     {
                         dmd.result+=
-                            !bit_is_all_zero((bm::wordop_t*)blk, 
-                                              (bm::wordop_t*)blk + bm::set_block_size);
+                            !bm::bit_is_all_zero(blk);
                     }
                     break;
                  case bm::COUNT_B:
                     if (g2)
-                        dmd.result += !gap_is_all_zero(g2, bm::gap_max_bits);
+                        dmd.result += !gap_is_all_zero(g2);
                     break;
+                 default:
+                     BM_ASSERT(0);
                  } // switch
                                      
              } // for it
@@ -567,12 +564,6 @@ void combine_any_operation_with_block(const bm::word_t* blk,
      //
      // Here we combine two plain bitblocks 
 
-     const bm::word_t* blk_end;
-     const bm::word_t* arg_end;
-
-     blk_end = blk + (bm::set_block_size);
-     arg_end = arg_blk + (bm::set_block_size);
-
      for (distance_metric_descriptor* it = dmit; it < dmit_end; ++it)
      {
         distance_metric_descriptor& dmd = *it;
@@ -585,34 +576,34 @@ void combine_any_operation_with_block(const bm::word_t* blk,
         {
         case bm::COUNT_AND:
             dmd.result += 
-            bit_operation_and_any(blk, blk_end, arg_blk);
+            bit_operation_and_any(blk, arg_blk);
             break;
         case bm::COUNT_OR:
             dmd.result += 
-            bit_operation_or_any(blk, blk_end, arg_blk);
+            bit_operation_or_any(blk, arg_blk);
             break;
         case bm::COUNT_SUB_AB:
             dmd.result += 
-            bit_operation_sub_any(blk, blk_end, arg_blk);
+            bit_operation_sub_any(blk, arg_blk);
             break;
         case bm::COUNT_SUB_BA:
             dmd.result += 
-            bit_operation_sub_any(arg_blk, arg_end, blk);
+            bit_operation_sub_any(arg_blk, blk);
             break;
         case bm::COUNT_XOR:
             dmd.result += 
-            bit_operation_xor_any(blk, blk_end, arg_blk);
+            bit_operation_xor_any(blk, arg_blk);
             break;
         case bm::COUNT_A:
             if (blk)
-                dmd.result += !bit_is_all_zero((bm::wordop_t*)blk, 
-                                               (bm::wordop_t*)blk_end);
+                dmd.result += !bit_is_all_zero(blk);
             break;
         case bm::COUNT_B:
             if (arg_blk)
-                dmd.result += !bit_is_all_zero((bm::wordop_t*)arg_blk, 
-                                               (bm::wordop_t*)arg_end);
+                dmd.result += !bit_is_all_zero(arg_blk);
             break;
+        default:
+            BM_ASSERT(0);
         } // switch
 
      } // for it
@@ -627,16 +618,12 @@ void combine_any_operation_with_block(const bm::word_t* blk,
 */
 inline
 unsigned combine_count_operation_with_block(const bm::word_t* blk,
-//                                            unsigned gap,
                                             const bm::word_t* arg_blk,
-//                                            int arg_gap,
-                                            //bm::word_t* temp_blk,
                                             distance_metric metric)
 {
     distance_metric_descriptor dmd(metric);
     combine_count_operation_with_block(blk, //gap, 
                                        arg_blk, //arg_gap, 
-                                       //temp_blk,
                                        &dmd, &dmd+1);
     return dmd.result;
 }
@@ -651,7 +638,7 @@ inline
 unsigned combine_any_operation_with_block(const bm::word_t* blk,
                                           unsigned gap,
                                           const bm::word_t* arg_blk,
-                                          int arg_gap,
+                                          unsigned arg_gap,
                                           distance_metric metric)
 {
     distance_metric_descriptor dmd(metric);
@@ -701,7 +688,6 @@ void distance_stage(const distance_metric_descriptor* dmit,
     \ingroup  distance
     
 */
-
 template<class BV>
 void distance_operation(const BV& bv1, 
                         const BV& bv2, 
@@ -714,7 +700,7 @@ void distance_operation(const BV& bv1,
     bool is_all_and = true; // flag is distance operation is just COUNT_AND
     distance_stage(dmit, dmit_end, &is_all_and);
 
-    bm::word_t*** blk_root = bman1.get_rootblock();
+    bm::word_t*** blk_root = bman1.top_blocks_root();
     unsigned block_idx = 0;
     unsigned i, j;
     
@@ -723,14 +709,14 @@ void distance_operation(const BV& bv1,
 
     BM_SET_MMX_GUARD
 
-    unsigned effective_top_block_size = bman1.effective_top_block_size();
-    unsigned ebs2 = bman2.effective_top_block_size();
-    if (ebs2 > effective_top_block_size)
-        effective_top_block_size = ebs2;
+    unsigned top_block_size = bman1.top_block_size();
+    unsigned ebs2 = bman2.top_block_size();
+    if (ebs2 > top_block_size)
+        top_block_size = ebs2;
 
-    for (i = 0; i < effective_top_block_size; ++i)
+    for (i = 0; i < top_block_size; ++i)
     {
-        bm::word_t** blk_blk = blk_root[i];
+        bm::word_t** blk_blk = blk_root ? blk_root[i] : 0;
 
         if (blk_blk == 0) // not allocated
         {
@@ -762,20 +748,18 @@ void distance_operation(const BV& bv1,
 
         for (j = 0; j < bm::set_array_size; ++j, ++block_idx)
         {
-            blk = BLOCK_ADDR_SAN(blk_blk[j]);
+            blk = bman1.get_block(i, j);
             if (blk == 0 && is_all_and)
                 continue;
 
             arg_blk = bman2.get_block(i, j);
 
-            if (blk == 0 && arg_blk == 0)
+            if (!blk & !arg_blk)
                 continue;
                 
             combine_count_operation_with_block(blk, 
                                                arg_blk, 
                                                dmit, dmit_end);
-            
-
         } // for j
 
     } // for i
@@ -791,25 +775,24 @@ void distance_operation(const BV& bv1,
 \ingroup  distance
 
 */
-
 template<class BV>
 unsigned distance_and_operation(const BV& bv1, 
                                 const BV& bv2)
 {
     const typename BV::blocks_manager_type& bman1 = bv1.get_blocks_manager();
     const typename BV::blocks_manager_type& bman2 = bv2.get_blocks_manager();
+    
+    if (!bman1.is_init() || !bman2.is_init())
+        return 0;
 
-    bm::word_t*** blk_root = bman1.get_rootblock();
-    bm::word_t*** blk_root_arg = bman2.get_rootblock();
+    bm::word_t*** blk_root     = bman1.top_blocks_root();
+    bm::word_t*** blk_root_arg = bman2.top_blocks_root();
     unsigned count = 0;
 
-    BM_SET_MMX_GUARD
+    unsigned top_block_size =
+        bm::min_value(bman1.top_block_size(),bman2.top_block_size());
 
-    unsigned effective_top_block_size = 
-        bm::min_value(bman1.effective_top_block_size(), 
-                      bman2.effective_top_block_size());
-
-    for (unsigned i = 0; i < effective_top_block_size; ++i)
+    for (unsigned i = 0; i < top_block_size; ++i)
     {
         bm::word_t** blk_blk;
         bm::word_t** blk_blk_arg;
@@ -858,9 +841,7 @@ unsigned distance_and_operation(const BV& bv1,
                       computation is added to "result" field
     \param dmit_end - pointer to (last+1) element of metric descriptors array
     \ingroup  distance
-    
 */
-
 template<class BV>
 void distance_operation_any(const BV& bv1, 
                             const BV& bv2, 
@@ -873,7 +854,7 @@ void distance_operation_any(const BV& bv1,
     bool is_all_and = true; // flag is distance operation is just COUNT_AND
     distance_stage(dmit, dmit_end, &is_all_and);
   
-    bm::word_t*** blk_root = bman1.get_rootblock();
+    bm::word_t*** blk_root = bman1.top_blocks_root();
     unsigned block_idx = 0;
     unsigned i, j;
     
@@ -882,16 +863,14 @@ void distance_operation_any(const BV& bv1,
     bool  blk_gap;
     bool  arg_gap;
 
-    BM_SET_MMX_GUARD
+    unsigned top_block_size = bman1.top_block_size();
+    unsigned ebs2 = bman2.top_block_size();
+    if (ebs2 > top_block_size)
+        top_block_size = ebs2;
 
-    unsigned effective_top_block_size = bman1.effective_top_block_size();
-    unsigned ebs2 = bman2.effective_top_block_size();
-    if (ebs2 > effective_top_block_size)
-        effective_top_block_size = ebs2;
-
-    for (i = 0; i < effective_top_block_size; ++i)
+    for (i = 0; i < top_block_size; ++i)
     {
-        bm::word_t** blk_blk = blk_root[i];
+        bm::word_t** blk_blk = blk_root ? blk_root[i] : 0;
 
         if (blk_blk == 0) // not allocated
         {
@@ -915,10 +894,10 @@ void distance_operation_any(const BV& bv1,
             for (j = 0; j < bm::set_array_size; ++j,++block_idx)
             {                
                 arg_blk = bman2.get_block(i, j);
-                arg_gap = BM_IS_GAP(arg_blk);
-
                 if (!arg_blk) 
                     continue;
+                arg_gap = BM_IS_GAP(arg_blk);
+                
                 combine_any_operation_with_block(blk, blk_gap,
                                                  arg_blk, arg_gap,
                                                  dmit, dmit_end);
@@ -944,7 +923,8 @@ void distance_operation_any(const BV& bv1,
 
         for (j = 0; j < bm::set_array_size; ++j, ++block_idx)
         {
-            blk = BLOCK_ADDR_SAN(blk_blk[j]);
+            blk = bman1.get_block(i, j);
+            //BLOCK_ADDR_SAN(blk_blk[j]);
             if (blk == 0 && is_all_and)
                 continue;
 
@@ -1152,12 +1132,15 @@ template<class BV, class It>
 void combine_or(BV& bv, It  first, It last)
 {
     typename BV::blocks_manager_type& bman = bv.get_blocks_manager();
+    if (!bman.is_init())
+        bman.init_tree();
+    
     unsigned max_id = 0;
 
     while (first < last)
     {
         unsigned nblock = unsigned((*first) >> bm::set_block_shift);     
-        It right = block_range_scan(first, last, nblock, &max_id);
+        It right = bm::block_range_scan(first, last, nblock, &max_id);
 
         if (max_id >= bv.size())
         {
@@ -1189,7 +1172,7 @@ void combine_or(BV& bv, It  first, It last)
                 unsigned nbit   = (*first) & bm::set_block_mask; 
                 
                 unsigned new_block_len =
-                    gap_set_value(true, gap_blk, nbit, &is_set);
+                    bm::gap_set_value(true, gap_blk, nbit, &is_set);
                 if (new_block_len > threshold) 
                 {
                     bman.extend_gap_block(nblock, gap_blk);
@@ -1205,7 +1188,7 @@ void combine_or(BV& bv, It  first, It last)
                 unsigned nbit   = unsigned(*first & bm::set_block_mask); 
                 unsigned nword  = unsigned(nbit >> bm::set_word_shift); 
                 nbit &= bm::set_word_mask;
-                blk[nword] |= (bm::word_t)1 << nbit;
+                blk[nword] |= (1u << nbit);
             } // for
         }
     } // while
@@ -1231,6 +1214,9 @@ template<class BV, class It>
 void combine_xor(BV& bv, It  first, It last)
 {
     typename BV::blocks_manager_type& bman = bv.get_blocks_manager();
+    if (!bman.is_init())
+        bman.init_tree();
+    
     unsigned max_id = 0;
 
     while (first < last)
@@ -1267,7 +1253,7 @@ void combine_xor(BV& bv, It  first, It last)
                 unsigned is_set;
                 unsigned nbit   = (*first) & bm::set_block_mask; 
                 
-                is_set = gap_test(gap_blk, nbit);
+                is_set = bm::gap_test_unr(gap_blk, nbit);
                 BM_ASSERT(is_set <= 1);
                 is_set ^= 1; 
                 
@@ -1288,7 +1274,7 @@ void combine_xor(BV& bv, It  first, It last)
                 unsigned nbit   = unsigned(*first & bm::set_block_mask); 
                 unsigned nword  = unsigned(nbit >> bm::set_word_shift); 
                 nbit &= bm::set_word_mask;
-                blk[nword] ^= (bm::word_t)1 << nbit;
+                blk[nword] ^= (1u << nbit);
             } // for
         }
     } // while
@@ -1315,6 +1301,9 @@ template<class BV, class It>
 void combine_sub(BV& bv, It  first, It last)
 {
     typename BV::blocks_manager_type& bman = bv.get_blocks_manager();
+    if (!bman.is_init())
+        bman.init_tree();
+    
     unsigned max_id = 0;
 
     while (first < last)
@@ -1352,7 +1341,7 @@ void combine_sub(BV& bv, It  first, It last)
                 unsigned is_set;
                 unsigned nbit   = (*first) & bm::set_block_mask; 
                 
-                is_set = gap_test(gap_blk, nbit);
+                is_set = bm::gap_test_unr(gap_blk, nbit);
                 if (!is_set)
                     continue;
                 
@@ -1440,6 +1429,7 @@ void combine_and(BV& bv, It  first, It last)
     intervals of 1s and 0s.
     <pre>
     For example: 
+      empty vector   - 1 interval
       00001111100000 - gives us 3 intervals
       10001111100000 - 4 intervals
       00000000000000 - 1 interval
@@ -1452,8 +1442,11 @@ template<class BV>
 bm::id_t count_intervals(const BV& bv)
 {
     const typename BV::blocks_manager_type& bman = bv.get_blocks_manager();
+    
+    if (!bman.is_init())
+        return 1;
 
-    bm::word_t*** blk_root = bman.get_rootblock();
+    bm::word_t*** blk_root = bman.top_blocks_root();
     typename BV::blocks_manager_type::block_count_change_func func(bman);
     for_each_block(blk_root, bman.top_block_size(), func);
 
@@ -1474,21 +1467,24 @@ bm::id_t count_intervals(const BV& bv)
 
     \ingroup setalgo
 */
-template<class BV, class It>
+template<typename BV, class It>
 void export_array(BV& bv, It first, It last)
 {
     typename BV::blocks_manager_type& bman = bv.get_blocks_manager();
+    if (!bman.is_init())
+        bman.init_tree();
+    
     unsigned inp_word_size = sizeof(*first);
-    size_t array_size = last - first;
+    size_t array_size = size_t(last - first);
     size_t bit_cnt = array_size * inp_word_size * 8;
     int block_type;
     bm::word_t tmp;
-    unsigned b1, b2, b3, b4;
+    bm::word_t b1, b2, b3, b4;
 
     if (bit_cnt >= bv.size())
-		bv.resize((bm::id_t)bit_cnt + 1);
+        bv.resize((bm::id_t)bit_cnt + 1);
     else 
-		bv.set_range((bm::id_t)bit_cnt, bv.size() - 1, false);
+        bv.set_range((bm::id_t)bit_cnt, bv.size() - 1, false);
     
     switch (inp_word_size)
     {
@@ -1505,40 +1501,40 @@ void export_array(BV& bv, It first, It last)
                                               false /*no NULL ret*/);
                 if (block_type == 1) // gap
                 {
-                    blk = bman.convert_gap2bitset(i, BMGAP_PTR(blk));
+                    blk = bman.deoptimize_block(i);
                 }
                 
                 bm::word_t* wrd_ptr = blk;
                 if (word_cnt >= bm::set_block_size) {
                     bm::word_t* wrd_end = blk + bm::set_block_size;
                     do {
-                        b1 = *first++; b2 = *first++;
-                        b3 = *first++; b4 = *first++;
-                        tmp = b1 | (b2 << 8) | (b3 << 16) | (b4 << 24);
+                        b1 = bm::word_t(*first++); b2 = bm::word_t(*first++);
+                        b3 = bm::word_t(*first++); b4 = bm::word_t(*first++);
+                        tmp = b1 | (b2 << 8u) | (b3 << 16u) | (b4 << 24u);
                         *wrd_ptr++ = tmp;
                     } while (wrd_ptr < wrd_end);
                     word_cnt -= bm::set_block_size;
                 } 
                 else 
                 {
-                    size_t to_convert = last - first;
+                    size_t to_convert = size_t(last - first);
                     for (size_t j = 0; j < to_convert / 4; ++j)
                     {
-                        b1 = *first++; b2 = *first++;
-                        b3 = *first++; b4 = *first++;
-                        tmp = b1 | (b2 << 8) | (b3 << 16) | (b4 << 24);
+                        b1 = bm::word_t(*first++); b2 = bm::word_t(*first++);
+                        b3 = bm::word_t(*first++); b4 = bm::word_t(*first++);
+                        tmp = b1 | (b2 << 8u) | (b3 << 16u) | (b4 << 24u);
                         *wrd_ptr++ = tmp;
                     }
                     while (1)
                     {
                         if (first == last) break;
-                        *wrd_ptr = *first++;
+                        *wrd_ptr = bm::word_t(*first++);
                         if (first == last) break;
-                        *wrd_ptr |= (*first++) << 8;
+                        *wrd_ptr |= bm::word_t(*first++) << 8u;
                         if (first == last) break;
-                        *wrd_ptr |= (*first++) << 16;
+                        *wrd_ptr |= bm::word_t(*first++) << 16u;
                         if (first == last) break;
-                        *wrd_ptr |= (*first++) << 24;
+                        *wrd_ptr |= bm::word_t(*first++) << 24u;
                         ++wrd_ptr;
                     }
                 }
@@ -1557,16 +1553,14 @@ void export_array(BV& bv, It first, It last)
                                               BM_BIT, 
                                               &block_type,
                                               false /*no NULL ret*/);
-                if (block_type == 1) // gap
-                {
-                    blk = bman.convert_gap2bitset(i, BMGAP_PTR(blk));
-                }
+                if (block_type) // gap
+                    blk = bman.deoptimize_block(i);
                 
                 bm::word_t* wrd_ptr = blk;
                 if (word_cnt >= bm::set_block_size) {
                     bm::word_t* wrd_end = blk + bm::set_block_size;
                     do {
-                        b1 = *first++; b2 = *first++;
+                        b1 = bm::word_t(*first++); b2 = bm::word_t(*first++);
                         tmp = b1 | (b2 << 16);
                         *wrd_ptr++ = tmp;
                     } while (wrd_ptr < wrd_end);
@@ -1574,19 +1568,19 @@ void export_array(BV& bv, It first, It last)
                 } 
                 else 
                 {
-                    size_t to_convert = last - first;
+                    size_t to_convert = size_t(last - first);
                     for (unsigned j = 0; j < to_convert / 2; ++j)
                     {
-                        b1 = *first++; b2 = *first++;
-                        tmp = b1 | (b2 << 16);
+                        b1 = bm::word_t(*first++); b2 = bm::word_t(*first++);
+                        tmp = b1 | (b2 << 16u);
                         *wrd_ptr++ = tmp;
                     }
                     while (1)
                     {
                         if (first == last) break;
-                        *wrd_ptr = *first++;
+                        *wrd_ptr = bm::word_t(*first++);
                         if (first == last) break;
-                        *wrd_ptr |= (*first++) << 16;
+                        *wrd_ptr |= bm::word_t((*first++) << 16u);
                         ++wrd_ptr;
                     }
                 }
@@ -1606,15 +1600,14 @@ void export_array(BV& bv, It first, It last)
                                               &block_type,
                                               false /*no NULL ret*/);
                 if (block_type == 1) // gap
-                {
-                    blk = bman.convert_gap2bitset(i, BMGAP_PTR(blk));
-                }
+                    blk = bman.deoptimize_block(i);
                 
                 bm::word_t* wrd_ptr = blk;
                 if (word_cnt >= bm::set_block_size) {
                     bm::word_t* wrd_end = blk + bm::set_block_size;
-                    do {
-                        *wrd_ptr++ = *first++;
+                    do
+                    {
+                        *wrd_ptr++ = bm::word_t(*first++);
                     } while (wrd_ptr < wrd_end);
                     word_cnt -= bm::set_block_size;
                 } 
@@ -1623,7 +1616,7 @@ void export_array(BV& bv, It first, It last)
                     while (1)
                     {
                         if (first == last) break;
-                        *wrd_ptr = *first++;
+                        *wrd_ptr = bm::word_t(*first++);
                         ++wrd_ptr;
                     }
                 }
@@ -1637,6 +1630,67 @@ void export_array(BV& bv, It first, It last)
 
 }
 
+
+/*!
+   \brief for-each visitor, calls a special visitor functor for each 1 bit group
+ 
+   \param block - bit block buffer pointer
+   \param offset - global block offset (number of bits)
+   \param bit_functor - functor must support .add_bits(offset, bits_ptr, size)
+ 
+   @ingroup bitfunc
+   @internal
+*/
+template<class Func>
+void for_each_bit_blk(const bm::word_t* block, bm::id_t offset,
+                      Func&  bit_functor)
+{
+    unsigned char bits[bm::set_bitscan_wave_size*32];
+
+    unsigned offs = offset;
+    unsigned cnt;
+    const word_t* block_end = block + bm::set_block_size;
+    do
+    {
+        cnt = bm::bitscan_wave(block, bits);
+        if (cnt)
+            bit_functor.add_bits(offs, bits, cnt);
+        offs += bm::set_bitscan_wave_size * 32;
+        block += bm::set_bitscan_wave_size;
+    } while (block < block_end);
+}
+
+
+/*!
+   \brief for-each visitor, calls a special visitor functor for each 1 bit range
+ 
+   \param block - bit block buffer pointer
+   \param offset - global block offset (number of bits)
+   \param bit_functor - functor must support .add_range(offset, bits_ptr, size)
+ 
+   @ingroup gapfunc
+   @internal
+*/
+template<typename T, typename Func>
+void for_each_gap_blk(const T* buf, bm::id_t offset,
+                      Func&  bit_functor)
+{
+    const T* pcurr = buf + 1;
+    const T* pend = buf + (*buf >> 3);
+
+    if (*buf & 1)
+    {
+        bit_functor.add_range(offset, *pcurr + 1);
+        ++pcurr;
+    }
+    for (++pcurr; pcurr <= pend; pcurr += 2)
+    {
+        T prev = *(pcurr-1);
+        bit_functor.add_range(offset + prev + 1, *pcurr - prev);
+    }
+}
+
+
 } // namespace bm
 
 #ifdef _MSC_VER
