@@ -1,8 +1,24 @@
-// Copyright(c) 2002-2009 Anatoliy Kuznetsov(anatoliy_kuznetsov at yahoo.com)
-//
-// BM library internal header
-//
-// Set all required preprocessor defines
+/*
+Copyright(c) 2002-2017 Anatoliy Kuznetsov(anatoliy_kuznetsov at yahoo.com)
+
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
+
+    http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
+
+For more information please visit:  http://bitmagic.io
+*/
+
+/*! \file bmdef.h
+    \brief Definitions(internal)
+*/
 
 #include <climits>
 
@@ -41,7 +57,7 @@
 // macro to define/undefine unaligned memory access (x86, PowerPC)
 //
 #if defined(__i386) || defined(__x86_64) || defined(__ppc__) || \
-	defined(__ppc64__) || defined(_M_IX86) || defined(_M_AMD64) || \
+    defined(__ppc64__) || defined(_M_IX86) || defined(_M_AMD64) || \
     defined(_M_IX86) || defined(_M_AMD64) || defined(_M_X64) || \
     (defined(_M_MPPC) && !defined(BM_FORBID_UNALIGNED_ACCESS))
 #define BM_UNALIGNED_ACCESS_OK 1
@@ -53,6 +69,23 @@
 #define BM_x86
 #endif
 
+// cxx11 features
+//
+#if defined(BM_NO_CXX11)  ||  (defined(_MSC_VER)  &&  _MSC_VER < 1900)
+# define BMNOEXEPT
+#else
+# ifndef BMNOEXEPT
+#  define BMNOEXEPT noexcept
+# endif
+#endif
+
+
+// disable 'register' keyword, which is obsolete in C++11
+//
+#ifndef BMREGISTER
+# define BMREGISTER
+#endif
+
 
 // Enable MSVC 8.0 (2005) specific optimization options
 //
@@ -75,6 +108,7 @@
 #  endif
 #endif
 
+
 #ifndef BM_ASSERT
 
 # ifndef BM_NOASSERT
@@ -89,15 +123,24 @@
 #endif
 
 
+#if defined(__x86_64) || defined(_M_AMD64) || defined(_WIN64) || \
+    defined(__LP64__) || defined(_LP64) || ( __WORDSIZE == 64 )
+#ifndef BM64OPT
+# define BM64OPT
+#endif
+#endif
+
+
+
 #define FULL_BLOCK_REAL_ADDR bm::all_set<true>::_block._p
 #define FULL_BLOCK_FAKE_ADDR bm::all_set<true>::_block._p_fullp
 #define BLOCK_ADDR_SAN(addr) (addr == FULL_BLOCK_FAKE_ADDR) ? FULL_BLOCK_REAL_ADDR : addr
-//#define IS_VALID_ADDR(addr) bool(addr && (addr != FULL_BLOCK_ADDR)
 #define IS_VALID_ADDR(addr) bm::all_set<true>::is_valid_block_addr(addr)
-//#define IS_FULL_BLOCK(addr) bool(addr == FULL_BLOCK_ADDR)
 #define IS_FULL_BLOCK(addr) bm::all_set<true>::is_full_block(addr)
 #define IS_EMPTY_BLOCK(addr) bool(addr == 0)
 
+#define BM_BLOCK_TYPE(addr) bm::all_set<true>::block_type(addr)
+
 // Macro definitions to manipulate bits in pointers
 // This trick is based on the fact that pointers allocated by malloc are
 // aligned and bit 0 is never set. It means we are safe to use it.
@@ -148,17 +191,42 @@
 
 
 // --------------------------------
-// SSE optmization
+// SSE optmization macros
 //
 
-#if !(defined(BMSSE2OPT) || defined(BMSSE42OPT)) 
+#ifdef BMSSE42OPT
+# if defined(BM64OPT) || defined(__x86_64) || defined(_M_AMD64) || defined(_WIN64) || \
+    defined(__LP64__) || defined(_LP64)
+#   undef BM64OPT
+#   define BM64_SSE4
+# endif
+# undef BMSSE2OPT
+#endif
+
+#ifdef BMAVX2OPT
+# if defined(BM64OPT) || defined(__x86_64) || defined(_M_AMD64) || defined(_WIN64) || \
+    defined(__LP64__) || defined(_LP64)
+#   undef BM64OPT
+#   undef  BM64_SSE4
+#   define BM64_AVX2
+# endif
+# undef BMSSE2OPT
+# undef BMSSE42OPT
+#endif
+
 
 # ifndef BM_SET_MMX_GUARD
 #  define BM_SET_MMX_GUARD
 # endif
 
+
+#if !(defined(BMSSE2OPT) || defined(BMSSE42OPT) || defined(BMAVX2OPT))
+
+
 #define BM_ALIGN16 
 #define BM_ALIGN16ATTR
+#define BM_ALIGN32
+#define BM_ALIGN32ATTR
 
 #else  
 
@@ -173,6 +241,12 @@
 #  define BM_ALIGN16ATTR
 #endif
 
+#ifndef BM_ALIGN32
+#  define BM_ALIGN32 __declspec(align(32))
+#  define BM_ALIGN32ATTR
+#endif
+
+
 # else // GCC
 
 #ifndef BM_ALIGN16
@@ -180,19 +254,41 @@
 #  define BM_ALIGN16ATTR __attribute__((aligned(16)))
 #endif
 
+#ifndef BM_ALIGN32
+#  define BM_ALIGN32
+#  define BM_ALIGN32ATTR __attribute__((aligned(32)))
+#endif
+
+
 #endif
 
 #endif
 
+#if (defined(BMSSE2OPT) || defined(BMSSE42OPT))
+#   define BM_VECT_ALIGN BM_ALIGN16
+#   define BM_VECT_ALIGN_ATTR BM_ALIGN16ATTR
+#else
+#   if defined(BMAVX2OPT)
+#       define BM_VECT_ALIGN BM_ALIGN32
+#       define BM_VECT_ALIGN_ATTR BM_ALIGN32ATTR
+#   else
+#       define BM_VECT_ALIGN
+#       define BM_VECT_ALIGN_ATTR
+#   endif
+#endif
+
+
+
+
 /*! 
-	Define calculates number of 1 bits in 32-bit word.
+    Define calculates number of 1 bits in 32-bit word.
     @ingroup bitfunc 
 */
 #ifndef BM_INCWORD_BITCOUNT
 
-#ifdef BMSSE42OPT
+#if (defined(BMSSE42OPT) || defined(BMAVX2OPT))
 
-# define BM_INCWORD_BITCOUNT(cnt, w) cnt += _mm_popcnt_u32(w);
+# define BM_INCWORD_BITCOUNT(cnt, w) cnt += unsigned(_mm_popcnt_u32(w));
 
 #else
 
@@ -202,8 +298,17 @@
      bm::bit_count_table<true>::_count[(unsigned char)((w) >> 16)] + \
      bm::bit_count_table<true>::_count[(unsigned char)((w) >> 24)];
 
+
 #endif
 
+// throw redefinintion for compatibility with language wrappers
+//
+#ifndef BM_ASSERT_THROW
+#define BM_ASSERT_THROW(x, xerrcode)
+#endif
+
+
+
 #endif
 
 
