@@ -1,36 +1,34 @@
 #ifndef BMTRANS__H__INCLUDED__
 #define BMTRANS__H__INCLUDED__
-
 /*
-Copyright(c) 2002-2009 Anatoliy Kuznetsov(anatoliy_kuznetsov at yahoo.com)
+Copyright(c) 2002-2017 Anatoliy Kuznetsov(anatoliy_kuznetsov at yahoo.com)
 
-Permission is hereby granted, free of charge, to any person 
-obtaining a copy of this software and associated documentation 
-files (the "Software"), to deal in the Software without restriction, 
-including without limitation the rights to use, copy, modify, merge, 
-publish, distribute, sublicense, and/or sell copies of the Software, 
-and to permit persons to whom the Software is furnished to do so, 
-subject to the following conditions:
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
 
-The above copyright notice and this permission notice shall be included 
-in all copies or substantial portions of the Software.
+    http://www.apache.org/licenses/LICENSE-2.0
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
-EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
-OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
-DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
-ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
-OTHER DEALINGS IN THE SOFTWARE.
-
-For more information please visit:  http://bmagic.sourceforge.net
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
 
+For more information please visit:  http://bitmagic.io
 */
+
+/*! \file bmtrans.h
+    \brief Utilities for bit transposition (internal) (experimental!)
+*/
+
 #ifdef _MSC_VER
 #pragma warning( push )
 #pragma warning( disable : 4311 4312 4127)
 #endif
 
+#include "bmdef.h"
+
 namespace bm
 {
 
@@ -43,7 +41,7 @@ struct tmatrix
 {
     typedef T value_type;
     
-    T BM_ALIGN16 value[ROWS][COLS] BM_ALIGN16ATTR;
+    T BM_VECT_ALIGN value[ROWS][COLS] BM_VECT_ALIGN_ATTR;
 
     enum params
     {
@@ -63,8 +61,16 @@ struct tmatrix
     static unsigned rows() { return ROWS; }
     static unsigned cols() { return COLS; }
 
-    const T* row(unsigned row_idx) const { return value[row_idx]; }
-          T* row(unsigned row_idx)       { return value[row_idx]; }
+    const T* row(unsigned row_idx) const
+    {
+        BM_ASSERT(row_idx < ROWS);
+        return value[row_idx];
+    }
+    T* row(unsigned row_idx)
+    {
+        BM_ASSERT(row_idx < ROWS);
+        return value[row_idx];
+    }
 };
 
 
@@ -129,22 +135,22 @@ struct bit_grabber<unsigned short, 16>
     static
     unsigned get(const unsigned short* arr, unsigned j)
     {
-        return  (((arr[0] >> j) & 1) << 0) |
-                (((arr[1] >> j) & 1) << 1) |
-                (((arr[2] >> j) & 1) << 2) |
-                (((arr[3] >> j) & 1) << 3) |
-                (((arr[4] >> j) & 1) << 4) |
-                (((arr[5] >> j) & 1) << 5) |
-                (((arr[6] >> j) & 1) << 6) |
-                (((arr[7] >> j) & 1) << 7) |
-                (((arr[8] >> j) & 1) << 8) |
-                (((arr[9] >> j) & 1) << 9) |
-                (((arr[10]>> j) & 1) << 10)|
-                (((arr[11]>> j) & 1) << 11)|
-                (((arr[12]>> j) & 1) << 12)|
-                (((arr[13]>> j) & 1) << 13)|
-                (((arr[14]>> j) & 1) << 14)|
-                (((arr[15]>> j) & 1) << 15);
+        return  (((arr[0] >> j) & 1u) << 0u) |
+                (((arr[1] >> j) & 1u) << 1u) |
+                (((arr[2] >> j) & 1u) << 2u) |
+                (((arr[3] >> j) & 1u) << 3u) |
+                (((arr[4] >> j) & 1u) << 4u) |
+                (((arr[5] >> j) & 1u) << 5u) |
+                (((arr[6] >> j) & 1u) << 6u) |
+                (((arr[7] >> j) & 1u) << 7u) |
+                (((arr[8] >> j) & 1u) << 8u) |
+                (((arr[9] >> j) & 1u) << 9u) |
+                (((arr[10]>> j) & 1u) << 10u)|
+                (((arr[11]>> j) & 1u) << 11u)|
+                (((arr[12]>> j) & 1u) << 12u)|
+                (((arr[13]>> j) & 1u) << 13u)|
+                (((arr[14]>> j) & 1u) << 14u)|
+                (((arr[15]>> j) & 1u) << 15u);
     }
 };
 
@@ -155,14 +161,15 @@ struct bit_grabber<unsigned char, 8>
     static
     unsigned get(const unsigned char* arr, unsigned j)
     {
-        return  (((arr[0] >> j) & 1) << 0) |
+        return  unsigned(
+                (((arr[0] >> j) & 1) << 0) |
                 (((arr[1] >> j) & 1) << 1) |
                 (((arr[2] >> j) & 1) << 2) |
                 (((arr[3] >> j) & 1) << 3) |
                 (((arr[4] >> j) & 1) << 4) |
                 (((arr[5] >> j) & 1) << 5) |
                 (((arr[6] >> j) & 1) << 6) |
-                (((arr[7] >> j) & 1) << 7);
+                (((arr[7] >> j) & 1) << 7));
     }
 };
 
@@ -355,14 +362,14 @@ void tmatrix_distance(const T  tmatrix[BPC][BPS],
     for (unsigned i = 0; i < BPC; ++i)
     {
         const T* r1 = tmatrix[i];
-        const T* r1_end = r1 + BPS;
+//        const T* r1_end;// = r1 + BPS;
         distance[i][i] = 
-            bm::bit_block_calc_count((bm::word_t*)r1, (bm::word_t*)r1_end);
+            bm::bit_block_count((bm::word_t*)r1);
 
         for (unsigned j = i + 1; j < BPC; ++j)
         {
             r1 = tmatrix[i];
-            r1_end = r1 + BPS;
+            //r1_end = r1 + BPS;
             unsigned count = 0;
 
             {
@@ -435,6 +442,8 @@ void bit_iblock_make_pcv(
         case total_possible_max:
             pc_vector[i] = ibpc_all_one; 
             continue;
+        default:
+            break;
         }
         
         // Dense-populated set, leave it as is
@@ -606,12 +615,12 @@ void tmatrix_restore(TMatrix& tmatrix,
     typedef typename TMatrix::value_type value_type;
 
     unsigned cols = effective_cols ? effective_cols : tmatrix.cols();
-    for (int row = tmatrix.rows()-1; row >= 0; --row)
+    for (unsigned row = tmatrix.rows()-1u; 1; --row)
     {
-        unsigned ibpc = pc_vector[row] & 7;  
-        int n_row = pc_vector[row] >> 3;
+        unsigned ibpc = pc_vector[row] & 7u;
+        unsigned n_row = pc_vector[row] >> 3u;
 
-        value_type* r1 = tmatrix.row(row);
+        value_type* r1 = tmatrix.row(unsigned(row));
 
         switch(ibpc)
         {
@@ -645,6 +654,9 @@ void tmatrix_restore(TMatrix& tmatrix,
             BM_ASSERT(0);
             break;
         } // switch
+        
+        if (row == 0)
+            break;
     }  // for
 
 }
@@ -716,8 +728,8 @@ void compute_tmatrix_rstat(const TMatrix& tmatrix,
             bm::bit_count_change32((bm::word_t*)r1, (bm::word_t*)r1_end, 
                                     &rstat[i].bit_count, &rstat[i].gap_count);
 
-            const unsigned bitset_size = sizeof(value_type) * cols;
-            const unsigned total_possible_max_bits = sizeof(value_type)*8*cols;
+            const unsigned bitset_size = unsigned(sizeof(value_type) * cols);
+            const unsigned total_possible_max_bits = unsigned(sizeof(value_type)*8*cols);
 
             rstat[i].best_rep = 
                 bm::best_representation(rstat[i].bit_count,
@@ -891,6 +903,7 @@ public:
 
 } // namespace bm
 
+
 #ifdef _MSC_VER
 #pragma warning( pop )
 #endif
