@@ -1,31 +1,28 @@
 #ifndef BMVMIN__H__INCLUDED__
 #define BMVMIN__H__INCLUDED__
 /*
-Copyright(c) 2002-2005 Anatoliy Kuznetsov(anatoliy_kuznetsov at yahoo.com)
+Copyright(c) 2002-2017 Anatoliy Kuznetsov(anatoliy_kuznetsov at yahoo.com)
 
-Permission is hereby granted, free of charge, to any person 
-obtaining a copy of this software and associated documentation 
-files (the "Software"), to deal in the Software without restriction, 
-including without limitation the rights to use, copy, modify, merge, 
-publish, distribute, sublicense, and/or sell copies of the Software, 
-and to permit persons to whom the Software is furnished to do so, 
-subject to the following conditions:
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
 
-The above copyright notice and this permission notice shall be included 
-in all copies or substantial portions of the Software.
+    http://www.apache.org/licenses/LICENSE-2.0
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
-EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
-OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
-DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
-ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
-OTHER DEALINGS IN THE SOFTWARE.
-
-For more information please visit:  http://bmagic.sourceforge.net
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
 
+For more information please visit:  http://bitmagic.io
 */
 
+/*! \file bmvmin.h
+    \brief Mini bitset for testing and utility purposes (internal)
+*/
+
+
 #ifdef _MSC_VER
 #pragma warning( push )
 #pragma warning( disable : 4100)
@@ -49,10 +46,7 @@ namespace bm
 
 /*!
     @brief Template class implements memory saving set functionality
-    
-    Template can be used as template parameter for bvector if we 
-    want to tune bvector for minimal memory consumption.
-
+    @ingroup mset
     @sa bvmini
 */
 template <class A, size_t N> class miniset
@@ -177,7 +171,7 @@ private:
     void init_gapbuf(bm::word_t* buf)
     {
         unsigned arr_size = 
-            BM_MINISET_GAPLEN / (sizeof(bm::word_t) / sizeof(bm::gap_word_t));
+            unsigned(BM_MINISET_GAPLEN / (sizeof(bm::word_t) / sizeof(bm::gap_word_t)));
         m_buf = A::allocate(arr_size, 0);
         if (buf)
         {
@@ -198,7 +192,7 @@ private:
 
         gap_convert_to_bitset(buf, (gap_word_t*) m_buf, arr_size);
         arr_size = 
-            BM_MINISET_GAPLEN / (sizeof(bm::word_t) / sizeof(bm::gap_word_t));
+            unsigned(BM_MINISET_GAPLEN / (sizeof(bm::word_t) / sizeof(bm::gap_word_t)));
         A::deallocate(m_buf, arr_size);
         m_buf = buf;
         m_type = 0;
@@ -212,10 +206,7 @@ private:
 
 /*!
     @brief Mini bitvector used in bvector template to keep block type flags
-    
-    Template is used as a default template parameter MS for bvector  
-    Offers maximum performance comparing to miniset.
-
+    @ingroup mset
     @sa miniset
 */
 template<size_t N> class bvmini
@@ -323,18 +314,18 @@ public:
         unsigned char  mask = (unsigned char)(0x1 << (pos & 7));
         unsigned char* offs = (unsigned char*)m_buf + (pos >> 3); 
 
-        *offs &= ~mask;
+        *offs &= (unsigned char)~mask;
     }
 
     /// Counts number of bits ON 
     unsigned bit_count() const
     {
-        register unsigned count = 0;
+        unsigned count = 0;
         const unsigned* end = m_buf + (m_size / 32)+1;    
 
         for (unsigned* start = m_buf; start < end; ++start)
         {
-            register unsigned value = *start;
+            unsigned value = *start;
             for (count += (value!=0); value &= value - 1; ++count);
         }
         return count;
@@ -384,21 +375,19 @@ public:
         unsigned pos = 0;
         const unsigned char* ptr = (unsigned char*) m_buf;
 
-        for (unsigned i = 0; i < (m_size/8)+1; ++i)
+        for (unsigned i = 0; i < ((m_size/8)+1); ++i)
         {
-            register unsigned char w = ptr[i];
-
-
+            unsigned char w = ptr[i];
             if (w != 0)
             {
-                while ((w & 1) == 0)
+                while ((w & 1u) == 0)
                 {
-                    w >>= 1;
+                    w = (unsigned char)(w >> 1u);
                     ++pos;
                 }
                 return pos;
             }
-            pos += sizeof(unsigned char) * 8;
+            pos = unsigned(pos + sizeof(unsigned char) * 8u);
         }
         return 0;
     }
@@ -407,7 +396,7 @@ public:
     /// Returns index of next bit, which is ON
     unsigned get_next(unsigned idx) const
     {
-        register unsigned i;
+        unsigned i;
 
         for (i = idx+1; i < m_size; ++i)
         {
@@ -482,7 +471,7 @@ public:
     const unsigned* get_buf() const { return m_buf; }
     unsigned mem_used() const
     {
-        return sizeof(bvector_mini) + (m_size / 32) + 1;
+        return unsigned(sizeof(bvector_mini) + (m_size / 32) + 1);
     }
 
     void swap(bvector_mini& bvm)
