@@ -1,7 +1,7 @@
 #ifndef CHECKSUM__HPP
 #define CHECKSUM__HPP
 
-/*  $Id: checksum.hpp 573617 2018-10-30 14:36:09Z ivanov $
+/*  $Id: checksum.hpp 578926 2019-01-28 18:17:50Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -436,7 +436,7 @@ public:
     };
 
     /// Translate from an error code value to its string representation.
-    virtual const char* GetErrCodeString(void) const;
+    virtual const char* GetErrCodeString(void) const override;
 
     // Standard exception boilerplate code.
     NCBI_EXCEPTION_DEFAULT(CChecksumException, CCoreException);
