@@ -1,7 +1,7 @@
 #ifndef UTIL___FILE_MANIFEST_HPP__
 #define UTIL___FILE_MANIFEST_HPP__
 
-/*  $Id: file_manifest.hpp 492854 2016-02-22 16:33:44Z elisovdn $
+/*  $Id: file_manifest.hpp 578926 2019-01-28 18:17:50Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -99,7 +99,7 @@ public:
         , eInvalidFilePath
     };
 
-    virtual const char* GetErrCodeString() const
+    virtual const char* GetErrCodeString() const override
     {
         switch (GetErrCode()) {
             case eEmptyManifestName: 
