@@ -1,6 +1,6 @@
 #ifndef CORELIB___NCBI_CACHE__HPP
 #define CORELIB___NCBI_CACHE__HPP
-/*  $Id: ncbi_cache.hpp 560849 2018-03-28 12:46:32Z ivanov $
+/*  $Id: ncbi_cache.hpp 578926 2019-01-28 18:17:50Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -311,7 +311,7 @@ public:
         eOtherError
     };
 
-    virtual const char* GetErrCodeString(void) const;
+    virtual const char* GetErrCodeString(void) const override;
 
     NCBI_EXCEPTION_DEFAULT(CCacheException, CException);
 };
