@@ -1,7 +1,7 @@
 #ifndef UTIL___ROW_READER_IANA_CSV__HPP
 #define UTIL___ROW_READER_IANA_CSV__HPP
 
-/*  $Id: row_reader_iana_csv.hpp 564203 2018-05-23 12:13:23Z ivanov $
+/*  $Id: row_reader_iana_csv.hpp 578926 2019-01-28 18:17:50Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -56,7 +56,7 @@ public:
         eUnexpectedDoubleQuote
     };
 
-    virtual const char * GetErrCodeString(void) const
+    virtual const char * GetErrCodeString(void) const override
     {
         switch (GetErrCode()) {
             case eUnbalancedDoubleQuote:
