@@ -1,7 +1,7 @@
 #ifndef UTIL___ROW_READER_IANA_TSV__HPP
 #define UTIL___ROW_READER_IANA_TSV__HPP
 
-/*  $Id: row_reader_iana_tsv.hpp 538310 2017-06-09 13:27:14Z satskyse $
+/*  $Id: row_reader_iana_tsv.hpp 578926 2019-01-28 18:17:50Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -82,7 +82,7 @@ public:
         eValidateConversion = 7
     };
 
-    virtual const char * GetErrCodeString(void) const
+    virtual const char * GetErrCodeString(void) const override
     {
         switch (GetErrCode()) {
             case eNumberOfFieldsMismatch:
