@@ -1,7 +1,7 @@
 #ifndef UTIL___SYNC_QUEUE__HPP
 #define UTIL___SYNC_QUEUE__HPP
 
-/*  $Id: sync_queue.hpp 534859 2017-05-03 12:47:35Z ivanov $
+/*  $Id: sync_queue.hpp 578926 2019-01-28 18:17:50Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -998,7 +998,7 @@ public:
         eGuardedCopy
     };
 
-    virtual const char* GetErrCodeString(void) const;
+    virtual const char* GetErrCodeString(void) const override;
 
     NCBI_EXCEPTION_DEFAULT(CSyncQueueException, CException);
 };
