@@ -1,7 +1,7 @@
 #ifndef UTIL__THREAD_POOL__HPP
 #define UTIL__THREAD_POOL__HPP
 
-/*  $Id: thread_pool.hpp 400966 2013-05-24 17:50:38Z vakatov $
+/*  $Id: thread_pool.hpp 578926 2019-01-28 18:17:50Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -619,7 +619,7 @@ public:
                          ///< by means of methods of another ThreadPool or
                          ///< invalid parameters in the constructor
     };
-    virtual const char* GetErrCodeString(void) const;
+    virtual const char* GetErrCodeString(void) const override;
     NCBI_EXCEPTION_DEFAULT(CThreadPoolException, CException);
 };
 
