@@ -1,7 +1,7 @@
 #ifndef UTIL___VALUE_CONV_POLICY__HPP
 #define UTIL___VALUE_CONV_POLICY__HPP
 
-/* $Id: value_convert_policy.hpp 499250 2016-04-25 12:27:42Z gouriano $
+/* $Id: value_convert_policy.hpp 578926 2019-01-28 18:17:50Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NTOICE
@@ -47,7 +47,7 @@ public:
     enum EErrCode {
         eInvalidConversion
     };
-    virtual const char* GetErrCodeString(void) const
+    virtual const char* GetErrCodeString(void) const override
     {
         return "Invalid run-time type conversion."; 
     }
