@@ -1,6 +1,6 @@
 #! /bin/sh
 
-# $Id: check_make_cfg.sh 544511 2017-08-23 18:50:29Z ivanov $
+# $Id: check_make_cfg.sh 566095 2018-06-25 15:21:27Z ivanov $
 # Author:  Vladimir Ivanov, NCBI 
 #
 ###########################################################################
@@ -134,6 +134,7 @@ if test -n "$NCBI_AUTOMATED_BUILD"; then
          vs2012 ) signature="VS_2012"   ;;
          vs2013 ) signature="VS_2013"   ;;
          vs2015 ) signature="VS_2015"   ;;
+         vs2017 ) signature="VS_2017"   ;;
       esac
       signature="$signature-${x_cfg}"
       case "$x_cfg" in
@@ -188,7 +189,9 @@ res_list="\$res_script.list"
 res_concat="\$res_script.out"
 res_concat_err="\$res_script.out_err"
 
-is_db_load=false
+# Define both senses to accommodate shells lacking !
+is_run=false
+no_run=true
 
 
 ##  Printout USAGE info and exit
@@ -207,8 +210,6 @@ USAGE:  $x_script_name {run | concat | concat_err}
 
 ERROR:  \$1
 EOF_usage
-# Undocumented command:
-#     load_to_db  Load data about test to database keeping all statistics.
 
     exit 1
 }
@@ -228,6 +229,8 @@ method="\$1"
 case "\$method" in
 #----------------------------------------------------------
    run )
+      is_run=true
+      no_run=false
       # See RunTest() below
       ;;
 #----------------------------------------------------------
@@ -267,19 +270,26 @@ case "\$method" in
       ) >> \$res_concat_err
       exit 0
       ;;
-#----------------------------------------------------------
-   load_to_db )
-      is_db_load=true
-      # See RunTest() below
-      ;;
 #----------------------------------------------------------
    * )
-      Usage "Invalid method name."
+      Usage "Invalid method name \$method."
       ;;
 esac
 
 
-## Run
+# Check for automated build
+is_automated=false
+is_db_load=false
+if test -n "\$NCBI_AUTOMATED_BUILD"; then
+   is_automated=true
+   if test -n "\$NCBI_CHECK_DB_LOAD"; then
+      is_db_load=true
+   fi
+fi
+
+
+#//////////////////////////////////////////////////////////////////////////
+
 
 trap "touch \$check_dir/check.failed; exit 1"  1 2 15
 rm \$check_dir/check.failed \$check_dir/check.success > /dev/null 2>&1 
@@ -331,19 +341,22 @@ if test -z "\$NCBI_TEST_DATA_PATH"; then
 fi
 
 
+## Run
+
 count_ok=0
 count_err=0
 count_timeout=0
 count_absent=0
 configurations="$x_confs"
 
+if \$is_run; then
+    rm -f "\$res_journal"
+    rm -f "\$res_log"
+fi
 if \$is_db_load; then
     echo "--------------------------------------------------" >> "\$build_dir/test_stat_load.log" 2>&1
     echo "Loading tests from: \$check_dir"                    >> "\$build_dir/test_stat_load.log" 2>&1
     echo "--------------------------------------------------" >> "\$build_dir/test_stat_load.log" 2>&1
-else
-    rm -f "\$res_journal"
-    rm -f "\$res_log"
 fi
 
 
@@ -355,7 +368,7 @@ RunTest() {
     x_test="\$2"
     x_app="\$3"
     x_run="\${4:-\$x_app}"
-    x_real_name="\$5"
+    x_alias="\$5"
     x_name="\${5:-\$x_run}"  
     x_ext="\$6"
     x_timeout="\$7"
@@ -378,6 +391,8 @@ RunTest() {
     # Run test under all specified check tools   
     for tool in \$NCBI_CHECK_TOOLS; do
 
+        saved_phid=''
+
         tool_lo=\`echo \$tool | tr '[A-Z]' '[a-z]'\`
         tool_up=\`echo \$tool | tr '[a-z]' '[A-Z]'\`
         
@@ -390,40 +405,18 @@ RunTest() {
            x_test_out="\$x_work_dir/\$x_app.test_out\$x_ext"
            x_test_rep="\$x_work_dir/\$x_app.test_rep\$x_ext"
            x_boost_rep="\$x_work_dir/\$x_app.boost_rep\$x_ext"
-           x_applog_sh="\$x_work_dir/\$x_app.applog\$x_ext.sh"
         else
            x_cmd="[\$build_tree/\$build_cfg/\$x_wdir] \$tool_up \$x_name"
            x_test_out="\$x_work_dir/\$x_app.test_out\$x_ext.\$tool_lo"
            x_test_rep="\$x_work_dir/\$x_app.test_rep\$x_ext.\$tool_lo"
            x_boost_rep="\$x_work_dir/\$x_app.boost_rep\$x_ext.\$tool_lo"
-           x_applog_sh="\$x_work_dir/\$x_app.applog\$x_ext.\$tool_lo.sh"
         fi
 
-        if \$is_db_load; then
-           echo "\$x_path_app:" >> "\$build_dir/test_stat_load.log" 2>&1
-           case "$x_compiler" in
-             MSVC )
-                test_stat_load "\$(cygpath -w "\$x_test_rep")" "\$(cygpath -w "\$x_test_out")" "\$(cygpath -w "\$x_boost_rep")" "\$(cygpath -w "\$top_srcdir/build_info")" "\$(cygpath -w "\$x_applog_sh")" >> "\$build_dir/test_stat_load.log" 2>&1
-                ;;        
-             XCODE ) 
-                $NCBI/bin/_production/CPPCORE/test_stat_load "\$x_test_rep" "\$x_test_out" "\$x_boost_rep" "\$top_srcdir/build_info" >> "\$build_dir/test_stat_load.log" 2>&1
-                ;;
-           esac
-           if test -f \$x_applog_sh; then
-              chmod a+x \$x_applog_sh 2>&1
-              echo "\$x_path_app:" >> "\$build_dir/test_stat_load_applog.log" 2>&1
-              \$x_applog_sh        >> "\$build_dir/test_stat_load_applog.log" 2>&1
-           else
-              echo "Error generating \$x_applog_sh "
-           fi
-           continue
-        fi
-   
-        if test -n "\$NCBI_AUTOMATED_BUILD"; then
-           echo "\$signature"   >> "\$x_test_rep"
-           echo "\$x_wdir"      >> "\$x_test_rep"
-           echo "\$x_run"       >> "\$x_test_rep"
-           echo "\$x_real_name" >> "\$x_test_rep"
+        if \$is_run && \$is_automated; then
+           echo "\$signature" >> "\$x_test_rep"
+           echo "\$x_wdir"    >> "\$x_test_rep"
+           echo "\$x_run"     >> "\$x_test_rep"
+           echo "\$x_alias"   >> "\$x_test_rep"
            case "$x_compiler" in
               MSVC )
                  export NCBI_BOOST_REPORT_FILE="\$(cygpath -w "\$x_boost_rep")"
@@ -488,9 +481,28 @@ RunTest() {
 
 
            # Run check
+           
            rm -f check_exec.pid > /dev/null 2>&1
            start_time="\`date +'$x_date_format'\`"
            check_exec="\$root_dir/scripts/common/check/check_exec.sh"
+           
+           # Generate PHID and SID for a new test
+           logfile=\$NCBI_CONFIG__LOG__FILE
+           NCBI_CONFIG__LOG__FILE=
+           export NCBI_CONFIG__LOG__FILE
+           eval "\`ncbi_applog generate -phid -sid -format=shell-export | tr -d '\r'\`"
+           if \$is_run && \$is_db_load; then
+              # Use generated PHID for test statistics, and sub-PHID.1 for test itself
+              saved_phid=\$NCBI_LOG_HIT_ID
+              NCBI_LOG_HIT_ID=\$NCBI_LOG_HIT_ID.1
+              export NCBI_LOG_HIT_ID
+              # Create a file in the cirrent directory with initial sub-PHID
+              # (will be incremented by $CHECK_EXEC, if any)
+              echo "0" > \$NCBI_LOG_HIT_ID
+           fi
+           NCBI_CONFIG__LOG__FILE=\$logfile
+           export NCBI_CONFIG__LOG__FILE
+           
            \$check_exec $x_time \`eval echo \$xx_run\` > \$x_test_out.\$\$ 2>&1
            result=\$?
            stop_time="\`date +'$x_date_format'\`"
@@ -501,7 +513,7 @@ RunTest() {
                s/ ["][$][@]["].*$//
             }' \$x_test_out.\$\$ >> \$x_test_out
 
-           # RunID
+           # Compute RunID
            runpid='?'
            test -f check_exec.pid  &&  runpid="\`cat check_exec.pid\`"
            runid="\`date -u +%y%m%d%H%M%S\`-\$runpid-\`uname -n\`"
@@ -533,40 +545,40 @@ RunTest() {
               echo "DIS --  \$x_cmd"
               echo "DIS --  \$x_cmd" >> \$res_log
               count_absent=\`expr \$count_absent + 1\`
-              test -n "\$NCBI_AUTOMATED_BUILD" && echo "DIS" >> "\$x_test_rep"
+              \$is_automated && echo "DIS" >> "\$x_test_rep"
       
            elif grep NCBI_UNITTEST_SKIPPED \$x_test_out >/dev/null; then
               echo "SKP --  \$x_cmd"
               echo "SKP --  \$x_cmd" >> \$res_log
               count_absent=\`expr \$count_absent + 1\`
-              test -n "\$NCBI_AUTOMATED_BUILD" && echo "SKP" >> "\$x_test_rep"
+              \$is_automated && echo "SKP" >> "\$x_test_rep"
       
            elif grep NCBI_UNITTEST_TIMEOUTS_BUT_NO_ERRORS \$x_test_out >/dev/null; then
               echo "TO  --  \$x_cmd"
               echo "TO  --  \$x_cmd" >> \$res_log
               count_timeout=\`expr \$count_timeout + 1\`
-              test -n "\$NCBI_AUTOMATED_BUILD" && echo "TO" >> "\$x_test_rep"
+              \$is_automated && echo "TO" >> "\$x_test_rep"
 
            elif egrep "Maximum execution .* is exceeded" \$x_test_out >/dev/null; then
               echo "TO  --  \$x_cmd     (\$exec_time)"
               echo "TO  --  \$x_cmd     (\$exec_time)" >> \$res_log
               count_timeout=\`expr \$count_timeout + 1\`
-              test -n "\$NCBI_AUTOMATED_BUILD" && echo "TO" >> "\$x_test_rep"
+              \$is_automated && echo "TO" >> "\$x_test_rep"
 
            elif test \$result -eq 0; then
               echo "OK  --  \$x_cmd     (\$exec_time)"
               echo "OK  --  \$x_cmd     (\$exec_time)" >> \$res_log
               count_ok=\`expr \$count_ok + 1\`
-              test -n "\$NCBI_AUTOMATED_BUILD" && echo "OK" >> "\$x_test_rep"
+              \$is_automated && echo "OK" >> "\$x_test_rep"
 
            else
-               echo "ERR [\$result] --  \$x_cmd     (\$exec_time)"
-               echo "ERR [\$result] --  \$x_cmd     (\$exec_time)" >> \$res_log
-               count_err=\`expr \$count_err + 1\`
-               test -n "\$NCBI_AUTOMATED_BUILD" && echo "ERR" >> "\$x_test_rep"
+              echo "ERR [\$result] --  \$x_cmd     (\$exec_time)"
+              echo "ERR [\$result] --  \$x_cmd     (\$exec_time)" >> \$res_log
+              count_err=\`expr \$count_err + 1\`
+              \$is_automated && echo "ERR" >> "\$x_test_rep"
            fi
    
-           if test -n "\$NCBI_AUTOMATED_BUILD"; then
+           if \$is_automated; then
               echo "\$start_time" >> "\$x_test_rep"
               echo "\$result"     >> "\$x_test_rep"
               echo "\$exec_time"  >> "\$x_test_rep"
@@ -575,17 +587,39 @@ RunTest() {
               echo "\$runid"      >> "\$x_test_rep"
            fi
 
-        else  # Check existence of the test's application directory
-           # Test application is absent
-           echo "ABS --  \$x_cmd - \$x_test"
-           echo "ABS --  \$x_cmd - \$x_test" >> \$res_log
-           count_absent=\`expr \$count_absent + 1\`
-           if test -n "\$NCBI_AUTOMATED_BUILD"; then
-              echo "ABS" >> "\$x_test_rep"
-              echo "\`date +'$x_date_format'\`" >> "\$x_test_rep"
+        else  # Run test if it exist
+           if \$is_run; then
+              # Test application is absent
+              echo "ABS --  \$x_cmd - \$x_test"
+              echo "ABS --  \$x_cmd - \$x_test" >> \$res_log
+              count_absent=\`expr \$count_absent + 1\`
+              if \$is_automated; then
+                 echo "ABS" >> "\$x_test_rep"
+                 echo "\`date +'$x_date_format'\`" >> "\$x_test_rep"
+              fi
            fi
         fi
         
+        # Load test results to Database and Applog immediately after a test.
+        # Always load test results for automated builds on a 'run' command.
+        
+        if \$is_run && \$is_db_load; then
+           echo "\$x_path_app:" >> "\$build_dir/test_stat_load.log" 2>&1
+           NCBI_LOG_HIT_ID=\$saved_phid
+           export NCBI_LOG_HIT_ID
+           case "$x_compiler" in
+             MSVC )
+                test_stat_load "\$(cygpath -w "\$x_test_rep")" "\$(cygpath -w "\$x_test_out")" "\$(cygpath -w "\$x_boost_rep")" "\$(cygpath -w "\$top_srcdir/build_info")" >> "\$build_dir/test_stat_load.log" 2>&1
+                ;;        
+             XCODE ) 
+                $NCBI/bin/_production/CPPCORE/test_stat_load "\$x_test_rep" "\$x_test_out" "\$x_boost_rep" "\$top_srcdir/build_info" >> "\$build_dir/test_stat_load.log" 2>&1
+                ;;
+           esac
+        fi
+        if test \$is_run  -a  -n "\$saved_phid"; then
+            rm -f \$saved_phid* > /dev/null 2>&1
+        fi
+
     done  # Run test under all specified check tools   
 }
 
@@ -711,7 +745,7 @@ cat >> $x_out <<EOF
 PATH="\$saved_path"
 
 # Write result of the tests execution
-if ! \$is_db_load; then
+if \$is_run; then
    echo
    echo "Succeeded : \$count_ok"
    echo "Timeout   : \$count_timeout"
