@@ -1,6 +1,6 @@
 #! /bin/sh
 
-# $Id: check_make_unix.sh 544511 2017-08-23 18:50:29Z ivanov $
+# $Id: check_make_unix.sh 571786 2018-10-02 17:43:34Z ivanov $
 # Author:  Vladimir Ivanov, NCBI 
 #
 ###########################################################################
@@ -158,6 +158,9 @@ compile_dir="$x_compile_dir"
 bin_dir="$x_bin_dir"
 script="$x_out"
 cygwin=$cygwin
+signature="$x_signature"
+sendmail=''
+domain='@ncbi.nlm.nih.gov'
 
 res_journal="\$script.journal"
 res_log="\$script.log"
@@ -166,13 +169,11 @@ res_concat="\$script.out"
 res_concat_err="\$script.out_err"
 
 # Define both senses to accommodate shells lacking !
+is_run=false
+no_run=true
 is_report_err=false
 no_report_err=true
-is_db_load=false
-no_db_load=true
-signature="$x_signature"
-sendmail=''
-domain='@ncbi.nlm.nih.gov'
+
 
 # Include COMMON.SH
 . \${root_dir}/scripts/common/common.sh
@@ -198,7 +199,6 @@ ERROR:  \$1
 EOF_usage
 # Undocumented commands:
 #     report_err  Report failed tests directly to developers.
-#     load_to_db  Load data about test to database keeping all statistics.
 
     exit 1
 }
@@ -218,6 +218,8 @@ method="\$1"
 case "\$method" in
 #----------------------------------------------------------
    run )
+      is_run=true
+      no_run=false
       # See RunTest() below
       ;;
 #----------------------------------------------------------
@@ -282,20 +284,27 @@ case "\$method" in
       no_report_err=false
       # See RunTest() below
       ;;
-#----------------------------------------------------------
-   load_to_db )
-      is_db_load=true
-      no_db_load=false
-      rm -f "$x_build_dir/test_stat_load.log"
-      # See RunTest() below
-      ;;
 #----------------------------------------------------------
    * )
-      Usage "Invalid method name."
+      Usage "Invalid method name \$method."
       ;;
 esac
 
 
+# Check for automated build
+is_automated=false
+is_db_load=false
+if test -n "\$NCBI_AUTOMATED_BUILD"; then
+   is_automated=true
+   if test -n "\$NCBI_CHECK_DB_LOAD"; then
+      is_db_load=true
+   fi
+fi
+
+
+#//////////////////////////////////////////////////////////////////////////
+
+
 trap "touch $x_target_dir/check.failed; exit 1"  1 2 15
 rm $x_target_dir/check.failed $x_target_dir/check.success > /dev/null 2>&1 
 
@@ -336,6 +345,9 @@ if (valgrind --ncbi --help) >/dev/null 2>&1; then
     VALGRIND_CMD="--ncbi \$VALGRIND_CMD" # --ncbi must be the first option!
 fi
 
+LSAN_OPTIONS="suppressions=\$root_dir/scripts/common/check/lsan.supp"
+export LSAN_OPTIONS
+
 # Export some global vars
 top_srcdir="\$root_dir"
 export top_srcdir
@@ -349,8 +361,8 @@ if test -z "\$NCBI_EXPORT_PROJECT"; then
     export NCBI_CONFIG__LOG__FILE
 fi
 
-# Add additional necessaary directories to PATH: current, build, scripts, utility.
-PATH=".:\${build_dir}:\${root_dir}/scripts/common/impl:\$NCBI/bin/_production/CPPCORE:\${PATH}"
+# Add additional necessary directories to PATH: current, build, scripts, utility and $HOME/bin (for Ubuntu).
+PATH=".:\${build_dir}:\${root_dir}/scripts/common/impl:\$NCBI/bin/_production/CPPCORE:\$HOME/bin:\${PATH}"
 export PATH
 
 # Export bin and lib pathes
@@ -433,9 +445,10 @@ count_timeout=0
 count_absent=0
 count_total=0
 
-if \$no_report_err && \$no_db_load; then
+if \$is_run; then
    rm -f "\$res_journal"
    rm -f "\$res_log"
+   rm -f "$x_build_dir/test_stat_load.log"
 fi
 
 if test "\$NCBI_CHECK_SETLIMITS" != "0"; then
@@ -462,7 +475,7 @@ RunTest()
     x_test="\$2"
     x_app="\$3"
     x_run="\${4:-\$x_app}"
-    x_real_name="\$5"
+    x_alias="\$5"
     x_name="\${5:-\$x_run}"
     x_ext="\$6"
     x_timeout="\$7"
@@ -480,6 +493,8 @@ RunTest()
     # Run test under all specified check tools   
     for tool in \$NCBI_CHECK_TOOLS; do
 
+        saved_phid=''
+
         tool_lo=\`echo \$tool | tr '[A-Z]' '[a-z]'\`
         tool_up=\`echo \$tool | tr '[a-z]' '[A-Z]'\`
         
@@ -494,42 +509,21 @@ RunTest()
            x_test_out="\$x_work_dir/\$x_test.test_out\$x_ext"
            x_test_rep="\$x_work_dir/\$x_test.test_rep\$x_ext"
            x_boost_rep="\$x_work_dir/\$x_test.boost_rep\$x_ext"
-           x_applog_sh="\$x_work_dir/\$x_test.applog\$x_ext.sh"
         else
            #x_cmd="[\$x_work_dir_tail] \$tool_up \$x_name"
            x_test_out="\$x_work_dir/\$x_test.test_out\$x_ext.\$tool_lo"
            x_test_rep="\$x_work_dir/\$x_test.test_rep\$x_ext.\$tool_lo"
            x_boost_rep="\$x_work_dir/\$x_test.boost_rep\$x_ext.\$tool_lo"
-           x_applog_sh="\$x_work_dir/\$x_test.applog\$x_ext.\$tool_lo.sh"
         fi
 
-        if \$is_db_load; then
-            case \`uname -s\` in
-              CYGWIN* )
-                test_stat_load "\$(cygpath -w "\$x_test_rep")" "\$(cygpath -w "\$x_test_out")" "\$(cygpath -w "\$x_boost_rep")" "\$(cygpath -w "\$top_srcdir/build_info")" "\$(cygpath -w "\$x_applog_sh")" >> "$x_build_dir/test_stat_load.log" 2>&1 ;;
-              IRIX* )
-                test_stat_load.sh "\$x_test_rep" "\$x_test_out" "\$x_boost_rep" "\$top_srcdir/build_info" "\$x_applog_sh" >> "$x_build_dir/test_stat_load.log" 2>&1 ;;
-              * )
-                test_stat_load "\$x_test_rep" "\$x_test_out" "\$x_boost_rep" "\$top_srcdir/build_info" "\$x_applog_sh" >> "$x_build_dir/test_stat_load.log" 2>&1 ;;
-            esac
-            if test -f \$x_applog_sh; then
-               chmod a+x \$x_applog_sh 2>&1
-               \$x_applog_sh >> "$x_build_dir/test_stat_load_applog.log" 2>&1
-            else
-               echo "Error generating \$x_applog_sh "
-            fi
-            continue
-        fi
-        
-        if \$no_report_err; then
-           if test -n "\$NCBI_AUTOMATED_BUILD"; then
-               echo "\$signature \$NCBI_CHECK_OS_NAME" > "\$x_test_rep"
-               echo "\$x_work_dir_tail" >> "\$x_test_rep"
-               echo "\$x_run" >> "\$x_test_rep"
-               echo "\$x_real_name" >> "\$x_test_rep"
-               NCBI_BOOST_REPORT_FILE="\$x_boost_rep"
-               export NCBI_BOOST_REPORT_FILE
-           fi
+   
+        if \$is_run && \$is_automated; then
+           echo "\$signature \$NCBI_CHECK_OS_NAME" > "\$x_test_rep"
+           echo "\$x_work_dir_tail" >> "\$x_test_rep"
+           echo "\$x_run" >> "\$x_test_rep"
+           echo "\$x_alias" >> "\$x_test_rep"
+           NCBI_BOOST_REPORT_FILE="\$x_boost_rep"
+           export NCBI_BOOST_REPORT_FILE
         fi
 
         # Check existence of the test's application directory
@@ -547,7 +541,6 @@ RunTest()
                 # Fix empty parameters (replace "" to \"\", '' to \'\')
                 x_run_fix=\`echo "\$x_run" | sed -e 's/""/\\\\\\\\\\"\\\\\\\\\\"/g' -e "s/''/\\\\\\\\\\'\\\\\\\\\\'/g"\`
 
-
                 # Define check tool variables
                 NCBI_CHECK_TOOL=\`eval echo "\$"NCBI_CHECK_\${tool_up}""\`
                 case "\$tool_lo" in
@@ -607,11 +600,30 @@ RunTest()
                     fi
                 ) > \$x_test_out 2>&1
 
-                # Remove old core file if it exist (for clarity of the test)
+                # Remove old core file if any
                 corefile="\$x_work_dir/core"
                 rm -f "\$corefile" > /dev/null 2>&1
                 rm -f check_exec.pid > /dev/null 2>&1
 
+                # Generate PHID and SID, to use it by any application in the current test,
+                # and for loading test statistics later (test_stat_load -> ncbi_applog),
+                # to have same values in Applog.
+                logfile=\$NCBI_CONFIG__LOG__FILE
+                NCBI_CONFIG__LOG__FILE=
+                export NCBI_CONFIG__LOG__FILE
+                eval "\`ncbi_applog generate -phid -sid -format=shell-export | tr -d '\r'\`"
+                if \$is_run && \$is_db_load; then
+                   # Use generated PHID for test statistics, and sub-PHID.1 for test itself
+                   saved_phid=\$NCBI_LOG_HIT_ID
+                   NCBI_LOG_HIT_ID=\$NCBI_LOG_HIT_ID.1
+                   export NCBI_LOG_HIT_ID
+                   # Create a file in the cirrent directory with initial sub-PHID
+                   # (will be incremented by $CHECK_EXEC, if any)
+                   echo "0" > \$NCBI_LOG_HIT_ID
+                fi
+                NCBI_CONFIG__LOG__FILE=\$logfile
+                export NCBI_CONFIG__LOG__FILE
+
                 # Run check
                 start_time="\`date +'$x_date_format'\`"
                         
@@ -651,7 +663,7 @@ EOF_launch
                                     -e 's/%$//'      \\
                                     -e 's/%/, /g'    \\
                                     -e 's/[ ] */ /g' \\
-                                    -e 's/.*\(Maximum execution .* is exceeded\).*$/\1/' \\
+                                    -e 's/^.*\(Maximum execution [0-9][0-9]* is exceeded\).*$/\1/' \\
                                     -e 's/^.*\(real [0-9][0-9]*[.][0-9][0-9]*\)/\1/' \\
                                     -e 's/\(sys [0-9][0-9]*[.][0-9][0-9]*\).*/\1/'\`
                 else
@@ -694,40 +706,40 @@ EOF_launch
                     echo "DIS --  \$x_cmd"
                     echo "DIS --  \$x_cmd" >> \$res_log
                     count_absent=\`expr \$count_absent + 1\`
-                    test -n "\$NCBI_AUTOMATED_BUILD" && echo "DIS" >> "\$x_test_rep"
+                    \$is_automated && echo "DIS" >> "\$x_test_rep"
 
                 elif grep NCBI_UNITTEST_SKIPPED \$x_test_out >/dev/null; then
                     echo "SKP --  \$x_cmd"
                     echo "SKP --  \$x_cmd" >> \$res_log
                     count_absent=\`expr \$count_absent + 1\`
-                    test -n "\$NCBI_AUTOMATED_BUILD" && echo "SKP" >> "\$x_test_rep"
+                    \$is_automated && echo "SKP" >> "\$x_test_rep"
 
                 elif grep NCBI_UNITTEST_TIMEOUTS_BUT_NO_ERRORS \$x_test_out >/dev/null; then
                     echo "TO  --  \$x_cmd"
                     echo "TO  --  \$x_cmd" >> \$res_log
                     count_timeout=\`expr \$count_timeout + 1\`
-                    test -n "\$NCBI_AUTOMATED_BUILD" && echo "TO" >> "\$x_test_rep"
+                    \$is_automated && echo "TO" >> "\$x_test_rep"
 
                 elif echo "\$exec_time" | egrep 'Maximum execution .* is exceeded' >/dev/null || egrep "Maximum execution .* is exceeded" \$x_test_out >/dev/null; then
                     echo "TO  --  \$x_cmd     (\$exec_time)"
                     echo "TO  --  \$x_cmd     (\$exec_time)" >> \$res_log
                     count_timeout=\`expr \$count_timeout + 1\`
-                    test -n "\$NCBI_AUTOMATED_BUILD" && echo "TO" >> "\$x_test_rep"
+                    \$is_automated && echo "TO" >> "\$x_test_rep"
 
                 elif test \$result -eq 0; then
                     echo "OK  --  \$x_cmd     (\$exec_time)"
                     echo "OK  --  \$x_cmd     (\$exec_time)" >> \$res_log
                     count_ok=\`expr \$count_ok + 1\`
-                    test -n "\$NCBI_AUTOMATED_BUILD" && echo "OK" >> "\$x_test_rep"
+                    \$is_automated && echo "OK" >> "\$x_test_rep"
 
                 else
                     echo "ERR [\$result] --  \$x_cmd     (\$exec_time)"
                     echo "ERR [\$result] --  \$x_cmd     (\$exec_time)" >> \$res_log
                     count_err=\`expr \$count_err + 1\`
-                    test -n "\$NCBI_AUTOMATED_BUILD" && echo "ERR" >> "\$x_test_rep"
+                    \$is_automated && echo "ERR" >> "\$x_test_rep"
                 fi
 
-                if test -n "\$NCBI_AUTOMATED_BUILD"; then
+                if \$is_automated; then
                     echo "\$start_time" >> "\$x_test_rep"
                     echo "\$result"     >> "\$x_test_rep"
                     echo "\$exec_time"  >> "\$x_test_rep"
@@ -737,12 +749,12 @@ EOF_launch
                 fi
 
             else  # Run test if it exist
-                if \$no_report_err; then
+                if \$is_run; then
                     echo "ABS --  \$x_cmd"
                     echo "ABS --  \$x_cmd" >> \$res_log
                     count_absent=\`expr \$count_absent + 1\`
 
-                    if test -n "\$NCBI_AUTOMATED_BUILD"; then
+                    if \$is_automated; then
                         echo "ABS"         >> "\$x_test_rep"
                         echo "\`date +'$x_date_format'\`" >> "\$x_test_rep"
                         echo "\$x_authors" >> "\$x_test_rep"
@@ -751,19 +763,38 @@ EOF_launch
             fi
 
         else  # Check existence of the test's application directory
-            if \$no_report_err; then
+            if \$is_run; then
                 # Test application is absent
                 echo "ABS -- \$x_work_dir - \$x_test"
                 echo "ABS -- \$x_work_dir - \$x_test" >> \$res_log
                 count_absent=\`expr \$count_absent + 1\`
 
-                if test -n "\$NCBI_AUTOMATED_BUILD"; then
+                if \$is_automated; then
                     echo "ABS"         >> "\$x_test_rep"
                     echo "\`date +'$x_date_format'\`" >> "\$x_test_rep"
                     echo "\$x_authors" >> "\$x_test_rep"
                 fi
             fi
         fi
+
+        # Load test results to Database and Applog immediately after a test.
+        # Always load test results for automated builds on a 'run' command.
+        
+        if \$is_run && \$is_db_load; then
+           NCBI_LOG_HIT_ID=\$saved_phid
+           export NCBI_LOG_HIT_ID
+           case \`uname -s\` in
+              CYGWIN* )
+                test_stat_load "\$(cygpath -w "\$x_test_rep")" "\$(cygpath -w "\$x_test_out")" "\$(cygpath -w "\$x_boost_rep")" "\$(cygpath -w "\$top_srcdir/build_info")" >> "$x_build_dir/test_stat_load.log" 2>&1 ;;
+              IRIX* )
+                test_stat_load.sh "\$x_test_rep" "\$x_test_out" "\$x_boost_rep" "\$top_srcdir/build_info" >> "$x_build_dir/test_stat_load.log" 2>&1 ;;
+              * )
+                test_stat_load "\$x_test_rep" "\$x_test_out" "\$x_boost_rep" "\$top_srcdir/build_info" >> "$x_build_dir/test_stat_load.log" 2>&1 ;;
+            esac
+        fi
+        if test \$is_run  -a  -n "\$saved_phid"; then
+            rm -f \$saved_phid* > /dev/null 2>&1
+        fi
         
     done  # Run test under all specified check tools   
 }
@@ -898,7 +929,7 @@ done # for x_row in x_tests
 # Write ending code into the script 
 cat >> $x_out <<EOF
 
-if \$no_report_err  &&  \$no_db_load; then
+if \$is_run; then
    # Write result of the tests execution
    echo
    echo "Succeeded : \$count_ok"
