@@ -14,11 +14,11 @@ suppress_dir="$(cygpath -w $script_dir)\\inspxe-suppressions"
 
 # Check Intel Inspector XE 2015 presence
 
-if test -z "$INSPECTOR_XE_2015_DIR"; then
-   echo "Cannot find Intel Inspector XE 2015."
+if test -z "$INSPECTOR_2018_DIR"; then
+   echo "Cannot find Intel Inspector 2018."
    exit 1 
 fi
-inspxe=`echo $(cygpath -u $INSPECTOR_XE_2015_DIR) | sed 's| |\ |g'`/bin32/inspxe-cl
+inspxe=`echo $(cygpath -u $INSPECTOR_2018_DIR) | sed 's| |\ |g'`/bin32/inspxe-cl
 
 
 # Result directory name
