@@ -1,4 +1,4 @@
-// $Id: project_utilits.js 525933 2017-01-30 16:20:38Z gouriano $
+// $Id: project_utilits.js 548209 2017-10-11 17:35:14Z gouriano $
 ////////////////////////////////////////////////////////////////////////////////////
 // Shared part of new_project.wsf and import_project.wsf
 // global settings
@@ -9,10 +9,12 @@ var g_open_solution = true;
 
 var g_def_branch = "toolkit/trunk/internal/c++";
 var g_branch     = "toolkit/trunk/internal/c++";
+//var g_def_branch = "toolkit/trunk/c++";
+//var g_branch     = "toolkit/trunk/c++";
 
 // valid:   "120", "120x64", "140", "140x64"
-var g_def_msvcver = "120x64";
-var g_msvcver     = "120x64";
+var g_def_msvcver = "140x64";
+var g_msvcver     = "140x64";
 
 ////////////////////////////////////////////////////////////////////////////////////
 // Utility functions :
@@ -691,6 +693,11 @@ function GetSvnRepositoryRoot()
     return "https://svn.ncbi.nlm.nih.gov/repos/";
 }
 
+function GetSvnRepositoryToolkitRoot()
+{
+    return GetSvnRepositoryRoot() + "toolkit";
+}
+
 function GetRepositoryRoot()
 {
     return GetSvnRepositoryRoot() + GetBranch();
@@ -724,6 +731,44 @@ function SearchRepository(oShell, abs_path, rel_path)
         while( repo.length == 0 ) {
             while (repo.length == 0 && !externals.StdOut.AtEndOfStream) {
                 line = externals.StdOut.ReadLine();
+                var line_parts = line.split(/\s/);
+                var line_parts_size = line_parts.length;
+                if (line_parts_size > 1) {
+                    var test = "";
+                    var j;
+                    for (j=i; j<rel_path_size; ++j) {
+                        test += rel_path_array[j];
+                        if (line_parts[line_parts_size - 1] == test) {
+                            repo = "";
+                            var p;
+                            for (p = 0; p < line_parts_size - 1; ++p) {
+                                var t = line_parts[p];
+                                if (t.charAt(0) == '^') {
+                                    t = t.replace(/\^/, GetSvnRepositoryToolkitRoot());
+                                }
+                                if (t.indexOf("http") == 0) {
+                                    var curdir = oFso.GetAbsolutePathName("") + "\\";
+                                    t = oFso.GetAbsolutePathName(t);
+                                    t = t.substr(curdir.length);
+                                    t = ForwardSlashes(t);
+                                    if (t.indexOf("http:/") == 0) {
+                                        t = "http://" + t.substr(6);
+                                    }
+                                    else if (t.indexOf("https:/") == 0) {
+                                        t = "https://" + t.substr(7);
+                                    }
+                                    for (j=j+1; j<rel_path_size; ++j) {
+                                        t += "/" + rel_path_array[j];
+                                    }
+                                }
+                                repo += " " + t;
+                            }
+                            break;
+                        }
+                        test += "/";
+                    }
+                }
+/*
                 var test = "";
                 var j;
                 for (j=i; j<rel_path_size; ++j) {
@@ -737,6 +782,7 @@ function SearchRepository(oShell, abs_path, rel_path)
                     }
                     test += "/";
                 }
+*/
             }
             if (repo.length != 0) {
                 while (!externals.StdOut.AtEndOfStream) {
