@@ -25,7 +25,7 @@ setup()
 
 prep_binary_component_package() 
 {
-    BLAST_BINS="blastn blastp blastx tblastn tblastx psiblast rpsblast rpstblastn blast_formatter deltablast legacy_blast.pl update_blastdb.pl "
+    BLAST_BINS="blastn blastp blastx tblastn tblastx psiblast rpsblast rpstblastn blast_formatter deltablast legacy_blast.pl update_blastdb.pl get_species_taxids.sh"
     MASKING_BINS="windowmasker dustmasker segmasker"
     DB_BINS="blastdbcmd makeblastdb makeprofiledb makembindex blastdb_aliastool convert2blastmask blastdbcheck"
     ALL_BINS="$BLAST_BINS $MASKING_BINS $DB_BINS"
