@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 """Script to create a source/binary RPM.
 """
-# $Id: make_rpm.py 496041 2016-03-23 14:47:36Z camacho $
+# $Id: make_rpm.py 548925 2017-10-18 21:56:59Z fukanchi $
 
 from __future__ import print_function
 import sys, os, shutil
@@ -120,7 +120,7 @@ def run_rpm(blast_version):
     dest = os.path.join(RPMBUILD_HOME, "SPECS", rpm_spec)
     shutil.copyfile(src, dest)
     update_blast_version(dest, blast_version)
-    cmd = "/usr/bin/rpmbuild -ba " + dest
+    cmd = "/usr/bin/rpmbuild -v -ba " + dest
     safe_exec(cmd)
 
 def move_rpms_to_installdir(installdir):
