@@ -1,6 +1,6 @@
 #! /usr/bin/env python3
 """Script to create the Windows installer for BLAST command line applications"""
-# $Id: make_win.py 555910 2018-01-24 16:33:11Z camacho $
+# $Id: make_win.py 578261 2019-01-16 12:43:00Z camacho $
 #
 # Author: Christiam camacho
 
@@ -62,7 +62,8 @@ def main():
              "blast_formatter.exe",
              "deltablast.exe",
              "legacy_blast.pl",
-             "update_blastdb.pl"
+             "update_blastdb.pl",
+             "get_species_taxids.sh"
              ]
     
     cwd = os.getcwd()
