@@ -1,4 +1,4 @@
-/*  $Id: blast_nucl_options.cpp 504861 2016-06-20 15:45:40Z boratyng $
+/*  $Id: blast_nucl_options.cpp 574439 2018-11-13 15:48:07Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -244,7 +244,7 @@ CBlastNucleotideOptionsHandle::SetHitSavingOptionsDefaults()
     // effective lengths and Karlin-Altschul params in BLAST_Cutoffs_simple
     // and passed to the engine in the params structure
 
-    SetLowScorePerc(0.15);
+    SetLowScorePerc(0);
     SetQueryCovHspPerc(0);
 }
 void
@@ -264,7 +264,7 @@ CBlastNucleotideOptionsHandle::SetMBHitSavingOptionsDefaults()
     // effective lengths and Karlin-Altschul params in BLAST_Cutoffs_simple
     // and passed to the engine in the params structure
 
-    SetLowScorePerc(0.15);
+    SetLowScorePerc(0);
 
     SetQueryCovHspPerc(0);
 }
