@@ -1,4 +1,4 @@
-/*  $Id: blast_options_builder.cpp 512725 2016-09-02 14:13:18Z fongah2 $
+/*  $Id: blast_options_builder.cpp 577721 2019-01-08 17:58:10Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -89,6 +89,9 @@ CBlastOptionsBuilder::ComputeProgram(const string & program,
             p = "deltablast";
             found = true;
         }
+        else if (s == "indexed") {
+            found = true;
+        }
     } else if (p == "blastn") {
         if (s == "megablast") {
             p = "megablast";
@@ -98,7 +101,7 @@ CBlastOptionsBuilder::ComputeProgram(const string & program,
             p = "vecscreen";
             found = true;
         }
-        if ((s == "sra") || (s == "wgs")) {
+        if ((s == "sra") || (s == "wgs") || (s=="indexed")) {
         	found = true;
         }
     } else if (p == "tblastn") {
