@@ -1,4 +1,4 @@
-/*  $Id: blast_results.cpp 542542 2017-08-01 12:44:33Z dicuccio $
+/*  $Id: blast_results.cpp 573790 2018-11-01 15:46:35Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -267,7 +267,7 @@ CSearchResults::GetErrorStrings() const
     }
     ITERATE(TQueryMessages, iter, m_Errors) {
         if ((**iter).GetSeverity() >= eBlastSevError) {
-            retval += (*iter)->GetMessage() + " ";
+            retval += (*iter)->GetMessage(false) + " ";
         }
     }
     return retval;
@@ -286,7 +286,7 @@ CSearchResults::GetWarningStrings() const
     }
     ITERATE(TQueryMessages, iter, m_Errors) {
         if ((**iter).GetSeverity() == eBlastSevWarning) {
-            retval += (*iter)->GetMessage() + " ";
+            retval += (*iter)->GetMessage(false) + " ";
         }
     }
     return retval;
