@@ -1,4 +1,4 @@
-/* $Id: magicblast.cpp 574908 2018-11-20 18:35:55Z ivanov $
+/* $Id: magicblast.cpp 580452 2019-02-14 18:01:37Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -227,19 +227,11 @@ static void s_ComputeBtopAndIdentity(const HSPChain* chain,
             int query_gap = hsp->query.offset - prev->query.end;
             if (query_gap > 0) {
                 btop += (string)"_" + NStr::IntToString(query_gap) + "_";
+                len += hsp->query.offset - prev->query.end;
             }
             else if (query_gap < 0) {
                 btop += (string)"(" + NStr::IntToString(-query_gap) + ")";
             }
-
-            // gap in query on exon edge
-            if (hsp->query.offset > prev->query.end) {
-                btop += (string)"_" +
-                    NStr::IntToString(hsp->query.offset - prev->query.end) +
-                    "_";
-
-                len += hsp->query.offset - prev->query.end;
-            }
         }
 
         int query_pos = hsp->query.offset;
