@@ -66,9 +66,10 @@ CBlastPrelimSearch::CBlastPrelimSearch(CRef<IQueryFactory> query_factory,
     m_Options(options), m_DbAdapter(NULL), m_DbInfo(&dbinfo)
 {
     BlastSeqSrc* seqsrc = CSetupFactory::CreateBlastSeqSrc(dbinfo);
+    CRef<TBlastSeqSrc> wrapped_src(new TBlastSeqSrc(seqsrc, BlastSeqSrcFree));
     x_Init(query_factory, options, CRef<CPssmWithParameters>(), seqsrc);
 
-    m_InternalData->m_SeqSrc.Reset(new TBlastSeqSrc(seqsrc, BlastSeqSrcFree));
+    m_InternalData->m_SeqSrc = wrapped_src;
 }
 
 CBlastPrelimSearch::CBlastPrelimSearch(CRef<IQueryFactory> query_factory,
@@ -149,7 +150,7 @@ CBlastPrelimSearch::x_LaunchMultiThreadedSearch(SInternalData& internal_data)
 
     // -RMH- This appears to be a problem right now.  When used...this
     // can cause all the work to go to a single thread!  (-MN- This is fixed in SB-768)
-    BlastSeqSrcSetNumberOfThreads(m_InternalData->m_SeqSrc->GetPointer(), 
+    BlastSeqSrcSetNumberOfThreads(m_InternalData->m_SeqSrc->GetPointer(),
                                   GetNumberOfThreads());
 
     // Create the threads ...
@@ -199,12 +200,12 @@ CBlastPrelimSearch::Run()
     if (! BlastSeqSrcGetNumSeqs(m_InternalData->m_SeqSrc->GetPointer())) {
         string msg =
             "GI or TI list filtering resulted in an empty database.";
-        
+
         m_Messages.AddMessageAllQueries(eBlastSevWarning,
-                                        kBlastMessageNoContext, 
+                                        kBlastMessageNoContext,
                                         msg);
     }
-    
+
     BlastSeqSrcResetChunkIterator(m_InternalData->m_SeqSrc->GetPointer());
 
     CEffectiveSearchSpacesMemento eff_memento(m_Options);
@@ -227,9 +228,9 @@ CBlastPrelimSearch::Run()
 
         for (Uint4 i = 0; i < query_splitter->GetNumberOfChunks(); i++) {
             try {
-                CRef<IQueryFactory> chunk_qf = 
+                CRef<IQueryFactory> chunk_qf =
                     query_splitter->GetQueryFactoryForChunk(i);
-                _TRACE("Query chunk " << i << "/" << 
+                _TRACE("Query chunk " << i << "/" <<
                        query_splitter->GetNumberOfChunks());
                 CRef<SInternalData> chunk_data =
                     SplitQuery_CreateChunkData(chunk_qf, m_Options,
@@ -238,16 +239,16 @@ CBlastPrelimSearch::Run()
 
                 CRef<ILocalQueryData> query_data(
                         chunk_qf->MakeLocalQueryData( &*m_Options ) );
-                BLAST_SequenceBlk * chunk_queries = 
+                BLAST_SequenceBlk * chunk_queries =
                     query_data->GetSequenceBlk();
                 GetDbIndexSetUsingThreadsFn()( IsMultiThreaded() );
-                GetDbIndexRunSearchFn()( 
+                GetDbIndexRunSearchFn()(
                         chunk_queries, lut_options, word_options );
 
                 if (IsMultiThreaded()) {
                      x_LaunchMultiThreadedSearch(*chunk_data);
                 } else {
-                    retval = 
+                    retval =
                         CPrelimSearchRunner(*chunk_data, opts_memento.get())();
                     if (retval) {
                         NCBI_THROW(CBlastException, eCoreBlastError,
@@ -264,7 +265,7 @@ CBlastPrelimSearch::Run()
                 // free this as the query_splitter keeps a reference to the
                 // chunk factories, which in turn keep a reference to the local
                 // query data.
-                query_data->FlushSequenceData();        
+                query_data->FlushSequenceData();
             } catch (const CBlastException& e) {
                 // This error message is safe to ignore for a given chunk,
                 // because the chunks might end up producing a region of
@@ -313,7 +314,7 @@ int
 CBlastPrelimSearch::CheckInternalData()
 {
     int retval = 0;
-    retval = BlastScoreBlkCheck(m_InternalData->m_ScoreBlk->GetPointer()); 
+    retval = BlastScoreBlkCheck(m_InternalData->m_ScoreBlk->GetPointer());
     return retval;
 }
 
@@ -341,7 +342,7 @@ CBlastPrelimSearch::ComputeBlastHSPResults(BlastHSPStream* stream,
            hit_param,
            max_num_hsps,
            removed_hsps);
-    if( rm_hsps_info){ 
+    if( rm_hsps_info){
 	rm_hsps_info->reserve(m_InternalData->m_QueryInfo->num_queries );
         for( int query_index = 0 ; query_index < m_InternalData->m_QueryInfo->num_queries ; query_index ++ ){
 	  (*rm_hsps_info)[ query_index ] = removed_hsps[query_index] == FALSE ? false : true;
@@ -419,9 +420,10 @@ bool CBlastPrelimSearch::x_BuildStdSegList( vector<list<CRef<CStd_seg> > >  & l
 	IBlastSeqInfoSrc * s_seqInfoSrc = m_DbAdapter->MakeSeqInfoSrc();
 	EBlastProgramType program = hsp_stream->program;
 
-	BlastHSPResults * results = ComputeBlastHSPResults(hsp_stream );
+	CStructWrapper<BlastHSPResults> results
+            (ComputeBlastHSPResults(hsp_stream), Blast_HSPResultsFree);
 
-	if(NULL == results)
+	if(NULL == results.GetPointer())
 		return false;
 
 	int num_queries = results->num_queries;
