@@ -1,7 +1,3 @@
-#ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] =
-    "$Id: query_data.cpp 315260 2011-07-22 13:48:03Z camacho $";
-#endif /* SKIP_DOXYGEN_PROCESSING */
 
 /* ===========================================================================
  *
