@@ -1,4 +1,4 @@
-/*  $Id: blastn_args.cpp 500404 2016-05-04 14:59:01Z camacho $
+/*  $Id: blastn_args.cpp 565102 2018-06-06 17:25:02Z rackerst $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -69,7 +69,8 @@ CBlastnAppArgs::CBlastnAppArgs()
     arg.Reset(m_StdCmdLineArgs);
     m_Args.push_back(arg);
 
-    arg.Reset(new CGenericSearchArgs(kQueryIsProtein, false, true));
+    arg.Reset(new CGenericSearchArgs(kQueryIsProtein, false, true,
+                                     false, false, true));
     m_Args.push_back(arg);
 
     arg.Reset(new CNuclArgs);
