@@ -1,4 +1,4 @@
-/*  $Id: cmdline_flags.cpp 562577 2018-04-24 15:51:23Z fongah2 $
+/*  $Id: cmdline_flags.cpp 579216 2019-01-31 16:18:17Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -62,6 +62,10 @@ const string kArgNegativeSeqidList("negative_seqidlist");
 const string kArgDbSoftMask("db_soft_mask");
 const string kArgDbHardMask("db_hard_mask");
 
+const string kArgIpgList("ipglist");
+const string kArgNegativeIpgList("negative_ipglist");
+
+
 const string kTask("task");
 
 const string kArgQueryGeneticCode("query_gencode");
