@@ -1,4 +1,4 @@
-/*  $Id: igblastn_args.cpp 546749 2017-09-21 11:22:02Z ivanov $
+/*  $Id: igblastn_args.cpp 556289 2018-01-29 17:33:51Z jianye $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -94,7 +94,7 @@ CIgBlastnAppArgs::CIgBlastnAppArgs()
     arg.Reset(m_QueryOptsArgs);
     m_Args.push_back(arg);
 
-    m_FormattingArgs.Reset(new CFormattingArgs(true));
+    m_FormattingArgs.Reset(new CFormattingArgs(true, CFormattingArgs::eIsAirrRearrangement));
     arg.Reset(m_FormattingArgs);
     m_Args.push_back(arg);
 
