@@ -1,4 +1,4 @@
-/*  $Id: rpstblastn_args.cpp 448376 2014-10-06 09:52:42Z mcelhany $
+/*  $Id: rpstblastn_args.cpp 505234 2016-06-23 13:16:57Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -31,11 +31,6 @@
  * Implementation of the RPSTBLASTN command line arguments
  */
 
-#ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] 
-    = "$Id: rpstblastn_args.cpp 448376 2014-10-06 09:52:42Z mcelhany $";
-#endif
-
 #include <ncbi_pch.hpp>
 #include <algo/blast/blastinput/rpstblastn_args.hpp>
 #include <algo/blast/api/rpstblastn_options.hpp>
@@ -49,11 +44,13 @@ USING_SCOPE(objects);
 CRPSTBlastnAppArgs::CRPSTBlastnAppArgs()
 {
     CRef<IBlastCmdLineArgs> arg;
+    bool const kFilterByDefault = false;
     static const string kProgram("rpstblastn");
     arg.Reset(new CProgramDescriptionArgs(kProgram,
                                "Translated Reverse Position Specific BLAST"));
     const bool kQueryIsProtein = false;
     const bool kIsRpsBlast = true;
+    const bool kIsCBS2and3Supported = false;
     m_Args.push_back(arg);
     m_ClientId = kProgram + " " + CBlastVersion().Print();
 
@@ -78,7 +75,7 @@ CRPSTBlastnAppArgs::CRPSTBlastnAppArgs()
 
     // N.B.: query is not protein because the filtering is applied on the 
     // translated query
-    arg.Reset(new CFilteringArgs( !kQueryIsProtein ));
+    arg.Reset(new CFilteringArgs( !kQueryIsProtein, kFilterByDefault));
     m_Args.push_back(arg);
 
     arg.Reset(new CWindowSizeArg);
@@ -106,6 +103,10 @@ CRPSTBlastnAppArgs::CRPSTBlastnAppArgs()
     m_DebugArgs.Reset(new CDebugArgs);
     arg.Reset(m_DebugArgs);
     m_Args.push_back(arg);
+
+    arg.Reset(new CCompositionBasedStatsArgs(kIsCBS2and3Supported, kDfltArgCompBasedStatsRPS ));
+    m_Args.push_back(arg);
+
 }
 
 CRef<CBlastOptionsHandle> 
