@@ -1,4 +1,4 @@
-/* $Id: blast_engine.c 574995 2018-11-21 15:39:18Z ivanov $
+/* $Id: blast_engine.c 581728 2019-03-05 16:37:30Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -79,9 +79,9 @@
 #define CONV_NUCL2PROT_COORDINATES(length) (length) / CODON_LENGTH
 
 NCBI_XBLAST_EXPORT const int   kBlastMajorVersion = 2;
-NCBI_XBLAST_EXPORT const int   kBlastMinorVersion = 8;
-NCBI_XBLAST_EXPORT const int   kBlastPatchVersion = 1;
-NCBI_XBLAST_EXPORT const char* kBlastReleaseDate = "December-17-2018";
+NCBI_XBLAST_EXPORT const int   kBlastMinorVersion = 9;
+NCBI_XBLAST_EXPORT const int   kBlastPatchVersion = 0;
+NCBI_XBLAST_EXPORT const char* kBlastReleaseDate = "April-02-2019";
 
 /** Structure to be passed to s_BlastSearchEngineCore, containing pointers
     to various preallocated structures and arrays. */
@@ -1800,8 +1800,11 @@ Blast_RunPreliminarySearchWithInterrupt(EBlastProgramType program,
         BLAST_GapAlignSetUp(program, seq_src, score_options,
                             eff_len_options, ext_options, hit_options,
                             query_info, sbp, &score_params, &ext_params,
-                            &hit_params, &eff_len_params, &gap_align)) != 0)
+                            &hit_params, &eff_len_params, &gap_align)) != 0) {
+      /* Blast_DiagnosticsUpdate(diagnostics, local_diagnostics); */
+      Blast_DiagnosticsFree(local_diagnostics);
       return status;
+    }
 
     if ((status=
         BLAST_PreliminarySearchEngine(program, query, query_info,
@@ -1810,8 +1813,16 @@ Blast_RunPreliminarySearchWithInterrupt(EBlastProgramType program,
                                       ext_params, hit_params, eff_len_params,
                                       psi_options, db_options, hsp_stream,
                                       local_diagnostics, interrupt_search,
-                                      progress_info)) != 0)
+                                      progress_info)) != 0) {
+      gap_align = BLAST_GapAlignStructFree(gap_align);
+      score_params = BlastScoringParametersFree(score_params);
+      hit_params = BlastHitSavingParametersFree(hit_params);
+      ext_params = BlastExtensionParametersFree(ext_params);
+      eff_len_params = BlastEffectiveLengthsParametersFree(eff_len_params);
+      /* Blast_DiagnosticsUpdate(diagnostics, local_diagnostics); */
+      Blast_DiagnosticsFree(local_diagnostics);
       return status;
+    }
 
     /* Do not destruct score block here */
     gap_align->sbp = NULL;
