@@ -1,4 +1,4 @@
-/*  $Id: blast_psi_priv.h 169756 2009-09-02 18:25:42Z ivanov $
+/*  $Id: blast_psi_priv.h 347365 2011-12-16 13:17:19Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -249,6 +249,7 @@ typedef struct _PSIInternalPssmData {
     int**       pssm;           /**< PSSM (scores) */
     int**       scaled_pssm;    /**< scaled PSSM (scores) */
     double**    freq_ratios;    /**< frequency ratios */
+    double*     pseudocounts;   /**< pseudocount constant for each column */
 } _PSIInternalPssmData;
 
 /** Allocates a new _PSIInternalPssmData structure.
@@ -334,6 +335,9 @@ typedef struct _PSISequenceWeights {
     Uint4 posDistinctDistrib_size; /**< Kept to deallocate field above. */
     int* posNumParticipating; /**< number of sequences at each position.  Copied from
                                 posit.h:posSearchItems.posNumParticipating  */
+    double* independent_observations; /**< Number of independent sequences
+                                       per column */
+
 } _PSISequenceWeights;
 
 /** Allocates and initializes the _PSISequenceWeights structure.
@@ -382,6 +386,8 @@ _PSISequenceWeightsFree(_PSISequenceWeights* seq_weights);
 #define PSIERR_STARTINGGAP      (-10)
 /** Found flanking gap at end of alignment */
 #define PSIERR_ENDINGGAP        (-11)
+/** Errors in conserved domain profile */
+#define PSIERR_BADPROFILE       (-12)
 /** Unknown error */
 #define PSIERR_UNKNOWN          (-255)
 
@@ -447,6 +453,21 @@ _PSIComputeSequenceWeights(const _PSIMsa* msa,
                            Boolean nsg_compatibility_mode,
                           _PSISequenceWeights* seq_weights);
 
+/** Main function to calculate CD weights and combine weighted residue counts
+ * from matched CDs
+ * @param cd_msa multiple alignment of conserved domains data structure [in]
+ * @param sbp BLAST score block [in]
+ * @param options CDD-related options [in]
+ * @param seq_weights data structure with CD frequencies [out]
+ */
+NCBI_XBLAST_EXPORT 
+int
+_PSIComputeFrequenciesFromCDs(const PSICdMsa* cd_msa,
+                              BlastScoreBlk* sbp,
+                              const PSIBlastOptions* options,
+                              _PSISequenceWeights* seq_weights);
+    
+
 /** Main function to compute the PSSM's frequency ratios (stage 5).
  * Implements formula 2 in Nucleic Acids Research, 2001, Vol 29, No 14.
  * Corresponds to posit.c:posComputePseudoFreqs
@@ -473,6 +494,24 @@ _PSIComputeFreqRatios(const _PSIMsa* msa,
                       Boolean nsg_compatibility_mode,
                       _PSIInternalPssmData* internal_pssm);
 
+/** Main function to compute CD-based PSSM's frequency ratios
+ * @param cd_msa multiple alignment of CDs [in]
+ * @param seq_weights contains weighted residue frequencies and effective number
+ * of observations [in]
+ * @param sbp initialized score block data structure [in]
+ * @param pseudo_count pseudo count constant [in]
+ * @param internal_pssm PSSM [out]
+ * @return status
+ */
+NCBI_XBLAST_EXPORT 
+int
+_PSIComputeFreqRatiosFromCDs(const PSICdMsa* cd_msa,
+                             const _PSISequenceWeights* seq_weights,
+                             const BlastScoreBlk* sbp,
+                             Int4 pseudo_count,
+                             _PSIInternalPssmData* internal_pssm);
+
+
 /** Converts the PSSM's frequency ratios obtained in the previous stage to a 
  * PSSM of scores. (stage 6) 
  * @param internal_pssm PSSM being computed [in|out]
@@ -621,6 +660,22 @@ _PSISaveDiagnostics(const _PSIMsa* msa,
                     const _PSIInternalPssmData* internal_pssm,
                     PSIDiagnosticsResponse* diagnostics);
 
+/** Collects diagnostic information from the process of creating the CDD-based
+ * PSSM 
+ * @param cd_msa multiple alignment of CDs data structure [in]
+ * @param seq_weights sequence weights data structure [in]
+ * @param internal_pssm structure containing PSSM's frequency ratios [in]
+ * @param diagnostics output parameter [out]
+ * @return PSI_SUCCESS on success, PSIERR_OUTOFMEM if memory allocation fails
+ * or PSIERR_BADPARAM if any of its arguments is NULL
+ */
+int
+_PSISaveCDDiagnostics(const PSICdMsa* msa,
+                      const _PSISequenceWeights* seq_weights,
+                      const _PSIInternalPssmData* internal_pssm,
+                      PSIDiagnosticsResponse* diagnostics);
+
+
 /** Calculates the information content from the scoring matrix
  * @param score_mat alphabet by alphabet_sz matrix of scores (const) [in]
  * @param std_prob standard residue probabilities [in]
@@ -685,6 +740,16 @@ NCBI_XBLAST_EXPORT
 int
 _PSIValidateMSA_StructureGroup(const _PSIMsa* msa);
 
+/** Validation of multiple alignment of conserved domains structure
+ * @param cd_msa multiple alignment of CDs [in]
+ * @param alphabet_size alphabet size [in]
+ * @return One of the errors defined above if validation fails or bad
+ * parameter is passed in, else PSI_SUCCESS
+ */
+NCBI_XBLAST_EXPORT 
+int
+_PSIValidateCdMSA(const PSICdMsa* cd_msa, Uint4 alphabet_size);
+
 #ifdef __cplusplus
 }
 #endif
