@@ -1,4 +1,4 @@
-/* $Id: blast_traceback.c 525268 2017-01-23 15:04:37Z fongah2 $
+/* $Id: blast_traceback.c 573792 2018-11-01 15:47:14Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -227,7 +227,9 @@ s_HSPListPostTracebackUpdate(EBlastProgramType program_number,
 
       /* For nucleotide search, if match score is = 2, the odd scores
          are rounded down to the nearest even number. */
+#if 0
       Blast_HSPListAdjustOddBlastnScores(hsp_list, kGapped, sbp);
+#endif
 
       Blast_HSPListGetEvalues(program_number, query_info, subject_length,
                               hsp_list, kGapped, FALSE, sbp, 0,
@@ -849,8 +851,19 @@ s_FilterBlastResults(BlastHSPResults* results, const BlastHitSavingOptions* hit_
     	  	  }
     	  	  if(hit_options->query_cov_hsp_perc) {
     	  		  Blast_HSPListReapByQueryCoverage(hsp_list, hit_options, query_info, program_number);
+    	  		  if(hsp_list->hspcnt == 0){
+    	  			hit_list->hsplist_array[subject_index] =  Blast_HSPListFree(hsp_list);
+    	  		  }
     	  	  }
-      }
+    	  	  if((hit_options->hsp_filt_opt != NULL) && (hit_options->hsp_filt_opt->subject_besthit_opts != NULL)) {
+    	  		  Blast_HSPListSubjectBestHit(program_number,
+    	  		  				           hit_options->hsp_filt_opt->subject_besthit_opts,
+    	  		  				           query_info, hsp_list);
+    	  	  }
+       }
+       if(hit_options->query_cov_hsp_perc) {
+            Blast_HitListPurgeNullHSPLists(hit_list);
+       }
    }
 }
 
@@ -873,7 +886,7 @@ s_BlastPruneExtraHits(BlastHSPResults* results, Int4 hitlist_size)
       for (subject_index = hitlist_size;
            subject_index < hit_list->hsplist_count; ++subject_index) {
          hit_list->hsplist_array[subject_index] =
-            Blast_HSPListFree(hit_list->hsplist_array[subject_index]);
+         Blast_HSPListFree(hit_list->hsplist_array[subject_index]);
       }
       hit_list->hsplist_count = MIN(hit_list->hsplist_count, hitlist_size);
    }
@@ -1734,7 +1747,8 @@ BLAST_ComputeTraceback_MT(EBlastProgramType program_number,
         Blast_HSPResultsSortByEvalue(results);
     }
 
-    if(hit_params->options->query_cov_hsp_perc > 0 || hit_params->options->max_hsps_per_subject > 0) {
+    if(hit_params->options->query_cov_hsp_perc > 0 || hit_params->options->max_hsps_per_subject > 0 ||
+       (hit_params->options->hsp_filt_opt != NULL && hit_params->options->hsp_filt_opt->subject_besthit_opts != NULL)) {
     	s_FilterBlastResults(results, hit_params->options, query_info, program_number);
     }
 
