@@ -1,4 +1,4 @@
-/* $Id: greedy_align.c 567051 2018-07-12 13:09:32Z fongah2 $
+/* $Id: greedy_align.c 581785 2019-03-05 19:21:30Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -877,6 +877,11 @@ Int4 BLAST_AffineGreedyAlign (const Uint1* seq1, Int4 len1,
     index = s_FindFirstMismatch(seq1, seq2, len1, len2, 0, 0,
                                 fence_hit, reverse, rem);
 
+    if (fence_hit && *fence_hit) {
+        return -1;
+    }
+
+
     /* update the extents of the alignment, and bail out
        early if no further work is needed */
 
@@ -1067,6 +1072,11 @@ Int4 BLAST_AffineGreedyAlign (const Uint1* seq1, Int4 len1,
                                         seq1_index, seq2_index,
                                         fence_hit, reverse, rem);
 
+            if (fence_hit && *fence_hit) {
+                return -1;
+            }
+
+
             if (index > longest_match_run) {
                 seed->start_q = seq1_index;
                 seed->start_s = seq2_index;
