@@ -1,4 +1,4 @@
-/*  $Id: jumper.h 533522 2017-04-17 15:50:34Z boratyng $
+/*  $Id: jumper.h 577781 2019-01-08 18:27:42Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -305,6 +305,10 @@ Int2 FilterQueriesForMapping(Uint1* sequence, Int4 length, Int4 offset,
                              BlastSeqLoc** seq_loc);
 
 
+/** Get alignment cutoff score for a given query length. The function
+    assumes that score for match is 1 */
+Int4 GetCutoffScore(Int4 query_length);
+
 #ifdef __cplusplus
 }
 #endif
