@@ -50,6 +50,8 @@ Author: Jason Papadopoulos
 #include <algo/blast/format/blastxml2_format.hpp>
 #include <algo/blast/format/data4xml2format.hpp>       /* NCBI_FAKE_WARNING */
 #include <algo/blast/format/build_archive.hpp>
+#include <misc/jsonwrapp/jsonwrapp.hpp>
+#include <objtools/blast/seqdb_reader/seqdb.hpp>   // for CSeqDB
 #include <serial/objostrxml.hpp>
 
 #include <corelib/ncbistre.hpp>
@@ -187,6 +189,8 @@ CBlastFormat::CBlastFormat(const blast::CBlastOptions& options,
         const CNcbiRegistry& registry = app->GetConfig();
         m_LongSeqId = (registry.Get("BLAST", "LONG_SEQID") == "1");
     }
+    m_HitsSortOption = -1;
+    m_HspsSortOption = -1;    
 }
 
 CBlastFormat::CBlastFormat(const blast::CBlastOptions& opts, 
@@ -287,13 +291,14 @@ CBlastFormat::CBlastFormat(const blast::CBlastOptions& opts,
     m_IsIterative = opts.IsIterativeSearch();
     if (m_FormatType == CFormattingArgs::eSAM) {
     	x_InitSAMFormatter();
-    }
-
+    }    
     CNcbiApplication* app = CNcbiApplication::Instance();
     if (app) {
         const CNcbiRegistry& registry = app->GetConfig();
         m_LongSeqId = (registry.Get("BLAST", "LONG_SEQID") == "1");
     }
+    m_HitsSortOption = -1;
+    m_HspsSortOption = -1;
 }
 
 CBlastFormat::~CBlastFormat()
@@ -486,7 +491,11 @@ CBlastFormat::x_ConfigCShowBlastDefline(CShowBlastDefline& showdef,
     if (m_LongSeqId) {
         flags |= CShowBlastDefline::eLongSeqId;
     }
-
+    if(m_HitsSortOption >= 0) {
+        flags |= CShowBlastDefline::eShowPercentIdent;
+        flags |= CShowBlastDefline::eShowTotalScore;
+        flags |= CShowBlastDefline::eShowQueryCoverage;
+    }
     showdef.SetOption(flags);
     showdef.SetDbName(m_DbName);
     showdef.SetDbType(!m_DbIsAA);
@@ -517,7 +526,7 @@ CBlastFormat::x_SplitSeqAlign(CConstRef<CSeq_align_set> full_alignment,
             new_seqs.Set().push_back(*alignment);
         }
         count++;
-        if(count >= m_NumSummary)
+        if(count >= (unsigned int)m_NumSummary)
         	break;
     }
 }
@@ -739,11 +748,14 @@ CBlastFormat::x_PrintTabularReport(const blast::CSearchResults& results,
     if (m_FormatType == CFormattingArgs::eTabular ||
         m_FormatType == CFormattingArgs::eTabularWithComments ||
         m_FormatType == CFormattingArgs::eCommaSeparatedValues) {
-        const CBlastTabularInfo::EFieldDelimiter kDelim =
+        const CBlastTabularInfo::EFieldDelimiter kDelim = 
             (m_FormatType == CFormattingArgs::eCommaSeparatedValues
              ? CBlastTabularInfo::eComma : CBlastTabularInfo::eTab);
-
+        
         CBlastTabularInfo tabinfo(m_Outfile, m_CustomOutputFormatSpec, kDelim);
+        if(!m_CustomDelim.empty()) {
+            tabinfo.SetCustomDelim(m_CustomDelim);
+        }
         tabinfo.SetParseLocalIds(m_BelieveQuery);
         if((m_IsBl2Seq && (!m_BelieveQuery))|| m_IsRemoteSearch) {
         	tabinfo.SetParseSubjectDefline(true);
@@ -1080,6 +1092,280 @@ CBlastFormat::WriteArchive(objects::CPssmWithParameters & pssm,
 }
 
 
+void CBlastFormat::x_CreateDeflinesJson(CConstRef<CSeq_align_set> aln_set)
+{
+    
+    int delineFormatOption = 0;
+    CShowBlastDefline deflines(*aln_set, *m_Scope,kFormatLineLength,m_NumSummary);
+         
+    deflines.SetQueryNumber(1);//m_Query_number
+    deflines.SetDbType (!m_DbIsAA);
+    deflines.SetDbName(m_DbName);    
+    delineFormatOption |= CShowBlastDefline::eHtml;
+    delineFormatOption |= CShowBlastDefline::eShowPercentIdent;        
+    deflines.SetOption(delineFormatOption); //m_defline_option
+
+    //Next three lines are for proper initialization in formatting of defline
+    CShowBlastDefline::SDeflineTemplates *deflineTemplates = new CShowBlastDefline::SDeflineTemplates;      
+    deflineTemplates->advancedView = true;
+    deflines.SetDeflineTemplates (deflineTemplates);    
+    
+
+    vector <CShowBlastDefline::SDeflineFormattingInfo *> sdlFortInfoVec = deflines.GetFormattingInfo();
+    CJson_Document doc;
+    CJson_Object top_obj = doc.SetObject();
+    CJson_Array defline_array = top_obj.insert_array("deflines");
+
+    for(size_t i = 0; i < sdlFortInfoVec.size(); i++) {
+        CJson_Object obj = defline_array.push_back_object();
+        
+        obj.insert("dfln_url",sdlFortInfoVec[i]->dfln_url);
+        obj.insert("dfln_rid",sdlFortInfoVec[i]->dfln_rid);
+        obj.insert("dfln_gi",sdlFortInfoVec[i]->dfln_gi);    
+        obj.insert("dfln_seqid",sdlFortInfoVec[i]->dfln_seqid);
+        obj.insert("full_dfln_defline",sdlFortInfoVec[i]->full_dfln_defline);
+        obj.insert("dfln_defline",sdlFortInfoVec[i]->dfln_defline);
+        obj.insert("dfln_id",sdlFortInfoVec[i]->dfln_id);
+        obj.insert("dflnFrm_id",sdlFortInfoVec[i]->dflnFrm_id);
+        obj.insert("dflnFASTA_id",sdlFortInfoVec[i]->dflnFASTA_id);
+        obj.insert("dflnAccs",sdlFortInfoVec[i]->dflnAccs);
+            
+        obj.insert("score_info",sdlFortInfoVec[i]->score_info);
+        obj.insert("dfln_hspnum",sdlFortInfoVec[i]->dfln_hspnum);
+        obj.insert("dfln_alnLen",sdlFortInfoVec[i]->dfln_alnLen);
+        obj.insert("dfln_blast_rank",sdlFortInfoVec[i]->dfln_blast_rank);
+	    obj.insert("total_bit_string",sdlFortInfoVec[i]->total_bit_string);
+        obj.insert("percent_coverage",sdlFortInfoVec[i]->percent_coverage);
+        obj.insert("evalue_string",sdlFortInfoVec[i]->evalue_string);
+        obj.insert("percent_identity",sdlFortInfoVec[i]->percent_identity);
+    }
+    doc.Write(m_Outfile);    
+}
+
+
+void CBlastFormat::x_DisplayDeflinesWithTemplates(CConstRef<CSeq_align_set> aln_set)
+{
+    x_InitDeflineTemplates();
+    _ASSERT(m_DeflineTemplates);
+
+    int delineFormatOption = 0;
+    CShowBlastDefline deflines(*aln_set, *m_Scope,kFormatLineLength,m_NumSummary);
+         
+    deflines.SetQueryNumber(1);//m_Query_number
+    deflines.SetDbType (!m_DbIsAA);
+    deflines.SetDbName(m_DbName);    
+    delineFormatOption |= CShowBlastDefline::eHtml;
+    delineFormatOption |= CShowBlastDefline::eShowPercentIdent;        
+    deflines.SetOption(delineFormatOption); //m_defline_option
+    deflines.SetDeflineTemplates (m_DeflineTemplates);    
+      
+    deflines.Init();    
+    deflines.Display(m_Outfile);        
+}
+
+
+void CBlastFormat::x_DisplayAlignsWithTemplates(CConstRef<CSeq_align_set> aln_set,const blast::CSearchResults& results)
+{
+    x_InitAlignTemplates();
+    _ASSERT(m_AlignTemplates);
+
+    TMaskedQueryRegions masklocs;
+    results.GetMaskedQueryRegions(masklocs);
+
+    CSeq_align_set copy_aln_set;        
+    CBlastFormatUtil::PruneSeqalign(*aln_set, copy_aln_set, m_NumAlignments);
+
+    CRef<CSeq_align_set> seqAlnSet(const_cast<CSeq_align_set*>(&copy_aln_set));      
+    if(!m_AlignSeqList.empty()) {                    
+        CAlignFormatUtil::ExtractSeqAlignForSeqList(seqAlnSet, m_AlignSeqList);
+    }                          
+
+    CDisplaySeqalign display(*seqAlnSet, *m_Scope, &masklocs, NULL, m_MatrixName);
+    x_SetAlignParameters(display);
+    display.SetAlignTemplates(m_AlignTemplates);    
+         
+    display.DisplaySeqalign(m_Outfile);    
+}
+
+void CBlastFormat::x_InitDeflineTemplates(void)
+{
+    CNcbiApplication* app = CNcbiApplication::Instance();
+    if(!app) return;
+    const CNcbiRegistry& reg = app->GetConfig();
+        
+      
+    m_DeflineTemplates = new CShowBlastDefline::SDeflineTemplates;      
+    string defLineTmpl;
+    
+    m_DeflineTemplates->defLineTmpl = reg.Get("Templates", "DFL_TABLE_ROW");    
+    m_DeflineTemplates->scoreInfoTmpl = reg.Get("Templates", "DFL_TABLE_SCORE_INFO");    
+    m_DeflineTemplates->seqInfoTmpl = reg.Get("Templates", "DFL_TABLE_SEQ_INFO");      
+    m_DeflineTemplates->advancedView = true;
+}
+
+void CBlastFormat::x_InitAlignTemplates(void)
+{
+    CNcbiApplication* app = CNcbiApplication::Instance();
+    if(!app) return;
+    const CNcbiRegistry& reg = app->GetConfig();
+
+    m_AlignTemplates = new CDisplaySeqalign::SAlignTemplates;
+    
+    m_AlignTemplates->alignHeaderTmpl = reg.Get("Templates", "BLAST_ALIGN_HEADER");           
+    string blastAlignParamsTemplData = reg.Get("Templates", "BLAST_ALIGN_PARAMS");     
+    string blastAlignParamsTag = (m_Program == "blastn") ? "ALIGN_PARAMS_NUC" : "ALIGN_PARAMS_PROT";     
+    string blastAlignProtParamsTable = reg.Get("Templates", blastAlignParamsTag);     
+    m_AlignTemplates->alignInfoTmpl = CAlignFormatUtil::MapTemplate(blastAlignParamsTemplData,"align_params",blastAlignProtParamsTable);    
+    m_AlignTemplates->sortInfoTmpl = reg.Get("Templates", "SORT_ALIGNS_SEQ");    
+    m_AlignTemplates->alignFeatureTmpl = reg.Get("Templates", "ALN_FEATURES");     
+    m_AlignTemplates->alignFeatureLinkTmpl = reg.Get("Templates", "ALN_FEATURES_LINK");
+
+	m_AlignTemplates->alnDefLineTmpl = reg.Get("Templates", "ALN_DEFLINE_ROW");     
+    m_AlignTemplates->alnTitlesLinkTmpl = reg.Get("Templates", "ALN_DEFLINE_TITLES_LNK");    
+    m_AlignTemplates->alnTitlesTmpl = reg.Get("Templates", "ALN_DEFLINE_TITLES");   
+    m_AlignTemplates->alnSeqInfoTmpl = reg.Get("Templates", "ALN_DEFLINE_SEQ_INFO");         
+    m_AlignTemplates->alignRowTmpl = reg.Get("Templates", "BLAST_ALIGN_ROWS");
+    m_AlignTemplates->alignRowTmplLast = reg.Get("Templates", "BLAST_ALIGN_ROWS_LST");
+}
+
+
+
+void CBlastFormat::x_SetAlignParameters(CDisplaySeqalign& cds)
+{
+ 
+    int AlignOption = 0;
+    
+    AlignOption += CDisplaySeqalign::eShowMiddleLine;
+            
+    if (m_Program == "tblastx") {
+        AlignOption += CDisplaySeqalign::eTranslateNucToNucAlignment;
+    }
+    AlignOption += CDisplaySeqalign::eShowBlastInfo;
+    AlignOption += CDisplaySeqalign::eShowBlastStyleId;    
+    AlignOption += CDisplaySeqalign::eHtml;
+    AlignOption += CDisplaySeqalign::eShowSortControls;//*******????    
+    AlignOption += CDisplaySeqalign::eDynamicFeature;
+    cds.SetAlignOption(AlignOption);
+
+    cds.SetDbName(m_DbName);
+    cds.SetDbType(!m_DbIsAA);
+    cds.SetLineLen(m_LineLength);
+
+    if (m_Program == "blastn" || m_Program == "megablast") {
+        cds.SetMiddleLineStyle (CDisplaySeqalign::eBar);
+        cds.SetAlignType(CDisplaySeqalign::eNuc);
+    } else {
+        cds.SetMiddleLineStyle (CDisplaySeqalign::eChar);
+        cds.SetAlignType(CDisplaySeqalign::eProt);
+    }
+    cds.SetQueryNumber(1); //m_Query_number    
+    cds.SetSeqLocChar (CDisplaySeqalign::eLowerCase);
+    cds.SetSeqLocColor ( CDisplaySeqalign::eGrey);    
+    cds.SetMasterGeneticCode(m_QueryGenCode);
+    cds.SetSlaveGeneticCode(m_DbGenCode);
+}
+         
+                    
+
+static string s_GetMolType(const CBioseq_Handle& bioseqHandle)
+{
+    int molType = bioseqHandle.GetBioseqMolType();    
+    string molTypeString;
+
+    switch(molType) {
+        case CSeq_inst::eMol_not_set:
+            molTypeString = "cdna";
+            break;
+        case CSeq_inst::eMol_dna:   
+            molTypeString = "dna";
+            break;
+        case CSeq_inst::eMol_rna:   
+            molTypeString = "rna";
+            break;
+        case CSeq_inst::eMol_aa:   
+            molTypeString = "amino acid";
+            break;
+        case CSeq_inst::eMol_na:
+            molTypeString = "nucleic acid";
+            break;
+        default:
+            molTypeString = "Unknown";
+    }        
+    return  molTypeString;
+}
+
+void
+CBlastFormat::PrintReport(const blast::CSearchResults& results,
+                          CBlastFormat::DisplayOption displayOption)                        
+{
+    if (displayOption == eMetadata) {//Metadata in json format
+        CBioseq_Handle bhandle = m_Scope->GetBioseqHandle(*results.GetSeqId(), CScope::eGetBioseq_All);
+        CConstRef<CBioseq> bioseq = bhandle.GetBioseqCore();
+
+        //string seqID = CAlignFormatUtil::GetSeqIdString(*bioseq, m_BelieveQuery);
+        string seqID;
+        CConstRef <CSeq_id> queryID = sequence::GetId(bhandle).GetSeqId();                          
+        CSeq_id::ELabelType labelType = (queryID->IsLocal()) ? CSeq_id::eDefault : CSeq_id::eContent;
+        queryID->GetLabel(&seqID,labelType);
+        
+        
+        string seqDescr = CBlastFormatUtil::GetSeqDescrString(*bioseq);
+        seqDescr = seqDescr.empty() ? "None" : seqDescr;
+
+        string molType = s_GetMolType(bhandle);  
+
+        int length = 0;
+        if(bioseq->IsSetInst() && bioseq->GetInst().CanGetLength()){            
+            length = bioseq->GetInst().GetLength();
+        }
+        
+        CJson_Document doc;
+        CJson_Object obj = doc.SetObject();
+        obj.insert("Query",seqID);
+        obj.insert("Query_descr",seqDescr);
+        obj.insert("IsQueryLocal",queryID->IsLocal());
+        obj.insert("Length",NStr::IntToString(length));
+        obj.insert("Moltype",molType);
+        obj.insert("Database",m_DbName);
+        string dbTitle;
+        try {
+            CRef<CSeqDB> seqdb;    
+            seqdb = new CSeqDB(m_DbName, m_DbIsAA ? CSeqDB::eProtein : CSeqDB::eNucleotide);        
+            dbTitle = seqdb->GetTitle();    
+        }
+        catch (...) {/*ignore exceptions for now*/}
+        obj.insert("Database_descr",dbTitle);
+        obj.insert("IsDBProtein",m_DbIsAA);
+        obj.insert("Program",m_Program);        
+        
+        
+        if (results.HasErrors()) {     
+            obj.insert("Error",results.GetErrorStrings());            
+        }
+        if (results.HasWarnings()) {
+            obj.insert("Warning",results.GetWarningStrings());            
+        }
+        doc.Write(m_Outfile);
+    }
+    else {
+        CConstRef<CSeq_align_set> aln_set = results.GetSeqAlign();
+        _ASSERT(results.HasAlignments());
+        if (m_IsUngappedSearch) {
+            aln_set.Reset(CDisplaySeqalign::PrepareBlastUngappedSeqalign(*aln_set));
+        }
+        
+        if (displayOption == eDescriptionsWithTemplates) {//Descriptions with html templates
+            x_DisplayDeflinesWithTemplates(aln_set);
+        }            
+        if (displayOption == eDescriptions) {//Descriptions with html templates
+            x_CreateDeflinesJson(aln_set);
+        }            
+        else if (displayOption == eAlignments) {// print the alignments with html templates
+            x_DisplayAlignsWithTemplates(aln_set,results);
+        }     
+    }
+}
+
 void
 CBlastFormat::PrintOneResultSet(const blast::CSearchResults& results,
                         CConstRef<blast::CBlastQueryVector> queries,
@@ -1191,6 +1477,15 @@ CBlastFormat::PrintOneResultSet(const blast::CSearchResults& results,
     if (m_IsUngappedSearch) {
         aln_set.Reset(CDisplaySeqalign::PrepareBlastUngappedSeqalign(*aln_set));
     }
+    
+    //invoke sorting only for m_HitsSortOption > CAlignFormatUtil::eEvalue or m_HspsSortOption > CAlignFormatUtil::eHspEvalue
+    if(m_HitsSortOption > 0 || m_HspsSortOption > 0) {     
+        aln_set = CBlastFormatUtil::SortSeqalignForSortableFormat(
+                                      *(const_cast<CSeq_align_set*>(aln_set.GetPointer())),
+                                      (m_Program == "tblastx") ? true : false,                                                                              
+                                      m_HitsSortOption, 
+                                      m_HspsSortOption);
+    }
 
     const bool kIsGlobal = s_IsGlobalSeqAlign(aln_set);
 
