@@ -1,4 +1,4 @@
-/* $Id: blastxml2_format.cpp 465639 2015-04-23 13:17:46Z ivanov $
+/* $Id: blastxml2_format.cpp 484963 2015-11-17 18:10:43Z fongah2 $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -53,6 +53,7 @@
 #include <serial/objostrjson.hpp>
 
 #include <algo/blast/api/version.hpp>
+#include <sstream>
 
 #include <algorithm>
 
@@ -700,6 +701,29 @@ static void s_FillBlastOutput(blastxml2::CBlastOutput2 & bxmlout, const IBlastXM
 	}
 
 }
+class CBlastOStreamXml : public CObjectOStreamXml
+{
+public:
+
+    CBlastOStreamXml (CNcbiOstream& stream, EOwnership deleteOut)
+        : CObjectOStreamXml(stream , deleteOut) {}
+    virtual ~CBlastOStreamXml (void) {};
+    // WriteFileHeader() is a dummy to keep xml prolog, doctype
+    // from being printed with each object
+    virtual void WriteFileHeader(TTypeInfo type) {;};
+};
+
+static void
+s_WriteXML2ObjectNoHeader(blastxml2::CBlastOutput2 & bxmlout, CNcbiOstream *out_stream)
+{
+    TTypeInfo typeInfo = bxmlout.GetThisTypeInfo();
+    auto_ptr<CBlastOStreamXml> xml_out(new CBlastOStreamXml (*out_stream, eNoOwnership));
+    xml_out->SetEncoding(eEncoding_Ascii);
+    xml_out->SetVerifyData( eSerialVerifyData_No );
+    xml_out->SetEnforcedStdXml();
+    xml_out->Write(&bxmlout, typeInfo );
+}
+
 
 static void
 s_WriteXML2Object(blastxml2::CBlastOutput2 & bxmlout, CNcbiOstream *out_stream)
@@ -723,12 +747,12 @@ s_WriteXML2Object(blastxml2::CBlastOutput2 & bxmlout, CNcbiOstream *out_stream)
 ///             retrieved [in]
 /// @param out_stream Output  stream for incremental output, ignore if NULL [out]
 void
-BlastXML2_FormatReport(const IBlastXML2ReportData* data, CNcbiOstream *out_stream )
+BlastXML2_FormatReport(const IBlastXML2ReportData* data, CNcbiOstream *out_stream)
 {
 	blastxml2::CBlastOutput2 bxmlout;
 	try {
 		s_FillBlastOutput(bxmlout, data);
-		s_WriteXML2Object(bxmlout, out_stream);
+		s_WriteXML2ObjectNoHeader(bxmlout, out_stream);
 	}
 	catch(CException &e){
 	    ERR_POST(Error << e.GetMsg() << e.what() );
@@ -744,25 +768,38 @@ void
 BlastXML2_FormatReport(const IBlastXML2ReportData* data, string file_name)
 {
 	blastxml2::CBlastOutput2 bxmlout;
-	try {
 		CNcbiOfstream out_stream;
 		out_stream.open(file_name.c_str(), IOS_BASE::out);
 		if(!out_stream.is_open())
-			 NCBI_THROW(CException, eInvalid, "Cannot open output file");
+			 NCBI_THROW(CArgException, eInvalidArg, "Cannot open output file");
 
 		s_FillBlastOutput(bxmlout, data);
 		s_WriteXML2Object(bxmlout, &out_stream);
-	}
-	catch(CException &e){
-	    ERR_POST(Error << e.GetMsg() << e.what() );
-	    return;
-	}
-	catch(...){
-	    ERR_POST(Error << "XML format failed" );
-	    return;
-	}
 }
 
+void
+BlastXML2_PrintHeader(CNcbiOstream *out_stream)
+{
+	CNcbiOstrstream ostr;
+	auto_ptr<CObjectOStreamXml> xml_out(new CObjectOStreamXml (ostr, eNoOwnership));
+	xml_out->SetEncoding(eEncoding_Ascii);
+	xml_out->SetVerifyData( eSerialVerifyData_No );
+	xml_out->SetReferenceSchema();
+    xml_out->SetUseSchemaLocation(true);
+    xml_out->SetEnforcedStdXml();
+    xml_out->SetDTDFilePrefix("http://www.ncbi.nlm.nih.gov/data_specs/schema_alt/");
+    xml_out->SetDefaultSchemaNamespace("http://www.ncbi.nlm.nih.gov");
+
+    blastxml2::CBlastXML2 xml2;
+	TTypeInfo typeInfo = xml2.GetThisTypeInfo();
+    xml_out->Write(&xml2, typeInfo);
+
+    string out_str = string(CNcbiOstrstreamToString(ostr));
+    string::size_type end_pos = out_str.find("</BlastXML2>");
+    out_str.erase(end_pos);
+
+    *out_stream << out_str;
+}
 
 void
 BlastXML2_FormatError(int exit_code, string err_msg,
@@ -776,6 +813,41 @@ BlastXML2_FormatError(int exit_code, string err_msg,
 	s_WriteXML2Object(bxmlout, out_stream);
 }
 
+class CBlastOStreamJson : public CObjectOStreamJson
+{
+public:
+
+    CBlastOStreamJson (CNcbiOstream& stream, EOwnership deleteOut)
+        : CObjectOStreamJson(stream , deleteOut) {}
+    virtual ~CBlastOStreamJson (void) {};
+    // WriteFileHeader() is a dummy to keep xml prolog, doctype
+    // from being printed with each object
+    virtual void WriteFileHeader(TTypeInfo type) {m_Output.IncIndentLevel();};
+    virtual void EndOfWrite(void) {
+    	m_Output.DecIndentLevel();
+    	m_Output.PutEol();
+    	CObjectOStream::EndOfWrite();
+    };
+};
+
+void
+BlastJSON_PrintHeader(CNcbiOstream *out_stream)
+{
+    *out_stream << "{\n\"BlastOutput2\": [\n";
+}
+
+static void
+s_WriteJSONObjectNoHeader(blastxml2::CBlastOutput2 & bxmlout, CNcbiOstream *out_stream)
+{
+    TTypeInfo typeInfo = bxmlout.GetThisTypeInfo();
+    auto_ptr<CObjectOStreamJson> json_out(new CBlastOStreamJson (*out_stream, eNoOwnership));
+    json_out->SetDefaultStringEncoding(eEncoding_Ascii);
+    //json_out.SetUseIndentation(true);
+    //json_out.SetUseEol(true);
+    json_out->Write(&bxmlout, typeInfo );
+}
+
+
 static void
 s_WriteJSONObject(blastxml2::CBlastOutput2 & bxmlout, CNcbiOstream *out_stream)
 {
@@ -792,23 +864,13 @@ void
 BlastJSON_FormatReport(const IBlastXML2ReportData* data, string file_name)
 {
 	blastxml2::CBlastOutput2 bxmlout;
-	try {
 		CNcbiOfstream out_stream;
 		out_stream.open(file_name.c_str(), IOS_BASE::out);
 		if(!out_stream.is_open())
-			 NCBI_THROW(CException, eInvalid, "Cannot open output file");
+			 NCBI_THROW(CArgException, eInvalidArg, "Cannot open output file");
 
 		s_FillBlastOutput(bxmlout, data);
 		s_WriteJSONObject(bxmlout, &out_stream);
-	}
-	catch(CException &e){
-	    ERR_POST(Error << e.GetMsg() << e.what() );
-	    return;
-	}
-	catch(...){
-	    ERR_POST(Error << "JSON format failed" );
-	    return;
-	}
 }
 
 void
@@ -817,7 +879,7 @@ BlastJSON_FormatReport(const IBlastXML2ReportData* data, CNcbiOstream *out_strea
 	blastxml2::CBlastOutput2 bxmlout;
 	try {
 		s_FillBlastOutput(bxmlout, data);
-		s_WriteJSONObject(bxmlout, out_stream);
+		s_WriteJSONObjectNoHeader(bxmlout, out_stream);
 	}
 	catch(CException &e){
 	    ERR_POST(Error << e.GetMsg() << e.what() );
