@@ -1,4 +1,4 @@
-/* $Id: blastxml_format.cpp 401131 2013-05-28 18:34:25Z grichenk $
+/* $Id: blastxml_format.cpp 504861 2016-06-20 15:45:40Z boratyng $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -30,11 +30,6 @@
 
 /// @file blastxml_format.cpp
 /// Formatting of BLAST results in XML form, using the BLAST XML specification.
-
-#ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: blastxml_format.cpp 401131 2013-05-28 18:34:25Z grichenk $";
-#endif /* SKIP_DOXYGEN_PROCESSING */
-
 #include <ncbi_pch.hpp>
 #include <objmgr/object_manager.hpp>
 #include <objects/seqloc/Seq_interval.hpp>
@@ -661,6 +656,7 @@ s_GetBlastPublication(EProgram program)
 
     switch (program) {
     case eMegablast:
+    case eMapper:
         publication = CReference::eMegaBlast; break;
     case ePHIBlastp: case ePHIBlastn:
         publication = CReference::ePhiBlast; break;
