@@ -1,4 +1,4 @@
-/*  $Id: tracebacksearch_unit_test.cpp 520431 2016-11-28 18:26:12Z ivanov $
+/*  $Id: tracebacksearch_unit_test.cpp 531968 2017-03-30 17:34:38Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -415,7 +415,12 @@ BOOST_AUTO_TEST_CASE(TracebackEntrez) {
     x_FindUsedGis(*rset[0].GetSeqAlign(), use_these);
     
     BOOST_REQUIRE_EQUAL((int)use_these.size(), 1);
-    BOOST_REQUIRE(*(use_these.begin()) == "gi:158292535");
+    if ( !CSeq_id::PreferAccessionOverGi() ) {
+        BOOST_REQUIRE(*(use_these.begin()) == "gi:158292535");
+    }
+    else {
+        BOOST_REQUIRE(*(use_these.begin()) == "seqid:XP_558472.3");
+    }
 }
 
 BOOST_AUTO_TEST_SUITE_END()
