@@ -1,4 +1,4 @@
-/*  $Id: seqalignfilter_unit_test.cpp 401131 2013-05-28 18:34:25Z grichenk $
+/*  $Id: seqalignfilter_unit_test.cpp 489876 2016-01-19 15:33:56Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -152,12 +152,12 @@ static bool s_AreListsEqual(list<TGi>& list1, list<TGi>& list2)
 /////////////////////////////////////////////////////////////////////////////
 // Vector-based static helper functions
 
-static bool s_IsGiInVector(TGi gi, vector<int>& vec_gis)
+static bool s_IsGiInVector(TGi gi, vector<TGi>& vec_gis)
 {
-    return binary_search(vec_gis.begin(), vec_gis.end(), GI_TO(int, gi));
+    return binary_search(vec_gis.begin(), vec_gis.end(), gi);
 }
 
-static bool s_GetFilteredGiList(CRef<CSeq_align> sa, vector<int>& vec_all_gis,
+static bool s_GetFilteredGiList(CRef<CSeq_align> sa, vector<TGi>& vec_all_gis,
                                list<TGi>& list_sa_filtered)
 {
     list<TGi> list_sa_full;
@@ -195,8 +195,8 @@ s_Check_GiListConsistency(CRef<CSeq_align> /*sa_orig*/,
 static void s_Check_GiEquivalenceInDB(TGi gi1, TGi gi2, CRef<CSeqDB> db)
 {
     int oid1 = -1, oid2 = -1;
-    db->GiToOid(GI_TO(int, gi1), oid1);
-    db->GiToOid(GI_TO(int, gi2), oid2);
+    db->GiToOid(gi1, oid1);
+    db->GiToOid(gi2, oid2);
 
     BOOST_REQUIRE(oid1 > 0);
     BOOST_REQUIRE(oid2 > 0);
@@ -207,7 +207,7 @@ static void s_Check_GiEquivalenceInDB(TGi gi1, TGi gi2, CRef<CSeqDB> db)
 // Pre-processing and testing individual seqaligns
 
 static void s_DoConsistencyCheck(CRef<CSeq_align> sa_orig, CRef<CSeq_align> sa_new,
-                                        vector<int>& vec_all_gis)
+                                        vector<TGi>& vec_all_gis)
 {
     list<TGi> list_orig_filtered;
     list<TGi> list_new, list_new_filtered;
@@ -297,8 +297,8 @@ BOOST_AUTO_TEST_CASE(s_TestDBBasedFiltering)
     CSeqAlignFilter filter;
     CRef<CSeqDB> db;
 
-    BOOST_REQUIRE_NO_THROW(db = filter.PrepareSeqDB(db_name, use_prot, fname_gis);)
-    BOOST_REQUIRE_NO_THROW(filter.FilterSeqalignsExt(fname_in, fname_out, db);)
+    BOOST_REQUIRE_NO_THROW(db = filter.PrepareSeqDB(db_name, use_prot, fname_gis););
+    BOOST_REQUIRE_NO_THROW(filter.FilterSeqalignsExt(fname_in, fname_out, db););
 
     // check the results
 
@@ -308,7 +308,7 @@ BOOST_AUTO_TEST_CASE(s_TestDBBasedFiltering)
     CSeq_align_set aln_filtered;
     s_LoadSeqAlignsFromFile(aln_filtered, fname_out);
 
-    vector<int> vec_gis;    // sorted vector of all available gi's
+    vector<TGi> vec_gis;    // sorted vector of all available gi's
     filter.ReadGiVector(fname_gis, vec_gis, true);
 
     ITERATE(CSeq_align_set::Tdata, iter, aln_all.Get())
