@@ -1,4 +1,4 @@
-/*  $Id: seq_masker.cpp 435370 2014-05-15 15:50:57Z morgulis $
+/*  $Id: seq_masker.cpp 464810 2015-04-14 16:29:42Z vakatov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -31,9 +31,6 @@
  */
 
 #include <ncbi_pch.hpp>
-#include <algorithm>
-#include <memory>
-#include <sstream>
 
 #include <corelib/ncbi_limits.h>
 
@@ -49,9 +46,26 @@
 #include <algo/winmask/seq_masker_istat_factory.hpp>
 #include <algo/winmask/seq_masker_cache_boost.hpp>
 
+#include <algorithm>
+#include <memory>
+#include <sstream>
+
+
 BEGIN_NCBI_SCOPE
 USING_SCOPE(objects);
 
+#define WIN_MASK_ALGO_NAME "window-masker-algorithm"
+#define WIN_MASK_ALGO_VER_MAJOR 1
+#define WIN_MASK_ALGO_VER_MINOR 0
+#define WIN_MASK_ALGO_VER_PATCH 0
+
+//-------------------------------------------------------------------------
+CSeqMaskerVersion CSeqMasker::AlgoVersion(
+        WIN_MASK_ALGO_NAME,
+        WIN_MASK_ALGO_VER_MAJOR,
+        WIN_MASK_ALGO_VER_MINOR,
+        WIN_MASK_ALGO_VER_PATCH
+);
 
 //-------------------------------------------------------------------------
 CSeqMasker::CSeqMasker( const string & lstat_name,
@@ -150,18 +164,6 @@ CSeqMasker::DoMask(
 {
     ustat->total_ = 0;
     auto_ptr<TMaskList> mask(new TMaskList);
-
-    /* This message is confusing. Particularly because DUST later
-       can still mask part of the sequence.
-    */
-    /*
-    if( window_size > data.size() )
-    {
-        ERR_POST( Warning 
-                  << "length of data is shorter than the window size" );
-    }
-    */
-
     Uint4 cutoff_score = ustat->get_threshold();
     Uint4 textend = ustat->get_textend();
     Uint1 nbits = discontig ? CSeqMaskerUtil::BitCount( pattern ) : 0;
