@@ -1,4 +1,4 @@
-/*  $Id: blast_app_util.hpp 527685 2017-02-15 14:00:17Z camacho $
+/*  $Id: blast_app_util.hpp 571362 2018-09-25 16:52:58Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,6 +35,7 @@
 #define APP__BLAST_APP_UTIL__HPP
 
 #include <objmgr/object_manager.hpp>
+#include <objtools/readers/reader_exception.hpp>        
 #include <objtools/blast/seqdb_reader/seqdb.hpp>
 #include <algo/blast/blastinput/blast_args.hpp>
 
@@ -173,6 +174,10 @@ string RegisterOMDataLoader(CRef<CSeqDB> db_handle);
         LOG_POST(Error << "Command line argument error: " << e.GetMsg());   \
         exit_code = BLAST_INPUT_ERROR;                                      \
     }                                                                       \
+    catch (const CObjReaderParseException& e) {                             \
+        LOG_POST(Error << "BLAST query error: " << e.GetMsg());             \
+        exit_code = BLAST_INPUT_ERROR;                                      \
+    }                                                                       \
     catch (const CSeqDBException& e) {                                      \
         LOG_POST(Error << "BLAST Database error: " << e.GetMsg());          \
         exit_code = BLAST_DATABASE_ERROR;                                   \
@@ -325,6 +330,9 @@ private :
 /// @param msg list of errors and warning to be added to blast4 archive for printing
 void PrintErrorArchive(const CArgs & a, const list<CRef<CBlast4_error> > & msg);
 
+/// Clean up formatter scope and release
+void QueryBatchCleanup();
+
 END_NCBI_SCOPE
 
 #endif /* APP__BLAST_APP_UTIL__HPP */
