@@ -1,4 +1,4 @@
-/*  $Id: blast_formatter.cpp 509107 2016-08-03 14:09:45Z fongah2 $
+/*  $Id: blast_formatter.cpp 577764 2019-01-08 18:10:59Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -318,7 +318,10 @@ int CBlastFormatterApp::PrintFormattedOutput(void)
                            opts.GetMBIndexLoaded(),
                            NULL, NULL,
                            GetCmdlineArgs(GetArguments()));
-    formatter.SetLineLength(fmt_args.GetLineLength());
+    formatter.SetLineLength(fmt_args.GetLineLength());    
+    formatter.SetHitsSortOption(fmt_args.GetHitsSortOption());
+    formatter.SetHspsSortOption(fmt_args.GetHspsSortOption());
+    formatter.SetCustomDelimiter(fmt_args.GetCustomDelimiter());
     if(UseXInclude(fmt_args, args[kArgOutput].AsString())) {
        	formatter.SetBaseFile(args[kArgOutput].AsString());
     }
