@@ -1,4 +1,4 @@
-/*  $Id: blastn_app.cpp 571362 2018-09-25 16:52:58Z ivanov $
+/*  $Id: blastn_app.cpp 577771 2019-01-08 18:12:51Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -161,6 +161,9 @@ int CBlastnApp::Run(void)
                                
         formatter.SetQueryRange(query_opts->GetRange());
         formatter.SetLineLength(fmt_args->GetLineLength());
+        formatter.SetHitsSortOption(fmt_args->GetHitsSortOption());
+        formatter.SetHspsSortOption(fmt_args->GetHspsSortOption());
+        formatter.SetCustomDelimiter(fmt_args->GetCustomDelimiter());
         if(UseXInclude(*fmt_args, args[kArgOutput].AsString())) {
         	formatter.SetBaseFile(args[kArgOutput].AsString());
         }
