@@ -1,4 +1,4 @@
-/*  $Id: deltablast_app.cpp 509107 2016-08-03 14:09:45Z fongah2 $
+/*  $Id: deltablast_app.cpp 571365 2018-09-25 16:53:46Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -279,7 +279,7 @@ int CDeltaBlastApp::Run(void)
         formatter.PrintProlog();
 
         /*** Process the input ***/
-        for (; !input.End(); formatter.ResetScopeHistory()) {
+        for (; !input.End(); formatter.ResetScopeHistory(), QueryBatchCleanup()) {
 
             CRef<CBlastQueryVector> query_batch(input.GetNextSeqBatch(*scope));
             CRef<blast::IQueryFactory> queries(
