@@ -1,4 +1,4 @@
-/*  $Id: psiblast_app.cpp 509107 2016-08-03 14:09:45Z fongah2 $
+/*  $Id: psiblast_app.cpp 571365 2018-09-25 16:53:46Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -499,7 +499,7 @@ int CPsiBlastApp::Run(void)
         {   // Value may be modified for 2nd (PSSM) iteration, so save to reset for next query.
             ECompoAdjustModes comp_stats_original = opts_hndl->GetOptions().GetCompositionBasedStats();
 
-            for (; !input->End(); formatter.ResetScopeHistory()) {
+            for (; !input->End(); formatter.ResetScopeHistory(), QueryBatchCleanup()) {
 
 		CRef<CBlastQueryVector> query_batch(input->GetNextSeqBatch(*scope));
 
