@@ -1,4 +1,4 @@
-/*  $Id: rpstblastn_app.cpp 536658 2017-05-22 15:48:20Z zaretska $
+/*  $Id: rpstblastn_app.cpp 571365 2018-09-25 16:53:46Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -160,7 +160,7 @@ int CRPSTBlastnApp::Run(void)
         formatter.PrintProlog();
 
         /*** Process the input ***/
-        for (; !input.End(); formatter.ResetScopeHistory()) {
+        for (; !input.End(); formatter.ResetScopeHistory(), QueryBatchCleanup()) {
 
             CRef<CBlastQueryVector> query_batch(input.GetNextSeqBatch(*scope));
             CRef<IQueryFactory> queries(new CObjMgr_QueryFactory(*query_batch));
