@@ -1,4 +1,4 @@
-/*  $Id: tblastn_app.cpp 571365 2018-09-25 16:53:46Z ivanov $
+/*  $Id: tblastn_app.cpp 577771 2019-01-08 18:12:51Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -175,7 +175,10 @@ int CTblastnApp::Run(void)
 
         formatter.SetQueryRange(query_opts->GetRange());
         formatter.SetLineLength(fmt_args->GetLineLength());
-        if(UseXInclude(*fmt_args, args[kArgOutput].AsString())) {
+        formatter.SetHitsSortOption(fmt_args->GetHitsSortOption());
+        formatter.SetHspsSortOption(fmt_args->GetHspsSortOption());
+        formatter.SetCustomDelimiter(fmt_args->GetCustomDelimiter());
+	if(UseXInclude(*fmt_args, args[kArgOutput].AsString())) {
         	formatter.SetBaseFile(args[kArgOutput].AsString());
         }
         formatter.PrintProlog();
