@@ -1,5 +1,5 @@
 #!/usr/bin/perl
-# $Id: update_blastdb.pl 574684 2018-11-16 14:53:37Z ivanov $
+# $Id: update_blastdb.pl 581818 2019-03-06 12:53:38Z ivanov $
 # ===========================================================================
 #
 #                            PUBLIC DOMAIN NOTICE
@@ -27,8 +27,7 @@
 # Author:  Christiam Camacho
 #
 # File Description:
-#   Script to download the pre-formatted BLAST databases from the NCBI ftp
-#   server.
+#   Script to download the pre-formatted BLAST databases.
 #
 # ===========================================================================
 
@@ -40,6 +39,8 @@ use Pod::Usage;
 use File::stat;
 use Digest::MD5;
 use Archive::Tar;
+use File::Temp;
+use JSON;
 
 use constant NCBI_FTP => "ftp.ncbi.nlm.nih.gov";
 use constant BLAST_DB_DIR => "/blast/db";
@@ -49,6 +50,12 @@ use constant DEBUG => 0;
 use constant MAX_DOWNLOAD_ATTEMPTS => 3;
 use constant EXIT_FAILURE => 2;
 
+use constant GCS_URL => "https://storage.googleapis.com";
+use constant GCP_URL => "http://metadata.google.internal/computeMetadata/v1/instance/id";
+use constant GCP_BUCKET => "blast-db";
+use constant GCP_MANIFEST => "blastdb-manifest.json";
+use constant GCP_MANIFEST_VERSION => "1.0";
+
 # Process command line options
 my $opt_verbose = 1;
 my $opt_quiet = 0;
@@ -57,27 +64,37 @@ my $opt_help = 0;
 my $opt_passive = 1;
 my $opt_blastdb_ver = 4;
 my $opt_timeout = 120;
-my $opt_showall = 0;
+my $opt_showall = undef;
 my $opt_show_version = 0;
 my $opt_decompress = 0;
+my $opt_source;
+my $opt_nt = &get_num_cores();
 my $result = GetOptions("verbose+"      =>  \$opt_verbose,
                         "quiet"         =>  \$opt_quiet,
                         "force"         =>  \$opt_force_download,
                         "passive:s"     =>  \$opt_passive,
                         "timeout=i"     =>  \$opt_timeout,
-                        "showall"       =>  \$opt_showall,
+                        "showall:s"     =>  \$opt_showall,
                         "version"       =>  \$opt_show_version,
                         "blastdb_version:i"=>  \$opt_blastdb_ver,
                         "decompress"    =>  \$opt_decompress,
+                        "source=s"      =>  \$opt_source,
+                        "num_threads=i" =>  \$opt_nt,
                         "help"          =>  \$opt_help);
 $opt_verbose = 0 if $opt_quiet;
 die "Failed to parse command line options\n" unless $result;
 pod2usage({-exitval => 0, -verbose => 2}) if $opt_help;
+if (length($opt_passive) and ($opt_passive !~ /1|no/i)) {
+    pod2usage({-exitval => 1, -verbose => 0,
+            -msg => "Invalid value for passive option: '$opt_passive'"});
+}
 pod2usage({-exitval => 0, -verbose => 2}) unless (scalar @ARGV or 
-                                                  $opt_showall or
+                                                  defined($opt_showall) or
                                                   $opt_show_version);
 pod2usage({-exitval => 1, -verbose => 0, -msg => "Invalid BLAST database version"}) 
     unless ($opt_blastdb_ver == 4 or $opt_blastdb_ver == 5);
+pod2usage({-exitval => 1, -verbose => 0, -msg => "Invalid number of threads"}) 
+    if ($opt_nt <= 0);
 if (length($opt_passive) and $opt_passive =~ /n|no/i) {
     $opt_passive = 0;
 } else {
@@ -86,26 +103,113 @@ if (length($opt_passive) and $opt_passive =~ /n|no/i) {
 my $exit_code = 0;
 $|++;
 
-# Connect and download files
-my $ftp = &connect_to_ftp() unless ($opt_show_version);
-if ($opt_show_version) {
-    my $revision = '$Revision: 574684 $';
-    $revision =~ s/\$Revision: | \$//g;
-    print "$0 version $revision\n";
-} elsif ($opt_showall) {
-    print "$_\n" foreach (sort(&get_available_databases()));
-} else {
-    my @files = sort(&get_files_to_download());
-    my @files2decompress;
-    $exit_code = &download(\@files, \@files2decompress);
-    if ($exit_code == 1) {
-        foreach (@files2decompress) {
-            $exit_code = &decompress($_);
-            last if ($exit_code != 1);
-        }
+my $location = "NCBI";
+unless ($^O =~ /mswin/i) {
+    $location = system("/usr/bin/curl -sfo /dev/null -H 'Metadata-Flavor: Google' " . GCP_URL) == 0 ? "GCP" : "NCBI";
+}
+# Override data source, only for testing
+if (defined($opt_source)) {
+    if ($opt_source =~ /^ncbi/i) {
+        $location = "NCBI";
+    } elsif ($opt_source =~ /^gc/i and $^O !~ /mswin/i) {
+        $location = "GCP";
     }
 }
-$ftp->quit() unless ($opt_show_version);
+
+if ($opt_show_version) {
+    my $revision = '$Revision: 581818 $';
+    $revision =~ s/\$Revision: | \$//g;
+    print "$0 version $revision\n";
+    exit($exit_code);
+}
+
+my $ftp;
+
+if ($location eq "GCP") {
+    #die "Only BLASTDB vesion 5 is supported at GCP\n" if ($opt_blastdb_ver == 4);
+    my $latest_dir = &get_gcs_latest_dir();
+    my ($json, $url) = &get_gcs_blastdb_metadata($latest_dir);
+    unless (length($json)) {
+        print STDERR "ERROR: Missing manifest file $url, please report to blast-help\@ncbi.nlm.nih.gov\n";
+        exit(2);
+    }
+    print "Connected to $location\n" if $opt_verbose;
+    my $metadata = from_json($json);
+    unless (exists($$metadata{version}) and ($$metadata{version} eq GCP_MANIFEST_VERSION)) {
+        print STDERR "ERROR: Invalid version in manifest file $url, please report to blast-help\@ncbi.nlm.nih.gov\n";
+        exit(2);
+    }
+    if (defined($opt_showall)) {
+        my $print_header = 1;
+        foreach my $db (sort keys %$metadata) {
+            next if ($db =~ /^version$/);
+            if ($opt_showall =~ /tsv/i) {
+                printf("%s\t%s\t%9.4f\t%s\n", $db, $$metadata{$db}{description}, 
+                    $$metadata{$db}{size}, $$metadata{$db}{last_updated});
+            } elsif ($opt_showall =~ /pretty/i) {
+                if ($print_header) {
+                    printf("%-60s %-120s %-11s %15s\n", "BLASTDB", 
+                        "DESCRIPTION", "SIZE (GB)", "LAST_UPDATED");
+                    $print_header = 0;
+                }
+                printf("%-60s %-120s %9.4f %15s\n", $db, $$metadata{$db}{description}, 
+                    $$metadata{$db}{size}, $$metadata{$db}{last_updated});
+            } else {
+                print "$db\n";
+            }
+        }
+    } else {
+        my @files2download;
+        for my $requested_db (@ARGV) {
+            if (exists $$metadata{$requested_db}) {
+                push @files2download, @{$$metadata{$requested_db}{files}};
+            } else {
+                print STDERR "Warning: $requested_db does not exist in $location ($latest_dir)\n";
+            }
+        }
+        if (@files2download) {
+            my $gsutil = &get_gsutil_path();
+            my $cmd;
+            my $fh = File::Temp->new();
+            if (defined($gsutil)) {
+                $cmd = "$gsutil " . ($opt_nt > 1 ? "-m" : "" ) . " -q cp ";
+                $cmd .= join(" ", @files2download) . " .";
+            } else { # fall back to  curl
+                my $url = GCS_URL;
+                s,gs://,$url/, foreach (@files2download);
+                if ($opt_nt > 1 and -f "/usr/bin/xargs") {
+                    print $fh join("\n", @files2download);
+                    $cmd = "/usr/bin/xargs -P $opt_nt -a $fh -n 1";
+                    $cmd .= " -t" if $opt_verbose > 3;
+                    $cmd .= " /usr/bin/curl -sOR";
+                } else {
+                    $cmd = "/usr/bin/curl -sR";
+                    $cmd .= " -O $_" foreach (@files2download);
+                }
+            }
+            print "$cmd\n" if $opt_verbose > 3;
+            system($cmd);
+        }
+    }
+
+} else {
+    # Connect and download files
+    $ftp = &connect_to_ftp();
+    if (defined $opt_showall) {
+        print "$_\n" foreach (sort(&get_available_databases($ftp->ls())));
+    } else {
+        my @files = sort(&get_files_to_download());
+        my @files2decompress;
+        $exit_code = &download(\@files, \@files2decompress);
+        if ($exit_code == 1) {
+            foreach (@files2decompress) {
+                $exit_code = &decompress($_);
+                last if ($exit_code != 1);
+            }
+        }
+    }
+    $ftp->quit();
+}
 
 exit($exit_code);
 
@@ -124,7 +228,7 @@ sub connect_to_ftp
     $ftp_path .= "/v5" if ($opt_blastdb_ver == 5);
     $ftp->cwd($ftp_path);
     $ftp->binary();
-    print "Connected to NCBI\n" if $opt_verbose;
+    print "Connected to $location\n" if $opt_verbose;
     return $ftp;
 }
 
@@ -338,11 +442,52 @@ sub get_num_volumes
     return $retval + 1;
 }
 
+# Retrieves the name of the 'subdirectory' where the latest BLASTDBs residue in GCP
+sub get_gcs_latest_dir
+{
+    my $cmd = "/usr/bin/curl -s " . GCS_URL . "/" . GCP_BUCKET . "/latest-dir";
+    chomp(my $retval = `$cmd`);
+    return $retval;
+}
+
+# Fetches the JSON text containing the BLASTDB metadata in GCS
+sub get_gcs_blastdb_metadata
+{
+    my $latest_dir = shift;
+    my $url = GCS_URL . "/" . GCP_BUCKET . "/$latest_dir/" . GCP_MANIFEST;
+    chomp(my $retval = `/usr/bin/curl -sf $url`);
+    return ($retval, $url);
+}
+
+# Returns the path to the gsutil utility or undef  if it is not found
+sub get_gsutil_path
+{
+    foreach (qw(/google/google-cloud-sdk/bin /usr/local/bin /usr/bin /snap/bin)) {
+        my $path = "$_/gsutil";
+        return $path if (-f $path);
+    }
+    return undef;
+}
+
+# Returns the number of cores, or 1 if unknown
+sub get_num_cores
+{
+    my $retval = 1;
+    if ($^O =~ /linux/i) {
+        open my $fh, "/proc/cpuinfo" or return $retval;
+        $retval = scalar(map /^processor/, <$fh>);
+        close($fh);
+    } elsif ($^O =~ /darwin/i) {
+        chomp($retval = `/usr/sbin/sysctl -n hw.ncpu`);
+    }
+    return $retval;
+}
+
 __END__
 
 =head1 NAME
 
-B<update_blastdb.pl> - Download pre-formatted BLAST databases from NCBI
+B<update_blastdb.pl> - Download pre-formatted BLAST databases
 
 =head1 SYNOPSIS
 
@@ -364,6 +509,13 @@ Show all available pre-formatted BLAST databases (default: false). The output
 of this option lists the database names which should be used when
 requesting downloads or updates using this script.
 
+It accepts the optional arguments: 'tsv' and 'pretty' to produce tab-separated values
+and a human-readable format respectively. These parameters elicit the display of
+additional metadata if this is available to the program.
+This metadata is displayed in columnar format; the columns represent:
+
+name, description, size in gigabytes, date of last update (YYYY-MM-DD format).
+
 =item B<--blastdb_version>
 
 Specify which BLAST database version to download (default: 4).
@@ -396,6 +548,11 @@ Produce no output (default: false). Overrides the B<--verbose> option.
 
 Prints this script's version. Overrides all other options.
 
+=item B<--num_cores>
+
+Sets the number of cores to utilize to perform downloads in parallel when data comes from GCS.
+Defaults to all cores (Linux and macos only).
+
 =back
 
 =head1 DESCRIPTION
