@@ -1,4 +1,4 @@
-/*  $Id: makeblastdb.cpp 560817 2018-03-27 17:24:45Z rackerst $
+/*  $Id: makeblastdb.cpp 577767 2019-01-08 18:11:46Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -526,11 +526,12 @@ private:
 
 void CMakeBlastDBApp::x_AddSeqEntries(CNcbiIstream & data, TFormat fmt)
 {
+    bool found = false;
 	try {
         while(!data.eof())
         {
             CSeqEntrySource src(data, fmt, m_SkipUnver);
-            m_DB->AddSequences(src);
+            found = found || m_DB->AddSequences(src);
         }
     } catch (const CEofException& e) {
         if (e.GetErrCode() == CEofException::eEof) {
@@ -539,6 +540,9 @@ void CMakeBlastDBApp::x_AddSeqEntries(CNcbiIstream & data, TFormat fmt)
             throw e;
         }
     }
+    if (!found) {
+        ERR_POST(Warning << "No sequences written");
+    }
 }
 
 class CRawSeqDBSource : public IRawSequenceSource {
