@@ -1,4 +1,4 @@
-/* $Id: mac_prj_generator.cpp 570296 2018-09-06 16:51:11Z ivanov $
+/* $Id: mac_prj_generator.cpp 576669 2018-12-19 13:11:04Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -202,7 +202,7 @@ void CMacProjectGenerator::Generate(const string& solution)
                 AddString( *dataspec_dependencies, proj_dependency);
             }
             if (prj.m_ProjType != CProjKey::eLib && prj.m_ProjType != CProjKey::eDll &&
-                prj.m_ProjType != CProjKey::eApp && prj.m_ProjType != CProjKey::eDataSpec &&
+                prj.m_ProjType != CProjKey::eApp && //prj.m_ProjType != CProjKey::eDataSpec &&
                 prj.m_ProjType != CProjKey::eMsvc) {
                 continue;
             }
@@ -234,6 +234,12 @@ void CMacProjectGenerator::Generate(const string& solution)
         AddString( *file_groups,
             CreateProjectFileGroups(prj, prj_files, *dict_objects, *build_files));
 
+        // project custom script phase
+        string proj_prebuild_script(
+            CreateProjectCustomScriptPhase(prj, prj_files, *dict_objects, "PreBuild"));
+        if (!proj_prebuild_script.empty()) {
+            AddString( *build_phases, proj_prebuild_script);
+        }
         // project script phase
         string proj_script(
             CreateProjectScriptPhase(prj, prj_files, *dict_objects));
@@ -248,7 +254,7 @@ void CMacProjectGenerator::Generate(const string& solution)
         }
         // project custom script phase
         string proj_cust_script(
-            CreateProjectCustomScriptPhase(prj, prj_files, *dict_objects));
+            CreateProjectCustomScriptPhase(prj, prj_files, *dict_objects, "CustomScript"));
         if (!proj_cust_script.empty()) {
             AddString( *build_phases, proj_cust_script);
         }
@@ -301,7 +307,11 @@ void CMacProjectGenerator::Generate(const string& solution)
             } else if (prj.m_ProjType == CProjKey::eApp) {
                 AddString( *dict_product, "path", prj.m_ID);
             } else if (prj.m_ProjType == CProjKey::eLib) {
-                AddString( *dict_product, "path", string("lib") + prj.m_ID + string(".a"));
+                if (prj.m_IsMetallib) {
+                    AddString( *dict_product, "path", prj.m_ID + string(".metallib"));
+                } else {
+                    AddString( *dict_product, "path", string("lib") + prj.m_ID + string(".a"));
+                }
             }
             AddString( *dict_product, "sourceTree", "BUILT_PRODUCTS_DIR");
         }
@@ -622,6 +632,7 @@ string CMacProjectGenerator::CreateProjectScriptPhase(
         script += "if test $? -ne 0; then\n";
         script += "cp -p " + infile + " " + outfile + "\n";
         script += "fi\n";
+        AddString( *outputs, outfile);
     }
     // datatool
     if (!prj.m_DatatoolSources.empty()) {
@@ -657,6 +668,7 @@ string CMacProjectGenerator::CreateProjectScriptPhase(
                 script += "export PTB_PLATFORM=\"$ARCHS\"\n";
                 if (spec_ext == ".proto") {
                     script += "export GENERATOR_PATH=" + GetApp().GetSite().GetConfigureEntry("XCode_CustomCodeGenerator" + spec_ext) + "\n";
+                    script += "export PROJECT_REQUIRES=" + NStr::Join(prj.m_Requires,";") + "\n";
                 } else {
                     script += "export DATATOOL_PATH=" + m_OutputDir + "../static/bin/ReleaseDLL\n";
                 }
@@ -711,14 +723,14 @@ string CMacProjectGenerator::CreateProjectScriptPhase(
 
 string CMacProjectGenerator::CreateProjectCustomScriptPhase(
     const CProjItem& prj, const CProjectFileCollector& prj_files,
-    CDict& dict_objects)
+    CDict& dict_objects, const string& section)
 {
     SCustomScriptInfo info;
-    prj_files.GetProjectContext().GetMsvcProjectMakefile().GetCustomScriptInfo(info);
+    prj_files.GetProjectContext().GetMsvcProjectMakefile().GetCustomScriptInfo(info, section);
 
     if (!info.m_Script.empty()) {
 #if USE_VERBOSE_NAMES
-        string proj_script(   GetProjId(       prj) + "_cust_script");
+        string proj_script(   GetProjId(       prj) + section);
 #else
         string proj_script(   GetUUID());
 #endif
@@ -748,6 +760,7 @@ string CMacProjectGenerator::CreateProjectCustomScriptPhase(
         }
         AddString( *dict_script, "shellPath", info.m_Shell);
         AddString( *dict_script, "shellScript",
+            CDirEntry::IsAbsolutePath(info.m_Script) ? info.m_Script :
             GetRelativePath(CDirEntry::ConcatPath(script_loc,info.m_Script)));
         AddString( *dict_script, "showEnvVarsInLog", "0");
         return proj_script;
@@ -1235,7 +1248,8 @@ void CMacProjectGenerator::CreateProjectBuildSettings(
         AddString( *settings, "OTHER_LDFLAGS", ldlib);
     }
 
-    AddString( *settings, "PRODUCT_NAME", GetTargetName(prj));
+//    AddString( *settings, "PRODUCT_NAME", GetTargetName(prj));
+    AddString( *settings, "PRODUCT_NAME", prj.m_ID);
     if (prj.m_ProjType == CProjKey::eDataSpec || prj.m_ProjType == CProjKey::eMsvc) {
         return;
     }
@@ -1490,6 +1504,8 @@ string CMacProjectGenerator::AddFile(CDict& dict, const string& name, bool style
         filetype = "text.xml";
     } else if (ext == ".jsd") {
         filetype = "text.json";
+    } else if (ext == ".metal") {
+        filetype = "sourcecode.metal";
     } else {
         filetype = "text";
     }
@@ -1675,6 +1691,13 @@ string CMacProjectGenerator::GetTargetName( const CProjItem& prj)
     if (prj.m_ProjType == CProjKey::eLib) {
         return /*string("lib") +*/ prj.m_ID;
     }
+    if (prj.m_ProjType == CProjKey::eApp) {
+        CProjKey klib(CProjKey::eLib, prj.m_ID), kdll(CProjKey::eDll, prj.m_ID);
+        if (m_Projects_tree.m_Projects.find(klib) != m_Projects_tree.m_Projects.end() ||
+            m_Projects_tree.m_Projects.find(kdll) != m_Projects_tree.m_Projects.end()) {
+            return string(prj.m_ID) + ".exe";
+        }
+    }
     return prj.m_ID;
 }
 
@@ -1704,7 +1727,11 @@ string CMacProjectGenerator::GetMachOType(  const CProjItem& prj)
 string CMacProjectGenerator::GetProductType( const CProjItem& prj)
 {
     if (prj.m_ProjType == CProjKey::eLib) {
-        return "com.apple.product-type.library.static";
+        if (prj.m_IsMetallib) {
+            return "com.apple.product-type.metal-library";
+        } else {
+            return "com.apple.product-type.library.static";
+        }
     } else if (prj.m_ProjType == CProjKey::eDll) {
         return "com.apple.product-type.library.dynamic";
     } else if (prj.m_ProjType == CProjKey::eApp) {
@@ -1718,7 +1745,11 @@ string CMacProjectGenerator::GetProductType( const CProjItem& prj)
 string CMacProjectGenerator::GetExplicitType( const CProjItem& prj)
 {
     if (prj.m_ProjType == CProjKey::eLib) {
-        return "archive.ar";
+        if (prj.m_IsMetallib) {
+            return "archive.metal-library";
+        } else {
+            return "archive.ar";
+        }
     } else if (prj.m_ProjType == CProjKey::eDll) {
         return "compiled.mach-o.dylib";
     } else if (prj.m_ProjType == CProjKey::eApp) {
