@@ -1,7 +1,7 @@
 #ifndef PROJECT_TREE_BUILDER__MAC_PRJ_GENERATOR__HPP
 #define PROJECT_TREE_BUILDER__MAC_PRJ_GENERATOR__HPP
 
-/* $Id: mac_prj_generator.hpp 431139 2014-04-01 19:11:21Z ucko $
+/* $Id: mac_prj_generator.hpp 576669 2018-12-19 13:11:04Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -79,7 +79,7 @@ private:
         CDict& dict_objects);
     string CreateProjectCustomScriptPhase(
         const CProjItem& prj, const CProjectFileCollector& prj_files,
-        CDict& dict_objects);
+        CDict& dict_objects, const string& section);
     string CreateProjectCopyBinScript(
         const CProjItem& prj, const CProjectFileCollector& prj_files,
         CDict& dict_objects);
@@ -149,7 +149,7 @@ private:
     static string GetProjHeaders(  const CProjItem& prj);
     static string GetProjDependency(  const CProjItem& prj);
     static string GetProjContainer(   const CProjItem& prj);
-    static string GetTargetName(   const CProjItem& prj);
+    string GetTargetName(   const CProjItem& prj);
 
     static string GetMachOType(    const CProjItem& prj);
     static string GetProductType(  const CProjItem& prj);
