@@ -1,4 +1,4 @@
-/* $Id: msvc_configure.cpp 572337 2018-10-11 16:38:17Z ivanov $
+/* $Id: msvc_configure.cpp 576267 2018-12-12 17:34:36Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -311,7 +311,7 @@ void CMsvcConfigure::WriteBuildVer(CMsvcSite& site, const string& root_dir, cons
     if (!cfg.empty()) {
         NStr::ReplaceInPlace(cfg_root_inc, cfg, config.GetConfigFullName());
     }
-    string extra = site.GetConfigureEntry("BuildVerPath");
+    string extra = CDirEntry::ConvertToOSPath(site.GetConfigureEntry("BuildVerPath"));
     string filename = CDirEntry::ConcatPath(cfg_root_inc, extra);
 
     if (extra.empty()) {
