@@ -1,4 +1,4 @@
-/* $Id: msvc_makefile.cpp 485908 2015-11-30 14:28:08Z gouriano $
+/* $Id: msvc_makefile.cpp 576669 2018-12-19 13:11:04Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -443,7 +443,7 @@ void CMsvcProjectMakefile::GetAdditionalIncludeDirs(const SConfigInfo& config,
     string dirs_string = 
         GetOpt(m_MakeFile, "AddToProject", "IncludeDirs", config);
     
-    NStr::Split(dirs_string, LIST_SEPARATOR, *dirs, NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
+    NStr::Split(dirs_string, LIST_SEPARATOR, *dirs, NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate | NStr::fSplit_CanQuote);
 }
 
 void CMsvcProjectMakefile::GetHeadersInInclude(const SConfigInfo& config, 
@@ -528,9 +528,8 @@ CMsvcProjectMakefile::GetCustomBuildInfo(list<SCustomBuildInfo>* info) const
 }
 
 void
-CMsvcProjectMakefile::GetCustomScriptInfo(SCustomScriptInfo& info) const
+CMsvcProjectMakefile::GetCustomScriptInfo(SCustomScriptInfo& info, const string& sec) const
 {
-    string sec("CustomScript");
     info.m_Input  = m_MakeFile.GetString(sec, "Input");
     info.m_Output = m_MakeFile.GetString(sec, "Output");
     info.m_Shell  = m_MakeFile.GetString(sec, "Shell");
@@ -785,9 +784,9 @@ void CMsvcCombinedProjectMakefile::GetCustomBuildInfo
 }
 
 void CMsvcCombinedProjectMakefile::GetCustomScriptInfo
-                                           (SCustomScriptInfo& info) const
+                                           (SCustomScriptInfo& info, const string& section) const
 {
-    m_ProjectMakefile->GetCustomScriptInfo(info);
+    m_ProjectMakefile->GetCustomScriptInfo(info, section);
 }
 
 
