@@ -1,6 +1,6 @@
 #ifndef PROJECT_TREE_BULDER__MSVC_MAKEFILE__HPP
 #define PROJECT_TREE_BULDER__MSVC_MAKEFILE__HPP
-/* $Id: msvc_makefile.hpp 554978 2018-01-11 15:08:49Z gouriano $
+/* $Id: msvc_makefile.hpp 576669 2018-12-19 13:11:04Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -256,7 +256,7 @@ public:
 
     virtual void GetExtraFiles           (map<string, list<string> >*  files) const;
 
-    void GetCustomScriptInfo             (SCustomScriptInfo& info) const;
+    void GetCustomScriptInfo             (SCustomScriptInfo& info, const string& section) const;
 
 
     string m_ProjectBaseDir;
@@ -360,7 +360,7 @@ public:
     virtual void GetExtraFiles           (map<string, list<string> >*  files) const;
 
 
-    void GetCustomScriptInfo             (SCustomScriptInfo& info) const;
+    void GetCustomScriptInfo             (SCustomScriptInfo& info, const string& section) const;
 
 private:
     typedef const CMsvcProjectMakefile*    TProjectMakefile;
