@@ -1,4 +1,4 @@
-/* $Id: msvc_prj_utils.cpp 568727 2018-08-09 18:33:41Z gouriano $
+/* $Id: msvc_prj_utils.cpp 576272 2018-12-12 17:36:12Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -445,6 +445,10 @@ string SourceFileExt(const string& file_path)
     if ( CFile(file).Exists() ) {
         return ".c.in";
     }
+    file += ".metal";
+    if ( CFile(file).Exists() ) {
+        return ".metal";
+    }
     return "";
 }
 
