@@ -1,4 +1,4 @@
-/* $Id: prj_file_collector.cpp 568727 2018-08-09 18:33:41Z gouriano $
+/* $Id: prj_file_collector.cpp 576272 2018-12-12 17:36:12Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -250,7 +250,7 @@ bool CProjectFileCollector::GetIncludeDirs(list<string>& inc_dirs, const SConfig
     inc_dirs.clear();
     string alldirs = m_ProjContext.AdditionalIncludeDirectories(cfg);
     list<string> dirs;
-    NStr::Split(alldirs, LIST_SEPARATOR, dirs, NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
+    NStr::Split(alldirs, LIST_SEPARATOR, dirs, NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate | NStr::fSplit_CanQuote);
     ITERATE( list<string>, i, dirs) {
         string dir;
 #ifdef PSEUDO_XCODE
@@ -273,7 +273,7 @@ bool CProjectFileCollector::GetLibraryDirs(list<string>& lib_dirs, const SConfig
     lib_dirs.clear();
     string alldirs = m_ProjContext.AdditionalLibraryDirectories(cfg);
     list<string> dirs;
-    NStr::Split(alldirs, LIST_SEPARATOR, dirs, NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
+    NStr::Split(alldirs, LIST_SEPARATOR, dirs, NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate | NStr::fSplit_CanQuote);
     ITERATE( list<string>, i, dirs) {
         string dir;
 #ifdef PSEUDO_XCODE
@@ -317,7 +317,7 @@ string CProjectFileCollector::GetFileExtension(const string& file)
             return ".cpp";
         }
     }
-    string ext_in[]  = {".cpp", ".cpp.in", ".c", ".c.in", kEmptyStr};
+    string ext_in[]  = {".cpp", ".cpp.in", ".c", ".c.in", ".metal", kEmptyStr};
     for (int i=0; !ext_in[i].empty(); ++i) {
         if ( CFile(file + ext_in[i]).Exists() ) {
             return ext_in[i];
