@@ -1,4 +1,4 @@
-/* $Id: proj_tree.cpp 568727 2018-08-09 18:33:41Z gouriano $
+/* $Id: proj_tree.cpp 576272 2018-12-12 17:36:12Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -203,6 +203,12 @@ void CProjectItemsTree::CreateFrom(const string& root_src,
                                             applib_mfilepath,
                                             makemsvc, tree, info.m_MakeType);
                 }
+                else if (info.m_Type == SMakeProjectT::SMakeInInfo::eMetal) {
+                    SLibProjectT::DoCreate(source_base_dir, 
+                                            proj_name, 
+                                            applib_mfilepath, 
+                                            makelib, tree, info.m_MakeType);
+                }
             }
         }
     }
