@@ -1,7 +1,7 @@
 #ifndef PROJECT_TREE_BUILDER__PROJ_UTILS__HPP
 #define PROJECT_TREE_BUILDER__PROJ_UTILS__HPP
 
-/* $Id: proj_utils.hpp 426480 2014-02-10 19:58:23Z gouriano $
+/* $Id: proj_utils.hpp 554978 2018-01-11 15:08:49Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -59,7 +59,7 @@ struct SProjectTreeInfo
 
     /// Subtree to buil (default is m_Src).
     /// More enhanced version of "subtree to build"
-    auto_ptr<IProjectFilter> m_IProjectFilter;
+    unique_ptr<IProjectFilter> m_IProjectFilter;
 
     /// Branch of tree to be implicit exclude from build
     list<string> m_ImplicitExcludedAbsDirs;
