@@ -1,4 +1,4 @@
-/*  $Id: cgiapp.cpp 534862 2017-05-03 12:54:15Z ivanov $
+/*  $Id: cgiapp.cpp 568892 2018-08-13 18:06:54Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -554,7 +554,8 @@ int CCgiApplication::Run(void)
                 GetDiagContext().SetAppState(eDiagAppState_Request);
                 if (x_ProcessHelpRequest() ||
                     x_ProcessVersionRequest() ||
-                    CCgiContext::ProcessCORSRequest(m_Context->GetRequest(), m_Context->GetResponse())) {
+                    CCgiContext::ProcessCORSRequest(m_Context->GetRequest(), m_Context->GetResponse()) ||
+                    x_ProcessAdminRequest()) {
                     result = 0;
                 }
                 else {
@@ -747,6 +748,20 @@ void CCgiApplication::LogRequest(void) const
             GetDiagContext().Extra().Print("USER_AGENT", str);
         }
     }
+    // Print NCBI_LOG_FIELDS
+    CNcbiLogFields f("http");
+    map<string, string> env;
+    list<string> names;
+    const CNcbiEnvironment& rq_env = ctx.GetRequest().GetEnvironment();
+    rq_env.Enumerate(names);
+    ITERATE(list<string>, it, names) {
+        if (!NStr::StartsWith(*it, "HTTP_")) continue;
+        string name = it->substr(5);
+        NStr::ToLower(name);
+        NStr::ReplaceInPlace(name, "_", "-");
+        env[name] = rq_env.Get(*it);
+    }
+    f.LogFields(env);
 }
 
 
@@ -934,6 +949,7 @@ CCgiApplication::CCgiApplication(void)
    m_OutputBroken(false),
    m_IsResultReady(true),
    m_ShouldExit(false),
+   m_CaughtSigterm(false),
    m_RequestStartPrinted(false),
    m_ErrorStatus(false)
 {
@@ -1061,28 +1077,6 @@ const CArgs& CCgiApplication::GetArgs(void) const
 }
 
 
-class CExtraEntryCollector : public CEntryCollector_Base {
-public:
-    CExtraEntryCollector(void) {}
-    virtual ~CExtraEntryCollector(void) {}
-    
-    virtual void AddEntry(const string& name,
-                          const string& value,
-                          const string& filename,
-                          bool          is_index)
-    {
-        _ASSERT(!is_index  ||  value.empty());
-        m_Args.push_back(CDiagContext_Extra::TExtraArg(name,
-            filename.empty() ? value : filename + "/" + value));
-    }
-
-    CDiagContext_Extra::TExtraArgs& GetArgs(void) { return m_Args; }
-
-private:
-    CDiagContext_Extra::TExtraArgs m_Args;
-};
-
-
 void CCgiApplication::x_OnEvent(EEvent event, int status)
 {
     switch ( event ) {
@@ -1733,7 +1727,7 @@ NCBI_PARAM_DEF_EX(bool, CGI, EnableHelpRequest, true,
 typedef NCBI_PARAM_TYPE(CGI, EnableHelpRequest) TEnableHelpRequest;
 
 
-bool CCgiApplication::x_ProcessHelpRequest()
+bool CCgiApplication::x_ProcessHelpRequest(void)
 {
     if (!TEnableHelpRequest::GetDefault()) return false;
     CCgiRequest& request = m_Context->GetRequest();
@@ -1873,7 +1867,7 @@ NCBI_PARAM_DEF_EX(string, CGI, EnableVersionRequest, "t",
 typedef NCBI_PARAM_TYPE(CGI, EnableVersionRequest) TEnableVersionRequest;
 
 
-bool CCgiApplication::x_ProcessVersionRequest()
+bool CCgiApplication::x_ProcessVersionRequest(void)
 {
     CCgiRequest& request = m_Context->GetRequest();
     if (request.GetRequestMethod() != CCgiRequest::eMethod_GET) return false;
@@ -1974,6 +1968,49 @@ void CCgiApplication::ProcessVersionRequest(EVersionType ver_type)
 }
 
 
+bool CCgiApplication::x_ProcessAdminRequest(void)
+{
+    CCgiRequest& request = m_Context->GetRequest();
+    if (request.GetRequestMethod() != CCgiRequest::eMethod_GET) return false;
+
+    bool found = false;
+    string cmd_name = request.GetEntry("ncbi_admin_cmd", &found);
+    if ( !found ) {
+        // Check if PATH_INFO contains a command.
+        string path_info = request.GetProperty(eCgi_PathInfo);
+        NStr::TrimSuffixInPlace(path_info, "/");
+        NStr::TrimPrefixInPlace(path_info, "/");
+        if ( path_info.empty() ) return false;
+        cmd_name = path_info;
+    }
+    EAdminCommand cmd = eAdmin_Unknown;
+    if ( NStr::EqualNocase(cmd_name, "health") ) {
+        cmd = eAdmin_Health;
+    }
+    else if ( NStr::EqualNocase(cmd_name, "deep-health") ) {
+        cmd = eAdmin_HealthDeep;
+    }
+
+    // If the overriden method failed or refused to process a command,
+    // fallback to the default processing which returns true for all known commands.
+    return ProcessAdminRequest(cmd) || CCgiApplication::ProcessAdminRequest(cmd);
+}
+
+
+bool CCgiApplication::ProcessAdminRequest(EAdminCommand cmd)
+{
+    if (cmd == eAdmin_Unknown) return false;
+
+    // By default report status 200 and write headers for any command.
+    CCgiResponse& response = m_Context->GetResponse();
+    response.SetContentType("text/plain");
+    SetHTTPStatus(CCgiException::e200_Ok,
+        CCgiException::GetStdStatusMessage(CCgiException::e200_Ok));
+    response.WriteHeader();
+    return true;
+}
+
+
 ///////////////////////////////////////////////////////
 // CCgiStatistics
 //
