@@ -1,4 +1,4 @@
-/*  $Id: cgictx.cpp 544444 2017-08-23 13:08:37Z ivanov $
+/*  $Id: cgictx.cpp 554017 2017-12-27 15:07:37Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -341,33 +341,45 @@ const string& CCgiContext::GetSelfURL(void) const
         return m_SelfURL;
     }
 
-    // Compose self URL
-    string server(GetRequest().GetProperty(eCgi_ServerName));
-    if ( server.empty() ) {
-        return kEmptyStr;
+    bool secure
+        = AStrEquiv
+        (GetRequest().GetRandomProperty("HTTPS", false), "on",
+            PNocase())
+        || AStrEquiv
+        (GetRequest().GetRandomProperty("X_FORWARDED_PROTO"), "https",
+            PNocase());
+
+    string host_port;
+    // Check HTTP_X_FORWARDED_HOST for host:port
+    const string& x_fwd_host = GetRequest().GetRandomProperty("X_FORWARDED_HOST");
+    if (!x_fwd_host.empty()) {
+        host_port = x_fwd_host;
+    }
+    else {
+        string server(GetRequest().GetProperty(eCgi_ServerName));
+        if (server.empty()) {
+            return kEmptyStr;
+        }
+
+        host_port = server;
+        string port = GetRequest().GetProperty(eCgi_ServerPort);
+        // Skip port if it's default for the selected scheme
+        if ((secure  &&  port == "443") || (!secure  &&  port == "80")
+            || (server.size() >= port.size() + 2 && NStr::EndsWith(server, port)
+                && server[server.size() - port.size() - 1] == ':')) {
+            port.erase();
+        }
+        if (!port.empty()) {
+            host_port += ':';
+            host_port += port;
+        }
     }
 
-    bool secure
-        =   AStrEquiv
-        (GetRequest().GetRandomProperty("HTTPS", false), "on",
-         PNocase())
-        ||  AStrEquiv
-        (GetRequest().GetRandomProperty("X_FORWARDED_PROTO"), "https",
-         PNocase());
+    // Compose self URL
     m_SecureMode = secure ? eSecure_On : eSecure_Off;
     m_SelfURL    = secure ? "https://" : "http://";
-    m_SelfURL += server;
-    string port = GetRequest().GetProperty(eCgi_ServerPort);
-    // Skip port if it's default for the selected scheme
-    if ((secure  &&  port == "443")  ||  (!secure  &&  port == "80")
-        ||  (server.size() >= port.size() + 2  &&  NStr::EndsWith(server, port)
-             &&  server[server.size() - port.size() - 1] == ':')) {
-        port.erase();
-    }
-    if ( !port.empty() ) {
-        m_SelfURL += ':';
-        m_SelfURL += port;
-    }
+    m_SelfURL += host_port;
+
     string script_uri;
     script_uri = GetRequest().GetRandomProperty("SCRIPT_URL", false);
     if ( script_uri.empty() ) {
@@ -568,7 +580,7 @@ typedef NCBI_PARAM_TYPE(CGI, CORS_Enable) TCORS_Enable;
 
 // Access-Control-Allow-Headers
 NCBI_PARAM_DECL(string, CGI, CORS_Allow_Headers);
-NCBI_PARAM_DEF_EX(string, CGI, CORS_Allow_Headers, kEmptyStr,
+NCBI_PARAM_DEF_EX(string, CGI, CORS_Allow_Headers, "",
                   eParam_NoThread, CGI_CORS_ALLOW_HEADERS);
 typedef NCBI_PARAM_TYPE(CGI, CORS_Allow_Headers) TCORS_AllowHeaders;
 
@@ -597,13 +609,13 @@ typedef NCBI_PARAM_TYPE(CGI, CORS_Allow_Credentials) TCORS_AllowCredentials;
 
 // Access-Control-Expose-Headers
 NCBI_PARAM_DECL(string, CGI, CORS_Expose_Headers);
-NCBI_PARAM_DEF_EX(string, CGI, CORS_Expose_Headers, kEmptyStr,
+NCBI_PARAM_DEF_EX(string, CGI, CORS_Expose_Headers, "",
                   eParam_NoThread, CGI_CORS_EXPOSE_HEADERS);
 typedef NCBI_PARAM_TYPE(CGI, CORS_Expose_Headers) TCORS_ExposeHeaders;
 
 // Access-Control-Max-Age
 NCBI_PARAM_DECL(string, CGI, CORS_Max_Age);
-NCBI_PARAM_DEF_EX(string, CGI, CORS_Max_Age, kEmptyStr,
+NCBI_PARAM_DEF_EX(string, CGI, CORS_Max_Age, "",
                   eParam_NoThread, CGI_CORS_MAX_AGE);
 typedef NCBI_PARAM_TYPE(CGI, CORS_Max_Age) TCORS_MaxAge;
 
