@@ -1,4 +1,4 @@
-/*  $Id: fcgi_run.cpp 523180 2016-12-29 14:05:52Z grichenk $
+/*  $Id: fcgi_run.cpp 561540 2018-04-09 17:20:49Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,6 +34,7 @@
 #include <cgi/cgiapp.hpp>
 #include <cgi/cgi_exception.hpp>
 #include <corelib/request_ctx.hpp>
+#include <signal.h>
 
 
 #if !defined(HAVE_LIBFASTCGI)
@@ -300,6 +301,21 @@ bool s_CheckMemoryLimit(Uint8 total_memory_limit)
 }
 
 
+void s_ScheduleFastCGIExit(void)
+{
+    CCgiApplication* cgiapp = dynamic_cast<CCgiApplication*>(CNcbiApplication::Instance());
+    if (!cgiapp) return;
+    cgiapp->FASTCGI_ScheduleExit();
+    cgiapp->m_CaughtSigterm = true;
+}
+
+
+extern "C" void SignalHandler(int)
+{
+    s_ScheduleFastCGIExit();
+}
+
+
 bool CCgiApplication::x_RunFastCGI(int* result, unsigned int def_iter)
 {
     // Reset the result (which is in fact an error counter here)
@@ -363,6 +379,7 @@ bool CCgiApplication::x_RunFastCGI(int* result, unsigned int def_iter)
 
     // Max. number of the Fast-CGI loop iterations
     unsigned int max_iterations;
+    bool handle_sigterm = false;
     {{
         int x_iterations =
             reg.GetInt("FastCGI", "Iterations", (int) def_iter, 0,
@@ -380,6 +397,8 @@ bool CCgiApplication::x_RunFastCGI(int* result, unsigned int def_iter)
 
         _TRACE("CCgiApplication::Run: FastCGI limited to "
                << max_iterations << " iterations");
+
+        handle_sigterm = reg.GetBool("FastCGI", "Complete_Request_On_Sigterm", false);
     }}
 
     // Watcher file -- to allow for stopping the Fast-CGI loop "prematurely"
@@ -444,7 +463,6 @@ bool CCgiApplication::x_RunFastCGI(int* result, unsigned int def_iter)
     // Diag.prefix related preparations
     const string prefix_pid(NStr::NumericToString(CProcess::GetCurrentPid()) + "-");
 
-
     // Main Fast-CGI loop
     CTime mtime = s_GetModTime(GetArguments().GetProgramName());
 
@@ -544,6 +562,13 @@ bool CCgiApplication::x_RunFastCGI(int* result, unsigned int def_iter)
             break;
         }
 
+        // Optionally turn on SIGTERM handler while processing request to allow
+        // the request to be completed before termination.
+        void (*old_sig_handler)(int) = SIG_DFL;
+        if (handle_sigterm) {
+             old_sig_handler = signal(SIGTERM, SignalHandler);
+        }
+
         // Process the request
         CTime start_time(CTime::eCurrent);
         bool skip_stat_log = false;
@@ -553,6 +578,15 @@ bool CCgiApplication::x_RunFastCGI(int* result, unsigned int def_iter)
         try {
             // Initialize CGI context with the new request data
             CNcbiEnvironment env(penv);
+
+            // Propagate environment values to PassThroughProperties.
+            CRequestContext& rctx = CDiagContext::GetRequestContext();
+            list<string> names;
+            env.Enumerate(names);
+            ITERATE(list<string>, it, names) {
+                rctx.AddPassThroughProperty(*it, env.Get(*it));
+            }
+
             PushDiagPostPrefix(env.Get(m_DiagPrefixEnv).c_str());
 
             CCgiObuffer       obuf(pfout);
@@ -642,7 +676,8 @@ bool CCgiApplication::x_RunFastCGI(int* result, unsigned int def_iter)
                     GetDiagContext().SetAppState(eDiagAppState_Request);
                     if (x_ProcessHelpRequest() ||
                         x_ProcessVersionRequest() ||
-                        CCgiContext::ProcessCORSRequest(m_Context->GetRequest(), m_Context->GetResponse())) {
+                        CCgiContext::ProcessCORSRequest(m_Context->GetRequest(), m_Context->GetResponse()) ||
+                        x_ProcessAdminRequest()) {
                         x_result = 0;
                     }
                     else {
@@ -757,6 +792,14 @@ bool CCgiApplication::x_RunFastCGI(int* result, unsigned int def_iter)
         }
         GetDiagContext().SetAppState(eDiagAppState_RequestEnd);
 
+        // Reset SIGTERM handler to allow termination between requests.
+        if (handle_sigterm) {
+            signal(SIGTERM, old_sig_handler);
+            if (m_ShouldExit  &&  m_CaughtSigterm) {
+                ERR_POST(Message << "Caught SIGTERM and performed graceful shutdown.");
+            }
+        }
+
         // Close current request
         _TRACE("CCgiApplication::x_RunFastCGI: FINISHING");
 
