@@ -1,4 +1,4 @@
-/*  $Id: ncbicgir.cpp 492273 2016-02-16 15:26:05Z ivanov $
+/*  $Id: ncbicgir.cpp 506589 2016-07-08 18:45:26Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -220,12 +220,12 @@ void CCgiResponse::x_RestoreOutputExceptions(void)
 }
 
 
-void CCgiResponse::SetOutput(CNcbiOstream* out, int fd)
+void CCgiResponse::SetOutput(CNcbiOstream* output, int fd)
 {
     x_RestoreOutputExceptions();
 
     m_HeaderWritten = false;
-    m_Output        = out;
+    m_Output        = output;
     m_OutputFD      = fd;
 
     // Make the output stream to throw on write if it's in a bad state
@@ -341,8 +341,8 @@ CNcbiOstream& CCgiResponse::WriteHeader(CNcbiOstream& os) const
         self->m_Cookies.Add(*m_TrackingCookie);
         self->SetHeaderValue(TCGI_TrackingTagName::GetDefault(),
             m_TrackingCookie->GetValue());
-        self->SetHeaderValue("NCBI-PHID",
-            GetDiagContext().GetRequestContext().GetHitID());
+        CRequestContext& rctx = GetDiagContext().GetRequestContext();
+        self->SetHeaderValue("NCBI-PHID", rctx.GetNextSubHitID("m_"));
         // Prevent storing the page in public caches.
         string cc = GetHeaderValue(sm_CacheControl);
         if ( cc.empty() ) {
