@@ -1,4 +1,4 @@
-/*  $Id: user_agent.cpp 514600 2016-09-22 17:56:14Z ivanov $
+/*  $Id: user_agent.cpp 569769 2018-08-29 12:03:50Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -160,24 +160,24 @@ bool CCgiUserAgent::IsBrowser(void) const
 // or names that should be excluded from the check.
 
 NCBI_PARAM_DECL(string, CGI, Bots); 
-NCBI_PARAM_DEF (string, CGI, Bots, kEmptyStr);
+NCBI_PARAM_DEF (string, CGI, Bots, "");
 NCBI_PARAM_DECL(string, CGI, NotBots); 
-NCBI_PARAM_DEF (string, CGI, NotBots, kEmptyStr);
+NCBI_PARAM_DEF (string, CGI, NotBots, "");
 
 NCBI_PARAM_DECL(string, CGI, PhoneDevices); 
-NCBI_PARAM_DEF (string, CGI, PhoneDevices, kEmptyStr);
+NCBI_PARAM_DEF (string, CGI, PhoneDevices, "");
 NCBI_PARAM_DECL(string, CGI, NotPhoneDevices); 
-NCBI_PARAM_DEF (string, CGI, NotPhoneDevices, kEmptyStr);
+NCBI_PARAM_DEF (string, CGI, NotPhoneDevices, "");
 
 NCBI_PARAM_DECL(string, CGI, TabletDevices); 
-NCBI_PARAM_DEF (string, CGI, TabletDevices, kEmptyStr);
+NCBI_PARAM_DEF (string, CGI, TabletDevices, "");
 NCBI_PARAM_DECL(string, CGI, NotTabletDevices); 
-NCBI_PARAM_DEF (string, CGI, NotTabletDevices, kEmptyStr);
+NCBI_PARAM_DEF (string, CGI, NotTabletDevices, "");
 
 NCBI_PARAM_DECL(string, CGI, MobileDevices); 
-NCBI_PARAM_DEF (string, CGI, MobileDevices, kEmptyStr);
+NCBI_PARAM_DEF (string, CGI, MobileDevices, "");
 NCBI_PARAM_DECL(string, CGI, NotMobileDevices); 
-NCBI_PARAM_DEF (string, CGI, NotMobileDevices, kEmptyStr);
+NCBI_PARAM_DEF (string, CGI, NotMobileDevices, "");
 
 
 // Helper method to check devices/bots against external patterns.
@@ -648,6 +648,7 @@ const SBrowser s_Browsers[] = {
     { CCgiUserAgent::eCrawler,      "yacy",                     "yacy.net",                 CCgiUserAgent::eEngine_Bot,     CCgiUserAgent::ePlatform_Unknown,      fAny },
     { CCgiUserAgent::eCrawler,      "Yahoo! Slurp",             "Yahoo! Slurp",             CCgiUserAgent::eEngine_Bot,     CCgiUserAgent::ePlatform_Unknown,      fAppComment },
     { CCgiUserAgent::eCrawler,      "YahooSeeker",              "YahooSeeker",              CCgiUserAgent::eEngine_Bot,     CCgiUserAgent::ePlatform_Unknown,      fAppProduct },
+    { CCgiUserAgent::eCrawler,      "Yandex",                   "yandex",                   CCgiUserAgent::eEngine_Bot,     CCgiUserAgent::ePlatform_Unknown,      fAny },
     { CCgiUserAgent::eCrawler,      "Zao",                      "www.kototoi.org/zao/",     CCgiUserAgent::eEngine_Bot,     CCgiUserAgent::ePlatform_Unknown,      fAppComment },
     { CCgiUserAgent::eCrawler,      "zspider",                  "zspider",                  CCgiUserAgent::eEngine_Bot,     CCgiUserAgent::ePlatform_Unknown,      fAppProduct },
     { CCgiUserAgent::eCrawler,      "ZyBorg",                   "ZyBorg",                   CCgiUserAgent::eEngine_Bot,     CCgiUserAgent::ePlatform_Unknown,      fApp },
