@@ -1,4 +1,4 @@
-/* $Id: ncbi_conn_test.cpp 533072 2017-04-12 19:09:37Z lavr $
+/* $Id: ncbi_conn_test.cpp 544468 2017-08-23 16:38:24Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -127,6 +127,7 @@ EIO_Status CConnTest::Execute(EStage& stage, string* reason)
             break;
         }
     } while (EStage(s++) < stage);
+
     if (status != eIO_Success  &&  status != eIO_Interrupt)
         ExtraCheckOnFailure();
     return status;
@@ -270,22 +271,15 @@ EIO_Status CConnTest::ExtraCheckOnFailure(void)
     } kTests[] = {
         // 0. NCBI default
         { eURL_Http,
-          "",                           0                      }, // NCBI
+          0,                            0                      }, // NCBI
         // 1. External server(s)
         { eURL_Https,
           "www.google.com",             0                      },
-        //    NB: Google's public DNS (@8.8.4.4/8.8), responds at :80 as well
-        { eURL_Http,
-          0,                            "www.google.com"       },
         // 2. NCBI servers, explicitly
         { eURL_Https,
           "www.be-md.ncbi.nlm.nih.gov", "www.ncbi.nlm.nih.gov" }, // NCBI main
         { eURL_Https,
-          "www.st-va.ncbi.nlm.nih.gov", "www.ncbi.nlm.nih.gov" }, // NCBI colo
-        { eURL_Https,
-          "130.14.29.110",              "www.ncbi.nlm.nih.gov" }, // NCBI main
-        { eURL_Https,
-          "165.112.7.20",               "www.ncbi.nlm.nih.gov" }  // NCBI colo
+          "www.st-va.ncbi.nlm.nih.gov", "www.ncbi.nlm.nih.gov" }  // NCBI colo
     };
 
     m_CheckPoint.clear();
@@ -314,17 +308,17 @@ EIO_Status CConnTest::ExtraCheckOnFailure(void)
     vector< AutoPtr<CConn_HttpStream> > http;
     for (n = 0;  n < sizeof(kTests) / sizeof(kTests[0]);  ++n) {
         char user_header[80];
-        net_info->scheme = kTests[n].scheme;
-        const char* host = kTests[n].host;
-        if (!host)
-            host = rand() & 1 ? "8.8.4.4" : "8.8.8.8";
-        if (*host) {
+        net_info->scheme  = kTests[n].scheme;
+        const char* host  = kTests[n].host;
+        const char* vhost = kTests[n].vhost;
+        if (host) {
             _ASSERT(::strlen(host) < sizeof(net_info->host) - 1);
-            ::strcpy(net_info->host, host);
+            unsigned int ip = vhost ? CSocketAPI::gethostbyname(host) : 0;
+            ::strcpy(net_info->host, ip ? CSocketAPI::ntoa(ip).c_str() : host);
         }
-        if (kTests[n].vhost) {
-            _ASSERT(::strlen(kTests[n].vhost) + 6 < sizeof(user_header) - 1);
-            ::sprintf(user_header, "Host: %s", kTests[n].vhost);
+        if (vhost) {
+            _ASSERT(::strlen(vhost) + 6 < sizeof(user_header) - 1);
+            ::sprintf(user_header, "Host: %s", vhost);
         } else
             *user_header = '\0';
         SAuxData* auxdata = new SAuxData(m_Canceled, 0);
@@ -1165,38 +1159,46 @@ EIO_Status CConnTest::StatefulOkay(string* reason)
     CTime  time(CTime::eCurrent, CTime::eLocal);
     time_t seed = time.GetTimeT();
 
-    char buf[(1 << 10) + (8 + 1)];
-    sprintf(buf, "%08X", (unsigned int) seed);
-    size_t size = rnd(2, (sizeof(buf) - (8 + 1)) >> 3);
+    char send[(1 << 10) + (8 + 1)];
+    char recv[sizeof(send)];
+    sprintf(send, "%08X", (unsigned int) seed);
+    size_t size = rnd(2, (sizeof(send) - (8 + 1)) >> 3);
 
     size_t i;
     for (i = 1;  i < size;  ++i)
-        sprintf(buf + (i << 3), "%08X", (unsigned int) rand());
-    memset(&buf[size <<= 3], 0, 8);
-    size += 8;
+        sprintf(send + (i << 3), "%08X", (unsigned int) rand());
+    memset(&send[size <<= 3], 0, 8);
+    send[size += 8] = '\0';
 
     CConn_ServiceStream echo(kEcho, fSERV_Any, net_info, 0/*xtra*/, m_Timeout);
     echo.SetCanceledCallback(m_Canceled);
 
     streamsize n = 0;
-    bool iofail = !echo.write(buf, size)  ||  !echo.flush()
-        ||  !(n = CStreamUtils::Readsome(echo, buf, size));
+    bool iofail = !echo.write(send, size)  ||  !echo.flush()
+        ||  !(n = CStreamUtils::Readsome(echo, recv, size));
     if (!iofail) {
         if (n < (streamsize) size) {
-            if (!echo.read(buf + n, (streamsize) size - n))
+            if (!echo.read(recv + n, (streamsize) size - n))
                 iofail = true;
             else
                 n += echo.gcount();
         }
         if (n == (streamsize) size) {
-            time_t now = (time_t) NStr::StringToNumeric<unsigned int>
-                (CTempString(buf, 8), NStr::fConvErr_NoThrow, 16/*radix*/);
-            time.SetTimeT(now);
+            for (i = 8;  i < size;  ++i) {
+                if (send[i] != recv[i])
+                    break;
+            }
+            if (i >= size) {
+                time_t now = (time_t) NStr::StringToNumeric<unsigned int>
+                    (CTempString(recv,8), NStr::fConvErr_NoThrow, 16/*radix*/);
+                time.SetTimeT(now);
+            } else
+                iofail = true;
         } else
             iofail = true;
     }
     EIO_Status status = ConnStatus(iofail, &echo);
-    buf[n] = '\0';
+    recv[n] = '\0';
 
     string temp;
     if (status == eIO_Interrupt)
@@ -1255,7 +1257,7 @@ EIO_Status CConnTest::StatefulOkay(string* reason)
             }
         } else if (!str) {
             if (n  &&  net_info  &&  net_info->http_proxy_port
-                &&  NStr::strncasecmp(buf, kFWSign, (size_t) n) == 0) {
+                &&  NStr::strncasecmp(recv, kFWSign, (size_t) n) == 0) {
                 temp += "NCBI Firewall";
                 if (!net_info->firewall)
                     temp += " (Connection Relay)";
