@@ -1,4 +1,4 @@
-/* $Id: ncbi_connection.c 532676 2017-04-07 14:32:22Z lavr $
+/* $Id: ncbi_connection.c 565391 2018-06-11 18:09:29Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -108,6 +108,7 @@
           static const STimeout* timeout = 0/*dummy*/;                  \
           CONN_LOG_EX(subcode, func_name, eLOG_Critical,                \
                       "Corrupted connection handle", 0);                \
+          assert(0);                                                    \
       }                                                                 \
   } while (0)
 
@@ -138,7 +139,8 @@ typedef enum {
 } ECONN_State;
 
 
-/* Connection internal data:  meta *must* come first
+/* Connection internal data.
+ * NOTE:  meta *must* come first!
  */
 typedef struct SConnectionTag {
     SMetaConnector  meta;        /* VTable of operations and list            */
@@ -280,10 +282,9 @@ static EIO_Status x_ReInit(CONN conn, CONNECTOR connector, int/*bool*/ close)
         /* erase unread data */
         BUF_Erase(conn->buf);
 
-        if (!x_conn) {
-            /* re-init with same connector does not cause the callback */
+        if (!x_conn)
             status = x_Callback(conn, eCONN_OnClose, 0);
-        }
+        /* else: re-init with same connector does not cause the callback */
 
         if (conn->state & eCONN_Open) {
             /* call current connector's "CLOSE" method */
@@ -331,6 +332,7 @@ static EIO_Status x_ReInit(CONN conn, CONNECTOR connector, int/*bool*/ close)
             return status;
         assert(conn->meta.list);
         conn->state = eCONN_Closed;
+        assert(conn->meta.default_timeout != kDefaultTimeout);
     }
 
     assert(conn->state != eCONN_Open  &&  conn->state != eCONN_Bad);
@@ -376,7 +378,7 @@ static EIO_Status s_Open(CONN conn)
     } else
         status  = eIO_NotSupported;
     if (status == eIO_Success) {
-        conn->flags   &= ~fCONN_Flush;
+        conn->flags   &= (TCONN_Flags)(~fCONN_Flush);
         conn->r_status = eIO_Success;
         conn->w_status = eIO_Success;
         conn->state    = eCONN_Open;
@@ -405,7 +407,7 @@ extern EIO_Status CONN_CreateEx
         conn = (SConnection*) calloc(1, sizeof(SConnection));
 
         if (conn) {
-            conn->flags     = flags & ~fCONN_Flush;
+            conn->flags     = flags & (TCONN_Flags)(~fCONN_Flush);
             conn->state     = eCONN_Unusable;
             conn->o_timeout = kDefaultTimeout;
             conn->r_timeout = kDefaultTimeout;
@@ -426,7 +428,7 @@ extern EIO_Status CONN_CreateEx
         CONN_LOG(2, Create, eLOG_Error, "NULL connector");
     }
 
-    assert(!conn == (status != eIO_Success));
+    assert(!conn == !(status == eIO_Success));
     CONN_CALLTRACE(Create);
     *connection = conn;
     return status;
@@ -682,7 +684,7 @@ static EIO_Status s_CONN_Write
 
         if (*n_written) {
             conn->w_pos += *n_written;
-            conn->flags &= ~fCONN_Flush;
+            conn->flags &= (TCONN_Flags)(~fCONN_Flush);
             break;
         }
         if (!size  ||  status != eIO_Timeout)
@@ -778,7 +780,7 @@ extern EIO_Status CONN_Write
 
 extern EIO_Status CONN_Pushback
 (CONN        conn,
- const void* buf,
+ const void* data,
  size_t      size)
 {
     CONN_NOT_NULL(19, Pushback);
@@ -792,7 +794,7 @@ extern EIO_Status CONN_Pushback
     if (!conn->meta.read)
         return eIO_NotSupported;
 
-    return BUF_Pushback(&conn->buf, buf, size) ? eIO_Success : eIO_Unknown;
+    return BUF_Pushback(&conn->buf, data, size) ? eIO_Success : eIO_Unknown;
 }
 
 
@@ -850,7 +852,7 @@ static EIO_Status s_CONN_Read
     else for (;;) {
         size_t x_read;
 
-        /* read data from the internal peek buffer, if any */
+        /* read data from the internal peek/pushback buffer, if any */
         if (size) {
             x_read = (peek
                       ? BUF_Peek(conn->buf, buf, size - *n_read)
@@ -1194,7 +1196,7 @@ extern EIO_Status CONN_SetFlags(CONN conn, TCONN_Flags flags)
     if (!conn)
         return eIO_InvalidArg;
 
-    flags &=              ~fCONN_Flush;
+    flags &= (TCONN_Flags)(~fCONN_Flush);
     flags |= conn->flags & fCONN_Flush;
     conn->flags = flags;
     return eIO_Success;
@@ -1205,7 +1207,7 @@ extern TCONN_Flags CONN_GetFlags(CONN conn)
 {
     CONN_CALLTRACE(GetFlags);
 
-    return conn ? conn->flags & ~fCONN_Flush : 0;
+    return conn ? conn->flags & (TCONN_Flags)(~fCONN_Flush) : 0;
 }
 
 
