@@ -1,4 +1,4 @@
-/* $Id: ncbi_connutil.c 533696 2017-04-18 21:33:46Z lavr $
+/* $Id: ncbi_connutil.c 569141 2018-08-16 16:45:11Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -51,24 +51,32 @@
 static TNCBI_BigCount s_FWPorts[1024 / sizeof(TNCBI_BigCount)] = { 0 };
 
 
-static char* x_StrcatCRLF(char* dst, const char* src)
+static int/*bool*/ x_StrcatCRLF(char** dstp, const char* src)
 {
+    char*  dst = *dstp;
     size_t dstlen = dst  &&  *dst ? strlen(dst) : 0;
     size_t srclen = src  &&  *src ? strlen(src) : 0;
-    if (dstlen  ||  srclen) {
+    if (dstlen  &&  dst[dstlen - 1] == '\n') {
+        if (--dstlen  &&  dst[dstlen - 1] == '\r')
+            --dstlen;
+    }
+    while (srclen) {
+        if (!isspace((unsigned char)(*src)))
+            break;
+        --srclen;
+        ++src;
+    }
+    while (srclen) {
+        if (!isspace((unsigned char) src[srclen - 1]))
+            break;
+        --srclen;
+    }
+    if (dstlen | srclen) {
         size_t len;
         char*  temp;
-        if (dstlen  &&  dst[dstlen - 1] == '\n') {
-            if (--dstlen  &&  dst[dstlen - 1] == '\r')
-                --dstlen;
-        }
-        if (srclen  &&  src[srclen - 1] == '\n') {
-            if (--srclen  &&  src[srclen - 1] == '\r')
-                --srclen;
-        }
         len = (dstlen ? dstlen + 2 : 0) + (srclen ? srclen + 2 : 0) + 1;
         if (!(temp = (char*)(dst ? realloc(dst, len) : malloc (len))))
-            return 0;
+            return 0/*failure*/;
         dst = temp;
         if (dstlen) {
             temp += dstlen;
@@ -81,7 +89,8 @@ static char* x_StrcatCRLF(char* dst, const char* src)
             memcpy(temp, "\r\n", 3);
         }
     }
-    return dst;
+    *dstp = dst;
+    return 1/*success*/;
 }
 
 
@@ -171,7 +180,7 @@ static const char* x_GetValue(const char* svc, size_t svclen,
     }
     CORE_UNLOCK;
 
-    /* Last resort: Search for 'param' in default registry section */
+    /* Last resort: Search for 'param' in default registry section [CONN] */
     s += sizeof(DEF_CONN_REG_SECTION);
     CORE_REG_GET(DEF_CONN_REG_SECTION, s, value, value_size, def_value);
     return value;
@@ -220,7 +229,7 @@ extern const char* ConnNetInfo_GetValue(const char* service, const char* param,
         return 0;
 
     if (service) {
-        if (!*service)
+        if (!*service  ||  strpbrk(service, "*?"))
             service = 0;
         else if (!(service = SERV_ServiceName(service)))
             return 0;
@@ -339,7 +348,7 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
     double dbl;
     char*  e;
 
-    if (service  &&  *service) {
+    if (service  &&  *service  &&  !strpbrk(service, "?*")) {
         if (!(service = SERV_ServiceName(service)))
             return 0;
         assert(*service);
@@ -378,7 +387,7 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
 
     /* external mode */
     REG_VALUE(REG_CONN_EXTERNAL, str, DEF_CONN_EXTERNAL);
-    info->external = ConnNetInfo_Boolean(str);
+    info->external = ConnNetInfo_Boolean(str) ? 1 : 0;
     
     /* firewall mode */
     REG_VALUE(REG_CONN_FIREWALL, str, DEF_CONN_FIREWALL);
@@ -386,11 +395,11 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
 
     /* stateless client */
     REG_VALUE(REG_CONN_STATELESS, str, DEF_CONN_STATELESS);
-    info->stateless = ConnNetInfo_Boolean(str);
+    info->stateless = ConnNetInfo_Boolean(str) ? 1 : 0;
 
     /* prohibit use of the local load balancer */
     REG_VALUE(REG_CONN_LB_DISABLE, str, DEF_CONN_LB_DISABLE);
-    info->lb_disable = ConnNetInfo_Boolean(str);
+    info->lb_disable = ConnNetInfo_Boolean(str) ? 1 : 0;
 
     /* level of debug printout */
     REG_VALUE(REG_CONN_DEBUG_PRINTOUT, str, DEF_CONN_DEBUG_PRINTOUT);
@@ -415,7 +424,7 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
     /* port # */
     REG_VALUE(REG_CONN_PORT, str, DEF_CONN_PORT);
     errno = 0;
-    if (*str  &&  (val = strtoul(str, &e, 10)) > 0  &&  !errno
+    if (*str  &&  (val = (long) strtoul(str, &e, 10)) > 0  &&  !errno
         &&  !*e  &&  val < (1 << 16)) {
         info->port = (unsigned short) val;
     } else
@@ -433,7 +442,7 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
     if (info->http_proxy_host[0]) {
         REG_VALUE(REG_CONN_HTTP_PROXY_PORT, str, DEF_CONN_HTTP_PROXY_PORT);
         errno = 0;
-        if (*str  &&  (val = strtoul(str, &e, 10)) > 0
+        if (*str  &&  (val = (long) strtoul(str, &e, 10)) > 0
             &&  !errno  &&  !*e  &&  val < (1 << 16)) {
             info->http_proxy_port = (unsigned short) val;
         } else
@@ -446,7 +455,7 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
                   DEF_CONN_HTTP_PROXY_PASS);
         /* HTTP proxy leakout */
         REG_VALUE(REG_CONN_HTTP_PROXY_LEAK, str, DEF_CONN_HTTP_PROXY_LEAK);
-        info->http_proxy_leak    =   ConnNetInfo_Boolean(str);
+        info->http_proxy_leak    = ConnNetInfo_Boolean(str) ? 1 : 0;
     } else {
         info->http_proxy_port    =   0;
         info->http_proxy_user[0] = '\0';
@@ -456,7 +465,7 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
 
     /* push HTTP auth tags */
     REG_VALUE(REG_CONN_HTTP_PUSH_AUTH, str, DEF_CONN_HTTP_PUSH_AUTH);
-    info->http_push_auth = ConnNetInfo_Boolean(str);
+    info->http_push_auth = ConnNetInfo_Boolean(str) ? 1 : 0;
 
     /* max. # of attempts to establish connection */
     REG_VALUE(REG_CONN_MAX_TRY, str, 0);
@@ -466,7 +475,8 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
     /* connection timeout */
     REG_VALUE(REG_CONN_TIMEOUT, str, 0);
     val = (long) strlen(str);
-    if (val < 3  ||  8 < val  ||  strncasecmp(str, "infinite", val) != 0) {
+    if (val < 3  ||  8 < val
+        ||  strncasecmp(str, "infinite", (size_t) val) != 0) {
         if (!*str || (dbl = NCBI_simple_atof(str, &e)) < 0.0 || errno || *e)
             dbl = DEF_CONN_TIMEOUT;
         info->tmo.sec      = (unsigned int)  dbl;
@@ -478,8 +488,12 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
         info->timeout      = kInfiniteTimeout/*0*/;
 
     /* HTTP user header */
+    info->http_user_header = 0;
     REG_VALUE(REG_CONN_HTTP_USER_HEADER, str, DEF_CONN_HTTP_USER_HEADER);
-    info->http_user_header = *str ? x_StrcatCRLF(NULL, str) : 0;
+    if (!x_StrcatCRLF((char**) &info->http_user_header, str)) {
+        free(info);
+        goto out;
+    }
 
     /* default referer */
     ConnNetInfo_GetValue(0, REG_CONN_HTTP_REFERER, str, sizeof(str),
@@ -492,6 +506,7 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
     /* magic */
     info->magic = CONN_NET_INFO_MAGIC;
 
+ out:
     if (len)
         free((void*) service);
 
@@ -699,33 +714,20 @@ extern int/*bool*/ ConnNetInfo_SetUserHeader(SConnNetInfo* info,
     if (!s_InfoIsValid(info))
         return 0/*failure*/;
 
-    if (info->http_user_header)
+    if (info->http_user_header) {
         free((void*) info->http_user_header);
-    if (!user_header  ||  !*user_header)
         info->http_user_header = 0;
-    else if (!(info->http_user_header = x_StrcatCRLF(NULL, user_header)))
-        return 0/*failure*/;
-    return 1/*success*/;
+    }
+    return x_StrcatCRLF((char**) &info->http_user_header, user_header);
 }
 
 
 extern int/*bool*/ ConnNetInfo_AppendUserHeader(SConnNetInfo* info,
                                                 const char*   user_header)
 {
-    char* new_header;
-
     if (!s_InfoIsValid(info))
         return 0/*failure*/;
-
-    if (!info->http_user_header  ||  !*info->http_user_header)
-        return ConnNetInfo_SetUserHeader(info, user_header);
-
-    new_header = (char*) info->http_user_header;
-    if (!(new_header = x_StrcatCRLF(new_header, user_header)))
-        return 0/*failure*/;
-
-    info->http_user_header = new_header;
-    return 1/*success*/;
+    return x_StrcatCRLF((char**) &info->http_user_header, user_header);
 }
 
 
@@ -779,13 +781,13 @@ static int/*bool*/ s_ModifyUserHeader(SConnNetInfo*      info,
     if (!s_InfoIsValid(info))
         return 0/*failure*/;
 
-    if (!user_header || !(newhdrlen = strlen(user_header)))
+    if (!user_header  ||  !(newhdrlen = strlen(user_header)))
         return 1/*success*/;
 
-    if (!(hdr = (char*) info->http_user_header) || !(hdrlen = strlen(hdr))) {
+    if (!(hdr = (char*) info->http_user_header)  ||  !(hdrlen = strlen(hdr))) {
         if (op == eUserHeaderOp_Delete)
             return 1/*success*/;
-        if (!hdr && !(hdr = strdup("")))
+        if (!hdr  &&  !(hdr = strdup("")))
             return 0/*failure*/;
         hdrlen = 0;
     }
@@ -930,6 +932,12 @@ static int/*bool*/ s_ModifyUserHeader(SConnNetInfo*      info,
         }
     }
 
+    assert(hdr);
+    if (!*hdr) {
+        assert(!hdrlen);
+        free(hdr);
+        hdr = 0;
+    }
     info->http_user_header = hdr;
     if (retval  &&  op != eUserHeaderOp_Delete)
         retval = ConnNetInfo_AppendUserHeader(info, newhdr);
@@ -1276,7 +1284,7 @@ static const char* x_Port(unsigned short port, char buf[])
 static const char* x_ReqMethod(TReqMethod req_method, char buf[])
 {
     int/*bool*/ v1 = req_method & eReqMethod_v1 ? 1/*true*/ : 0/*false*/;
-    req_method &= ~eReqMethod_v1;
+    req_method &= (TReqMethod)(~eReqMethod_v1);
     switch (req_method) {
     case eReqMethod_Any:
         return v1 ? "ANY/1.1"     : "ANY";
@@ -1324,7 +1332,7 @@ static const char* x_Firewall(unsigned int firewall)
 
 static const char* x_CredInfo(NCBI_CRED cred, char buf[])
 {
-    int who, what;
+    unsigned int who, what;
     if (!cred)
         return "NULL";
     who  = (cred->type / 100) * 100;
@@ -1428,13 +1436,14 @@ extern void ConnNetInfo_Log(const SConnNetInfo* info, ELOG_Level sev, LOG lg)
     else
         s_SaveKeyval(s, "client_host",     "(default)");
     s_SaveKeyval    (s, "scheme",         (info->scheme
-                                           ? x_Scheme((EURLScheme)info->scheme,
-                                                      buf)
+                                           ? x_Scheme((EURLScheme)
+                                                      info->scheme, buf)
                                            : "(unspec)"));
-    s_SaveKeyval    (s, "req_method",      x_ReqMethod(info->req_method
-                                                       | (info->version
-                                                          ? eReqMethod_v1
-                                                          : 0), buf));
+    s_SaveKeyval    (s, "req_method",      x_ReqMethod((TReqMethod)
+                                                        (info->req_method
+                                                         | (info->version
+                                                            ? eReqMethod_v1
+                                                            : 0)), buf));
 #if defined(_DEBUG)  &&  !defined(NDEBUG)
     s_SaveString    (s, "user",            info->user);
 #else
@@ -1515,7 +1524,7 @@ extern char* ConnNetInfo_URL(const SConnNetInfo* info)
     if (!info  ||  !s_InfoIsValid(info))
         return 0/*failed*/;
 
-    req_method = info->req_method & ~eReqMethod_v1;
+    req_method = info->req_method & (TReqMethod)(~eReqMethod_v1);
     scheme = x_Scheme((EURLScheme) info->scheme, buf);
     assert(!scheme  ||  *scheme);
     if (scheme  &&  !isalpha((unsigned char)(*scheme)))
@@ -1541,14 +1550,15 @@ extern char* ConnNetInfo_URL(const SConnNetInfo* info)
     if (url) {
         assert(scheme  &&  args);
         strlwr((char*) memcpy(url, scheme, schlen + 1));
-        len  = schlen;
-        len += sprintf(url + len, &"://%s"[schlen
-                                           ? 0
-                                           : req_method == eReqMethod_Connect
-                                           ? 3
-                                           : 1], info->host);
+        len = schlen;
+        len += (size_t) sprintf(url + len,
+                                &"://%s"[schlen
+                                         ? 0
+                                         : req_method == eReqMethod_Connect
+                                         ? 3
+                                         : 1], info->host);
         if (info->port  ||  !path/*req_method == eReqMethod_Connect*/)
-            len += sprintf(url + len, ":%hu", info->port);
+            len += (size_t) sprintf(url + len, ":%hu", info->port);
         sprintf(url + len, "%s%s%s%s",
                 &"/"[! path  ||  *path == '/'], path ? path : "",
                 &"?"[!*args  ||  *args == '#'], args);
@@ -1575,8 +1585,7 @@ extern int/*bool*/ ConnNetInfo_SetTimeout(SConnNetInfo*   info,
 extern void ConnNetInfo_Destroy(SConnNetInfo* info)
 {
     if (info) {
-        ConnNetInfo_SetUserHeader(info, 0);
-        if (info->http_referer) {
+        if (ConnNetInfo_SetUserHeader(info, 0)  &&  info->http_referer) {
             free((void*) info->http_referer);
             info->http_referer = 0;
         }
@@ -1632,8 +1641,7 @@ extern EIO_Status URL_ConnectEx
     size_t      user_hdr_len = user_hdr  &&  *user_hdr ? strlen(user_hdr) : 0;
 
     /* sanity check first */
-    if (!sock  ||  !host  ||  !*host  ||  !path  ||  !*path
-        ||  (user_hdr  &&  *user_hdr  &&  user_hdr[user_hdr_len - 1] != '\n')){
+    if (!sock  ||  !host  ||  !*host  ||  !path  ||  !*path) {
         CORE_LOG_X(2, eLOG_Critical, "[URL_Connect]  Bad argument(s)");
         if (sock) {
             s = *sock;
@@ -1663,8 +1671,8 @@ extern EIO_Status URL_ConnectEx
     while (user_hdr_len) {
         if (!isspace((unsigned char) user_hdr[0]))
             break;
-        ++user_hdr;
         --user_hdr_len;
+        ++user_hdr;
     }
     while (user_hdr_len) {
         if (!isspace((unsigned char) user_hdr[user_hdr_len - 1]))
@@ -1673,7 +1681,7 @@ extern EIO_Status URL_ConnectEx
     }
 
     http = kHttp[req_method < eReqMethod_v1 ? 0 : 1];
-    x_req_meth = (EReqMethod)(req_method & ~eReqMethod_v1);
+    x_req_meth = (EReqMethod)(req_method & (TReqMethod)(~eReqMethod_v1));
     /* select request method and its verbal representation */
     if (x_req_meth == eReqMethod_Any)
         x_req_meth  = content_length ? eReqMethod_Post : eReqMethod_Get;
@@ -1871,11 +1879,13 @@ extern SOCK URL_Connect
                     sprintf(x_host + x_len, ":%hu", port);
                 else
                     x_host[x_len] = '\0';
-                if (!(x_hdr = x_StrcatCRLF(x_host, user_hdr))) {
+                if (!x_StrcatCRLF(&x_host, user_hdr)) {
                     x_hdr = user_hdr;
                     free(x_host);
-                }
-            }
+                } else
+                    x_hdr = x_host;
+            } else
+                x_hdr = user_hdr;
         } else
             x_hdr = user_hdr;
 
@@ -2154,11 +2164,11 @@ extern EIO_Status BUF_StripToPattern
  */
 static int s_HexChar(char ch)
 {
-    unsigned int rc = ch - '0';
+    unsigned int rc = (unsigned int)(ch - '0');
     if (rc <= 9)
-        return rc;
-    rc = (ch | ' ') - 'a';
-    return rc <= 5 ? (int) rc + 10 : -1;
+        return (int)rc;
+    rc = (unsigned int)((ch | ' ') - 'a');
+    return rc <= 5 ? (int)(rc + 10) : -1;
 }
 
 
@@ -2211,8 +2221,8 @@ extern int/*bool*/ URL_DecodeEx
  size_t*     dst_written,
  const char* allow_symbols)
 {
-    unsigned char* src = (unsigned char*) src_buf;
-    unsigned char* dst = (unsigned char*) dst_buf;
+    char* src = (char*) src_buf;
+    char* dst = (char*) dst_buf;
 
     *src_read    = 0;
     *dst_written = 0;
@@ -2233,7 +2243,7 @@ extern int/*bool*/ URL_DecodeEx
             if (*src_read + 2 < src_size) {
                 if ((i1 = s_HexChar(src[1])) != -1  &&
                     (i2 = s_HexChar(src[2])) != -1) {
-                    *dst = (unsigned char)((i1 << 4) + i2);
+                    *dst = (char)((i1 << 4) + i2);
                     *src_read += 2;
                     src       += 2;
                     break;
@@ -2257,8 +2267,8 @@ extern int/*bool*/ URL_DecodeEx
         }/*switch*/
     }
 
-    assert(src == (unsigned char*) src_buf + *src_read   );
-    assert(dst == (unsigned char*) dst_buf + *dst_written);
+    assert(src == (char*) src_buf + *src_read   );
+    assert(dst == (char*) dst_buf + *dst_written);
     return 1/*true*/;
 }
 
@@ -2285,8 +2295,8 @@ extern void URL_EncodeEx
  size_t*     dst_written,
  const char* allow_symbols)
 {
-    unsigned char* src = (unsigned char*) src_buf;
-    unsigned char* dst = (unsigned char*) dst_buf;
+    char* src = (char*) src_buf;
+    char* dst = (char*) dst_buf;
 
     *src_read    = 0;
     *dst_written = 0;
@@ -2297,7 +2307,7 @@ extern void URL_EncodeEx
           (*src_read)++, (*dst_written)++, src++, dst++) {
         const char* subst = allow_symbols ? strchr(allow_symbols, *src) : 0;
         if (!subst)
-            subst = s_EncodeTable[*src];
+            subst = s_EncodeTable[*((unsigned char*) src)];
         if (*subst != '%') {
             *dst = *subst;
         } else if (*dst_written < dst_size - 2) {
@@ -2308,8 +2318,8 @@ extern void URL_EncodeEx
         } else
             return;
     }
-    assert(src == (unsigned char*) src_buf + *src_read   );
-    assert(dst == (unsigned char*) dst_buf + *dst_written);
+    assert(src == (char*) src_buf + *src_read   );
+    assert(dst == (char*) dst_buf + *dst_written);
 }
 
 
@@ -2518,8 +2528,8 @@ int/*bool*/ SERV_IsFirewallPort(unsigned short port)
 
 void SERV_PrintFirewallPorts(char* buf, size_t bufsize, EFWMode mode)
 {
+    unsigned short m;
     size_t len, n;
-    unsigned int m;
 
     assert(buf  &&  bufsize > 1);
     switch (mode) {
@@ -2532,20 +2542,22 @@ void SERV_PrintFirewallPorts(char* buf, size_t bufsize, EFWMode mode)
     default:
         break;
     }
+    m = 0;
     len = 0;
-    for (n = m = 0; n < SizeOf(s_FWPorts); ++n, m += sizeof(s_FWPorts[0])<<3) {
+    for (n = 0;  n < SizeOf(s_FWPorts);  ++n) {
         unsigned short p;
         TNCBI_BigCount mask = s_FWPorts[n];
-        for (p = (unsigned short) m + 1;  mask;  ++p, mask >>= 1) {
+        for (p = (unsigned short)(m + 1);  mask;  ++p, mask >>= 1) {
             if (mask & 1) {
                 char port[10];
-                int  k = sprintf(port, &" %hu"[!len], p);
+                size_t k = (size_t) sprintf(port, &" %hu"[!len], p);
                 if (len + k < bufsize) {
                     memcpy(buf + len, port, k);
                     len += k;
                 }
             }
         }
+        m = (unsigned short)(m + (sizeof(s_FWPorts[0]) << 3));
     }
     buf[len] = '\0';
 }
