@@ -1,4 +1,4 @@
-/* $Id: ncbi_core.c 545593 2017-09-07 18:06:33Z ivanov $
+/* $Id: ncbi_core.c 568303 2018-08-02 14:55:14Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -55,18 +55,19 @@
 
 extern const char* IO_StatusStr(EIO_Status status)
 {
-    static const char* kStatusStr[eIO_Unknown + 1] = {
+    static const char* kStatusStr[EIO_N_STATUS] = {
         "Success",
         "Timeout",
-        "Closed",
+        "",
         "Interrupt",
         "Invalid argument",
         "Not supported",
-        "Unknown"
+        "Unknown",
+        "Closed"
     };
 
-    assert(eIO_Success <= status  &&  status <= eIO_Unknown);
-    return eIO_Success <= status  &&  status <= eIO_Unknown
+    assert(eIO_Success <= status  &&  status <= eIO_Closed);
+    return eIO_Success <= status  &&  status <= eIO_Closed
         ? kStatusStr[status]
         : 0;
 }
@@ -263,9 +264,9 @@ extern int/*bool*/ MT_LOCK_DoInternal(MT_LOCK lk, EMT_Lock how)
  */
 
 /* Lock/unlock the logger */
-#define LOG_LOCK_WRITE  verify(MT_LOCK_Do(lg->lock, eMT_Lock))
-#define LOG_LOCK_READ   verify(MT_LOCK_Do(lg->lock, eMT_LockRead))
-#define LOG_UNLOCK      verify(MT_LOCK_Do(lg->lock, eMT_Unlock))
+#define LOG_LOCK_WRITE  verify(MT_LOCK_Do(lg->lock, eMT_Lock)     != 0)
+#define LOG_LOCK_READ   verify(MT_LOCK_Do(lg->lock, eMT_LockRead) != 0)
+#define LOG_UNLOCK      verify(MT_LOCK_Do(lg->lock, eMT_Unlock)   != 0)
 
 
 /* Check the validity of the logger */
@@ -450,9 +451,9 @@ extern void LOG_Write
  */
 
 /* Lock/unlock the registry  */
-#define REG_LOCK_WRITE  verify(MT_LOCK_Do(rg->lock, eMT_Lock))
-#define REG_LOCK_READ   verify(MT_LOCK_Do(rg->lock, eMT_LockRead))
-#define REG_UNLOCK      verify(MT_LOCK_Do(rg->lock, eMT_Unlock))
+#define REG_LOCK_WRITE  verify(MT_LOCK_Do(rg->lock, eMT_Lock)     != 0)
+#define REG_LOCK_READ   verify(MT_LOCK_Do(rg->lock, eMT_LockRead) != 0)
+#define REG_UNLOCK      verify(MT_LOCK_Do(rg->lock, eMT_Unlock)   != 0)
 
 
 /* Check the validity of the registry */
