@@ -1,4 +1,4 @@
-/* $Id: ncbi_file_connector.c 454485 2014-12-15 20:41:15Z lavr $
+/* $Id: ncbi_file_connector.c 565391 2018-06-11 18:09:29Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -156,7 +156,7 @@ static EIO_Status s_VT_Open
         if (!(xxx->fout = fopen(xxx->ofname, mode)))
             return eIO_Unknown;
         if (xxx->attr.w_mode == eFCM_Seek  &&  xxx->attr.w_pos
-            &&  fseek(xxx->fout, xxx->attr.w_pos, SEEK_SET) != 0) {
+            &&  fseek(xxx->fout, (long) xxx->attr.w_pos, SEEK_SET) != 0) {
             fclose(xxx->fout);
             xxx->fout = 0;
             return eIO_Unknown;
@@ -173,7 +173,7 @@ static EIO_Status s_VT_Open
             return eIO_Unknown;
         }
         if (xxx->attr.r_pos
-            &&  fseek(xxx->finp, xxx->attr.r_pos, SEEK_SET) != 0) {
+            &&  fseek(xxx->finp, (long) xxx->attr.r_pos, SEEK_SET) != 0) {
             fclose(xxx->finp);
             xxx->finp = 0;
             if (xxx->fout) {
