@@ -1,4 +1,4 @@
-/* $Id: ncbi_ftp_connector.c 532866 2017-04-10 17:55:22Z lavr $
+/* $Id: ncbi_ftp_connector.c 565391 2018-06-11 18:09:29Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -36,13 +36,13 @@
  *   See <connect/ncbi_connector.h> for the detailed specification of
  *   the connector's methods and structures.
  *
- *   Note:  We do not implement transfers of files whose names include
- *          CR or LF characters:  for those to work, all FTP commands will
- *          have to be required to terminate with CRLF at the user level
- *          (currently, LF alone acts as the command terminator), and all
- *          solitary CRs to be recoded as 'CR\0' (per the RFC), yet all
- *          solitary LFs to be passed through.  Nonetheless, we escape all
- *          IACs for the sake of safety of the control connection.
+ *   Note:  We do not implement transfers of files whose names include CR or LF
+ *          character(s):  for those to work, all FTP commands will have to be
+ *          required to terminate with CRLF at the user level (currently, LF
+ *          alone acts as the command terminator), and all solitary CRs to be
+ *          recoded as 'CR\0' (per the RFC), yet all solitary LFs to be passed
+ *          through.  Nonetheless, we escape all IACs for the sake of safety of
+ *          the control connection.
  *
  */
 
@@ -348,11 +348,13 @@ static EIO_Status s_FTPCommandEx(SFTPConnector* xxx,
         return eIO_Closed;
 
     cmdlen  = strlen(cmd);
-    arglen  = arg ? strlen(arg) : 0;
     linelen = cmdlen + 2;
-    if (arg)
+    if (arg) {
+        arglen = *arg ? strlen(arg) : 0;
         linelen += 1 + arglen;
-    line    = linelen < sizeof(x_buf) ? x_buf : (char*) malloc(linelen + 1);
+    } else
+        arglen = 0/*unused*/;
+    line = linelen < sizeof(x_buf) ? x_buf : (char*) malloc(linelen + 1);
 
     if (line) {
         ESwitch log = eDefault;
@@ -365,8 +367,9 @@ static EIO_Status s_FTPCommandEx(SFTPConnector* xxx,
         line[cmdlen++] = '\r';
         line[cmdlen++] = '\n';
         line[cmdlen]   = '\0';
+        assert(cmdlen == linelen);
         log = off ? SOCK_SetDataLogging(xxx->cntl, eOff) : eOff;
-        status = SOCK_Write(xxx->cntl, line, cmdlen, 0, eIO_WritePersist);
+        status = SOCK_Write(xxx->cntl, line, linelen, 0, eIO_WritePersist);
         if (off  &&  log != eOff) {
             SOCK_SetDataLogging(xxx->cntl, log);
             if (log == eOn  ||  SOCK_SetDataLoggingAPI(eDefault) == eOn) {
@@ -403,69 +406,69 @@ static EIO_Status x_FTPHelpCB(SFTPConnector* xxx, int code,
         assert(code == 211  ||  code == 214);
         if ((s = x_4Word(line, "NOOP")) != 0) {  /* RFC 959 */
             if (s[4 + strspn(s + 4, " \t")] != '*')
-                xxx->feat |=  fFtpFeature_NOOP;
+                xxx->feat |=                  fFtpFeature_NOOP;
             else
-                xxx->feat &= ~fFtpFeature_NOOP;
+                xxx->feat &= (TFTP_Features)(~fFtpFeature_NOOP);
         }
         if ((s = x_4Word(line, "SYST")) != 0) {  /* RFC 959 */
             if (s[4 + strspn(s + 4, " \t")] != '*')
-                xxx->feat |=  fFtpFeature_SYST;
+                xxx->feat |=                  fFtpFeature_SYST;
             else
-                xxx->feat &= ~fFtpFeature_SYST;
+                xxx->feat &= (TFTP_Features)(~fFtpFeature_SYST);
         }
         if ((s = x_4Word(line, "SITE")) != 0) {  /* RFC 959, 1123 */
             if (s[4 + strspn(s + 4, " \t")] != '*')
-                xxx->feat |=  fFtpFeature_SITE;
+                xxx->feat |=                  fFtpFeature_SITE;
             else
-                xxx->feat &= ~fFtpFeature_SITE;
+                xxx->feat &= (TFTP_Features)(~fFtpFeature_SITE);
         }
         if ((s = x_4Word(line, "FEAT")) != 0) {  /* RFC 3659 */
             if (s[4 + strspn(s + 4, " \t")] != '*')
-                xxx->feat |=  fFtpFeature_FEAT;
+                xxx->feat |=                  fFtpFeature_FEAT;
             else
-                xxx->feat &= ~fFtpFeature_FEAT;
+                xxx->feat &= (TFTP_Features)(~fFtpFeature_FEAT);
         }
         if ((s = x_4Word(line, "MDTM")) != 0) {  /* RFC 3659 */
             if (s[4 + strspn(s + 4, " \t")] != '*')
-                xxx->feat |=  fFtpFeature_MDTM;
+                xxx->feat |=                  fFtpFeature_MDTM;
             else
-                xxx->feat &= ~fFtpFeature_MDTM;
+                xxx->feat &= (TFTP_Features)(~fFtpFeature_MDTM);
         }
         if ((s = x_4Word(line, "SIZE")) != 0) {  /* RFC 3659 */
             if (s[4 + strspn(s + 4, " \t")] != '*')
-                xxx->feat |=  fFtpFeature_SIZE;
+                xxx->feat |=                  fFtpFeature_SIZE;
             else
-                xxx->feat &= ~fFtpFeature_SIZE;
+                xxx->feat &= (TFTP_Features)(~fFtpFeature_SIZE);
         }
         if ((s = x_4Word(line, "REST")) != 0) {  /* RFC 3659, NB: FEAT */
             if (s[4 + strspn(s + 4, " \t")] != '*')
-                xxx->feat |=  fFtpFeature_REST;
+                xxx->feat |=                  fFtpFeature_REST;
             else
-                xxx->feat &= ~fFtpFeature_REST;
+                xxx->feat &= (TFTP_Features)(~fFtpFeature_REST);
         }
         if ((s = x_4Word(line, "MLST")) != 0) {  /* RFC 3659 */
             if (s[4 + strspn(s + 4, " \t")] != '*')
-                xxx->feat |=  fFtpFeature_MLSx;
+                xxx->feat |=                  fFtpFeature_MLSx;
             else
-                xxx->feat &= ~fFtpFeature_MLSx;
+                xxx->feat &= (TFTP_Features)(~fFtpFeature_MLSx);
         }
         if ((s = x_4Word(line, "MLSD")) != 0) {  /* RFC 3659 */
             if (s[4 + strspn(s + 4, " \t")] != '*')
-                xxx->feat |=  fFtpFeature_MLSx;
+                xxx->feat |=                  fFtpFeature_MLSx;
             else
-                xxx->feat &= ~fFtpFeature_MLSx;
+                xxx->feat &= (TFTP_Features)(~fFtpFeature_MLSx);
         }
         if ((s = x_4Word(line, "EPRT")) != 0) {  /* RFC 2428 */
             if (s[4 + strspn(s + 4, " \t")] != '*')
-                xxx->feat |=  fFtpFeature_EPRT;
+                xxx->feat |=                  fFtpFeature_EPRT;
             else
-                xxx->feat &= ~fFtpFeature_EPRT;
+                xxx->feat &= (TFTP_Features)(~fFtpFeature_EPRT);
         }
         if ((s = x_4Word(line, "EPSV")) != 0) {  /* RFC 2428 (cf 1579) */
             if (s[4 + strspn(s + 4, " \t")] != '*')
-                xxx->feat |=  fFtpFeature_EPSV;
+                xxx->feat |=                  fFtpFeature_EPSV;
             else
-                xxx->feat &= ~fFtpFeature_EPSV;
+                xxx->feat &= (TFTP_Features)(~fFtpFeature_EPSV);
         }
     } /* else last line */
     return eIO_Success;
@@ -822,7 +825,7 @@ static EIO_Status x_FTPAbort(SFTPConnector*  xxx,
                 status = eIO_Unknown;
         } else if (status == eIO_Timeout  &&  !code)
             sync = 0/*false*/;
-        xxx->sync = sync;
+        xxx->sync = sync ? 1 : 0;
     }
     return status;
 }
@@ -894,7 +897,7 @@ static EIO_Status x_FTPPasv(SFTPConnector*  xxx,
         for (i = 0;  i < (unsigned int)(sizeof(o) / sizeof(o[0]));  i++) {
             if (sscanf(c + len, &",%d%n"[!i], &o[i], &code) < 1)
                 break;
-            len += code;
+            len += (size_t) code;
         }
         if (i >= (unsigned int)(sizeof(o) / sizeof(o[0])))
             break;
@@ -908,10 +911,10 @@ static EIO_Status x_FTPPasv(SFTPConnector*  xxx,
         if (o[i] < 0  ||  o[i] > 255)
             return eIO_Unknown;
     }
-    if (!(i = (((((o[0] << 8) | o[1]) << 8) | o[2]) << 8) | o[3]))
+    if (!(i = (unsigned int)((((((o[0]<<8) | o[1])<<8) | o[2])<<8) | o[3])))
         return eIO_Unknown;
     *host = SOCK_HostToNetLong(i);
-    if (!(i = (o[4] << 8) | o[5]))
+    if (!(i = (unsigned int)((o[4]<<8) | o[5])))
         return eIO_Unknown;
     *port = (unsigned short) i;
     return eIO_Success;
@@ -929,15 +932,16 @@ static EIO_Status x_FTPPassive(SFTPConnector*  xxx,
     if ((xxx->feat & fFtpFeature_APSV) == fFtpFeature_APSV) {
         /* first time here, try to set EPSV ALL */
         if (x_FTPEpsv(xxx, 0, 0) == eIO_Success)
-            xxx->feat &= ~fFtpFeature_EPSV;                     /* APSV mode */
-        else
-            xxx->feat &= ~fFtpFeature_APSV | fFtpFeature_EPSV;  /* EPSV mode */
+            xxx->feat &= (TFTP_Features)(~fFtpFeature_EPSV); /* APSV mode */
+
+        else                                                 /* EPSV mode */
+            xxx->feat &= (TFTP_Features)(~fFtpFeature_APSV | fFtpFeature_EPSV);
     }
     if (xxx->feat & fFtpFeature_APSV) {
         status = x_FTPEpsv(xxx, &host, &port);
         switch (status) {
         case eIO_NotSupported:
-            xxx->feat &= ~fFtpFeature_EPSV;
+            xxx->feat &= (TFTP_Features)(~fFtpFeature_EPSV);
             port = 0;
             break;
         case eIO_Success:
@@ -1051,7 +1055,7 @@ static EIO_Status x_FTPActive(SFTPConnector*  xxx,
         status = x_FTPEprt(xxx, host, port);
         if (status != eIO_NotSupported)
             return status;
-        xxx->feat &= ~fFtpFeature_EPRT;
+        xxx->feat &= (TFTP_Features)(~fFtpFeature_EPRT);
     }
     return x_FTPPort(xxx, host, port);
 }
@@ -1075,6 +1079,8 @@ static EIO_Status x_FTPOpenData(SFTPConnector*  xxx,
              * still, better safe than sorry */
             return status;
         }
+        if (!xxx->cntl)
+            return status;
     }
     status = x_FTPActive(xxx, lsock, timeout);
     if (status != eIO_Success) {
@@ -1140,7 +1146,7 @@ static EIO_Status x_FTPXfer(SFTPConnector*  xxx,
                 assert(xxx->data);
                 if (xxx->send) {
                     if (!(xxx->flag & fFTP_UncorkUpload))
-                        SOCK_SetCork(xxx->data, 1);
+                        SOCK_SetCork(xxx->data, 1/*true*/);
                     assert(xxx->open);
                     xxx->size = 0;
                 }
@@ -1357,8 +1363,8 @@ static EIO_Status x_FTPParseMdtm(SFTPConnector* xxx, char* timestamp)
      *    input format specifiers in the format string (%-based);
      * 2. None to any spaces are allowed to match a space in the format, whilst
      *    an MDTM response must not contain even a single space. */
-    for (n = 0;  n < 6;  n++) {
-        size_t len = n ? 2 : 4/*year*/;
+    for (n = 0;  n < 6;  ++n) {
+        len = n ? 2 : 4/*year*/;
         if (len != strlen(strncpy0(buf, timestamp, len))  ||
             len != strspn(buf, kDigits)) {
             return eIO_Unknown;
@@ -1663,11 +1669,12 @@ static EIO_Status s_FTPPollCntl(SFTPConnector* xxx, const STimeout* timeout)
                 xxx->sync = 1/*true*/;
                 status = eIO_Closed;
             }
-        }
+        } else if (!xxx->cntl)
+            status  = eIO_Closed;
         if (status == eIO_Closed)
             break;
     }
-    xxx->abor = abor;
+    xxx->abor = abor ? 1 : 0;
     return status;
 }
 
@@ -2285,7 +2292,7 @@ extern CONNECTOR s_CreateConnector(const SConnNetInfo*  info,
         strcpy(xxx->info->user, user  &&  *user ? user : "ftp");
         strcpy(xxx->info->pass, pass            ? pass : "-none@");
         strcpy(xxx->info->path, path            ? path : "");
-        flag &= ~fFTP_IgnorePath;
+        flag &= (TFTP_Flags)(~fFTP_IgnorePath);
     } else if (!(flag & fFTP_LogAll)) {
         switch (info->debug_printout) {
         case eDebugPrintout_Some:
