@@ -1,4 +1,4 @@
-/* $Id: ncbi_gnutls.c 545593 2017-09-07 18:06:33Z ivanov $
+/* $Id: ncbi_gnutls.c 568537 2018-08-07 14:35:10Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -86,7 +86,7 @@ static int gcry_user_mutex_lock(void** lock)
 }
 static int gcry_user_mutex_unlock(void** lock)
 {
-    return MT_LOCK_Do((MT_LOCK)(*lock), eMT_Unlock) ? 0 : NCBI_NOTSUPPORTED;
+    return MT_LOCK_Do((MT_LOCK)(*lock), eMT_Unlock)!=0 ? 0 : NCBI_NOTSUPPORTED;
 }
 static struct gcry_thread_cbs gcry_threads_user = {
     GCRY_THREAD_OPTION_USER, NULL/*gcry_user_init*/,
@@ -121,7 +121,7 @@ static int gtls_user_mutex_lock(void** lock)
 }
 static int gtls_user_mutex_unlock(void** lock)
 {
-    return MT_LOCK_Do((MT_LOCK)(*lock), eMT_Unlock) ? 0 : NCBI_NOTSUPPORTED;
+    return MT_LOCK_Do((MT_LOCK)(*lock), eMT_Unlock)!=0 ? 0 : NCBI_NOTSUPPORTED;
 }
 #    ifdef __cplusplus
 }
@@ -173,8 +173,8 @@ static const int kGnuTlsCompPrio[] = {
 static int                              s_GnuTlsLogLevel;
 static gnutls_anon_client_credentials_t s_GnuTlsCredAnon;
 static gnutls_certificate_credentials_t s_GnuTlsCredCert;
-static FSSLPull                         s_Pull;
-static FSSLPush                         s_Push;
+static volatile FSSLPull                s_Pull;
+static volatile FSSLPush                s_Push;
 
 
 static void x_GnuTlsLogger(int level, const char* message)
@@ -236,20 +236,20 @@ static EIO_Status x_ErrorToStatus(int* error, gnutls_session_t session,
     else if (*error == GNUTLS_E_PULL_ERROR
              &&  sock->r_status != eIO_Success
              &&  sock->r_status != eIO_Unknown) {
-        status = sock->r_status;
+        status = (EIO_Status) sock->r_status;
     }
     else if (*error == GNUTLS_E_PUSH_ERROR
              &&  sock->w_status != eIO_Success
              &&  sock->w_status != eIO_Unknown) {
-        status = sock->w_status;
+        status = (EIO_Status) sock->w_status;
     }
     else if (gnutls_error_is_fatal(*error))
         status = eIO_Closed;
     else
         status = eIO_Unknown;
 
-    CORE_LOGF(eLOG_Trace, ("GNUTLS error %d -> CONNECT GNUTLS status %s",
-                           *error, IO_StatusStr(status)));
+    CORE_TRACEF(("GNUTLS error %d -> CONNECT GNUTLS status %s",
+                 *error, IO_StatusStr(status)));
 
     return status;
 }
@@ -269,9 +269,6 @@ static int x_StatusToError(EIO_Status status, SOCK sock, EIO_Event direction)
     case eIO_Timeout:
         error = EAGAIN;
         break;
-    case eIO_Closed:
-        error = SOCK_ENOTCONN;
-        break;
     case eIO_Interrupt:
         error = SOCK_EINTR;
         break;
@@ -281,6 +278,9 @@ static int x_StatusToError(EIO_Status status, SOCK sock, EIO_Event direction)
     case eIO_Unknown:
         error = 0/*keep*/;
         break;
+    case eIO_Closed:
+        error = SOCK_ENOTCONN;
+        break;
     default:
         /*NB:eIO_InvalidArg*/
         error = EINVAL;
@@ -288,12 +288,11 @@ static int x_StatusToError(EIO_Status status, SOCK sock, EIO_Event direction)
     }
 
     {{
-        const char* x_what = error ? "error" : "errno";
-        int        x_error = error ?  error  :  errno;
-        CORE_LOGF(eLOG_Trace, ("CONNECT GNUTLS status %s -> %s %d",
-                               IO_StatusStr(status), x_what, x_error));
+        int x_err = error ? error : errno;
+        CORE_TRACEF(("CONNECT GNUTLS status %s -> %s %d",
+                     IO_StatusStr(status), error ? "error" : "errno", x_err));
         if (!error)
-            errno = x_error; /* restore errno that may be clobbered by log */
+            errno = x_err; /* restore errno that may be clobbered by log */
     }}
 
     return error;
@@ -396,7 +395,7 @@ static EIO_Status s_GnuTlsOpen(void* session, int* error, char** desc)
         *error = x_error;
     } else {
 #  if LIBGNUTLS_VERSION_NUMBER >= 0x030110
-        char* temp = gnutls_session_get_desc(session);
+        char* temp = gnutls_session_get_desc((gnutls_session_t) session);
         if (temp) {
             *desc = strdup(temp);
             gnutls_free(temp);
@@ -481,8 +480,8 @@ static ssize_t x_GnuTlsPush(gnutls_transport_ptr_t ptr,
             } else {
                 assert(status == eIO_Success);
                 assert(x_written <= size);
-                n_written += x_written;
-                size      -= x_written;
+                n_written += (ssize_t)            x_written;
+                size      -=                      x_written;
                 data       = (const char*) data + x_written;
             }
         } while (size);
@@ -564,7 +563,7 @@ static EIO_Status s_GnuTlsRead(void* session, void* buf, size_t n_todo,
     } else
         status = eIO_Success;
 
-    *n_done = x_read;
+    *n_done = (size_t) x_read;
     return status;
 }
 
@@ -589,7 +588,7 @@ static EIO_Status x_GnuTlsWrite(void* session, const void* data, size_t n_todo,
     } else
         status = eIO_Success;
 
-    *n_done = x_written;
+    *n_done = (size_t) x_written;
     return status;
 }
 
@@ -672,7 +671,9 @@ static EIO_Status s_GnuTlsInit(FSSLPull pull, FSSLPush push)
     if (!pull  ||  !push)
         return eIO_InvalidArg;
 
-    val = ConnNetInfo_GetValue(0, "TLS_LOGLEVEL", buf, sizeof(buf), 0);
+    val = ConnNetInfo_GetValue(0, "GNUTLS_LOGLEVEL", buf, sizeof(buf), 0);
+    if (!val  ||  !*val)
+        val = ConnNetInfo_GetValue(0, "TLS_LOGLEVEL", buf, sizeof(buf), 0);
     CORE_LOCK_READ;
     if (!val  ||  !*val)
         val = getenv("GNUTLS_DEBUG_LEVEL"); /* GNUTLS proprietary setting */
@@ -752,12 +753,14 @@ static void s_GnuTlsExit(void)
 }
 
  
-static const char* s_GnuTlsError(void* session/*unused*/, int error,
+static const char* s_GnuTlsError(void* session, int error,
                                  char* buf/*unused*/, size_t size/*unused*/)
 {
     /* GNUTLS defines only negative error codes */
-    return error >= 0 ? 0 : error < GNUTLS_E_APPLICATION_ERROR_MAX
-        ? gnutls_alert_get_name(GNUTLS_E_APPLICATION_ERROR_MAX - error)
+    return error >= 0 ? 0 :
+        error == GNUTLS_E_WARNING_ALERT_RECEIVED  ||
+        error == GNUTLS_E_FATAL_ALERT_RECEIVED
+        ? gnutls_alert_get_strname(gnutls_alert_get((gnutls_session_t)session))
         : gnutls_strerror(error);
 }
 
