@@ -1,4 +1,4 @@
-/* $Id: ncbi_http_session.cpp 567636 2018-07-21 15:53:54Z mcelhany $
+/* $Id: ncbi_http_session.cpp 576759 2018-12-20 14:37:49Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -831,7 +831,9 @@ void CHttpRequest::x_InitConnection(bool use_form_data)
 
 bool CHttpRequest::x_CanSendData(void) const
 {
-    return m_Method == eReqMethod_Post  ||  m_Method == eReqMethod_Put;
+    return m_Method == eReqMethod_Post  ||
+		m_Method == eReqMethod_Put  ||
+		m_Method == eReqMethod_Patch;
 }
 
 
