@@ -121,14 +121,8 @@ static void           s_LBOS_DestroyData   (SLBOS_Data*   data);
 static SSERV_Info*    s_LBOS_GetNextInfo   (SERV_ITER     iter,
                                             HOST_INFO*    host_info);
 static void           s_LBOS_Initialize    (void);
-static int/*bool*/    s_LBOS_Feedback      (SERV_ITER     a,
-                                            double        b,
-                                            int           c);
 static void           s_LBOS_Close         (SERV_ITER     iter);
 static void           s_LBOS_Reset         (SERV_ITER     iter);
-static int/*bool*/    s_LBOS_Update        (SERV_ITER     iter,
-                                            const char*   text,
-                                            int           code);
 #if defined NCBI_OS_LINUX || defined NCBI_OS_MSWIN
 static const char*    s_LBOS_ReadLbosresolver(void);
 #endif /* defined NCBI_OS_LINUX || defined NCBI_OS_MSWIN */
@@ -342,8 +336,8 @@ clear_and_exit:
 static const SSERV_VTable s_lbos_op =  {
         s_LBOS_GetNextInfo, /**< Use open()'ed iterator to get next found
                              *   server*/
-        s_LBOS_Feedback,    /**< Not used                                    */
-        s_LBOS_Update,      /**< Not used                                    */
+        0,                  /**< Feedback, not used                          */
+        0,                  /**< Update, Not used                            */
         s_LBOS_Reset,       /**< Free iterator's 'data'                      */
         s_LBOS_Close,       /**< Close iterator and free its 'data'          */
         "lbos"              /**< name of mapper                              */
@@ -984,7 +978,7 @@ static SSERV_Info** s_LBOS_ResolveIPPort(const char* lbos_address,
     size_t length;
     size_t user_dtab_length;
     char* new_dtab = NULL;
-    char* user_dtab_end;
+    const char* user_dtab_end;
     /* Allocate space for answer (will be expanded later, if needed) */
     infos = (SSERV_Info**)calloc(2, sizeof(SSERV_Info*));
     if (infos == NULL) {
@@ -1073,6 +1067,7 @@ static SSERV_Info** s_LBOS_ResolveIPPort(const char* lbos_address,
     x_JSON_Array  *serviceEndpoints;
     x_JSON_Object *serviceEndpoint;
     unsigned int j = 0;
+    time_t now = 0;
     root_value = x_json_parse_string(lbos_answer);
     if (x_json_value_get_type(root_value) != JSONObject) {
         goto clean_and_exit;
@@ -1085,7 +1080,6 @@ static SSERV_Info** s_LBOS_ResolveIPPort(const char* lbos_address,
     serviceEndpoints = 
         x_json_object_get_array(services, x_json_object_get_name(services, 0));
     /* Iterate through endpoints */
-    time_t now = 0;
     for (j = 0;  j < x_json_array_get_count(serviceEndpoints);  j++) {
         const char *host, *rate, *extra, *type;
         char* server_description;
@@ -1130,7 +1124,7 @@ static SSERV_Info** s_LBOS_ResolveIPPort(const char* lbos_address,
         length = strlen(descr_format) + strlen(type) + strlen(host) + 
                  5 /*length of port*/ + strlen(extra) + strlen(rate) +
                  40/*time*/;
-        server_description = malloc(sizeof(char) * length);
+        server_description = (char*) malloc(sizeof(char) * length);
         if (!now)
             now = time(0);
         sprintf(server_description, descr_format, type, host, 
@@ -1682,7 +1676,7 @@ static EHTTP_HeaderParse s_LBOS_ParseHeader(const char*      header,
     SLBOS_UserData* response_output;
     int             status_code = 0/*success code if any*/;
     /* For all we know, status message ends before \r\n */
-    char*           header_end;
+    const char*     header_end;
     char*           status_message;
     unsigned int    content_length;
     char*           content_length_pos;
@@ -1792,20 +1786,6 @@ static void s_LBOS_Reset(SERV_ITER iter)
 }
 
 
-/** Not implemented in LBOS client. */
-static int/*bool*/ s_LBOS_Feedback (SERV_ITER a, double b, int c)
-{
-    return 0;
-}
-
-
-/** Not implemented in LBOS client. */
-static int/*bool*/ s_LBOS_Update(SERV_ITER iter, const char* text, int code)
-{
-    return 1;
-}
-
-
 static void s_LBOS_Close (SERV_ITER iter)
 {
     SLBOS_Data* data;
@@ -1884,6 +1864,9 @@ const SSERV_VTable* SERV_LBOS_Open( SERV_ITER            iter,
     SLBOS_Data* data;
     char* new_name = NULL; /* if we need to add dbaf */
     const char* orig_serv_name = iter->name; /* we may modify name with dbaf */
+
+    CORE_LOG(eLOG_Error, "LBOS is deprecated, consider using LBSMD instead.");
+
     if (!s_TurnOn()) {
         return NULL;
     }
