@@ -1,4 +1,4 @@
-/* $Id: ncbi_local.c 481584 2015-10-14 04:40:34Z lavr $
+/* $Id: ncbi_local.c 565391 2018-06-11 18:09:29Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -80,7 +80,7 @@ static int/*bool*/ s_AddService(const SSERV_Info* info,
         data->cand   = temp;
     }
 
-    n = rand() % ++data->n_cand;
+    n = (size_t) rand() % ++data->n_cand;
     if (n < data->n_cand - 1) {
         temp = data->cand + n++;
         memmove(temp + 1, temp, (data->n_cand - n) * sizeof(*data->cand));
@@ -92,8 +92,9 @@ static int/*bool*/ s_AddService(const SSERV_Info* info,
 
 static int/*bool*/ s_LoadSingleService(const char* name, SERV_ITER iter)
 {
+    TSERV_Type types
+        = iter->types & (TSERV_Type)(~(fSERV_Stateless | fSERV_Firewall));
     struct SLOCAL_Data* data = (struct SLOCAL_Data*) iter->data;
-    const TSERV_Type types = iter->types & ~fSERV_Firewall;
     char key[sizeof(REG_CONN_LOCAL_SERVER) + 10];
     int/*bool*/ ok = 0/*failed*/;
     SSERV_Info* info;
@@ -101,7 +102,7 @@ static int/*bool*/ s_LoadSingleService(const char* name, SERV_ITER iter)
 
     info = 0;
     strcpy(key, REG_CONN_LOCAL_SERVER "_");
-    for (n = 0;  n <= 100;  n++) {
+    for (n = 0;  n <= 100;  ++n) {
         const char* svc;
         char buf[1024];
 
@@ -122,15 +123,17 @@ static int/*bool*/ s_LoadSingleService(const char* name, SERV_ITER iter)
             unsigned int localhost = SOCK_GetLocalHostAddress(eDefault);
             if (!info->host)
                 info->host = localhost;
-            if ((info->site & fSERV_Private)  &&  info->host != localhost)
+            if (!iter->ok_private  &&  (info->site & fSERV_Private)
+                &&  info->host != localhost) {
                 continue;  /* private server */
+            }
         }
         if (!iter->reverse_dns  &&  info->type != fSERV_Dns) {
             if (types != fSERV_Any  &&  !(types & info->type))
                 continue;  /* type doesn't match */
             if (types == fSERV_Any  &&  info->type == fSERV_Dns)
                 continue;  /* DNS entries have to be req'd explicitly */
-            if (iter->stateless  &&  (info->mode & fSERV_Stateful))
+            if ((iter->types & fSERV_Stateless)&&(info->mode & fSERV_Stateful))
                 continue;  /* skip stateful only servers */
         }
         if (!info->rate)
@@ -218,8 +221,8 @@ static SLB_Candidate* s_GetCandidate(void* user_data, size_t i)
 
 static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
 {
+    const TSERV_Type types = iter->types & (TSERV_Type)(~fSERV_Firewall);
     struct SLOCAL_Data* data = (struct SLOCAL_Data*) iter->data;
-    const TSERV_Type types = iter->types & ~fSERV_Firewall;
     int/*bool*/ dns_info_seen = 0/*false*/;
     SSERV_Info* info;
     size_t i, n;
@@ -236,7 +239,7 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
     i = 0;
     data->i_cand = 0;
     while (i < data->n_cand) {
-        /* NB all servers have been loaded in accordance with iter->external */
+        /* NB all servers have been loaded in accordance with their locality */
         info = (SSERV_Info*) data->cand[i].info;
         if (info->rate > 0.0  ||  iter->ok_down) {
             const char* c = SERV_NameOfInfo(info);
