@@ -1,4 +1,4 @@
-/* $Id: ncbi_pipe.cpp 488579 2016-01-01 00:38:12Z lavr $
+/* $Id: ncbi_pipe.cpp 570101 2018-09-04 15:48:46Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -474,7 +474,8 @@ void CPipeHandle::OpenSelf(void)
 
     NcbiCout.flush();
     ::fflush(stdout);
-    if ( !::FlushFileBuffers(m_ChildStdIn) ) {
+    if ( m_ChildStdIn != INVALID_HANDLE_VALUE
+         &&  !::FlushFileBuffers(m_ChildStdIn) ) {
         PIPE_THROW(::GetLastError(), "Failed FlushFileBuffers(stdout)");
     }
     if ((m_ChildStdIn = ::GetStdHandle(STD_OUTPUT_HANDLE))
@@ -485,7 +486,8 @@ void CPipeHandle::OpenSelf(void)
         == INVALID_HANDLE_VALUE) {
         PIPE_THROW(::GetLastError(), "Failed GetStdHandle(stdin)");
     }
-    m_ProcHandle = ::GetCurrentProcess();
+    // NB: GetCurrentProcess() returns HANDLE(-1) which is INVALID_HANDLE_VALUE
+    m_ProcHandle = GetCurrentProcess();
 
     m_SelfHandles = true;
 }
@@ -578,7 +580,7 @@ EIO_Status CPipeHandle::Read(void* buf, size_t count, size_t* n_read,
     EIO_Status status = eIO_Unknown;
 
     try {
-        if (m_ProcHandle == INVALID_HANDLE_VALUE) {
+        if (m_ProcHandle == INVALID_HANDLE_VALUE  &&  !m_SelfHandles) {
             status = eIO_Closed;
             throw string("Pipe closed");
         }
@@ -661,7 +663,7 @@ EIO_Status CPipeHandle::Write(const void* buf, size_t count,
     EIO_Status status = eIO_Unknown;
 
     try {
-        if (m_ProcHandle == INVALID_HANDLE_VALUE) {
+        if (m_ProcHandle == INVALID_HANDLE_VALUE  &&  !m_SelfHandles) {
             status = eIO_Closed;
             throw string("Pipe closed");
         }
@@ -728,7 +730,7 @@ CPipe::TChildPollMask CPipeHandle::Poll(CPipe::TChildPollMask mask,
     CPipe::TChildPollMask poll = 0;
 
     try {
-        if (m_ProcHandle == INVALID_HANDLE_VALUE) {
+        if (m_ProcHandle == INVALID_HANDLE_VALUE  &&  !m_SelfHandles) {
             throw string("Pipe closed");
         }
         if (m_ChildStdIn  == INVALID_HANDLE_VALUE  &&
@@ -2056,7 +2058,11 @@ CPipe::EFinish CPipe::ExecWait(const string&           cmd,
     bool err_done = false;
     bool in_done  = false;
     
+#ifndef NCBI_OS_LINUX
     const size_t buf_size = 16 * 1024;
+#else
+    const size_t buf_size = 512 * 1024;
+#endif
     size_t total_bytes_written = 0;
     size_t bytes_in_inbuf = 0;
     char   inbuf[buf_size];
