@@ -1,4 +1,4 @@
-/* $Id: ncbi_sendmail.c 534370 2017-04-26 19:29:20Z lavr $
+/* $Id: ncbi_sendmail.c 569143 2018-08-16 16:46:54Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -285,7 +285,7 @@ static void x_Sendmail_InitEnv(void)
         tmo = 120.0/*2 min*/;
     }
     if (!ConnNetInfo_GetValue(0, "MX_PORT", buf, sizeof(buf), 0)
-        ||  !(port = atoi(buf))  ||  port > 65535) {
+        ||  !(port = (unsigned int) atoi(buf))  ||  port > 65535) {
         port = CONN_PORT_SMTP;
     }
     if (!ConnNetInfo_GetValue(0, "MX_HOST", buf, sizeof(buf), 0)
@@ -301,7 +301,7 @@ static void x_Sendmail_InitEnv(void)
     s_MxTimeout.sec  = (unsigned int)  tmo;
     s_MxTimeout.usec = (unsigned int)((tmo - s_MxTimeout.sec) * 1000000.0);
     strcpy(s_MxHost, buf);
-    s_MxPort = port;
+    s_MxPort = (unsigned short) port;
     CORE_UNLOCK;
 }
 
@@ -335,11 +335,10 @@ extern const char* CORE_SendMail(const char* to,
 }
 
 
-/* In two macros below the smartest (or, weak-minded?) Sun
- * C compiler warns about unreachable end-of-loop condition
- * (well, it thinks "a condition" is there, dumb!).
+/* In two macros below the smartest (or, weak-minded?) Sun C compiler would
+ * have warned about unreachable end-of-loop condition (well, it thinks "a
+ * condition" is there, dumb!) -- so we had to add the "if (!sock)" thing.
  */
-
 #define SENDMAIL_RETURN(subcode, reason)                                \
     do {                                                                \
         if (sock) {                                                     \
@@ -367,15 +366,15 @@ extern const char* CORE_SendMail(const char* to,
                     ("[SendMail]  %s: %s", reason, explanation));       \
         if (!sock) {                                                    \
             if (info->mx_options & fSendMail_ExtendedErrInfo) {         \
-                size_t len = strlen(reason);                            \
-                char*  retval = (char*) malloc(len + 3                  \
-                                               + strlen(explanation));  \
+                size_t lenr = strlen(reason);                           \
+                size_t lene = strlen(explanation) + 1;                  \
+                char*  retval = (char*) malloc(lene + 2 + lenr);        \
                 if (!retval)                                            \
                     return "";                                          \
-                memcpy(retval, reason, len);                            \
-                retval[len++] = ':';                                    \
-                retval[len++] = ' ';                                    \
-                strcpy(retval + len, explanation);                      \
+                memcpy(retval, reason, lenr);                           \
+                retval[lenr++] = ':';                                   \
+                retval[lenr++] = ' ';                                   \
+                memcpy(retval + lenr, explanation, lene);               \
                 return retval;                                          \
             }                                                           \
             return reason;                                              \
@@ -484,7 +483,7 @@ extern const char* CORE_SendMailEx(const char*          to,
 
     info = uinfo ? uinfo : SendMailInfo_Init(&ainfo);
     if (info->magic != MX_SENDMAIL_MAGIC)
-        SENDMAIL_RETURN(6, "SSendMailInfo inited improperly: invalid magic");
+        SENDMAIL_RETURN(6, "SSendMailInfo inited improperly: Invalid magic");
 
     if ((!to         ||  !*to)        &&
         (!info->cc   ||  !*info->cc)  &&
@@ -552,7 +551,7 @@ extern const char* CORE_SendMailEx(const char*          to,
     if (!SENDMAIL_READ_RESPONSE(354, buffer))
         SENDMAIL_RETURN2(16, "Protocol error in DATA command", buffer);
 
-    (void) SOCK_SetCork(sock, eOn);
+    (void) SOCK_SetCork(sock, 1/*true*/);
 
     if (!(info->mx_options & fSendMail_NoMxHeader)) {
         if (!(info->mx_options & fSendMail_Old822Headers)) {
@@ -718,7 +717,7 @@ extern const char* CORE_SendMailEx(const char*          to,
     } else if (!s_SockWrite(sock, "." MX_CRLF, sizeof(MX_CRLF)))
         SENDMAIL_RETURN(28, "Write error while finalizing message");
 
-    (void) SOCK_SetCork(sock, eOff);
+    (void) SOCK_SetCork(sock, 0/*false*/);
 
     if (!SENDMAIL_READ_RESPONSE(250, buffer))
         SENDMAIL_RETURN2(29, "Protocol error in sending message", buffer);
