@@ -1,4 +1,4 @@
-/* $Id: ncbi_server_info.c 548391 2017-10-13 14:22:34Z ivanov $
+/* $Id: ncbi_server_info.c 568159 2018-07-31 18:36:43Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -55,7 +55,7 @@
  *   SSERV_Info::u's string parameters are stored contiguously following the
  *                 "fixed" area (SSERV_Ops::SizeOf() returns the size of the
  *                 type-dependent "u" plus all the string parameters);
- *   SSERV_Info::vhost (if non-zero) is stored past the parameters;
+ *   SSERV_Info::vhost (if non-zero) is stored past the parameters (+'\0');
  *   SSERV_Info::extra defines the size of "extra" opaque bytes (which may be
  *                 used to keep private information) that follow vhost (or the
  *                 SSERV_Info::u's block if vhost is empty), zero means none;
@@ -104,8 +104,8 @@ static struct {
     /* must be ordered longer-to-shorter */
     { kRegularInter, sizeof(kRegularInter)-1, eSERV_Regular, fSERV_Interzone },
     { kBlastInter,   sizeof(kBlastInter)-1,   eSERV_Blast,   fSERV_Interzone },
-    { kRegular,      sizeof(kRegular)-1,      eSERV_Regular, 0               },
-    { kBlast,        sizeof(kBlast)-1,        eSERV_Blast,   0               }
+    { kRegular,      sizeof(kRegular)-1,      eSERV_Regular, (ESERV_Site) 0  },
+    { kBlast,        sizeof(kBlast)-1,        eSERV_Blast,   (ESERV_Site) 0  }
 };
 
 
@@ -173,11 +173,14 @@ extern char* SERV_WriteInfo(const SSERV_Info* info)
         memcpy(s, attr->tag, attr->len);
         s += attr->len;
         *s++ = ' ';
-        if (info->host == SOCK_HostToNetLong(-1L)) {
-            int/*bool*/ ipv6 = !NcbiIsIPv4(&info->addr);
+        if (info->host == SOCK_HostToNetLong((unsigned int)(-1L))) {
+            int/*bool*/ ipv6 = !NcbiIsIPv4(&info->addr)  &&  info->port;
             if (ipv6)
                 *s++ = '[';
-            s = NcbiAddrToString(s, reserve, &info->addr);
+            if (!(s = NcbiAddrToString(s, MAX_IP_ADDR_LEN, &info->addr))) {
+                free(str);
+                return 0;
+            }
             if (ipv6)
                 *s++ = ']';
             if (info->port)
@@ -212,7 +215,8 @@ extern char* SERV_WriteInfo(const SSERV_Info* info)
             s += sprintf(s, " S=%s", k_NY[info->mode & fSERV_Stateful]);
         if (info->type != fSERV_Dns  &&  (info->mode & fSERV_Secure))
             s += sprintf(s, " $=yes");
-        sprintf(s, " T=%lu", (unsigned long) info->time);
+        if (info->time)
+            sprintf(s, " T=%lu", (unsigned long) info->time);
         if (info->site & fSERV_Interzone)
             s += sprintf(s, " X=yes");
     }
@@ -266,32 +270,38 @@ SSERV_Info* SERV_ReadInfoEx(const char* str,
         }
         if (end) {
             int/*bool*/ ipv6 = *str == '['  &&  len > 2 ? 1/*T*/ : 0/*F*/;
-            const char* tmp = NcbiIPToAddr(&addr, str + ipv6, len - ipv6);
-            if (tmp  &&  (!ipv6  ||  *tmp == ']')) {
-                str   = tmp + ipv6;
-                if (str != end  &&  *str != ':')
+            const char* tmp = NcbiIPToAddr(&addr, str + ipv6, len - !!ipv6);
+            if (tmp  &&  (!ipv6  ||  (*tmp++ == ']'  &&  *tmp == ':'))) {
+                if (tmp != end  &&  *tmp != ':')
                     return 0;
+                str   = tmp;
                 ipv6  = 1;
                 vhost = 0;
+            } else if (*str == '[') {
+                /* not a valid IP(v4/v6) address */
+                return 0;
             } else {
+                assert(!ipv6);
                 vhost = attr->type & SERV_VHOSTABLE ? str : 0;
-                ipv6  = 0;
             }
             if (str == end)
-                port = 0;
+                port = 0/*NB: "ipv6" case only*/;
             else if (!(str = SOCK_StringToHostPortEx(str, &host, &port, lazy)))
                 return 0;
             else if (str != end)
                 return 0;
+            else if (host == SOCK_HostToNetLong((unsigned int)(-1L)))
+                vhost = 0;
             if (!ipv6)
                 NcbiIPv4ToIPv6(&addr, host, 0);
             else if (NcbiIsIPv4(&addr))
                 host = NcbiIPv6ToIPv4(&addr, 0);
             else
-                host = SOCK_HostToNetLong(-1L);
+                host = SOCK_HostToNetLong((unsigned int)(-1L));
             while (*str  &&  isspace((unsigned char)(*str)))
                 ++str;
             if (vhost) {
+                /*NB: NcbiIPToAddr() can parse full-quad IPv4, so !vhost*/
                 int/*bool*/ dot = 0/*false*/;
                 for (len = 0;  len < (size_t)(end - vhost);  ++len) {
                     if (vhost[len] == ':')
@@ -301,7 +311,6 @@ SSERV_Info* SERV_ReadInfoEx(const char* str,
                 }
                 if (len > 1  &&  dot) {
                     tmp = strndup(vhost, len);
-                    /*NB: NcbiIPToAddr() can parse full-quad IPv4, so !vhost*/
                     assert(!tmp  ||  !SOCK_isipEx(tmp, 1/*full-quad*/));
                     if (tmp) {
                         if (SOCK_isip(tmp)/*NB: very unlikely*/)
@@ -346,7 +355,7 @@ SSERV_Info* SERV_ReadInfoEx(const char* str,
     while (*str  &&  isspace((unsigned char)(*str)))
         ++str;
     while (*str) {
-        if (str[1] == '='  &&  !isspace((unsigned char) str[2])) {
+        if (str[1] == '='  &&  str[2]  &&  !isspace((unsigned char) str[2])) {
             char*          e;
             int            n;
             double         d;
@@ -361,7 +370,7 @@ SSERV_Info* SERV_ReadInfoEx(const char* str,
                 if (algo)
                     break;
                 algo = 1/*true*/;
-                switch (*str) {
+                switch ((unsigned char)(*++str)) {
                 case 'B':
                     info->algo = eSERV_Blast;
                     ++str;
@@ -400,7 +409,7 @@ SSERV_Info* SERV_ReadInfoEx(const char* str,
                     info->mime_e = mime_e;
                     /* skip the entire token */
                     while (*str  &&  !isspace((unsigned char)(*str)))
-                        str++;
+                        ++str;
                 }
                 break;
             case 'H':
@@ -415,6 +424,8 @@ SSERV_Info* SERV_ReadInfoEx(const char* str,
                 }
                 assert(len);
                 vhost = str - len;
+                if (len == 2  &&  strncmp(vhost, "''", 2) == 0)
+                    vhost = 0;
                 break;
             case 'L':
                 if (locl)
@@ -422,10 +433,10 @@ SSERV_Info* SERV_ReadInfoEx(const char* str,
                 locl = 1/*true*/;
                 if (sscanf(++str, "%3s%n", s, &n) >= 1) {
                     if (strcasecmp(s, "YES") == 0) {
-                        info->site |=  fSERV_Local;
+                        info->site |=               fSERV_Local;
                         str += n;
                     } else if (strcasecmp(s, "NO") == 0) {
-                        info->site &= ~fSERV_Local;
+                        info->site &= (TSERV_Site)(~fSERV_Local);
                         str += n;
                     }
                 }
@@ -436,10 +447,10 @@ SSERV_Info* SERV_ReadInfoEx(const char* str,
                 priv = 1/*true*/;
                 if (sscanf(++str, "%3s%n", s, &n) >= 1) {
                     if (strcasecmp(s, "YES") == 0) {
-                        info->site |=  fSERV_Private;
+                        info->site |=               fSERV_Private;
                         str += n;
                     } else if (strcasecmp(s, "NO") == 0) {
-                        info->site &= ~fSERV_Private;
+                        info->site &= (TSERV_Site)(~fSERV_Private);
                         str += n;
                     }
                 }
@@ -466,10 +477,10 @@ SSERV_Info* SERV_ReadInfoEx(const char* str,
                 sful = 1/*true*/;
                 if (sscanf(++str, "%3s%n", s, &n) >= 1) {
                     if (strcasecmp(s, "YES") == 0) {
-                        info->mode |=  fSERV_Stateful;
+                        info->mode |=               fSERV_Stateful;
                         str += n;
                     } else if (strcasecmp(s, "NO") == 0) {
-                        info->mode &= ~fSERV_Stateful;
+                        info->mode &= (TSERV_Mode)(~fSERV_Stateful);
                         str += n;
                     }
                 }
@@ -480,10 +491,10 @@ SSERV_Info* SERV_ReadInfoEx(const char* str,
                 secu = 1/*true*/;
                 if (sscanf(++str, "%3s%n", s, &n) >= 1) {
                     if (strcasecmp(s, "YES") == 0) {
-                        info->mode |=  fSERV_Secure;
+                        info->mode |=               fSERV_Secure;
                         str += n;
                     } else if (strcasecmp(s, "NO") == 0) {
-                        info->mode &= ~fSERV_Secure;
+                        info->mode &= (TSERV_Mode)(~fSERV_Secure);
                         str += n;
                     }
                 }
@@ -503,10 +514,10 @@ SSERV_Info* SERV_ReadInfoEx(const char* str,
                 cros = 1/*true*/;
                 if (sscanf(++str, "%3s%n", s, &n) >= 1) {
                     if (strcasecmp(s, "YES") == 0) {
-                        info->site |=  fSERV_Interzone;
+                        info->site |=               fSERV_Interzone;
                         str += n;
                     } else if (strcasecmp(s, "NO") == 0) {
-                        info->mode &= ~fSERV_Interzone;
+                        info->site &= (TSERV_Site)(~fSERV_Interzone);
                         str += n;
                     }
                 }
@@ -521,7 +532,7 @@ SSERV_Info* SERV_ReadInfoEx(const char* str,
                         if (cros)
                             break;
                         cros = 1/*true*/;
-                        info->site |= kFlags[i].site;
+                        info->site |= (TSERV_Site) kFlags[i].site;
                     }
                     info->algo = kFlags[i].algo;
                     str += kFlags[i].len;
@@ -540,6 +551,7 @@ SSERV_Info* SERV_ReadInfoEx(const char* str,
         if (!vh  &&  !(secu  ||  (info->type & fSERV_Http)))
             vhost = 0;
         if (vhost) {
+            assert(len);
             if (len > 255)
                 len = 255;
             strncpy0((char*) &info->u + attr->ops.SizeOf(&info->u),
@@ -610,7 +622,7 @@ extern size_t SERV_SizeOfInfo(const SSERV_Info *info)
     return attr
         ? (sizeof(*info) - sizeof(info->u)
            + attr->ops.SizeOf(&info->u)
-           + info->vhost
+           + (info->vhost ? (size_t) info->vhost + 1 : 0)
            + info->extra)
         : 0;
 }
@@ -888,7 +900,7 @@ SSERV_Info* SERV_CreateHttpInfoEx(ESERV_Type     type,
 {
     SSERV_Info* info;
 
-    if (type & ~fSERV_Http)
+    if (type & (unsigned int)(~fSERV_Http))
         return 0;
     add += (path ? strlen(path) : 0) + 1 + (args ? strlen(args) : 0);
     if ((info = (SSERV_Info*) malloc(sizeof(SSERV_Info) + add + 1)) != 0) {
