@@ -1,4 +1,4 @@
-/* $Id: ncbi_service_connector.c 533405 2017-04-14 20:36:09Z lavr $
+/* $Id: ncbi_service_connector.c 565391 2018-06-11 18:09:29Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -97,10 +97,7 @@ extern "C" {
 
 static int/*bool*/ s_OpenDispatcher(SServiceConnector* uuu)
 {
-    TSERV_Type types = uuu->types;
-    if (uuu->net_info->stateless)
-        types |= fSERV_Stateless;
-    if (!(uuu->iter = SERV_Open(uuu->service, types,
+    if (!(uuu->iter = SERV_Open(uuu->service, uuu->types,
                                 SERV_LOCALHOST, uuu->net_info))) {
         CORE_LOGF_X(5, eLOG_Error,
                     ("[%s]  Service not found", uuu->service));
@@ -184,7 +181,10 @@ static EHTTP_HeaderParse s_ParseHeader(const char* header,
                                                  [header[m] == '$'])))) {
                     break/*failed - unreadable connection info*/;
                 }
-                o1 = i1; o2 = i2; o3 = i3; o4 = i4;
+                o1 = (unsigned char) i1;
+                o2 = (unsigned char) i2;
+                o3 = (unsigned char) i3;
+                o4 = (unsigned char) i4;
                 sprintf(ipaddr, "%u.%u.%u.%u", o1, o2, o3, o4);
                 if (strncmp(header, ipaddr, n) != 0
                     ||  !(uuu->host = SOCK_gethostbyname(ipaddr))
@@ -268,7 +268,7 @@ static int/*bool*/ s_IsContentTypeDefined(const char*         service,
                 const char* c;
                 size_t len;
                 char* t;
-                for (s += 15;  *s;  s++) {
+                for (s += 14;  *s;  s++) {
                     if (!isspace((unsigned char)(*s)))
                         break;
                 }
@@ -433,12 +433,13 @@ static CONNECTOR s_SocketConnectorBuilder(SConnNetInfo* net_info,
         *status = HTTP_CreateTunnel(net_info, fHTTP_NoAutoRetry, &sock);
         assert(!sock ^ !(*status != eIO_Success));
         if (*status == eIO_Success
-            &&  (size  ||  (flags & ~(fSOCK_LogOn | fSOCK_LogDefault)))) {
+            &&  (size  ||  (flags & (TSOCK_Flags)(~(fSOCK_LogOn |
+                                                    fSOCK_LogDefault))))) {
             /* push initial data through the proxy, as-is (i.e. clear-text) */
             TSOCK_Flags tempf = flags;
             if (size  &&  (flags & fSOCK_Secure)) {
                 tempf &=  fSOCK_LogOn | fSOCK_LogDefault;
-                tempf &= ~fSOCK_Secure;
+                tempf &=  (TSOCK_Flags)(~fSOCK_Secure);
             }
             memset(&init, 0, sizeof(init));
             init.data = data;
@@ -466,7 +467,7 @@ static CONNECTOR s_SocketConnectorBuilder(SConnNetInfo* net_info,
         TSOCK_Flags tempf = flags;
         if (size  &&  (flags & fSOCK_Secure)) {
             tempf &=  fSOCK_LogOn | fSOCK_LogDefault;
-            tempf &= ~fSOCK_Secure;
+            tempf &=  (TSOCK_Flags)(~fSOCK_Secure);
         }
         if (!proxy  &&  net_info->debug_printout) {
             net_info->scheme = eURL_Unspec;
@@ -534,10 +535,10 @@ static int/*bool*/ x_SetHostPort(SConnNetInfo* net_info,
         }
         free(tag);
     }
-    if (info->host == SOCK_HostToNetLong(-1L)) {
+    if (info->host == SOCK_HostToNetLong((unsigned int)(-1L))) {
         int/*bool*/ ipv6 = !NcbiIsIPv4(&info->addr);
-        char* end = NcbiAddrToString(net_info->host         +   ipv6,
-                                     sizeof(net_info->host) - 2*ipv6,
+        char* end = NcbiAddrToString(net_info->host + ipv6,
+                                     sizeof(net_info->host) - (size_t)(2*ipv6),
                                      &info->addr);
         if (!end) {
             *net_info->host = '\0';
@@ -939,11 +940,11 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
         ||  uuu->extra.adjust(net_info, uuu->extra.data, (unsigned int)(-1))
         ? HTTP_CreateConnectorEx(net_info,
                                  (uuu->extra.flags
-                                  & (fHTTP_Flushable   |
-                                     fHTTP_NoAutoRetry |
-                                     (uuu->extra.adjust
-                                      ? fHTTP_AdjustOnRedirect
-                                      : 0)))
+                                  & (THTTP_Flags)(fHTTP_Flushable   |
+                                                  fHTTP_NoAutoRetry |
+                                                  (uuu->extra.adjust
+                                                   ? fHTTP_AdjustOnRedirect
+                                                   : 0)))
                                  | fHTTP_AutoReconnect,
                                  s_ParseHeaderUCB, uuu/*user_data*/,
                                  s_Adjust, 0/*cleanup*/)
@@ -1034,23 +1035,6 @@ static EIO_Status s_VT_Open(CONNECTOR connector, const STimeout* timeout)
                  ||  strcasecmp(SERV_MapperName(uuu->iter), "local") == 0)) {
             break;
         }
-        if ((uuu->types & ~fSERV_Firewall)
-            &&  info  &&  ((info->type != fSERV_Firewall
-                            &&  !(uuu->types & info->type))  ||
-                           (info->type == fSERV_Firewall
-                            &&  !(uuu->types & info->u.firewall.type)))) {
-            const char* type = SERV_TypeStr(info->type == fSERV_Firewall
-                                            ? info->u.firewall.type
-                                            : info->type);
-            char buf[40];
-            sprintf(buf,
-                    *type ? " (0x%hX)" : "0x%hX", (unsigned short) info->type);
-            CORE_LOGF_X(10, eLOG_Critical,
-                        ("[%s]  Mismatched server type %s%s not within 0x%hX",
-                         uuu->service, type, buf, (unsigned short)uuu->types));
-            status = eIO_Unknown;
-            continue;
-        }
         if (uuu->type) {
             free((void*) uuu->type);
             uuu->type = 0;
@@ -1126,7 +1110,7 @@ static EIO_Status s_VT_Open(CONNECTOR connector, const STimeout* timeout)
             CORE_LOGF_X(6, eLOG_Error,
                         ("[%s]  %s connection failure (%s) usually"
                          " indicates possible firewall configuration"
-                         " problems; please consult <%s>", uuu->service,
+                         " problem(s); please consult <%s>", uuu->service,
                          uuu->net_info->firewall ? "Firewall":"Stateful relay",
                          IO_StatusStr(status), kFWDLink));
         }
@@ -1135,7 +1119,7 @@ static EIO_Status s_VT_Open(CONNECTOR connector, const STimeout* timeout)
     }
     if (status != eIO_Success  &&  !uuu->warned
         &&  uuu->retry > 1  &&  uuu->retry >= uuu->net_info->max_try) {
-        CORE_LOGF_X(11, eLOG_Error,
+        CORE_LOGF_X(10, eLOG_Error,
                     ("[%s]  Too many failed attempts (%hu), giving up",
                      uuu->service, uuu->retry));
     }
@@ -1224,7 +1208,7 @@ extern CONNECTOR SERVICE_CreateConnectorEx
     ccc->setup    = s_Setup;
     ccc->destroy  = s_Destroy;
 
-    xxx->types    = types;
+    xxx->types    = (TSERV_TypeOnly) types;
     x_net_info    = (net_info
                      ? ConnNetInfo_Clone(net_info)
                      : ConnNetInfo_Create(x_service));
@@ -1244,13 +1228,17 @@ extern CONNECTOR SERVICE_CreateConnectorEx
         x_net_info->stateless = 1/*true*/;
     if ((types & fSERV_Firewall)  &&  !x_net_info->firewall)
         x_net_info->firewall = eFWMode_Adaptive;
-    if (x_net_info->max_try < 1)
+    if (x_net_info->max_try < 1
+        ||  (extra  &&  (extra->flags & fHTTP_NoAutoRetry))) {
         x_net_info->max_try = 1;
-    if (!s_OpenDispatcher(xxx)) {
-        s_Destroy(ccc);
-        return 0;
     }
-    assert(xxx->iter);
+    if (!(types & fSERV_DelayOpen)) {
+        if (!s_OpenDispatcher(xxx)) {
+            s_Destroy(ccc);
+            return 0;
+        }
+        assert(xxx->iter);
+    }
 
     /* finally, store all callback extras */
     if (extra)
