@@ -1,4 +1,4 @@
-/* $Id: ncbi_util.c 545592 2017-09-07 18:06:13Z ivanov $
+/* $Id: ncbi_util.c 565407 2018-06-11 19:48:10Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -42,6 +42,9 @@
 #  include <stdlib.h>
 #  include <time.h>
 #endif
+#if !defined(NCBI_OS_DARWIN)  &&  defined(HAVE_POLL_H)
+#  include <poll.h>
+#endif /*!NCBI_OS_DARWIN && HAVE_POLL_H*/
 #if defined(NCBI_OS_UNIX)
 #  ifndef HAVE_GETPWUID
 #    error "HAVE_GETPWUID is undefined on a UNIX system!"
@@ -198,7 +201,7 @@ extern size_t UTIL_PrintableStringSize(const char* data, size_t size)
 
 
 extern char* UTIL_PrintableString(const char* data, size_t size,
-                                  char* buf, int/*bool*/ full_octal)
+                                  char* buf, int/*bool*/ flags)
 {
     const unsigned char* s;
     unsigned char* d;
@@ -209,7 +212,7 @@ extern char* UTIL_PrintableString(const char* data, size_t size,
         size = strlen(data);
 
     d = (unsigned char*) buf;
-    for (s = (const unsigned char*) data;  size;  size--, s++) {
+    for (s = (const unsigned char*) data;  size;  --size, ++s) {
         switch (*s) {
         case '\t':
             *d++ = '\\';
@@ -238,6 +241,8 @@ extern char* UTIL_PrintableString(const char* data, size_t size,
         case '\n':
             *d++ = '\\';
             *d++ = 'n';
+            if (flags & eUTIL_PrintableNoNewLine)
+                continue;
             /*FALLTHRU*/
         case '\\':
         case '\'':
@@ -248,23 +253,23 @@ extern char* UTIL_PrintableString(const char* data, size_t size,
             if (!isascii(*s)  ||  !isprint(*s)) {
                 int/*bool*/ reduce;
                 unsigned char v;
-                if (full_octal)
+                if (flags & eUTIL_PrintableFullOctal)
                     reduce = 0/*false*/;
                 else {
                     reduce = (size == 1  ||
-                              s[1] < '0' || s[1] > '7' ? 1/*t*/ : 0/*f*/);
+                              s[1] < '0' || s[1] > '7' ? 1/*T*/ : 0/*F*/);
                 }
                 *d++     = '\\';
                 v =  *s >> 6;
                 if (v  ||  !reduce) {
-                    *d++ = '0' + v;
+                    *d++ = (unsigned char)('0' + v);
                     reduce = 0;
                 }
                 v = (*s >> 3) & 7;
                 if (v  ||  !reduce)
-                    *d++ = '0' + v;
-                v =  *s       & 7;
-                *d++     = '0' + v;
+                    *d++ = (unsigned char)('0' + v);
+                v = *s & 7;
+                *d++ = (unsigned char)('0' + v);
                 continue;
             }
             break;
@@ -338,7 +343,7 @@ extern const char* NcbiMessagePlusError
     mlen += 7;
 
     if (error)
-        mlen += sprintf(buf + mlen, "%d%s", error, &","[!*descr]);
+        mlen += (size_t) sprintf(buf + mlen, "%d%s", error, &","[!*descr]);
 
     memcpy((char*) memcpy(buf + mlen, descr, dlen) + dlen, "}", 2);
     if (release)
@@ -353,10 +358,10 @@ extern char* LOG_ComposeMessage
 (const SLOG_Message* mess,
  TLOG_FormatFlags    flags)
 {
-    static const char kRawData_Begin[] =
+    static const char kRawData_Beg[] =
         "\n#################### [BEGIN] Raw Data (%lu byte%s):\n";
     static const char kRawData_End[] =
-        "\n#################### [END] Raw Data\n";
+        "\n#################### [_END_] Raw Data\n";
 
     const char *function, *level = 0;
     char *str, *s, datetime[32];
@@ -439,7 +444,7 @@ extern char* LOG_ComposeMessage
         message_len = strlen(mess->message);
 
     if (mess->raw_size) {
-        data_len = (sizeof(kRawData_Begin)
+        data_len = (sizeof(kRawData_Beg)
                     + 20 + UTIL_PrintableStringSize((const char*)
                                                     mess->raw_data,
                                                     mess->raw_size) +
@@ -491,14 +496,15 @@ extern char* LOG_ComposeMessage
         s += message_len;
     }
     if (data_len) {
-        s += sprintf(s, kRawData_Begin,
+        s += sprintf(s, kRawData_Beg,
                      (unsigned long) mess->raw_size,
                      &"s"[mess->raw_size == 1]);
 
         s = UTIL_PrintableString((const char*)
                                  mess->raw_data,
                                  mess->raw_size,
-                                 s, flags & fLOG_FullOctal);
+                                 s, flags & fLOG_FullOctal
+                                 ? eUTIL_PrintableFullOctal : 0);
 
         memcpy(s, kRawData_End, sizeof(kRawData_End));
     } else
@@ -945,23 +951,19 @@ extern void CORE_Msdelay(unsigned long ms)
 #if   defined(NCBI_OS_MSWIN)
     Sleep(ms);
 #elif defined(NCBI_OS_UNIX)
-#  ifdef HAVE_USLEEP
-    if (usleep(ms * 1000) < 0  &&  errno == EINVAL) {
-#  endif /*HAVE_USLEEP*/
-#  if defined(HAVE_NANOSLEEP)
+#  if    defined(HAVE_NANOSLEEP)
     struct timespec ts;
-    ts.tv_sec  =  ms / 1000;
-    ts.tv_nsec = (ms % 1000) * 1000000;
+    ts.tv_sec  = (time_t)(ms / 1000);
+    ts.tv_nsec = (long) ((ms % 1000) * 1000000);
     nanosleep(&ts, 0);
+#  elif !defined(NCBI_OS_DARWIN)  &&  defined(HAVE_POLL_H)
+    poll(0, 0, (int) ms);
 #  else
     struct timeval tv;
-    tv.tv_sec  =  ms / 1000;
-    tv.tv_usec = (ms % 1000) * 1000;
+    tv.tv_sec  = (long) (ms / 1000);
+    tv.tv_usec = (long)((ms % 1000) * 1000);
     select(0, 0, 0, 0, &tv);
 #  endif /*HAVE_NANOSLEEP*/
-#  ifdef HAVE_USLEEP
-    }
-#  endif /*HAVE_USLEEP*/
 #else
 #  error "Unsupported platform."
 #endif /*NCBI_OS*/
@@ -1241,8 +1243,8 @@ extern int/*bool*/ UTIL_MatchesMaskEx(const char* name, const char* mask,
         } else {
             d = *name++;
             if (ignore_case) {
-                c = tolower((unsigned char) c);
-                d = tolower((unsigned char) d);
+                c = (char) tolower((unsigned char) c);
+                d = (char) tolower((unsigned char) d);
             }
             if (c != d)
                 return 0/*false*/;
