@@ -1,24 +1,23 @@
 /*
- Parson ( http://kgabis.github.com/parson/ )
- Copyright (c) 2012 - 2016 Krzysztof Gabis
+Parson ( http://kgabis.github.com/parson/ )
+Copyright (c) 2012 - 2016 Krzysztof Gabis
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
 
- Permission is hereby granted, free of charge, to any person obtaining a copy
- of this software and associated documentation files (the "Software"), to deal
- in the Software without restriction, including without limitation the rights
- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- copies of the Software, and to permit persons to whom the Software is
- furnished to do so, subject to the following conditions:
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
 
- The above copyright notice and this permission notice shall be included in
- all copies or substantial portions of the Software.
-
- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- THE SOFTWARE.
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
 */
 #ifdef _MSC_VER
 #ifndef _CRT_SECURE_NO_WARNINGS
@@ -994,6 +993,24 @@ x_JSON_Value * x_json_object_get_value_at(const x_JSON_Object *object, size_t in
     return object->values[index];
 }
 
+int x_json_object_has_value (const x_JSON_Object *object, const char *name) {
+    return x_json_object_get_value(object, name) != NULL;
+}
+
+int x_json_object_has_value_of_type(const x_JSON_Object *object, const char *name, x_JSON_Value_Type type) {
+    x_JSON_Value *val = x_json_object_get_value(object, name);
+    return val != NULL && x_json_value_get_type(val) == type;
+}
+
+int x_json_object_dothas_value (const x_JSON_Object *object, const char *name) {
+    return x_json_object_dotget_value(object, name) != NULL;
+}
+
+int x_json_object_dothas_value_of_type(const x_JSON_Object *object, const char *name, x_JSON_Value_Type type) {
+    x_JSON_Value *val = x_json_object_dotget_value(object, name);
+    return val != NULL && x_json_value_get_type(val) == type;
+}
+
 /* JSON Array API */
 x_JSON_Value * x_json_array_get_value(const x_JSON_Array *array, size_t index) {
     if (array == NULL || index >= x_json_array_get_count(array))
@@ -1764,4 +1781,4 @@ int x_json_boolean(const x_JSON_Value *value) {
 void x_json_set_allocation_functions(x_JSON_Malloc_Function malloc_fun, x_JSON_Free_Function free_fun) {
     parson_malloc = malloc_fun;
     parson_free = free_fun;
-}
+}
\ No newline at end of file
