@@ -1,4 +1,4 @@
-/*  $Id: grid_control_thread.cpp 513338 2016-09-09 15:19:49Z sadyrovr $
+/*  $Id: grid_control_thread.cpp 571360 2018-09-25 16:51:11Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -63,9 +63,9 @@ public:
         CGridWorkerNode node(control_server->GetWorkerNode());
         const CGridWorkerNode::TVersion version(node.GetAppVersion());
 
-        os << "OK:version=" << NStr::URLEncode(version[0]) <<
-                "&build_date=" << NStr::URLEncode(version[1]) <<
-                "&build_tag=" << NStr::URLEncode(version[2]) << "\n";
+        os << "OK:version=" << NStr::URLEncode(version.first) <<
+                "&build_date=" << NStr::URLEncode(version.second.date) <<
+                "&build_tag=" << NStr::URLEncode(version.second.tag) << "\n";
     }
 };
 
@@ -169,9 +169,13 @@ public:
         const CGridWorkerNode::TVersion version(node.GetAppVersion());
 
         os << "OK:Application: " << node.GetAppName() <<
-                "\nVersion: " << version[0] <<
-                "\nBuild date: " << version[1] <<
-                "\nBuild tag: " << version[2] << "\n";
+                "\nVersion: " << version.first <<
+                "\nBuild date: " << version.second.date <<
+                "\nBuild tag: " << version.second.tag << "\n";
+
+        for (const auto& p : version.second.m_extra) {
+            os << SBuildInfo::ExtraName(p.first) << ": " << p.second << '\n';
+        }
 
         {{
             CMutexGuard guard(CNcbiApplication::GetInstanceMutex());
@@ -227,6 +231,11 @@ public:
         }
         os << "\n";
 
+        auto registry = node->m_SynRegistry;
+        _ASSERT(registry);
+
+        registry->Alerts(os);
+
         os << "OK:END\n";
     }
 };
@@ -269,6 +278,60 @@ public:
     }
 };
 
+class CGetConfProcessor : public CWorkerNodeControlServer::IRequestProcessor
+{
+public:
+    void Process(const string& request, CNcbiOstream& reply, CWorkerNodeControlServer* control_server) override;
+};
+
+void CGetConfProcessor::Process(const string&, CNcbiOstream& reply, CWorkerNodeControlServer* control_server)
+{
+    _ASSERT(control_server);
+
+    auto node = static_cast<SGridWorkerNodeImpl*>(control_server->GetWorkerNode());
+    _ASSERT(node);
+
+    auto registry = node->m_SynRegistry;
+    _ASSERT(registry);
+
+    registry->Report(reply);
+    reply << "OK:END\n";
+}
+
+class CAckAlertProcessor : public CWorkerNodeControlServer::IRequestProcessor
+{
+public:
+    void Process(const string& request, CNcbiOstream& reply, CWorkerNodeControlServer* control_server) override;
+};
+
+void CAckAlertProcessor::Process(const string& request, CNcbiOstream& reply, CWorkerNodeControlServer* control_server)
+{
+    _ASSERT(control_server);
+
+    auto node = static_cast<SGridWorkerNodeImpl*>(control_server->GetWorkerNode());
+    _ASSERT(node);
+
+    auto registry = node->m_SynRegistry;
+    _ASSERT(registry);
+
+    const string kAlertIDPrefix = " alert_";
+    auto pos = NStr::Find(request, kAlertIDPrefix, NStr::eNocase);
+
+    if (pos == NPOS) {
+        reply << "ERR:Alert ID is required\n";
+        return;
+    }
+
+    const auto kFlags = NStr::fConvErr_NoThrow | NStr::fAllowTrailingSymbols;
+    auto id = NStr::StringToUInt(request.c_str() + pos + kAlertIDPrefix.size(), kFlags);
+
+    if (!registry->AckAlert(id)) {
+        reply << "ERR:Failed to find an alert with such ID (" << id << ")\n";
+    } else {
+        reply << "OK:\n";
+    }
+}
+
 class CUnknownProcessor : public CWorkerNodeControlServer::IRequestProcessor
 {
 public:
@@ -306,6 +369,12 @@ CWorkerNodeControlServer::IRequestProcessor*
     if (NStr::StartsWith(cmd, TEMP_STRING_CTOR("GETLOAD")))
         return new CGetLoadProcessor;
 
+    if (NStr::StartsWith(cmd, TEMP_STRING_CTOR("GETCONF")))
+        return new CGetConfProcessor;
+
+    if (NStr::StartsWith(cmd, TEMP_STRING_CTOR("ACKALERT")))
+        return new CAckAlertProcessor;
+
     return new CUnknownProcessor;
 }
 
