@@ -1,7 +1,7 @@
 #ifndef CONNECT_SERVICES___NETCACHE_API_IMPL__HPP
 #define CONNECT_SERVICES___NETCACHE_API_IMPL__HPP
 
-/*  $Id: netcache_api_impl.hpp 535008 2017-05-04 16:02:37Z sadyrovr $
+/*  $Id: netcache_api_impl.hpp 555398 2018-01-17 20:27:02Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -54,32 +54,37 @@ struct SNetCacheServerProperties : public INetServerProperties
 class NCBI_XCONNECT_EXPORT CNetCacheServerListener : public INetServerConnectionListener
 {
 public:
-    CRef<INetServerProperties> AllocServerProperties() override;
+    void SetAuthString(const string& auth) { m_Auth = auth; }
+
+    TPropCreator GetPropCreator() const override;
+    INetServerConnectionListener* Clone() override;
 
-    void OnPreInit(CObject* api_impl, ISynRegistry& registry, SRegSynonyms& sections, string& client_name) override;
-    void OnInit(CObject* api_impl, ISynRegistry& registry, SRegSynonyms& sections) override;
     void OnConnected(CNetServerConnection& connection) override;
-    void OnError(const string& err_msg, CNetServer& server) override;
-    void OnWarning(const string& warn_msg, CNetServer& server) override;
 
-    static CRef<SNetCacheServerProperties> x_GetServerProperties(SNetServerImpl* server_impl);
+private:
+    void OnErrorImpl(const string& err_msg, CNetServer& server) override;
+    void OnWarningImpl(const string& warn_msg, CNetServer& server) override;
 
     string m_Auth;
 };
 
 struct NCBI_XCONNECT_EXPORT SNetCacheAPIImpl : public CObject
 {
-    SNetCacheAPIImpl(SConfigOrRegistry conf_or_reg, const string& section,
+    SNetCacheAPIImpl(CSynRegistryBuilder registry_builder, const string& section,
             const string& service, const string& client_name,
             CNetScheduleAPI::TInstance ns_api);
 
     // For use by SNetICacheClientImpl
-    SNetCacheAPIImpl(const string& api_name, const string& service, const string& client_name,
-            INetServerConnectionListener* listener);
+    SNetCacheAPIImpl();
 
     // Special constructor for CNetCacheAPI::GetServer().
     SNetCacheAPIImpl(SNetServerInPool* server, SNetCacheAPIImpl* parent);
 
+    CNetCacheServerListener* GetListener()
+    {
+        return static_cast<CNetCacheServerListener*>(m_Service->m_Listener.GetPointer());
+    }
+
     CNetCacheReader* GetPartReader(
         const string& blob_id,
         size_t offset,
@@ -97,7 +102,9 @@ struct NCBI_XCONNECT_EXPORT SNetCacheAPIImpl : public CObject
     virtual CNetServerConnection InitiateWriteCmd(CNetCacheWriter* nc_writer,
             const CNetCacheAPIParameters* parameters);
 
-    void AppendClientIPSessionID(string* cmd);
+    void AppendClientIPSessionID(string* cmd, CRequestContext& req);
+    void AppendHitID(string* cmd, CRequestContext& req);
+    void AppendClientIPSessionIDHitID(string* cmd);
     void AppendClientIPSessionIDPasswordAgeHitID(string* cmd,
             const CNetCacheAPIParameters* parameters);
     string MakeCmd(const char* cmd_base, const CNetCacheKey& key,
@@ -111,10 +118,9 @@ struct NCBI_XCONNECT_EXPORT SNetCacheAPIImpl : public CObject
         bool multiline_output,
         const CNetCacheAPIParameters* parameters,
         SNetServiceImpl::EServerErrorHandling error_handling =
-            SNetServiceImpl::eRethrowServerErrors,
-        INetServerConnectionListener* conn_listener = NULL);
+            SNetServiceImpl::eRethrowServerErrors);
 
-    void Init(ISynRegistry& registry, const SRegSynonyms& sections);
+    void Init(CSynRegistry& registry, const SRegSynonyms& sections);
 
     CNetService m_Service;
 
@@ -136,18 +142,14 @@ struct NCBI_XCONNECT_EXPORT SNetCacheAPIImpl : public CObject
 
 struct SNetCacheAdminImpl : public CObject
 {
-    SNetCacheAdminImpl(SNetCacheAPIImpl* nc_api_impl);
+    SNetCacheAdminImpl(SNetCacheAPIImpl* nc_api_impl) : m_API(nc_api_impl) {}
 
-    string MakeAdminCmd(const char* cmd);
+    void ExecOnAllServers(string cmd);
+    void PrintCmdOutput(string cmd, CNcbiOstream& output_stream, bool multiline_output = true);
 
     CNetCacheAPI m_API;
 };
 
-inline SNetCacheAdminImpl::SNetCacheAdminImpl(SNetCacheAPIImpl* nc_api_impl) :
-    m_API(nc_api_impl)
-{
-}
-
 class NCBI_XCONNECT_EXPORT SNetCacheMirrorTraversal : public IServiceTraversal
 {
 public:
