@@ -1,7 +1,7 @@
 #ifndef CONN___NETCACHE_RW__HPP
 #define CONN___NETCACHE_RW__HPP
 
-/*  $Id: netcache_rw.hpp 446411 2014-09-15 18:56:36Z kazimird $
+/*  $Id: netcache_rw.hpp 477209 2015-08-27 13:39:00Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -117,6 +117,7 @@ class NCBI_XCONNECT_EXPORT CNetCacheWriter : public IEmbeddedStreamWriter
 public:
     CNetCacheWriter(SNetCacheAPIImpl* impl,
         string* blob_id,
+        const string& key,
         ENetCacheResponseType response_type,
         const CNetCacheAPIParameters* parameters);
 
@@ -137,6 +138,7 @@ public:
     ENetCacheResponseType GetResponseType() const {return m_ResponseType;}
 
     const string& GetBlobID() const {return m_BlobID;}
+    const string& GetKey() const {return m_Key;}
 
     void SetBlobID(const string& blob_id) {m_BlobID = blob_id;}
 
@@ -144,6 +146,7 @@ private:
     bool IsConnectionOpen() { return m_TransmissionWriter.get() != NULL; }
     void ResetWriters();
     void AbortConnection();
+    EIO_Status TransmitImpl(const char* buf, size_t count);
     void Transmit(const void* buf, size_t count, size_t* bytes_written);
 
     void EstablishConnection();
@@ -156,6 +159,7 @@ private:
 
     CNetCacheAPI m_NetCacheAPI;
     string m_BlobID;
+    string m_Key; // Only for ICache mode.
     const CNetCacheAPIParameters* m_Parameters;
 
     CFileIO m_CacheFile;
