@@ -1,4 +1,4 @@
-/*  $Id: neticache_client.cpp 535008 2017-05-04 16:02:37Z sadyrovr $
+/*  $Id: neticache_client.cpp 555600 2018-01-19 16:46:14Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -62,12 +62,6 @@
 
 BEGIN_NCBI_SCOPE
 
-class CNetICacheServerListener : public CNetCacheServerListener
-{
-protected:
-    void OnInit(CObject* api_impl, ISynRegistry& registry, SRegSynonyms& sections) override;
-};
-
 const char* const kNetICacheDriverName = "netcache";
 
 static string s_CheckKeySubkey(
@@ -125,27 +119,33 @@ static string s_KeyVersionSubkeyToBlobID(
 
 static const char s_NetICacheAPIName[] = "NetICacheClient";
 
-struct SNetICacheClientImpl : public SNetCacheAPIImpl, protected CConnIniter
+struct SNetICacheClientImpl : public SNetCacheAPIImpl
 {
-    SNetICacheClientImpl(SConfigOrRegistry conf_or_reg,
+    SNetICacheClientImpl(CSynRegistryBuilder registry_builder,
             const string& section,
             const string& service_name,
             const string& client_name,
             const string& cache_name) :
-        SNetCacheAPIImpl(s_NetICacheAPIName, service_name, client_name,
-                new CNetICacheServerListener),
         m_CacheFlags(ICache::fBestPerformance)
     {
         m_DefaultParameters.SetCacheName(cache_name);
-        m_Service->Init(this, conf_or_reg,
-                { section, "netcache_api", "netcache_client", kNetICacheDriverName });
+        SRegSynonyms sections{ section, "netcache_api", "netcache_client", kNetICacheDriverName };
+        m_Service = SNetServiceImpl::Create(s_NetICacheAPIName, service_name, client_name,
+                new CNetCacheServerListener,
+                registry_builder, sections);
+        Init(registry_builder, sections);
+    }
+
+    SNetICacheClientImpl(SNetServerInPool* server, SNetICacheClientImpl* parent) :
+        SNetCacheAPIImpl(server, parent),
+        m_CacheFlags(parent->m_CacheFlags)
+    {
     }
 
     CNetServer::SExecResult ChooseServerAndExec(const string& cmd,
             const string& key,
             bool multiline_output,
-            const CNetCacheAPIParameters* parameters,
-            INetServerConnectionListener* conn_listener = NULL);
+            const CNetCacheAPIParameters* parameters);
 
     string MakeStdCmd(const char* cmd_base, const string& blob_id,
         const CNetCacheAPIParameters* parameters,
@@ -171,23 +171,15 @@ struct SNetICacheClientImpl : public SNetCacheAPIImpl, protected CConnIniter
         size_t* blob_size_ptr,
         const CNamedParameterList* optional);
 
-    void Init(ISynRegistry& registry, const SRegSynonyms& sections);
+    void Init(CSynRegistry& registry, const SRegSynonyms& sections);
 
     ICache::TFlags m_CacheFlags;
 };
 
-void CNetICacheServerListener::OnInit(CObject* api_impl, ISynRegistry& registry, SRegSynonyms& sections)
+void SNetICacheClientImpl::Init(CSynRegistry& registry, const SRegSynonyms& sections)
 {
-    CNetCacheServerListener::OnInit(api_impl, registry, sections);
+    SNetCacheAPIImpl::Init(registry, sections);
 
-    SNetICacheClientImpl* icache_impl =
-        static_cast<SNetICacheClientImpl*>(api_impl);
-
-    icache_impl->Init(registry, sections);
-}
-
-void SNetICacheClientImpl::Init(ISynRegistry& registry, const SRegSynonyms& sections)
-{
     auto cache_name = m_DefaultParameters.GetCacheName();
 
     if (cache_name.empty()) cache_name = registry.Get(sections, { "name", "cache_name" }, "default_cache");
@@ -302,8 +294,7 @@ CNetServer::SExecResult SNetICacheClientImpl::ChooseServerAndExec(
         const string& cmd,
         const string& key,
         bool multiline_output,
-        const CNetCacheAPIParameters* parameters,
-        INetServerConnectionListener* conn_listener)
+        const CNetCacheAPIParameters* parameters)
 {
     CNetServer selected_server(parameters->GetServerToUse());
     CNetServer* server_last_used_ptr(parameters->GetServerLastUsedPtr());
@@ -312,7 +303,7 @@ CNetServer::SExecResult SNetICacheClientImpl::ChooseServerAndExec(
         if (selected_server) {
             CNetServer::SExecResult exec_result(
                     selected_server.ExecWithRetry(cmd,
-                        multiline_output, conn_listener));
+                        multiline_output));
 
             if (server_last_used_ptr) *server_last_used_ptr = selected_server;
             return exec_result;
@@ -324,7 +315,7 @@ CNetServer::SExecResult SNetICacheClientImpl::ChooseServerAndExec(
         do {
             try {
                 exec_result = (*it).ExecWithRetry(cmd,
-                        multiline_output, conn_listener);
+                        multiline_output);
                 selected_server = *it;
                 break;
             }
@@ -354,14 +345,12 @@ CNetServer::SExecResult SNetICacheClientImpl::ChooseServerAndExec(
                 selected_server, server_check);
 
         m_Service->IterateUntilExecOK(cmd, multiline_output, exec_result,
-                &mirror_traversal, SNetServiceImpl::eIgnoreServerErrors,
-                conn_listener);
+                &mirror_traversal, SNetServiceImpl::eIgnoreServerErrors);
     } else {
         SWeightedServiceTraversal service_traversal(m_Service, key);
 
         m_Service->IterateUntilExecOK(cmd, multiline_output, exec_result,
-                &service_traversal, SNetServiceImpl::eIgnoreServerErrors,
-                conn_listener);
+                &service_traversal, SNetServiceImpl::eIgnoreServerErrors);
     }
 
     if (server_last_used_ptr != NULL)
@@ -664,33 +653,23 @@ bool CNetICacheClient::HasBlobs(const string&  key,
 bool CNetICacheClient::HasBlob(const string& key, const string& subkey,
         const CNamedParameterList* optional)
 {
-    try {
-        CNetCacheAPIParameters parameters(&m_Impl->m_DefaultParameters);
+    CNetCacheAPIParameters parameters(&m_Impl->m_DefaultParameters);
 
-        parameters.LoadNamedParameters(optional);
+    parameters.LoadNamedParameters(optional);
 
-        string response(m_Impl->ExecStdCmd("HASB", key, 0, subkey, &parameters));
-        return (response[0] == '1'|| NStr::StartsWith(response, "0, VER="));
-    }
-    // Workaround for a bug in NC v6.6.1 (CXX-4095)
-    // TODO: Throw away after all NC servers are upgraded to v6.6.2+
-    catch (CNetCacheException& e) {
-        if (e.GetErrCode() == CNetCacheException::eBlobNotFound)
-            return false;
-        throw;
-    }
+    string response(m_Impl->ExecStdCmd("HASB", key, 0, subkey, &parameters));
+    return (response[0] == '1'|| NStr::StartsWith(response, "0, VER="));
 }
 
-void CNetICacheClient::Purge(time_t access_timeout, EKeepVersions keep_last_version)
+void CNetICacheClient::Purge(time_t access_timeout)
 {
-    Purge(kEmptyStr, kEmptyStr, access_timeout, keep_last_version);
+    Purge(kEmptyStr, kEmptyStr, access_timeout);
 }
 
 
 void CNetICacheClient::Purge(const string&    key,
                              const string&    subkey,
-                             time_t           access_timeout,
-                             EKeepVersions)
+                             time_t           access_timeout)
 {
     if (access_timeout) {
         NCBI_THROW(CNetCacheException, eNotImplemented, "Not implemented");
@@ -704,20 +683,8 @@ void CNetICacheClient::Purge(const string&    key,
         return RemoveBlob(key, 0, subkey);
     }
 
-    // TODO: Replace with a command from CXX-8948 when it is deployed everywhere
-    using namespace ncbi::grid::netcache::search;
-
-    // NB: 'size >= 0' condition is used to get all blobs
-    auto result = Search(key.empty() ? fields::size >= 0 : fields::key == key);
-
-    for (auto& blob_info : result) {
-        try {
-            RemoveBlob(blob_info[fields::key], 0, blob_info[fields::subkey]);
-        }
-        catch (CNetCacheException& e) {
-            if (e.GetErrCode() != CNetCacheException::eBlobNotFound) throw;
-        }
-    }
+    const auto cmd = m_Impl->MakeStdCmd("PURGE2", "'" + key + "'", &m_Impl->m_DefaultParameters);
+    m_Impl->ChooseServerAndExec(cmd, key, false, &m_Impl->m_DefaultParameters);
 }
 
 IReader* CNetICacheClient::GetReadStream(
@@ -834,68 +801,49 @@ IReader* SNetICacheClientImpl::ReadCurrentBlobNotOlderThan(const string& key,
     }
 }
 
-class CSetValidWarningSuppressor : public CNetICacheServerListener
+class CSetValidWarningSuppressor
 {
 public:
     CSetValidWarningSuppressor(
-            INetServerConnectionListener* delegate_listener,
+            INetServerConnectionListener* listener,
             const string& key,
             const string& subkey,
             int version) :
-        m_DelegateListener(delegate_listener),
+        m_Listener(listener),
         m_Key(key),
         m_Subkey(subkey),
         m_Version(version)
     {
+        _ASSERT(m_Listener);
+
+        auto warning_handler = [&](const string& m, CNetServer s) {
+            return OnWarning(m, s);
+        };
+
+        m_Listener->SetWarningHandler(warning_handler);
     }
 
-    CRef<INetServerProperties> AllocServerProperties() override;
-    void OnPreInit(CObject* api_impl, ISynRegistry& registry, SRegSynonyms& sections, string& client_name) override;
-    void OnInit(CObject* api_impl, ISynRegistry& registry, SRegSynonyms& sections) override;
-    void OnConnected(CNetServerConnection& connection) override;
-    void OnError(const string& err_msg, CNetServer& server) override;
-    void OnWarning(const string& warn_msg, CNetServer& server) override;
+    ~CSetValidWarningSuppressor()
+    {
+        m_Listener->SetWarningHandler(nullptr);
+    }
 
-    CRef<INetServerConnectionListener> m_DelegateListener;
-    string m_Key;
-    string m_Subkey;
-    int m_Version;
+private:
+    bool OnWarning(const string& warn_msg, CNetServer server);
+
+    CRef<INetServerConnectionListener> m_Listener;
+    const string m_Key;
+    const string m_Subkey;
+    const int m_Version;
 };
 
-CRef<INetServerProperties> CSetValidWarningSuppressor::AllocServerProperties()
-{
-    return m_DelegateListener->AllocServerProperties();
-}
-
-void CSetValidWarningSuppressor::OnPreInit(CObject* api_impl, ISynRegistry& registry, SRegSynonyms& sections, string& client_name)
-{
-    m_DelegateListener->OnPreInit(api_impl, registry, sections, client_name);
-}
-
-void CSetValidWarningSuppressor::OnInit(CObject* api_impl, ISynRegistry& registry, SRegSynonyms& sections)
-{
-    m_DelegateListener->OnInit(api_impl, registry, sections);
-}
-
-void CSetValidWarningSuppressor::OnConnected(CNetServerConnection& connection)
-{
-    m_DelegateListener->OnConnected(connection);
-}
-
-void CSetValidWarningSuppressor::OnError(
-        const string& err_msg, CNetServer& server)
-{
-    m_DelegateListener->OnError(err_msg, server);
-}
-
-void CSetValidWarningSuppressor::OnWarning(
-        const string& warn_msg, CNetServer& server)
+bool CSetValidWarningSuppressor::OnWarning(const string& warn_msg, CNetServer)
 {
     SIZE_TYPE ver_pos = NStr::FindCase(warn_msg,
             CTempString("VER=", sizeof("VER=") - 1));
 
     if (ver_pos == NPOS)
-        m_DelegateListener->OnWarning(warn_msg, server);
+        return false;
     else {
         int version = atoi(warn_msg.c_str() + ver_pos + sizeof("VER=") - 1);
         if (version < m_Version) {
@@ -905,14 +853,14 @@ void CSetValidWarningSuppressor::OnWarning(
                     m_Version << " to " << version);
         }
     }
+
+    return true;
 }
 
 void CNetICacheClient::SetBlobVersionAsCurrent(const string& key,
         const string& subkey, int version)
 {
-    CRef<INetServerConnectionListener> warning_suppressor(
-            new CSetValidWarningSuppressor(m_Impl->m_Service->m_Listener,
-                    key, subkey, version));
+    CSetValidWarningSuppressor warning_suppressor(m_Impl->m_Service->m_Listener, key, subkey, version);
 
     CNetServer::SExecResult exec_result(
             m_Impl->ChooseServerAndExec(
@@ -921,8 +869,7 @@ void CNetICacheClient::SetBlobVersionAsCurrent(const string& key,
                             &m_Impl->m_DefaultParameters),
                     key,
                     false,
-                    &m_Impl->m_DefaultParameters,
-                    warning_suppressor));
+                    &m_Impl->m_DefaultParameters));
 
     if (!exec_result.response.empty()) {
         ERR_POST("SetBlobVersionAsCurrent(\"" << key << "\", " <<
@@ -1017,9 +964,9 @@ bool CNetICacheClient::SameCacheParams(const TCacheParams*) const
     return false;
 }
 
-void CNetICacheClient::SetEventHandler(INetEventHandler* event_handler)
+CNetICacheClientExt CNetICacheClientExt::GetServer(CNetServer::TInstance server)
 {
-    m_Impl->m_Service->SetEventHandler(event_handler);
+    return new SNetICacheClientImpl(server->m_ServerInPool, m_Impl);
 }
 
 vector<CNetICacheClient::CBlobInfo> CNetICacheClient::Search(
@@ -1066,10 +1013,19 @@ void CNetICacheClientExt::ProlongBlobLifetime(const string& key,
     cmd += "\" \"\" ttl=";
     cmd += NStr::NumericToString((unsigned)ttl.GetAsDouble());
 
-    m_Impl->AppendClientIPSessionID(&cmd);
+    m_Impl->AppendClientIPSessionIDHitID(&cmd);
     m_Impl->ChooseServerAndExec(cmd, key, false, &m_Impl->m_DefaultParameters);
 }
 
+SNetCacheAPIImpl* CNetICacheClientExt::GetNetCacheAPI()
+{
+    return new SNetCacheAPIImpl(m_Impl.GetObject());
+}
+
+const SNetCacheAPIImpl* CNetICacheClientExt::GetNetCacheAPI() const
+{
+    return new SNetCacheAPIImpl(m_Impl.GetObject());
+}
 
 /// Class factory for NetCache implementation of ICache
 ///
@@ -1085,14 +1041,15 @@ public:
     {
     }
 
-    virtual ICache* CreateInstance(
+private:
+    virtual ICache* x_CreateInstance(
                    const string&    driver  = kEmptyStr,
                    CVersionInfo     version = NCBI_INTERFACE_VERSION(ICache),
                    const TPluginManagerParamTree* params = 0) const;
 };
 
 
-ICache* CNetICacheCF::CreateInstance(
+ICache* CNetICacheCF::x_CreateInstance(
            const string&                  driver,
            CVersionInfo                   version,
            const TPluginManagerParamTree* params) const
