@@ -1,4 +1,4 @@
-/*  $Id: netschedule_api.cpp 541657 2017-07-20 13:36:33Z ivanov $
+/*  $Id: netschedule_api.cpp 555398 2018-01-17 20:27:02Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,13 +37,11 @@
 #include <connect/ncbi_socket.hpp>
 #include <connect/ncbi_conn_exception.hpp>
 #include <connect/ncbi_userhost.hpp>
-#include <connect/ncbi_core_cxx.hpp>
 
 #include <corelib/ncbi_system.hpp>
 #include <corelib/plugin_manager_impl.hpp>
 
-#include <util/ncbi_url.hpp>
-
+#include <array>
 #include <memory>
 #include <stdio.h>
 
@@ -70,29 +68,27 @@ void ThrowIllegalChar(const string& name, const string& value, char c)
 
 using namespace grid::netschedule;
 
-void g_AppendClientIPSessionIDHitID(string& cmd, bool use_next_sub_hit_id)
-{
-    CRequestContext& req = CDiagContext::GetRequestContext();
-    g_AppendClientIPAndSessionID(cmd, req);
-    g_AppendHitID(cmd, use_next_sub_hit_id ? req.GetNextSubHitID() : req.GetCurrentSubHitID());
-}
-
 SNetScheduleNotificationThread::SNetScheduleNotificationThread(
         SNetScheduleAPIImpl* ns_api) :
     m_API(ns_api),
     m_StopThread(false)
+{
+}
+
+SNetScheduleNotificationReceiver::SNetScheduleNotificationReceiver()
 {
     STimeout rto;
     rto.sec = rto.usec = 0;
-    m_UDPSocket.SetTimeout(eIO_Read, &rto);
+    socket.SetDataLogging(TServConn_ConnDataLogging::GetDefault() ? eOn : eOff);
+    socket.SetTimeout(eIO_Read, &rto);
 
-    EIO_Status status = m_UDPSocket.Bind(0);
+    EIO_Status status = socket.Bind(0);
     if (status != eIO_Success) {
         NCBI_THROW_FMT(CException, eUnknown,
             "Could not bind a UDP socket: " << IO_StatusStr(status));
     }
 
-    m_UDPPort = m_UDPSocket.GetLocalPort(eNH_HostByteOrder);
+    port = socket.GetLocalPort(eNH_HostByteOrder);
 }
 
 void SNetScheduleNotificationThread::CmdAppendPortAndTimeout(
@@ -109,17 +105,25 @@ void SNetScheduleNotificationThread::CmdAppendPortAndTimeout(
 
 CNetScheduleNotificationHandler::CNetScheduleNotificationHandler()
 {
-    STimeout rto;
-    rto.sec = rto.usec = 0;
-    m_UDPSocket.SetTimeout(eIO_Read, &rto);
+}
 
-    EIO_Status status = m_UDPSocket.Bind(0);
-    if (status != eIO_Success) {
-        NCBI_THROW_FMT(CException, eUnknown,
-            "Could not bind a UDP socket: " << IO_StatusStr(status));
-    }
+CUrlArgs s_CreateCUrlArgs(const string& output)
+try {
+    return CUrlArgs(output);
+}
+catch (CUrlParserException&) {
+    return CUrlArgs();
+}
 
-    m_UDPPort = m_UDPSocket.GetLocalPort(eNH_HostByteOrder);
+SNetScheduleOutputParser::SNetScheduleOutputParser(const string& output) :
+        CUrlArgs(s_CreateCUrlArgs(output))
+{
+}
+
+const string& SNetScheduleOutputParser::operator()(const string& param) const
+{
+    auto it = FindFirst(param);
+    return it != GetArgs().end() ? it->value : kEmptyStr;
 }
 
 int g_ParseNSOutput(const string& attr_string, const char* const* attr_names,
@@ -151,29 +155,24 @@ int g_ParseNSOutput(const string& attr_string, const char* const* attr_names,
     return -1;
 }
 
-#define JOB_NOTIF_ATTR_COUNT 3
-
 SNetScheduleNotificationThread::ENotificationType
         SNetScheduleNotificationThread::CheckNotification(string* ns_node)
 {
-    static const char* const attr_names[JOB_NOTIF_ATTR_COUNT] =
-        {"reason", "ns_node", "queue"};
+    _ASSERT(ns_node);
 
-    string attr_values[JOB_NOTIF_ATTR_COUNT];
+    SNetScheduleOutputParser parser(m_Receiver.message);
 
-    int defined_attrs = g_ParseNSOutput(m_Message,
-            attr_names, attr_values, JOB_NOTIF_ATTR_COUNT);
+    if (parser("queue") != m_API->m_Queue) return eNT_Unknown;
 
-    if (defined_attrs == -1 || attr_values[2] != m_API->m_Queue)
-        return eNT_Unknown;
+    *ns_node = parser("ns_node");
 
-    *ns_node = attr_values[1];
+    const auto reason = parser("reason");
 
-    if (defined_attrs != JOB_NOTIF_ATTR_COUNT)
+    if (reason.empty())
         return eNT_GetNotification;
-    else if (NStr::CompareCase(attr_values[0], CTempString("get", 3)) == 0)
+    else if (NStr::CompareCase(reason, CTempString("get", 3)) == 0)
         return eNT_GetNotification;
-    else if (NStr::CompareCase(attr_values[0], CTempString("read", 4)) == 0)
+    else if (NStr::CompareCase(reason, CTempString("read", 4)) == 0)
         return eNT_ReadNotification;
     else
         return eNT_Unknown;
@@ -236,20 +235,14 @@ void* SNetScheduleNotificationThread::Main()
 
     static const STimeout two_seconds = {2, 0};
 
-    size_t msg_len;
     string server_host;
 
     while (!m_StopThread)
-        if (m_UDPSocket.Wait(&two_seconds) == eIO_Success) {
+        if (m_Receiver.socket.Wait(&two_seconds) == eIO_Success) {
             if (m_StopThread)
                 break;
 
-            if (m_UDPSocket.Recv(m_Buffer, sizeof(m_Buffer), &msg_len,
-                    &server_host, NULL) == eIO_Success) {
-                while (msg_len > 0 && m_Buffer[msg_len - 1] == '\0')
-                    --msg_len;
-                m_Message.assign(m_Buffer, msg_len);
-
+            if (m_Receiver(&server_host)) {
                 string ns_node;
 
                 ENotificationType notif_type = CheckNotification(&ns_node);
@@ -269,21 +262,27 @@ void* SNetScheduleNotificationThread::Main()
     return NULL;
 }
 
-bool CNetScheduleNotificationHandler::ReceiveNotification(string* server_host)
+bool SNetScheduleNotificationReceiver::operator()(string* server_host)
 {
+    array<char, 64 * 1024> buffer; // Enough to hold any UDP
     size_t msg_len;
 
-    if (m_UDPSocket.Recv(m_Buffer, sizeof(m_Buffer), &msg_len,
+    if (socket.Recv(buffer.data(), buffer.size(), &msg_len,
             server_host, NULL) != eIO_Success)
         return false;
 
-    while (msg_len > 0 && m_Buffer[msg_len - 1] == '\0')
-        --msg_len;
-    m_Message.assign(m_Buffer, msg_len);
+    _ASSERT(buffer.size() > msg_len);
+    buffer[msg_len] = '\0'; // Make it null-terminated in case it's not
+    message.assign(buffer.data()); // Ignore everything after the first null character
 
     return true;
 }
 
+bool CNetScheduleNotificationHandler::ReceiveNotification(string* server_host)
+{
+    return m_Receiver(server_host);
+}
+
 bool CNetScheduleNotificationHandler::WaitForNotification(
         const CDeadline& deadline, string* server_host)
 {
@@ -295,7 +294,7 @@ bool CNetScheduleNotificationHandler::WaitForNotification(
         if (timeout.sec == 0 && timeout.usec == 0)
             return false;
 
-        switch (m_UDPSocket.Wait(&timeout)) {
+        switch (m_Receiver.socket.Wait(&timeout)) {
         case eIO_Timeout:
             return false;
 
@@ -339,7 +338,7 @@ SNetScheduleAPIImpl::~SNetScheduleAPIImpl()
         if (m_NotificationThread != NULL) {
             m_NotificationThread->m_StopThread = true;
             CDatagramSocket().Send("INTERRUPT", sizeof("INTERRUPT"),
-                    "127.0.0.1", m_NotificationThread->m_UDPPort);
+                    "127.0.0.1", m_NotificationThread->m_Receiver.port);
             m_NotificationThread->Join();
         }
     }
@@ -378,10 +377,10 @@ CTempString s_GetSection(bool ns_conf)
     return ns_conf ? "netschedule_conf_from_netschedule" : "netcache_conf_from_netschedule";
 }
 
-CNetScheduleConfigLoader::CNetScheduleConfigLoader(ISynRegistry& registry, SRegSynonyms& sections, bool ns_conf) :
+CNetScheduleConfigLoader::CNetScheduleConfigLoader(CSynRegistry& registry, SRegSynonyms& sections, bool ns_conf) :
     m_Registry(registry), m_Sections(sections), m_NsConf(ns_conf), m_Mode(eImplicit)
 {
-    sections.insert(sections.begin(), s_GetSection(m_NsConf));
+    sections.Insert(s_GetSection(m_NsConf));
 
     const auto param = "load_config_from_ns";
 
@@ -404,6 +403,9 @@ bool CNetScheduleConfigLoader::Transform(const string& prefix, string& name) con
         }
     }
 
+    // Do not load client_name from server
+    if (name == "client_name") return false;
+
     // Only params starting with provided prefix are used
     if (NStr::StartsWith(name, prefix)) {
         name.erase(0, prefix.size());
@@ -467,6 +469,32 @@ string SNetScheduleAPIImpl::MakeAuthString()
 {
     string auth(m_Service->MakeAuthString());
 
+    const CVersion* version = nullptr;
+    string name;
+
+    {{
+        CMutexGuard guard(CNcbiApplication::GetInstanceMutex());
+
+        if (CNcbiApplication* app = CNcbiApplication::Instance()) {
+            version = &app->GetFullVersion();
+            name = app->GetProgramDisplayName();
+        }
+    }}
+
+    if (version && m_ProgramVersion.empty()) {
+        m_ProgramVersion += name;
+        auto package_name = version->GetPackageName();
+
+        if (!package_name.empty()) {
+            m_ProgramVersion += ": ";
+            m_ProgramVersion += package_name;
+            m_ProgramVersion += ' ';
+            m_ProgramVersion += version->GetPackageVersion().Print();
+            m_ProgramVersion += " built on ";
+            m_ProgramVersion += version->GetBuildInfo().date;
+        }
+    }
+
     if (!m_ProgramVersion.empty()) {
         auth += " prog=\"";
         auth += m_ProgramVersion;
@@ -505,15 +533,11 @@ string SNetScheduleAPIImpl::MakeAuthString()
         auth += '\"';
     }
 
-    {{
-        CMutexGuard guard(CNcbiApplication::GetInstanceMutex());
-        CNcbiApplication* app = CNcbiApplication::Instance();
-        if (app != NULL) {
-            auth += " client_version=\"";
-            auth += app->GetFullVersion().GetVersionInfo().Print();
-            auth += '\"';
-        }
-    }}
+    if (version) {
+        auth += " client_version=\"";
+        auth += version->GetVersionInfo().Print();
+        auth += '\"';
+    }
 
     ITERATE(SNetScheduleAPIImpl::TAuthParams, it, m_AuthParams) {
         auth += it->second;
@@ -529,50 +553,25 @@ string SNetScheduleAPIImpl::MakeAuthString()
     // like the "queue not found" error.
     if (m_Mode & fNonWnCompatible) {
         auth += "\r\nVERSION";
-        g_AppendClientIPSessionIDHitID(auth);
     }
 
     return auth;
 }
 
-bool CNetScheduleServerListener::NeedToSubmitAffinities(
-        SNetServerImpl* server_impl)
+INetServerConnectionListener::TPropCreator CNetScheduleServerListener::GetPropCreator() const
 {
-    return !x_GetServerProperties(server_impl)->affs_synced;
+    return [] { return new SNetScheduleServerProperties; };
 }
 
-void CNetScheduleServerListener::SetAffinitiesSynced(
-        SNetServerImpl* server_impl, bool affs_synced)
+INetServerConnectionListener* CNetScheduleServerListener::Clone()
 {
-    x_GetServerProperties(server_impl)->affs_synced = affs_synced;
+    return new CNetScheduleServerListener(*this);
 }
 
-CRef<SNetScheduleServerProperties>
-        CNetScheduleServerListener::x_GetServerProperties(
-                SNetServerImpl* server_impl)
+void SNetScheduleAPIImpl::Init(CSynRegistry& registry, SRegSynonyms& sections)
 {
-    return CRef<SNetScheduleServerProperties>(
-            static_cast<SNetScheduleServerProperties*>(
-                    server_impl->m_ServerInPool->
-                            m_ServerProperties.GetPointerOrNull()));
-}
-
-CRef<INetServerProperties> CNetScheduleServerListener::AllocServerProperties()
-{
-    return CRef<INetServerProperties>(new SNetScheduleServerProperties);
-}
-
-void CNetScheduleServerListener::OnInit(CObject* api_impl, ISynRegistry& registry, SRegSynonyms& sections)
-{
-    SNetScheduleAPIImpl* ns_impl = static_cast<SNetScheduleAPIImpl*>(api_impl);
-    _ASSERT(ns_impl);
-
     SetDiagUserAndHost();
-    ns_impl->Init(registry, sections);
-}
 
-void SNetScheduleAPIImpl::Init(ISynRegistry& registry, SRegSynonyms& sections)
-{
     if (!m_Queue.empty()) limits::Check<limits::SQueueName>(m_Queue);
 
     const string& user(GetDiagContext().GetUsername());
@@ -581,13 +580,11 @@ void SNetScheduleAPIImpl::Init(ISynRegistry& registry, SRegSynonyms& sections)
         (user.empty() ? kEmptyStr : user + '@') +
         GetDiagContext().GetHost();
 
-    m_Config.Init(registry, sections);
-
     CNetScheduleConfigLoader loader(registry, sections);
 
     bool affinities_initialized = false;
 
-    // There are two phases of OnInit in case we need to load config from server
+    // There are two phases of Init in case we need to load config from server
     // 1) Setup as much as possible and try to get config from server
     // 2) Setup everything using received config from server
     do {
@@ -642,17 +639,16 @@ void CNetScheduleServerListener::OnConnected(CNetServerConnection& connection)
 
         // Usually, all attributes come together, so no need to check version
         if (!ns_node.empty() && !ns_session.empty()) {
-            CRef<SNetScheduleServerProperties> server_props =
-                x_GetServerProperties(connection->m_Server);
+            auto server_props = connection->m_Server->Get<SNetScheduleServerProperties>();
 
             // Version cannot change without session, so no need to compare, too
             if (server_props->ns_node != ns_node ||
                     server_props->ns_session != ns_session) {
-                CFastMutexGuard guard(m_ServerByNodeMutex);
+                CFastMutexGuard guard(m_SharedData->m_ServerByNodeMutex);
                 server_props->ns_node = ns_node;
                 server_props->ns_session = ns_session;
                 server_props->version = version;
-                m_ServerByNode[ns_node] = connection->m_Server->m_ServerInPool;
+                m_SharedData->m_ServerByNode[ns_node] = connection->m_Server->m_ServerInPool;
                 server_props->affs_synced = false;
             }
         }
@@ -666,7 +662,7 @@ void CNetScheduleServerListener::OnConnected(CNetServerConnection& connection)
         connection->WriteLine(m_Auth);
 }
 
-void CNetScheduleServerListener::OnError(
+void CNetScheduleServerListener::OnErrorImpl(
     const string& err_msg, CNetServer& server)
 {
     string code;
@@ -682,10 +678,6 @@ void CNetScheduleServerListener::OnError(
     // Map code into numeric value
     CException::TErrCode err_code = CNetScheduleExceptionMap::GetCode(code);
 
-    if (m_EventHandler && m_EventHandler->OnError(err_code)) {
-        return;
-    }
-
     switch (err_code) {
     case CException::eInvalid:
         NCBI_THROW(CNetServiceException, eCommunicationError, err_msg);
@@ -706,41 +698,34 @@ void CNetScheduleServerListener::OnError(
     }
 }
 
-void CNetScheduleServerListener::OnWarning(const string& warn_msg,
+void CNetScheduleServerListener::OnWarningImpl(const string& warn_msg,
         CNetServer& server)
 {
-    if (m_EventHandler)
-        m_EventHandler->OnWarning(warn_msg, server);
-    else {
         LOG_POST(Warning << server->m_ServerInPool->m_Address.AsString() <<
                 ": " << warn_msg);
-    }
 }
 
 const char* const kNetScheduleAPIDriverName = "netschedule_api";
 
-SNetScheduleAPIImpl::SNetScheduleAPIImpl(SConfigOrRegistry conf_or_reg, const string& section) :
-    m_Mode(GetMode(false, true)),
-    m_Service(new SNetServiceImpl("NetScheduleAPI", kEmptyStr, kEmptyStr,
-                new CNetScheduleServerListener(m_Mode & fNonWnCompatible)))
-{
-    m_Service->Init(this, conf_or_reg, { section, kNetScheduleAPIDriverName });
-}
-
-SNetScheduleAPIImpl::SNetScheduleAPIImpl(const string& service_name, const string& client_name,
+SNetScheduleAPIImpl::SNetScheduleAPIImpl(CSynRegistryBuilder registry_builder, const string& section,
+        const string& service_name, const string& client_name,
         const string& queue_name, bool wn, bool try_config) :
     m_Mode(GetMode(wn, try_config)),
-    m_Service(new SNetServiceImpl("NetScheduleAPI", service_name, client_name,
-                new CNetScheduleServerListener(m_Mode & fNonWnCompatible))),
+    m_SharedData(new SNetScheduleSharedData),
     m_Queue(queue_name)
 {
-    m_Service->Init(this, nullptr, kNetScheduleAPIDriverName);
+    SRegSynonyms sections{ section, kNetScheduleAPIDriverName };
+    m_Service = SNetServiceImpl::Create("NetScheduleAPI", service_name, client_name,
+            new CNetScheduleServerListener(m_Mode & fNonWnCompatible, m_SharedData),
+            registry_builder, sections);
+    Init(registry_builder, sections);
 }
 
 SNetScheduleAPIImpl::SNetScheduleAPIImpl(
         SNetServerInPool* server, SNetScheduleAPIImpl* parent) :
     m_Mode(parent->m_Mode),
-    m_Service(new SNetServiceImpl(server, parent->m_Service)),
+    m_SharedData(parent->m_SharedData),
+    m_Service(SNetServiceImpl::Clone(server, parent->m_Service)),
     m_Queue(parent->m_Queue),
     m_ProgramVersion(parent->m_ProgramVersion),
     m_ClientNode(parent->m_ClientNode),
@@ -769,7 +754,7 @@ CNetScheduleAPI::CNetScheduleAPI(CConfig* conf, const string& conf_section) :
 
 CNetScheduleAPI::CNetScheduleAPI(const string& service_name,
         const string& client_name, const string& queue_name) :
-    m_Impl(new SNetScheduleAPIImpl(service_name, client_name, queue_name))
+    m_Impl(new SNetScheduleAPIImpl(nullptr, kEmptyStr, service_name, client_name, queue_name))
 {
 }
 
@@ -943,30 +928,20 @@ CNetScheduleAPI::EJobStatus CNetScheduleAPI::GetJobDetails(
         time_t* job_exptime,
         ENetScheduleQueuePauseMode* pause_mode)
 {
-    string resp = m_Impl->x_ExecOnce("STATUS2", job);
+    string cmd(g_MakeBaseCmd("STATUS2", job.job_id));
+    g_AppendClientIPSessionIDHitID(cmd);
+    cmd += " need_progress_msg=1";
 
-    static const char* const s_JobStatusAttrNames[] = {
-            "job_status",       // 0
-            "job_exptime",      // 1
-            "input",            // 2
-            "output",           // 3
-            "ret_code",         // 4
-            "err_msg",          // 5
-            "pause",            // 6
-    };
+    CNetServer::SExecResult exec_result;
 
-#define NUMBER_OF_STATUS_ATTRS (sizeof(s_JobStatusAttrNames) / \
-    sizeof(*s_JobStatusAttrNames))
+    m_Impl->GetServer(job)->ConnectAndExec(cmd, false, exec_result);
 
-    string attr_values[NUMBER_OF_STATUS_ATTRS];
+    SNetScheduleOutputParser parser(exec_result.response);
 
-    g_ParseNSOutput(resp, s_JobStatusAttrNames,
-            attr_values, NUMBER_OF_STATUS_ATTRS);
+    const auto status = StringToStatus(parser("job_status"));
 
-    EJobStatus status = StringToStatus(attr_values[0]);
-
-    s_SetJobExpTime(job_exptime, attr_values[1]);
-    s_SetPauseMode(pause_mode, attr_values[6]);
+    s_SetJobExpTime(job_exptime, parser("job_exptime"));
+    s_SetPauseMode(pause_mode, parser("pause"));
 
     switch (status) {
     case ePending:
@@ -977,11 +952,10 @@ CNetScheduleAPI::EJobStatus CNetScheduleAPI::GetJobDetails(
     case eReading:
     case eConfirmed:
     case eReadFailed:
-        job.input = attr_values[2];
-        job.output = attr_values[3];
-        job.ret_code = NStr::StringToInt(attr_values[4],
-                NStr::fConvErr_NoThrow);
-        job.error_msg = attr_values[5];
+        job.input = parser("input");
+        job.output = parser("output");
+        job.ret_code = NStr::StringToInt(parser("ret_code"), NStr::fConvErr_NoThrow);
+        job.error_msg = parser("err_msg");
         break;
 
     default:
@@ -993,7 +967,7 @@ CNetScheduleAPI::EJobStatus CNetScheduleAPI::GetJobDetails(
 
     job.affinity.erase();
     job.mask = CNetScheduleAPI::eEmptyMask;
-    job.progress_msg.erase();
+    job.progress_msg = parser("msg");
 
     return status;
 }
@@ -1017,22 +991,12 @@ CNetScheduleAPI::EJobStatus SNetScheduleAPIImpl::GetJobStatus(const string& cmd,
         return CNetScheduleAPI::eJobNotFound;
     }
 
-    static const char* const s_AttrNames[] = {
-        "job_status",       // 0
-        "job_exptime",      // 1
-        "pause",            // 2
-    };
+    SNetScheduleOutputParser parser(response);
 
-#define NUMBER_OF_ATTRS (sizeof(s_AttrNames) / sizeof(*s_AttrNames))
+    s_SetJobExpTime(job_exptime, parser("job_exptime"));
+    s_SetPauseMode(pause_mode, parser("pause"));
 
-    string attr_values[NUMBER_OF_ATTRS];
-
-    g_ParseNSOutput(response, s_AttrNames, attr_values, NUMBER_OF_ATTRS);
-
-    s_SetJobExpTime(job_exptime, attr_values[1]);
-    s_SetPauseMode(pause_mode, attr_values[2]);
-
-    return CNetScheduleAPI::StringToStatus(attr_values[0]);
+    return CNetScheduleAPI::StringToStatus(parser("job_status"));
 }
 
 bool SNetScheduleAPIImpl::GetServerByNode(const string& ns_node,
@@ -1041,14 +1005,11 @@ bool SNetScheduleAPIImpl::GetServerByNode(const string& ns_node,
     SNetServerInPool* known_server; /* NCBI_FAKE_WARNING */
 
     {{
-        CNetScheduleServerListener* listener = GetListener();
+        CFastMutexGuard guard(m_SharedData->m_ServerByNodeMutex);
 
-        CFastMutexGuard guard(listener->m_ServerByNodeMutex);
+        auto server_props_it = m_SharedData->m_ServerByNode.find(ns_node);
 
-        CNetScheduleServerListener::TServerByNode::iterator
-            server_props_it(listener->m_ServerByNode.find(ns_node));
-
-        if (server_props_it == listener->m_ServerByNode.end())
+        if (server_props_it == m_SharedData->m_ServerByNode.end())
             return false;
 
         known_server = server_props_it->second;
@@ -1231,7 +1192,7 @@ void SNetScheduleAPIImpl::SetAuthParam(const string& param_name,
     UpdateAuthString();
 }
 
-void SNetScheduleAPIImpl::InitAffinities(ISynRegistry& registry, const SRegSynonyms& sections)
+void SNetScheduleAPIImpl::InitAffinities(CSynRegistry& registry, const SRegSynonyms& sections)
 {
     const bool claim_new_affinities = registry.Get(sections, "claim_new_affinities", false);
     const bool process_any_job =      registry.Get(sections, "process_any_job", false);
@@ -1293,17 +1254,6 @@ void SNetScheduleAPIImpl::InitAffinities(ISynRegistry& registry, const SRegSynon
     }
 }
 
-void SNetScheduleAPIImpl::SConfig::Init(ISynRegistry& registry, SRegSynonyms& sections)
-{
-    const auto kDOCTimeoutDefault = 0.03;
-
-    double doc_timeout = registry.Get(sections, "direct_output_connect_timeout", kDOCTimeoutDefault);
-
-    if (doc_timeout <= 0) doc_timeout = kDOCTimeoutDefault;
-
-    g_CTimeoutToSTimeout(CTimeout(doc_timeout), direct_output_connect_timeout);
-}
-
 ///////////////////////////////////////////////////////////////////////////////
 
 /// @internal
@@ -1391,11 +1341,6 @@ void CNetScheduleAPIExt::UseOldStyleAuth()
     m_Impl->UseOldStyleAuth();
 }
 
-void CNetScheduleAPIExt::SetEventHandler(INetEventHandler* event_handler)
-{
-    m_Impl->m_Service->SetEventHandler(event_handler);
-}
-
 CCompoundIDPool CNetScheduleAPIExt::GetCompoundIDPool()
 {
     return m_Impl->m_CompoundIDPool;
@@ -1424,14 +1369,14 @@ CNetScheduleAPI::TInstance
 CNetScheduleAPIExt::CreateWnCompat(const string& service_name,
         const string& client_name)
 {
-    return new SNetScheduleAPIImpl(service_name, client_name, kEmptyStr, true, false);
+    return new SNetScheduleAPIImpl(nullptr, kEmptyStr, service_name, client_name, kEmptyStr, true, false);
 }
 
 CNetScheduleAPI::TInstance
 CNetScheduleAPIExt::CreateNoCfgLoad(const string& service_name,
         const string& client_name, const string& queue_name)
 {
-    return new SNetScheduleAPIImpl(service_name, client_name, queue_name, false, false);
+    return new SNetScheduleAPIImpl(nullptr, kEmptyStr, service_name, client_name, queue_name, false, false);
 }
 
 
