@@ -1,4 +1,4 @@
-/*  $Id: netschedule_api_executor.cpp 537163 2017-05-26 15:32:47Z sadyrovr $
+/*  $Id: netschedule_api_executor.cpp 551694 2017-11-21 16:59:53Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -53,8 +53,6 @@ bool s_DoParseGet2JobResponse(CNetScheduleJob& job, const string& response)
         eClientSessionID,
         ePageHitID,
         eJobMask,
-        eSubmitterNotifHost,
-        eSubmitterNotifPort,
         eNumberOfJobBits
     };
     int job_bits = 0;
@@ -92,24 +90,7 @@ bool s_DoParseGet2JobResponse(CNetScheduleJob& job, const string& response)
         } else if (field->name == "ncbi_phid") {
             job_bits |= (1 << ePageHitID);
             job.page_hit_id = field->value;
-
-        } else if (field->name == "submitter_notif_host") {
-            job_bits |= (1 << eSubmitterNotifHost);
-            job.submitter.first = field->value;
-
-        } else if (field->name == "submitter_notif_port") {
-            job_bits |= (1 << eSubmitterNotifPort);
-            job.submitter.second = static_cast<unsigned short>(stoul(field->value));
-
-        // TODO: Remove after there are no NetSchedule servers of version 4.30.0 (with this typo)
-        } else if (field->name == "sumbitter_notif_host") {
-            job_bits |= (1 << eSubmitterNotifHost);
-            job.submitter.first = field->value;
-        } else if (field->name == "sumbitter_notif_port") {
-            job_bits |= (1 << eSubmitterNotifPort);
-            job.submitter.second = static_cast<unsigned short>(stoul(field->value));
         }
-        // TODO: End
 
         if (job_bits == (1 << eNumberOfJobBits) - 1)
             break;
@@ -210,11 +191,10 @@ void SNetScheduleExecutorImpl::ClaimNewPreferredAffinity(
                         (*it).GetServerAddress() <<
                         " of a new affinity " << e);
 
-                CNetScheduleServerListener* listener = m_API->GetListener();
+                CFastMutexGuard sync_guard(m_API->m_SharedData->m_AffinitySubmissionMutex);
+                auto& affs_synced = it.GetServer()->Get<SNetScheduleServerProperties>()->affs_synced;
 
-                CFastMutexGuard sync_guard(listener->m_AffinitySubmissionMutex);
-
-                listener->SetAffinitiesSynced(it.GetServer(), false);
+                affs_synced = false;
             }
     }
 }
@@ -252,13 +232,12 @@ bool SNetScheduleExecutorImpl::ExecGET(SNetServerImpl* server,
             throw;
 
         {
-            CNetScheduleServerListener* listener = m_API->GetListener();
+            CFastMutexGuard guard(m_API->m_SharedData->m_AffinitySubmissionMutex);
+            auto& affs_synced = server->Get<SNetScheduleServerProperties>()->affs_synced;
 
-            CFastMutexGuard guard(listener->m_AffinitySubmissionMutex);
-
-            listener->SetAffinitiesSynced(server, false);
+            affs_synced = false;
             server->ConnectAndExec(MkSETAFFCmd(), false,  exec_result);
-            listener->SetAffinitiesSynced(server, true);
+            affs_synced = true;
         }
 
         server->ConnectAndExec(get_cmd, false,
@@ -451,23 +430,14 @@ bool CNetScheduleNotificationHandler::RequestJob(
     return true;
 }
 
-#define GET2_NOTIF_ATTR_COUNT 2
-
 bool CNetScheduleNotificationHandler::CheckRequestJobNotification(
         CNetScheduleExecutor::TInstance executor, CNetServer* server)
 {
-    static const char* const attr_names[GET2_NOTIF_ATTR_COUNT] =
-        {"ns_node", "queue"};
+    SNetScheduleOutputParser parser(m_Receiver.message);
 
-    string attr_values[GET2_NOTIF_ATTR_COUNT];
+    if (parser("queue") != executor->m_API.GetQueueName()) return false;
 
-    if (g_ParseNSOutput(m_Message,
-            attr_names, attr_values, GET2_NOTIF_ATTR_COUNT) !=
-                    GET2_NOTIF_ATTR_COUNT ||
-            attr_values[1] != executor->m_API.GetQueueName())
-        return false;
-
-    return executor->m_API->GetServerByNode(attr_values[0], server);
+    return executor->m_API->GetServerByNode(parser("ns_node"), server);
 }
 
 inline
@@ -701,14 +671,12 @@ void CNetScheduleGETCmdListener::OnExec(
         {
             CNetServerConnection conn(conn_impl);
 
-            CNetScheduleServerListener* listener =
-                    m_Executor->m_API->GetListener();
+            CFastMutexGuard guard(m_Executor->m_API->m_SharedData->m_AffinitySubmissionMutex);
+            auto& affs_synced = conn->m_Server->Get<SNetScheduleServerProperties>()->affs_synced;
 
-            CFastMutexGuard guard(listener->m_AffinitySubmissionMutex);
-
-            if (listener->NeedToSubmitAffinities(conn_impl->m_Server)) {
+            if (!affs_synced) {
                 conn.Exec(m_Executor->MkSETAFFCmd(), false);
-                listener->SetAffinitiesSynced(conn_impl->m_Server, true);
+                affs_synced  = true;
             }
         }
         break;
