@@ -1,7 +1,7 @@
 #ifndef CONN_SERVICES___NETSCHEDULE_API_IMPL__HPP
 #define CONN_SERVICES___NETSCHEDULE_API_IMPL__HPP
 
-/*  $Id: netschedule_api_impl.hpp 537390 2017-05-31 15:32:22Z sadyrovr $
+/*  $Id: netschedule_api_impl.hpp 555378 2018-01-17 19:24:12Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -137,29 +137,6 @@ using namespace grid::netschedule;
 
 #define SERVER_PARAMS_ASK_MAX_COUNT 100
 
-inline
-void g_AppendHitID(string& cmd, const string& sub_hit_id)
-{
-    cmd += " ncbi_phid=\"";
-    cmd += sub_hit_id;
-    cmd += '"';
-}
-
-inline
-void g_AppendClientIPAndSessionID(string& cmd, const CRequestContext& req)
-{
-    if (req.IsSetClientIP()) {
-        cmd += " ip=\"";
-        cmd += req.GetClientIP();
-        cmd += '"';
-    }
-
-    cmd += " sid=\"";
-    cmd += NStr::PrintableString(req.GetSessionID());
-    cmd += '"';
-}
-
-
 inline string g_MakeBaseCmd(const string& cmd_name, const string& job_key)
 {
     string cmd(cmd_name);
@@ -190,43 +167,51 @@ struct SNetScheduleServerProperties : public INetServerProperties
 class CNetScheduleConfigLoader
 {
 public:
-    CNetScheduleConfigLoader(ISynRegistry& registry, SRegSynonyms& sections, bool ns_conf = true);
+    CNetScheduleConfigLoader(CSynRegistry& registry, SRegSynonyms& sections, bool ns_conf = true);
 
     bool operator()(SNetScheduleAPIImpl* impl);
 
 private:
     bool Transform(const string& prefix, string& name) const;
 
-    ISynRegistry& m_Registry;
+    CSynRegistry& m_Registry;
     const SRegSynonyms& m_Sections;
     const bool m_NsConf;
     enum { eOff, eImplicit, eExplicit } m_Mode;
 };
 
+struct SNetScheduleSharedData;
+
 class CNetScheduleServerListener : public INetServerConnectionListener
 {
 public:
-    CNetScheduleServerListener(bool non_wn) : m_NonWn(non_wn) {}
+    CNetScheduleServerListener(bool non_wn, SNetScheduleSharedData* shared_data) :
+        m_NonWn(non_wn),
+        m_SharedData(shared_data)
+    {
+        _ASSERT(shared_data);
+    }
 
     void SetAuthString(const string& auth) { m_Auth = auth; }
     string& Scope() { return m_Scope; }
 
-    bool NeedToSubmitAffinities(SNetServerImpl* server_impl);
-    void SetAffinitiesSynced(SNetServerImpl* server_impl, bool affs_synced);
+    TPropCreator GetPropCreator() const override;
+    INetServerConnectionListener* Clone() override;
 
-    static CRef<SNetScheduleServerProperties> x_GetServerProperties(SNetServerImpl* server_impl);
-
-    CRef<INetServerProperties> AllocServerProperties() override;
-
-    void OnInit(CObject* api_impl, ISynRegistry& registry, SRegSynonyms& sections) override;
     void OnConnected(CNetServerConnection& connection) override;
-    void OnError(const string& err_msg, CNetServer& server) override;
-    void OnWarning(const string& warn_msg, CNetServer& server) override;
 
 private:
-    string m_Auth;
+    void OnErrorImpl(const string& err_msg, CNetServer& server) override;
+    void OnWarningImpl(const string& warn_msg, CNetServer& server) override;
 
-public:
+    string m_Auth;
+    const bool m_NonWn;
+    string m_Scope;
+    CRef<SNetScheduleSharedData> m_SharedData;
+};
+
+struct SNetScheduleSharedData : CObject
+{
     CFastMutex m_ServerByNodeMutex;
     typedef map<string, SNetServerInPool*> TServerByNode;
     TServerByNode m_ServerByNode;
@@ -234,10 +219,6 @@ public:
     // Make sure the worker node does not attempt to submit its
     // preferred affinities from two threads.
     CFastMutex m_AffinitySubmissionMutex;
-
-private:
-    const bool m_NonWn;
-    string m_Scope;
 };
 
 // Structure that governs NetSchedule server notifications.
@@ -295,21 +276,15 @@ struct SNetScheduleNotificationThread : public CThread
 
     virtual void* Main();
 
-    unsigned short GetPort() const {return m_UDPPort;}
+    unsigned short GetPort() const { return m_Receiver.port; }
 
-    const string& GetMessage() const {return m_Message;}
-
-    void PrintPortNumber();
+    const string& GetMessage() const { return m_Receiver.message; }
 
     void CmdAppendPortAndTimeout(string* cmd, unsigned remaining_seconds);
 
     SNetScheduleAPIImpl* m_API;
 
-    CDatagramSocket m_UDPSocket;
-    unsigned short m_UDPPort;
-
-    char m_Buffer[1024];
-    string m_Message;
+    SNetScheduleNotificationReceiver m_Receiver;
 
     bool m_StopThread;
 
@@ -333,14 +308,13 @@ private:
     {
         if (wn) return fWorkerNode;
         if (try_config) return fNetSchedule | fConfigLoading;
-                        return fNetSchedule;
+        return fNetSchedule;
     }
 
 public:
-    SNetScheduleAPIImpl(SConfigOrRegistry conf_or_reg, const string& section);
-
-    SNetScheduleAPIImpl(const string& service_name, const string& client_name,
-        const string& queue_name, bool wn = false, bool try_config = true);
+    SNetScheduleAPIImpl(CSynRegistryBuilder registry_builder, const string& section,
+        const string& service_name = kEmptyStr, const string& client_name = kEmptyStr,
+        const string& queue_name = kEmptyStr, bool wn = false, bool try_config = true);
 
     // Special constructor for CNetScheduleAPI::GetServer().
     SNetScheduleAPIImpl(SNetServerInPool* server, SNetScheduleAPIImpl* parent);
@@ -400,25 +374,16 @@ public:
     void UseOldStyleAuth();
     void SetAuthParam(const string& param_name, const string& param_value);
     CCompoundIDPool GetCompoundIDPool() { return m_CompoundIDPool; }
-    void Init(ISynRegistry& registry, SRegSynonyms& sections);
-    void InitAffinities(ISynRegistry& registry, const SRegSynonyms& sections);
+    void Init(CSynRegistry& registry, SRegSynonyms& sections);
+    void InitAffinities(CSynRegistry& registry, const SRegSynonyms& sections);
     string MakeAuthString();
 
-    struct SConfig
-    {
-        STimeout direct_output_connect_timeout;
-
-        void Init(ISynRegistry& registry, SRegSynonyms& sections);
-    };
-
-    const SConfig& GetConfig() const { return m_Config; }
-
 private:
     const TMode m_Mode;
-    SConfig m_Config;
 
 public:
     CNetScheduleAPI::EClientType m_ClientType = CNetScheduleAPI::eCT_Auto;
+    CRef<SNetScheduleSharedData> m_SharedData;
     CNetService m_Service;
 
     string m_Queue;
@@ -464,7 +429,7 @@ struct SNetScheduleSubmitterImpl : public CObject
         const string& error_message);
 
     CNetScheduleAPI::EJobStatus SubmitJobAndWait(CNetScheduleJob& job,
-            unsigned wait_time, time_t* job_exptime = NULL, pair<bool*, CNcbiOstream*> = {});
+            unsigned wait_time, time_t* job_exptime = NULL);
 
     void AppendClientIPSessionIDHitID(string& cmd, const string& job_group);
 
