@@ -1,4 +1,4 @@
-/*  $Id: netservice_api.cpp 540945 2017-07-12 15:49:46Z ivanov $
+/*  $Id: netservice_api.cpp 573310 2018-10-25 16:53:14Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -49,11 +49,12 @@
 #include <corelib/ncbi_system.hpp>
 #include <corelib/ncbi_config.hpp>
 #include <corelib/ncbi_message.hpp>
-#include <corelib/env_reg.hpp>
 
 #include <util/random_gen.hpp>
 #include <util/checksum.hpp>
 
+#include <deque>
+
 #define NCBI_USE_ERRCODE_X   ConnServ_Connection
 
 #define LBSMD_PENALIZED_RATE_BOUNDARY -0.01
@@ -63,33 +64,32 @@ BEGIN_NCBI_SCOPE
 // The purpose of this class is to execute commands suppressing possible errors and avoiding retries
 class SNetServiceImpl::CTry
 {
-    struct SHandler : public INetEventHandler
-    {
-        bool OnError(CException::TErrCode) { return true; }
-    };
-
 public:
     CTry(SNetServiceImpl* service) :
         m_Service(service)
     {
         _ASSERT(m_Service);
-        Swap(new SHandler);
+
+        auto error_handler = [](const string&, CNetServer) {
+            return true;
+        };
+
+        Swap(error_handler);
     }
 
     ~CTry()
     {
-        Swap(m_OriginalHandler);
+        Swap(nullptr);
     }
 
 private:
-    void Swap(INetEventHandler* handler)
+    void Swap(CNetService::TEventHandler error_handler)
     {
-        m_OriginalHandler = m_Service->SetEventHandler(handler);
+        m_Service->m_Listener->SetErrorHandler(error_handler);
         swap(m_MaxRetries, m_Service->m_ConnectionMaxRetries);
     }
 
     CNetRef<SNetServiceImpl> m_Service;
-    CRef<INetEventHandler> m_OriginalHandler;
     unsigned m_MaxRetries = 0;
 };
 
@@ -119,6 +119,7 @@ void SDiscoveredServers::DeleteThis()
 
 CNetServer CNetServiceIterator::GetServer()
 {
+    m_Impl->m_ServerGroup->m_Service->m_RebalanceStrategy->OnResourceRequested();
     return new SNetServerImpl(m_Impl->m_ServerGroup->m_Service,
             m_Impl->m_ServerGroup->m_Service->m_ServerPool->
             ReturnServer(m_Impl->m_Position->first));
@@ -313,18 +314,56 @@ bool SNetServiceIterator_Weighted::Prev()
     return true;
 }
 
-SNetServerPoolImpl::SNetServerPoolImpl(INetServerConnectionListener* listener,
-        bool old_style_auth) :
-    m_Listener(listener),
+SNetServerPoolImpl::SNetServerPoolImpl(INetServerConnectionListener* listener) :
+    m_PropCreator(listener->GetPropCreator()),
     m_EnforcedServer(0, 0),
     m_MaxTotalTime(CTimeout::eInfinite),
-    m_UseOldStyleAuth(old_style_auth)
+    m_UseOldStyleAuth(false)
 {
 }
 
+SNetServiceImpl::SNetServiceImpl(const string& api_name, const string& service_name, const string& client_name,
+        INetServerConnectionListener* listener) :
+    m_Listener(listener),
+    m_ServerPool(new SNetServerPoolImpl(listener)),
+    m_ServiceName(service_name),
+    m_APIName(api_name),
+    m_ClientName(client_name)
+{
+}
+
+SNetServiceImpl::SNetServiceImpl(SNetServerInPool* server, SNetServiceImpl* prototype) :
+    m_Listener(prototype->m_Listener->Clone()),
+    m_ServerPool(prototype->m_ServerPool),
+    m_ServiceName(server->m_Address.AsString()),
+    m_APIName(prototype->m_APIName),
+    m_ClientName(prototype->m_ClientName),
+    m_UseSmartRetries(prototype->m_UseSmartRetries),
+    m_ConnectionMaxRetries(prototype->m_ConnectionMaxRetries),
+    m_ConnectionRetryDelay(prototype->m_ConnectionRetryDelay),
+    m_NetInfo(prototype->m_NetInfo)
+{
+    Construct(server);
+}
+
+SNetServiceImpl::SNetServiceImpl(const string& service_name, SNetServiceImpl* prototype) :
+    m_Listener(prototype->m_Listener->Clone()),
+    m_ServerPool(prototype->m_ServerPool),
+    m_ServiceName(service_name),
+    m_RebalanceStrategy(new CSimpleRebalanceStrategy(prototype->m_RebalanceStrategy)),
+    m_APIName(prototype->m_APIName),
+    m_ClientName(prototype->m_ClientName),
+    m_UseSmartRetries(prototype->m_UseSmartRetries),
+    m_ConnectionMaxRetries(prototype->m_ConnectionMaxRetries),
+    m_ConnectionRetryDelay(prototype->m_ConnectionRetryDelay),
+    m_NetInfo(prototype->m_NetInfo)
+{
+    Construct();
+}
+
 void SNetServiceImpl::Construct(SNetServerInPool* server)
 {
-    m_ServiceType = CNetService::eSingleServerService;
+    m_ServiceType = eSingleServerService;
     m_DiscoveredServers = AllocServerGroup(0);
     CFastMutexGuard server_mutex_lock(m_ServerPool->m_ServerMutex);
     m_DiscoveredServers->m_Servers.push_back(TServerRate(server, 1));
@@ -338,13 +377,33 @@ void SNetServiceImpl::Construct()
         string host, port;
 
         if (!NStr::SplitInTwo(m_ServiceName, ":", host, port))
-            m_ServiceType = CNetService::eLoadBalancedService;
+            m_ServiceType = eLoadBalancedService;
         else
-            Construct(m_ServerPool->FindOrCreateServerImpl(SServerAddress(host,
+            Construct(m_ServerPool->FindOrCreateServerImpl(CNetServer::SAddress(host,
                     (unsigned short) NStr::StringToInt(port))));
     }
 }
 
+SNetServiceImpl* SNetServiceImpl::Create(
+        const string& api_name, const string& service_name, const string& client_name,
+        INetServerConnectionListener* listener,
+        CSynRegistry& registry, SRegSynonyms& sections, const string& ns_client_name)
+{
+    CNetRef<SNetServiceImpl> rv(new SNetServiceImpl(api_name, service_name, client_name, listener));
+    rv->Init(registry, sections, ns_client_name);
+    return rv.Release();
+}
+
+SNetServiceImpl* SNetServiceImpl::Clone(SNetServerInPool* server, SNetServiceImpl* prototype)
+{
+    return new SNetServiceImpl(server, prototype);
+}
+
+SNetServiceImpl* SNetServiceImpl::Clone(const string& service_name, SNetServiceImpl* prototype)
+{
+    return new SNetServiceImpl(service_name, prototype);
+}
+
 #ifdef NCBI_GRID_XSITE_CONN_SUPPORT
 void CNetService::AllowXSiteConnections()
 {
@@ -371,7 +430,7 @@ bool SNetServiceXSiteAPI::IsUsingXSiteProxy()
     return m_AllowXSiteConnections.load();
 }
 
-void SNetServiceXSiteAPI::InitXSite(ISynRegistry& registry, const SRegSynonyms& sections)
+void SNetServiceXSiteAPI::InitXSite(CSynRegistry& registry, const SRegSynonyms& sections)
 {
     if (registry.Get({ "netservice_api", sections }, "allow_xsite_conn", false)) {
         AllowXSiteConnections();
@@ -380,83 +439,98 @@ void SNetServiceXSiteAPI::InitXSite(ISynRegistry& registry, const SRegSynonyms&
 
 void SNetServiceXSiteAPI::ConnectXSite(CSocket& socket,
         SNetServerImpl::SConnectDeadline& deadline,
-        const SServerAddress& original)
+        const CNetServer::SAddress& original, const string& service)
 {
-    SServerAddress actual(original);
+    CNetServer::SAddress actual(original);
+    _ASSERT(actual.port);
     ticket_t ticket = 0;
 
     if (IsForeignAddr(actual.host)) {
         union {
-            SFWDRequestReply rq;
-            char buffer[FWD_MAX_RR_SIZE + 1];
+            SFWDRequestReply rr;
+            char buf[FWD_RR_MAX_SIZE + 1];
         };
+        memset(&rr, 0, sizeof(rr));
 
-        memset(buffer, 0, sizeof(buffer));
-        rq.host =                     actual.host;
-        rq.port = SOCK_HostToNetShort(actual.port);
-        rq.flag = SOCK_HostToNetShort(1);
-        _ASSERT(offsetof(SFWDRequestReply, text) +
-                sizeof(kXSiteFwd) < sizeof(buffer));
-        memcpy(rq.text, kXSiteFwd, sizeof(kXSiteFwd));
+        rr.host =                     actual.host;
+        rr.port = SOCK_HostToNetShort(actual.port);
+        rr.flag = SOCK_HostToNetShort(FWD_RR_FIREWALL | FWD_RR_KEEPALIVE);
+
+        auto text_max = sizeof(buf)-1 - offsetof(SFWDRequestReply,text);
+        auto text_len = service.size() ? min(service.size() + 1, text_max) : 0;
+        memcpy(rr.text, service.c_str(), text_len);
 
         size_t len = 0;
 
         CConn_ServiceStream svc(kXSiteFwd);
-        if (svc.write((const char*) &rq.ticket/*0*/, sizeof(rq.ticket)) &&
-                svc.write(buffer, offsetof(SFWDRequestReply, text) +
-                        sizeof(kXSiteFwd))) {
-            svc.read(buffer, sizeof(buffer) - 1);
+        svc.rdbuf()->PUBSETBUF(0, 0);  // quick way to make stream unbuffered
+        if (svc.write((const char*) &rr.ticket/*0*/, sizeof(rr.ticket))  &&
+            svc.write(buf, offsetof(SFWDRequestReply,text) + text_len)) {
+            svc.read(buf, sizeof(buf)-1);
             len = (size_t) svc.gcount();
-            _ASSERT(len < sizeof(buffer));
+            _ASSERT(len < sizeof(buf));
         }
 
-        memset(buffer + len, 0, sizeof(buffer) - len);
+        memset(buf + len, 0, sizeof(buf) - len); // NB: terminates "text" field
 
-        if (len < offsetof(SFWDRequestReply, text) ||
-            (rq.flag & 0xF0F0) || rq.port == 0) {
+        if (len < offsetof(SFWDRequestReply,text)
+            ||  (rr.flag & FWD_RR_ERRORMASK)  ||  !rr.port) {
             const char* err;
             if (len == 0)
                 err = "Connection refused";
-            else if (len < offsetof(SFWDRequestReply, text))
+            else if (len < offsetof(SFWDRequestReply,text))
                 err = "Short response received";
-            else if (!(rq.flag & 0xF0F0))
-                err = rq.flag & 0x0F0F ? "Client rejected" : "Unknown error";
-            else if (NStr::strncasecmp(buffer, "NCBI", 4) == 0)
-                err = buffer;
-            else if (rq.text[0])
-                err = rq.text;
+            else if (!(rr.flag & FWD_RR_ERRORMASK))
+                err = rr.flag & FWD_RR_REJECTMASK
+                    ? "Client rejected" : "Unknown error";
+            else if (memcmp(buf, "NCBI", 4) != 0)
+                err = rr.text[0] ? rr.text : "Unspecified error";
             else
-                err = "Unspecified error";
+                err = buf;
             NCBI_THROW_FMT(CNetSrvConnException, eConnectionFailure,
-                           "Error while acquiring an auth ticket from a "
-                           "cross-site connection proxy: " << err);
+                           "Error while acquiring auth ticket from"
+                           " cross-site connection proxy "
+                           << kXSiteFwd << ": " << err);
         }
 
-        if (rq.ticket != 0) {
-            actual.host =                     rq.host;
-            actual.port = SOCK_NetToHostShort(rq.port);
+        if (rr.ticket) {
+            ticket      = rr.ticket;
+            actual.host =                     rr.host;
+            actual.port = SOCK_NetToHostShort(rr.port);
         } else {
             SOCK sock;
-            actual.port = 0;
             EIO_Status io_st = CONN_GetSOCK(svc.GetCONN(), &sock);
-            if (sock != NULL)
-                SOCK_CreateOnTop(sock, 0, &sock);
-            if (io_st != eIO_Success  ||  sock == NULL) {
-                NCBI_THROW(CNetSrvConnException, eConnectionFailure,
-                           "Error while connecting to proxy.");
+            if (sock)
+                io_st = SOCK_CreateOnTop(sock, 0, &sock);
+            _ASSERT(!sock == !(io_st == eIO_Success));
+            if (sock) {
+                // excess read data to return into sock
+                text_len  = strlen(rr.text) + 1/*'\0'-terminated*/;
+                if (text_len > text_max)
+                    text_len = text_max;
+                text_len += offsetof(SFWDRequestReply,text);
+                _ASSERT(text_len <= len);
+                io_st = SOCK_Pushback(sock, buf + text_len, len - text_len);
+            }
+            if (io_st != eIO_Success) {
+                SOCK_Destroy(sock);
+                const char* err = IO_StatusStr(io_st);
+                NCBI_THROW_FMT(CNetSrvConnException, eConnectionFailure,
+                               "Error while tunneling through proxy "
+                               << kXSiteFwd << ": " << err);
             }
             socket.Reset(sock, eTakeOwnership, eCopyTimeoutsToSOCK);
+            actual.port = 0;
         }
-        ticket = rq.ticket;
     }
 
     if (actual.port) {
         SNetServerImpl::ConnectImpl(socket, deadline, actual, original);
     }
 
-    if (ticket && socket.Write(&ticket, sizeof(ticket)) != eIO_Success) {
+    if (ticket  &&  socket.Write(&ticket, sizeof(ticket)) != eIO_Success) {
         NCBI_THROW(CNetSrvConnException, eConnectionFailure,
-                "Error while sending proxy auth ticket.");
+                   "Error while sending proxy auth ticket");
     }
 }
 
@@ -469,7 +543,8 @@ int SNetServiceXSiteAPI::GetDomain(unsigned int ip)
     NcbiIsLocalIPEx(&addr, &info);
 
     if (!info.num) {
-        NCBI_THROW(CNetSrvConnException, eLBNameNotFound, "Cannot determine local domain");
+        NCBI_THROW(CNetSrvConnException, eLBNameNotFound,
+                   "Cannot determine local domain");
     }
     
     return info.num;
@@ -487,23 +562,21 @@ atomic<bool> SNetServiceXSiteAPI::m_AllowXSiteConnections{false};
 
 #else
 
-void SNetServiceXSiteAPI::InitXSite(ISynRegistry&, const SRegSynonyms&)
+void SNetServiceXSiteAPI::InitXSite(CSynRegistry&, const SRegSynonyms&)
 {
 }
 
 void SNetServiceXSiteAPI::ConnectXSite(CSocket& socket,
         SNetServerImpl::SConnectDeadline& deadline,
-        const SServerAddress& original)
+        const CNetServer::SAddress& original, const string&)
 {
     SNetServerImpl::ConnectImpl(socket, deadline, original, original);
 }
 
 #endif
 
-void SNetServiceImpl::Init(CObject* api_impl, const IRegistry* top_registry, SRegSynonyms sections)
+void SNetServiceImpl::Init(CSynRegistry& registry, SRegSynonyms& sections, const string& ns_client_name)
 {
-    _ASSERT(m_Listener);
-
     // Initialize the connect library and LBSM structures
     // used in DiscoverServersIfNeeded().
     {
@@ -515,34 +588,6 @@ void SNetServiceImpl::Init(CObject* api_impl, const IRegistry* top_registry, SRe
         conn_initer.NoOp();
     }
 
-    CSynRegistry syn_registry;
-    CCachedSynRegistry registry(syn_registry);
-
-    {
-        CMutexGuard guard(CNcbiApplication::GetInstanceMutex());
-
-        if (CNcbiApplication* app = CNcbiApplication::Instance()) {
-            syn_registry.Add(app->GetConfig());
-        } else {
-            syn_registry.Add(*new CEnvironmentRegistry);
-        }
-    }
-
-    if (top_registry) syn_registry.Add(*top_registry);
-
-    // Remove empty sections
-    for (auto it = sections.begin(); it != sections.end(); ) {
-        if (it->empty()) {
-            it = sections.erase(it);
-        } else {
-            ++it;
-        }
-    }
-
-    if (sections.empty()) {
-        NCBI_THROW_FMT(CArgException, eNoValue, "Configuration section was not provided");
-    }
-
     NStr::TruncateSpacesInPlace(m_ServiceName);
 
     // TODO:
@@ -552,7 +597,8 @@ void SNetServiceImpl::Init(CObject* api_impl, const IRegistry* top_registry, SRe
     // Do not override explicitly set client name
     if (m_ClientName.empty()) m_ClientName = registry.Get(sections, { "client_name", "client" }, "");
 
-    m_Listener->OnPreInit(api_impl, registry, sections, m_ClientName);
+    // Use client name from NetSchedule API if it's not provided for NetCache API
+    if (m_ClientName.empty()) m_ClientName = ns_client_name;
 
     if (m_ServiceName.empty()) {
         m_ServiceName = registry.Get(sections, { "service", "service_name" }, "");
@@ -587,18 +633,17 @@ void SNetServiceImpl::Init(CObject* api_impl, const IRegistry* top_registry, SRe
         m_ClientName = app->GetProgramDisplayName();
     }
 
-    m_ServerPool->Init(registry, sections, m_Listener.GetPointerOrNull());
+    auto max_requests = registry.Get(sections, "rebalance_requests", CSimpleRebalanceStrategy::DefaultMaxRequests());
+    auto max_seconds = registry.Get(sections, "rebalance_time", CSimpleRebalanceStrategy::DefaultMaxSeconds());
+    m_RebalanceStrategy = new CSimpleRebalanceStrategy(max_requests, max_seconds);
+
+    m_ServerPool->Init(registry, sections);
 
     Construct();
-
-    m_Listener->OnInit(api_impl, registry, sections);
 }
 
-void SNetServerPoolImpl::Init(ISynRegistry& registry, const SRegSynonyms& sections, INetServerConnectionListener* listener)
+void SNetServerPoolImpl::Init(CSynRegistry& registry, const SRegSynonyms& sections)
 {
-    int max_requests = CSimpleRebalanceStrategy::DefaultMaxRequests();
-    double max_seconds = CSimpleRebalanceStrategy::DefaultMaxSeconds();
-
     const auto kConnTimeoutDefault = 2.0;
     const auto kCommTimeoutDefault = 12.0;
     const auto kFirstServerTimeoutDefault = 0.3;
@@ -631,57 +676,7 @@ void SNetServerPoolImpl::Init(ISynRegistry& registry, const SRegSynonyms& sectio
     double max_total_time = registry.Get(sections, "max_connection_time", 0.0);
     if (max_total_time > 0) m_MaxTotalTime = CTimeout(max_total_time);
 
-    max_requests = registry.Get(sections, "rebalance_requests", max_requests);
-    max_seconds = registry.Get(sections, "rebalance_time", max_seconds);
-
     m_ThrottleParams.Init(registry, sections);
-
-    m_RebalanceStrategy = new CSimpleRebalanceStrategy(max_requests, max_seconds);
-
-    m_Listener = listener;
-}
-
-void SNetServerPoolImpl::SThrottleParams::Init(ISynRegistry& registry, const SRegSynonyms& sections)
-{
-    m_ServerThrottlePeriod = registry.Get(sections, "throttle_relaxation_period", 0);
-
-    if (m_ServerThrottlePeriod <= 0) return;
-
-    m_MaxConsecutiveIOFailures = registry.Get(sections,
-            { "throttle_by_consecutive_connection_failures", "throttle_by_subsequent_connection_failures" }, 0);
-
-    m_ThrottleUntilDiscoverable = registry.Get(sections, "throttle_hold_until_active_in_lb", false);
-
-    // These values must correspond to each other
-    const auto default_error_rate = "0/1";
-    m_IOFailureThresholdNumerator = 0;
-    m_IOFailureThresholdDenominator = 1;
-
-    const string error_rate = registry.Get(sections, "throttle_by_connection_error_rate", default_error_rate);
-
-    if (error_rate == default_error_rate || error_rate.empty()) return;
-
-    string numerator_str, denominator_str;
-
-    if (!NStr::SplitInTwo(error_rate, "/", numerator_str, denominator_str)) return;
-
-    const auto flags = NStr::fConvErr_NoThrow | NStr::fAllowLeadingSpaces | NStr::fAllowTrailingSpaces;
-
-    int numerator = NStr::StringToInt(numerator_str, flags);
-    int denominator = NStr::StringToInt(denominator_str, flags);
-
-    if (numerator < 0) numerator = 0;
-
-    if (denominator < 1) {
-        denominator = 1;
-
-    } else if (denominator > CONNECTION_ERROR_HISTORY_MAX) {
-        numerator = (numerator * CONNECTION_ERROR_HISTORY_MAX) / denominator;
-        denominator = CONNECTION_ERROR_HISTORY_MAX;
-    }
-
-    m_IOFailureThresholdNumerator = numerator;
-    m_IOFailureThresholdDenominator = denominator;
 }
 
 SDiscoveredServers* SNetServiceImpl::AllocServerGroup(
@@ -709,7 +704,7 @@ string SNetServiceImpl::MakeAuthString()
     auth += '\"';
 
     if (!m_ServerPool->m_UseOldStyleAuth) {
-        if (m_ServiceType == CNetService::eLoadBalancedService) {
+        if (m_ServiceType == eLoadBalancedService) {
             auth += " svc=\"";
             auth += NStr::PrintableString(m_ServiceName);
             auth += '\"';
@@ -737,19 +732,14 @@ CNetServerPool CNetService::GetServerPool()
     return m_Impl->m_ServerPool;
 }
 
-CNetService::EServiceType CNetService::GetServiceType() const
-{
-    return m_Impl->m_ServiceType;
-}
-
 bool CNetService::IsLoadBalanced() const
 {
-    return m_Impl->m_ServiceType == CNetService::eLoadBalancedService;
+    return m_Impl->IsLoadBalanced();
 }
 
 void CNetServerPool::StickToServer(const string& host, unsigned short port)
 {
-    m_Impl->m_EnforcedServer = SServerAddress(host, port);
+    m_Impl->m_EnforcedServer = CNetServer::SAddress(host, port);
 }
 
 void CNetService::PrintCmdOutput(const string& cmd,
@@ -800,7 +790,7 @@ void CNetService::PrintCmdOutput(const string& cmd,
 }
 
 SNetServerInPool* SNetServerPoolImpl::FindOrCreateServerImpl(
-        const SServerAddress& server_address)
+        CNetServer::SAddress server_address)
 {
     pair<TNetServerByAddress::iterator, bool> loc(m_Servers.insert(
             TNetServerByAddress::value_type(server_address,
@@ -809,8 +799,7 @@ SNetServerInPool* SNetServerPoolImpl::FindOrCreateServerImpl(
     if (!loc.second)
         return loc.first->second;
 
-    SNetServerInPool* server = new SNetServerInPool(server_address.host, server_address.port,
-            m_Listener->AllocServerProperties().GetPointerOrNull());
+    auto* server = new SNetServerInPool(move(server_address), m_PropCreator(), m_ThrottleParams);
 
     loc.first->second = server;
 
@@ -820,38 +809,37 @@ SNetServerInPool* SNetServerPoolImpl::FindOrCreateServerImpl(
 CRef<SNetServerInPool> SNetServerPoolImpl::ReturnServer(
         SNetServerInPool* server_impl)
 {
-    m_RebalanceStrategy->OnResourceRequested();
-
     CFastMutexGuard server_mutex_lock(m_ServerMutex);
 
     server_impl->m_ServerPool = this;
     return CRef<SNetServerInPool>(server_impl);
 }
 
-CNetServer SNetServiceImpl::GetServer(const SServerAddress& server_address)
+CNetServer SNetServerPoolImpl::GetServer(SNetServiceImpl* service, CNetServer::SAddress server_address)
 {
-    m_ServerPool->m_RebalanceStrategy->OnResourceRequested();
+    CFastMutexGuard server_mutex_lock(m_ServerMutex);
 
-    CFastMutexGuard server_mutex_lock(m_ServerPool->m_ServerMutex);
+    auto* server = FindOrCreateServerImpl(m_EnforcedServer.host == 0 ? move(server_address) : m_EnforcedServer);
+    server->m_ServerPool = this;
 
-    SNetServerInPool* server = m_ServerPool->FindOrCreateServerImpl(
-            m_ServerPool->m_EnforcedServer.host == 0 ?
-            server_address : m_ServerPool->m_EnforcedServer);
+    return new SNetServerImpl(service, server);
+}
 
-    server->m_ServerPool = m_ServerPool;
-
-    return new SNetServerImpl(this, server);
+CNetServer SNetServiceImpl::GetServer(CNetServer::SAddress server_address)
+{
+    m_RebalanceStrategy->OnResourceRequested();
+    return m_ServerPool->GetServer(this, move(server_address));
 }
 
 CNetServer CNetService::GetServer(const string& host,
         unsigned short port)
 {
-    return m_Impl->GetServer(SServerAddress(host, port));
+    return m_Impl->GetServer(CNetServer::SAddress(host, port));
 }
 
 CNetServer CNetService::GetServer(unsigned host, unsigned short port)
 {
-    return m_Impl->GetServer(SServerAddress(host, port));
+    return m_Impl->GetServer(CNetServer::SAddress(host, port));
 }
 
 class SRandomServiceTraversal : public IServiceTraversal
@@ -884,12 +872,11 @@ CNetServer::SExecResult SNetServiceImpl::FindServerAndExec(const string& cmd,
         bool multiline_output)
 {
     switch (m_ServiceType) {
-    default: // CNetService::eServiceNotDefined
+    default: // eServiceNotDefined
         NCBI_THROW_FMT(CNetSrvConnException, eSrvListEmpty,
-                m_APIName << ": command '" << cmd <<
-                        "' requires a server but none specified");
+                m_APIName << ": service name is not set");
 
-    case CNetService::eLoadBalancedService:
+    case eLoadBalancedService:
         {
             CNetServer::SExecResult exec_result;
 
@@ -897,14 +884,14 @@ CNetServer::SExecResult SNetServiceImpl::FindServerAndExec(const string& cmd,
 
             IterateUntilExecOK(cmd, multiline_output,
                     exec_result, &random_traversal,
-                    SNetServiceImpl::eIgnoreServerErrors,
-                    m_Listener);
+                    SNetServiceImpl::eIgnoreServerErrors);
 
             return exec_result;
         }
 
-    case CNetService::eSingleServerService:
+    case eSingleServerService:
         {
+            m_RebalanceStrategy->OnResourceRequested();
             CNetServer server(new SNetServerImpl(this,
                     m_ServerPool->ReturnServer(
                     m_DiscoveredServers->m_Servers.front().first)));
@@ -928,15 +915,15 @@ void CNetService::ExecOnAllServers(const string& cmd)
 
 void SNetServiceImpl::DiscoverServersIfNeeded()
 {
-    if (m_ServiceType == CNetService::eServiceNotDefined) {
+    if (m_ServiceType == eServiceNotDefined) {
         NCBI_THROW_FMT(CNetSrvConnException, eSrvListEmpty,
             m_APIName << ": service name is not set");
     }
 
-    if (m_ServiceType == CNetService::eLoadBalancedService) {
+    if (m_ServiceType == eLoadBalancedService) {
         // The service is load-balanced, check if rebalancing is required.
-        m_ServerPool->m_RebalanceStrategy->OnResourceRequested();
-        if (m_ServerPool->m_RebalanceStrategy->NeedRebalance())
+        m_RebalanceStrategy->OnResourceRequested();
+        if (m_RebalanceStrategy->NeedRebalance())
             ++m_LatestDiscoveryIteration;
 
         if (m_DiscoveredServers == NULL ||
@@ -1001,11 +988,8 @@ void SNetServiceImpl::DiscoverServersIfNeeded()
                 if (sinfo->time > 0 && sinfo->time != NCBI_TIME_INFINITE &&
                         sinfo->rate != 0.0) {
                     SNetServerInPool* server = m_ServerPool->
-                            FindOrCreateServerImpl(SServerAddress(sinfo->host, sinfo->port));
-                    {{
-                        CFastMutexGuard guard(server->m_ThrottleLock);
-                        server->m_DiscoveredAfterThrottling = true;
-                    }}
+                            FindOrCreateServerImpl(CNetServer::SAddress(sinfo->host, sinfo->port));
+                    server->m_ThrottleStats.Discover();
 
                     TServerRate server_rate(server, sinfo->rate);
 
@@ -1061,16 +1045,30 @@ bool SNetServiceImpl::IsInService(CNetServer::TInstance server)
     return false;
 }
 
+struct SFailOnlyWarnings : deque<pair<string, CNetServer>>
+{
+    SFailOnlyWarnings(CRef<INetServerConnectionListener> listener) : m_Listener(listener) {}
+    ~SFailOnlyWarnings() { IssueAndClear(); }
+
+    void IssueAndClear()
+    {
+        for (auto& w : *this) {
+            m_Listener->OnWarning(w.first, w.second);
+        }
+
+        clear();
+    }
+
+private:
+    CRef<INetServerConnectionListener> m_Listener;
+};
+
 void SNetServiceImpl::IterateUntilExecOK(const string& cmd,
     bool multiline_output,
     CNetServer::SExecResult& exec_result,
     IServiceTraversal* service_traversal,
-    SNetServiceImpl::EServerErrorHandling error_handling,
-    INetServerConnectionListener* conn_listener)
+    SNetServiceImpl::EServerErrorHandling error_handling)
 {
-    if (conn_listener == NULL)
-        conn_listener = m_Listener;
-
     int retry_count = m_ConnectionMaxRetries;
 
     const unsigned long retry_delay = m_ConnectionRetryDelay;
@@ -1097,14 +1095,15 @@ void SNetServiceImpl::IterateUntilExecOK(const string& cmd,
     STimeout* timeout = retry_count <= 0 && !m_UseSmartRetries ?
             NULL : &m_ServerPool->m_FirstServerTimeout;
 
-    CMessageListener_Basic err_listener;
+    SFailOnlyWarnings fail_only_warnings(m_Listener);
 
     for (;;) {
         skip_server = false;
 
         try {
             server->ConnectAndExec(cmd, multiline_output, exec_result,
-                    timeout, NULL, conn_listener);
+                    timeout);
+            fail_only_warnings.clear();
             return;
         }
         catch (CNetCacheBlobTooOldException& /*ex rethrown*/) {
@@ -1119,9 +1118,7 @@ void SNetServiceImpl::IterateUntilExecOK(const string& cmd,
             } else if (error_handling == eRethrowServerErrors)
                 throw;
             else
-                conn_listener->OnWarning(ex.GetMsg(), server);
-
-            err_listener.PostMessage(CMessage_Basic(ex.GetMsg(), eDiag_Warning));
+                m_Listener->OnWarning(ex.GetMsg(), server);
         }
         catch (CNetScheduleException& ex) {
             if (retry_count <= 0 && !m_UseSmartRetries)
@@ -1129,32 +1126,29 @@ void SNetServiceImpl::IterateUntilExecOK(const string& cmd,
             if (ex.GetErrCode() == CNetScheduleException::eSubmitsDisabled) {
                 ++ns_with_submits_disabled;
                 skip_server = true;
+                fail_only_warnings.emplace_back(ex.GetMsg(), server);
             } else if (error_handling == eRethrowServerErrors)
                 throw;
             else
-                conn_listener->OnWarning(ex.GetMsg(), server);
-
-            err_listener.PostMessage(CMessage_Basic(ex.GetMsg(), eDiag_Warning));
+                m_Listener->OnWarning(ex.GetMsg(), server);
         }
         catch (CNetSrvConnException& ex) {
             if (retry_count <= 0 && !m_UseSmartRetries)
                 throw;
             switch (ex.GetErrCode()) {
             case CNetSrvConnException::eReadTimeout:
-                break;
             case CNetSrvConnException::eConnectionFailure:
-                conn_listener->OnWarning(ex.GetMsg(), server);
+                m_Listener->OnWarning(ex.GetMsg(), server);
                 break;
 
             case CNetSrvConnException::eServerThrottle:
                 ++servers_throttled;
+                fail_only_warnings.emplace_back(ex.GetMsg(), server);
                 break;
 
             default:
                 throw;
             }
-
-            err_listener.PostMessage(CMessage_Basic(ex.GetMsg(), eDiag_Warning));
         }
 
         ++number_of_servers;
@@ -1187,13 +1181,15 @@ void SNetServiceImpl::IterateUntilExecOK(const string& cmd,
                 NCBI_THROW_FMT(CNetSrvConnException, eSrvListEmpty,
                         "Cannot execute ["  << cmd <<
                         "]: all NetSchedule servers are "
-                        "in REFUSESUBMITS mode.");
+                        "in REFUSESUBMITS mode for the " + m_ServiceName + " service.");
             }
+
             if (number_of_servers == servers_throttled) {
                 NCBI_THROW_FMT(CNetSrvConnException, eSrvListEmpty,
                         "Cannot execute ["  << cmd <<
-                        "]: all servers are throttled.");
+                        "]: all servers are throttled for the " + m_ServiceName + " service.");
             }
+
             if (retry_count <= 0 || servers_to_retry.empty()) {
                 if (blob_not_found) {
                     NCBI_THROW_FMT(CNetCacheException, eBlobNotFound,
@@ -1201,14 +1197,10 @@ void SNetServiceImpl::IterateUntilExecOK(const string& cmd,
                 }
                 NCBI_THROW_FMT(CNetSrvConnException, eSrvListEmpty,
                         "Unable to execute [" << cmd <<
-                        "] on any of the discovered servers.");
+                        "] on any of the discovered servers for the " + m_ServiceName + " service.");
             }
 
-            for (size_t i = 0; i < err_listener.Count(); ++i) {
-                LOG_POST(Warning << err_listener.GetMessage(i).GetText());
-            }
-
-            err_listener.Clear();
+            fail_only_warnings.IssueAndClear();
             LOG_POST(Warning << "Unable to send [" << cmd << "] to any "
                     "of the discovered servers; will retry after delay.");
 
@@ -1400,7 +1392,17 @@ CNetService CNetService::Clone(const string& name)
 {
     _ASSERT(m_Impl);
     return name == m_Impl->m_ServiceName ? m_Impl :
-        new SNetServiceImpl(name, m_Impl);
+        SNetServiceImpl::Clone(name, m_Impl);
+}
+
+void CNetService::SetErrorHandler(TEventHandler error_handler)
+{
+    m_Impl->m_Listener->SetErrorHandler(error_handler);
+}
+
+void CNetService::SetWarningHandler(TEventHandler warning_handler)
+{
+    m_Impl->m_Listener->SetWarningHandler(warning_handler);
 }
 
 CNetService SNetServiceMap::GetServiceByName(const string& service_name,
@@ -1418,7 +1420,7 @@ CNetService SNetServiceMap::GetServiceByNameImpl(const string& service_name,
 
     return !loc.second ? loc.first->second :
             (loc.first->second =
-                    new SNetServiceImpl(service_name, prototype));
+                    SNetServiceImpl::Clone(service_name, prototype));
 }
 
 bool SNetServiceMap::IsAllowed(const string& service_name) const
@@ -1450,10 +1452,9 @@ void SNetServiceMap::AddToAllowed(const string& service_name)
 }
 
 CJsonNode g_ExecToJson(IExecToJson& exec_to_json, CNetService service,
-        CNetService::EServiceType service_type,
         CNetService::EIterationMode iteration_mode)
 {
-    if (service_type == CNetService::eSingleServerService)
+    if (!service.IsLoadBalanced())
         return exec_to_json.ExecOn(service.Iterate().GetServer());
 
     CJsonNode result(CJsonNode::NewObjectNode());
@@ -1469,22 +1470,26 @@ CNetService g_DiscoverService(const string& service_name,
 {
     struct SNoOpConnectionListener : public INetServerConnectionListener
     {
-        CRef<INetServerProperties> AllocServerProperties() override
-        {
-            return CRef<INetServerProperties>(new INetServerProperties);
-        }
-
-        void OnInit(CObject*, ISynRegistry&, SRegSynonyms&) override {}
+        INetServerConnectionListener* Clone() override { return new SNoOpConnectionListener(*this); }
         void OnConnected(CNetServerConnection&) override {}
-        void OnError(const string&, CNetServer&) override {}
-        void OnWarning(const string&, CNetServer&) override {}
+
+    private:
+        void OnErrorImpl(const string&, CNetServer&) override {}
+        void OnWarningImpl(const string&, CNetServer&) override {}
     };
 
-    CNetService service(new SNetServiceImpl("Discovery", service_name, client_name, new SNoOpConnectionListener));
+    CSynRegistryBuilder registry_builder;
+    SRegSynonyms sections("discovery");
 
-    service->Init(nullptr, nullptr, "discovery");
+    return SNetServiceImpl::Create("Discovery", service_name, client_name, new SNoOpConnectionListener,
+            registry_builder, sections);
+}
 
-    return service;
+void g_AppendClientIPSessionIDHitID(string& cmd, bool use_next_sub_hit_id)
+{
+    CRequestContext& req = CDiagContext::GetRequestContext();
+    g_AppendClientIPAndSessionID(cmd, req);
+    g_AppendHitID(cmd, req, use_next_sub_hit_id);
 }
 
 END_NCBI_SCOPE
