@@ -1,7 +1,7 @@
 #ifndef CONNECT_SERVICES___NETSERVICE_API_IMPL__HPP
 #define CONNECT_SERVICES___NETSERVICE_API_IMPL__HPP
 
-/*  $Id: netservice_api_impl.hpp 537425 2017-05-31 17:34:36Z sadyrovr $
+/*  $Id: netservice_api_impl.hpp 573297 2018-10-25 16:49:18Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,6 +33,7 @@
 
 #include "srv_connections_impl.hpp"
 #include "balancing.hpp"
+#include <corelib/request_ctx.hpp>
 #include <connect/ncbi_connutil.h>
 
 #include <map>
@@ -48,7 +49,7 @@ BEGIN_NCBI_SCOPE
 
 typedef pair<SNetServerInPool*, double> TServerRate;
 typedef vector<TServerRate> TNetServerList;
-typedef map<SServerAddress, SNetServerInPool*> TNetServerByAddress;
+typedef map<CNetServer::SAddress, SNetServerInPool*> TNetServerByAddress;
 typedef map<string, CNetService> TNetServiceByName;
 
 struct SDiscoveredServers : public CObject
@@ -90,44 +91,25 @@ struct SDiscoveredServers : public CObject
 
 struct NCBI_XCONNECT_EXPORT SNetServerPoolImpl : public CObject
 {
-    struct SThrottleParams
-    {
-        // The parameters below describe different conditions that trigger server throttling.
-        int m_MaxConsecutiveIOFailures;
-
-        // Connection failure rate (numerator/denominator), which is when reached, triggers server throttling.
-        int m_IOFailureThresholdNumerator;
-        int m_IOFailureThresholdDenominator; // Cannot be greater than CONNECTION_ERROR_HISTORY_MAX
-
-        // How many seconds the API should wait before attempting to connect to a misbehaving server again.
-        // Throttling is off if period is less or equal to zero.
-        int m_ServerThrottlePeriod;
-
-        // Whether to check with LBSMD before re-enabling the server.
-        bool m_ThrottleUntilDiscoverable;
-
-        void Init(ISynRegistry& registry, const SRegSynonyms& sections);
-    };
-
     // Construct a new object.
-    SNetServerPoolImpl(INetServerConnectionListener* listener,
-            bool old_style_auth);
+    SNetServerPoolImpl(INetServerConnectionListener* listener);
 
-    void Init(ISynRegistry& registry, const SRegSynonyms& sections, INetServerConnectionListener* listener);
+    void Init(CSynRegistry& registry, const SRegSynonyms& sections);
 
-    SNetServerInPool* FindOrCreateServerImpl(const SServerAddress& server_address);
+    SNetServerInPool* FindOrCreateServerImpl(CNetServer::SAddress server_address);
     CRef<SNetServerInPool> ReturnServer(SNetServerInPool* server_impl);
+    CNetServer GetServer(SNetServiceImpl* service, CNetServer::SAddress server_address);
 
     void ResetServerConnections();
     const SThrottleParams& GetThrottleParams() const { return m_ThrottleParams; }
 
     virtual ~SNetServerPoolImpl();
 
-    CRef<INetServerConnectionListener> m_Listener;
+private:
+    INetServerConnectionListener::TPropCreator m_PropCreator;
 
-    SServerAddress m_EnforcedServer;
-
-    CRef<CSimpleRebalanceStrategy> m_RebalanceStrategy;
+public:
+    CNetServer::SAddress m_EnforcedServer;
 
     // LBSM affinity name and value
     pair<string, const char*> m_LBSMAffinity;
@@ -270,9 +252,9 @@ public:
 
 struct SNetServiceXSiteAPI : public CObject
 {
-    static void InitXSite(ISynRegistry& registry, const SRegSynonyms& sections);
+    static void InitXSite(CSynRegistry& registry, const SRegSynonyms& sections);
     static void ConnectXSite(CSocket&, SNetServerImpl::SConnectDeadline&,
-            const SServerAddress&);
+            const CNetServer::SAddress&, const string&);
 
 #ifdef NCBI_GRID_XSITE_CONN_SUPPORT
     static bool IsUsingXSiteProxy();
@@ -289,52 +271,36 @@ private:
 
 struct NCBI_XCONNECT_EXPORT SNetServiceImpl : SNetServiceXSiteAPI
 {
+    enum EServiceType {
+        eServiceNotDefined,
+        eLoadBalancedService,
+        eSingleServerService
+    };
+
     class CTry;
 
+    static SNetServiceImpl* Create(const string& api_name, const string& service_name, const string& client_name,
+            INetServerConnectionListener* listener,
+            CSynRegistry& registry, SRegSynonyms& sections, const string& ns_client_name = kEmptyStr);
+
+    static SNetServiceImpl* Clone(SNetServerInPool* server, SNetServiceImpl* prototype);
+    static SNetServiceImpl* Clone(const string& service_name, SNetServiceImpl* prototype);
+
+private:
     // Construct a new object.
     SNetServiceImpl(const string& api_name, const string& service_name, const string& client_name,
-            INetServerConnectionListener* listener, bool old_style_auth = false) :
-        m_Listener(listener),
-        m_ServerPool(new SNetServerPoolImpl(listener, old_style_auth)),
-        m_ServiceName(service_name),
-        m_APIName(api_name),
-        m_ClientName(client_name)
-    {
-    }
+            INetServerConnectionListener* listener);
 
     // Constructors for 'spawning'.
-    SNetServiceImpl(SNetServerInPool* server, SNetServiceImpl* prototype) :
-        m_Listener(prototype->m_Listener),
-        m_ServerPool(prototype->m_ServerPool),
-        m_ServiceName(server->m_Address.AsString()),
-        m_APIName(prototype->m_APIName),
-        m_ClientName(prototype->m_ClientName),
-        m_UseSmartRetries(prototype->m_UseSmartRetries),
-        m_ConnectionMaxRetries(prototype->m_ConnectionMaxRetries),
-        m_ConnectionRetryDelay(prototype->m_ConnectionRetryDelay),
-        m_NetInfo(prototype->m_NetInfo)
-    {
-        Construct(server);
-    }
-    SNetServiceImpl(const string& service_name, SNetServiceImpl* prototype) :
-        m_Listener(prototype->m_Listener),
-        m_ServerPool(prototype->m_ServerPool),
-        m_ServiceName(service_name),
-        m_APIName(prototype->m_APIName),
-        m_ClientName(prototype->m_ClientName),
-        m_UseSmartRetries(prototype->m_UseSmartRetries),
-        m_ConnectionMaxRetries(prototype->m_ConnectionMaxRetries),
-        m_ConnectionRetryDelay(prototype->m_ConnectionRetryDelay),
-        m_NetInfo(prototype->m_NetInfo)
-    {
-        Construct();
-    }
+    SNetServiceImpl(SNetServerInPool* server, SNetServiceImpl* prototype);
+    SNetServiceImpl(const string& service_name, SNetServiceImpl* prototype);
 
     void Construct(SNetServerInPool* server);
     void Construct();
 
-    void Init(CObject* api_impl, const IRegistry* top_registry, SRegSynonyms sections);
+    void Init(CSynRegistry& registry, SRegSynonyms& sections, const string& ns_client_name);
 
+public:
     string MakeAuthString();
 
     CNetServer::SExecResult FindServerAndExec(const string& cmd,
@@ -353,23 +319,16 @@ struct NCBI_XCONNECT_EXPORT SNetServiceImpl : SNetServiceXSiteAPI
         bool multiline_output,
         CNetServer::SExecResult& exec_result,
         IServiceTraversal* service_traversal,
-        EServerErrorHandling error_handling,
-        INetServerConnectionListener* conn_listener);
+        EServerErrorHandling error_handling);
 
     SDiscoveredServers* AllocServerGroup(unsigned discovery_iteration);
-    CNetServer GetServer(const SServerAddress& server_address);
+    CNetServer GetServer(CNetServer::SAddress server_address);
 
     const string& GetClientName() const { return m_ClientName; }
 
-    CRef<INetEventHandler> SetEventHandler(INetEventHandler* new_handler)
-    {
-        CRef<INetEventHandler> old_handler(m_Listener->m_EventHandler.ReleaseOrNull());
-        m_Listener->m_EventHandler.Reset(new_handler);
-        return old_handler;
-    }
-
     unsigned GetConnectionMaxRetries() const { return m_ConnectionMaxRetries; }
     unsigned long GetConnectionRetryDelay() const { return m_ConnectionRetryDelay; }
+    bool IsLoadBalanced() const { return m_ServiceType == eLoadBalancedService; }
     shared_ptr<CTry> GetTryGuard();
 
     virtual ~SNetServiceImpl();
@@ -381,12 +340,13 @@ struct NCBI_XCONNECT_EXPORT SNetServiceImpl : SNetServiceXSiteAPI
     CNetServerPool m_ServerPool;
 
     string m_ServiceName;
-    CNetService::EServiceType m_ServiceType = CNetService::eServiceNotDefined;
+    EServiceType m_ServiceType = eServiceNotDefined;
 
     CFastMutex m_DiscoveryMutex;
     SDiscoveredServers* m_DiscoveredServers = nullptr;
     SDiscoveredServers* m_ServerGroupPool = nullptr;
     unsigned m_LatestDiscoveryIteration = 0;
+    CRef<CSimpleRebalanceStrategy> m_RebalanceStrategy;
 
 private:
     string m_APIName;
@@ -447,6 +407,29 @@ private:
     bool m_Allowed = true;
 };
 
+inline
+void g_AppendHitID(string& cmd, CRequestContext& req, bool use_next_sub_hit_id)
+{
+    auto sub_hit_id = use_next_sub_hit_id ? req.GetNextSubHitID() : req.GetCurrentSubHitID();
+    cmd += " ncbi_phid=\"";
+    cmd += sub_hit_id;
+    cmd += '"';
+}
+
+inline
+void g_AppendClientIPAndSessionID(string& cmd, const CRequestContext& req)
+{
+    if (req.IsSetClientIP()) {
+        cmd += " ip=\"";
+        cmd += req.GetClientIP();
+        cmd += '"';
+    }
+
+    cmd += " sid=\"";
+    cmd += req.GetSessionID();
+    cmd += '"';
+}
+
 END_NCBI_SCOPE
 
 #endif  /* CONNECT_SERVICES___NETSERVICE_API_IMPL__HPP */
