@@ -1,4 +1,4 @@
-/*  $Id: netstorage_rpc.cpp 533663 2017-04-18 19:29:12Z sadyrovr $
+/*  $Id: netstorage_rpc.cpp 555398 2018-01-17 20:27:02Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -433,25 +433,21 @@ public:
     {
     }
 
-    CRef<INetServerProperties> AllocServerProperties() override;
+    INetServerConnectionListener* Clone() override;
 
-    void OnInit(CObject* api_impl, ISynRegistry& registry, SRegSynonyms& sections) override;
     void OnConnected(CNetServerConnection& connection) override;
-    void OnError(const string& err_msg, CNetServer& server) override;
-    void OnWarning(const string& warn_msg, CNetServer& server) override;
  
 private:
+    void OnErrorImpl(const string& err_msg, CNetServer& server) override;
+    void OnWarningImpl(const string& warn_msg, CNetServer& server) override;
+
     const CJsonNode m_Hello;
     const SNetStorage::SConfig::EErrMode m_ErrMode;
 };
 
-CRef<INetServerProperties> CNetStorageServerListener::AllocServerProperties()
-{
-    return CRef<INetServerProperties>(new INetServerProperties);
-}
-
-void CNetStorageServerListener::OnInit(CObject*, ISynRegistry&, SRegSynonyms&)
+INetServerConnectionListener* CNetStorageServerListener::Clone()
 {
+    return new CNetStorageServerListener(*this);
 }
 
 void CNetStorageServerListener::OnConnected(
@@ -464,7 +460,7 @@ void CNetStorageServerListener::OnConnected(
     s_ReadMessage(m_Hello, connection, m_ErrMode, *this);
 }
 
-void CNetStorageServerListener::OnError(const string& err_msg,
+void CNetStorageServerListener::OnErrorImpl(const string& err_msg,
         CNetServer& server)
 {
     LOG_POST(Error << "NetStorage server " <<
@@ -472,16 +468,12 @@ void CNetStorageServerListener::OnError(const string& err_msg,
             " issued error " << err_msg);
 }
 
-void CNetStorageServerListener::OnWarning(const string& warn_msg,
+void CNetStorageServerListener::OnWarningImpl(const string& warn_msg,
         CNetServer& server)
 {
-    if (m_EventHandler)
-        m_EventHandler->OnWarning(warn_msg, server);
-    else {
         LOG_POST(Warning << "NetStorage server " <<
                 server->m_ServerInPool->m_Address.AsString() <<
                 " issued warning " << warn_msg);
-    }
 }
 
 class CJsonOverUTTPExecHandler : public INetServerExecHandler
@@ -786,16 +778,17 @@ SNetStorageRPC::SNetStorageRPC(const TConfig& config,
     hello.SetString("ProtocolVersion", NST_PROTOCOL_VERSION);
     if (!m_Config.ticket.empty()) hello.SetString("Ticket", m_Config.ticket);
 
-    m_Service = new SNetServiceImpl("NetStorageAPI", m_Config.service, m_Config.client_name,
-            new CNetStorageServerListener(hello, m_Config.err_mode));
-
-    m_Service->Init(this, nullptr, "netstorage_api");
+    CSynRegistryBuilder registry_builder;
+    SRegSynonyms sections("netstorage_api");
+    m_Service = SNetServiceImpl::Create("NetStorageAPI", m_Config.service, m_Config.client_name,
+            new CNetStorageServerListener(hello, m_Config.err_mode),
+            registry_builder, sections);
 }
 
 SNetStorageRPC::SNetStorageRPC(SNetServerInPool* server,
         SNetStorageRPC* parent) :
     m_DefaultFlags(parent->m_DefaultFlags),
-    m_Service(new SNetServiceImpl(server, parent->m_Service)),
+    m_Service(SNetServiceImpl::Clone(server, parent->m_Service)),
     m_Config(parent->m_Config),
     m_CompoundIDPool(parent->m_CompoundIDPool),
     m_NetCacheAPI(parent->m_NetCacheAPI),
@@ -1407,11 +1400,6 @@ CNetStorageAdmin CNetStorageAdmin::GetServer(CNetServer::TInstance server)
     return new SNetStorageRPC(server->m_ServerInPool, m_Impl->m_NetStorageRPC);
 }
 
-void CNetStorageAdmin::SetEventHandler(INetEventHandler* event_handler)
-{
-    m_Impl->m_NetStorageRPC->m_Service->SetEventHandler(event_handler);
-}
-
 CNetStorageObject CNetStorageAdmin::Open(const string& object_loc)
 {
     return m_Impl->m_NetStorageRPC->Open(object_loc);
