@@ -1,4 +1,4 @@
-/*  $Id: srv_connections.cpp 540944 2017-07-12 15:49:36Z ivanov $
+/*  $Id: srv_connections.cpp 573309 2018-10-25 16:52:53Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -38,6 +38,8 @@
 
 #include <corelib/ncbi_system.hpp>
 
+#include <sstream>
+
 #ifdef NCBI_OS_LINUX
 # include <sys/socket.h>
 # include <netinet/in.h>
@@ -60,25 +62,25 @@ SNetServerMultilineCmdOutputImpl::~SNetServerMultilineCmdOutputImpl()
         m_Connection->Close();
 }
 
-bool CNetServerMultilineCmdOutput::ReadLine(string& output)
+bool SNetServerMultilineCmdOutputImpl::ReadLine(string& output)
 {
-    _ASSERT(!m_Impl->m_ReadCompletely);
+    _ASSERT(!m_ReadCompletely);
 
-    if (!m_Impl->m_FirstLineConsumed) {
-        output = m_Impl->m_FirstOutputLine;
-        m_Impl->m_FirstOutputLine = kEmptyStr;
-        m_Impl->m_FirstLineConsumed = true;
-    } else if (!m_Impl->m_NetCacheCompatMode)
-        m_Impl->m_Connection->ReadCmdOutputLine(output, true);
+    if (!m_FirstLineConsumed) {
+        output = m_FirstOutputLine;
+        m_FirstOutputLine = kEmptyStr;
+        m_FirstLineConsumed = true;
+    } else if (!m_NetCacheCompatMode)
+        m_Connection->ReadCmdOutputLine(output, true);
     else {
         try {
-            m_Impl->m_Connection->ReadCmdOutputLine(output, true);
+            m_Connection->ReadCmdOutputLine(output, true);
         }
         catch (CNetSrvConnException& e) {
             if (e.GetErrCode() != CNetSrvConnException::eConnClosedByServer)
                 throw;
 
-            m_Impl->m_ReadCompletely = true;
+            m_ReadCompletely = true;
             return false;
         }
     }
@@ -86,13 +88,58 @@ bool CNetServerMultilineCmdOutput::ReadLine(string& output)
     if (output != END_OF_MULTILINE_OUTPUT)
         return true;
     else {
-        m_Impl->m_ReadCompletely = true;
+        m_ReadCompletely = true;
         return false;
     }
 }
 
-void INetServerConnectionListener::OnPreInit(CObject*, ISynRegistry&, SRegSynonyms&, string&)
+bool CNetServerMultilineCmdOutput::ReadLine(string& output)
 {
+    return m_Impl->ReadLine(output);
+}
+
+INetServerConnectionListener::TPropCreator INetServerConnectionListener::GetPropCreator() const
+{
+    return [] { return new INetServerProperties; };
+}
+
+void INetServerConnectionListener::OnError(const string& err_msg, CNetServer& server)
+{
+    if (m_ErrorHandler && m_ErrorHandler(err_msg, server)) return;
+
+    OnErrorImpl(err_msg, server);
+}
+
+void INetServerConnectionListener::OnWarning(const string& warn_msg, CNetServer& server)
+{
+    if (m_WarningHandler && m_WarningHandler(warn_msg, server)) return;
+
+    OnWarningImpl(warn_msg, server);
+}
+
+INetServerConnectionListener::INetServerConnectionListener(const INetServerConnectionListener&)
+{
+    // No m_ErrorHandler/m_WarningHandler sharing
+}
+
+INetServerConnectionListener& INetServerConnectionListener::operator=(const INetServerConnectionListener&)
+{
+    // No m_ErrorHandler/m_WarningHandler sharing
+    return *this;
+}
+
+void INetServerConnectionListener::SetErrorHandler(TEventHandler error_handler)
+{
+    // Event handlers are not allowed to be changed, only to be set or reset
+    _ASSERT(!(m_ErrorHandler && error_handler));
+    m_ErrorHandler = error_handler;
+}
+
+void INetServerConnectionListener::SetWarningHandler(TEventHandler warning_handler)
+{
+    // Event handlers are not allowed to be changed, only to be set or reset
+    _ASSERT(!(m_WarningHandler && warning_handler));
+    m_WarningHandler = warning_handler;
 }
 
 inline SNetServerConnectionImpl::SNetServerConnectionImpl(
@@ -134,8 +181,7 @@ void SNetServerConnectionImpl::DeleteThis()
 #define WARNING_PREFIX_LEN STRING_LEN(WARNING_PREFIX)
 
 void SNetServerConnectionImpl::ReadCmdOutputLine(string& result,
-        bool multiline_output,
-        INetServerConnectionListener* conn_listener)
+        bool multiline_output)
 {
     switch (m_Socket.ReadLine(result))
     {
@@ -159,6 +205,8 @@ void SNetServerConnectionImpl::ReadCmdOutputLine(string& result,
                 "Communication error while reading");
     }
 
+    auto& conn_listener = m_Server->m_Service->m_Listener;
+
     if (NStr::StartsWith(result, "OK:")) {
         const char* reply = result.c_str() + STRING_LEN("OK:");
         size_t reply_len = result.length() - STRING_LEN("OK:");
@@ -167,8 +215,6 @@ void SNetServerConnectionImpl::ReadCmdOutputLine(string& result,
             reply += WARNING_PREFIX_LEN;
             reply_len -= WARNING_PREFIX_LEN;
             const char* semicolon = strchr(reply, ';');
-            if (conn_listener == NULL)
-                conn_listener = m_Server->m_Service->m_Listener;
             if (semicolon == NULL) {
                 conn_listener->OnWarning(string(reply, reply + reply_len),
                         m_Server);
@@ -183,8 +229,7 @@ void SNetServerConnectionImpl::ReadCmdOutputLine(string& result,
     } else if (NStr::StartsWith(result, "ERR:")) {
         result.erase(0, STRING_LEN("ERR:"));
         result = NStr::ParseEscapes(result);
-        (conn_listener != NULL ? conn_listener :
-                m_Server->m_Service->m_Listener)->OnError(result, m_Server);
+        conn_listener->OnError(result, m_Server);
         result = multiline_output ? string(END_OF_MULTILINE_OUTPUT) : kEmptyStr;
     }
 }
@@ -231,8 +276,7 @@ void SNetServerConnectionImpl::WriteLine(const string& line)
 
 string CNetServerConnection::Exec(const string& cmd,
         bool multiline_output,
-        STimeout* timeout,
-        INetServerConnectionListener* conn_listener)
+        STimeout* timeout)
 {
     CTimeoutKeeper timeout_keeper(&m_Impl->m_Socket, timeout);
 
@@ -247,7 +291,7 @@ string CNetServerConnection::Exec(const string& cmd,
 
     string output;
 
-    m_Impl->ReadCmdOutputLine(output, multiline_output, conn_listener);
+    m_Impl->ReadCmdOutputLine(output, multiline_output);
 
     return output;
 }
@@ -259,23 +303,22 @@ CNetServerMultilineCmdOutput::CNetServerMultilineCmdOutput(
 }
 
 /*************************************************************************/
-SNetServerInPool::SNetServerInPool(unsigned host, unsigned short port,
-        INetServerProperties* server_properties) :
-    m_Address(host, port),
-    m_ServerProperties(server_properties)
+SNetServerInPool::SNetServerInPool(CNetServer::SAddress address,
+        INetServerProperties* server_properties, SThrottleParams throttle_params) :
+    m_Address(move(address)),
+    m_ServerProperties(server_properties),
+    m_ThrottleStats(move(throttle_params))
 {
     m_CurrentConnectionGeneration.Set(0);
 
     m_FreeConnectionListHead = NULL;
     m_FreeConnectionListSize = 0;
 
-    ResetThrottlingParameters();
-
     m_RankBase = 1103515245 *
             // XOR the network prefix bytes of the IP address with the port
             // number (in network byte order) and convert the result
             // to host order so that it can be used in arithmetic operations.
-            CSocketAPI::NetToHostLong(host ^ CSocketAPI::HostToNetShort(port)) +
+            CSocketAPI::NetToHostLong(m_Address.host ^ CSocketAPI::HostToNetShort(m_Address.port)) +
             12345;
 }
 
@@ -373,53 +416,49 @@ SNetServerInfoImpl::SNetServerInfoImpl(const string& version_string)
     m_NextAttribute = m_Attributes->begin();
 }
 
-bool CNetServerInfo::GetNextAttribute(string& attr_name, string& attr_value)
+bool SNetServerInfoImpl::GetNextAttribute(string& attr_name, string& attr_value)
 {
-    if (m_Impl->m_NextAttribute == m_Impl->m_Attributes->end())
+    if (m_NextAttribute == m_Attributes->end())
         return false;
 
-    attr_name = m_Impl->m_NextAttribute->name;
-    attr_value = m_Impl->m_NextAttribute->value;
-    ++m_Impl->m_NextAttribute;
+    attr_name = m_NextAttribute->name;
+    attr_value = m_NextAttribute->value;
+    ++m_NextAttribute;
     return true;
 }
 
-unsigned CNetServer::GetHost() const
+bool CNetServerInfo::GetNextAttribute(string& attr_name, string& attr_value)
 {
-    return m_Impl->m_ServerInPool->m_Address.host;
+    return m_Impl->GetNextAttribute(attr_name, attr_value);
 }
 
-unsigned short CNetServer::GetPort() const
+const CNetServer::SAddress& CNetServer::GetAddress() const
 {
-    return m_Impl->m_ServerInPool->m_Address.port;
+    return m_Impl->m_ServerInPool->m_Address;
 }
 
-string CNetServer::GetServerAddress() const
-{
-    return m_Impl->m_ServerInPool->m_Address.AsString();
-}
 
-CNetServerConnection SNetServerImpl::GetConnectionFromPool()
+CNetServerConnection SNetServerInPool::GetConnectionFromPool(SNetServerImpl* server)
 {
     for (;;) {
-        TFastMutexGuard guard(m_ServerInPool->m_FreeConnectionListLock);
+        TFastMutexGuard guard(m_FreeConnectionListLock);
 
-        if (m_ServerInPool->m_FreeConnectionListSize == 0)
+        if (m_FreeConnectionListSize == 0)
             return NULL;
 
         // Get an existing connection object from the connection pool.
-        CNetServerConnection conn = m_ServerInPool->m_FreeConnectionListHead;
+        CNetServerConnection conn = m_FreeConnectionListHead;
 
-        m_ServerInPool->m_FreeConnectionListHead = conn->m_NextFree;
-        --m_ServerInPool->m_FreeConnectionListSize;
-        conn->m_Server = this;
+        m_FreeConnectionListHead = conn->m_NextFree;
+        --m_FreeConnectionListSize;
+        conn->m_Server = server;
 
         guard.Release();
 
         // Check if the socket is already connected.
         if (conn->m_Socket.GetStatus(eIO_Open) != eIO_Success ||
                 conn->m_Generation !=
-                        m_ServerInPool->m_CurrentConnectionGeneration.Get())
+                        m_CurrentConnectionGeneration.Get())
             continue;
 
         SSOCK_Poll conn_socket_poll_struct = {
@@ -440,7 +479,8 @@ struct SNetServerImpl::SConnectDeadline
 {
     SConnectDeadline(const STimeout& conn_timeout) :
         try_timeout(Min(conn_timeout , kMaxTryTimeout)),
-        deadline(CTimeout(conn_timeout.sec, conn_timeout.usec))
+        total_timeout(conn_timeout.sec, conn_timeout.usec),
+        deadline(total_timeout)
     {}
 
     const STimeout* GetRemaining() const { return &try_timeout; }
@@ -456,6 +496,8 @@ struct SNetServerImpl::SConnectDeadline
         return false;
     }
 
+    CTimeout GetTotal() const { return total_timeout; }
+
 private:
     static STimeout Min(const STimeout& t1, const STimeout& t2)
     {
@@ -466,6 +508,7 @@ private:
     }
 
     STimeout try_timeout;
+    CTimeout total_timeout;
     CDeadline deadline;
 
     static const STimeout kMaxTryTimeout;
@@ -477,36 +520,30 @@ const STimeout SNetServerImpl::SConnectDeadline::kMaxTryTimeout = {0, 250 * 1000
 const STimeout SNetServerImpl::SConnectDeadline::kMaxTryTimeout = {1, 250 * 1000};
 #endif
 
-CNetServerConnection SNetServerImpl::Connect(STimeout* timeout,
-        INetServerConnectionListener* conn_listener)
+CNetServerConnection SNetServerInPool::Connect(SNetServerImpl* server, STimeout* timeout)
 {
-    CNetServerConnection conn = new SNetServerConnectionImpl(this);
+    CNetServerConnection conn = new SNetServerConnectionImpl(server);
 
-    auto& server_pool = m_ServerInPool->m_ServerPool;
-    SConnectDeadline deadline(timeout ? *timeout : server_pool->m_ConnTimeout);
+    SNetServerImpl::SConnectDeadline deadline(timeout ? *timeout : m_ServerPool->m_ConnTimeout);
     auto& socket = conn->m_Socket;
 
-    SNetServiceXSiteAPI::ConnectXSite(socket, deadline, m_ServerInPool->m_Address);
-
-    m_ServerInPool->AdjustThrottlingParameters(SNetServerInPool::eCOR_Success);
+    SNetServiceXSiteAPI::ConnectXSite(socket, deadline, m_Address, server->m_Service->m_ServiceName);
 
     socket.SetDataLogging(TServConn_ConnDataLogging::GetDefault() ? eOn : eOff);
     socket.SetTimeout(eIO_ReadWrite, timeout ? timeout :
-                              &server_pool->m_CommTimeout);
+                              &m_ServerPool->m_CommTimeout);
     socket.DisableOSSendDelay();
     socket.SetReuseAddress(eOn);
 
-    if (!conn_listener) conn_listener = m_Service->m_Listener;
+    server->m_Service->m_Listener->OnConnected(conn);
 
-    conn_listener->OnConnected(conn);
-
-    if (timeout) socket.SetTimeout(eIO_ReadWrite, &server_pool->m_CommTimeout);
+    if (timeout) socket.SetTimeout(eIO_ReadWrite, &m_ServerPool->m_CommTimeout);
 
     return conn;
 }
 
 void SNetServerImpl::ConnectImpl(CSocket& socket, SConnectDeadline& deadline,
-        const SServerAddress& actual, const SServerAddress& original)
+        const CNetServer::SAddress& actual, const CNetServer::SAddress& original)
 {
     EIO_Status io_st;
 
@@ -520,22 +557,23 @@ void SNetServerImpl::ConnectImpl(CSocket& socket, SConnectDeadline& deadline,
 
     socket.Close();
 
-    NCBI_THROW(CNetSrvConnException, eConnectionFailure,
-        FORMAT(original.AsString() <<
-            ": Could not connect: " << IO_StatusStr(io_st)));
+    ostringstream os;
+    os << original.AsString() << ": Could not connect: " << IO_StatusStr(io_st);
+
+    if (io_st == eIO_Timeout) os << " (" << deadline.GetTotal().GetAsDouble() << "s)";
+
+    NCBI_THROW(CNetSrvConnException, eConnectionFailure, os.str());
 }
 
-void SNetServerImpl::TryExec(INetServerExecHandler& handler,
-        STimeout* timeout, INetServerConnectionListener* conn_listener)
+void SNetServerInPool::TryExec(SNetServerImpl* server, INetServerExecHandler& handler,
+        STimeout* timeout)
 {
-    m_ServerInPool->CheckIfThrottled();
-
     CNetServerConnection conn;
 
     // Silently reconnect if the connection was taken
     // from the pool and it was closed by the server
     // due to inactivity.
-    while ((conn = GetConnectionFromPool()) != NULL) {
+    while ((conn = GetConnectionFromPool(server)) != NULL) {
         try {
             handler.Exec(conn, timeout);
             return;
@@ -545,21 +583,30 @@ void SNetServerImpl::TryExec(INetServerExecHandler& handler,
             if (err_code != CNetSrvConnException::eWriteFailure &&
                 err_code != CNetSrvConnException::eConnClosedByServer)
             {
-                m_ServerInPool->AdjustThrottlingParameters(
-                        SNetServerInPool::eCOR_Failure);
                 throw;
             }
         }
     }
 
+    handler.Exec(Connect(server, timeout), timeout);
+}
+
+void SNetServerImpl::TryExec(INetServerExecHandler& handler, STimeout* timeout)
+{
+    auto& server_in_pool = *m_ServerInPool;
+    auto& throttle_stats = server_in_pool.m_ThrottleStats;
+
+    throttle_stats.Check(this);
+
     try {
-        handler.Exec(Connect(timeout, conn_listener), timeout);
+        server_in_pool.TryExec(this, handler, timeout);
     }
-    catch (CNetSrvConnException&) {
-        m_ServerInPool->AdjustThrottlingParameters(
-                SNetServerInPool::eCOR_Failure);
+    catch (CNetSrvConnException& e) {
+        throttle_stats.Adjust(this, e.GetErrCode());
         throw;
     }
+
+    throttle_stats.Adjust(this, -1); // Success
 }
 
 class CNetServerExecHandler : public INetServerExecHandler
@@ -568,13 +615,11 @@ public:
     CNetServerExecHandler(const string& cmd,
             bool multiline_output,
             CNetServer::SExecResult& exec_result,
-            INetServerExecListener* exec_listener,
-            INetServerConnectionListener* conn_listener) :
+            INetServerExecListener* exec_listener) :
         m_Cmd(cmd),
         m_MultilineOutput(multiline_output),
         m_ExecResult(exec_result),
-        m_ExecListener(exec_listener),
-        m_ConnListener(conn_listener)
+        m_ExecListener(exec_listener)
     {
     }
 
@@ -585,7 +630,6 @@ public:
     bool m_MultilineOutput;
     CNetServer::SExecResult& m_ExecResult;
     INetServerExecListener* m_ExecListener;
-    INetServerConnectionListener* m_ConnListener;
 };
 
 void CNetServerExecHandler::Exec(CNetServerConnection::TInstance conn_impl,
@@ -598,108 +642,119 @@ void CNetServerExecHandler::Exec(CNetServerConnection::TInstance conn_impl,
 
     m_ExecResult.response = m_ExecResult.conn.Exec(m_Cmd,
             m_MultilineOutput,
-            timeout, m_ConnListener);
+            timeout);
 }
 
 void SNetServerImpl::ConnectAndExec(const string& cmd,
         bool multiline_output,
         CNetServer::SExecResult& exec_result, STimeout* timeout,
-        INetServerExecListener* exec_listener,
-        INetServerConnectionListener* conn_listener)
+        INetServerExecListener* exec_listener)
 {
-    if (conn_listener == NULL)
-        conn_listener = m_Service->m_Listener;
-
     CNetServerExecHandler exec_handler(cmd, multiline_output,
-            exec_result, exec_listener, conn_listener);
+            exec_result, exec_listener);
 
-    TryExec(exec_handler, timeout, conn_listener);
+    TryExec(exec_handler, timeout);
 }
 
-void SNetServerInPool::AdjustThrottlingParameters(EConnOpResult op_result)
+void SThrottleStats::Adjust(SNetServerImpl* server_impl, int err_code)
 {
-    const auto& params = m_ServerPool->GetThrottleParams();
+    _ASSERT(server_impl);
 
-    if (params.m_ServerThrottlePeriod <= 0)
+    if (m_Params.throttle_period <= 0)
         return;
 
-    CFastMutexGuard guard(m_ThrottleLock);
+    const auto op_result = err_code >= 0; // True, if failure
 
-    if (params.m_MaxConsecutiveIOFailures > 0) {
-        if (op_result != eCOR_Success)
-            ++m_NumberOfConsecutiveIOFailures;
-        else if (m_NumberOfConsecutiveIOFailures < params.m_MaxConsecutiveIOFailures)
+    if (op_result && m_Params.connect_failures_only &&
+            (err_code != CNetSrvConnException::eConnectionFailure)) return;
+
+    CFastMutexGuard guard(m_ThrottleLock);
+    const auto& address = server_impl->m_ServerInPool->m_Address;
+
+    if (m_Params.max_consecutive_io_failures > 0) {
+        if (!op_result)
             m_NumberOfConsecutiveIOFailures = 0;
+
+        else if (++m_NumberOfConsecutiveIOFailures >= m_Params.max_consecutive_io_failures) {
+            m_Throttled = true;
+            m_ThrottleMessage = "Server " + address.AsString() +
+                " reached the maximum number of connection failures in a row";
+        }
     }
 
-    if (params.m_IOFailureThresholdNumerator > 0) {
-        if (m_IOFailureRegister[m_IOFailureRegisterIndex] != op_result) {
-            if ((m_IOFailureRegister[m_IOFailureRegisterIndex] =
-                    op_result) == eCOR_Success)
-                --m_IOFailureCounter;
-            else
-                ++m_IOFailureCounter;
+    if (m_Params.io_failure_threshold.numerator > 0) {
+        auto& reg = m_IOFailureRegister.first;
+        auto& index = m_IOFailureRegister.second;
+
+        if (reg[index] != op_result) {
+            reg[index] = op_result;
+
+            if (op_result && (reg.count() >= m_Params.io_failure_threshold.numerator)) {
+                m_Throttled = true;
+                m_ThrottleMessage = "Connection to server " + address.AsString() +
+                    " aborted as it was considered bad/overloaded";
+            }
         }
 
-        if (++m_IOFailureRegisterIndex >= params.m_IOFailureThresholdDenominator)
-            m_IOFailureRegisterIndex = 0;
+        if (++index >= m_Params.io_failure_threshold.denominator) index = 0;
+    }
+
+    if (m_Throttled) {
+        m_DiscoveredAfterThrottling = false;
+        m_ThrottledUntil.SetCurrent();
+        CNetServer server(server_impl);
+        server_impl->m_Service->m_Listener->OnWarning(m_ThrottleMessage, server);
+        m_ThrottleMessage += " on " + m_ThrottledUntil.AsString();
+        m_ThrottledUntil.AddSecond(m_Params.throttle_period);
     }
 }
 
-void SNetServerInPool::CheckIfThrottled()
+void SThrottleStats::Check(SNetServerImpl* server_impl)
 {
-    const auto& params = m_ServerPool->GetThrottleParams();
+    _ASSERT(server_impl);
 
-    if (params.m_ServerThrottlePeriod <= 0)
+    if (m_Params.throttle_period <= 0)
         return;
 
     CFastMutexGuard guard(m_ThrottleLock);
 
     if (m_Throttled) {
         CTime current_time(GetFastLocalTime());
-        if (current_time >= m_ThrottledUntil &&
-                (!params.m_ThrottleUntilDiscoverable || m_DiscoveredAfterThrottling)) {
-            ResetThrottlingParameters();
+        auto duration = current_time - m_ThrottledUntil;
+
+        if ((duration >= CTimeSpan(0, 0)) &&
+                (!m_Params.throttle_until_discoverable || m_DiscoveredAfterThrottling)) {
+            duration += CTimeSpan(m_Params.throttle_period, 0);
+            Reset();
+            const auto& address = server_impl->m_ServerInPool->m_Address;
+            ostringstream os;
+            os << "Disabling throttling for server " << address.AsString() <<
+                    " before new attempt after " << duration.AsString() << " seconds wait" << 
+                    (m_Params.throttle_until_discoverable ? " and rediscovery" : "");
+            CNetServer server(server_impl);
+            server_impl->m_Service->m_Listener->OnWarning(os.str(), server);
             return;
         }
         NCBI_THROW(CNetSrvConnException, eServerThrottle, m_ThrottleMessage);
     }
-
-    if (params.m_MaxConsecutiveIOFailures > 0 &&
-        m_NumberOfConsecutiveIOFailures >= params.m_MaxConsecutiveIOFailures) {
-        m_Throttled = true;
-        m_DiscoveredAfterThrottling = false;
-        m_ThrottleMessage = "Server " + m_Address.AsString();
-        m_ThrottleMessage += " has reached the maximum number "
-            "of connection failures in a row";
-    }
-
-    if (params.m_IOFailureThresholdNumerator > 0 &&
-            m_IOFailureCounter >= params.m_IOFailureThresholdNumerator) {
-        m_Throttled = true;
-        m_DiscoveredAfterThrottling = false;
-        m_ThrottleMessage = "Connection to server " + m_Address.AsString();
-        m_ThrottleMessage += " aborted as it is considered bad/overloaded";
-    }
-
-    if (m_Throttled) {
-        m_ThrottledUntil.SetCurrent();
-        m_ThrottledUntil.AddSecond(params.m_ServerThrottlePeriod);
-        NCBI_THROW(CNetSrvConnException, eServerThrottle, m_ThrottleMessage);
-    }
 }
 
-void SNetServerInPool::ResetThrottlingParameters()
+void SThrottleStats::Reset()
 {
     m_NumberOfConsecutiveIOFailures = 0;
-    memset(m_IOFailureRegister, 0, sizeof(m_IOFailureRegister));
-    m_IOFailureCounter = m_IOFailureRegisterIndex = 0;
+    m_IOFailureRegister.first.reset();
+    m_IOFailureRegister.second = 0;
     m_Throttled = false;
 }
 
+void SThrottleStats::Discover()
+{
+    CFastMutexGuard guard(m_ThrottleLock);
+    m_DiscoveredAfterThrottling = true;
+}
+
 CNetServer::SExecResult CNetServer::ExecWithRetry(const string& cmd,
-        bool multiline_output,
-        INetServerConnectionListener* conn_listener)
+        bool multiline_output)
 {
     CNetServer::SExecResult exec_result;
 
@@ -709,9 +764,6 @@ CNetServer::SExecResult CNetServer::ExecWithRetry(const string& cmd,
     unsigned attempt = 0;
     auto& service = m_Impl->m_Service;
 
-    if (conn_listener == NULL)
-        conn_listener = service->m_Listener;
-
     const auto max_retries = service->GetConnectionMaxRetries();
     const auto retry_delay = service->GetConnectionRetryDelay();
 
@@ -720,7 +772,7 @@ CNetServer::SExecResult CNetServer::ExecWithRetry(const string& cmd,
 
         try {
             m_Impl->ConnectAndExec(cmd, multiline_output,
-                    exec_result, NULL, NULL, conn_listener);
+                    exec_result);
             return exec_result;
         }
         catch (CNetSrvConnException& e) {
@@ -755,7 +807,7 @@ CNetServer::SExecResult CNetServer::ExecWithRetry(const string& cmd,
         warning += " of ";
         warning += NStr::NumericToString(max_retries);
 
-        conn_listener->OnWarning(warning, *this);
+        service->m_Listener->OnWarning(warning, *this);
 
         SleepMilliSec(retry_delay);
     }
@@ -763,7 +815,9 @@ CNetServer::SExecResult CNetServer::ExecWithRetry(const string& cmd,
 
 CNetServerInfo CNetServer::GetServerInfo()
 {
-    string response(ExecWithRetry("VERSION", false).response);
+    string cmd("VERSION");
+    g_AppendClientIPSessionIDHitID(cmd);
+    auto response = ExecWithRetry(cmd, false).response;
 
     // Keep these two lines separate to let the Exec method
     // above succeed before constructing the object below.
@@ -776,4 +830,81 @@ CNetServerInfo g_ServerInfoFromString(const string& server_info)
     return new SNetServerInfoImpl(server_info);
 }
 
+void SThrottleParams::Init(CSynRegistry& registry, const SRegSynonyms& sections)
+{
+    throttle_period = registry.Get(sections, "throttle_relaxation_period", 0);
+
+    if (throttle_period <= 0) return;
+
+    max_consecutive_io_failures = registry.Get(sections,
+            { "throttle_by_consecutive_connection_failures", "throttle_by_subsequent_connection_failures" }, 0);
+
+    throttle_until_discoverable = registry.Get(sections, "throttle_hold_until_active_in_lb", false);
+    connect_failures_only = registry.Get(sections, "throttle_connect_failures_only", false);
+
+    io_failure_threshold.Init(registry, sections);
+}
+
+void SThrottleParams::SIOFailureThreshold::Init(CSynRegistry& registry, const SRegSynonyms& sections)
+{
+    // These values must correspond to each other
+    const string error_rate = registry.Get(sections, "throttle_by_connection_error_rate", kEmptyStr);
+
+    if (error_rate.empty()) return;
+
+    string numerator_str, denominator_str;
+
+    if (!NStr::SplitInTwo(error_rate, "/", numerator_str, denominator_str)) return;
+
+    const auto flags = NStr::fConvErr_NoThrow | NStr::fAllowLeadingSpaces | NStr::fAllowTrailingSpaces;
+
+    int n = NStr::StringToInt(numerator_str, flags);
+    int d = NStr::StringToInt(denominator_str, flags);
+
+    if (n > 0) numerator = static_cast<size_t>(n);
+    if (d > 1) denominator = static_cast<size_t>(d);
+
+    if (denominator > kMaxDenominator) {
+        numerator = (numerator * kMaxDenominator) / denominator;
+        denominator = kMaxDenominator;
+    }
+}
+
+CNetServer::SAddress::SAddress(unsigned h, unsigned short p) :
+    host(h),
+    port(p),
+    name(host, {})
+{
+}
+
+CNetServer::SAddress::SAddress(string n, unsigned short p) :
+    host(g_NetService_gethostbyname(n)),
+    port(p),
+    name(host, move(n))
+{
+}
+
+bool operator==(const CNetServer::SAddress& lhs, const CNetServer::SAddress& rhs)
+{
+    return lhs.host == rhs.host && lhs.port == rhs.port;
+}
+
+bool operator< (const CNetServer::SAddress& lhs, const CNetServer::SAddress& rhs)
+{
+    if (lhs.host != rhs.host) return lhs.host < rhs.host;
+
+    return lhs.port < rhs.port;
+}
+
+string CNetServer::SAddress::AsString() const
+{
+    // Name was not looked up yet or host changed
+    if (name.second.empty() || name.first != host) {
+        name.first = host;
+        name.second = g_NetService_gethostnamebyaddr(host);
+    }
+
+    return name.second + ':' + NStr::UIntToString(port);
+}
+
 END_NCBI_SCOPE
