@@ -1,4 +1,4 @@
-/*  $Id: wn_main_loop.cpp 537169 2017-05-26 15:43:05Z sadyrovr $
+/*  $Id: wn_main_loop.cpp 571654 2018-10-01 12:45:42Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -158,7 +158,7 @@ CNcbiIstream& CWorkerNodeJobContext::GetIStream()
 
 CNcbiOstream& SWorkerNodeJobContextImpl::GetOStream()
 {
-    return m_GridWrite(m_NetCacheAPI, m_WorkerNode->m_QueueEmbeddedOutputSize, m_WorkerNode->m_DirectOutput, m_Job);
+    return m_GridWrite(m_NetCacheAPI, m_WorkerNode->m_QueueEmbeddedOutputSize, m_Job.output);
 }
 
 CNcbiOstream& CWorkerNodeJobContext::GetOStream()
@@ -549,6 +549,7 @@ void SWorkerNodeJobContextImpl::x_RunJob()
 
 void* CMainLoopThread::Main()
 {
+    const auto kRetryDelay = static_cast<unsigned long>(TServConn_RetryDelay::GetDefault() * kMilliSecondsPerSecond);
     SetCurrentThreadName(m_ThreadName);
     CDeadline max_wait_for_servers(
             TWorkerNode_MaxWaitForServers::GetDefault());
@@ -592,7 +593,7 @@ void* CMainLoopThread::Main()
                 CDeadline(TWorkerNode_MaxWaitForServers::GetDefault());
         }
         catch (CNetSrvConnException& e) {
-            SleepMilliSec(s_GetRetryDelay());
+            SleepMilliSec(kRetryDelay);
             if (e.GetErrCode() == CNetSrvConnException::eConnectionFailure &&
                     !max_wait_for_servers.GetRemainingTime().IsZero())
                 continue;
@@ -607,7 +608,7 @@ void* CMainLoopThread::Main()
                 CGridGlobals::GetInstance().RequestShutdown(
                     CNetScheduleAdmin::eShutdownImmediate);
             } else {
-                SleepMilliSec(s_GetRetryDelay());
+                SleepMilliSec(kRetryDelay);
                 continue;
             }
         }
@@ -723,7 +724,9 @@ bool CMainLoopThread::x_GetNextJob(CNetScheduleJob& job)
     if (!m_WorkerNode->WaitForExclusiveJobToFinish())
         return false;
 
-    if (m_Timeline.GetJob(CTimeout::eInfinite, job, NULL) != CNetScheduleGetJob::eJob) {
+    const bool any_affinity = m_Impl.m_API->m_AffinityLadder.empty();
+
+    if (m_Timeline.GetJob(CDeadline::eInfinite, job, NULL, any_affinity) != CNetScheduleGetJob::eJob) {
         return false;
     }
 
