@@ -1,4 +1,4 @@
-/*  $Id: expr.cpp 408848 2013-08-02 13:32:46Z ivanov $
+/*  $Id: expr.cpp 500405 2016-05-04 15:18:35Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -779,7 +779,7 @@ void CExprParser::Parse(const char* str)
             }
         }
 
-        int n_args = 1;
+        n_args = 1;
 
         while (sm_lpr[m_OStack[m_o_sp-1]] >= sm_rpr[oper]) { 
             int cop = m_OStack[--m_o_sp]; 
