@@ -1,4 +1,4 @@
-/*  $Id: metareg.cpp 487768 2015-12-21 20:05:25Z ucko $
+/*  $Id: metareg.cpp 548806 2017-10-18 12:19:30Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -331,8 +331,7 @@ void CMetaRegistry::GetDefaultSearchPath(CMetaRegistry::TSearchPath& path)
     const TXChar* cfg_path = NcbiSys_getenv(_TX("NCBI_CONFIG_PATH"));
     TSearchPath   path_tail;
     if (cfg_path) {
-        NStr::Split(_T_STDSTRING(cfg_path), kConfigPathDelim, path,
-                    NStr::fSplit_NoMergeDelims);
+        NStr::Split(_T_STDSTRING(cfg_path), kConfigPathDelim, path);
         TSearchPath::iterator it = find(path.begin(), path.end(), kEmptyStr);
         if (it == path.end()) {
             return;
