@@ -1,4 +1,4 @@
-/*  $Id: ncbi_stack_win32.cpp 500861 2016-05-09 16:32:20Z grichenk $
+/*  $Id: ncbi_stack_win32.cpp 534857 2017-05-03 12:26:07Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -113,7 +113,7 @@ static bool s_FillModuleListTH32(TModules& modules, DWORD pid)
         };
 
         const char* libname = dllname[0];
-        auto_ptr<CDll> dll;
+        unique_ptr<CDll> dll;
         while (libname) {
             dll.reset(new CDll(libname, CDll::eLoadNow, CDll::eAutoUnload));
 
