@@ -1,4 +1,4 @@
-/*  $Id: ncbi_stack_win64.cpp 456287 2015-01-09 14:29:15Z grichenk $
+/*  $Id: ncbi_stack_win64.cpp 500861 2016-05-09 16:32:20Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -416,7 +416,7 @@ void CStackTraceImpl::Expand(CStackTrace::TStack& stack)
 
     try {
         ITERATE(TStack, it, m_Stack) {
-            CStackTrace::SStackFrameInfo sf_info;
+            CStackTrace::SStackFrameInfo sf_info((void*)it->AddrPC.Offset);
             sf_info.func = "<cannot get function name for this address>";
 
             if ( !SymGetSymFromAddr64(curr_proc, it->AddrPC.Offset, &offs64, pSym) ) {
