@@ -1,4 +1,4 @@
-/* $Id: ncbi_system.cpp 500405 2016-05-04 15:18:35Z ivanov $
+/* $Id: ncbi_system.cpp 525410 2017-01-24 15:14:24Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -65,6 +65,7 @@ extern "C" {
 #  if defined(NCBI_OS_IRIX)
 #    include <sys/sysmp.h>
 #  endif //NCBI_OS_IRIX
+#  include "ncbi_os_unix_p.hpp"
 #  define USE_SETMEMLIMIT
 #  define USE_SETCPULIMIT
 #  define HAVE_MADVISE 1
@@ -87,6 +88,7 @@ extern "C" {
 #  include <crtdbg.h>
 #  include <stdlib.h>
 #  include <windows.h>
+#  include "ncbi_os_mswin_p.hpp"
 
 struct SProcessMemoryCounters
 {
@@ -1210,4 +1212,16 @@ extern int GetProcessThreadCount(void)
 }
 
 
+string GetProcessUserName(void)
+{
+#if defined(NCBI_OS_UNIX)
+    return CUnixFeature::GetUserNameByUID(geteuid());
+#elif defined(NCBI_OS_MSWIN)
+    return CWinSecurity::GetUserName();
+#else
+    return string();
+#endif
+}
+
+
 END_NCBI_SCOPE
