@@ -1,4 +1,4 @@
-/*  $Id: ncbi_url.cpp 534857 2017-05-03 12:26:07Z ivanov $
+/*  $Id: ncbi_url.cpp 566197 2018-06-26 18:22:05Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -463,7 +463,15 @@ void CUrl::SetUrl(const string& orig_url, const IUrlEncoder* encoder)
             }
             host.resize(pos);
         }
-        if (NStr::EqualNocase(m_Scheme, NCBI_SCHEME_SERVICE)) {
+        size_t scheme_svc_pos = NStr::Find(m_Scheme,
+                                           string("+") + NCBI_SCHEME_SERVICE,
+                                           NStr::eNocase, NStr::eReverseSearch);
+        if (scheme_svc_pos != NPOS  &&  scheme_svc_pos > 0) {
+            x_SetScheme(m_Scheme.substr(0, scheme_svc_pos), *encoder);
+            x_SetService(host);
+        }
+        else if (NStr::EqualNocase(m_Scheme, NCBI_SCHEME_SERVICE)) {
+            m_Scheme.clear();
             x_SetService(host);
         }
         else {
@@ -520,11 +528,36 @@ string CUrl::ComposeUrl(CUrlArgs::EAmpEncoding amp_enc,
         encoder = GetDefaultEncoder();
     }
     string url;
+
+    // Host or service name only require some special processing.
+    bool host_only =  (!m_Host.empty() || !m_Service.empty())
+        && m_Scheme.empty()
+        && !m_IsGeneric
+        && m_User.empty()
+        && m_Password.empty()
+        && m_Port.empty()
+        && m_Path.empty()
+        && m_Fragment.empty() &&
+        !HaveArgs();
+    if (host_only && IsService()) {
+        // Do not add ncbilb:// if only service name is set.
+        return NStr::URLEncode(m_Service, NStr::eUrlEnc_ProcessMarkChars);
+    }
+
     if ( !m_Scheme.empty() ) {
         url += m_Scheme;
+    }
+    if (IsService() && m_Scheme != NCBI_SCHEME_SERVICE) {
+        if ( !m_Scheme.empty() ) {
+            url += "+";
+        }
+        url += NCBI_SCHEME_SERVICE;
+    }
+    if (IsService() || !m_Scheme.empty()) {
         url += ":";
     }
-    if (m_IsGeneric) {
+
+    if (host_only || m_IsGeneric || IsService()) {
         url += "//";
     }
     bool have_user_info = false;
@@ -559,6 +592,29 @@ string CUrl::ComposeUrl(CUrlArgs::EAmpEncoding amp_enc,
 }
 
 
+void CUrl::SetScheme(const string& value)
+{
+    size_t pos = value.find(NCBI_SCHEME_SERVICE);
+    if (pos != NPOS
+        &&  (pos == 0 || value[pos - 1] == '+')
+        &&  value.substr(pos) == NCBI_SCHEME_SERVICE) {
+        // Strip "ncbilb" scheme, switch to service.
+        if (!IsService()) {
+            x_SetService(GetHost());
+        }
+        if (pos == 0) {
+            m_Scheme.clear();
+        }
+        else {
+            m_Scheme = value.substr(0, pos - 1);
+        }
+    }
+    else {
+        m_Scheme = value;
+    }
+}
+
+
 const CUrlArgs& CUrl::GetArgs(void) const
 {
     if ( !m_ArgsList.get() ) {
@@ -577,6 +633,12 @@ void CUrl::Adjust(const CUrl& other, TAdjustFlags flags)
     static const int fFragment_Mask = fFragment_Replace | fFragment_ReplaceIfEmpty;
     static const int fArgs_Mask = fArgs_Replace | fArgs_Append | fArgs_Merge;
 
+    if ( !other.m_Scheme.empty() ) {
+        if (flags & fScheme_Replace) {
+            m_Scheme = other.m_Scheme;
+        }
+    }
+
     if ((flags & fUser_Mask) == fUser_Mask) {
         NCBI_THROW(CUrlException, eFlags, "Multiple fUser_* flags are set.");
     }
