@@ -1,4 +1,4 @@
-/*  $Id: ncbiapp.cpp 563865 2018-05-16 15:00:13Z ucko $
+/*  $Id: ncbiapp.cpp 576975 2018-12-27 12:53:02Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1097,17 +1097,9 @@ void CNcbiApplication::SetStdioFlags(TStdioSetupFlags stdio_flags)
 
 void CNcbiApplication::x_SetupStdio(void)
 {
-#if 1//!defined(NCBI_COMPILER_GCC)  ||  NCBI_COMPILER_VERSION >= 411
-    // CAUTION:  http://gcc.gnu.org/bugzilla/show_bug.cgi?id=26777
-    //           fix applied Mar 29, 2006, scheduled for 4.1.1
-    if ((m_StdioFlags & fDefault_SyncWithStdio) == 0) {
-        // SUN WorkShop STL stream library has significant performance loss
-        // (due to the multiple gratuitous lseeks() in std i/o)
-        // when sync_with_stdio is TRUE (default),
-        // so we turn off sync_with_stdio here.
+    if ((m_StdioFlags & fNoSyncWithStdio) != 0) {
         IOS_BASE::sync_with_stdio(false);
     }
-#endif
 
     if ((m_StdioFlags & fDefault_CinBufferSize) == 0
 #ifdef NCBI_OS_UNIX
