@@ -1,4 +1,4 @@
-/*  $Id: ncbiexec.cpp 544803 2017-08-28 14:35:55Z ivanov $
+/*  $Id: ncbiexec.cpp 558614 2018-02-28 15:34:58Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -141,6 +141,7 @@ s_SpawnUnix(ESpawnFunc func, CExec::EMode full_mode,
 
     // Replace the current process image with a new process image.
     if (mode == CExec::eOverlay) {
+        GetDiagContext().PrintStop();
         switch (func) {
         case eV:
             return execv(cmdname, const_cast<char**>(argv));
@@ -550,7 +551,11 @@ CExec::SpawnL(EMode mode, const char *cmdname, const char *argv, ...)
 
 #if defined(NCBI_OS_MSWIN)
     _flushall();
-    status = NcbiSys_spawnv(s_GetRealMode(mode), _T_XCSTRING(cmdname), a_args);
+    int realmode = s_GetRealMode(mode);
+    if (realmode == P_OVERLAY) {
+        GetDiagContext().PrintStop();
+    }
+    status = NcbiSys_spawnv(realmode, _T_XCSTRING(cmdname), a_args);
 #elif defined(NCBI_OS_UNIX)
     status = s_SpawnUnix(eV, mode, cmdname, a_args);
 #endif
@@ -567,7 +572,11 @@ CExec::SpawnLE(EMode mode, const char *cmdname,  const char *argv, ...)
 
 #if defined(NCBI_OS_MSWIN)
     _flushall();
-    status = NcbiSys_spawnve(s_GetRealMode(mode), _T_XCSTRING(cmdname), a_args, a_envs);
+    int realmode = s_GetRealMode(mode);
+    if (realmode == P_OVERLAY) {
+        GetDiagContext().PrintStop();
+    }
+    status = NcbiSys_spawnve(realmode, _T_XCSTRING(cmdname), a_args, a_envs);
 #elif defined(NCBI_OS_UNIX)
     status = s_SpawnUnix(eVE, mode, cmdname, a_args, a_envs);
 #endif
@@ -583,7 +592,11 @@ CExec::SpawnLP(EMode mode, const char *cmdname, const char *argv, ...)
 
 #if defined(NCBI_OS_MSWIN)
     _flushall();
-    status = NcbiSys_spawnvp(s_GetRealMode(mode), _T_XCSTRING(cmdname), a_args);
+    int realmode = s_GetRealMode(mode);
+    if (realmode == P_OVERLAY) {
+        GetDiagContext().PrintStop();
+    }
+    status = NcbiSys_spawnvp(realmode, _T_XCSTRING(cmdname), a_args);
 #elif defined(NCBI_OS_UNIX)
     status = s_SpawnUnix(eVP, mode, cmdname, a_args);
 #endif
@@ -600,7 +613,11 @@ CExec::SpawnLPE(EMode mode, const char *cmdname, const char *argv, ...)
 
 #if defined(NCBI_OS_MSWIN)
     _flushall();
-    status = NcbiSys_spawnvpe(s_GetRealMode(mode), _T_XCSTRING(cmdname), a_args, a_envs);
+    int realmode = s_GetRealMode(mode);
+    if (realmode == P_OVERLAY) {
+        GetDiagContext().PrintStop();
+    }
+    status = NcbiSys_spawnvpe(realmode, _T_XCSTRING(cmdname), a_args, a_envs);
 #elif defined(NCBI_OS_UNIX)
     status = s_SpawnUnix(eVPE, mode, cmdname, a_args, a_envs);
 #endif
@@ -616,7 +633,11 @@ CExec::SpawnV(EMode mode, const char *cmdname, const char *const *argv)
 
 #if defined(NCBI_OS_MSWIN)
     _flushall();
-    status = NcbiSys_spawnv(s_GetRealMode(mode), _T_XCSTRING(cmdname), a_args);
+    int realmode = s_GetRealMode(mode);
+    if (realmode == P_OVERLAY) {
+        GetDiagContext().PrintStop();
+    }
+    status = NcbiSys_spawnv(realmode, _T_XCSTRING(cmdname), a_args);
 #elif defined(NCBI_OS_UNIX)
     status = s_SpawnUnix(eV, mode, cmdname, a_args);
 #endif
@@ -634,7 +655,11 @@ CExec::SpawnVE(EMode mode, const char *cmdname,
 
 #if defined(NCBI_OS_MSWIN)
     _flushall();
-    status = NcbiSys_spawnve(s_GetRealMode(mode), _T_XCSTRING(cmdname), a_args, a_envs);
+    int realmode = s_GetRealMode(mode);
+    if (realmode == P_OVERLAY) {
+        GetDiagContext().PrintStop();
+    }
+    status = NcbiSys_spawnve(realmode, _T_XCSTRING(cmdname), a_args, a_envs);
 #elif defined(NCBI_OS_UNIX)
     status = s_SpawnUnix(eVE, mode, cmdname, a_args, a_envs);
 #endif
@@ -650,7 +675,11 @@ CExec::SpawnVP(EMode mode, const char *cmdname, const char *const *argv)
 
 #if defined(NCBI_OS_MSWIN)
     _flushall();
-    status = NcbiSys_spawnvp(s_GetRealMode(mode), _T_XCSTRING(cmdname), a_args);
+    int realmode = s_GetRealMode(mode);
+    if (realmode == P_OVERLAY) {
+        GetDiagContext().PrintStop();
+    }
+    status = NcbiSys_spawnvp(realmode, _T_XCSTRING(cmdname), a_args);
 #elif defined(NCBI_OS_UNIX)
     status = s_SpawnUnix(eVP, mode, cmdname, a_args);
 #endif
@@ -668,7 +697,11 @@ CExec::SpawnVPE(EMode mode, const char *cmdname,
 
 #if defined(NCBI_OS_MSWIN)
     _flushall();
-    status = NcbiSys_spawnvpe(s_GetRealMode(mode), _T_XCSTRING(cmdname), a_args, a_envs);
+    int realmode = s_GetRealMode(mode);
+    if (realmode == P_OVERLAY) {
+        GetDiagContext().PrintStop();
+    }
+    status = NcbiSys_spawnvpe(realmode, _T_XCSTRING(cmdname), a_args, a_envs);
 #elif defined(NCBI_OS_UNIX)
     status = s_SpawnUnix(eVPE, mode, cmdname, a_args, a_envs);
 #endif
@@ -788,7 +821,7 @@ CExec::CResult CExec::RunSilent(EMode mode, const char *cmdname,
                       dwCreateFlags, NULL, NULL, &StartupInfo, &ProcessInfo))
     {
         if (mode == eOverlay) {
-            // destroy ourselves
+            GetDiagContext().PrintStop();
             _exit(0);
         }
         else if (mode == eWait) {
