@@ -1,4 +1,4 @@
-/*  $Id: ncbiexpt.cpp 534857 2017-05-03 12:26:07Z ivanov $
+/*  $Id: ncbiexpt.cpp 552221 2017-11-30 16:25:33Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -41,6 +41,7 @@
 #include <corelib/ncbithr.hpp>
 #include <corelib/ncbi_safe_static.hpp>
 #include <corelib/ncbi_param.hpp>
+#include <corelib/request_ctx.hpp>
 #include <corelib/error_codes.hpp>
 #include "ncbisys.hpp"
 #include <errno.h>
@@ -147,6 +148,16 @@ typedef NCBI_PARAM_TYPE(EXCEPTION, Abort_If_Critical) TAbortIfCritical;
 static CSafeStatic<TAbortIfCritical> s_AbortIfCritical;
 
 
+class CRequestContextRef
+{
+public:
+    CRequestContextRef(CRequestContext& ctx) : m_Context(&ctx) {}
+    CRequestContext& GetRequestContext(void) { return *m_Context; }
+private:
+    CRef<CRequestContext> m_Context;
+};
+
+
 /////////////////////////////////////////////////////////////////////////////
 // CException implementation
 
@@ -164,7 +175,8 @@ CException::CException(const CDiagCompileInfo& info,
   m_Predecessor(0),
   m_InReporter(false),
   m_MainText(true),
-  m_Flags(flags)
+  m_Flags(flags),
+  m_Retriable(eRetriable_Unknown)
 {
     if (CompareDiagPostLevel(severity, eDiag_Critical) >= 0  &&
         s_AbortIfCritical->Get()) {
@@ -185,7 +197,8 @@ CException::CException(const CDiagCompileInfo& info,
   m_Predecessor(0),
   m_InReporter(false),
   m_MainText(true),
-  m_Flags(flags)
+  m_Flags(flags),
+  m_Retriable(eRetriable_Unknown)
 {
     if (CompareDiagPostLevel(severity, eDiag_Critical) >= 0  &&
         s_AbortIfCritical->Get()) {
@@ -205,7 +218,8 @@ CException::CException(const CDiagCompileInfo& info,
   m_Predecessor(0),
   m_InReporter(false),
   m_MainText(true),
-  m_Flags(args.GetFlags())
+  m_Flags(args.GetFlags()),
+  m_Retriable(eRetriable_Unknown)
 {
     if (CompareDiagPostLevel(m_Severity, eDiag_Critical) >= 0  &&
         s_AbortIfCritical->Get()) {
@@ -232,7 +246,9 @@ CException::CException(void)
   m_Predecessor(0),
   m_InReporter(false),
   m_MainText(true),
-  m_Flags(0)
+  m_Flags(0),
+  m_Retriable(eRetriable_Unknown),
+  m_RequestContext(new CRequestContextRef(GetDiagContext().GetRequestContext()))
 {
 // this only is called in case of multiple inheritance
 }
@@ -451,6 +467,12 @@ const string& CException::GetMsg(void) const
 }
 
 
+CRequestContext& CException::GetRequestContext(void) const
+{
+    return m_RequestContext->GetRequestContext();
+}
+
+
 void CException::x_ReportToDebugger(void) const
 {
 #if defined(NCBI_OS_MSWIN)  &&  defined(_DEBUG)
@@ -495,6 +517,7 @@ void CException::x_Init(const CDiagCompileInfo& info,const string& message,
     if (!m_Predecessor && prev_exception) {
         m_Predecessor = prev_exception->x_Clone();
     }
+    m_RequestContext.reset(new CRequestContextRef(GetDiagContext().GetRequestContext()));
     x_GetStackTrace();
 }
 
@@ -504,6 +527,7 @@ void CException::x_InitArgs(const CExceptionArgs_Base& args)
     if ( args.IsSetModule() ) {
         m_Module = args.GetModule();
     }
+    m_Retriable = args.GetRetriable();
 }
 
 
@@ -526,6 +550,8 @@ void CException::x_Assign(const CException& src)
         m_StackTrace.reset(new CStackTrace(*src.m_StackTrace));
     }
     m_Flags = src.m_Flags;
+    m_Retriable = src.m_Retriable;
+    m_RequestContext.reset(new CRequestContextRef(src.m_RequestContext->GetRequestContext()));
 }
 
 
@@ -666,6 +692,7 @@ void CExceptionReporter::ReportDefaultEx(int err_code, int err_subcode,
                               flags);
     } else {
         CNcbiDiag d(info, cex->GetSeverity(), flags);
+        d.SetOmitStackTrace(true);
         d << ErrCode(err_code, err_subcode) << title << " " <<  *cex;
     }
 }
