@@ -1,4 +1,4 @@
-/*  $Id: ncbireg.cpp 540943 2017-07-12 15:49:26Z ivanov $
+/*  $Id: ncbireg.cpp 549365 2017-10-24 14:30:17Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1826,22 +1826,27 @@ bool CCompoundRWRegistry::LoadBaseRegistries(TFlags flags, int metareg_flags,
     list<string> names;
     {{
         string s = m_MainRegistry->Get("NCBI", ".Inherits");
-        if (s.empty()) {
+
+        REVERSE_ITERATE(CCompoundRegistry::TPriorityMap, it, m_AllRegistries->m_PriorityMap) {
+            s += ',';
+            s += it->second->Get("NCBI", ".Inherits");
+        }
+
+        {
             if (dynamic_cast<CNcbiRegistry*>(this) != NULL) {
                 _TRACE("LoadBaseRegistries(" << this
                        << "): trying file registry");
-                s = FindByName(CNcbiRegistry::sm_FileRegName)
+                s += ',';
+                s += FindByName(CNcbiRegistry::sm_FileRegName)
                     ->Get("NCBI", ".Inherits");
             }
-            if (s.empty()) {
-                return false;
-            }
         }
-        _TRACE("LoadBaseRegistries(" << this << "): using " << s);
         NStr::Split(s, ", ", names,
                     NStr::fSplit_CanSingleQuote | 
                     NStr::fSplit_MergeDelimiters | 
                     NStr::fSplit_Truncate);
+        if (names.empty()) return false;
+        _TRACE("LoadBaseRegistries(" << this << "): using " << s);
     }}
 
     typedef pair<string, CRef<IRWRegistry> > TNewBase;
