@@ -1,4 +1,4 @@
-/*  $Id: request_ctx.cpp 508631 2016-07-29 15:53:51Z grichenk $
+/*  $Id: request_ctx.cpp 568672 2018-08-09 13:05:38Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,6 +37,7 @@
 #include <corelib/ncbi_param.hpp>
 #include <corelib/ncbi_strings.h>
 #include <corelib/error_codes.hpp>
+#include <corelib/ncbi_mask.hpp>
 
 
 #define NCBI_USE_ERRCODE_X   Corelib_Diag
@@ -49,10 +50,11 @@ static const char* kPassThrough_Phid = "ncbi_phid";
 static const char* kPassThrough_ClientIp = "ncbi_client_ip";
 static const char* kPassThrough_Dtab = "ncbi_dtab";
 
+
 CRequestContext::CRequestContext(TContextFlags flags)
     : m_RequestID(0),
       m_AppState(eDiagAppState_NotSet),
-      m_LoggedHitID(false),
+      m_HitIDLoggedFlag(0),
       m_ReqStatus(0),
       m_ReqTimer(CStopWatch::eStop),
       m_BytesRd(0),
@@ -61,8 +63,10 @@ CRequestContext::CRequestContext(TContextFlags flags)
       m_IsRunning(false),
       m_AutoIncOnPost(false),
       m_Flags(flags),
-      m_OwnerTID(-1)
+      m_OwnerTID(-1),
+      m_IsReadOnly(false)
 {
+    x_LoadEnvContextProperties();
 }
 
 
@@ -80,7 +84,7 @@ CRequestContext::TCount CRequestContext::GetNextRequestID(void)
 
 void CRequestContext::x_LogHitID(bool ignore_app_state) const
 {
-    if (m_LoggedHitID || m_HitID.Empty()) return;
+    if ((m_HitIDLoggedFlag & fLoggedOnRequest) || m_HitID.Empty()) return;
 
     // ignore_app_state can be set by CDiagContext in case if hit-id
     // was set for request context only (no default one), but request
@@ -94,13 +98,15 @@ void CRequestContext::x_LogHitID(bool ignore_app_state) const
         m_AppState != eDiagAppState_RequestEnd) return;
     GetDiagContext().Extra().Print(g_GetNcbiString(eNcbiStrings_PHID),
         m_HitID.GetHitId());
-    m_LoggedHitID = true;
+    m_HitIDLoggedFlag |= fLoggedOnRequest;
 }
 
 
 const string& CRequestContext::SetHitID(void)
 {
-    SetHitID(GetDiagContext().GetNextHitID());
+    if (x_CanModify()) {
+        SetHitID(GetDiagContext().GetNextHitID());
+    }
     return m_HitID.GetHitId();
 }
 
@@ -111,6 +117,7 @@ string CRequestContext::x_GetHitID(CDiagContext::EDefaultHitIDFlags flag) const
         x_LogHitID();
         return m_HitID.GetHitId();
     }
+    if (!x_CanModify()) return kEmptyStr;
     CSharedHitId phid = GetDiagContext().x_GetDefaultHitID(CDiagContext::eHitID_NoCreate);
     if (!phid.Empty()) {
         const_cast<CRequestContext*>(this)->x_SetHitID(phid);
@@ -144,12 +151,14 @@ EDiagAppState CRequestContext::GetAppState(void) const
 
 void CRequestContext::SetAppState(EDiagAppState state)
 {
+    if (!x_CanModify()) return;
     m_AppState = state;
 }
 
 
 void CRequestContext::Reset(void)
 {
+    if (!x_CanModify()) return;
     m_AppState = eDiagAppState_NotSet; // Use global AppState
     UnsetRequestID();
     UnsetClientIP();
@@ -160,11 +169,13 @@ void CRequestContext::Reset(void)
     UnsetBytesWr();
     m_ReqTimer.Reset();
     m_PassThroughProperties.clear();
+    x_LoadEnvContextProperties();
 }
 
 
 void CRequestContext::SetProperty(const string& name, const string& value)
 {
+    if (!x_CanModify()) return;
     m_Properties[name] = value;
 }
 
@@ -184,6 +195,7 @@ bool CRequestContext::IsSetProperty(const string& name) const
 
 void CRequestContext::UnsetProperty(const string& name)
 {
+    if (!x_CanModify()) return;
     m_Properties.erase(name);
 }
 
@@ -193,6 +205,7 @@ static const char* kBadIP = "0.0.0.0";
 
 void CRequestContext::SetClientIP(const string& client)
 {
+    if (!x_CanModify()) return;
     x_SetProp(eProp_ClientIP);
 
     // Verify IP
@@ -208,6 +221,8 @@ void CRequestContext::SetClientIP(const string& client)
 
 void CRequestContext::StartRequest(void)
 {
+    if (!x_CanModify()) return;
+    x_LoadEnvContextProperties();
     if (m_Flags & fResetOnStart) {
         UnsetRequestStatus();
         SetBytesRd(0);
@@ -221,7 +236,8 @@ void CRequestContext::StartRequest(void)
 
 void CRequestContext::StopRequest(void)
 {
-    if (!m_LoggedHitID) {
+    if (!x_CanModify()) return;
+    if ((m_HitIDLoggedFlag & fLoggedOnRequest) == 0) {
         // Hit id has not been set or logged yet. Try to log the default one.
         x_GetHitID(CDiagContext::eHitID_NoCreate);
     }
@@ -321,8 +337,9 @@ static bool IsValidHitID(const string& hit) {
 
 void CRequestContext::x_SetHitID(const CSharedHitId& hit_id)
 {
+    if (!x_CanModify()) return;
     const string& hit = hit_id.GetHitId();
-    if ( m_LoggedHitID ) {
+    if (m_HitIDLoggedFlag & fLoggedOnRequest) {
         // Show warning when changing hit id after is has been logged.
         ERR_POST_X(28, Warning << "Changing hit ID after one has been logged. "
             "New hit id is: " << hit);
@@ -352,35 +369,49 @@ void CRequestContext::x_SetHitID(const CSharedHitId& hit_id)
 
     m_SubHitIDCache.clear();
     m_HitID = hit_id;
-    m_LoggedHitID = false;
+    m_HitIDLoggedFlag = 0;
     x_LogHitID();
 }
 
 
 void CRequestContext::SetHitID(const string& hit)
 {
+    if (!x_CanModify()) return;
     x_SetHitID(CSharedHitId(hit));
 }
 
 
+// Use old output format if the flag is set
+NCBI_PARAM_DECL(unsigned int, Log, Issued_SubHit_Limit);
+NCBI_PARAM_DEF_EX(unsigned int, Log, Issued_SubHit_Limit, 200, eParam_NoThread,
+    LOG_ISSUED_SUBHIT_LIMIT);
+typedef NCBI_PARAM_TYPE(Log, Issued_SubHit_Limit) TIssuedSubHitLimitParam;
+
+
 void CRequestContext::x_UpdateSubHitID(bool increment, CTempString prefix)
 {
     _ASSERT(IsSetHitID());
 
     // Use global sub-hit counter for default hit id to prevent
     // duplicate phids in different threads.
-    m_SubHitIDCache = GetHitID();
+    string hit_id = GetHitID();
 
     unsigned int sub_hit_id = increment ?
         m_HitID.GetNextSubHitId() : m_HitID.GetCurrentSubHitId();
 
     // Cache the string so that C code can use it.
-    m_SubHitIDCache += "." + prefix + NStr::NumericToString(sub_hit_id);
+    string subhit = prefix + NStr::NumericToString(sub_hit_id);
+    hit_id += "." + subhit;
+    m_SubHitIDCache = hit_id;
+    if (increment  &&  sub_hit_id <= TIssuedSubHitLimitParam::GetDefault()) {
+        GetDiagContext().Extra().Print("issued_subhit", subhit);
+    }
 }
 
 
 void CRequestContext::SetSessionID(const string& session)
 {
+    if (!x_CanModify()) return;
     if ( !IsValidSessionID(session) ) {
         EOnBadSessionID action = GetBadSessionIDAction();
         switch ( action ) {
@@ -506,7 +537,7 @@ CRef<CRequestContext> CRequestContext::Clone(void) const
     ret->m_SessionID.SetString(m_SessionID.GetOriginalString());
     m_HitID.SetShared();
     ret->m_HitID = m_HitID;
-    ret->m_LoggedHitID = m_LoggedHitID;
+    ret->m_HitIDLoggedFlag = m_HitIDLoggedFlag;
     ret->m_SubHitIDCache = m_SubHitIDCache;
     ret->m_Dtab = m_Dtab;
     ret->m_ReqStatus = m_ReqStatus;
@@ -518,6 +549,7 @@ CRef<CRequestContext> CRequestContext::Clone(void) const
     ret->m_IsRunning = m_IsRunning;
     ret->m_AutoIncOnPost = m_AutoIncOnPost;
     ret->m_Flags = m_Flags;
+    ret->m_IsReadOnly = m_IsReadOnly;
     return ret;
 }
 
@@ -573,6 +605,7 @@ void CRequestContext::x_SetPassThroughProp(CTempString name,
                                            CTempString value,
                                            bool update) const
 {
+    if (!x_CanModify()) return;
     m_PassThroughProperties[name] = value;
     if ( update ) x_UpdateStdContextProp(name);
 }
@@ -580,6 +613,7 @@ void CRequestContext::x_SetPassThroughProp(CTempString name,
 
 void CRequestContext::x_ResetPassThroughProp(CTempString name, bool update) const
 {
+    if (!x_CanModify()) return;
     TPassThroughProperties::iterator found = m_PassThroughProperties.find(name);
     if (found != m_PassThroughProperties.end()) {
         m_PassThroughProperties.erase(found);
@@ -687,6 +721,93 @@ void CRequestContext::x_UpdateStdContextProp(CTempString name) const
 }
 
 
+NCBI_PARAM_DECL(string, Context, Fields);
+NCBI_PARAM_DEF_EX(string, Context, Fields, "",
+                  eParam_NoThread,
+                  NCBI_CONTEXT_FIELDS);
+typedef NCBI_PARAM_TYPE(Context, Fields) TNcbiContextFields;
+
+DEFINE_STATIC_MUTEX(s_ContextFieldsMutex);
+unique_ptr<CMaskFileName> CRequestContext::sm_ContextFields;
+unique_ptr<CRequestContext::TPassThroughProperties> CRequestContext::sm_EnvContextProperties;
+
+
+const CMask& CRequestContext::sx_GetContextFieldsMask(void)
+{
+    if ( !sm_ContextFields.get() ) {
+        CMutexGuard guard(s_ContextFieldsMutex);
+        if ( !sm_ContextFields.get() ) {
+            sm_ContextFields.reset(new CMaskFileName());
+            string fields_var = TNcbiContextFields::GetDefault();
+            if ( !fields_var.empty() ) {
+                list<string> fields;
+                NStr::Split(fields_var, " ", fields, NStr::fSplit_MergeDelimiters);
+                ITERATE(list<string>, field, fields) {
+                    string norm_field = sx_NormalizeContextPropertyName(*field);
+                    sm_ContextFields->Add(norm_field);
+                }
+            }
+            else {
+                // By default exclude everything.
+                sm_ContextFields->AddExclusion("*");
+            }
+        }
+    }
+    return *sm_ContextFields;
+}
+
+
+string CRequestContext::sx_NormalizeContextPropertyName(const string& name)
+{
+    return NStr::Replace(name, "_", "-");
+}
+
+
+void CRequestContext::x_LoadEnvContextProperties(void)
+{
+    if (!x_CanModify()) return;
+    // Parse environment only once.
+    if ( !sm_EnvContextProperties.get() ) {
+        TPassThroughProperties props;
+        {
+            CMutexGuard guard(CNcbiApplication::GetInstanceMutex());
+            CNcbiApplication* app = CNcbiApplication::Instance();
+            if ( !app ) return;
+            const CNcbiEnvironment& env = app->GetEnvironment();
+            list<string> names;
+            env.Enumerate(names);
+            ITERATE(list<string>, it, names) {
+                props[*it] = env.Get(*it);
+            }
+        }
+
+        CMutexGuard guard(s_ContextFieldsMutex);
+        if ( !sm_EnvContextProperties.get() ) {
+            sm_EnvContextProperties.reset(new TPassThroughProperties);
+            const CMask& mask = sx_GetContextFieldsMask();
+            ITERATE(TPassThroughProperties, it, props) {
+                string norm_prop = sx_NormalizeContextPropertyName(it->first);
+                if ( mask.Match(norm_prop, NStr::eNocase) ) {
+                    (*sm_EnvContextProperties)[norm_prop] = it->second;
+                }
+            }
+        }
+    }
+    m_PassThroughProperties.insert(sm_EnvContextProperties->begin(), sm_EnvContextProperties->end());
+}
+
+
+void CRequestContext::AddPassThroughProperty(const string& name, const string& value)
+{
+    if (!x_CanModify()) return;
+    const CMask& mask = sx_GetContextFieldsMask();
+    string norm_prop = sx_NormalizeContextPropertyName(name);
+    if ( mask.Match(norm_prop, NStr::eNocase) ) {
+        m_PassThroughProperties[norm_prop] = value;
+    }
+}
+
+
 string CRequestContext_PassThrough::Serialize(EFormat format) const
 {
     m_Context->x_UpdateStdPassThroughProp("");
@@ -736,4 +857,64 @@ const char* CRequestContextException::GetErrCodeString(void) const
 }
 
 
+CRequestContextGuard_Base::CRequestContextGuard_Base(CRequestContext* context, TFlags flags)
+    : m_Flags(flags)
+{
+    CDiagContext& ctx = GetDiagContext();
+    if ( context ) {
+        m_SavedContext.Reset(&ctx.GetRequestContext());
+        m_RequestContext.Reset(context);
+        ctx.SetRequestContext(context);
+    }
+    else {
+        m_RequestContext.Reset(&ctx.GetRequestContext());
+    }
+
+    if (m_Flags & fPrintRequestStart) {
+        // This also updates app state to RB/R
+        ctx.PrintRequestStart();
+    }
+}
+
+
+CRequestContextGuard_Base::~CRequestContextGuard_Base(void)
+{
+    // If released, do not perform any actions.
+    if ( !m_RequestContext ) return;
+
+    if ( uncaught_exception() ) {
+        SetStatus(m_ErrorStatus);
+    }
+    else if ( !m_RequestContext->IsSetRequestStatus() ) {
+        m_RequestContext->SetRequestStatus(CRequestStatus::e200_Ok);
+    }
+
+    CDiagContext& ctx = GetDiagContext();
+    // This also updates app state to RE/P
+    ctx.PrintRequestStop();
+
+    if ( m_SavedContext ) {
+        ctx.SetRequestContext(m_SavedContext.GetNonNullPointer());
+    }
+}
+
+
+void CRequestContextGuard_Base::Release(void)
+{
+    if ( m_SavedContext ) {
+        GetDiagContext().SetRequestContext(m_SavedContext.GetNonNullPointer());
+        m_SavedContext.Reset();
+    }
+    m_RequestContext.Reset();
+}
+
+
+void CRequestContextGuard_Base::SetDefaultErrorStatus(int status)
+{
+    // If the guard has been released throw null pointer exception.
+    m_RequestContext.GetNonNullPointer();
+    m_ErrorStatus = status;
+}
+
+
 END_NCBI_SCOPE
