@@ -1,4 +1,4 @@
-/*  $Id: syslog.cpp 460070 2015-02-24 19:18:49Z ucko $
+/*  $Id: syslog.cpp 534306 2017-04-26 14:16:05Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -249,7 +249,7 @@ void CSysLog::x_Connect(void)
 #endif
 }
 
-void CSysLog::HonorRegistrySettings(IRegistry* reg)
+void CSysLog::HonorRegistrySettings(const IRegistry* reg)
 {
     if (reg == NULL  &&  CNcbiApplication::Instance()) {
         reg = &CNcbiApplication::Instance()->GetConfig();
