@@ -1,4 +1,4 @@
-/*  $Id: test_boost.cpp 534439 2017-04-27 11:53:27Z ivanov $
+/*  $Id: test_boost.cpp 570860 2018-09-17 13:59:01Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -95,8 +95,15 @@
 
 #if BOOST_VERSION >= 106000
 #  define attr_value utils::attr_value
-#  define RTCFG(type, new_name, old_name) \
-    but::runtime_config::get<type >(but::runtime_config::new_name)
+#  if BOOST_VERSION >= 106400
+     // Everything old is new again, apparently...
+#    define RTCFG(type, new_name, old_name) \
+      but::runtime_config::get<type >(but::runtime_config::btrt_##old_name)
+#    define CONFIGURED_FILTERS RTCFG(std::vector<std::string>, _, run_filters)
+#  else
+#    define RTCFG(type, new_name, old_name) \
+      but::runtime_config::get<type >(but::runtime_config::new_name)
+#  endif
 #else
 #  define RTCFG(type, new_name, old_name) but::runtime_config::old_name()
 #  if BOOST_VERSION >= 105900
@@ -112,8 +119,10 @@
 #  endif
 #endif
 
-#define CONFIGURED_FILTERS \
+#ifndef CONFIGURED_FILTERS
+  #define CONFIGURED_FILTERS \
     RTCFG(std::vector<std::string>, RUN_FILTERS, test_to_run)
+#endif
 
 #ifdef NCBI_COMPILER_MSVC
 #  pragma warning(pop)
@@ -278,11 +287,19 @@ public:
     virtual
     void entry_context_start(ostream& ostr, but::log_level l);
 
+#  if BOOST_VERSION >= 106500
+    virtual
+    void log_entry_context(ostream& os, but::log_level l, but::const_string v);
+
+    virtual
+    void entry_context_finish(ostream& os, but::log_level l);
+#  else
     virtual
     void log_entry_context(ostream& ostr, but::const_string value);
 
     virtual
     void entry_context_finish (ostream& ostr);
+#  endif
 #endif
 
 private:
@@ -874,8 +891,7 @@ CNcbiTestsTreeBuilder::test_suite_start(but::test_suite const& suite)
 void
 CNcbiTestsTreeBuilder::test_suite_finish(but::test_suite const& suite)
 {
-    _ASSERT(m_CurElem->GetTestUnit()
-                            == &static_cast<const but::test_unit&>(suite));
+    _ASSERT(m_CurElem->GetTestUnit() == &static_cast<const but::test_unit&>(suite));
     m_CurElem = m_CurElem->GetParent();
 }
 
@@ -906,6 +922,30 @@ CNcbiTestsTreeBuilder::FixUnitsOrder(void)
     }
 }
 
+// List of memory chunks to delete at exit
+static CNcbiTestMemoryCleanupList* s_TestMemoryCleanupList = NULL;
+
+CNcbiTestMemoryCleanupList* CNcbiTestMemoryCleanupList::GetInstance(void)
+{
+    if (!s_TestMemoryCleanupList) {
+        s_TestMemoryCleanupList = new CNcbiTestMemoryCleanupList();
+    }
+    return s_TestMemoryCleanupList;
+}
+
+CNcbiTestMemoryCleanupList::~CNcbiTestMemoryCleanupList()
+{
+    for (auto& x : s_TestMemoryCleanupList->m_List) {
+        free(x);
+    }
+    m_List.clear();
+}
+
+void CNcbiTestMemoryCleanupList::Add(void* ptr) {
+    m_List.push_back(ptr);
+}
+
+
 
 static CNcbiTestApplication* s_TestApp = NULL;
 
@@ -935,8 +975,9 @@ CNcbiTestApplication::~CNcbiTestApplication(void)
 static CNcbiTestApplication&
 s_GetTestApp(void)
 {
-    if (!s_TestApp)
+    if (!s_TestApp) {
         s_TestApp = new CNcbiTestApplication();
+    }
     return *s_TestApp;
 }
 
@@ -1456,7 +1497,7 @@ CNcbiTestApplication::x_ReadConfiguration(void)
         but::test_unit* tu = GetTestUnit(test_name);
         if (tu) {
             list<CTempString> koef_lst;
-            NStr::Split(reg_value, ";", koef_lst, NStr::fSplit_NoMergeDelims);
+            NStr::Split(reg_value, ";", koef_lst);
             ITERATE(list<CTempString>, it_koef, koef_lst) {
                 CTempString koef_str, koef_cond;
                 if (NStr::SplitInTwo(*it_koef, ":", koef_str, koef_cond)) {
@@ -2124,6 +2165,19 @@ void CNcbiBoostLogger::entry_context_start(ostream& ostr, but::log_level l)
     m_Upper->entry_context_start(ostr, l);
 }
 
+#  if BOOST_VERSION >= 106500
+void CNcbiBoostLogger::log_entry_context(ostream& ostr,
+                                         but::log_level l,
+                                         but::const_string value)
+{
+    m_Upper->log_entry_context(ostr, l, value);
+}
+
+void CNcbiBoostLogger::entry_context_finish(ostream& ostr, but::log_level l)
+{
+    m_Upper->entry_context_finish(ostr, l);
+}
+#  else
 void CNcbiBoostLogger::log_entry_context(ostream& ostr,
                                          but::const_string value)
 {
@@ -2134,6 +2188,7 @@ void CNcbiBoostLogger::entry_context_finish (ostream& ostr)
 {
     m_Upper->entry_context_finish(ostr);
 }
+#  endif
 #endif
 
 void
@@ -2252,12 +2307,11 @@ main(int argc, char* argv[])
                 ncbi_args.push_back(argv[i]);
             }
         }
-        s_NcbiArgc = ncbi_args.size();
+        s_NcbiArgc = (int)ncbi_args.size();
         s_NcbiArgv = ncbi_args.data();
-        framework::init
-            (&init_unit_test_suite, boost_args.size(), boost_args.data());
+        framework::init(&init_unit_test_suite, (int)boost_args.size(), boost_args.data());
 #else
-        framework::init( &init_unit_test_suite, argc, argv );
+        framework::init(&init_unit_test_suite, argc, argv);
 #endif
 
         ncbi::s_GetTestApp().InitTestsBeforeRun();
@@ -2316,7 +2370,7 @@ main(int argc, char* argv[])
 
         if (
 #if BOOST_VERSION >= 106000
-            runtime_config::get<bool>( runtime_config::RESULT_CODE )
+            RTCFG(bool, RESULT_CODE, result_code)
 #else
             !runtime_config::no_result_code()
 #endif
@@ -2385,6 +2439,22 @@ main(int argc, char* argv[])
 #endif
 
     delete NCBI_NS_NCBI::s_TestApp;
+    delete NCBI_NS_NCBI::s_TestMemoryCleanupList;
+
     NCBI_NS_NCBI::GetDiagContext().SetExitCode(result_code);
     return result_code;
 }
+
+// Provide 'wmain()' too, and convert cmdline args from UNICODE to UTF-8.
+#if defined(NCBI_OS_MSWIN) && defined(_UNICODE)
+int NcbiSys_main(int argc, ncbi::TXChar* argv[])
+{
+    std::vector<std::string>  args_storage(argc);
+    std::vector<char*>        args(argc);
+    for (int i = 0; i < argc; ++i) {
+        args_storage[i] = ncbi::CUtf8::AsUTF8(argv[i]);
+        args[i] = const_cast<char*>(args_storage[i].c_str());
+    }
+    return main(argc, args.data());
+}
+#endif
