@@ -1,4 +1,4 @@
-/* $Id: test_mt.cpp 534857 2017-05-03 12:26:07Z ivanov $
+/* $Id: test_mt.cpp 548878 2017-10-18 17:48:25Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -70,7 +70,7 @@ static string s_GroupsCount(void)
     return NStr::UIntToString( (unsigned int)sqrt((double)s_NumThreads));
 }
 NCBI_PARAM_DECL(string, TEST_MT, GroupsCount);
-NCBI_PARAM_DEF_WITH_INIT(string, TEST_MT, GroupsCount, kEmptyStr, s_GroupsCount);
+NCBI_PARAM_DEF_WITH_INIT(string, TEST_MT, GroupsCount, "", s_GroupsCount);
 
 // group.has_sync_point = (rand() % 100) < threshold;
 NCBI_PARAM_DECL(unsigned int, TEST_MT, IntragroupSyncPoint);
