@@ -1,4 +1,4 @@
-/*  $Id: sqlitewrapp.cpp 535286 2017-05-08 13:31:07Z ivanov $
+/*  $Id: sqlitewrapp.cpp 566371 2018-06-28 14:31:38Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -346,16 +346,6 @@ CSQLITE_Global::SetCustomPageCache(sqlite3_pcache_methods* methods)
     }
 }
 
-void
-CSQLITE_Global::SetCustomMallocFuncs(sqlite3_mem_methods* methods)
-{
-    int res = sqlite3_config(SQLITE_CONFIG_MALLOC, methods);
-    if (res != SQLITE_OK) {
-        NCBI_THROW_FMT(CSQLITE_Exception, eBadCall,
-                       "Custom malloc functions are not set, err_code = " << res);
-    }
-}
-
 sqlite3_vfs*
 CSQLITE_Global::GetDefaultVFS(void)
 {
@@ -476,7 +466,7 @@ CSQLITE_Connection::CSQLITE_Connection(CTempString     file_name,
     : m_FileName(file_name),
       m_Flags(flags),
       m_PageSize(kSQLITE_DefPageSize),
-      m_CacheSize(0),
+      m_CacheSize((unsigned int)(-1)),
       m_HandlePool(CSQLITE_HandleFactory(this)),
       m_InMemory(0)
 {
@@ -512,9 +502,11 @@ CSQLITE_Connection::SetupNewConnection(sqlite3* handle)
     x_ExecuteSql(handle, "PRAGMA count_changes = 0");
     x_ExecuteSql(handle, "PRAGMA legacy_file_format = OFF");
     x_ExecuteSql(handle, "PRAGMA page_size = "
-                                            + NStr::IntToString(m_PageSize));
-    x_ExecuteSql(handle, "PRAGMA cache_size = "
-                                            + NStr::IntToString(m_CacheSize));
+        + NStr::IntToString(m_PageSize));
+    if (m_CacheSize != (unsigned int)(-1)) {
+        x_ExecuteSql(handle, "PRAGMA cache_size = "
+            + NStr::IntToString(m_CacheSize));
+    }
 
     switch (m_Flags & eAllTemp) {
     case fTempToMemory:
