@@ -1,4 +1,4 @@
-/* $Id: conn_impl.cpp 507084 2016-07-14 16:04:41Z ucko $
+/* $Id: conn_impl.cpp 572640 2018-10-17 16:56:06Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -23,7 +23,7 @@
 *
 * ===========================================================================
 *
-* File Name:  $Id: conn_impl.cpp 507084 2016-07-14 16:04:41Z ucko $
+* File Name:  $Id: conn_impl.cpp 572640 2018-10-17 16:56:06Z ivanov $
 *
 * Author:  Michael Kholodov
 *
@@ -51,6 +51,11 @@
 
 BEGIN_NCBI_SCOPE
 
+NCBI_PARAM_DECL(bool, dbapi, set_xact_abort);
+typedef NCBI_PARAM_TYPE(dbapi, set_xact_abort) TDbapi_SetXactAbort;
+NCBI_PARAM_DEF_EX(bool, dbapi, set_xact_abort, true, eParam_NoThread, NULL);
+
+
 // Implementation
 CConnection::CConnection(CDataSource* ds, EOwnership ownership)
     : m_ds(ds), m_connection(0), m_connCounter(1), m_connUsed(false),
@@ -131,6 +136,7 @@ void CConnection::Connect(const CDBConnParams& params)
     m_connection = m_ds->GetDriverContext()->MakeConnection(params);
     // Explicitly set member value ...
     m_database = m_connection? m_connection->DatabaseName(): string();
+    x_SendXactAbort();
 }
 
 
@@ -266,6 +272,7 @@ IConnection* CConnection::CloneConnection(EOwnership ownership)
     conn->AddListener(m_ds);
     m_ds->AddListener(conn);
 
+    conn->x_SendXactAbort();
     return conn;
 }
 
@@ -526,10 +533,10 @@ void CConnection::MsgToEx(bool v)
 
 CToMultiExHandler* CConnection::GetHandler()
 {
-    if(m_multiExH == 0 ) {
+    if (m_multiExH.Empty()) {
         m_multiExH = new CToMultiExHandler;
     }
-    return m_multiExH;
+    return m_multiExH.GetPointerOrNull();
 }
 
 CDB_MultiEx* CConnection::GetErrorAsEx()
@@ -565,4 +572,58 @@ void CConnection::DeleteConn(CConnection* conn)
     return;
 }
 */
+
+void CConnection::x_SendXactAbort(void)
+{
+    static TDbapi_SetXactAbort  sx_set_xact_abort;
+
+    if (sx_set_xact_abort.Get()) {
+        // Note: SET XACT_ABORT ON is supported by MS SQL and is not supported
+        // by Sybase. So we first check the server brand.
+        bool    is_ms_sql = false;
+
+        unique_ptr<CDB_LangCmd> cmd(m_connection->LangCmd("SELECT @@version"));
+        cmd->Send();
+
+        while (cmd->HasMoreResults()) {
+            unique_ptr<CDB_Result>  result;
+            try {
+                // Note: in case of connecting to PSOS cmd->HasFailed() returns
+                // false however cmd->Result() generates an exception. The way
+                // to identify an error in the command execution without an
+                // exception was NOT found so here are try-catch blocks.
+                // The error code was picked as the one used in PSOS scenario
+                // at lang_cmd.cpp:698 (DATABASE_DRIVER_WARNING macro).
+                result.reset(cmd->Result());
+            } catch (const CDB_ClientEx &  ex) {
+                if (ex.GetDBErrCode() == 120016)
+                    return;
+                throw;
+            }
+
+            if (!result.get())
+                continue;
+
+            while (result->Fetch()) {
+                CDB_VarChar             value;
+                result->GetItem(&value);
+                if (NStr::Find(value.AsString(),
+                               "Microsoft", NStr::eNocase) != NPOS)
+                    is_ms_sql = true;
+            }
+        }
+
+        if (is_ms_sql) {
+            unique_ptr<CDB_LangCmd> xact_cmd(
+                m_connection->LangCmd("SET XACT_ABORT ON"));
+            xact_cmd->Send();
+
+            // Dump results triggers calling the Check() method a few levels
+            // below which properly checks if there were problems during the
+            // command execution. If so then an exception is thrown.
+            xact_cmd->DumpResults();
+        }
+    }
+}
+
 END_NCBI_SCOPE
