@@ -1,4 +1,4 @@
-/* $Id: dbapi_driver_exception_storage.cpp 530219 2017-03-13 14:42:37Z ivanov $
+/* $Id: dbapi_driver_exception_storage.cpp 554054 2017-12-27 18:56:08Z satskyse $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -58,7 +58,9 @@ struct SUnLock
 
 /////////////////////////////////////////////////////////////////////////////
 CDBExceptionStorage::CDBExceptionStorage(void)
-    : m_ClosingConnect(false)
+    : m_ClosingConnect(false),
+      m_Retriable(eRetriable_Unknown),
+      m_HasTimeout(false)
 {
 }
 
@@ -74,12 +76,27 @@ CDBExceptionStorage::~CDBExceptionStorage(void) throw()
 }
 
 
-void CDBExceptionStorage::Accept(CDB_Exception const& e)
+void CDBExceptionStorage::Accept(unique_ptr<CDB_Exception>& e)
 {
     CFastMutexGuard mg(m_Mutex);
 
-    CDB_Exception* ex = e.Clone(); // for debugging ...
-    m_Exceptions.push_back(ex);
+    // Sometimes the very same exception is pushed to the storage via different
+    // paths. To avoid double output in a log file the accumulated list is
+    // checked if there is this exception here already.
+    for (const auto &  item: m_Exceptions) {
+        if (item->GetDBErrCode() == e->GetDBErrCode())
+            if (item->GetMsg() == e->GetMsg())
+                return;     // the same exception has already been registered
+    }
+
+    CDB_Exception *  raw_pointer = e.get();
+    m_Exceptions.push_back(raw_pointer);
+    e.release();
+
+    const CDB_TimeoutEx *  timeout_exc =
+                            dynamic_cast<const CDB_TimeoutEx *>(raw_pointer);
+    if (timeout_exc)
+        m_HasTimeout = true;
 }
 
 
@@ -94,15 +111,32 @@ void CDBExceptionStorage::Handle(const CDBHandlerStack& handler,
         CFastMutexGuard mg(m_Mutex);
         TGuard guard(m_Exceptions);
 
-        handler.HandleExceptions(m_Exceptions, dbg_info, conn, par);
+        ERetriable retriable = m_Retriable;
+        m_Retriable = eRetriable_Unknown;
+
+        m_HasTimeout = false;
+
+        try {
+            handler.HandleExceptions(m_Exceptions, dbg_info, conn, par);
+        } catch (CException &  exc) {
+            exc.SetRetriable(retriable);
+            throw;
+        }
     }
 }
 
+void CDBExceptionStorage::SetRetriable(ERetriable retriable)
+{
+    if (m_Retriable == eRetriable_No)
+        return;
+    m_Retriable = retriable;
+}
+
 }
 
 void s_DelExceptionStorage(impl::CDBExceptionStorage* storage, void* /* data */)
 {
-	delete storage;
+    delete storage;
 }
 
 END_NCBI_SCOPE
