@@ -1,4 +1,4 @@
-/*  $Id: dbapi_impl_context.cpp 548536 2017-10-16 15:16:58Z ivanov $
+/*  $Id: dbapi_impl_context.cpp 570973 2018-09-18 16:55:16Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -58,6 +58,20 @@ NCBI_PARAM_DEF_EX(bool, dbapi, conn_use_encrypt_data, false, eParam_NoThread, NU
 namespace impl
 {
 
+inline
+static bool s_Matches(CConnection* conn, const string& pool_name,
+                      const string& server_name)
+{
+    if (pool_name.empty()) {
+        // There is no server name check here.  We assume that a connection
+        // pool contains connections with appropriate server names only.
+        return conn->ServerName() == server_name
+            ||  conn->GetRequestedServer() == server_name;
+    } else {
+        return conn->PoolName() == pool_name;
+    }
+}
+
 ///////////////////////////////////////////////////////////////////////////
 //  CDriverContext::
 //
@@ -280,7 +294,10 @@ void CDriverContext::x_Recycle(CConnection* conn, bool conn_reusable)
 
 #ifdef NCBI_THREADS
     bool for_sem = false;
-    if ( !m_PoolSemSubject.empty()  &&  conn->PoolName() == m_PoolSemSubject) {
+    if ( !m_PoolSemSubject.empty()
+         &&  (m_PoolSemSubjectHasPoolName
+              ? conn->PoolName() == m_PoolSemSubject
+              : s_Matches(conn, kEmptyStr, m_PoolSemSubject))) {
         for_sem = true;
         m_PoolSemSubject.erase();
         m_PoolSemConn = NULL;
@@ -289,7 +306,31 @@ void CDriverContext::x_Recycle(CConnection* conn, bool conn_reusable)
     }
 #endif
 
+    bool keep = false;
     if (conn_reusable  &&  conn->IsOpeningFinished()  &&  conn->IsValid()) {
+        keep = true;
+        // Tentatively; account for temporary overflow below, bearing
+        // in mind that that conn is already gone from m_InUse and not
+        // yet in m_NotInUse, and as such uncounted.
+        if (conn->m_PoolMaxSize <= m_InUse.size() + m_NotInUse.size()) {
+            unsigned int n;
+            if (conn->m_Pool.empty()) {
+                n = NofConnections(conn->m_RequestedServer);
+            } else {
+                n = NofConnections(kEmptyStr, conn->m_Pool);
+            }
+            if (n >= conn->m_PoolMaxSize) {
+                keep = false;
+            }
+        }
+
+        if (keep && conn->m_ReuseCount + 1 > conn->m_PoolMaxConnUse) {
+            // CXX-4420: limit the number of time the connection is reused
+            keep = false;
+        }
+    }
+
+    if (keep) {
         if (conn->m_PoolIdleTimeParam.GetSign() != eNegative) {
             CTime now(CTime::eCurrent);
             conn->m_CleanupTime = now + conn->m_PoolIdleTimeParam;
@@ -353,7 +394,7 @@ void CDriverContext::CloseUnusedConnections(const string&   srv_name,
     TConnPool::value_type con;
 
     // close all connections first
-    NON_CONST_ITERATE(TConnPool, it, m_NotInUse) {
+    ERASE_ITERATE(TConnPool, it, m_NotInUse) {
         con = *it;
 
         if ( !srv_name.empty()  && srv_name != con->ServerName()
@@ -362,7 +403,6 @@ void CDriverContext::CloseUnusedConnections(const string&   srv_name,
         if((!pool_name.empty()) && pool_name.compare(con->PoolName())) continue;
 
         it = m_NotInUse.erase(it);
-        --it;
         x_AdjustCounts(con, -1);
         delete con;
         if (--max_closings == 0) {
@@ -430,91 +470,35 @@ CDB_Connection* CDriverContext::MakeCDBConnection(CConnection* connection,
     return new CDB_Connection(connection);
 }
 
-class CDBConnParams_Unpooled : public CDBConnParamsDelegate
-{
-public:
-    CDBConnParams_Unpooled(const CDBConnParams& other)
-        : CDBConnParamsDelegate(other)
-        { }
-
-    string GetParam(const string& key) const
-        {
-            if (key == "is_pooled") {
-                return "false";
-            } else {
-                return CDBConnParamsDelegate::GetParam(key);
-            }
-        }
-};
-    
 CDB_Connection*
 CDriverContext::MakePooledConnection(const CDBConnParams& params)
 {
     if (params.GetParam("is_pooled") == "true") {
         CMutexGuard mg(m_PoolMutex);
 
-        string pool_name(params.GetParam("pool_name"));
-        if (!m_NotInUse.empty()) {
-            if (!pool_name.empty()) {
-                // use a pool name
-                ERASE_ITERATE(TConnPool, it, m_NotInUse) {
-                    CConnection* t_con(*it);
+        string pool_name  (params.GetParam("pool_name")),
+               server_name(params.GetServerName());
 
-                    // There is no pool name check here. We assume that a connection
-                    // pool contains connections with appropriate server names only.
-                    if (pool_name == t_con->PoolName()) {
-                        it = m_NotInUse.erase(it);
-                        CDB_Connection* dbcon = MakeCDBConnection(t_con, 0);
-                        if (dbcon->Refresh()) {
-                            /* Future development ...
-                            if (!params.GetDatabaseName().empty()) {
-                                return SetDatabase(dbcon, params);
-                            } else {
-                                return dbcon;
-                            }
-                            */
-
-                            return dbcon;
-                        }
-                        else {
-                            x_AdjustCounts(t_con, -1);
-                            t_con->m_Reusable = false;
-                            delete dbcon;
-                        }
-                    }
+        ERASE_ITERATE(TConnPool, it, m_NotInUse) {
+            CConnection* t_con(*it);
+            if (s_Matches(t_con, pool_name, server_name)) {
+                it = m_NotInUse.erase(it);
+                CDB_Connection* dbcon = MakeCDBConnection(t_con, 0);
+                if (dbcon->Refresh()) {
+                    /* Future development ...
+                       if (!params.GetDatabaseName().empty()) {
+                           return SetDatabase(dbcon, params);
+                       } else {
+                           return dbcon;
+                       }
+                    */
+ 
+                    return dbcon;
                 }
-            }
-            else {
-                string server_name = params.GetServerName();
-                if ( server_name.empty() ) {
-                    return NULL;
-                }
-
-                // try to use a server name
-                ERASE_ITERATE(TConnPool, it, m_NotInUse) {
-                    CConnection* t_con(*it);
-
-                    if (server_name == t_con->ServerName()
-                        ||  server_name == t_con->GetRequestedServer()) {
-                        it = m_NotInUse.erase(it);
-                        CDB_Connection* dbcon = MakeCDBConnection(t_con, 0);
-                        if (dbcon->Refresh()) {
-                            /* Future development ...
-                            if (!params.GetDatabaseName().empty()) {
-                                return SetDatabase(dbcon, params);
-                            } else {
-                                return dbcon;
-                            }
-                            */
-
-                            return dbcon;
-                        }
-                        else {
-                            x_AdjustCounts(t_con, -1);
-                            t_con->m_Reusable = false;
-                            delete dbcon;
-                        }
-                    }
+                else {
+                    x_AdjustCounts(t_con, -1);
+                    t_con->m_Reusable = false;
+                    delete dbcon;
                 }
             }
         }
@@ -528,8 +512,9 @@ CDriverContext::MakePooledConnection(const CDBConnParams& params)
                 int total_cnt = 0;
                 ITERATE(TConnPool, it, m_InUse) {
                     CConnection* t_con(*it);
-                    if (pool_name == t_con->PoolName())
+                    if (s_Matches(t_con, pool_name, server_name)) {
                         ++total_cnt;
+                    }
                 }
                 if (total_cnt >= pool_max) {
 #ifdef NCBI_THREADS
@@ -539,7 +524,13 @@ CDriverContext::MakePooledConnection(const CDBConnParams& params)
                         timeout_val = NStr::StringToDouble(timeout_str);
                     }
                     CTimeout timeout(timeout_val);
-                    m_PoolSemSubject = pool_name;
+                    if (pool_name.empty()) {
+                        m_PoolSemSubject = server_name;
+                        m_PoolSemSubjectHasPoolName = false;
+                    } else {
+                        m_PoolSemSubject = pool_name;
+                        m_PoolSemSubjectHasPoolName = true;
+                    }
                     mg.Release();
                     if (m_PoolSem.TryWait(timeout)) {
                         mg.Guard(m_PoolMutex);
@@ -558,10 +549,7 @@ CDriverContext::MakePooledConnection(const CDBConnParams& params)
                     } else
 #endif
                     if (params.GetParam("pool_allow_temp_overflow")
-                        == "true") {
-                        return MakePooledConnection
-                            (CDBConnParams_Unpooled(params));
-                    } else {
+                        != "true") {
                         return NULL;
                     }
                 }
@@ -574,16 +562,12 @@ CDriverContext::MakePooledConnection(const CDBConnParams& params)
     }
 
     // Precondition check.
-    if (params.GetServerName().empty() ||
-        (!TDbapi_CanUseKerberos::GetDefault()
-         &&  (params.GetUserName().empty()
-              ||  params.GetPassword().empty())))
+    if (!TDbapi_CanUseKerberos::GetDefault()
+        &&  (params.GetUserName().empty()
+             ||  params.GetPassword().empty()))
     {
-        string err_msg("Insufficient info/credentials to connect.");
+        string err_msg("Insufficient credentials to connect.");
 
-        if (params.GetServerName().empty()) {
-            err_msg += " Server name has not been set.";
-        }
         if (params.GetUserName().empty()) {
             err_msg += " User name has not been set.";
         }
@@ -819,6 +803,7 @@ SDBConfParams::Clear(void)
     pool_idle_time.clear();
     pool_wait_time.clear();
     pool_allow_temp_overflow.clear();
+    pool_max_conn_use.clear();
     args.clear();
 }
 
@@ -938,6 +923,12 @@ CDriverContext::ReadDBConfParams(const string&  service_name,
         params->continue_after_raiserror
             = reg.Get(section_name, "continue_after_raiserror");
     }
+    if (reg.HasEntry(section_name, "conn_pool_max_conn_use",
+                     IRegistry::fCountCleared)) {
+        params->flags += SDBConfParams::fPoolMaxConnUseSet;
+        params->pool_max_conn_use
+            = reg.Get(section_name, "conn_pool_max_conn_use");
+    }
     if (reg.HasEntry(section_name, "args", IRegistry::fCountCleared)) {
         params->flags += SDBConfParams::fArgsSet;
         params->args = reg.Get(section_name, "args");
@@ -956,11 +947,12 @@ CDriverContext::SatisfyPoolMinimum(const CDBConnParams& params)
     if (pool_min <= 0)
         return true;
 
-    string pool_name = params.GetParam("pool_name");
+    string pool_name   = params.GetParam("pool_name"),
+           server_name = params.GetServerName();
     int total_cnt = 0;
     ITERATE(TConnPool, it, m_InUse) {
         CConnection* t_con(*it);
-        if (t_con->IsReusable()  &&  pool_name == t_con->PoolName()
+        if (t_con->IsReusable()  &&  s_Matches(t_con, pool_name, server_name)
             &&  t_con->IsValid()  &&  t_con->IsAlive())
         {
             ++total_cnt;
@@ -968,7 +960,7 @@ CDriverContext::SatisfyPoolMinimum(const CDBConnParams& params)
     }
     ITERATE(TConnPool, it, m_NotInUse) {
         CConnection* t_con(*it);
-        if (t_con->IsReusable()  &&  pool_name == t_con->PoolName()
+        if (t_con->IsReusable()  &&  s_Matches(t_con, pool_name, server_name)
             &&  t_con->IsAlive())
         {
             ++total_cnt;
@@ -993,6 +985,7 @@ CDriverContext::MakeConnection(const CDBConnParams& params)
     CMutexGuard mg(x_GetCtxMtx());
 
     CMakeConnActualParams act_params(params);
+
     SDBConfParams conf_params;
     conf_params.Clear();
     if (params.GetParam("do_not_read_conf") != "true") {
@@ -1115,7 +1108,7 @@ CDriverContext::MakeConnection(const CDBConnParams& params)
             }
             else {
                 act_params.SetParam
-                    ("pool_allow_temp_overflow", 
+                    ("pool_allow_temp_overflow",
                      NStr::BoolToString(
                          NStr::StringToBool(
                              conf_params.pool_allow_temp_overflow)));
@@ -1124,6 +1117,12 @@ CDriverContext::MakeConnection(const CDBConnParams& params)
         else if (params.GetParam("pool_allow_temp_overflow") == "default") {
             act_params.SetParam("pool_allow_temp_overflow", "false");
         }
+        if (conf_params.IsPoolMaxConnUseSet())
+            act_params.SetParam("pool_max_conn_use",
+                                conf_params.pool_max_conn_use);
+        else if (params.GetParam("pool_max_conn_use") == "default") {
+            act_params.SetParam("pool_max_conn_use", "0");
+        }
         if (conf_params.IsContinueAfterRaiserrorSet()) {
             if (conf_params.continue_after_raiserror.empty()) {
                 act_params.SetParam("continue_after_raiserror", "false");
@@ -1159,6 +1158,7 @@ CDriverContext::MakeConnection(const CDBConnParams& params)
             err += "' as user '" + act_params.GetUserName() + "'";
 
             CDB_ClientEx ex(DIAG_COMPILE_INFO, NULL, err, eDiag_Error, 100011);
+            ex.SetRetriable(eRetriable_No);
             CDB_UserHandler::TExceptions* expts = factory->GetExceptions();
             if (expts) {
                 NON_CONST_REVERSE_ITERATE(CDB_UserHandler::TExceptions, it, *expts) {
@@ -1183,24 +1183,32 @@ CDriverContext::MakeConnection(const CDBConnParams& params)
     return t_con.release();
 }
 
-void CDriverContext::CloseConnsForPool(const string& pool_name)
+size_t CDriverContext::CloseConnsForPool(const string& pool_name,
+                                         Uint4 keep_host_ip, Uint2 keep_port)
 {
+    size_t      invalidated_count = 0;
     CMutexGuard mg(m_PoolMutex);
 
     ITERATE(TConnPool, it, m_InUse) {
         CConnection* t_con(*it);
         if (t_con->IsReusable()  &&  pool_name == t_con->PoolName()) {
-            t_con->Invalidate();
+            if (t_con->Host() != keep_host_ip || t_con->Port() != keep_port) {
+                t_con->Invalidate();
+                ++invalidated_count;
+            }
         }
     }
     ERASE_ITERATE(TConnPool, it, m_NotInUse) {
         CConnection* t_con(*it);
         if (t_con->IsReusable()  &&  pool_name == t_con->PoolName()) {
-            m_NotInUse.erase(it);
-            x_AdjustCounts(t_con, -1);
-            delete t_con;
+            if (t_con->Host() != keep_host_ip || t_con->Port() != keep_port) {
+                m_NotInUse.erase(it);
+                x_AdjustCounts(t_con, -1);
+                delete t_con;
+            }
         }
     }
+    return invalidated_count;
 }
 
 
@@ -1212,18 +1220,30 @@ void CDriverContext::CloseOldIdleConns(unsigned int max_closings,
         return;
     }
 
-    set<string> at_min;
+    set<string> at_min_by_pool, at_min_by_server;
     CTime now(CTime::eCurrent);
     ERASE_ITERATE (TConnPool, it, m_NotInUse) {
         const string& pool_name_2 = (*it)->PoolName();
-        if (pool_name_2.empty()
-            ||  ( !pool_name.empty()  &&  pool_name != pool_name_2)
-            ||  at_min.find(pool_name_2) != at_min.end()
-            ||  (*it)->m_CleanupTime.IsEmpty()
-            ||  (*it)->m_CleanupTime > now) {
+        set<string>& at_min
+            = pool_name_2.empty() ? at_min_by_server : at_min_by_pool;
+        if (pool_name_2.empty()) {
+            if ( !pool_name.empty()
+                ||  at_min.find((*it)->GetRequestedServer()) != at_min.end()) {
+                continue;
+            }
+        } else if (( !pool_name.empty()  &&  pool_name != pool_name_2)
+                   ||  at_min.find(pool_name_2) != at_min.end()) {
             continue;
         }
-        unsigned int n = NofConnections(TSvrRef(), pool_name_2);
+        if ((*it)->m_CleanupTime.IsEmpty()  ||  (*it)->m_CleanupTime > now) {
+            continue;
+        }
+        unsigned int n;
+        if (pool_name_2.empty()) {
+            n = NofConnections((*it)->GetRequestedServer());
+        } else {
+            n = NofConnections(TSvrRef(), pool_name_2);
+        }
         if (n > (*it)->m_PoolMinSize) {
             x_AdjustCounts(*it, -1);
             delete *it;
