@@ -1,4 +1,4 @@
-/* $Id: parameters.cpp 487444 2015-12-17 18:38:53Z ucko $
+/* $Id: parameters.cpp 535289 2017-05-08 13:38:35Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -168,7 +168,7 @@ bool CDB_Params::SetParam(unsigned int param_no, const string& param_name,
 
 CDB_Params* CDB_Params::SemiShallowClone(void) const
 {
-    auto_ptr<CDB_Params> result(new CDB_Params);
+    unique_ptr<CDB_Params> result(new CDB_Params);
     SParam empty_param;
     ITERATE (deque<SParam>, it, m_Params) {
         result->m_Params.push_back(empty_param);
