@@ -1,4 +1,4 @@
-/* $Id: stmt_impl.cpp 497635 2016-04-08 13:52:30Z ucko $
+/* $Id: stmt_impl.cpp 535289 2017-05-08 13:38:35Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -23,7 +23,7 @@
 *
 * ===========================================================================
 *
-* File Name:  $Id: stmt_impl.cpp 497635 2016-04-08 13:52:30Z ucko $
+* File Name:  $Id: stmt_impl.cpp 535289 2017-05-08 13:38:35Z ivanov $
 *
 * Author:  Michael Kholodov
 *
@@ -364,7 +364,7 @@ void CStatement::PurgeResults()
     while (HasMoreResults())
     {
         if (HasRows()) {
-            auto_ptr<IResultSet> rs( GetResultSet() );
+            unique_ptr<IResultSet> rs( GetResultSet() );
             if (rs.get()) {
                 // Is it necessary???
                 while (rs->Next()) {
