@@ -1,4 +1,4 @@
-/* $Id: variant.cpp 498292 2016-04-14 19:07:55Z ucko $
+/* $Id: variant.cpp 563801 2018-05-15 15:55:00Z ucko $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -23,7 +23,7 @@
 *
 * ===========================================================================
 *
-* File Name:  $Id: variant.cpp 498292 2016-04-14 19:07:55Z ucko $
+* File Name:  $Id: variant.cpp 563801 2018-05-15 15:55:00Z ucko $
 *
 * Author:  Michael Kholodov
 *
@@ -168,6 +168,22 @@ CVariant CVariant::DateTime(CTime *p)
     return CVariant(p ? new CDB_DateTime(*p) : new CDB_DateTime());
 }
 
+CVariant CVariant::BigDateTime(CTime *p, EDateTimeFormat fmt)
+{
+    CDB_BigDateTime::ESQLType sql_type;
+    switch (fmt) {
+    case eLonger:         sql_type = CDB_BigDateTime::eDateTime;        break;
+    case eDateOnly:       sql_type = CDB_BigDateTime::eDate;            break;
+    case eTimeOnly:       sql_type = CDB_BigDateTime::eTime;            break;
+    case eDateTimeOffset: sql_type = CDB_BigDateTime::eDateTimeOffset;  break;
+    default:
+        NCBI_THROW(CVariantException, eVariant | Retriable(eRetriable_No),
+                   FORMAT("Unsupported BigDateTime format " << fmt));
+    }
+    return CVariant(p ? new CDB_BigDateTime(*p, sql_type)
+                    : new CDB_BigDateTime(CTime::eEmpty, sql_type));
+}
+
 CVariant CVariant::Numeric(unsigned int precision,
                            unsigned int scale,
                            const char* p)
@@ -196,7 +212,9 @@ CVariant::CVariant(EDB_Type type, size_t size)
     case eDB_LongChar:
         if( size == 0 )
         {
-            NCBI_THROW(CVariantException, eVariant, "Illegal argument, the size of LONGCHAR should not be 0");
+            NCBI_THROW(CVariantException,
+                       eVariant | Retriable(eRetriable_No),
+                       "Illegal argument, the size of LONGCHAR should not be 0");
         }
         m_data = new CDB_LongChar(size);
         return;
@@ -206,14 +224,16 @@ CVariant::CVariant(EDB_Type type, size_t size)
     case eDB_Char:
         if( size == 0 )
         {
-            NCBI_THROW(CVariantException, eVariant, "Illegal argument, the size of CHAR should not be 0");
+            NCBI_THROW(CVariantException, eVariant | Retriable(eRetriable_No),
+                       "Illegal argument, the size of CHAR should not be 0");
         }
         m_data = new CDB_Char(size);
         return;
     case eDB_LongBinary:
         if( size == 0 )
         {
-            NCBI_THROW(CVariantException, eVariant, "Illegal argument, the size of LONGBINARY should not be 0");
+            NCBI_THROW(CVariantException, eVariant | Retriable(eRetriable_No),
+                       "Illegal argument, the size of LONGBINARY should not be 0");
         }
         m_data = new CDB_LongBinary(size);
         return;
@@ -223,7 +243,8 @@ CVariant::CVariant(EDB_Type type, size_t size)
     case eDB_Binary:
         if( size == 0 )
         {
-            NCBI_THROW(CVariantException, eVariant, "Illegal argument, the size of BINARY should not be 0");
+            NCBI_THROW(CVariantException, eVariant | Retriable(eRetriable_No),
+                       "Illegal argument, the size of BINARY should not be 0");
         }
         m_data = new CDB_Binary(size);
         return;
@@ -236,6 +257,9 @@ CVariant::CVariant(EDB_Type type, size_t size)
     case eDB_DateTime:
         m_data = new CDB_DateTime();
         return;
+    case eDB_BigDateTime:
+        m_data = new CDB_BigDateTime();
+        return;
     case eDB_SmallDateTime:
         m_data = new CDB_SmallDateTime();
         return;
@@ -260,7 +284,7 @@ CVariant::CVariant(EDB_Type type, size_t size)
     case eDB_UnsupportedType:
         break;
     }
-    NCBI_THROW(CVariantException, eVariant,
+    NCBI_THROW(CVariantException, eVariant | Retriable(eRetriable_No),
                string("Unsupported type: ")
                + CDB_Object::GetTypeName(type, false));
 }
@@ -324,8 +348,20 @@ CVariant::CVariant(const CTime& v, EDateTimeFormat fmt)
     case eLong:
         m_data = new CDB_DateTime(v);
         break;
+    case eLonger:
+        m_data = new CDB_BigDateTime(v);
+        break;
+    case eDateOnly:
+        m_data = new CDB_BigDateTime(v, CDB_BigDateTime::eDate);
+        break;
+    case eTimeOnly:
+        m_data = new CDB_BigDateTime(v, CDB_BigDateTime::eTime);
+        break;
+    case eDateTimeOffset:
+        m_data = new CDB_BigDateTime(v, CDB_BigDateTime::eDateTimeOffset);
+        break;
     default:
-        NCBI_THROW(CVariantException, eVariant,
+        NCBI_THROW(CVariantException, eVariant | Retriable(eRetriable_No),
                    "CVariant::ctor(): unsupported datetime type "
                    + NStr::IntToString(fmt));
     }
@@ -355,7 +391,8 @@ CVariant::~CVariant(void)
 
 CDB_Object* CVariant::GetNonNullData() const {
     if( m_data == 0 )
-        NCBI_THROW(CVariantException, eVariant, "CVariant::GetNonNullData(): null data");
+        NCBI_THROW(CVariantException, eVariant | Retriable(eRetriable_No),
+                   "CVariant::GetNonNullData(): null data");
 
     return m_data;
 }
@@ -440,6 +477,7 @@ string CVariant::GetString(void) const
                 s = ((CDB_Numeric*)GetData())->Value();
                 break;
             case eDB_DateTime:
+            case eDB_BigDateTime:
             case eDB_SmallDateTime:
                 s = GetCTime().AsString();
                 break;
@@ -602,6 +640,9 @@ const CTime& CVariant::GetCTime() const
     case eDB_DateTime:
         ptr = const_cast<CTime*>(&((CDB_DateTime*)GetData())->Value());
         break;
+    case eDB_BigDateTime:
+        ptr = const_cast<CTime*>(&((CDB_BigDateTime*)GetData())->GetCTime());
+        break;
     case eDB_SmallDateTime:
         ptr = const_cast<CTime*>(&((CDB_SmallDateTime*)GetData())->Value());
         break;
@@ -823,6 +864,9 @@ CVariant& CVariant::operator=(const CTime& v)
     case eDB_DateTime:
         *((CDB_DateTime*)GetData()) = v;
         break;
+    case eDB_BigDateTime:
+        *((CDB_BigDateTime*)GetData()) = v;
+        break;
     case eDB_SmallDateTime:
         *((CDB_SmallDateTime*)GetData()) = v;
         break;
@@ -847,7 +891,7 @@ bool operator<(const CVariant& v1, const CVariant& v2)
     }
     else {
         if( v1.GetType() != v2.GetType() ) {
-            NCBI_THROW(CVariantException, eVariant,
+            NCBI_THROW(CVariantException, eVariant | Retriable(eRetriable_No),
                        string("Cannot compare different types ")
                        + CDB_Object::GetTypeName(v1.GetType(), false) + " and "
                        + CDB_Object::GetTypeName(v2.GetType(), false));
@@ -878,11 +922,12 @@ bool operator<(const CVariant& v1, const CVariant& v2)
             less = v1.GetDouble() < v2.GetDouble();
             break;
         case eDB_DateTime:
+        case eDB_BigDateTime:
         case eDB_SmallDateTime:
             less = v1.GetCTime() < v2.GetCTime();
             break;
         default:
-            NCBI_THROW(CVariantException, eVariant,
+            NCBI_THROW(CVariantException, eVariant | Retriable(eRetriable_No),
                        string("Type not supported: ")
                        + CDB_Object::GetTypeName(v1.GetType(), false));
         }
@@ -899,7 +944,7 @@ bool operator==(const CVariant& v1, const CVariant& v2)
     }
     else {
         if( v1.GetType() != v2.GetType() ) {
-            NCBI_THROW(CVariantException, eVariant,
+            NCBI_THROW(CVariantException, eVariant | Retriable(eRetriable_No),
                        string("Cannot compare different types ")
                        + CDB_Object::GetTypeName(v1.GetType(), false) + " and "
                        + CDB_Object::GetTypeName(v2.GetType(), false));
@@ -935,11 +980,12 @@ bool operator==(const CVariant& v1, const CVariant& v2)
             less = v1.GetDouble() == v2.GetDouble();
             break;
         case eDB_DateTime:
+        case eDB_BigDateTime:
         case eDB_SmallDateTime:
             less = v1.GetCTime() == v2.GetCTime();
             break;
         default:
-            NCBI_THROW(CVariantException, eVariant,
+            NCBI_THROW(CVariantException, eVariant | Retriable(eRetriable_No),
                        string("Type not supported: ")
                        + CDB_Object::GetTypeName(v1.GetType(), false));
         }
