@@ -1,7 +1,7 @@
 #ifndef OBJECTS_OBJMGR___SEQ_ID_TREE__HPP
 #define OBJECTS_OBJMGR___SEQ_ID_TREE__HPP
 
-/*  $Id: seq_id_tree.hpp 535901 2017-05-12 18:05:49Z vasilche $
+/*  $Id: seq_id_tree.hpp 574508 2018-11-14 16:02:44Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -89,7 +89,27 @@ struct PHashNocase {
 struct PEqualNocase {
     bool operator()(const string& s1, const string& s2) const
         {
-            return NStr::EqualNocase(s1, s2);
+            // in most cases letter cases match,
+            // so it's faster first to check that
+            // with more efficient direct string comparison
+            if ( s1 == s2 ) {
+                return true;
+            }
+            // otherwise we first check if lengths are the same
+            size_t len = s1.size();
+            if ( s2.size() != len ) {
+                return false;
+            }
+            for ( size_t i = 0; i < len; ++i ) {
+                char c1 = s1[i];
+                char c2 = s2[i];
+                if ( tolower((unsigned char)c1) != tolower((unsigned char)c2) ) {
+                    return false;
+                }
+            }
+            return true;
+            // commented out old less efficient comparison
+            //return NStr::EqualNocase(s1, s2);
         }
 };
 
@@ -158,6 +178,10 @@ protected:
         {
             return id.m_Info;
         }
+    static const CSeq_id* GetSeqId(const CSeq_id_Info* info)
+        {
+            return info->m_Seq_id.GetPointerOrNull();
+        }
     virtual void x_Unindex(const CSeq_id_Info* info) = 0;
 
     typedef CFastMutex TTreeLock;
@@ -276,12 +300,9 @@ protected:
 class CSeq_id_Gi_Info : public CSeq_id_Info
 {
 public:
-    CSeq_id_Gi_Info(CSeq_id_Mapper* mapper)
-        : CSeq_id_Info(CSeq_id::e_Gi, mapper)
-        {
-        }
+    CSeq_id_Gi_Info(CSeq_id_Mapper* mapper);
     
-    virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed) const;
+    virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant /*variant*/) const;
 };
 
 
@@ -297,8 +318,6 @@ public:
     virtual CSeq_id_Handle FindOrCreate(const CSeq_id& id);
     virtual CSeq_id_Handle GetGiHandle(TGi gi);
 
-    virtual void DropInfo(const CSeq_id_Info* info);
-
     virtual void FindMatchStr(const string& sid,
                               TSeq_id_MatchList& id_list) const;
 
@@ -311,8 +330,8 @@ protected:
     bool x_Check(const CSeq_id& id) const;
     TGi x_Get(const CSeq_id& id) const;
 
-    CConstRef<CSeq_id_Info> m_ZeroInfo;
-    CConstRef<CSeq_id_Info> m_SharedInfo;
+    CSeq_id_Info* m_ZeroInfo;
+    CSeq_id_Info* m_SharedInfo;
 };
 
 
@@ -321,6 +340,18 @@ protected:
 // e_Ddbj, e_Prf, e_Tpg, e_Tpe, e_Tpd trees
 
 
+class CSeq_id_Textseq_PlainInfo : public CSeq_id_Info
+{
+public:
+    CSeq_id_Textseq_PlainInfo(const CConstRef<CSeq_id>& seq_id, CSeq_id_Mapper* mapper);
+    
+    virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const;
+    
+    TVariant ParseCaseVariant(const string& acc) const;
+    TVariant ParseCaseVariant(const CTextseq_id& id) const;
+};
+
+
 class CSeq_id_Textseq_Info : public CSeq_id_Info {
 public:
     typedef CTextseq_id::TVersion TVersion;
@@ -333,13 +364,17 @@ public:
 
         unsigned m_Hash;
         TVersion m_Version;
-        string m_Prefix;
+        Uint1 m_PrefixLen;
+        enum {
+            kMaxPrefixLen = 7
+        };
+        char m_PrefixBuf[kMaxPrefixLen];
 
         DECLARE_OPERATOR_BOOL(m_Hash != 0);
 
         bool operator==(const TKey& b) const {
             return m_Hash == b.m_Hash && m_Version == b.m_Version &&
-                NStr::EqualNocase(m_Prefix, b.m_Prefix);
+                NStr::EqualNocase(GetAccPrefix(), b.GetAccPrefix());
         }
         bool operator!=(const TKey& b) const {
             return !(*this == b);
@@ -349,7 +384,7 @@ public:
                 (m_Hash == b.m_Hash &&
                  (m_Version < b.m_Version ||
                   (m_Version == b.m_Version &&
-                   NStr::CompareNocase(m_Prefix, b.m_Prefix) < 0)));
+                   NStr::CompareNocase(GetAccPrefix(), b.GetAccPrefix()) < 0)));
         }
 
         bool SameHash(const TKey& b) const {
@@ -360,7 +395,7 @@ public:
         }
         bool EqualAcc(const TKey& b) const {
             return SameHashNoVer(b) &&
-                NStr::EqualNocase(m_Prefix, b.m_Prefix);
+                NStr::EqualNocase(GetAccPrefix(), b.GetAccPrefix());
         }
 
         bool IsSetVersion(void) const {
@@ -381,6 +416,14 @@ public:
         int GetAccDigits(void) const {
             return (m_Hash & 0xff) >> 1;
         }
+        TVariant ParseCaseVariant(const string& acc) const;
+        
+        size_t GetPrefixLen() const {
+            return m_PrefixLen;
+        }
+        CTempString GetAccPrefix(void) const {
+            return CTempString(m_PrefixBuf, m_PrefixLen);
+        }
     };
     CSeq_id_Textseq_Info(CSeq_id::E_Choice type,
                          CSeq_id_Mapper* mapper,
@@ -390,10 +433,10 @@ public:
     const TKey& GetKey(void) const {
         return m_Key;
     }
-    const string& GetAccPrefix(void) const {
-        return m_Key.m_Prefix;
+    CTempString GetAccPrefix(void) const {
+        return m_Key.GetAccPrefix();
     }
-    bool GoodPrefix(const string& acc) const {
+    bool GoodPrefix(const CTempString& acc) const {
         return NStr::StartsWith(acc, GetAccPrefix(), NStr::eNocase);
     }
     int GetAccDigits(void) const {
@@ -405,22 +448,24 @@ public:
     const TVersion& GetVersion(void) const {
         return m_Key.GetVersion();
     }
-    void RestoreAccession(string& acc, TPacked param) const;
-    void Restore(CTextseq_id& id, TPacked param) const;
+    void RestoreAccession(string& acc, TPacked param, TVariant variant) const;
+    void Restore(CTextseq_id& id, TPacked param, TVariant variant) const;
 
     static TKey ParseAcc(const string& acc, const TVersion* ver);
-    static TKey ParseAcc(const string& acc, const CTextseq_id* tid) {
-        TVersion ver, *ver_ptr = 0;
-        if ( tid && tid->IsSetVersion() ) {
-            ver = tid->GetVersion();
+    static TKey ParseAcc(const string& acc, const CTextseq_id& tid) {
+        TVersion ver;
+        const TVersion *ver_ptr = 0;
+        if ( tid.IsSetVersion() ) {
+            ver = tid.GetVersion();
             ver_ptr = &ver;
         }
         return ParseAcc(acc, ver_ptr);
     }
     static TPacked Pack(const TKey& key, const string& acc);
     static TPacked Pack(const TKey& key, const CTextseq_id& id);
+    static TVariant ParseCaseVariant(const CSeq_id_Info* info, const string& acc);
     
-    virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed) const;
+    virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const;
     
 private:
     TKey m_Key;
@@ -468,12 +513,12 @@ protected:
         _ASSERT(text_id);
         return *text_id;
     }
-    CSeq_id_Info* x_FindStrInfo(CSeq_id::E_Choice type,
-                                const CTextseq_id& tid) const;
+    CSeq_id_Textseq_PlainInfo* x_FindStrInfo(CSeq_id::E_Choice type,
+                                             const CTextseq_id& tid) const;
     bool x_GetVersion(TVersion& version, const CSeq_id_Handle& id) const;
 
 private:
-    typedef multimap<string, CSeq_id_Info*, PNocase> TStringMap;
+    typedef multimap<string, CSeq_id_Textseq_PlainInfo*, PNocase> TStringMap;
     typedef TStringMap::value_type TStringMapValue;
     typedef TStringMap::const_iterator TStringMapCI;
     typedef pair<TStringMapCI, TStringMapCI> TVersions;
@@ -488,10 +533,10 @@ private:
                         const string& key,
                         const CSeq_id_Info* info);
 
-    CSeq_id_Info* x_FindStrInfo(const TStringMap& str_map,
-                                const string& str,
-                                CSeq_id::E_Choice type,
-                                const CTextseq_id& tid) const;
+    CSeq_id_Textseq_PlainInfo* x_FindStrInfo(const TStringMap& str_map,
+                                             const string& str,
+                                             CSeq_id::E_Choice type,
+                                             const CTextseq_id& tid) const;
 
     void x_FindMatchByAcc(TSeq_id_MatchList& id_list,
                           const string& acc,
@@ -656,6 +701,33 @@ public:
 // e_Local tree
 
 
+class CSeq_id_Local_Info : public CSeq_id_Info {
+public:
+    CSeq_id_Local_Info(const CObject_id& oid, CSeq_id_Mapper* mapper);
+    ~CSeq_id_Local_Info(void);
+
+    bool IsId() const {
+        return m_IsId;
+    }
+    bool HasMatchingId() const {
+        return m_HasMatchingId;
+    }
+    CObject_id::TId GetMatchingId() const {
+        return m_MatchingId;
+    }
+    
+    TVariant ParseCaseVariant(const string& str) const;
+    TVariant ParseCaseVariant(const CObject_id& oid) const;
+    
+    virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const;
+    
+private:
+    bool m_IsId;
+    bool m_HasMatchingId;
+    CObject_id::TId m_MatchingId;
+};
+
+
 class CSeq_id_Local_Tree : public CSeq_id_Which_Tree
 {
 public:
@@ -667,6 +739,10 @@ public:
     virtual CSeq_id_Handle FindInfo(const CSeq_id& id) const;
     virtual CSeq_id_Handle FindOrCreate(const CSeq_id& id);
 
+    // Get the list of matching seq-id (int id = str id).
+    virtual bool HaveMatch(const CSeq_id_Handle& id) const;
+    virtual void FindMatch(const CSeq_id_Handle& id,
+                           TSeq_id_MatchList& id_list) const;
     virtual void FindMatchStr(const string& sid,
                               TSeq_id_MatchList& id_list) const;
 
@@ -676,10 +752,12 @@ public:
 
 private:
     virtual void x_Unindex(const CSeq_id_Info* info);
-    CSeq_id_Info* x_FindInfo(const CObject_id& oid) const;
-
-    typedef unordered_map<string, CSeq_id_Info*, PHashNocase, PEqualNocase> TByStr;
-    typedef map<TPacked, CSeq_id_Info*>         TById;
+    CSeq_id_Local_Info* x_FindInfo(const CObject_id& oid) const;
+    CSeq_id_Local_Info* x_FindStrInfo(const string& str) const;
+    CSeq_id_Local_Info* x_FindIdInfo(CObject_id::TId id) const;
+    
+    typedef unordered_map<string, CSeq_id_Local_Info*, PHashNocase, PEqualNocase> TByStr;
+    typedef map<CObject_id::TId, CSeq_id_Local_Info*>         TById;
 
     TByStr m_ByStr;
     TById  m_ById;
@@ -704,12 +782,11 @@ public:
     const string& GetDbtag(void) const {
         return m_Key;
     }
-    void Restore(CDbtag& id, TPacked param) const;
+    void Restore(CDbtag& id, TPacked param, TVariant variant) const;
 
-    static TKey Parse(const CDbtag& id);
     static TPacked Pack(const TKey& key, const CDbtag& id);
     
-    virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed) const;
+    virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const;
     
 private:
     TKey m_Key;
@@ -719,16 +796,15 @@ private:
 class CSeq_id_General_Str_Info : public CSeq_id_Info {
 public:
     struct TKey {
-        // all upper case
         TPacked m_Key;
         string m_Db;
         string m_StrPrefix;
         string m_StrSuffix;
         bool operator==(const TKey& b) const {
             return m_Key == b.m_Key &&
-                NStr::EqualNocase(m_StrSuffix, b.m_StrSuffix) &&
-                NStr::EqualNocase(m_StrPrefix, b.m_StrPrefix) &&
-                NStr::EqualNocase(m_Db, b.m_Db);
+                PEqualNocase()(m_StrSuffix, b.m_StrSuffix) &&
+                PEqualNocase()(m_StrPrefix, b.m_StrPrefix) &&
+                PEqualNocase()(m_Db, b.m_Db);
         }
         bool operator!=(const TKey& b) const {
             return !(*this == b);
@@ -736,6 +812,7 @@ public:
         size_t GetStrDigits(void) const {
             return m_Key & 0xff;
         }
+        TVariant ParseCaseVariant(const CDbtag& dbtag) const;
     };
     struct PKeyLess {
         bool operator()(const TKey& a, const TKey& b) const {
@@ -752,6 +829,11 @@ public:
             return diff < 0;
         }
     };
+    struct PHash {
+        TPacked operator()(const TKey& a) const {
+            return a.m_Key;
+        }
+    };
 
     CSeq_id_General_Str_Info(CSeq_id_Mapper* mapper, const TKey& key);
     ~CSeq_id_General_Str_Info(void);
@@ -771,18 +853,28 @@ public:
     size_t GetStrDigits(void) const {
         return m_Key.GetStrDigits();
     }
-    void Restore(CDbtag& id, TPacked param) const;
+    void Restore(CDbtag& id, TPacked param, TVariant variant) const;
 
     static TKey Parse(const CDbtag& id);
     static TPacked Pack(const TKey& key, const CDbtag& id);
     
-    virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed) const;
+    virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const;
     
 private:
     TKey m_Key;
 };
 
 
+class CSeq_id_General_PlainInfo : public CSeq_id_Info {
+public:
+    CSeq_id_General_PlainInfo(const CDbtag& dbid, CSeq_id_Mapper* mapper);
+    
+    virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const;
+    
+    TVariant ParseCaseVariant(const CDbtag& dbtag) const;
+};
+
+
 class CSeq_id_General_Tree : public CSeq_id_Which_Tree
 {
 public:
@@ -807,12 +899,12 @@ public:
 
 private:
     virtual void x_Unindex(const CSeq_id_Info* info);
-    CSeq_id_Info* x_FindInfo(const CDbtag& dbid) const;
+    CSeq_id_General_PlainInfo* x_FindInfo(const CDbtag& dbid) const;
 
     struct STagMap {
     public:
-        typedef unordered_map<string, CSeq_id_Info*, PHashNocase, PEqualNocase> TByStr;
-        typedef map<TPacked, CSeq_id_Info*>         TById;
+        typedef unordered_map<string, CSeq_id_General_PlainInfo*, PHashNocase, PEqualNocase> TByStr;
+        typedef map<TPacked, CSeq_id_General_PlainInfo*>         TById;
         TByStr m_ByStr;
         TById  m_ById;
     };
@@ -821,8 +913,8 @@ private:
     typedef map<TPackedIdKey, CConstRef<CSeq_id_General_Id_Info>,
                 CSeq_id_General_Id_Info::PKeyLess> TPackedIdMap;
     typedef CSeq_id_General_Str_Info::TKey TPackedStrKey;
-    typedef map<TPackedStrKey, CConstRef<CSeq_id_General_Str_Info>,
-                CSeq_id_General_Str_Info::PKeyLess> TPackedStrMap;
+    typedef unordered_map<TPackedStrKey, CConstRef<CSeq_id_General_Str_Info>,
+                          CSeq_id_General_Str_Info::PHash> TPackedStrMap;
 
     TDbMap m_DbMap;
     TPackedIdMap m_PackedIdMap;
@@ -935,7 +1027,6 @@ public:
 
 private:
     virtual void x_Unindex(const CSeq_id_Info* info);
-    CSeq_id_Info* x_FindInfo(const CPDB_seq_id& pid) const;
 
     string x_IdToStrKey(const CPDB_seq_id& id) const;
 
