@@ -1,4 +1,4 @@
-/*  $Id: gb_release_file.cpp 371868 2012-08-13 15:10:25Z rafanovi $
+/*  $Id: gb_release_file.cpp 434519 2014-05-08 02:28:17Z ucko $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -29,17 +29,17 @@
 *
 */
 #include <ncbi_pch.hpp>
-#include <corelib/ncbistd.hpp>
-#include <serial/objhook.hpp>
-#include <serial/objistr.hpp>
-#include <serial/objectiter.hpp>
-#include <serial/objectio.hpp>
-#include <objects/seqset/Seq_entry.hpp>
-#include <objects/seqset/Bioseq_set.hpp>
-#include <vector>
-#include <algorithm>
 #include <objects/seqset/gb_release_file.hpp>
 
+#include <serial/objistr.hpp>
+#include <serial/objectio.hpp>
+
+#include <objects/seq/Seqdesc.hpp>
+#include <objects/seq/Seq_descr.hpp>
+#include <objects/seqset/seqset_macros.hpp>
+
+#include <algorithm>
+
 
 BEGIN_NCBI_SCOPE
 USING_SCOPE(objects);
@@ -55,8 +55,8 @@ class CGBReleaseFileImpl : public CReadClassMemberHook
 public:
     typedef CGBReleaseFile::ISeqEntryHandler*   THandler;
 
-    CGBReleaseFileImpl(const string& file_name);
-    CGBReleaseFileImpl(CObjectIStream& in);
+    CGBReleaseFileImpl(const string& file_name, bool propagate);
+    CGBReleaseFileImpl(CObjectIStream& in, bool propagate);
 
     void Read(void);
     void RegisterHandler(THandler handler);
@@ -68,19 +68,23 @@ private:
     auto_ptr<CObjectIStream>    m_In;
     CBioseq_set                 m_Seqset;
     bool                        m_Stopped;
+    bool                        m_Propagate;
 };
 
 
-CGBReleaseFileImpl::CGBReleaseFileImpl(const string& file_name) :
+CGBReleaseFileImpl::CGBReleaseFileImpl(const string& file_name, bool propagate) :
     m_In(CObjectIStream::Open(file_name, eSerial_AsnBinary)),
-    m_Stopped(false)
+    m_Stopped(false),
+    m_Propagate(propagate)
 {
     _ASSERT(m_In.get() != 0  &&  m_In->InGoodState());
 }
 
 
-CGBReleaseFileImpl::CGBReleaseFileImpl(CObjectIStream& in) :
-    m_In(&in), m_Stopped(false)
+CGBReleaseFileImpl::CGBReleaseFileImpl(CObjectIStream& in, bool propagate) :
+    m_In(&in),
+    m_Stopped(false),
+    m_Propagate(propagate)
 {
     _ASSERT(m_In.get() != 0  &&  m_In->InGoodState());
 }
@@ -117,11 +121,19 @@ void CGBReleaseFileImpl::ReadClassMember
     // remove the read hook
     member.ResetLocalReadHook(in);
 
+    CBioseq_set& parent_set = *CType<CBioseq_set>::Get(member.GetClassObject());
+
     // iterate over the sequence of entries
     for ( CIStreamContainerIterator it(in, member); it; ++it ) {
         CRef<CSeq_entry> se(new CSeq_entry);
         it >> *se;
         if ( se ) {
+            if ( m_Propagate ) {
+                EDIT_EACH_DESCRIPTOR_ON_SEQSET (it, parent_set) {
+                    CSeqdesc& desc = **it;
+                    ADD_DESCRIPTOR_TO_SEQENTRY (*se, CRef<CSeqdesc>(SerialClone(desc)));
+                }
+            }
             if ( !m_Handler->HandleSeqEntry(se) ) {
                 m_Stopped = true;
                 break;
@@ -136,15 +148,15 @@ void CGBReleaseFileImpl::ReadClassMember
 //  CGBReleaseFile
 
 
-CGBReleaseFile::CGBReleaseFile(const string& file_name) :
-    m_Impl(new CGBReleaseFileImpl(file_name))
+CGBReleaseFile::CGBReleaseFile(const string& file_name, bool propagate) :
+    m_Impl(new CGBReleaseFileImpl(file_name, propagate))
 {
     _ASSERT(m_Impl);
 }
 
 
-CGBReleaseFile::CGBReleaseFile(CObjectIStream& in) :
-    m_Impl(new CGBReleaseFileImpl(in))
+CGBReleaseFile::CGBReleaseFile(CObjectIStream& in, bool propagate) :
+    m_Impl(new CGBReleaseFileImpl(in, propagate))
 {
     _ASSERT(m_Impl);
 }
