@@ -1,4 +1,4 @@
-/*  $Id: annot_object.cpp 448090 2014-10-02 14:57:32Z ivanov $
+/*  $Id: annot_object.cpp 470193 2015-06-11 19:51:07Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -430,11 +430,16 @@ void CAnnotObject_Info::x_ProcessFeat(vector<CHandleRangeMap>& hrmaps,
     hrmaps.resize(feat.IsSetProduct()? 2: 1);
     hrmaps[0].clear();
     hrmaps[0].SetMasterSeq(master);
-    hrmaps[0].AddLocation(feat.GetLocation());
+    CHandleRangeMap::ETransSplicing mode = CHandleRangeMap::eNoTransSplicing;
+    if ( feat.IsSetExcept_text() &&
+         feat.GetExcept_text().find("trans-splicing") != NPOS ) {
+        mode = CHandleRangeMap::eTransSplicing;
+    }
+    hrmaps[0].AddLocation(feat.GetLocation(), mode);
     if ( feat.IsSetProduct() ) {
         hrmaps[1].clear();
         hrmaps[1].SetMasterSeq(master);
-        hrmaps[1].AddLocation(feat.GetProduct());
+        hrmaps[1].AddLocation(feat.GetProduct(), mode);
     }
 }
 /* static */
