@@ -1,4 +1,4 @@
-/*  $Id: bioseq_info.cpp 513580 2016-09-13 11:58:16Z ivanov $
+/*  $Id: bioseq_info.cpp 577156 2018-12-31 16:20:57Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -157,7 +157,10 @@ void CBioseq_Info::x_DoUpdate(TNeedUpdateFlags flags)
             x_LoadChunks(m_Seq_dataChunks);
         }
         if ( m_SeqMap ) {
-            m_SeqMap->x_UpdateSeq_inst(m_Object->SetInst());
+            CFastMutexGuard guard(m_SeqMap_Mtx);
+            if ( m_SeqMap ) {
+                m_SeqMap->x_UpdateSeq_inst(m_Object->SetInst());
+            }
         }
     }
     if ( flags & fNeedUpdate_assembly ) {
