@@ -1,4 +1,4 @@
-/*  $Id: bioseq_set_handle.cpp 571924 2018-10-04 17:41:25Z ivanov $
+/*  $Id: bioseq_set_handle.cpp 576406 2018-12-14 15:28:32Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -546,6 +546,12 @@ CRef<CSeqdesc> CBioseq_set_EditHandle::RemoveSeqdesc(const CSeqdesc& d) const
 }
 
 
+CRef<CSeqdesc> CBioseq_set_EditHandle::ReplaceSeqdesc(const CSeqdesc& old_desc, CSeqdesc& new_desc) const
+{
+    return x_RealReplaceSeqdesc(old_desc, new_desc);
+}
+
+
 void CBioseq_set_EditHandle::AddSeq_descr(TDescr& v) const
 {
     typedef CAddDescr_EditCommand<CBioseq_set_EditHandle> TCommand;
@@ -708,6 +714,12 @@ CRef<CSeqdesc> CBioseq_set_EditHandle::x_RealRemoveSeqdesc(const CSeqdesc& d) co
 }
 
 
+CRef<CSeqdesc> CBioseq_set_EditHandle::x_RealReplaceSeqdesc(const CSeqdesc& old_desc, CSeqdesc& new_desc) const
+{
+    return x_GetInfo().ReplaceSeqdesc(old_desc, new_desc);
+}
+
+
 void CBioseq_set_EditHandle::x_RealAddSeq_descr(TDescr& v) const
 {
     x_GetInfo().AddSeq_descr(v);
