@@ -1,4 +1,4 @@
-/*  $Id: graph_ci.cpp 439336 2014-06-27 16:08:17Z vasilche $
+/*  $Id: graph_ci.cpp 536422 2017-05-18 14:44:05Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -308,6 +308,28 @@ CGraph_CI::CGraph_CI(const CSeq_annot_Handle& annot,
 }
 
 
+CGraph_CI::CGraph_CI(const CSeq_loc& loc,
+                     const CSeq_annot_Handle& annot)
+    : CAnnotTypes_CI(CSeq_annot::C_Data::e_Graph,
+                     loc,
+                     annot)
+{
+    x_Update();
+}
+
+
+CGraph_CI::CGraph_CI(const CSeq_loc& loc,
+                     const CSeq_annot_Handle& annot,
+                     const SAnnotSelector& sel)
+    : CAnnotTypes_CI(CSeq_annot::C_Data::e_Graph,
+                     loc,
+                     annot,
+                     &sel)
+{
+    x_Update();
+}
+
+
 CGraph_CI::CGraph_CI(const CSeq_entry_Handle& entry)
     : CAnnotTypes_CI(CSeq_annot::C_Data::e_Graph, entry)
 {
