@@ -1,4 +1,4 @@
-/*  $Id: seq_entry_handle.cpp 194592 2010-06-15 18:54:05Z vasilche $
+/*  $Id: seq_entry_handle.cpp 576406 2018-12-14 15:28:32Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -126,12 +126,36 @@ CSeq_entry_Handle CSeq_entry_Handle::GetParentEntry(void) const
 }
 
 
+bool CSeq_entry_Handle::IsTopLevelEntry(void) const
+{
+    return !x_GetInfo().HasParent_Info();
+}
+
+
 CSeq_entry_Handle CSeq_entry_Handle::GetTopLevelEntry(void) const
 {
     return GetTSE_Handle();
 }
 
 
+bool CSeq_entry_Handle::IsTopLevelSeq_submit(void) const
+{
+    return IsTopLevelEntry() && GetTSE_Handle().IsTopLevelSeq_submit();
+}
+
+
+const CSeq_submit& CSeq_entry_Handle::GetTopLevelSeq_submit() const
+{
+    return GetTSE_Handle().GetTopLevelSeq_submit();
+}
+
+
+const CSubmit_block& CSeq_entry_Handle::GetTopLevelSubmit_block() const
+{
+    return GetTSE_Handle().GetTopLevelSubmit_block();
+}
+
+
 CBioseq_Handle CSeq_entry_Handle::GetBioseqHandle(const CSeq_id& id) const
 {
     return GetTSE_Handle().GetBioseqHandle(id);
@@ -244,6 +268,18 @@ CSeq_entry_EditHandle::CSeq_entry_EditHandle(CSeq_entry_Info& info,
 }
 
 
+CSubmit_block& CSeq_entry_EditHandle::SetTopLevelSubmit_block(void) const
+{
+    return GetTSE_Handle().SetTopLevelSubmit_block();
+}
+
+
+void CSeq_entry_EditHandle::SetTopLevelSubmit_block(CSubmit_block& sub) const
+{
+    GetTSE_Handle().SetTopLevelSubmit_block(sub);
+}
+
+
 CSeq_entry_EditHandle CSeq_entry_EditHandle::GetParentEntry(void) const
 {
     CSeq_entry_EditHandle ret;
@@ -348,6 +384,11 @@ CRef<CSeqdesc> CSeq_entry_EditHandle::RemoveSeqdesc(const CSeqdesc& v) const
 }
 
 
+CRef<CSeqdesc> CSeq_entry_EditHandle::ReplaceSeqdesc(const CSeqdesc& old_desc, CSeqdesc& new_desc) const
+{
+    return x_RealReplaceSeqdesc(old_desc, new_desc);
+}
+
 
 CBioseq_EditHandle
 CSeq_entry_EditHandle::AttachBioseq(CBioseq& seq, int index) const
@@ -708,6 +749,12 @@ CRef<CSeqdesc> CSeq_entry_EditHandle::x_RealRemoveSeqdesc(const CSeqdesc& v) con
 }
 
 
+CRef<CSeqdesc> CSeq_entry_EditHandle::x_RealReplaceSeqdesc(const CSeqdesc& old_desc, CSeqdesc& new_desc) const
+{
+    return x_GetInfo().ReplaceSeqdesc(old_desc, new_desc);
+}
+
+
 void CSeq_entry_EditHandle::x_RealAddSeq_descr(TDescr& v) const
 {
     x_GetInfo().AddSeq_descr(v);
