@@ -1,4 +1,4 @@
-/*  $Id: autodef_feature_clause_base.cpp 572646 2018-10-17 16:58:07Z ivanov $
+/*  $Id: autodef_feature_clause_base.cpp 578992 2019-01-29 13:03:37Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -67,9 +67,6 @@ CAutoDefFeatureClause_Base::CAutoDefFeatureClause_Base() :
 
 CAutoDefFeatureClause_Base::~CAutoDefFeatureClause_Base()
 {
-    for (auto it : m_ClauseList) {
-        delete it;
-    }
 }
 
 
@@ -99,7 +96,7 @@ bool CAutoDefFeatureClause_Base::IsuORF(const string& product)
 }
 
 
-void CAutoDefFeatureClause_Base::AddSubclause (CAutoDefFeatureClause_Base *subclause)
+void CAutoDefFeatureClause_Base::AddSubclause (CRef<CAutoDefFeatureClause_Base> subclause)
 {
     if (subclause) {
         m_ClauseList.push_back(subclause);
@@ -188,7 +185,7 @@ string CAutoDefFeatureClause_Base::PrintClause(bool print_typeword, bool typewor
 }
 
 
-bool CAutoDefFeatureClause_Base::DisplayAlleleName ()
+bool CAutoDefFeatureClause_Base::DisplayAlleleName () const
 {
     if (NStr::IsBlank(m_AlleleName)) {
         return false;
@@ -341,6 +338,21 @@ void CAutoDefFeatureClause_Base::RemoveGenesMentionedElsewhere()
 }
 
 
+bool HasAlleleChange(const CAutoDefFeatureClause_Base& c1, const CAutoDefFeatureClause_Base& c2)
+{
+    bool has_allele_1 = c1.DisplayAlleleName();
+    bool has_allele_2 = c2.DisplayAlleleName();
+    if ((has_allele_1 && !has_allele_2) ||
+        (!has_allele_1 && has_allele_2) ||
+        (has_allele_1 && has_allele_2)) {
+        return true;
+    } else {
+        return false;
+    }
+}
+
+
+
 string CAutoDefFeatureClause_Base::ListClauses(bool allow_semicolons, bool suppress_final_and, bool suppress_allele)
 {
     if (m_ClauseList.size() < 1) {
@@ -397,8 +409,7 @@ string CAutoDefFeatureClause_Base::ListClauses(bool allow_semicolons, bool suppr
             }
             if (onebefore_has_typeword_change  ||
                 onebefore_has_interval_change  ||
-                (m_ClauseList[k-1]->DisplayAlleleName()  &&
-                 m_ClauseList[k]->DisplayAlleleName())) {
+                HasAlleleChange(*m_ClauseList[k-1], *m_ClauseList[k])) {
                 onebefore_has_detail_change = true;  
             }
         }
@@ -414,8 +425,7 @@ string CAutoDefFeatureClause_Base::ListClauses(bool allow_semicolons, bool suppr
             }
             if (oneafter_has_typeword_change  ||
                 oneafter_has_interval_change  ||
-                (m_ClauseList[k+1]->DisplayAlleleName()  &&
-                 m_ClauseList[k]->DisplayAlleleName())) {
+                HasAlleleChange(*m_ClauseList[k], *m_ClauseList[k+1])) {
                 oneafter_has_detail_change = true;
             }
         }
@@ -593,7 +603,7 @@ size_t CAutoDefFeatureClause_Base::x_LastIntervalChangeBeforeEnd () const
     if (m_ClauseList.size() < 2) {
         return 0;
     }
-    string last_interval = m_ClauseList[m_ClauseList.size() - 1]->GetInterval();
+    string last_interval = m_ClauseList.back()->GetInterval();
     
     for (size_t k = m_ClauseList.size() - 2; k > 0; k--) {
         if (!NStr::Equal(m_ClauseList[k]->GetInterval(), last_interval)
@@ -667,33 +677,15 @@ void CAutoDefFeatureClause_Base::PluralizeDescription()
 
 void CAutoDefFeatureClause_Base::RemoveDeletedSubclauses()
 {
-    unsigned int k, j;
-    k = 0;
-    while (k < m_ClauseList.size()) {
-        j = k;
-        while (j < m_ClauseList.size() && (m_ClauseList[j] == NULL || m_ClauseList[j]->IsMarkedForDeletion())) {   
-            if (m_ClauseList[j] != NULL) {
-                delete m_ClauseList[j];         
-            }
-            j++;
+    auto it = m_ClauseList.begin();
+    while (it != m_ClauseList.end()) {
+        if (!(*it) || (*it)->IsMarkedForDeletion()) {
+            it = m_ClauseList.erase(it);
+        } else {
+            (*it)->RemoveDeletedSubclauses();
+            ++it;
         }
-        if (j > k) {
-           unsigned int num_removed = j - k;
-           while (j < m_ClauseList.size()) {
-                m_ClauseList[j - num_removed] = m_ClauseList[j];
-                j++;
-            }
-            while (num_removed > 0) {
-                m_ClauseList[m_ClauseList.size() - 1] = NULL;
-                m_ClauseList.pop_back();
-                num_removed --;
-            }
-        }
-        while (k < m_ClauseList.size() && m_ClauseList[k] != NULL && !m_ClauseList[k]->IsMarkedForDeletion()) {
-            m_ClauseList[k]->RemoveDeletedSubclauses();
-            k++;
-        }
-    }   
+    }
 }
 
 
@@ -1138,7 +1130,7 @@ void CAutoDefFeatureClause_Base::SetProductName(string product_name)
 
 void CAutoDefFeatureClause_Base::CountUnknownGenes()
 {
-    CAutoDefUnknownGeneList *unknown_list = new CAutoDefUnknownGeneList();
+    CRef<CAutoDefUnknownGeneList> unknown_list(new CAutoDefUnknownGeneList());
     bool any_found = false;
     
     for (unsigned int k = 0; k < m_ClauseList.size(); k++) {
@@ -1154,8 +1146,6 @@ void CAutoDefFeatureClause_Base::CountUnknownGenes()
     
     if (any_found) {
         AddSubclause(unknown_list);
-    } else {
-        delete unknown_list;
     }        
 }
 
@@ -1235,7 +1225,6 @@ void CAutoDefFeatureClause_Base::ExpandExonLists()
             TClauseList subclauses;
             subclauses.clear();
             m_ClauseList[k]->TransferSubclauses(subclauses);
-            delete m_ClauseList[k];
             for (unsigned int j = 0; j < subclauses.size(); j++) {
                 if (k + j < m_ClauseList.size()) {
                     m_ClauseList[k + j] = subclauses[j];
@@ -1535,7 +1524,7 @@ void CAutoDefFeatureClause_Base::RemoveuORFs()
 void CAutoDefFeatureClause_Base::RemoveOptionalMobileElements()
 {
     for (unsigned int k = 0; k < m_ClauseList.size(); k++) {
-        CAutoDefMobileElementClause* clause = dynamic_cast<CAutoDefMobileElementClause *>(m_ClauseList[k]);        
+        CAutoDefMobileElementClause* clause = dynamic_cast<CAutoDefMobileElementClause *>(m_ClauseList[k].GetPointer());        
         if (clause && clause->IsOptional()) {
             m_ClauseList[k]->MarkForDeletion();
         } else {
@@ -1624,7 +1613,7 @@ CRef<CSeq_loc> CAutoDefExonListClause::SeqLocIntersect (CRef<CSeq_loc> loc1, CRe
 }
 
 
-void CAutoDefExonListClause::AddSubclause (CAutoDefFeatureClause_Base *subclause)
+void CAutoDefExonListClause::AddSubclause (CRef<CAutoDefFeatureClause_Base> subclause)
 {
     CAutoDefFeatureClause_Base::AddSubclause(subclause);
     if (m_ClauseList.size() == 1) {
@@ -1998,22 +1987,22 @@ vector<string> CAutoDefFeatureClause_Base::GetFeatureClausePhrases(string commen
 }
 
 
-CAutoDefFeatureClause_Base * CAutoDefFeatureClause_Base::ClauseFromPhrase(const string& phrase, CBioseq_Handle bh, const CSeq_feat& cf, const CSeq_loc& mapped_loc, bool first, bool last)
+CRef<CAutoDefFeatureClause> CAutoDefFeatureClause_Base::ClauseFromPhrase(const string& phrase, CBioseq_Handle bh, const CSeq_feat& cf, const CSeq_loc& mapped_loc, bool first, bool last)
 {
     if (NStr::Equal(phrase, "control region") ||
         NStr::Equal(phrase, "D-loop")) {
         // create a clause of the appropriate type
-        CAutoDefParsedClause* other = new CAutoDefParsedClause(bh, cf, mapped_loc, first, last);
+        CAutoDefParsedClause * other(new CAutoDefParsedClause(bh, cf, mapped_loc, first, last));
         other->SetTypeword(phrase);
         other->SetTypewordFirst(false);
-        return other;
+        return CRef< CAutoDefFeatureClause> (other);
     } else if (x_GetRnaMiscWordType(phrase) != eMiscRnaWordType_Unrecognized) {
         CAutoDefParsedClause *new_clause = new CAutoDefParsedClause(bh, cf, mapped_loc, first, last);
         new_clause->SetMiscRNAWord(phrase);
-        return new_clause;
+        return CRef< CAutoDefFeatureClause>(new_clause);
     } else {
         CAutoDefParsedtRNAClause* trna = s_tRNAClauseFromNote(bh, cf, mapped_loc, phrase, first, last);
-        return trna;
+        return CRef< CAutoDefFeatureClause>(trna);
     }
 }
 
