@@ -1055,8 +1055,13 @@ void CBioseqIndex::x_InitGaps (void)
 
         SSeqMapSelector sel;
 
+        size_t resolveCount = 1;
+        if (m_Policy == CSeqEntryIndex::eInternal) {
+            resolveCount = 0;
+        }
+
         sel.SetFlags(CSeqMap::fFindGap)
-           .SetResolveCount(1);
+           .SetResolveCount(resolveCount);
 
         // explore gaps, pass original target BioseqHandle if using Bioseq sublocation
         for (CSeqMap_CI gap_it(m_OrigBsh, sel); gap_it; ++gap_it) {
@@ -1855,13 +1860,24 @@ void CBioseqIndex::x_InitFeats (void)
             sel.ExcludeFeatSubtype(CSeqFeatData::eSubtype_STS);
         }
         if ((m_Flags & CSeqEntryIndex::fHideExonFeats) != 0) {
+            sel.ExcludeNamedAnnots("Exon");
             sel.ExcludeFeatSubtype(CSeqFeatData::eSubtype_exon);
         }
         if ((m_Flags & CSeqEntryIndex::fHideIntronFeats) != 0) {
             sel.ExcludeFeatSubtype(CSeqFeatData::eSubtype_intron);
         }
         if ((m_Flags & CSeqEntryIndex::fHideMiscFeats) != 0) {
+            sel.ExcludeFeatType(CSeqFeatData::e_Site);
+            sel.ExcludeFeatType(CSeqFeatData::e_Bond);
+            sel.ExcludeFeatType(CSeqFeatData::e_Region);
+            sel.ExcludeFeatType(CSeqFeatData::e_Comment);
             sel.ExcludeFeatSubtype(CSeqFeatData::eSubtype_misc_feature);
+            sel.ExcludeFeatSubtype(CSeqFeatData::eSubtype_preprotein);
+        }
+
+        bool onlyGeneRNACDS = false;
+        if ((m_Flags & CSeqEntryIndex::fGeneRNACDSOnly) != 0) {
+            onlyGeneRNACDS = true;
         }
 
         // additional common settings
@@ -1909,6 +1925,17 @@ void CBioseqIndex::x_InitFeats (void)
             // iterate features on Bioseq
             for (CFeat_CI feat_it(m_Bsh, sel); feat_it; ++feat_it) {
                 const CMappedFeat mf = *feat_it;
+
+                if (onlyGeneRNACDS) {
+                    const CSeqFeatData& data = mf.GetData();
+                    CSeqFeatData::E_Choice type = data.Which();
+                    if (type != CSeqFeatData::e_Gene &&
+                        type != CSeqFeatData::e_Rna &&
+                        type != CSeqFeatData::e_Cdregion) {
+                        continue;
+                    }
+                }
+
                 CSeq_feat_Handle hdl = mf.GetSeq_feat_Handle();
 
                 CRef<CFeatureIndex> sfx(new CFeatureIndex(hdl, mf, *this));
@@ -2871,7 +2898,7 @@ CFeatureIndex::CFeatureIndex (CSeq_feat_Handle sfh,
       m_Mf(mf),
       m_Bsx(&bsx)
 {
-    const CSeqFeatData& data  = m_Mf.GetData();
+    const CSeqFeatData& data = m_Mf.GetData();
     m_Type = data.Which();
     m_Subtype = data.GetSubtype();
     const CSeq_feat& mpd = m_Mf.GetMappedFeature();
