@@ -1,4 +1,4 @@
-/*  $Id: objutil.cpp 573600 2018-10-30 11:55:27Z ivanov $
+/*  $Id: objutil.cpp 580156 2019-02-11 16:24:38Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -253,11 +253,25 @@ string ConvertQuotes(const string& str)
 // Strips all spaces in string in following manner. If the function
 // meet several spaces (spaces and tabs) in succession it replaces them
 // with one space. Strips all spaces after '(' and before ( ')' or ',' ).
-void StripSpaces(string& str)
+bool StripSpaces(string& str)
 {
     if (str.empty()) {
-        return;
+        return false;
     }
+    auto orig_len = str.length();
+
+    NStr::ReplaceInPlace(str, "\t", " ");
+    auto this_len = str.length();
+    NStr::ReplaceInPlace(str, "  ", " ");
+    while (str.length() != this_len) {
+        this_len = str.length();
+        NStr::ReplaceInPlace(str, "  ", " ");
+    }
+    NStr::ReplaceInPlace(str, "( ", "(");
+    NStr::ReplaceInPlace(str, " )", ")");
+    NStr::ReplaceInPlace(str, " ,", ",");
+
+#if 0
 
     string::iterator end = str.end();
     string::iterator it = str.begin();
@@ -277,6 +291,8 @@ void StripSpaces(string& str)
         }
     }
     str.erase(new_str, str.end());
+#endif
+    return (orig_len != str.length());
 }
 
 
