@@ -1,4 +1,4 @@
-/*  $Id: seq_loc_util.cpp 531968 2017-03-30 17:34:38Z grichenk $
+/*  $Id: seq_loc_util.cpp 572076 2018-10-09 13:32:45Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -300,6 +300,9 @@ bool IsValid(const CSeq_interval& interval, CScope* scope)
 bool IsSameBioseq(const CSeq_id& id1, const CSeq_id& id2, CScope* scope,
                   CScope::EGetBioseqFlag get_flag)
 {
+    if ( id1.Match(id2) ) {
+        return true;
+    }
     return IsSameBioseq(CSeq_id_Handle::GetHandle(id1),
                         CSeq_id_Handle::GetHandle(id2),
                         scope, get_flag);
@@ -382,7 +385,7 @@ CSeq_id_Handle GetIdHandle(const CSeq_loc& loc, CScope* scope)
 
 bool IsOneBioseq(const CSeq_loc& loc, CScope* scope)
 {
-    return s_GetId(loc, scope) != NULL;
+    return loc.GetId() != NULL || s_GetId(loc, scope) != NULL;
 }
 
 
@@ -542,8 +545,12 @@ void ChangeSeqId(CSeq_id* id, bool best, CScope* scope)
     }
 
     // Get CBioseq represented by *id
+    CBioseq_Handle bsh = scope->GetBioseqHandle(*id);
+    if (!bsh) {
+        return;
+    }
     CBioseq_Handle::TBioseqCore seq =
-        scope->GetBioseqHandle(*id).GetBioseqCore();
+        bsh.GetBioseqCore();
 
     // Get pointer to the best/worst id of *seq
     const CSeq_id* tmp_id;
