@@ -1,4 +1,4 @@
-/*  $Id: format_flags.cpp 516923 2016-10-19 14:23:23Z fongah2 $
+/*  $Id: format_flags.cpp 577748 2019-01-08 18:06:48Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -222,6 +222,11 @@ const size_t kDfltArgNumAlignments = 250;
 const string kArgProduceHtml("html");
 const bool kDfltArgProduceHtml = false;
 const size_t kDfltLineLength = 60;
+const string kArgAlignSeqList("alignseqlist");
+const string kArgMetadata("searchmetadata");
+const string kArgQueryIndex("queryindex");
+const string kArgSortHits("sorthits");
+const string kArgSortHSPs("sorthsps");
 
 const size_t kNumSAMOutputFormatSpecifiers = 2;
 const SSAMFormatSpec sc_SAMFormatSpecifiers[kNumSAMOutputFormatSpecifiers] = {
