@@ -1,4 +1,4 @@
-/*  $Id: aln_generators.cpp 360549 2012-04-24 15:29:03Z grichenk $
+/*  $Id: aln_generators.cpp 401311 2013-05-29 18:06:45Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -153,7 +153,7 @@ class CSegmentedRangeCollection : public CRangeCollection<CPairwiseAln::TPos>
 public:
     typedef ncbi::CRangeCollection<CPairwiseAln::TPos> TParent;
 
-    const_iterator CutAtPosition(position_type pos) {
+    iterator CutAtPosition(position_type pos) {
         iterator ret_it = TParent::m_vRanges.end();
         iterator it = find_nc(pos);
         if (it != TParent::end()  &&  it->GetFrom() < pos) {
