@@ -1,4 +1,4 @@
-/*  $Id: alnmap.cpp 461118 2015-03-06 13:29:39Z grichenk $
+/*  $Id: alnmap.cpp 554069 2017-12-27 20:58:55Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -1147,6 +1147,11 @@ void CAlnMap::x_GetChunks(CAlnChunkVec * vec,
     TSegTypeFlags type, test_type;
 
     _ASSERT(left_seg <= right_seg);
+    if (left_seg > right_seg) {
+        NCBI_THROW(CAlnException,
+            eInvalidSegment | Critical,
+            "Invalid segments range");
+    }
 
     size_t hint_idx = m_NumRows * left_seg + row;
 
