@@ -1,4 +1,4 @@
-/*  $Id: seq_writer_unit_test.cpp 509457 2016-08-05 16:24:09Z boratyng $
+/*  $Id: seq_writer_unit_test.cpp 548810 2017-10-18 13:38:41Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -116,7 +116,7 @@ BOOST_AUTO_TEST_CASE(TestRequestGiOidLength)
     in.getline(buffer, sizeof(buffer));
 
     vector<string> tokens;
-    NStr::Split(string(buffer), " ", tokens, NStr::fSplit_NoMergeDelims);
+    NStr::Split(string(buffer), " ", tokens);
 
     int gi = NStr::StringToInt(tokens[2], kConvFlags);
     int oid = NStr::StringToInt(tokens[5], kConvFlags);
@@ -235,7 +235,7 @@ BOOST_AUTO_TEST_CASE(TestRequestAccessionPIGTaxidTitle)
     in.getline(buffer, sizeof(buffer));
 
     vector<string> tokens;
-    NStr::Split(string(buffer), " ", tokens, NStr::fSplit_NoMergeDelims);
+    NStr::Split(string(buffer), " ", tokens);
 
     int gi = NStr::StringToInt(tokens[1], kConvFlags);
     string accession = tokens[4];
@@ -291,7 +291,7 @@ BOOST_AUTO_TEST_CASE(TestRequestSequenceDataLength)
     in.getline(buffer, sizeof(buffer));
 
     vector<string> tokens;
-    NStr::Split(string(buffer), "|", tokens, NStr::fSplit_NoMergeDelims);
+    NStr::Split(string(buffer), "|", tokens);
 
     int gi = NStr::StringToInt(tokens[0], kConvFlags);
     int len = NStr::StringToInt(tokens[1], kConvFlags);
