@@ -1,4 +1,4 @@
-/*  $Id: gene_info_reader_app.cpp 473030 2015-07-15 19:29:59Z camacho $
+/*  $Id: gene_info_reader_app.cpp 492284 2016-02-16 16:55:37Z camacho $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -254,6 +254,6 @@ void CReadFilesApp::Exit(void)
 int main(int argc, const char* argv[])
 {
     // Execute main application function
-    return CReadFilesApp().AppMain(argc, argv, 0, eDS_Default, "");
+    return CReadFilesApp().AppMain(argc, argv);
 }
 
