@@ -1,4 +1,4 @@
-/*  $Id: gene_info_reader.cpp 219583 2011-01-11 20:26:07Z camacho $
+/*  $Id: gene_info_reader.cpp 500404 2016-05-04 14:59:01Z camacho $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -29,11 +29,6 @@
 
 /// @file gene_info_reader.cpp
 /// Implementation of reading Gene information from files.
-
-#ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: gene_info_reader.cpp 219583 2011-01-11 20:26:07Z camacho $";
-#endif /* SKIP_DOXYGEN_PROCESSING */
-
 //==========================================================================//
 
 #include <ncbi_pch.hpp>
