@@ -1,4 +1,4 @@
-/*  $Id: seqdb_lmdb.cpp 559974 2018-03-16 15:23:03Z fongah2 $
+/*  $Id: seqdb_lmdb.cpp 581731 2019-03-05 16:42:09Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -124,8 +124,8 @@ CBlastLMDBManager::~CBlastLMDBManager()
 {
 	NON_CONST_ITERATE(list <CBlastEnv* >, itr, m_EnvList) {
 		delete *itr;
-		itr = m_EnvList.erase(itr);
 	}
+	m_EnvList.clear();
 }
 
 CSeqDBLMDB::CSeqDBLMDB(const string & fname)
@@ -517,6 +517,7 @@ void CSeqDBLMDB::GetOidsForTaxIds(const set<Int4> & tax_ids, vector<blastdb::TOi
     }
 }
 
+
 class CLookupTaxIds
 {
 public:
@@ -580,6 +581,18 @@ CSeqDBLMDB::NegativeTaxIdsToOids(const set<Int4>& tax_ids, vector<blastdb::TOid>
 	}
 }
 
+void CSeqDBLMDB::GetTaxIdsForOids(const vector<blastdb::TOid> & oids, set<Int4> & tax_ids) const
+{
+	CMemoryFile oid_file(m_Oid2TaxIdsFile);
+	CLookupTaxIds lookup(oid_file);
+	for(unsigned int i=0; i < oids.size(); i++) {
+		vector<Int4>  taxid_list;
+		lookup.GetTaxIdListForOid(oids[i], taxid_list);
+		tax_ids.insert(taxid_list.begin(), taxid_list.end());
+	}
+}
+
+
 string BuildLMDBFileName(const string& basename, bool is_protein, bool use_index, unsigned int index)
 {
     if (basename.empty()) {
