@@ -1,4 +1,4 @@
-/*  $Id: seqdbcol.cpp 536658 2017-05-22 15:48:20Z zaretska $
+/*  $Id: seqdbcol.cpp 579716 2019-02-05 16:53:21Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -103,7 +103,7 @@ CSeqDBColumn::CSeqDBColumn(const string   & basename,
                            const string   & index_extn,
                            const string   & data_extn,
                            CSeqDBLockHold * lockedp)
-    : m_AtlasHolder      (true, lockedp),
+    : m_AtlasHolder      (lockedp, true),
       m_Atlas            (m_AtlasHolder.Get()),
       m_IndexFile        (m_Atlas),      
       m_DataFile         (m_Atlas),     
