@@ -1,4 +1,4 @@
-/*  $Id: seqdbgilistset.cpp 553487 2017-12-18 14:23:38Z fongah2 $
+/*  $Id: seqdbgilistset.cpp 579228 2019-01-31 16:32:48Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -110,6 +110,10 @@ public:
                 break;
             case CSeqDBGiListSet::eSiList:
                 SeqDB_ReadMemorySiList(fbeginp, fendp, m_SisOids, & in_order);
+                break;
+            case CSeqDBGiListSet::ePigList:
+                SeqDB_ReadMemoryPigList(fbeginp, fendp, m_PigsOids, & in_order);
+                break;
             }
 
             if (in_order) {
@@ -160,19 +164,13 @@ void s_VerifySeqidlist(const SBlastSeqIdListInfo & list_info, const CSeqDBVolSet
 	 return;
 }
 
-
-CSeqDBGiListSet::CSeqDBGiListSet(CSeqDBAtlas            & atlas,
-                                 const CSeqDBVolSet     & volset,
-                                 CRef<CSeqDBGiList>       user_list,
-                                 CRef<CSeqDBNegativeList> neg_list,
-                                 CSeqDBLockHold         & locked,
-                                 const CSeqDBLMDBSet & lmdb_set)
-    : m_Atlas        (atlas),
-      m_UserList     (user_list),
-      m_NegativeList (neg_list)
+void
+CSeqDBGiListSet::x_ResolvePositiveList(CSeqDBAtlas            & atlas,
+                                       const CSeqDBVolSet     & volset,
+                                       CRef<CSeqDBGiList>       user_list,
+                                       CSeqDBLockHold         & locked,
+                                       const CSeqDBLMDBSet & lmdb_set)
 {
-    _ASSERT(user_list.Empty() || neg_list.Empty());
-
     if (m_UserList.NotEmpty() && m_UserList->NotEmpty()) {
     	if(user_list->GetNumSis() > 0) {
     		s_VerifySeqidlist(user_list->GetListInfo(), volset, lmdb_set);
@@ -196,7 +194,8 @@ CSeqDBGiListSet::CSeqDBGiListSet(CSeqDBAtlas            & atlas,
     			set<Int4> &  tax_ids = user_list->GetTaxIdsList();
     			lmdb_set.TaxIdsToOids(tax_ids, oids);
     		}
-    		if((user_list->GetNumGis() == 0) && (user_list->GetNumTis() == 0)) {
+    		if((user_list->GetNumGis() == 0) && (user_list->GetNumTis() == 0) &&
+    		   (user_list->GetNumPigs() == 0)) {
     			return;
     		}
     	}
@@ -237,7 +236,18 @@ CSeqDBGiListSet::CSeqDBGiListSet(CSeqDBAtlas            & atlas,
 
             vol->Vol()->IdsToOids(*m_UserList, locked);
         }
-    } else if (m_NegativeList.NotEmpty() && m_NegativeList->NotEmpty()) {
+    }
+}
+
+
+void
+CSeqDBGiListSet::x_ResolveNegativeList(CSeqDBAtlas            & atlas,
+                                       const CSeqDBVolSet     & volset,
+                                       CRef<CSeqDBNegativeList> neg_list,
+                                       CSeqDBLockHold         & locked,
+                                       const CSeqDBLMDBSet & lmdb_set)
+{
+    if (m_NegativeList.NotEmpty() && m_NegativeList->NotEmpty()) {
         // We don't bother to sort these since every ISAM mapping must
         // be examined for the negative ID list case.
     	if(m_NegativeList->GetNumSis() > 0) {
@@ -261,7 +271,8 @@ CSeqDBGiListSet::CSeqDBGiListSet(CSeqDBAtlas            & atlas,
     			lmdb_set.NegativeTaxIdsToOids(tax_ids, oids);
     		}
 
-    		if((m_NegativeList->GetNumGis() == 0) && (m_NegativeList->GetNumTis() == 0)) {
+    		if((m_NegativeList->GetNumGis() == 0) && (m_NegativeList->GetNumTis() == 0) &&
+    		   (m_NegativeList->GetNumPigs() == 0)) {
     			return;
     		}
     	}
@@ -269,6 +280,27 @@ CSeqDBGiListSet::CSeqDBGiListSet(CSeqDBAtlas            & atlas,
     	if((m_NegativeList->GetNumSis() > 0) && !(lmdb_set.IsBlastDBVersion5())) {
     		m_NegativeList->PreprocessIdsForISAMSiLookup();
     	}
+
+    	if (m_NegativeList->GetNumPigs() > 0) {
+    		CSeqDBGiList pigs;
+    		pigs.ReservePigs(m_NegativeList->GetNumPigs());
+    		ITERATE(vector<TPig>, p, m_NegativeList->GetPigList()){
+    			pigs.AddPig(*p);
+    		}
+    		for(int v = 0; v < volset.GetNumVols(); v++) {
+    		    const CSeqDBVolEntry * vol = volset.GetVolEntry(v);
+    		    vol->Vol()->IdsToOids(pigs, locked);
+    		}
+
+    		vector<blastdb::TOid> &  exclude_oid_list = m_NegativeList->SetExcludedOids();
+    		for(int o=0; o < pigs.GetNumPigs(); o++) {
+    			const CSeqDBGiList::SPigOid &  pig = pigs.GetPigOid(o);
+    			if(pig.oid != -1) {
+    				exclude_oid_list.push_back(pig.oid);
+    			}
+    		}
+
+    	}
         for(int i = 0; i < volset.GetNumVols(); i++) {
             const CSeqDBVolEntry * vol = volset.GetVolEntry(i);
 
@@ -279,6 +311,20 @@ CSeqDBGiListSet::CSeqDBGiListSet(CSeqDBAtlas            & atlas,
     }
 }
 
+CSeqDBGiListSet::CSeqDBGiListSet(CSeqDBAtlas            & atlas,
+                                 const CSeqDBVolSet     & volset,
+                                 CRef<CSeqDBGiList>       user_list,
+                                 CRef<CSeqDBNegativeList> neg_list,
+                                 CSeqDBLockHold         & locked,
+                                 const CSeqDBLMDBSet & lmdb_set)
+    : m_Atlas        (atlas),
+      m_UserList     (user_list),
+      m_NegativeList (neg_list)
+{
+	x_ResolvePositiveList(atlas, volset, user_list, locked, lmdb_set);
+	x_ResolveNegativeList(atlas, volset, neg_list, locked, lmdb_set);
+}
+
 CRef<CSeqDBGiList>
 CSeqDBGiListSet::GetNodeIdList(const CSeqDB_Path & filename,
                                const CSeqDBVol   * volp,
