@@ -1,4 +1,4 @@
-/*  $Id: seqdbimpl.cpp 560817 2018-03-27 17:24:45Z rackerst $
+/*  $Id: seqdbimpl.cpp 579716 2019-02-05 16:53:21Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -44,11 +44,11 @@ CSeqDBImpl::CSeqDBImpl(const string       & db_name_list,
                        char                 prot_nucl,
                        int                  oid_begin,
                        int                  oid_end,
-                       bool                 use_mmap,
                        CSeqDBGiList       * gi_list,
                        CSeqDBNegativeList * neg_list,
-                       CSeqDBIdSet          idset)
-    : m_AtlasHolder     (use_mmap, NULL),
+                       CSeqDBIdSet          idset,
+                       bool                 use_atlas_lock)
+    : m_AtlasHolder     (NULL, use_atlas_lock),
       m_Atlas           (m_AtlasHolder.Get()),
       m_DBNames         (db_name_list),
       m_Aliases         (m_Atlas, db_name_list, prot_nucl),
@@ -88,7 +88,6 @@ CSeqDBImpl::CSeqDBImpl(const string       & db_name_list,
         m_GiMask.Reset(new CSeqDBGiMask(m_Atlas, mask_list));
     }
 
-    _ASSERT((! gi_list) || (! neg_list));
 
     m_VolSet.OptimizeGiLists();
 
@@ -150,8 +149,8 @@ CSeqDBImpl::CSeqDBImpl(const string       & db_name_list,
     CHECK_MARKER();
 }
 
-CSeqDBImpl::CSeqDBImpl()
-    : m_AtlasHolder     (false, NULL),
+CSeqDBImpl::CSeqDBImpl(bool use_atlas_lock)
+    : m_AtlasHolder     (NULL, use_atlas_lock),
       m_Atlas           (m_AtlasHolder.Get()),
       m_Aliases         (m_Atlas, "", '-'),
       m_RestrictBegin   (0),
@@ -1314,12 +1313,26 @@ void CSeqDBImpl::TaxIdsToOids(set<Int4>& tax_ids, vector<blastdb::TOid>& rv)
     return;
 }
 
-void CSeqDBImpl::GetDBTaxIds(set<Int4> & tax_ids) const
+void CSeqDBImpl::GetDBTaxIds(set<Int4> & tax_ids)
 {
     CHECK_MARKER();
+    CSeqDBLockHold locked(m_Atlas);
+
+    if (! m_OidListSetup) {
+        x_GetOidList(locked);
+    }
     tax_ids.clear();
     if (m_LMDBSet.IsBlastDBVersion5()) {
-    	m_LMDBSet.GetDBTaxIds(tax_ids);
+    	if(m_OIDList.NotEmpty()){
+    		vector<blastdb::TOid>  oids;
+    	    for(int oid = 0; CheckOrFindOID(oid); oid++) {
+    	    	oids.push_back(oid);
+    	    }
+    		m_LMDBSet.GetTaxIdsForOids(oids, tax_ids);
+    	}
+    	else {
+    		m_LMDBSet.GetDBTaxIds(tax_ids);
+    	}
     }
     else {
     	NCBI_THROW(CSeqDBException, eArgErr,
@@ -1494,7 +1507,8 @@ CSeqDBImpl::FindVolumePaths(const string   & dbname,
                             bool             recursive,
                             bool             expand_links)
 {
-    CSeqDBAtlasHolder AH(true, NULL);
+    bool use_atlas_lock = true;
+    CSeqDBAtlasHolder AH(NULL, use_atlas_lock);
     CSeqDBAtlas & atlas(AH.Get());
 
     // This constructor handles its own locking.
@@ -1943,7 +1957,7 @@ void CSeqDBImpl::ListColumns(vector<string> & titles)
         m_VolSet.GetVolNonConst(vol_idx)->ListColumns(all, locked);
     }
 
-    titles.resize(SeqDB_VectorAssign(all, titles));
+    titles.assign(all.begin(), all.end());
 }
 
 int CSeqDBImpl::GetColumnId(const string & title)
