@@ -1,4 +1,4 @@
-/*  $Id: seqdbvol.cpp 559232 2018-03-08 14:41:08Z fongah2 $
+/*  $Id: seqdbvol.cpp 581731 2019-03-05 16:42:09Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1881,27 +1881,35 @@ CSeqDBVol::x_GetFilteredHeader(int                    oid,
 
             // Here we must pass both the user-gi and volume-gi test,
             // for each defline, but not necessarily for each Seq-id.
+            if (have_memb) {
+            	if (id_filter && defline.CanGetSeqid()) {
+            		have_memb = false;
+            		bool have_user = false, have_volume = false;
+            		ITERATE(list< CRef<CSeq_id> >, seqid, defline.GetSeqid()) {
+            			x_FilterHasId(**seqid, have_user, have_volume);
+            			if (have_user && have_volume) break;
+            		}
+            		have_memb = have_user && have_volume;
+            	}
 
-            if (have_memb && id_filter && defline.CanGetSeqid()) {
-                have_memb = false;
+            	if(have_memb && (!m_UserGiList.Empty()) && (m_UserGiList->GetNumTaxIds() > 0)) {
+               		have_memb = s_IncludeDefline_Taxid(defline, m_UserGiList->GetTaxIdsList());
+            	}
 
-                bool have_user = false, have_volume = false;
+            	if (!have_memb && !m_VolumeGiLists.empty()) {
+               		NON_CONST_ITERATE(TGiLists, vtaxid, m_VolumeGiLists) {
+               			if( (*vtaxid)->GetNumTaxIds() > 0) {
+               				have_memb = s_IncludeDefline_Taxid(defline, (*vtaxid)->GetTaxIdsList());
+               				if(have_memb){
+               					break;
+               				}
+               			}
+               		}
+            	}
 
-                ITERATE(list< CRef<CSeq_id> >, seqid, defline.GetSeqid()) {
-                    x_FilterHasId(**seqid, have_user, have_volume);
-
-                    if (have_user && have_volume)
-                        break;
-                }
-
-                have_memb = have_user && have_volume;
-            }
-
-            if(have_memb && (!m_UserGiList.Empty()) && (m_UserGiList->GetNumTaxIds() > 0)) {
-               	have_memb = s_IncludeDefline_Taxid(defline, m_UserGiList->GetTaxIdsList());
-            }
-            if(have_memb && (!m_NegativeList.Empty()) && (m_NegativeList->GetNumTaxIds() > 0)) {
-               	have_memb = s_IncludeDefline_NegativeTaxid(defline, m_NegativeList->GetTaxIdsList());
+            	if(have_memb && (!m_NegativeList.Empty()) && (m_NegativeList->GetNumTaxIds() > 0)) {
+               		have_memb = s_IncludeDefline_NegativeTaxid(defline, m_NegativeList->GetTaxIdsList());
+            	}
             }
 
             if (! have_memb) {
@@ -2194,6 +2202,17 @@ void CSeqDBVol::IdsToOids(CSeqDBGiList   & ids,
         }
     }
 
+    if (ids.GetNumPigs()) {
+        if (!m_PigFileOpened) x_OpenPigFile();
+        if (m_IsamPig.NotEmpty()) {
+            m_IsamPig->IdsToOids(m_VolStart, m_VolEnd, ids);
+        } else {
+            NCBI_THROW(CSeqDBException,
+                       eArgErr,
+                       "IPG list specified but no ISAM file found for IPG in " + m_VolName);
+        }
+    }
+
     if (ids.GetNumSis() && (GetLMDBFileName() == kEmptyStr)) {
         if (!m_StrFileOpened) x_OpenStrFile();
         if (m_IsamStr.NotEmpty()) {
