@@ -1,4 +1,4 @@
-/*  $Id: seqdb_perf.cpp 500404 2016-05-04 14:59:01Z camacho $
+/*  $Id: seqdb_perf.cpp 573139 2018-10-23 19:28:52Z fukanchi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -145,7 +145,7 @@ CSeqDBPerfApp::x_ScanDatabase()
         thread_id = omp_get_thread_num();
 #endif
         #pragma omp for schedule(static, (oids2iterate.size()/m_DbHandles.size())) nowait
-        for (size_t i = 0; i < oids2iterate.size(); i++) {
+        for (ssize_t i = 0; i < oids2iterate.size(); i++) {
             int oid = oids2iterate[i];
             const char* buffer = NULL;
             int seqlen = 0;
