@@ -1,4 +1,4 @@
-/*  $Id: multisource_util.cpp 568561 2018-08-07 18:29:39Z grichenk $
+/*  $Id: multisource_util.cpp 580237 2019-02-12 13:07:41Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -95,6 +95,11 @@ string AccessionToKey(const string & acc)
         } else {
             GetSeqIdKey(*seqid, str);
         }
+    } else {
+        string tempID = "lcl|" + acc;
+        if(CheckAccession(tempID, gi, seqid, specific)) {
+            GetSeqIdKey(*seqid, str);
+        }
     }
 
     return str;
