@@ -1,4 +1,4 @@
-/*  $Id: taxid_set.cpp 495728 2016-03-21 14:25:14Z ivanov $
+/*  $Id: taxid_set.cpp 548810 2017-10-18 13:38:41Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -30,11 +30,6 @@
 /** @file taxid_set.cpp
 *     Class which defines sequence id to taxid mapping.
 */
-
-#ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: taxid_set.cpp 495728 2016-03-21 14:25:14Z ivanov $";
-#endif /* SKIP_DOXYGEN_PROCESSING */
-
 #include <ncbi_pch.hpp>
 #include <objtools/blast/seqdb_writer/taxid_set.hpp>
 #include <objtools/blast/seqdb_writer/multisource_util.hpp>
@@ -58,7 +53,7 @@ void CTaxIdSet::SetMappingFromFile(CNcbiIstream & f)
         s = NStr::TruncateSpaces(s);
         
         vector<string> tokens;
-        NStr::Split(s, " \t", tokens, NStr::fSplit_NoMergeDelims);
+        NStr::Split(s, " \t", tokens);
 
         string gi_str = tokens.front();
         string tx_str;
