@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_WRITERS_WRITEDB__WRITEDB_IMPL_HPP
 #define OBJTOOLS_WRITERS_WRITEDB__WRITEDB_IMPL_HPP
 
-/*  $Id: writedb_impl.hpp 513693 2016-09-14 15:53:17Z boratyng $
+/*  $Id: writedb_impl.hpp 554873 2018-01-10 18:08:52Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -43,6 +43,7 @@
 #include <objects/blastdb/defline_extra.hpp>
 #include <objtools/blast/seqdb_writer/writedb.hpp>
 #include <objtools/blast/seqdb_reader/seqdbcommon.hpp>
+#include <objtools/blast/seqdb_writer/writedb_lmdb.hpp>
 #include "writedb_volume.hpp"
 #include "writedb_gimask.hpp"
 #include "mask_info_registry.hpp"
@@ -81,7 +82,8 @@ public:
                   EIndexType         indices,
                   bool               parse_ids,
                   bool               long_ids,
-                  bool               use_gi_mask);
+                  bool               use_gi_mask,
+                  EBlastDbVersion dbver = eBDB_Version4);
 
     /// Destructor.
     ~CWriteDB_Impl();
@@ -377,6 +379,7 @@ private:
     map<int, int> m_MaskAlgoMap;      ///< Mapping from algo_id to gi-mask id
     bool          m_ParseIDs;         ///< Generate ISAM files
     bool          m_UseGiMask;        ///< Generate GI-based mask files
+    EBlastDbVersion m_DbVersion;      ///< BLASTDB version
 
     /// Column titles.
     vector<string> m_ColumnTitles;
@@ -526,6 +529,7 @@ private:
                                   const vector< vector<int> >    & membbits,
                                   const vector< vector<int> >    & linkouts,
                                   int                              pig,
+                                  set<Int4>                      & tax_ids,
                                   int                              OID=-1,
                                   bool                             parse_ids=true,
                                   bool                             long_seqid=false);
@@ -604,6 +608,8 @@ private:
     /// Binary header in format that will be written to disk.
     string m_BinHdr;
 
+    set<Int4> m_TaxIds;
+
     // Volumes
 
     /// This volume is currently accepting sequences.
@@ -621,9 +627,18 @@ private:
     /// Registry for masking algorithms in this database.
     CMaskInfoRegistry m_MaskAlgoRegistry;
 
+    ///Write lmdb handle
+    CRef <CWriteDB_LMDB>    m_Lmdbdb;
+
+    ///Write tax info handle
+    CRef <CWriteDB_TaxID>    m_Taxdb;
+
     /// If true, use long sequence id format (database|accession) for all
     /// acessions
     bool m_LongSeqId;
+    
+    ///Current oid to use for lmdb
+    int m_LmdbOid;
 };
 
 END_NCBI_SCOPE
