@@ -258,29 +258,11 @@ private:
     void DbtagBC (CDbtag& dbt);
 
     void PubdescBC (CPubdesc& pub);
-    void PubEquivBC (CPub_equiv& pub_equiv);
-    EAction PubBC(CPub& pub, bool fix_initials);
-    EAction CitGenBC(CCit_gen& cg, bool fix_initials);
-    EAction CitSubBC(CCit_sub& cs, bool fix_initials);
-    EAction CitArtBC(CCit_art& ca, bool fix_initials);
-    EAction CitBookBC(CCit_book& cb, bool fix_initials);
-    EAction CitPatBC(CCit_pat& cp, bool fix_initials);
-    EAction CitLetBC(CCit_let& cl, bool fix_initials);
-    EAction CitProcBC(CCit_proc& cb, bool fix_initials);
-    EAction CitJourBC(CCit_jour &j, bool fix_initials);
-    EAction MedlineEntryBC(CMedline_entry& ml, bool fix_initials);
-    void AuthListBC( CAuth_list& al, bool fix_initials );
-    void AffilBC( CAffil& af );
-    enum EImprintBC {
-        eImprintBC_AllowStatusChange =  2,
-        eImprintBC_ForbidStatusChange
-    };
-    void ImprintBC( CImprint& imprint, EImprintBC is_status_change_allowed );
     void PubSetBC( CPub_set &pub_set );
 
     void ImpFeatBC( CSeq_feat& sf );
 
-    void SiteFeatBC( CSeqFeatData::ESite &site, CSeq_feat& sf );
+    void SiteFeatBC( const CSeqFeatData::ESite &site, CSeq_feat& sf );
 
     void SeqLocBC( CSeq_loc &loc );
     void ConvertSeqLocWholeToInt( CSeq_loc &loc );
@@ -520,14 +502,14 @@ private:
 
     void x_DecodeXMLMarkChanged( std::string & str );
 
-
 private:
     void x_SortSeqDescs( CSeq_entry & seq_entry );
 
-    void x_RemoveDupBioSource( CBioseq & bioseq );
     void x_FixStructuredCommentKeywords( CBioseq & bioseq );
 
     void x_RemoveDupBioSource( CBioseq_set & bioseq_set );
+    void x_RemoveDupBioSource(CBioseq & bioseq);
+    void x_RemoveDupBioSource(CSeq_entry& se, const CBioSource& src);
 
     void x_RemoveDupPubs(CSeq_descr & descr);
 
@@ -542,7 +524,6 @@ private:
     void x_RemoveEmptyUserObject( CSeq_descr & seq_descr );
     void x_SetMolInfoTechFromGenBankBlock(CSeq_descr& seq_descr, CGB_block& block);
     void x_SetMolInfoTechFromGenBankBlock(CSeq_descr& seq_descr);
-    static bool s_ShouldRemoveKeyword(const string& keyword, CMolInfo::TTech tech);
     static bool x_CleanGenbankKeywords(CGB_block& blk, CMolInfo::TTech tech);
     void x_CleanupGenbankBlock(CBioseq& seq);
     void x_CleanupGenbankBlock(CBioseq_set& set);
@@ -594,9 +575,6 @@ private:
     void x_MoveNpPub(CBioseq_set& np_set, CSeq_descr& descr);
     void x_MovePopPhyMutPub(CBioseq_set& bioseq_set);
     void x_RemovePub(CSeq_entry& se, const CPubdesc& pub);
-    void x_RemovePub(CBioseq& seq, const CPubdesc& pub);
-    void x_RemovePub(CBioseq_set& set, const CPubdesc& pub);
-    void x_RemovePub(CSeq_descr& descr, const CPubdesc& pub);
 
     bool x_IsDBLinkUserObj( const CSeqdesc & desc );
 
