@@ -1,4 +1,4 @@
-/*  $Id: gbloader.cpp 572532 2018-10-16 12:00:58Z ivanov $
+/*  $Id: gbloader.cpp 578552 2019-01-22 15:38:07Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -137,6 +137,8 @@ public:
 
     virtual bool GetAddWGSMasterDescr(void) const;
 
+    virtual EGBErrorAction GetPTISErrorAction(void) const;
+    
     friend class CGBDataLoader;
 
 private:
@@ -641,6 +643,28 @@ void CGBDataLoader::x_CreateDriver(const CGBLoaderParams& params)
             }
         }
     }
+    m_PTISErrorAction = eGBErrorAction_report;
+    if ( gb_params ) {
+        string param =
+            GetParam(gb_params, NCBI_GBLOADER_PARAM_PTIS_ERROR_ACTION);
+        if ( !param.empty() ) {
+            if ( NStr::EqualNocase(param, NCBI_GBLOADER_PARAM_PTIS_ERROR_ACTION_IGNORE) ) {
+                m_PTISErrorAction = eGBErrorAction_ignore;
+            }
+            else if ( NStr::EqualNocase(param, NCBI_GBLOADER_PARAM_PTIS_ERROR_ACTION_REPORT) ) {
+                m_PTISErrorAction = eGBErrorAction_report;
+            }
+            else if ( NStr::EqualNocase(param, NCBI_GBLOADER_PARAM_PTIS_ERROR_ACTION_THROW) ) {
+                m_PTISErrorAction = eGBErrorAction_throw;
+            }
+            else {
+                NCBI_THROW_FMT(CLoaderException, eBadConfig,
+                               "Bad value of parameter "
+                               NCBI_GBLOADER_PARAM_PTIS_ERROR_ACTION
+                               ": \""<<param<<"\"");
+            }
+        }
+    }
     
     m_Dispatcher = new CReadDispatcher;
     m_InfoManager = new CGBInfoManager(queue_size);
@@ -1946,6 +1970,12 @@ CGBReaderRequestResult::GetAddWGSMasterDescr(void) const
 }
 
 
+EGBErrorAction CGBReaderRequestResult::GetPTISErrorAction(void) const
+{
+    return m_Loader->GetPTISErrorAction();
+}
+
+
 /*
 bool CGBDataLoader::LessBlobId(const TBlobId& id1, const TBlobId& id2) const
 {
