@@ -1,4 +1,4 @@
-/*  $Id: request_result.cpp 493171 2016-02-24 19:31:13Z vasilche $
+/*  $Id: request_result.cpp 578552 2019-01-22 15:38:07Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -792,6 +792,12 @@ bool CReaderRequestResult::GetAddWGSMasterDescr(void) const
 }
 
 
+EGBErrorAction CReaderRequestResult::GetPTISErrorAction(void) const
+{
+    return eGBErrorAction_report;
+}
+
+
 CReaderRequestResult::~CReaderRequestResult(void)
 {
     ReleaseLocks();
