@@ -1,4 +1,4 @@
-/*  $Id: loc_edit.cpp 538916 2017-06-15 15:31:20Z ivanov $
+/*  $Id: loc_edit.cpp 564018 2018-05-18 17:59:03Z bollin $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -35,6 +35,10 @@
 #include <objtools/edit/cds_fix.hpp>
 #include <objects/seq/Seq_descr.hpp>
 #include <objects/seq/Seqdesc.hpp>
+#include <objects/seq/Delta_ext.hpp>
+#include <objects/seq/Delta_seq.hpp>
+#include <objects/seq/Seq_ext.hpp>
+#include <objects/seq/Seq_inst.hpp>
 #include <objects/seqfeat/Code_break.hpp>
 #include <objects/seqfeat/Cdregion.hpp>
 #include <objects/seqfeat/RNA_ref.hpp>
@@ -245,27 +249,51 @@ bool CLocationEditPolicy::Is3AtEndOfSeq(const CSeq_loc& loc, CBioseq_Handle bsh)
 }
 
 
-bool s_Know5WithoutBsh(const CSeq_loc& loc)
+bool CLocationEditPolicy::Is5AtEndOfSeq(const CSeq_loc& loc, CScope& scope, bool& confident)
 {
-    ENa_strand strand = loc.GetStrand();
-
-    if (strand == eNa_strand_minus) {
-        return false;
+    bool rval = false;
+    confident = true;
+    CSeq_loc_CI first_l(loc);
+    if (!first_l.IsSetStrand() || first_l.GetStrand() != eNa_strand_minus) {
+        // positive strand, just need to know if it's at zero
+        rval = (first_l.GetRange().GetFrom() == 0);
     } else {
-        return true;
+        // negative strand
+        try {
+            CBioseq_Handle bsh = scope.GetBioseqHandle(first_l.GetSeq_id());
+            rval = (first_l.GetRange().GetTo() == bsh.GetBioseqLength() - 1);
+        } catch (CException& ex) {
+            confident = false;
+        }
     }
+    return rval;
 }
 
 
-bool s_Know3WithoutBsh(const CSeq_loc& loc)
+bool CLocationEditPolicy::Is3AtEndOfSeq(const CSeq_loc& loc, CScope& scope, bool& confident)
 {
-    ENa_strand strand = loc.GetStrand();
-
-    if (strand == eNa_strand_minus) {
-        return true;
+    bool rval = false;
+    confident = true;
+    CSeq_loc_CI last_l(loc);
+    size_t num_intervals = last_l.GetSize();
+    last_l.SetPos(num_intervals - 1);
+    if (!last_l.IsSetStrand() || last_l.GetStrand() != eNa_strand_minus) {
+        // positive strand
+        try {
+            CBioseq_Handle bsh = scope.GetBioseqHandle(last_l.GetSeq_id());
+            if (bsh) {
+                rval = (last_l.GetRange().GetTo() == bsh.GetBioseqLength() - 1);
+            } else {
+                confident = false;
+            }
+        } catch (CException& ex) {
+            confident = false;
+        }
     } else {
-        return false;
+        // negative strand, just need to know if it's at zero
+        rval = (last_l.GetRange().GetFrom() == 0);
     }
+    return rval;
 }
 
 
@@ -278,7 +306,7 @@ bool CLocationEditPolicy::Interpret5Policy
     do_set_5_partial = false;
     do_clear_5_partial = false;
     const CSeq_loc& loc = orig_feat.GetLocation();
-    CBioseq_Handle bsh = scope.GetBioseqHandle(loc);
+   
 
     switch (m_PartialPolicy5) {
         case ePartialPolicy_eNoChange:
@@ -287,31 +315,36 @@ bool CLocationEditPolicy::Interpret5Policy
         case ePartialPolicy_eSet:
             if (!orig_feat.GetLocation().IsPartialStart(eExtreme_Biological)) {
                 do_set_5_partial = true;
-            } else if (m_Extend5 && !Is5AtEndOfSeq(loc, bsh)
-                       && (bsh || s_Know5WithoutBsh(loc))) {
-                do_set_5_partial = true;
+            } else if (m_Extend5) {
+                bool confident = false;
+                bool at_5 = Is5AtEndOfSeq(loc, scope, confident);
+                if (confident && !at_5) {
+                    do_set_5_partial = true;
+                }
             }
             break;
         case ePartialPolicy_eSetAtEnd:
-            if (Is5AtEndOfSeq(loc, bsh) 
-                && !orig_feat.GetLocation().IsPartialStart(eExtreme_Biological)
-                && (bsh || s_Know5WithoutBsh(loc))) {
-                do_set_5_partial = true;
+            if (!orig_feat.GetLocation().IsPartialStart(eExtreme_Biological)) {
+                bool confident = false;
+                if (Is5AtEndOfSeq(loc, scope, confident) && confident) {
+                    do_set_5_partial = true;
+                }
             }
             break;
         case ePartialPolicy_eSetForBadEnd:
             if (!orig_feat.GetLocation().IsPartialStart(eExtreme_Biological)
-                && orig_feat.GetData().IsCdregion()
-                && bsh) {
+                && orig_feat.GetData().IsCdregion()) {
                 string transl_prot;
+                bool confident = true;
                 try {
                     CSeqTranslator::Translate(orig_feat, scope, transl_prot,
                                                 false,   // do not include stop codons
                                                 false);  // do not remove trailing X/B/Z
 
-                } catch ( const runtime_error& ) {
+                } catch ( const CException& ) {
+                    confident = false;
                 }
-                if (!NStr::StartsWith(transl_prot, "M", NStr::eNocase)) {
+                if (confident && !NStr::StartsWith(transl_prot, "M", NStr::eNocase)) {
                     do_set_5_partial = true;
                 }
             }
@@ -331,10 +364,11 @@ bool CLocationEditPolicy::Interpret5Policy
             }
             break;
         case ePartialPolicy_eClearNotAtEnd:
-            if (orig_feat.GetLocation().IsPartialStart(eExtreme_Biological)
-                && !Is5AtEndOfSeq(orig_feat.GetLocation(), bsh)
-                && (bsh || s_Know5WithoutBsh(loc))) {
-                do_clear_5_partial = true;
+            if (orig_feat.GetLocation().IsPartialStart(eExtreme_Biological)) {
+                bool confident = false;
+                if (!Is5AtEndOfSeq(loc, scope, confident) && confident) {
+                    do_clear_5_partial = true;
+                }
             }
             break;
         case ePartialPolicy_eClearForGoodEnd:
@@ -342,17 +376,18 @@ bool CLocationEditPolicy::Interpret5Policy
                 && orig_feat.GetData().IsCdregion()
                 && (!orig_feat.GetData().GetCdregion().IsSetFrame() ||
                     orig_feat.GetData().GetCdregion().GetFrame() == CCdregion::eFrame_not_set ||
-                    orig_feat.GetData().GetCdregion().GetFrame() == CCdregion::eFrame_one)
-                && bsh) {
+                    orig_feat.GetData().GetCdregion().GetFrame() == CCdregion::eFrame_one)) {
                 string transl_prot;
+                bool confident = true;
                 try {
                     CSeqTranslator::Translate(orig_feat, scope, transl_prot,
                                                 false,   // do not include stop codons
                                                 false);  // do not remove trailing X/B/Z
 
-                } catch ( const runtime_error& ) {
+                } catch ( const CException& ) {
+                    confident = false;
                 }
-                if (NStr::StartsWith(transl_prot, "M", NStr::eNocase)) {
+                if (confident && NStr::StartsWith(transl_prot, "M", NStr::eNocase)) {
                     do_clear_5_partial = true;
                 }
             }
@@ -372,39 +407,45 @@ bool CLocationEditPolicy::Interpret3Policy
     do_clear_3_partial = false;
     
     const CSeq_loc& loc = orig_feat.GetLocation();
-    CBioseq_Handle bsh = scope.GetBioseqHandle(loc);
 
     switch (m_PartialPolicy3) {
         case ePartialPolicy_eNoChange:
             // do nothing
             break;
         case ePartialPolicy_eSet:
-            if (!orig_feat.GetLocation().IsPartialStop(eExtreme_Biological)) {
-                do_set_3_partial = true;
-            } else if (m_Extend3 && !Is3AtEndOfSeq(loc, bsh) && (bsh || s_Know3WithoutBsh(loc))) {
+            if (!loc.IsPartialStop(eExtreme_Biological)) {
                 do_set_3_partial = true;
+            } else if (m_Extend3) {
+                bool confident = false;
+                bool at_3 = Is3AtEndOfSeq(loc, scope, confident);
+                if (confident && !at_3) {
+                    do_set_3_partial = true;
+                }
             }
             break;
         case ePartialPolicy_eSetAtEnd:
-            if (!orig_feat.GetLocation().IsPartialStop(eExtreme_Biological) 
-                && Is3AtEndOfSeq(orig_feat.GetLocation(), bsh)
-                && (bsh || s_Know3WithoutBsh(loc))) {
-                do_set_3_partial = true;
+            if (!loc.IsPartialStop(eExtreme_Biological)) {
+                bool confident = false;
+                bool at_3 = Is3AtEndOfSeq(loc, scope, confident);
+                if (at_3 && confident) {
+                    do_set_3_partial = true;
+                }
             }
             break;
         case ePartialPolicy_eSetForBadEnd:
-            if (!orig_feat.GetLocation().IsPartialStop(eExtreme_Biological)
-                && orig_feat.GetData().IsCdregion()
-                && bsh) {
+            if (!loc.IsPartialStop(eExtreme_Biological)
+                && orig_feat.GetData().IsCdregion()) {
                 string transl_prot;
+                bool confident = true;
                 try {
                     CSeqTranslator::Translate(orig_feat, scope, transl_prot,
                                                 true,   // include stop codons
                                                 false);  // do not remove trailing X/B/Z
 
-                } catch ( const runtime_error& ) {
+                } catch ( const CException& ) {
+                    confident = false;
                 }
-                if (!NStr::EndsWith(transl_prot, "*", NStr::eNocase)) {
+                if (confident && !NStr::EndsWith(transl_prot, "*", NStr::eNocase)) {
                     do_set_3_partial = true;
                 }
             }
@@ -413,30 +454,33 @@ bool CLocationEditPolicy::Interpret3Policy
             // not allowed for 3' end
             break;
         case ePartialPolicy_eClear:
-            if (orig_feat.GetLocation().IsPartialStop(eExtreme_Biological)) {
+            if (loc.IsPartialStop(eExtreme_Biological)) {
                 do_clear_3_partial = true;
             }
             break;
         case ePartialPolicy_eClearNotAtEnd:
-            if (orig_feat.GetLocation().IsPartialStop(eExtreme_Biological)
-                && !Is3AtEndOfSeq(orig_feat.GetLocation(), bsh)
-                && (bsh || s_Know3WithoutBsh(loc))) {
-                do_clear_3_partial = true;
+            if (loc.IsPartialStop(eExtreme_Biological)) {
+                bool confident = false;
+                bool at_3 = Is3AtEndOfSeq(loc, scope, confident);
+                if (!at_3 && confident) {
+                    do_clear_3_partial = true;
+                }
             }
             break;
         case ePartialPolicy_eClearForGoodEnd:
-            if (orig_feat.GetLocation().IsPartialStop(eExtreme_Biological)
-                && orig_feat.GetData().IsCdregion()
-                && bsh) {
+            if (loc.IsPartialStop(eExtreme_Biological)
+                && orig_feat.GetData().IsCdregion()) {
                 string transl_prot;
+                bool confident = true;
                 try {
                     CSeqTranslator::Translate(orig_feat, scope, transl_prot,
                                                 true,   // include stop codons
                                                 false);  // do not remove trailing X/B/Z
 
-                } catch ( const runtime_error& ) {
+                } catch ( const CException& ) {
+                    confident = false;
                 }
-                if (NStr::EndsWith(transl_prot, "*", NStr::eNocase)) {
+                if (confident && NStr::EndsWith(transl_prot, "*", NStr::eNocase)) {
                     do_clear_3_partial = true;
                 }
             }
@@ -446,6 +490,61 @@ bool CLocationEditPolicy::Interpret3Policy
 }
 
 
+CRef<CSeq_loc> SeqLocExtend5(const CSeq_loc& loc, size_t pos, CScope* scope)
+{
+    CSeq_loc_CI first_l(loc);
+    CConstRef<CSeq_loc> first_loc = first_l.GetRangeAsSeq_loc();
+
+    size_t loc_start = first_loc->GetStart(eExtreme_Biological);
+    bool partial_start = first_loc->IsPartialStart(eExtreme_Biological);
+    ENa_strand strand = first_loc->IsSetStrand() ? first_loc->GetStrand() : eNa_strand_plus;
+    CRef<CSeq_loc> new_loc(NULL);
+
+    CRef<CSeq_id> id(new CSeq_id());
+    id->Assign(first_l.GetSeq_id());
+
+    if (pos < loc_start && strand != eNa_strand_minus) {
+        CRef<CSeq_loc> add(new CSeq_loc(*id, pos, loc_start - 1, strand));
+        add->SetPartialStart(partial_start, eExtreme_Positional);
+        new_loc = sequence::Seq_loc_Add(loc, *add, CSeq_loc::fSort | CSeq_loc::fMerge_AbuttingOnly, scope);
+    } else if (pos > loc_start && strand == eNa_strand_minus) {
+        CRef<CSeq_loc> add(new CSeq_loc(*id, loc_start + 1, pos, strand));
+        add->SetPartialStop(partial_start, eExtreme_Positional);
+        new_loc = sequence::Seq_loc_Add(loc, *add, CSeq_loc::fSort | CSeq_loc::fMerge_AbuttingOnly, scope);
+    }
+    return new_loc;
+}
+
+
+CRef<CSeq_loc> SeqLocExtend3(const CSeq_loc& loc, size_t pos, CScope* scope)
+{
+    CSeq_loc_CI last_l(loc);
+    size_t num_intervals = last_l.GetSize();
+    last_l.SetPos(num_intervals - 1);
+    CConstRef<CSeq_loc> last_loc = last_l.GetRangeAsSeq_loc();
+
+    size_t loc_stop = last_loc->GetStop(eExtreme_Biological);
+    bool partial_stop = last_loc->IsPartialStop(eExtreme_Biological);
+    ENa_strand strand = last_loc->IsSetStrand() ? last_loc->GetStrand() : eNa_strand_plus;
+    CRef<CSeq_loc> new_loc(NULL);
+
+    CRef<CSeq_id> id(new CSeq_id());
+    id->Assign(last_l.GetSeq_id());
+
+    if (pos > loc_stop && strand != eNa_strand_minus) {
+        CRef<CSeq_loc> add(new CSeq_loc(*id, loc_stop + 1, pos, strand));
+        add->SetPartialStop(partial_stop, eExtreme_Positional);
+        new_loc = sequence::Seq_loc_Add(loc, *add, CSeq_loc::fSort | CSeq_loc::fMerge_AbuttingOnly, scope);
+    } else if (pos < loc_stop && strand == eNa_strand_minus) {
+        CRef<CSeq_loc> add(new CSeq_loc(*id, pos, loc_stop - 1, strand));
+        add->SetPartialStart(partial_stop, eExtreme_Positional);
+        new_loc = sequence::Seq_loc_Add(loc, *add, CSeq_loc::fSort | CSeq_loc::fMerge_AbuttingOnly, scope);
+    }
+    return new_loc;
+}
+
+
+
 CRef<CSeq_loc> SeqLocExtend(const CSeq_loc& loc, size_t pos, CScope* scope)
 {
     size_t loc_start = loc.GetStart(eExtreme_Positional);
@@ -637,28 +736,56 @@ CRef<CSeq_loc> CLocationEditPolicy::ConvertToOrder(const CSeq_loc& orig_loc, boo
     return new_loc;
 }
 
+void AdjustFrameFor5Extension(CSeq_feat& feat, size_t diff)
+{
+    // adjust frame to maintain consistency
+    if (diff % 3 > 0 && feat.GetData().IsCdregion()) {
+        int orig_frame = 1;
+        if (feat.GetData().GetCdregion().IsSetFrame()) {
+            if (feat.GetData().GetCdregion().GetFrame() == CCdregion::eFrame_two) {
+                orig_frame = 2;
+            } else if (feat.GetData().GetCdregion().GetFrame() == CCdregion::eFrame_three) {
+                orig_frame = 3;
+            }
+        }
+        CCdregion::EFrame new_frame = CCdregion::eFrame_not_set;
+        switch ((orig_frame + diff % 3) % 3) {
+            case 1:
+                new_frame = CCdregion::eFrame_not_set;
+                break;
+            case 2:
+                new_frame = CCdregion::eFrame_two;
+                break;
+            case 0:
+                new_frame = CCdregion::eFrame_three;
+                break;
+        }
+        feat.SetData().SetCdregion().SetFrame(new_frame);
+    }
+}
+
+
 bool CLocationEditPolicy::Extend5(CSeq_feat& feat, CScope& scope)
 {
     bool extend = false;
-    CBioseq_Handle bsh = scope.GetBioseqHandle(feat.GetLocation());
-    if (bsh || s_Know5WithoutBsh(feat.GetLocation())) {
-        ENa_strand strand = feat.GetLocation().GetStrand();
-        size_t start = feat.GetLocation().GetStart(eExtreme_Biological);
+
+    bool confident = false;
+    if (!Is5AtEndOfSeq(feat.GetLocation(), scope, confident) && confident) {
         int diff = 0;
-        if (strand == eNa_strand_minus) {                
-            if (start < bsh.GetInst_Length() - 1) {
-                diff = bsh.GetInst_Length() - feat.GetLocation().GetStart(eExtreme_Biological) - 1;
-                CRef<CSeq_loc> new_loc = SeqLocExtend(feat.GetLocation(), bsh.GetInst_Length() - 1, &scope);
-                if (new_loc) {
-                    feat.SetLocation().Assign(*new_loc);
-                    extend = true;
-                } else {
-                    diff = 0;
-                }
+        CSeq_loc_CI first_l(feat.GetLocation());
+        if (first_l.IsSetStrand() && first_l.GetStrand() == eNa_strand_minus) {
+            CBioseq_Handle bsh = scope.GetBioseqHandle(first_l.GetSeq_id());
+            diff = bsh.GetInst().GetLength() - first_l.GetRange().GetTo() - 1;
+            CRef<CSeq_loc> new_loc = SeqLocExtend5(feat.GetLocation(), bsh.GetInst_Length() - 1, &scope);
+            if (new_loc) {
+                feat.SetLocation().Assign(*new_loc);
+                extend = true;
+            } else {
+                diff = 0;
             }
-        } else  if (start > 0) {
-            diff = start;
-            CRef<CSeq_loc> new_loc = SeqLocExtend(feat.GetLocation(), 0, &scope);
+        } else {
+            diff = first_l.GetRange().GetFrom();
+            CRef<CSeq_loc> new_loc = SeqLocExtend5(feat.GetLocation(), 0, &scope);
             if (new_loc) {
                 feat.SetLocation().Assign(*new_loc);
                 extend = true;
@@ -666,30 +793,9 @@ bool CLocationEditPolicy::Extend5(CSeq_feat& feat, CScope& scope)
                 diff = 0;
             }
         }
+
         // adjust frame to maintain consistency
-        if (diff % 3 > 0 && feat.GetData().IsCdregion()) {
-            int orig_frame = 1;
-            if (feat.GetData().GetCdregion().IsSetFrame()) {
-                if (feat.GetData().GetCdregion().GetFrame() == CCdregion::eFrame_two) {
-                    orig_frame = 2;
-                } else if (feat.GetData().GetCdregion().GetFrame() == CCdregion::eFrame_three) {
-                    orig_frame = 3;
-                }
-            }
-            CCdregion::EFrame new_frame = CCdregion::eFrame_not_set;
-            switch ((orig_frame + diff % 3) % 3) {
-                case 1:
-                    new_frame = CCdregion::eFrame_not_set;
-                    break;
-                case 2:
-                    new_frame = CCdregion::eFrame_two;
-                    break;
-                case 0:
-                    new_frame = CCdregion::eFrame_three;
-                    break;
-            }
-            feat.SetData().SetCdregion().SetFrame(new_frame);
-        }
+        AdjustFrameFor5Extension(feat, diff);
     }
     return extend;
 }
@@ -697,20 +803,23 @@ bool CLocationEditPolicy::Extend5(CSeq_feat& feat, CScope& scope)
 bool CLocationEditPolicy::Extend3(CSeq_feat& feat, CScope& scope)
 {
     bool extend = false;
-    CBioseq_Handle bsh = scope.GetBioseqHandle(feat.GetLocation());
-    if (bsh || s_Know3WithoutBsh(feat.GetLocation())) {
-        ENa_strand strand = feat.GetLocation().GetStrand();
-        size_t stop = feat.GetLocation().GetStop(eExtreme_Biological);
+
+    bool confident = false;
+    if (!Is3AtEndOfSeq(feat.GetLocation(), scope, confident) && confident) {
+        CSeq_loc_CI last_l(feat.GetLocation());
+        size_t num_intervals = last_l.GetSize();
+        last_l.SetPos(num_intervals - 1);
+
+        ENa_strand strand = last_l.GetStrand();
         if (strand == eNa_strand_minus) {                
-            if (stop > 0) {
-                CRef<CSeq_loc> new_loc = SeqLocExtend(feat.GetLocation(), 0, &scope);
-                if (new_loc) {
-                    feat.SetLocation().Assign(*new_loc);
-                    extend = true;
-                }
+            CRef<CSeq_loc> new_loc = SeqLocExtend3(feat.GetLocation(), 0, &scope);
+            if (new_loc) {
+                feat.SetLocation().Assign(*new_loc);
+                extend = true;
             }
-        } else  if (stop < bsh.GetInst_Length() - 1) {
-            CRef<CSeq_loc> new_loc = SeqLocExtend(feat.GetLocation(), bsh.GetInst_Length() - 1, &scope);
+        } else {
+            CBioseq_Handle bsh = scope.GetBioseqHandle(last_l.GetSeq_id());
+            CRef<CSeq_loc> new_loc = SeqLocExtend3(feat.GetLocation(), bsh.GetInst_Length() - 1, &scope);
             if (new_loc) {
                 feat.SetLocation().Assign(*new_loc);
                 extend = true;
@@ -746,12 +855,12 @@ bool ApplyPolicyToFeature(const CLocationEditPolicy& policy, const CSeq_feat& or
                 new_gene->Assign(*old_gene);
                 bool gene_change = false;
                 // adjust ends of gene to match ends of feature
-                CRef<CSeq_loc> new_loc = SeqLocExtend(new_gene->GetLocation(), feat_start, &scope);
+                CRef<CSeq_loc> new_loc = SeqLocExtend5(new_gene->GetLocation(), feat_start, &scope);
                 if (new_loc) {
                     new_gene->SetLocation().Assign(*new_loc);
                     gene_change = true;
                 }
-                new_loc = SeqLocExtend(new_gene->GetLocation(), feat_stop, &scope);
+                new_loc = SeqLocExtend3(new_gene->GetLocation(), feat_stop, &scope);
                 if (new_loc) {
                     new_gene->SetLocation().Assign(*new_loc);
                     gene_change = true;
@@ -771,7 +880,8 @@ bool ApplyPolicyToFeature(const CLocationEditPolicy& policy, const CSeq_feat& or
         // retranslate or resynch if coding region
         if (new_feat->IsSetProduct() && new_feat->GetData().IsCdregion()) {
             if (!retranslate_cds || !RetranslateCDS(*new_feat, scope)) {
-                AdjustForCDSPartials(*new_feat, scope.GetBioseqHandle(new_feat->GetLocation()).GetSeq_entry_Handle());
+                CSeq_loc_CI l(new_feat->GetLocation());
+                feature::AdjustForCDSPartials(*new_feat, scope);
             }
         }
     }
@@ -1632,17 +1742,16 @@ CRef<CSeq_interval> SplitLocationForGap(CSeq_interval& before,
     // Feature location
     TSeqPos feat_from = before.GetFrom();
     TSeqPos feat_to = before.GetTo();
-
     CRef<CSeq_interval> after(NULL);
     if (feat_to < start) {
         // gap completely after location
         return after;
     }
 
-    if (feat_from > start && !(options & eSplitLocOption_split_in_intron)) {
-        // if gap completely before location, but not splitting in introns,
-        // no change
-        return after;
+    if (feat_from > stop && !(options & eSplitLocOption_split_in_intron)) {
+      // if gap completely before location, but not splitting in introns,
+      // no change
+      return after;
     }
 
     if (feat_from < start && feat_to > stop) {
@@ -1697,8 +1806,6 @@ void SplitLocationForGap(CSeq_loc::TPacked_int& before_intervals,
             }
             if (after) {
                 after_intervals.Set().push_back(after);
-                // from here on, always move intervals to the right to the other loc
-                options |= eSplitLocOption_split_in_intron;
             }
         }
     }
@@ -1794,11 +1901,8 @@ void SplitLocationForGap(CSeq_loc& loc1, CSeq_loc& loc2,
                         }
                         if (after->Which() != CSeq_loc::e_not_set) {
                             after_mix->Set().push_back(after);
-                            // from here on, always move intervals to the right to the other loc
-                            options |= eSplitLocOption_split_in_intron;
                         }
                     }
-
                     // Update the original list
                     if (before_mix.Set().empty()) {
                         loc1.Reset();
@@ -2231,6 +2335,260 @@ bool CorrectIntervalOrder(CSeq_loc& loc)
 }
 
 
+bool IsExtendableLeft(TSeqPos left, const CBioseq& seq, CScope* scope, TSeqPos& extend_len)
+{
+    bool rval = false;
+    if (left < 3) {
+        rval = true;
+        extend_len = left;
+    } else if (seq.IsSetInst() && seq.GetInst().IsSetRepr() &&
+               seq.GetInst().GetRepr() == CSeq_inst::eRepr_delta &&
+               seq.GetInst().IsSetExt() &&
+               seq.GetInst().GetExt().IsDelta()) {
+        TSeqPos offset = 0;
+        TSeqPos last_gap_stop = 0;
+        ITERATE(CDelta_ext::Tdata, it, seq.GetInst().GetExt().GetDelta().Get()) {
+            if ((*it)->IsLiteral()) {
+                offset += (*it)->GetLiteral().GetLength();
+                if (!(*it)->GetLiteral().IsSetSeq_data()) {
+                    last_gap_stop = offset;
+                } else if ((*it)->GetLiteral().GetSeq_data().IsGap()) {
+                    last_gap_stop = offset;
+                }
+            } else if ((*it)->IsLoc()) {
+                offset += sequence::GetLength((*it)->GetLoc(), scope);
+            }
+            if (offset > left) {
+                break;
+            }
+        }
+        if (left >= last_gap_stop && left - last_gap_stop <= 3) {
+            rval = true;
+            extend_len = left - last_gap_stop;
+        }
+    }
+    return rval;
+}
+
+
+bool IsExtendableRight(TSeqPos right, const CBioseq& seq, CScope* scope, TSeqPos& extend_len)
+{
+    bool rval = false;
+    if (right > seq.GetLength() - 5) {
+        rval = true;
+        extend_len = seq.GetLength() - right - 1;
+    } else if (seq.IsSetInst() && seq.GetInst().IsSetRepr() &&
+        seq.GetInst().GetRepr() == CSeq_inst::eRepr_delta &&
+        seq.GetInst().IsSetExt() &&
+        seq.GetInst().GetExt().IsDelta()) {
+        TSeqPos offset = 0;
+        TSeqPos next_gap_start = 0;
+        ITERATE(CDelta_ext::Tdata, it, seq.GetInst().GetExt().GetDelta().Get()) {
+            if ((*it)->IsLiteral()) {
+                if (!(*it)->GetLiteral().IsSetSeq_data()) {
+                    next_gap_start = offset;
+                } else if ((*it)->GetLiteral().GetSeq_data().IsGap()) {
+                    next_gap_start = offset;
+                }
+                offset += (*it)->GetLiteral().GetLength();
+            } else if ((*it)->IsLoc()) {
+                offset += sequence::GetLength((*it)->GetLoc(), scope);
+            }
+            if (offset > right + 4) {
+                break;
+            }
+        }
+        if (next_gap_start > right && next_gap_start - right - 1 <= 3) {
+            rval = true;
+            extend_len = next_gap_start - right - 1;
+        }
+    }
+    return rval;
+}
+
+
+bool AdjustFeatureEnd5(CSeq_feat& cds, vector<CRef<CSeq_feat> > related_features, CScope& scope)
+{
+    if (!cds.GetLocation().IsPartialStart(eExtreme_Biological)) {
+        return false;
+    }
+
+    bool rval = false;
+
+    CSeq_loc_CI first_l(cds.GetLocation());
+    CBioseq_Handle bsh = scope.GetBioseqHandle(first_l.GetSeq_id());
+    CConstRef<CBioseq> seq = bsh.GetCompleteBioseq();
+
+    TSeqPos start = cds.GetLocation().GetStart(eExtreme_Biological);
+    TSeqPos new_start = start;
+    TSeqPos extend_len = 0;
+    bool extendable = false;
+
+    if (!first_l.IsSetStrand() || first_l.GetStrand() != eNa_strand_minus) {
+        // positive strand
+        if (start > 0) {
+            extendable = IsExtendableLeft(start, *seq, &scope, extend_len);
+            new_start = start - extend_len;
+        }
+    } else {
+        if (start < seq->GetInst().GetLength() - 1) {
+            extendable = IsExtendableRight(start, *seq, &scope, extend_len);
+            new_start = start + extend_len;
+        }
+    }
+    if (extendable) {
+        CRef<CSeq_loc> cds_loc = SeqLocExtend5(cds.GetLocation(), new_start, &scope);
+        if (cds_loc) {
+            for (auto it : related_features) {
+                if (it->GetLocation().GetStart(eExtreme_Biological) == start) {
+                    CRef<CSeq_loc> related_loc = SeqLocExtend5(it->GetLocation(), start - extend_len, &scope);
+                    if (related_loc) {
+                        it->SetLocation().Assign(*related_loc);
+                        if (it->IsSetData() && it->GetData().IsCdregion()) {
+                            AdjustFrameFor5Extension(cds, extend_len);
+                        }
+                    }
+                }
+            }
+            cds.SetLocation().Assign(*cds_loc);
+            if (cds.IsSetData() && cds.GetData().IsCdregion()) {
+                AdjustFrameFor5Extension(cds, extend_len);
+            }
+
+            rval = true;
+        }
+    }
+
+    return rval;
+}
+
+
+bool AdjustFeatureEnd3(CSeq_feat& cds, vector<CRef<CSeq_feat> > related_features, CScope& scope)
+{
+    if (!cds.GetLocation().IsPartialStop(eExtreme_Biological)) {
+        return false;
+    }
+
+    bool rval = false;
+
+    CSeq_loc_CI last_l(cds.GetLocation());
+    size_t num_intervals = last_l.GetSize();
+    last_l.SetPos(num_intervals - 1);
+
+    CBioseq_Handle bsh = scope.GetBioseqHandle(last_l.GetSeq_id());
+    CConstRef<CBioseq> seq = bsh.GetCompleteBioseq();
+
+    TSeqPos stop = cds.GetLocation().GetStop(eExtreme_Biological);
+    TSeqPos new_stop = stop;
+    TSeqPos extend_len = 0;
+    bool extendable = false;
+
+    if (!last_l.IsSetStrand() || last_l.GetStrand() != eNa_strand_minus) {
+        // positive strand
+        if (stop < seq->GetInst().GetLength() - 1) {
+            extendable = IsExtendableRight(stop, *seq, &scope, extend_len);
+            new_stop = stop + extend_len;
+        }
+    } else {
+        if (stop > 0) {
+            extendable = IsExtendableLeft(stop, *seq, &scope, extend_len);
+            new_stop = stop - extend_len;
+        }
+    }
+    if (extendable) {
+        CRef<CSeq_loc> cds_loc = SeqLocExtend3(cds.GetLocation(), new_stop, &scope);
+        if (cds_loc) {
+            for (auto it : related_features) {
+                if (it->GetLocation().GetStart(eExtreme_Biological) == stop) {
+                    CRef<CSeq_loc> related_loc = SeqLocExtend3(it->GetLocation(), new_stop, &scope);
+                    if (related_loc) {
+                        it->SetLocation().Assign(*related_loc);
+                    }
+                }
+            }
+            cds.SetLocation().Assign(*cds_loc);
+            rval = true;
+        }
+    }
+
+    return rval;
+}
+
+
+bool IsExtendable(const CSeq_feat& cds, CScope& scope)
+{
+    if (cds.GetLocation().IsPartialStart(eExtreme_Positional)) {
+        CSeq_loc_CI first_l(cds.GetLocation());
+        CBioseq_Handle bsh = scope.GetBioseqHandle(first_l.GetSeq_id());
+        CConstRef<CBioseq> seq = bsh.GetCompleteBioseq();
+        TSeqPos extend_len = 0;
+        TSeqPos start = first_l.GetRange().GetFrom();
+        if (IsExtendableLeft(start, *seq, &scope, extend_len) && extend_len > 0) {
+            return true;
+        }
+    }
+    if (cds.GetLocation().IsPartialStop(eExtreme_Positional)) {
+        CSeq_loc_CI last_l(cds.GetLocation());
+        size_t num_intervals = last_l.GetSize();
+        last_l.SetPos(num_intervals - 1);
+        CBioseq_Handle bsh = scope.GetBioseqHandle(last_l.GetSeq_id());
+        CConstRef<CBioseq> seq = bsh.GetCompleteBioseq();
+
+        TSeqPos stop = cds.GetLocation().GetStop(eExtreme_Positional);
+        TSeqPos extend_len = 0;
+
+        if (IsExtendableRight(stop, *seq, &scope, extend_len) && extend_len > 0) {
+            return true;
+        }
+    }
+    return false;
+}
+
+
+bool ExtendPartialFeatureEnds(CBioseq_Handle bsh)
+{
+    bool any_change = false;
+    CFeat_CI f(bsh);
+    CRef<feature::CFeatTree> tr(new feature::CFeatTree(f));
+    while (f) {
+        if (f->GetData().IsCdregion()) {
+            CMappedFeat gene = tr->GetBestGene(*f);
+            CMappedFeat mRNA = tr->GetParent(*f, CSeqFeatData::eSubtype_mRNA);
+            vector<CRef<CSeq_feat> > related_features;
+            if (gene) {
+                CRef<CSeq_feat> new_gene(new CSeq_feat());
+                new_gene->Assign(*(gene.GetOriginalSeq_feat()));
+                related_features.push_back(new_gene);
+            }
+            if (mRNA) {
+                CRef<CSeq_feat> new_mRNA(new CSeq_feat());
+                new_mRNA->Assign(*(mRNA.GetOriginalSeq_feat()));
+                related_features.push_back(new_mRNA);
+            }
+            CRef<CSeq_feat> new_cds(new CSeq_feat());
+            new_cds->Assign(*(f->GetOriginalSeq_feat()));
+
+            if (AdjustFeatureEnd5(*new_cds, related_features, bsh.GetScope()) ||
+                AdjustFeatureEnd3(*new_cds, related_features, bsh.GetScope())) {
+                CSeq_feat_EditHandle feh(*f);
+                feh.Replace(*new_cds);
+                if (gene) {
+                    CSeq_feat_EditHandle geh(gene);
+                    geh.Replace(*(related_features.front()));
+                }
+                if (mRNA) {
+                    CSeq_feat_EditHandle meh(mRNA);
+                    meh.Replace(*(related_features.back()));
+                }
+                any_change = true;
+            }
+        }
+        ++f;
+    }
+    return any_change;
+}
+
+
 END_SCOPE(edit)
 END_SCOPE(objects)
 END_NCBI_SCOPE
