@@ -1,4 +1,4 @@
-/*  $Id: remote_updater.cpp 539753 2017-06-27 14:26:39Z ivanov $
+/*  $Id: remote_updater.cpp 572177 2018-10-10 12:11:58Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -36,11 +36,9 @@
 #include <objects/taxon3/taxon3.hpp>
 #include <objects/mla/mla_client.hpp>
 
-#include <objtools/readers/message_listener.hpp>
-#include <objtools/format/items/reference_item.hpp>
-
 #include <objects/pub/Pub_equiv.hpp>
 #include <objects/pub/Pub.hpp>
+#include <objects/seq/Pubdesc.hpp>
 
 #include <objects/seqfeat/Org_ref.hpp>
 #include <objects/submit/Seq_submit.hpp>
@@ -99,11 +97,9 @@ void CreatePubPMID(CMLAClient& mlaClient, CPub_equiv::Tdata& arr, int id)
         {
             // authors come back in a weird format that we need
             // to convert to ISO
-            CReferenceItem::ChangeMedlineAuthorsToISO(new_pub);
             if (new_pub->IsSetAuthors())
                CRemoteUpdater::ConvertToStandardAuthors((CAuth_list&)new_pub->GetAuthors());
 
-
             arr.clear();
             CRef<CPub> new_pmid(new CPub);
             new_pmid->SetPmid().Set(id);
@@ -142,17 +138,24 @@ public:
         }
     }
 
-    CRef<COrg_ref> GetOrg(const COrg_ref& org, objects::ILineErrorListener* logger)
+    CRef<COrg_ref> GetOrg(const COrg_ref& org, CRemoteUpdater::FLogger f_logger)
     {
         CRef<COrg_ref> result;
         CRef<CT3Reply> reply = GetOrgReply(org);
-        if (reply->IsError() && logger)
+        if (reply->IsError() && f_logger)
         {
+            const string& error_message = 
+                "Taxon update: " +
+                (org.IsSetTaxname() ? org.GetTaxname() : NStr::IntToString(org.GetTaxId())) + ": " +
+                reply->GetError().GetMessage();
+
+/*
             logger->PutError(*auto_ptr<CLineError>(
                 CLineError::Create(ILineError::eProblem_Unset, eDiag_Warning, "", 0,
                 string("Taxon update: ") + 
                 (org.IsSetTaxname() ? org.GetTaxname() : NStr::IntToString(org.GetTaxId())) + ": " +
                 reply->GetError().GetMessage())));
+            */
         }
         else
         if (reply->IsData() && reply->SetData().IsSetOrg())
@@ -222,7 +225,7 @@ protected:
     auto_ptr<CCachedReplyMap> m_cache;
 };
 
-void CRemoteUpdater::UpdateOrgFromTaxon(objects::ILineErrorListener* logger, objects::CSeqdesc& obj)
+void CRemoteUpdater::UpdateOrgFromTaxon(FLogger logger, objects::CSeqdesc& obj)
 {
     if (obj.IsOrg())
     {
@@ -235,7 +238,7 @@ void CRemoteUpdater::UpdateOrgFromTaxon(objects::ILineErrorListener* logger, obj
     }
 }
 
-void CRemoteUpdater::xUpdateOrgTaxname(objects::ILineErrorListener* logger, COrg_ref& org)
+void CRemoteUpdater::xUpdateOrgTaxname(FLogger logger, COrg_ref& org)
 {
     CMutexGuard guard(m_Mutex);
 
@@ -374,7 +377,7 @@ namespace
     typedef set<CRef< CSeqdesc >* > TOwnerSet;
     typedef struct { TOwnerSet owner; CRef<COrg_ref> org_ref; } TOwner;
     typedef map<string, TOwner > TOrgMap;
-    void _UpdateOrgFromTaxon(objects::ILineErrorListener* logger, objects::CSeq_entry& entry, TOrgMap& m)
+    void _UpdateOrgFromTaxon(CRemoteUpdater::FLogger logger, objects::CSeq_entry& entry, TOrgMap& m)
     {
         if (entry.IsSet())
         {
@@ -432,7 +435,7 @@ namespace
         }
     }
 }
-void CRemoteUpdater::UpdateOrgFromTaxon(objects::ILineErrorListener* logger, objects::CSeq_entry& entry)
+void CRemoteUpdater::UpdateOrgFromTaxon(FLogger logger, objects::CSeq_entry& entry)
 {   
     TOrgMap org_to_update;
 
@@ -449,20 +452,23 @@ void CRemoteUpdater::UpdateOrgFromTaxon(objects::ILineErrorListener* logger, obj
         reflist.push_back(it->second.org_ref);
         CRef<CTaxon3_reply> reply = taxon.SendOrgRefList(reflist);
 
-        CTaxon3_reply::TReply::iterator reply_it = reply->SetReply().begin();
+        if (reply.NotNull())
         {
-            if ((*reply_it)->IsData() && (*reply_it)->SetData().IsSetOrg())
+            CTaxon3_reply::TReply::iterator reply_it = reply->SetReply().begin();
             {
-                (*reply_it)->SetData().SetOrg().ResetSyn();
-                (*reply_it)->SetData().SetOrg().SetOrgname().SetFormalNameFlag(false);
+                if ((*reply_it)->IsData() && (*reply_it)->SetData().IsSetOrg())
+                {
+                    (*reply_it)->SetData().SetOrg().ResetSyn();
+                    (*reply_it)->SetData().SetOrg().SetOrgname().SetFormalNameFlag(false);
 
-                xUpdate(it->second.owner, (*reply_it)->SetData().SetOrg());
+                    xUpdate(it->second.owner, (*reply_it)->SetData().SetOrg());
+                }
             }
         }
     }
 }
 
-void CRemoteUpdater::UpdateOrgFromTaxon(objects::ILineErrorListener* logger, objects::CSeq_entry_EditHandle& obj)
+void CRemoteUpdater::UpdateOrgFromTaxon(FLogger logger, objects::CSeq_entry_EditHandle& obj)
 {
     for (CBioseq_CI bioseq_it(obj); bioseq_it; ++bioseq_it)
     {
@@ -547,7 +553,7 @@ void FixMedLineList(CAuth_list& auth_list)
     auth_list.SetNames().SetStd().insert(auth_list.SetNames().SetStd().begin(), standard_names.begin(), standard_names.end());
 }
 
-
+ 
 }
 
 void CRemoteUpdater::ConvertToStandardAuthors(CAuth_list& auth_list)
@@ -555,7 +561,27 @@ void CRemoteUpdater::ConvertToStandardAuthors(CAuth_list& auth_list)
     if (!auth_list.IsSetNames()) {
         return;
     }
-    
+/*
+    const bool fix_suffix = false;
+    if (auth_list.GetNames().IsMl()) {
+        list<CRef<CAuthor>> new_authors;
+        for (const string& ml_name : auth_list.GetNames().GetMl()) {
+            new_authors.push_back(CAuthor::ConvertMlToStandard(ml_name, fix_suffix));
+        }
+        auth_list.SetNames().Reset();
+        auth_list.SetNames().SetStd().insert(auth_list.SetNames().SetStd().begin(), new_authors.begin(), new_authors.end());
+        return;
+    }
+
+    if (auth_list.GetNames().IsStd()) {
+        for (CRef<CAuthor> author : auth_list.SetNames().SetStd()) {
+            if (author->GetName().IsMl()) {
+                CRef<CAuthor> new_author = CAuthor::ConvertMlToStandard(*author, fix_suffix);
+                author->Assign(*new_author);
+            }
+        }
+    }
+*/
     if (auth_list.GetNames().IsMl()) {
         FixMedLineList(auth_list);
         return;
@@ -569,6 +595,7 @@ void CRemoteUpdater::ConvertToStandardAuthors(CAuth_list& auth_list)
     }
 }
 
+
 void CRemoteUpdater::PostProcessPubs(CSeq_entry& obj)
 {
     if (obj.IsSet())
@@ -598,7 +625,6 @@ void CRemoteUpdater::PostProcessPubs(CPubdesc& pubdesc)
 
     NON_CONST_ITERATE(CPubdesc::TPub::Tdata, it, pubdesc.SetPub().Set())
     {
-        CReferenceItem::ChangeMedlineAuthorsToISO(*it);
         if ((**it).IsSetAuthors())
         {
             ConvertToStandardAuthors((CAuth_list&)(**it).GetAuthors());
