@@ -1,4 +1,4 @@
-/*  $Id: basecount_item.cpp 400413 2013-05-21 12:32:09Z kornbluh $
+/*  $Id: basecount_item.cpp 578290 2019-01-16 16:34:11Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -52,6 +52,10 @@ CBaseCountItem::CBaseCountItem(CBioseqContext& ctx) :
     x_GatherInfo(ctx);
 }
 
+IFlatItem::EItem CBaseCountItem::GetItemType(void) const
+{
+    return eItem_BaseCount;
+}
 
 void CBaseCountItem::Format
 (IFormatter& formatter,
