@@ -1,4 +1,4 @@
-/*  $Id: comment_item.cpp 559026 2018-03-06 01:23:45Z kans $
+/*  $Id: comment_item.cpp 578290 2019-01-16 16:34:11Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -89,6 +89,11 @@ CCommentItem::CCommentItem(CBioseqContext& ctx, bool need_period) :
     swap(m_First, sm_FirstComment);
 }
 
+IFlatItem::EItem CCommentItem::GetItemType(void) const
+{
+    return eItem_Comment;
+}
+
 
 CCommentItem::CCommentItem
 (const string& comment,
