@@ -1,4 +1,4 @@
-/*  $Id: dbsource_item.cpp 568762 2018-08-09 21:34:53Z kans $
+/*  $Id: dbsource_item.cpp 578290 2019-01-16 16:34:11Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -72,6 +72,10 @@ CDBSourceItem::CDBSourceItem(CBioseqContext& ctx) :
     x_GatherInfo(ctx);
 }
 
+IFlatItem::EItem CDBSourceItem::GetItemType(void) const
+{
+    return eItem_DbSource;
+}
 
 void CDBSourceItem::Format
 (IFormatter& formatter,
