@@ -1,4 +1,4 @@
-/*  $Id: defline_item.cpp 567707 2018-07-23 17:19:47Z kans $
+/*  $Id: defline_item.cpp 578290 2019-01-16 16:34:11Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -53,6 +53,10 @@ CDeflineItem::CDeflineItem(CBioseqContext& ctx) :
     x_GatherInfo(ctx);
 }
 
+IFlatItem::EItem CDeflineItem::GetItemType(void) const
+{
+    return eItem_Defline;
+}
 
 void CDeflineItem::Format
 (IFormatter& formatter,
