@@ -1,4 +1,4 @@
-/*  $Id: flat_file_config.cpp 574582 2018-11-15 15:10:09Z ivanov $
+/*  $Id: flat_file_config.cpp 578289 2019-01-16 16:33:46Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -561,9 +561,10 @@ void CFlatFileConfig::AddArgumentDescriptions(CArgDescriptions& args)
                                  "Custom flat file output bits.  The value is the bitwise OR (logical addition) of:\n"
                                  "         1 - hide protein_id and transcript_id\n"
                                  "         2 - hide GI number\n"
-                                 "         4 - allow long locus lines"
-                                 "      1024 - disable annot references"
-                                 "      2048 - use SeqEntryIndexer",
+                                 "         4 - allow long locus lines\n"
+                                 "      1024 - disable annot references\n"
+                                 "      2048 - use SeqEntryIndexer\n"
+                                 "     16384 - only Gene, RNA, and CDS features",
 
                                  CArgDescriptions::eInteger, "0");
 
