@@ -1,4 +1,4 @@
-/*  $Id: flat_file_generator.cpp 573607 2018-10-30 11:57:34Z ivanov $
+/*  $Id: flat_file_generator.cpp 578289 2019-01-16 16:33:46Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -298,6 +298,18 @@ void CFlatFileGenerator::Generate
                 if ( m_Ctx->GetConfig().ShowCDDFeatures() ) {
                     flags |= CSeqEntryIndex::fShowCDDFeats;
                 }
+                if ( m_Ctx->GetConfig().HideExonFeatures() ) {
+                    flags |= CSeqEntryIndex::fHideExonFeats;
+                }
+                if ( m_Ctx->GetConfig().HideIntronFeatures() ) {
+                    flags |= CSeqEntryIndex::fHideIntronFeats;
+                }
+                if ( m_Ctx->GetConfig().HideMiscFeatures() ) {
+                    flags |= CSeqEntryIndex::fHideMiscFeats;
+                }
+                if ( m_Ctx->GetConfig().GeneRNACDSFeatures() ) {
+                    flags |= CSeqEntryIndex::fGeneRNACDSOnly;
+                }
                 CRef<CSeqEntryIndex> idx(new CSeqEntryIndex( topseh, policy, flags ));
                 m_Ctx->SetSeqEntryIndex(idx);
                 if (idx->IsIndexFailure()) {
