@@ -1,4 +1,4 @@
-/*  $Id: flat_qual_slots.cpp 514265 2016-09-20 19:15:12Z kans $
+/*  $Id: flat_qual_slots.cpp 564513 2018-05-29 17:40:10Z kans $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -305,7 +305,8 @@ CTempString GetStringOfSourceQual(ESourceQualifier eSourceQualifier)
         TYPICAL_SQ(whole_replicon),
         { eSQ_zero_orgmod, "?" },
         { eSQ_one_orgmod, "?" },
-        { eSQ_zero_subsrc, "?" }
+        { eSQ_zero_subsrc, "?" },
+        TYPICAL_SQ(submitter_seqid)
 #undef TYPICAL_SQ
     };
     typedef const CStaticPairArrayMap<ESourceQualifier, const char*> TSourceQualToNameMap;
