@@ -1,4 +1,4 @@
-/*  $Id: gap_item.cpp 341078 2011-10-17 13:24:43Z kornbluh $
+/*  $Id: gap_item.cpp 578290 2019-01-16 16:34:11Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -55,6 +55,11 @@ CGapItem::CGapItem
 {
 }
 
+IFlatItem::EItem CGapItem::GetItemType(void) const
+{
+    return eItem_Gap;
+}
+
 void CGapItem::Format(IFormatter& formatter, IFlatTextOStream& text_os) const
 {
     formatter.FormatGap(*this, text_os);
