@@ -1,4 +1,4 @@
-/*  $Id: genbank_formatter.cpp 545053 2017-08-31 13:54:54Z ivanov $
+/*  $Id: genbank_formatter.cpp 569187 2018-08-16 21:49:33Z kans $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -288,6 +288,16 @@ static string s_get_anchor_html(const string & sAnchorName, TGi iGi )
     return (string)CNcbiOstrstreamToString(result);
 }
 
+static string s_get_anchor_html(const string & sAnchorName, CBioseqContext *ctx )
+{
+    CNcbiOstrstream result;
+
+    result << "<a name=\"" << sAnchorName << "_"
+        << ctx->GetAccession() << "\"></a>";
+
+    return (string)CNcbiOstrstreamToString(result);
+}
+
 void CGenbankFormatter::EndSection
 (const CEndSectionItem& end_item,
  IFlatTextOStream& orig_text_os)
@@ -310,7 +320,7 @@ void CGenbankFormatter::EndSection
 
     if( bHtml && cfg.IsModeEntrez() ) {
         text_os.AddLine( 
-            s_get_anchor_html("slash", end_item.GetContext()->GetGI()), 
+            s_get_anchor_html("slash", end_item.GetContext()),
             0, IFlatTextOStream::eAddNewline_No );
     }
 
@@ -357,7 +367,7 @@ void CGenbankFormatter::FormatLocus
 (const CLocusItem& locus, 
  IFlatTextOStream& orig_text_os)
 {
-    static const string strands[]  = { "   ", "ss-", "ds-", "ms-" };
+    static const char* strands[]  = { "   ", "ss-", "ds-", "ms-" };
 
     CRef<IFlatTextOStream> p_text_os;
     IFlatTextOStream& text_os = s_WrapOstreamIfCallbackExists(p_text_os, locus, orig_text_os);
@@ -367,7 +377,7 @@ void CGenbankFormatter::FormatLocus
     list<string> l;
     CNcbiOstrstream locus_line;
 
-    string units = "bp";
+    const char* units = "bp";
     if ( !ctx.IsProt() ) {
         if ( ( ctx.IsWGSMaster() && ! ctx.IsRSWGSNuc() ) || 
             ctx.IsTSAMaster() || ctx.IsTLSMaster() ) 
@@ -377,22 +387,17 @@ void CGenbankFormatter::FormatLocus
     } else {
         units = "aa";
     }
-    string topology = (locus.GetTopology() == CSeq_inst::eTopology_circular) ?
-                "circular" : "linear  ";
+    const char* topology = (locus.GetTopology() == CSeq_inst::eTopology_circular) ?  "circular" : "linear  ";
 
-    string mol = s_GenbankMol[locus.GetBiomol()];
+    const string& mol = s_GenbankMol[locus.GetBiomol()];
 
     const CFlatFileConfig& cfg = GetContext().GetConfig();
 
     locus_line.setf(IOS_BASE::left, IOS_BASE::adjustfield);
 
-    string locusname;
-    if (cfg.LongLocusNames()) {
-        locusname = locus.GetFullName();
-    } else {
-        locusname = locus.GetName();
-    }
-    int locuslength = locusname.length();
+    const string& locusname =  cfg.LongLocusNames() ? locus.GetFullName() : locus.GetName();    
+    size_t locuslength = locusname.length();
+
     locus_line << setw(16) << locusname;
     // long LOCUS names may impinge on the length (e.g. gi 1449456)
     // I would consider this behavior conceptually incorrect; we should either fix the data
@@ -470,10 +475,14 @@ void CGenbankFormatter::FormatAccession
     CRef<IFlatTextOStream> p_text_os;
     IFlatTextOStream& text_os = s_WrapOstreamIfCallbackExists(p_text_os, acc, orig_text_os);
 
-    string acc_line = x_FormatAccession(acc, ' ');
+    string acc_str = x_FormatAccession(acc, ' ');
+    string link_base = (acc.IsNuc() ? strLinkBaseNuc : strLinkBaseProt);
+    string acc_line;
     if( acc.GetContext()->Config().DoHTML() && ! acc.GetContext()->GetLocation().IsWhole() ) {
         acc_line = "<a href=\"";
-        acc_line += strLinkBaseEntrezViewer + acc_line + "\">" + acc_line + "</a>";
+        acc_line += link_base + acc_str + "\">" + acc_str + "</a>";
+    } else {
+        acc_line = acc_str;
     }
     if ( acc.IsSetRegion() ) {
         acc_line += " REGION: ";
@@ -601,7 +610,7 @@ void CGenbankFormatter::FormatHtmlAnchor(
     CRef<IFlatTextOStream> p_text_os;
     IFlatTextOStream& text_os = s_WrapOstreamIfCallbackExists(p_text_os, html_anchor, orig_text_os);
 
-    text_os.AddLine( s_get_anchor_html(html_anchor.GetLabelCore(), html_anchor.GetGI()),
+    text_os.AddLine( s_get_anchor_html(html_anchor.GetLabelCore(), html_anchor.GetContext()),
         0, IFlatTextOStream::eAddNewline_No );
 }
 
@@ -1173,17 +1182,17 @@ CGenbankFormatter::x_LocusHtmlPrefix( string &first_line, CBioseqContext& ctx )
     }}
 
     // list of links that let us jump to sections
-    const TGi gi = ctx.GetGI();
+    const string& accn = ctx.GetAccession();
     result << "<div class=\"localnav\"><ul class=\"locals\">";
     if( has_comment ) {
-        result << "<li><a href=\"#comment_" << gi << "\" title=\"Jump to the comment section of this record\">Comment</a></li>";
+        result << "<li><a href=\"#comment_" << accn << "\" title=\"Jump to the comment section of this record\">Comment</a></li>";
     }
-    result << "<li><a href=\"#feature_" << gi << "\" title=\"Jump to the feature table of this record\">Features</a></li>";
+    result << "<li><a href=\"#feature_" << accn << "\" title=\"Jump to the feature table of this record\">Features</a></li>";
     if( has_contig ) {
-        result << "<li><a href=\"#contig_" << gi << "\" title=\"Jump to the contig section of this record\">Contig</a></li>";
+        result << "<li><a href=\"#contig_" << accn << "\" title=\"Jump to the contig section of this record\">Contig</a></li>";
     }
     if( has_sequence ) {
-        result << "<li><a href=\"#sequence_" << gi << "\" title=\"Jump to the sequence of this record\">Sequence</a></li>";
+        result << "<li><a href=\"#sequence_" << accn << "\" title=\"Jump to the sequence of this record\">Sequence</a></li>";
     }
     result << "</ul>";
 
@@ -1227,7 +1236,7 @@ CGenbankFormatter::x_GetFeatureSpanAndScriptStart(
 
     // The span
     CNcbiOstrstream pre_feature_html;
-    pre_feature_html << "<span id=\"feature_" << ctx.GetGI()
+    pre_feature_html << "<span id=\"feature_" << ctx.GetAccession()
         << "_" << strKey << "_" << feat_type_count << "\" class=\"feature\">";
 
     // The javascript
@@ -1647,19 +1656,21 @@ void CGenbankFormatter::FormatFeature
         ++ m_uFeatureCount;
     }
 
-	const string& strKey = feat->GetKey();
-  string fkey = strKey;
-	if (NStr::EqualNocase (fkey, "propeptide")) {
-      if (f.GetContext()->IsProt()) {
-      } else if (f.GetContext()->IsRefSeq()) {
-      } else if (f.GetContext()->Config().IsModeEntrez() || f.GetContext()->Config().IsModeRelease()) {
-	      // fkey = "misc_feature";
-	  }
-	}
-	// write <span...> and <script...> in HTML mode
-	if (bHtml && f.GetContext()->Config().IsModeEntrez() && f.GetContext()->Config().ShowSeqSpans()) {
-		x_GetFeatureSpanAndScriptStart(*text_os, fkey, f.GetLoc(), *f.GetContext());
-	}
+    const string& strKey = feat->GetKey();
+    string fkey = strKey;
+    if (NStr::EqualNocase(fkey, "propeptide")) {
+        if (f.GetContext()->IsProt()) {
+        }
+        else if (f.GetContext()->IsRefSeq()) {
+        }
+        else if (f.GetContext()->Config().IsModeEntrez() || f.GetContext()->Config().IsModeRelease()) {
+            // fkey = "misc_feature";
+        }
+    }
+    // write <span...> and <script...> in HTML mode
+    if (bHtml && f.GetContext()->Config().IsModeEntrez() && f.GetContext()->Config().ShowSeqSpans()) {
+        x_GetFeatureSpanAndScriptStart(*text_os, fkey, f.GetLoc(), *f.GetContext());
+    }
 
 #if 1
 	list<string>        l;
@@ -1826,7 +1837,7 @@ s_FormatRegularSequencePiece
  TSeqPos &base_count )
 {
     const bool bHtml = seq.GetContext()->Config().DoHTML() && seq.GetContext()->Config().ShowSeqSpans();
-    const TGi gi = seq.GetContext()->GetGI();
+    const string& accn = seq.GetContext()->GetAccession();
 
     // format of sequence position
     const size_t kSeqPosWidth = 9;
@@ -1840,8 +1851,8 @@ s_FormatRegularSequencePiece
     const static string kCloseSpan = "</span>";
     TSeqPos length_of_span_before_base_count = 0;
     if( bHtml ) {
-        string kSpan = " <span class=\"ff_line\" id=\"gi_";
-        kSpan += NStr::NumericToString(gi);
+        string kSpan = " <span class=\"ff_line\" id=\"";
+        kSpan += accn;
         kSpan += '_';
         copy( kSpan.begin(), kSpan.end(), line + kSeqPosWidth );
         length_of_span_before_base_count = kSpan.length();
@@ -1893,7 +1904,11 @@ s_FormatRegularSequencePiece
             for ( ; i < s_kChunkCount; ++i) {
                 ++linep;
                 for ( ; j < s_kChunkSize; ++j, ++iter, ++linep) {
-                    *linep = *iter;
+                    unsigned char ch = *iter;
+                    if (ch > 126) {
+                        ch = '?';
+                    }
+                    *linep = ch;
                 }
                 *linep = ' ';
                 j = 0;
@@ -1906,7 +1921,11 @@ s_FormatRegularSequencePiece
             for ( ; total > 0  &&  i < s_kChunkCount; ++i) {
                 ++linep;
                 for ( ; total > 0  &&  j < s_kChunkSize; ++j, ++iter, --total, ++linep) {
-                    *linep = *iter;
+                    unsigned char ch = *iter;
+                    if (ch > 126) {
+                        ch = '?';
+                    }
+                    *linep = ch;
                 }
                 *linep = ' ';
                 j = 0;
@@ -1945,7 +1964,8 @@ void CGenbankFormatter::FormatSequence
 
     const bool bGapsHiddenUntilClicked = ( 
         GetContext().GetConfig().DoHTML() && 
-        GetContext().GetConfig().IsModeEntrez() );
+        GetContext().GetConfig().IsModeEntrez() &&
+        !GetContext().GetConfig().ExpandGaps());
 
     const CSeqVector& vec = seq.GetSequence();
     TSeqPos from = seq.GetFrom();
@@ -1991,17 +2011,10 @@ void CGenbankFormatter::FormatSequence
                 }
 
                 // build gap size text and "Expand Ns" link
-                const static int kExpandedGapDisplay = 65536;
                 CNcbiOstrstream gap_link;
-                const char *mol_type = ( seq.GetContext()->IsProt() ? "aa" : "bp" );
-                gap_link << "          [gap " << gap_size << " " << mol_type << "]";
-                const TGi gi = seq.GetContext()->GetGI();
-                if( gi > ZERO_GI ) {
-                    gap_link << "    <a href=\"" << strLinkBaseEntrezViewer 
-                        << gi << "?fmt_mask=" << kExpandedGapDisplay
-                        << "\">Expand Ns</a>";
-                }
-
+                GetContext().GetConfig().GetHTMLFormatter().FormatGapLink(gap_link, gap_size, 
+                                                                          seq.GetContext()->GetAccession(), 
+                                                                          seq.GetContext()->IsProt());
                 text_os.AddLine( (string)CNcbiOstrstreamToString(gap_link) );
             } else {
                 // create a fake total so we stop before the next gap
@@ -2079,7 +2092,7 @@ void CGenbankFormatter::FormatWGS
 
     switch ( wgs.GetType() ) {
     case CWGSItem::eWGS_Projects:
-        tag += "WGS";
+        tag = "WGS";
         break;
 
     case CWGSItem::eWGS_ScaffoldList:
