@@ -1,4 +1,4 @@
-/*  $Id: genome_project_item.cpp 556353 2018-01-30 12:00:12Z bollin $
+/*  $Id: genome_project_item.cpp 578290 2019-01-16 16:34:11Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -54,6 +54,10 @@ CGenomeProjectItem::CGenomeProjectItem(CBioseqContext& ctx) :
     x_GatherInfo(ctx);
 }
 
+IFlatItem::EItem CGenomeProjectItem::GetItemType(void) const
+{
+    return eItem_GenomeProject;
+}
 
 void CGenomeProjectItem::Format
 (IFormatter& formatter,
