@@ -1,4 +1,4 @@
-/*  $Id: keywords_item.cpp 560480 2018-03-22 21:45:28Z kans $
+/*  $Id: keywords_item.cpp 580654 2019-02-19 12:40:00Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -64,6 +64,10 @@ CKeywordsItem::CKeywordsItem(CBioseqContext& ctx) :
     x_GatherInfo(ctx);
 }
 
+IFlatItem::EItem CKeywordsItem::GetItemType(void) const
+{
+    return eItem_Keywords;
+}
 
 void CKeywordsItem::Format
 (IFormatter& formatter,
@@ -278,6 +282,26 @@ void CKeywordsItem::x_GatherInfo(CBioseqContext& ctx)
     for (CSeqdesc_CI di(bsh, CSeqdesc::e_User); di; ++di) {
         const CUser_object& usr = di->GetUser();
         if ( ! CComment_rule::IsStructuredComment (usr) ) continue;
+        string pfx = CComment_rule::GetStructuredCommentPrefix ( usr, true );
+        if ( NStr::EqualNocase (pfx, "MIGS:5.0-Data" )) {
+            x_AddKeyword("GSC:MIxS");
+            x_AddKeyword("MIGS:5.0.");
+        } else if ( NStr::EqualNocase (pfx, "MIMS:5.0-Data" )) {
+            x_AddKeyword("GSC:MIxS");
+            x_AddKeyword("MIMS:5.0.");
+        } else if ( NStr::EqualNocase (pfx, "MIMARKS:5.0-Data" )) {
+            x_AddKeyword("GSC:MIxS");
+            x_AddKeyword("MIMARKS:5.0.");
+        } else if ( NStr::EqualNocase (pfx, "MISAG:5.0-Data" )) {
+            x_AddKeyword("GSC:MIxS");
+            x_AddKeyword("MISAG:5.0.");
+        } else if ( NStr::EqualNocase (pfx, "MIMAG:5.0-Data" )) {
+            x_AddKeyword("GSC:MIxS");
+            x_AddKeyword("MIMAG:5.0.");
+        } else if ( NStr::EqualNocase (pfx, "MIUVIG:5.0-Data" )) {
+            x_AddKeyword("GSC:MIxS");
+            x_AddKeyword("MIUVIG:5.0.");
+        }
         try {
             list<string> keywords = CComment_set::GetKeywords(usr);
             FOR_EACH_STRING_IN_LIST ( s_itr, keywords ) {
@@ -292,12 +316,15 @@ void CKeywordsItem::x_GatherInfo(CBioseqContext& ctx)
         x_AddKeyword("UNVERIFIED");
     }
     if ((unv & CBioseqContext::fUnverified_Organism) != 0) {
+        x_AddKeyword("UNVERIFIED");
         x_AddKeyword("UNVERIFIED_ORGANISM");
     }
     if ((unv & CBioseqContext::fUnverified_Misassembled) != 0) {
+        x_AddKeyword("UNVERIFIED");
         x_AddKeyword("UNVERIFIED_MISASSEMBLY");
     }
     if ((unv & CBioseqContext::fUnverified_Contaminant) != 0) {
+        x_AddKeyword("UNVERIFIED");
         x_AddKeyword("UNVERIFIED_CONTAMINANT");
     }
 
