@@ -1,4 +1,4 @@
-/*  $Id: locus_item.cpp 574584 2018-11-15 15:11:08Z ivanov $
+/*  $Id: locus_item.cpp 578290 2019-01-16 16:34:11Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -78,6 +78,10 @@ CLocusItem::CLocusItem(CBioseqContext& ctx) :
    x_GatherInfo(ctx);
 }
 
+IFlatItem::EItem CLocusItem::GetItemType(void) const
+{
+    return eItem_Locus;
+}
 
 void CLocusItem::Format
 (IFormatter& formatter,
