@@ -1,4 +1,4 @@
-/*  $Id: origin_item.cpp 213605 2010-11-24 15:12:46Z kornbluh $
+/*  $Id: origin_item.cpp 578290 2019-01-16 16:34:11Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -53,6 +53,11 @@ COriginItem::COriginItem(CBioseqContext& ctx) :
 }
 
 
+IFlatItem::EItem COriginItem::GetItemType(void) const
+{
+    return eItem_Origin;
+}
+
 void COriginItem::Format
 (IFormatter& formatter,
  IFlatTextOStream& text_os) const
