@@ -1,4 +1,4 @@
-/*  $Id: qualifiers.cpp 564513 2018-05-29 17:40:10Z kans $
+/*  $Id: qualifiers.cpp 578996 2019-01-29 13:12:39Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -865,43 +865,54 @@ void CFlatMolTypeQVal::Format(TFlatQuals& q, const CTempString& name,
 void CFlatSubmitterSeqidQVal::Format(TFlatQuals& q, const CTempString& name,
                             CBioseqContext& ctx, IFlatQVal::TFlags flags) const
 {
-    if ( ctx.Config().IsModeGBench()  ||  ctx.Config().IsModeDump() ) {
-        switch ( m_Tech ) {
-        case CMolInfo::eTech_wgs:
-        case CMolInfo::eTech_tsa:
-        case CMolInfo::eTech_targeted:
-            ITERATE (CBioseq::TId, itr, ctx.GetBioseqIds()) {
-                const CSeq_id& id = **itr;
-                if ( id.Which() != CSeq_id::e_General ) continue;
-                const CDbtag& dbtag = id.GetGeneral();
-                if ( ! dbtag.IsSetDb() ) continue;
-                string dbname = dbtag.GetDb();
-                if ( dbname.length() != 10 ) continue;
-                if ( ! NStr::StartsWith(dbname, "WGS:" ) && ! NStr::StartsWith(dbname, "TSA:" ) && ! NStr::StartsWith(dbname, "TLS:" ) ) continue;
-                bool bail = false;
-                for ( int i = 4; i < 8; i++ ) {
-                    char ch = dbname[i];
-                    if ( ! isupper(ch) && ! islower(ch) ) {
+    switch ( m_Tech ) {
+    case CMolInfo::eTech_wgs:
+    case CMolInfo::eTech_tsa:
+    case CMolInfo::eTech_targeted:
+        ITERATE (CBioseq::TId, itr, ctx.GetBioseqIds()) {
+            const CSeq_id& id = **itr;
+            if ( id.Which() != CSeq_id::e_General ) continue;
+            const CDbtag& dbtag = id.GetGeneral();
+            if ( ! dbtag.IsSetDb() ) continue;
+            string dbname = dbtag.GetDb();
+            if ( ! NStr::StartsWith(dbname, "WGS:" ) && ! NStr::StartsWith(dbname, "TSA:" ) && ! NStr::StartsWith(dbname, "TLS:" ) ) continue;
+            dbname.erase(0, 4);
+            if (NStr::StartsWith(dbname, "NZ_" )) {
+                dbname.erase(0, 3);
+            }
+            int num_letters = 0;
+            int num_digits = 0;
+            int len = dbname.length();
+            if ( len != 6 && len != 8 ) continue;
+            bool bail = false;
+            for ( int i = 0; i < len; i++ ) {
+                char ch = dbname[i];
+                if ( isupper(ch) || islower(ch) ) {
+                    num_letters++;
+                    if ( num_digits > 0 ) {
                         bail = true;
                     }
-                }
-                if ( bail ) continue;
-                for ( int i = 8; i < 10; i++ ) {
-                    char ch = dbname[i];
-                    if ( ! isdigit(ch) ) {
-                        bail = true;
-                    }
-                }
-                if ( bail ) continue;
-                if ( dbtag.IsSetTag() && dbtag.GetTag().IsStr() ) {
-                    string tag = dbtag.GetTag().GetStr();
-                    x_AddFQ(q, name, tag);
+                } else if ( isdigit(ch) ) {
+                    num_digits++;
+                } else {
+                    bail = true;
                 }
             }
-            break;
-        default:
-            break;
+            if ( num_letters != 4 && num_letters != 6 ) {
+                bail = true;
+            }
+            if ( num_digits != 2 ) {
+                bail = true;
+            }
+            if ( bail ) continue;
+            if ( dbtag.IsSetTag() && dbtag.GetTag().IsStr() ) {
+                string tag = dbtag.GetTag().GetStr();
+                x_AddFQ(q, name, tag);
+            }
         }
+        break;
+    default:
+        break;
     }
 }
 
