@@ -1,4 +1,4 @@
-/*  $Id: reference_item.cpp 557236 2018-02-12 22:36:27Z kans $
+/*  $Id: reference_item.cpp 578290 2019-01-16 16:34:11Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -239,6 +239,10 @@ CReferenceItem::CReferenceItem(const CSubmit_block& sub, CBioseqContext& ctx) :
 CReferenceItem::~CReferenceItem() {
 }
 
+IFlatItem::EItem CReferenceItem::GetItemType(void) const
+{
+    return eItem_Reference;
+}
 
 void CReferenceItem::SetLoc(const CConstRef<CSeq_loc>& loc)
 {
