@@ -1,4 +1,4 @@
-/*  $Id: source_item.cpp 563713 2018-05-14 16:53:23Z kans $
+/*  $Id: source_item.cpp 578290 2019-01-16 16:34:11Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -70,6 +70,10 @@ CSourceItem::CSourceItem(CBioseqContext& ctx) :
     x_GatherInfo(ctx);
 }
 
+IFlatItem::EItem CSourceItem::GetItemType(void) const
+{
+    return eItem_Source;
+}
 
 CSourceItem::CSourceItem(CBioseqContext& ctx, const CBioSource& bsrc, const CSerialObject& obj) :
     CFlatItem(&ctx),
