@@ -1,4 +1,4 @@
-/*  $Id: agp_read.cpp 543178 2017-08-08 11:22:20Z ivanov $
+/*  $Id: agp_read.cpp 548810 2017-10-18 13:38:41Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -120,7 +120,7 @@ void AgpRead(CNcbiIstream& is,
 
         // split into fields, as delimited by tabs
         fields.clear();
-        NStr::Split(line, "\t", fields, NStr::fSplit_NoMergeDelims);
+        NStr::Split(line, "\t", fields);
 
         // eliminate any empty fields at the end of the line
         int index;
