@@ -1,4 +1,4 @@
-/*  $Id: aln_reader.cpp 540293 2017-07-05 18:11:48Z foleyjp $
+/*  $Id: aln_reader.cpp 575409 2018-11-28 17:53:34Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -104,8 +104,7 @@ static char * ALIGNMENT_CALLBACK s_ReadLine(void *user_data)
         return 0;
     }
     string s;
-    NcbiGetline(*is, s, "\n");
-    NStr::ReplaceInPlace (s, "\r", "");
+    NcbiGetline(*is, s, "\r\n");
     return strdup(s.c_str());
 }
 
