@@ -1,4 +1,4 @@
-/*  $Id: bed_reader.cpp 572342 2018-10-11 17:14:40Z ivanov $
+/*  $Id: bed_reader.cpp 575513 2018-11-29 19:39:23Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -116,10 +116,10 @@ public:
     bool FillBuffer(
         size_t numLines)
     {
-        CTempString line;
+        string line;
         while (numLines  &&  !mLineReader.AtEOF()) {
             line = *++mLineReader;
-            NStr::TruncateSpacesInPlace(line);
+            CLinePreBuffer::StripSpaceCharsInPlace(line);
             mBuffer.push_back(line);
             if (!IsCommentLine(line)) {
                 --numLines;
@@ -151,7 +151,7 @@ public:
             }
             else {
                 temp = *++mLineReader;
-                NStr::TruncateSpacesInPlace(temp);
+                CLinePreBuffer::StripSpaceCharsInPlace(temp);
             }
             if (!IsCommentLine(temp)) {
                 line = temp;
@@ -195,6 +195,21 @@ public:
         }
     };
 
+    static void
+    StripSpaceCharsInPlace(
+        string& str)
+    {
+        auto newFirst = 0;
+        while (str[newFirst] == ' ') {
+            ++newFirst;
+        }
+        auto newLast = str.length() - 1;
+        while (str[newLast] == ' ') {
+            --newLast;
+        }
+        str = str.substr(newFirst, newLast - newFirst + 1);
+    };
+
 protected:
     ILineReader& mLineReader;
     deque<string> mBuffer;
@@ -345,19 +360,33 @@ bool CBedReader::xSplitColumns(
     vector<string>& columns)
 //  ----------------------------------------------------------------------------
 {
+    bool splitSuccessful = false;
     if (mColumnSeparator.empty()) {
         columns.clear();
         mColumnSeparator = "\t";
         NStr::Split(line, mColumnSeparator, columns, mColumnSplitFlags);
         if (columns.size() > 2) {
-            return true;
+            splitSuccessful = true;
+        }
+        else {
+            mColumnSeparator = " \t";
+            mColumnSplitFlags = NStr::fSplit_MergeDelimiters;
         }
-        mColumnSeparator = " \t";
-        mColumnSplitFlags = NStr::fSplit_MergeDelimiters;
     }
-    columns.clear();
-    NStr::Split(line, mColumnSeparator, columns, mColumnSplitFlags);
-    return (columns.size() > 2);
+    if (!splitSuccessful) {
+        columns.clear();
+        NStr::Split(line, mColumnSeparator, columns, mColumnSplitFlags);
+        if (columns.size() > 2) {
+            splitSuccessful = true;
+        }
+    }
+    if (!splitSuccessful) {
+        return false;
+    }
+    for (auto& column: columns) {
+        NStr::TruncateSpacesInPlace(column);
+    }
+    return true;
 }
 
 //  ----------------------------------------------------------------------------
@@ -397,7 +426,6 @@ bool CBedReader::xDetermineLikelyColumnCount(
             continue;
         }
 
-
         vector<string> columns;
         if (!xSplitColumns(line, columns)) {
             pErrColumnCount->Throw();
@@ -595,8 +623,6 @@ CBedReader::xParseFeature(
     ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
-	CTempString record_copy = NStr::TruncateSpaces_Unsafe(line);
-
     //  parse
     vector<string> fields;
     xSplitColumns(line, fields);
