@@ -1,4 +1,4 @@
-/*  $Id: fasta.cpp 541541 2017-07-19 13:48:44Z ivanov $
+/*  $Id: fasta.cpp 568147 2018-07-31 18:08:59Z gotvyans $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -217,41 +217,29 @@ inline static bool s_ASCII_IsUnAmbigNuc(unsigned char c)
     }
 }
 
-CFastaReader::CFastaReader(ILineReader& reader, TFlags flags)
+CFastaReader::CFastaReader(ILineReader& reader, TFlags flags, FIdCheck f_idcheck)
     : m_LineReader(&reader), m_MaskVec(0), 
-      m_IDGenerator(new CSeqIdGenerator), 
+      m_IDGenerator(new CSeqIdGenerator()), 
       m_gapNmin(0), m_gap_Unknown_length(0),
-      m_MaxIDLength(kMax_UI4)
+      m_MaxIDLength(kMax_UI4),
+      m_fIdCheck(f_idcheck)
 {
     m_Flags.push(flags);
     m_IDHandler = Ref(new CFastaIdHandler());
 }
 
-CFastaReader::CFastaReader(CNcbiIstream& in, TFlags flags)
-    : m_LineReader(ILineReader::New(in)), m_MaskVec(0),
-      m_IDGenerator(new CSeqIdGenerator), 
-      m_gapNmin(0), m_gap_Unknown_length(0),
-      m_MaxIDLength(kMax_UI4)
-{
-    m_Flags.push(flags);
-    m_IDHandler = Ref(new CFastaIdHandler());
-}
+CFastaReader::CFastaReader(CNcbiIstream& in, TFlags flags, FIdCheck f_idcheck)
+    : CFastaReader(*(ILineReader::New(in)), flags, f_idcheck) {}
 
-CFastaReader::CFastaReader(const string& path, TFlags flags)
-    : m_LineReader(ILineReader::New(path)), m_MaskVec(0),
-      m_IDGenerator(new CSeqIdGenerator), 
-      m_gapNmin(0), m_gap_Unknown_length(0),
-      m_MaxIDLength(kMax_UI4)
-{
-    m_Flags.push(flags);
-    m_IDHandler = Ref(new CFastaIdHandler());
-}
+CFastaReader::CFastaReader(const string& path, TFlags flags, FIdCheck f_idcheck)
+    : CFastaReader(*(ILineReader::New(path)), flags, f_idcheck) {}
 
-CFastaReader::CFastaReader(CReaderBase::TReaderFlags fBaseFlags, TFlags flags)
+CFastaReader::CFastaReader(CReaderBase::TReaderFlags fBaseFlags, TFlags flags, FIdCheck f_idcheck)
     : CReaderBase(fBaseFlags), m_MaskVec(0), 
       m_IDGenerator(new CSeqIdGenerator), 
       m_gapNmin(0), m_gap_Unknown_length(0),
-      m_MaxIDLength(kMax_UI4)
+      m_MaxIDLength(kMax_UI4),
+      m_fIdCheck(f_idcheck)
 {
     m_Flags.push(flags);
     m_IDHandler = Ref(new CFastaIdHandler());
@@ -324,6 +312,13 @@ CRef<CSeq_entry> CFastaReader::ReadOneSeq(ILineErrorListener * pMessageListener)
         }
         if (c == '>' ) {
             CTempString next_line = *++GetLineReader();
+            string strmodified;
+            if( NStr::StartsWith(next_line, ">?_") ) {
+                CTempString tmp = next_line.substr(3);
+                strmodified = ">";
+                strmodified.append(tmp.data(), tmp.length());
+                next_line = strmodified;
+            }
             if( NStr::StartsWith(next_line, ">?") ) {
                 // This is actually a data line. an assembly gap, in particular, which
                 // we handle farther below
@@ -339,18 +334,10 @@ CRef<CSeq_entry> CFastaReader::ReadOneSeq(ILineErrorListener * pMessageListener)
                     break;
                 }
             }
-        } else if (c == '[') {
-            return x_ReadSegSet(pMessageListener);
-        } else if (c == ']') {
-            if (need_defline) {
-                FASTA_ERROR(LineNumber(), 
-                    "CFastaReader: Reached unexpected end of segmented set around line " << LineNumber(),
-                    CObjReaderParseException::eEOF );
-            }
-            break;
         }
 
         CTempString line = NStr::TruncateSpaces_Unsafe(*++GetLineReader());
+
         if (line.empty()) {
             continue; // ignore lines containing only whitespace
         }
@@ -374,6 +361,13 @@ CRef<CSeq_entry> CFastaReader::ReadOneSeq(ILineErrorListener * pMessageListener)
 
         if ( !TestFlag(fNoSeqData) ) {
             try {
+				string strmodified;
+				if( NStr::StartsWith(line, ">?_") ) {
+					CTempString tmp = line.substr(3);
+					strmodified = ">";
+					strmodified.append(tmp.data(), tmp.length());
+					line = strmodified;
+				}
                 ParseDataLine(line, pMessageListener);
             } catch(CBadResiduesException & e) {
                 // we have to catch this exception so we can build up
@@ -486,11 +480,11 @@ CRef<CSeq_entry> CFastaReader::ReadSet(int max_seqs, ILineErrorListener * pMessa
         }
     }
 
-    entry->Parentize();
 
     if (entry->IsSet()  &&  entry->GetSet().GetSeq_set().size() == 1) {
         return entry->SetSet().SetSeq_set().front();
     } else {
+        entry->Parentize();
         return entry;
     }
 }
@@ -506,6 +500,13 @@ void CFastaReader::SetIDGenerator(CSeqIdGenerator& gen)
     m_IDHandler->SetGenerator(gen);
 }
 
+void CFastaReader::SetMaxIDLength(Uint4 max_len)
+{
+    CFastaDeflineReader::s_MaxLocalIDLength =
+    CFastaDeflineReader::s_MaxGeneralTagLength =
+    CFastaDeflineReader::s_MaxAccessionLength = m_MaxIDLength = max_len;
+}
+
 // For reasons of efficiency, this method does not use 
 // CRef<CSeq_interval> to access range information - RW-26
 void CFastaReader::ParseDefLine(const TStr& defLine,
@@ -518,16 +519,18 @@ void CFastaReader::ParseDefLine(const TStr& defLine,
                                 TSeqTitles& seqTitles, 
                                 ILineErrorListener* pMessageListener) 
 {
+    CFastaDeflineReader::SDeflineData data;
     CFastaDeflineReader::ParseDefline(
-        defLine, 
+        defLine,
         info,
-        ignoredErrors,
-        ids, 
-        hasRange,
-        rangeStart,
-        rangeEnd,
-        seqTitles,
+        data,
         pMessageListener);
+
+    ids = move(data.ids);
+    hasRange   = data.has_range;
+    rangeStart = data.range_start;
+    rangeEnd   = data.range_end;
+    seqTitles  = move(data.titles);
 }
 
 
@@ -578,28 +581,18 @@ bool CFastaReader::xSetSeqMol(const list<CRef<CSeq_id>>& ids, CSeq_inst_Base::EM
 
 void CFastaReader::ParseDefLine(const TStr& s, ILineErrorListener * pMessageListener)
 {
-    TSeqPos range_start, range_end;
-    bool has_range;
     SDefLineParseInfo parseInfo;
     parseInfo.fBaseFlags = m_iFlags;
     parseInfo.fFastaFlags = GetFlags();
     parseInfo.maxIdLength = m_MaxIDLength;
     parseInfo.lineNumber = LineNumber();
 
-    CBioseq::TId defline_ids;
+    CFastaDeflineReader::SDeflineData data;
+    CFastaDeflineReader::ParseDefline(s, parseInfo, data, pMessageListener, m_fIdCheck);
 
-    CFastaDeflineReader::ParseDefline(s,
-                 parseInfo,
-                 m_ignorable,
-                 defline_ids, 
-                 has_range,
-                 range_start,
-                 range_end,
-                 m_CurrentSeqTitles, 
-                 pMessageListener);
+    m_CurrentSeqTitles = move(data.titles);
 
-
-    if (defline_ids.empty()) {
+    if (data.ids.empty()) {
         if (TestFlag(fRequireID)) {
             // No [usable] IDs
             FASTA_ERROR(LineNumber(),
@@ -609,16 +602,16 @@ void CFastaReader::ParseDefLine(const TStr& s, ILineErrorListener * pMessageList
     }
     else if (!TestFlag(fForceType)) {
         CSeq_inst::EMol mol;
-        if (xSetSeqMol(defline_ids, mol)) { 
+        if (xSetSeqMol(data.ids, mol)) { 
             m_CurrentSeq->SetInst().SetMol(mol);
         }
     }
 
-    PostProcessIDs(defline_ids, s, has_range, range_start, range_end);
+    PostProcessIDs(data.ids, s, data.has_range, data.range_start, data.range_end);
 
     m_BestID = FindBestChoice(GetIDs(), CSeq_id::BestRank);
-    if(has_range) {
-        m_ExpectedEnd = range_end - range_start;
+    if(data.has_range) {
+        m_ExpectedEnd = data.range_end - data.range_start;
     }
     FASTA_PROGRESS("Processing Seq-id: " <<
         ( m_BestID ? m_BestID->AsFastaString() : string("UNKNOWN") ) );
@@ -738,7 +731,6 @@ bool CFastaReader::IsValidLocalID(const TStr& s) const
     return IsValidLocalID(s, GetFlags());
 }
 
-
 bool CFastaReader::IsValidLocalID(const TStr& idString, 
     const TFlags fFastaFlags)
 {
@@ -1915,7 +1907,10 @@ class CFastaMapper : public CFastaReader
 public:
     typedef CFastaReader TParent;
 
-    CFastaMapper(ILineReader& reader, SFastaFileMap* fasta_map, TFlags flags);
+    CFastaMapper(ILineReader& reader, 
+                 SFastaFileMap* fasta_map, 
+                 TFlags flags, 
+                 FIdCheck f_idcheck = CSeqIdCheck());
 
 protected:
     void ParseDefLine(const TStr& s, 
@@ -1929,9 +1924,11 @@ private:
     SFastaFileMap::SFastaEntry m_MapEntry;
 };
 
-CFastaMapper::CFastaMapper(ILineReader& reader, SFastaFileMap* fasta_map,
-                           TFlags flags)
-    : TParent(reader, flags), m_Map(fasta_map)
+CFastaMapper::CFastaMapper(ILineReader& reader, 
+                           SFastaFileMap* fasta_map,
+                           TFlags flags,
+                           FIdCheck f_idcheck)
+    : TParent(reader, flags, f_idcheck), m_Map(fasta_map)
 {
     _ASSERT(fasta_map);
     fasta_map->file_map.resize(0);
@@ -2004,6 +2001,21 @@ void ScanFastaFile(IFastaEntryScan* scanner,
     }
 }
 
+CSourceModParser* CFastaReader::xCreateSourceModeParser(
+    ILineErrorListener* pErrorListener)
+{
+    // first of all, honor any explicit fBadModThrow flag given to the reader:
+    if (TestFlag(fBadModThrow) || TestFlag(fUnknModThrow)) {
+        return new CSourceModParser(CSourceModParser::eHandleBadMod_Throw);
+    }
+    // otherwise, try to construct the parser around any given error listener:
+    if (pErrorListener != nullptr) {
+        return new CSourceModParser(pErrorListener, m_uLineNumber+1);
+    }
+    // ignore errors as a last resort:
+    return new CSourceModParser(CSourceModParser::eHandleBadMod_Ignore);
+}
+
 void CFastaReader::x_ApplyAllMods( 
     CBioseq & bioseq,
     TSeqPos iLineNum,
@@ -2013,13 +2025,10 @@ void CFastaReader::x_ApplyAllMods(
     // mods to be added because we want to give a warning if there
     // are mods when not expected.
 
-    CSourceModParser smp( TestFlag(fBadModThrow) ?
-        CSourceModParser::eHandleBadMod_Throw : 
-    CSourceModParser::eHandleBadMod_Ignore );
-    smp.SetModFilter( m_pModFilter );
+    auto_ptr<CSourceModParser> pSmp(xCreateSourceModeParser(pMessageListener));
+    pSmp->SetModFilter( m_pModFilter );
     CRef<CSeqdesc> title_desc;
 
-
     if( ! bioseq.IsSetDescr() && ! bioseq.GetDescr().IsSet() ) {
         return;
     }
@@ -2043,55 +2052,27 @@ void CFastaReader::x_ApplyAllMods(
     string& title = title_desc->SetTitle();
 
     if( TestFlag(fAddMods) ) {
-        title = smp.ParseTitle(title, CConstRef<CSeq_id>(bioseq.GetFirstId()) );
+        title = pSmp->ParseTitle(title, CConstRef<CSeq_id>(bioseq.GetFirstId()) );
 
-        smp.ApplyAllMods(bioseq);
-        if( TestFlag(fUnknModThrow) ) {
-            CSourceModParser::TMods unused_mods = smp.GetMods(CSourceModParser::fUnusedMods);
-            if( ! unused_mods.empty() ) 
-            {
-                // there are unused mods and user specified to throw if any
-                // unused 
-                CNcbiOstrstream err;
-                err << "CFastaReader: Inapplicable or unrecognized modifiers on ";
+        pSmp->ApplyAllMods(bioseq);
+        ApplySourceMods(bioseq, *pSmp);
+        pSmp->GetLabel(&title, CSourceModParser::fUnusedMods);
 
-                // get sequence ID
-                const CSeq_id* seq_id = bioseq.GetFirstId();
-                if( seq_id ) {
-                    err << seq_id->GetSeqIdString();
-                } else {
-                    // seq-id unknown
-                    err << "sequence";
-                }
-
-                err << ":";
-                ITERATE(CSourceModParser::TMods, mod_iter, unused_mods) {
-                    err << " [" << mod_iter->key << "=" << mod_iter->value << ']';
-                }
-                err << " around line " + NStr::NumericToString(iLineNum);
-                NCBI_THROW2(CObjReaderParseException, eUnusedMods,
-                    (string)CNcbiOstrstreamToString(err),
-                    iLineNum);
-            }
-        }
-
-        smp.GetLabel(&title, CSourceModParser::fUnusedMods);
-
-        copy( smp.GetBadMods().begin(), smp.GetBadMods().end(),
+        copy( pSmp->GetBadMods().begin(), pSmp->GetBadMods().end(),
             inserter(m_BadMods, m_BadMods.begin()) );
         CSourceModParser::TMods unused_mods = 
-            smp.GetMods(CSourceModParser::fUnusedMods);
+            pSmp->GetMods(CSourceModParser::fUnusedMods);
         copy( unused_mods.begin(), unused_mods.end(),
             inserter(m_UnusedMods, m_UnusedMods.begin() ) );
     } else {
         // user did not request fAddMods, so we warn if we found
         // mods anyway
-        smp.ParseTitle(
+        pSmp->ParseTitle(
             title, 
             CConstRef<CSeq_id>(bioseq.GetFirstId()),
             1 // "1" since we only care whether or not there are mods, not how many
             );
-        CSourceModParser::TMods unused_mods = smp.GetMods(CSourceModParser::fUnusedMods);
+        CSourceModParser::TMods unused_mods = pSmp->GetMods(CSourceModParser::fUnusedMods);
         if( ! unused_mods.empty() ) {
             FASTA_WARNING(iLineNum,
                 "FASTA-Reader: Ignoring FASTA modifier(s) found because "
@@ -2202,6 +2183,11 @@ void CFastaReader::IgnoreProblem(ILineError::EProblem problem)
     m_ignorable.push_back(problem);
 }
 
+void CFastaReader::ApplySourceMods(CBioseq&, CSourceModParser&)
+{
+}
+
+
 END_SCOPE(objects)
 END_NCBI_SCOPE
 
