@@ -1,4 +1,4 @@
-/*  $Id: gff2_data.cpp 561166 2018-04-03 13:59:57Z ludwigf $
+/*  $Id: gff2_data.cpp 575507 2018-11-29 19:36:24Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -343,7 +343,7 @@ bool CGff2Record::AssignFromGff(
 
     columns[8].Copy(m_strAttributes, 0, CTempString::npos);
     
-    return x_AssignAttributesFromGff(m_strType, columns[8]);
+    return xAssignAttributesFromGff(m_strType, columns[8]);
 }
 
 //  ----------------------------------------------------------------------------
@@ -522,7 +522,7 @@ bool x_GetNextAttribute(CTempString& input, CTempString& key, CTempString& value
     return !key.empty();
 }
 
-bool CGff2Record::x_AssignAttributesFromGff(
+bool CGff2Record::xAssignAttributesFromGff(
     const string& strType,
     const string& strRawAttributes )
 //  ----------------------------------------------------------------------------
@@ -541,7 +541,7 @@ bool CGff2Record::x_AssignAttributesFromGff(
 }
 
 //  ----------------------------------------------------------------------------
-bool CGff2Record::x_SplitGffAttributes(
+bool CGff2Record::xSplitGffAttributes(
     const string& strRawAttributes,
 	vector< string >& attributes) const
 //  ----------------------------------------------------------------------------
@@ -585,14 +585,14 @@ bool CGff2Record::InitializeFeature(
 //  ----------------------------------------------------------------------------
 {
     return (
-        x_InitFeatureLocation(flags, pFeature, seqidresolve)  &&
+        xInitFeatureLocation(flags, pFeature, seqidresolve)  &&
         xInitFeatureData(flags, pFeature)  &&
-        x_MigrateId(pFeature)  &&
-        x_MigrateStartStopStrand(pFeature)  &&
-        x_MigrateType(pFeature)  &&
-        x_MigrateScore(pFeature)  &&
-        x_MigratePhase(pFeature)  &&
-        x_MigrateAttributes(flags, pFeature) );
+        xMigrateId(pFeature)  &&
+        xMigrateStartStopStrand(pFeature)  &&
+        xMigrateType(pFeature)  &&
+        xMigrateScore(pFeature)  &&
+        xMigratePhase(pFeature)  &&
+        xMigrateAttributes(flags, pFeature) );
 }
 
 //  ----------------------------------------------------------------------------
@@ -600,22 +600,17 @@ bool CGff2Record::UpdateFeature(
     int flags,
     CRef<CSeq_feat> pFeature,
     SeqIdResolver seqidresolve ) const
-//  ----------------------------------------------------------------------------
+    //  ----------------------------------------------------------------------------
 {
-    // mss-582:
-    //  if the parent feature is a gene then don't mess with the gene's location
-    //
-    CSeqFeatData::ESubtype subtype = pFeature->GetData().GetSubtype();
-    if (subtype == CSeqFeatData::eSubtype_gene) {
-        return true;
-    }
+    auto subtype = pFeature->GetData().GetSubtype();
     auto recType = Type();
     NStr::ToLower(recType);
+
     const CSeq_loc& target = pFeature->GetLocation();
     CRef<CSeq_loc> pAddLoc = GetSeqLoc(flags, seqidresolve);
 
     if (target.IsInt()  &&  target.GetInt().GetFrom() <= SeqStart()  &&
-            target.GetInt().GetTo() >= SeqStop() ) {
+        target.GetInt().GetTo() >= SeqStop() ) {
         if (recType == "start_codon"  ||  recType == "stop_codon") {
             return true;
         }
@@ -691,7 +686,7 @@ bool CGff2Record::xUpdateFeatureData(
     
 
 //  ----------------------------------------------------------------------------
-bool CGff2Record::x_MigrateId(
+bool CGff2Record::xMigrateId(
     CRef<CSeq_feat> pFeature ) const
 //  ----------------------------------------------------------------------------
 {
@@ -703,7 +698,7 @@ bool CGff2Record::x_MigrateId(
 }
 
 //  ----------------------------------------------------------------------------
-bool CGff2Record::x_MigrateStartStopStrand(
+bool CGff2Record::xMigrateStartStopStrand(
     CRef<CSeq_feat> pFeature ) const
 //  ----------------------------------------------------------------------------
 {
@@ -711,7 +706,7 @@ bool CGff2Record::x_MigrateStartStopStrand(
 }
 
 //  ----------------------------------------------------------------------------
-bool CGff2Record::x_MigrateType(
+bool CGff2Record::xMigrateType(
     CRef<CSeq_feat> pFeature ) const
 //  ----------------------------------------------------------------------------
 {
@@ -720,7 +715,7 @@ bool CGff2Record::x_MigrateType(
 
 
 //  ----------------------------------------------------------------------------
-bool CGff2Record::x_MigrateScore(
+bool CGff2Record::xMigrateScore(
     CRef<CSeq_feat> pFeature ) const
 //  ----------------------------------------------------------------------------
 {
@@ -728,7 +723,7 @@ bool CGff2Record::x_MigrateScore(
 }
 
 //  ----------------------------------------------------------------------------
-bool CGff2Record::x_MigratePhase(
+bool CGff2Record::xMigratePhase(
     CRef<CSeq_feat> pFeature ) const
 //  ----------------------------------------------------------------------------
 {
@@ -736,7 +731,7 @@ bool CGff2Record::x_MigratePhase(
 }
 
 //  ----------------------------------------------------------------------------
-bool CGff2Record::x_MigrateAttributes(
+bool CGff2Record::xMigrateAttributes(
     int flags,
     CRef<CSeq_feat> pFeature ) const
 //  ----------------------------------------------------------------------------
@@ -994,10 +989,10 @@ bool CGff2Record::x_MigrateAttributes(
     }
 
     if (pFeature->GetData().IsBiosrc()) { 
-        if (!x_MigrateAttributesSubSource(flags, pFeature, attrs_left)) {
+        if (!xMigrateAttributesSubSource(flags, pFeature, attrs_left)) {
             return false;
         }
-        if (!x_MigrateAttributesOrgName(flags, pFeature, attrs_left)) {
+        if (!xMigrateAttributesOrgName(flags, pFeature, attrs_left)) {
             return false;
         }
     }
@@ -1064,7 +1059,7 @@ bool CGff2Record::xMigrateAttributeDefault(
 }
 
 //  ----------------------------------------------------------------------------
-bool CGff2Record::x_MigrateAttributesOrgName(
+bool CGff2Record::xMigrateAttributesOrgName(
     int flags,
     CRef<CSeq_feat> pFeature,
     TAttributes& attrs_left) const
@@ -1131,7 +1126,7 @@ bool CGff2Record::x_MigrateAttributesOrgName(
 }
 
 //  ----------------------------------------------------------------------------
-bool CGff2Record::x_MigrateAttributesSubSource(
+bool CGff2Record::xMigrateAttributesSubSource(
     int flags,
     CRef<CSeq_feat> pFeature,
     TAttributes& attrs_left) const
@@ -1205,7 +1200,7 @@ bool CGff2Record::x_MigrateAttributesSubSource(
 }
 
 //  ----------------------------------------------------------------------------
-bool CGff2Record::x_InitFeatureLocation(
+bool CGff2Record::xInitFeatureLocation(
     int flags,
     CRef<CSeq_feat> pFeature,
     SeqIdResolver seqidresolve ) const
