@@ -1,4 +1,4 @@
-/*  $Id: gff3_reader.cpp 559236 2018-03-08 14:54:50Z ludwigf $
+/*  $Id: gff3_reader.cpp 575509 2018-11-29 19:37:34Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -220,21 +220,7 @@ bool CGff3Reader::IsInGenbankMode() const
 }
 
 //  ----------------------------------------------------------------------------
-bool CGff3Reader::x_UpdateFeatureCds(
-    const CGff2Record& gff,
-    CRef<CSeq_feat> pFeature)
-//  ----------------------------------------------------------------------------
-{
-    CRef<CSeq_feat> pAdd = CRef<CSeq_feat>(new CSeq_feat);
-    if (!x_FeatureSetLocation(gff, pAdd)) {
-        return false;
-    }
-    pFeature->SetLocation().Add(pAdd->GetLocation());
-    return true;
-}
-
-//  ----------------------------------------------------------------------------
-bool CGff3Reader::x_UpdateAnnotFeature(
+bool CGff3Reader::xUpdateAnnotFeature(
     const CGff2Record& record,
     CRef< CSeq_annot > pAnnot,
     ILineErrorListener* pEC)
@@ -293,41 +279,34 @@ bool CGff3Reader::xUpdateAnnotExon(
 {
     list<string> parents;
     if (record.GetAttribute("Parent", parents)) {
-        if (!parents.empty()  &&  parents.front() == "ENST00000367927") {
-            cerr << "";
-        }
         for (list<string>::const_iterator it = parents.begin(); it != parents.end(); 
                 ++it) {
             const string& parentId = *it; 
-            if (!xVerifyExonLocation(parentId, record, pEC)) {
-                //create a free agent "exon" feature
-                if (!record.InitializeFeature(m_iFlags, pFeature)) {
-                    return false;
-                }
-                CRef<CSeq_feat> pParent;
-                if (!xGetParentFeature(*pFeature, pParent)) {
-                //Note: The below does not quite cut it as there are types of RNA that come
-                // as Imps.
-                //if (!xGetParentFeature(*pFeature, pParent)  ||  
-                //        (!pParent->GetData().IsGene()  &&  !pParent->GetData().IsRna())) {
-                    AutoPtr<CObjReaderLineException> pErr(
-                        CObjReaderLineException::Create(
+            CRef<CSeq_feat> pParent;
+            if (!x_GetFeatureById(parentId, pParent)) {
+                xAddPendingExon(parentId, record);
+                return true;
+            }
+            if (pParent->GetData().IsRna()  &&  !xVerifyExonLocation(parentId, record, pEC)) {
+                AutoPtr<CObjReaderLineException> pErr(
+                    CObjReaderLineException::Create(
                         eDiag_Error,
                         0,
                         "Bad data line: Exon record referring to non-existing mRNA or gene parent.",
                         ILineError::eProblem_FeatureBadStartAndOrStop));
-                    ProcessError(*pErr, pEC);
+                ProcessError(*pErr, pEC);
+                return false;
+            }
+            if (pParent->GetData().IsGene()) {
+                if  (!xInitializeFeature(record, pFeature)) {
                     return false;
                 }
-                if (! xAddFeatureToAnnot(pFeature, pAnnot)) {
-                    return false;
-                }
-                return true;
+                return xAddFeatureToAnnot(pFeature, pAnnot);            
             }
             IdToFeatureMap::iterator fit = m_MapIdToFeature.find(parentId);
             if (fit != m_MapIdToFeature.end()) {
                 CRef<CSeq_feat> pParent = fit->second;
-                if (!record.UpdateFeature(m_iFlags, pParent)) {
+                if (!pParent->GetData().IsGene()  &&  !record.UpdateFeature(m_iFlags, pParent)) {
                     return false;
                 }
             }
@@ -389,10 +368,8 @@ bool CGff3Reader::xUpdateAnnotCds(
         IdToFeatureMap::iterator featIt = m_MapIdToFeature.find(parentId);
         if (featIt != m_MapIdToFeature.end()) {
             CRef<CSeq_feat> pParent = featIt->second;
-            if (pParent->GetData().IsGene()) {
-                parentIsGene = true;
-            }
-            if (!record.UpdateFeature(m_iFlags, pParent)) {
+            parentIsGene = pParent->GetData().IsGene();
+            if (!parentIsGene  &&  !record.UpdateFeature(m_iFlags, pParent)) {
                 return false;
             }
             //rw-143:
@@ -445,7 +422,7 @@ bool CGff3Reader::xUpdateAnnotCds(
         else {
             //didn't find feature with that ID: create new one
             pFeature.Reset(new CSeq_feat);
-            record.InitializeFeature(m_iFlags, pFeature);
+            xInitializeFeature(record, pFeature);
             if (!parentId.empty()) {
                 xFeatureSetQualifier("Parent", parentId, pFeature);
                 xFeatureSetXrefParent(parentId, pFeature);
@@ -528,6 +505,43 @@ bool CGff3Reader::xFeatureSetXrefParent(
     return true;
 }
 
+//  ----------------------------------------------------------------------------
+bool CGff3Reader::xFindFeatureUnderConstruction(
+    const CGff2Record& record,
+    CRef<CSeq_feat>& underConstruction)
+//  ----------------------------------------------------------------------------
+{
+    string id;
+    if (!record.GetAttribute("ID", id)) {
+        return false;
+    }
+    IdToFeatureMap::iterator it = m_MapIdToFeature.find(id);
+    if (it == m_MapIdToFeature.end()) {
+        return false;
+    }
+
+    AutoPtr<CObjReaderLineException> pErr(CObjReaderLineException::Create(
+        eDiag_Fatal,
+        0,
+        string("Bad data line: Duplicate feature ID \"") + id + "\"",
+        ILineError::eProblem_DuplicateIDs) );
+    if (record.Id() != mIdToSeqIdMap[id]) {
+        pErr->Throw();
+    }
+    if (it->second->GetData().IsRna()) {
+        pErr->Throw();
+    }
+    CSeq_feat tempFeat;
+    if (CSoMap::SoTypeToFeature(record.Type(), tempFeat)) {
+        if (it->second->GetData().GetSubtype() != tempFeat.GetData().GetSubtype()) {
+            pErr->Throw();
+        }
+    }
+
+    underConstruction = it->second;
+    return true;
+}
+
 //  ----------------------------------------------------------------------------
 bool CGff3Reader::xUpdateAnnotGeneric(
     const CGff2Record& record,
@@ -536,12 +550,9 @@ bool CGff3Reader::xUpdateAnnotGeneric(
     ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
-    string id;
-    if (record.GetAttribute("ID", id)) {
-        IdToFeatureMap::iterator it = m_MapIdToFeature.find(id);
-        if (it != m_MapIdToFeature.end()) {
-            return record.UpdateFeature(m_iFlags, it->second);
-        }
+    CRef<CSeq_feat> pUnderConstruction(new CSeq_feat);
+    if (xFindFeatureUnderConstruction(record, pUnderConstruction)) {
+        return record.UpdateFeature(m_iFlags, pUnderConstruction);
     }
 
     string featType = record.Type();
@@ -573,8 +584,8 @@ bool CGff3Reader::xUpdateAnnotGeneric(
         CSeq_interval& cbLoc = pCodeBreak->SetLoc().SetInt();        
         CRef< CSeq_id > pId = mSeqIdResolve(record.Id(), m_iFlags, true);
         cbLoc.SetId(*pId);
-        cbLoc.SetFrom(record.SeqStart());
-        cbLoc.SetTo(record.SeqStop());
+        cbLoc.SetFrom(static_cast<TSeqPos>(record.SeqStart()));
+        cbLoc.SetTo(static_cast<TSeqPos>(record.SeqStop()));
         if (record.IsSetStrand()) {
             cbLoc.SetStrand(record.Strand());
         }
@@ -586,7 +597,7 @@ bool CGff3Reader::xUpdateAnnotGeneric(
         codeBreaks.push_back(pCodeBreak);
         return true;
     }
-    if (!record.InitializeFeature(m_iFlags, pFeature)) {
+    if (!xInitializeFeature(record, pFeature)) {
         return false;
     }
     if (! xAddFeatureToAnnot(pFeature, pAnnot)) {
@@ -596,7 +607,6 @@ bool CGff3Reader::xUpdateAnnotGeneric(
     if ( record.GetAttribute("ID", strId)) {
         m_MapIdToFeature[strId] = pFeature;
     }
-    auto st = pFeature->GetData().GetSubtype();
     if (pFeature->GetData().IsRna()  ||  pFeature->GetData().GetSubtype() == CSeqFeatData::eSubtype_misc_RNA) {
         CRef<CSeq_interval> rnaLoc(new CSeq_interval);
         rnaLoc->Assign(pFeature->GetLocation().GetInt());
@@ -613,30 +623,14 @@ bool CGff3Reader::xUpdateAnnotMrna(
     ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
-    string id;
-    if (record.GetAttribute("ID", id)) {
-        IdToFeatureMap::iterator it = m_MapIdToFeature.find(id);
-        if (it != m_MapIdToFeature.end()) {
-            return record.UpdateFeature(m_iFlags, it->second);
-        }
+    CRef<CSeq_feat> pUnderConstruction(new CSeq_feat);
+    if (xFindFeatureUnderConstruction(record, pUnderConstruction)) {
+        return record.UpdateFeature(m_iFlags, pUnderConstruction);
     }
 
-    if (!record.InitializeFeature(m_iFlags, pFeature)) {
+    if (!xInitializeFeature(record, pFeature)) {
         return false;
     }
-    CRef<CSeq_interval> mrnaLoc(new CSeq_interval);
-    CSeq_loc::E_Choice choice = pFeature->GetLocation().Which();
-    if (choice != CSeq_loc::e_Int) {
-        AutoPtr<CObjReaderLineException> pErr(
-            CObjReaderLineException::Create(
-            eDiag_Error,
-            0,
-            "Internal error: Unexpected location type.",
-            ILineError::eProblem_BadFeatureInterval));
-    }
-    mrnaLoc->Assign(pFeature->GetLocation().GetInt());
-    mMrnaLocs[id] = mrnaLoc;
-
     string parentsStr;
     if ((m_iFlags & fGeneXrefs)  &&  record.GetAttribute("Parent", parentsStr)) {
         list<string> parents;
@@ -656,13 +650,32 @@ bool CGff3Reader::xUpdateAnnotMrna(
         }
     }
 
-    if (! xAddFeatureToAnnot(pFeature, pAnnot)) {
-        return false;
-    }
     string strId;
     if ( record.GetAttribute("ID", strId)) {
         m_MapIdToFeature[strId] = pFeature;
     }
+    CRef<CSeq_interval> mrnaLoc(new CSeq_interval);
+    CSeq_loc::E_Choice choice = pFeature->GetLocation().Which();
+    if (choice != CSeq_loc::e_Int) {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+                eDiag_Error,
+                0,
+                "Internal error: Unexpected location type.",
+                ILineError::eProblem_BadFeatureInterval));
+    }
+    mrnaLoc->Assign(pFeature->GetLocation().GetInt());
+    mMrnaLocs[strId] = mrnaLoc;
+
+    list<CGff2Record> pendingExons;
+    xGetPendingExons(strId, pendingExons);
+    for (auto exonRecord: pendingExons) {
+        CRef< CSeq_feat > pFeature(new CSeq_feat);
+        xUpdateAnnotExon(exonRecord, pFeature, pAnnot, pEC);
+    }
+    if (! xAddFeatureToAnnot(pFeature, pAnnot)) {
+        return false;
+    }
     return true;
 }
 
@@ -811,5 +824,70 @@ bool CGff3Reader::xIsIgnoredFeatureType(
     return false;
 }
 
+//  ----------------------------------------------------------------------------
+bool
+CGff3Reader::xInitializeFeature(
+    const CGff2Record& record,
+    CRef<CSeq_feat> pFeature)
+//  ----------------------------------------------------------------------------
+{
+    if (!record.InitializeFeature(m_iFlags, pFeature)) {
+        return false;
+    }
+    const auto& attrs = record.Attributes();
+    const auto it = attrs.find("ID");
+    if (it != attrs.end()) {
+        mIdToSeqIdMap[it->second] = record.Id();
+    }
+    return true;
+}
+
+//  ----------------------------------------------------------------------------
+void
+CGff3Reader::xAddPendingExon(
+    const string& rnaId,
+    const CGff2Record& exonRecord)
+//  ----------------------------------------------------------------------------
+{
+    PENDING_EXONS::iterator it = mPendingExons.find(rnaId);
+    if (it == mPendingExons.end()) {
+        mPendingExons[rnaId] = list<CGff2Record>();
+    }
+    mPendingExons[rnaId].push_back(exonRecord);
+}
+
+//  ----------------------------------------------------------------------------
+void
+CGff3Reader::xGetPendingExons(
+    const string& rnaId,
+    list<CGff2Record>& pendingExons)
+//  ----------------------------------------------------------------------------
+{
+    PENDING_EXONS::iterator it = mPendingExons.find(rnaId);
+    if (it == mPendingExons.end()) {
+        return;
+    }
+    pendingExons.swap(mPendingExons[rnaId]);
+    mPendingExons.erase(rnaId);
+}
+
+//  ----------------------------------------------------------------------------
+void CGff3Reader::xPostProcessAnnot(
+    CRef<CSeq_annot>& pAnnot,
+    ILineErrorListener *pEC)
+    //  ----------------------------------------------------------------------------
+{
+    for (const auto& it: mPendingExons) {
+        AutoPtr<CObjReaderLineException> pErr(CObjReaderLineException::Create(
+            eDiag_Warning,
+            0,
+            "Bad data line: Record references non-existant Parent=" + it.first,
+            ILineError::eProblem_MissingContext) );
+        ProcessError(*pErr, pEC);
+    }
+    return CGff2Reader::xPostProcessAnnot(pAnnot, pEC);
+}
+
+
 END_objects_SCOPE
 END_NCBI_SCOPE
