@@ -1,4 +1,4 @@
-/*  $Id: glimmer_reader.cpp 389354 2013-02-15 00:27:40Z dicuccio $
+/*  $Id: glimmer_reader.cpp 515707 2016-10-05 12:32:29Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -110,7 +110,7 @@ CRef<CSeq_entry> CGlimmerReader::Read(CNcbiIstream& istr, CScope& scope,
             }
         } else {
             list<string> toks;
-            NStr::Split(line, " \t", toks);
+            NStr::Split(line, " \t", toks, NStr::fSplit_Tokenize);
             if (toks.size() != 5) {
                 CNcbiOstrstream ostr;
                 ostr << "CGlimmerReader::ReadAnnot(): line "
