@@ -1,4 +1,4 @@
- /*  $Id: gvf_reader.cpp 539953 2017-06-29 13:16:45Z ludwigf $
+ /*  $Id: gvf_reader.cpp 575504 2018-11-29 19:35:15Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -149,13 +149,13 @@ bool CGvfReadRecord::AssignFromGff(
 }
 
 //  ----------------------------------------------------------------------------
-bool CGvfReadRecord::x_AssignAttributesFromGff(
+bool CGvfReadRecord::xAssignAttributesFromGff(
     const string& strGffType,
     const string& strRawAttributes )
 //  ----------------------------------------------------------------------------
 {
     vector< string > attributes;
-    x_SplitGffAttributes(strRawAttributes, attributes);
+    xSplitGffAttributes(strRawAttributes, attributes);
 	for ( size_t u=0; u < attributes.size(); ++u ) {
         string strKey;
         string strValue;
