@@ -1,4 +1,4 @@
-/*  $Id: microarray_reader.cpp 515629 2016-10-04 17:46:33Z ivanov $
+/*  $Id: microarray_reader.cpp 536287 2017-05-17 13:05:21Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -160,7 +160,7 @@ CMicroArrayReader::ReadSeqAnnot(
 
         //  parse
         vector<string> fields;
-        NStr::Split( record_copy, " \t", fields, NStr::eMergeDelims );
+        NStr::Split(record_copy, " \t", fields, NStr::fSplit_MergeDelimiters);
         try {
             xCleanColumnValues(fields);
         }
