@@ -1,4 +1,4 @@
-/*  $Id: reader_base.cpp 542725 2017-08-02 11:32:01Z ivanov $
+/*  $Id: reader_base.cpp 572658 2018-10-17 17:01:41Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -275,7 +275,6 @@ CReaderBase::ProcessError(
             "Error allowance exceeded",
             ILineError::eProblem_GeneralParsingError) );
         pErr->Throw();
-        //err.Throw();
     }
 }
 
@@ -385,7 +384,7 @@ void CReaderBase::xSetBrowserRegion(
                 0,
                 "Bad browser line: cannot parse browser position" ) );
             ProcessError(*pErr, pEC);
-            location.Release();
+            location.Reset();
         }
     }
 
@@ -649,7 +648,7 @@ bool CReaderBase::xIsTrackTerminator(
     const CTempString& strLine)
 //  ----------------------------------------------------------------------------
 {
-    auto line = NStr::TruncateSpaces(strLine);
+    auto line = NStr::TruncateSpaces_Unsafe(strLine);
     if (NStr::StartsWith(line, "##sequence-region")) {
         return true;
     }
