@@ -1,4 +1,4 @@
-/*  $Id: source_mod_parser.cpp 542051 2017-07-25 16:55:29Z ivanov $
+/*  $Id: source_mod_parser.cpp 572183 2018-10-10 12:13:06Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -36,6 +36,7 @@
 #include <sstream>
 
 #include <objtools/readers/source_mod_parser.hpp>
+#include <objtools/readers/message_listener.hpp>
 
 #include <corelib/ncbiutil.hpp>
 #include <util/static_map.hpp>
@@ -65,7 +66,8 @@
 #include <objects/seqfeat/SubSource.hpp>
 #include <objects/seqloc/Seq_id.hpp>
 #include <objects/seqloc/Seq_loc.hpp>
-// #include <objects/submit/Submit_block.hpp>
+
+#include <objects/general/User_object.hpp>
 
 BEGIN_NCBI_SCOPE
 BEGIN_SCOPE(objects)
@@ -88,12 +90,16 @@ namespace
 #  error "STATIC_SMOD already defined"
 #endif
 
-// The macro makes sure that the var's name matches its key.
-// Due to kKeyCanonicalizationTable, it's okay to use '_' for '-'
-// because it will match both.
+    // The macro makes sure that the var's name matches its key.
+    // Due to kKeyCanonicalizationTable, it's okay to use '_' for '-'
+    // because it will match both.
+
+
 #define STATIC_SMOD(key_str) \
-    CSafeStatic<CSourceModParser::SMod> s_Mod_##key_str([]() { \
-        return new CSourceModParser::SMod(#key_str); }, nullptr);
+    const char   s_Mod_s_##key_str[] = #key_str; \
+    const size_t s_Mod_n_##key_str = sizeof(#key_str)-1; \
+    const CTempString s_Mod_##key_str(s_Mod_s_##key_str, s_Mod_n_##key_str)
+
 
     // For CBioseq
     STATIC_SMOD(topology);
@@ -116,9 +122,13 @@ namespace
     STATIC_SMOD(lat_long);
     STATIC_SMOD(latitude_longitude);
     STATIC_SMOD(fwd_primer_seq);
+    STATIC_SMOD(fwd_pcr_primer_seq);
     STATIC_SMOD(rev_primer_seq);
+    STATIC_SMOD(rev_pcr_primer_seq);
     STATIC_SMOD(fwd_primer_name);
+    STATIC_SMOD(fwd_pcr_primer_name);
     STATIC_SMOD(rev_primer_name);
+    STATIC_SMOD(rev_pcr_primer_name);
     STATIC_SMOD(dbxref);
     STATIC_SMOD(db_xref);
     STATIC_SMOD(division);
@@ -158,9 +168,13 @@ namespace
     STATIC_SMOD(keyword);
     STATIC_SMOD(keywords);
 
+    STATIC_SMOD(biosample);
+    STATIC_SMOD(bioproject);
     // For TPA Mods (CUser_object)
     STATIC_SMOD(primary);
     STATIC_SMOD(primary_accessions);
+    // For SRA (Sequence Read Archive) CUser_object
+    STATIC_SMOD(SRA);
 
     // For Genome Project DB Mods (CUser_object)
     STATIC_SMOD(project);
@@ -257,6 +271,8 @@ namespace
             // skip because handled specially elsewhere
             "fwd_primer_seq", "rev_primer_seq",
             "fwd_primer_name", "rev_primer_name",
+            "fwd_PCR_primer_seq", "rev_PCR_primer_seq",
+            "fwd_PCR_primer_name", "rev_PCR_primer_name",
             // skip because deprecated
             "transposon_name",
             "plastid_name",
@@ -275,8 +291,67 @@ namespace
                                                  
     CSafeStatic<TSModSubSrcSubtype> kSModSubSrcSubtypeMap(
         s_InitSModSubSrcSubtypeMap, nullptr);
+
+    bool x_FindBrackets(const CTempString& str, size_t& start, size_t& stop, size_t& eq_pos)
+    {
+        size_t i = start;
+        bool found = false;
+
+        eq_pos = CTempString::npos;
+
+        const char* s = str.data() + start;
+
+        int nested_brackets = -1;
+        while (i < str.size())
+        {
+            switch (*s)
+            {
+            case '[':
+                nested_brackets++;
+                if (nested_brackets == 0)
+                {
+                    start = i;
+                }
+                break;
+            case '=':
+                if (nested_brackets >= 0)
+                if (eq_pos == CTempString::npos)
+                    eq_pos = i;
+                break;
+            case ']':
+                if (nested_brackets == 0)
+                {
+                    stop = i;
+                    if (eq_pos == CTempString::npos)
+                        eq_pos = i;
+                    return true;
+                }
+                else
+                if (nested_brackets < 0)
+                    return false;
+                else
+                {
+                    nested_brackets--;
+                }
+            }
+            i++; s++;
+        }
+        return false;
+    };
+
+    void x_AppendIfNonEmpty(string& s, const CTempString& o)
+    {
+        if (!o.empty())
+        {
+            if (!s.empty())
+                s.push_back(' ');
+            s.append(o.data(), o.length());
+        }
+    }
+
 };
 
+
 CSafeStatic<CSourceModParser::SMod> CSourceModParser::kEmptyMod;
 
 // ASCII letters to lowercase, space and underscore to hyphen.
@@ -313,6 +388,57 @@ protected:
     mutable CRef<CBioseq_set> m_bioset;
 };
 
+class CAutoAddDBLink
+{
+public:
+    CAutoAddDBLink(CBioseq& seq, const CTempString& id)
+      :m_bioseq(seq), m_id(id)
+    {
+    }
+    bool IsInitialised() const
+    {
+        return !m_dblink.Empty();
+    }
+
+    CUser_field& Get()
+    {
+        if (m_dblink)
+            return *m_dblink;
+
+        for (auto& d : m_bioseq.SetDescr().Set())
+        {
+            if (d->IsUser() && d->GetUser().IsDBLink())
+            {
+                for (auto& u : d->SetUser().SetData())
+                {
+                    if (u->IsSetLabel() && u->GetLabel().IsStr() &&
+                        NStr::EqualCase(u->GetLabel().GetStr(), m_id))
+                    {
+                        m_dblink = u;
+                        return *m_dblink;
+                    }
+                }
+            }
+        }
+        if (m_dblink.Empty())
+        {
+            m_user_obj.Reset(new CSeqdesc);
+            m_user_obj->SetUser().SetType().SetStr() = "DBLink";
+            m_dblink.Reset(new CUser_field);
+            m_dblink->SetLabel().SetStr() = m_id;
+            m_user_obj->SetUser().SetData().push_back(m_dblink);
+            m_bioseq.SetDescr().Set().push_back(m_user_obj);
+        }
+
+        return *m_dblink;
+    }
+protected:
+    CBioseq& m_bioseq;
+    CTempString m_id;
+    CRef<CUser_field> m_dblink;
+    CRef<CSeqdesc> m_user_obj;
+};
+
 CSafeStaticRef<CSeq_descr> fake_descr;
 
 template<class _T>
@@ -399,6 +525,7 @@ void CAutoInitDesc<CGB_block>::_getfromdesc()
     m_ptr = &Set().SetGenbank();
 }
 
+
 string CSourceModParser::ParseTitle(const CTempString& title, 
     CConstRef<CSeq_id> seqid,
     size_t iMaxModsToParse )
@@ -411,44 +538,37 @@ string CSourceModParser::ParseTitle(const CTempString& title,
 
     mod.seqid = seqid;
 
-    size_t iModsFoundSoFar = 0;
-    for( ; (pos < title.size()) && (iModsFoundSoFar < iMaxModsToParse); 
+    size_t iModsFoundSoFar = 0;    
+    for (; (pos < title.size()) && (iModsFoundSoFar < iMaxModsToParse);
         ++iModsFoundSoFar )
     {
-        size_t lb_pos = title.find('[', pos), eq_pos = title.find('=', lb_pos),
-               end_pos = CTempString::npos;
-        if (eq_pos != CTempString::npos) {
-            mod.key = NStr::TruncateSpaces_Unsafe
-                (title.substr(lb_pos + 1, eq_pos - lb_pos - 1));
-            if (eq_pos + 3 < title.size()  &&  title[eq_pos + 1] == '"') {
-                end_pos = title.find('"', ++eq_pos + 1);
-            } else {
-                end_pos = title.find(']', eq_pos + 1);
+        size_t lb_pos, end_pos, eq_pos;
+        lb_pos = pos;
+        if (x_FindBrackets(title, lb_pos, end_pos, eq_pos))
+        {            
+            CTempString skipped = NStr::TruncateSpaces_Unsafe(title.substr(pos, lb_pos - pos));
+
+            if (eq_pos < end_pos) {
+                CTempString key = NStr::TruncateSpaces_Unsafe(title.substr(lb_pos+1, eq_pos - lb_pos - 1));
+                CTempString value = NStr::TruncateSpaces_Unsafe(title.substr(eq_pos + 1, end_pos - eq_pos - 1));
+
+                mod.key = key;
+                mod.value = value;
+                mod.pos = lb_pos;
+                mod.used = false;
+                m_Mods.emplace(mod);
             }
-        }
-        if (end_pos == CTempString::npos) {
-            stripped_title += title.substr(pos);
-            break;
-        } else {
-            mod.value = NStr::TruncateSpaces_Unsafe
-                (title.substr(eq_pos + 1, end_pos - eq_pos - 1));
-            if (title[end_pos] == '"') {
-                end_pos = title.find(']', end_pos + 1);
-                if (end_pos == CTempString::npos) {
-                    break;
-                }
-            }
-            mod.pos = lb_pos;
-            mod.used = false;
-            m_Mods.emplace(mod);
-            CTempString text = NStr::TruncateSpaces_Unsafe
-                (title.substr(pos, lb_pos - pos));
-            if ( !stripped_title.empty()  &&  !text.empty() ) {
-                stripped_title += ' ';
-            }
-            stripped_title += text;
+
+            x_AppendIfNonEmpty(stripped_title, skipped);
+
             pos = end_pos + 1;
         }
+        else
+        { // rest of the title is unparsed
+            CTempString rest = NStr::TruncateSpaces_Unsafe(title.substr(pos));
+            x_AppendIfNonEmpty(stripped_title, rest);
+            break;
+        }
     }
 
     return stripped_title;
@@ -567,6 +687,8 @@ void CSourceModParser::ApplyAllMods(CBioseq& seq, CTempString organism, CConstRe
         }
     }}
 
+    x_ApplyDBLinkMods(seq);
+
     {{
         CAutoInitRef<CUser_object> gpdb;
         x_ApplyGenomeProjectsDBMods(gpdb);
@@ -580,6 +702,11 @@ void CSourceModParser::ApplyAllMods(CBioseq& seq, CTempString organism, CConstRe
     {{
         ApplyPubMods(seq);
     }}
+
+    TMods unusedMods = GetMods(fUnusedMods);
+    for (TMods::const_iterator unused = unusedMods.begin(); unused != unusedMods.end(); ++unused) {
+        x_HandleUnkModValue(*unused); 
+    }
 };
 
 struct SMolTypeInfo {
@@ -630,7 +757,7 @@ void CSourceModParser::ApplyMods(CBioseq& seq)
     const SMod* mod = NULL;
 
     // top[ology]
-    if ((mod = FindMod(s_Mod_topology.Get(), s_Mod_top.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_topology, s_Mod_top)) != NULL) {
         if (NStr::EqualNocase(mod->value, "linear")) {
             seq.SetInst().SetTopology(CSeq_inst::eTopology_linear);
         } else if (NStr::EqualNocase(mod->value, "circular")) {
@@ -647,7 +774,7 @@ void CSourceModParser::ApplyMods(CBioseq& seq)
         bool bMolSetViaMolMod = false;
 
         // mol[ecule]
-        if ((mod = FindMod(s_Mod_molecule.Get(), s_Mod_mol.Get())) != NULL) {
+        if ((mod = FindMod(s_Mod_molecule, s_Mod_mol)) != NULL) {
             if (NStr::EqualNocase(mod->value, "dna")) {
                 seq.SetInst().SetMol( CSeq_inst::eMol_dna );
                 bMolSetViaMolMod = true;
@@ -663,7 +790,7 @@ void CSourceModParser::ApplyMods(CBioseq& seq)
 
         // mol[-]type
         if( ! bMolSetViaMolMod ) {
-            if ((mod = FindMod(s_Mod_moltype.Get(), s_Mod_mol_type.Get())) != NULL) {
+            if ((mod = FindMod(s_Mod_moltype, s_Mod_mol_type)) != NULL) {
                 TBiomolMap::const_iterator it = sc_BiomolMap.find(mod->value.c_str());
                 if (it == sc_BiomolMap.end()) {
                     x_HandleBadModValue(*mod);
@@ -676,7 +803,7 @@ void CSourceModParser::ApplyMods(CBioseq& seq)
     }
 
     // strand
-    if ((mod = FindMod(s_Mod_strand.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_strand)) != NULL) {
         if (NStr::EqualNocase(mod->value, "single")) {
             seq.SetInst().SetStrand( CSeq_inst::eStrand_ss );
         } else if (NStr::EqualNocase(mod->value, "double")) {
@@ -689,13 +816,143 @@ void CSourceModParser::ApplyMods(CBioseq& seq)
     }
 
     // comment
-    if ((mod = FindMod(s_Mod_comment.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_comment)) != NULL) {
         CRef<CSeqdesc> desc(new CSeqdesc);
         desc->SetComment( mod->value );
         seq.SetDescr().Set().push_back(desc);
     }
 }
 
+
+static void s_AddPrimers(const pair<string, string>& primer_info, CPCRPrimerSet& primer_set)
+{
+    vector<string> names;
+    NStr::Split(primer_info.first, ":", names, NStr::fSplit_Tokenize);
+    vector<string> seqs;
+    NStr::Split(primer_info.second, ":", seqs, NStr::fSplit_Tokenize);
+
+    const auto num_names = names.size();
+    const auto num_seqs = seqs.size();
+    const auto num_primers = max(num_names, num_seqs);
+
+    for(size_t i=0; i<num_primers; ++i) {
+        auto primer = Ref(new CPCRPrimer());
+
+        if (i<num_names && !NStr::IsBlank(names[i])) {
+            primer->SetName().Set(names[i]); 
+        }
+        if (i<num_seqs && !NStr::IsBlank(seqs[i])) {
+            primer->SetSeq().Set(seqs[i]);
+        }
+        primer_set.Set().push_back(primer);
+    }
+}
+
+
+static void s_GetPrimerInfo(const CSourceModParser::SMod* pNamesMod,
+                            const CSourceModParser::SMod* pSeqsMod,
+                            vector<pair<string, string>>& reaction_info)
+{
+    reaction_info.clear();
+    vector<string> names;
+    if (pNamesMod) {
+        NStr::Split(pNamesMod->value, ",", names, NStr::fSplit_Tokenize);
+    }
+
+    vector<string> seqs;
+    if (pSeqsMod) {
+        NStr::Split(pSeqsMod->value, ",", seqs, NStr::fSplit_Tokenize);
+        if (seqs.size()>1) {
+            if (seqs.front().front() == '(') {
+                seqs.front().erase(0,1);
+            }
+            if (seqs.back().back() == ')') {
+                seqs.back().erase(seqs.back().size()-1, 1);
+            }
+        }
+    }
+
+    const auto num_names = names.size();
+    const auto num_seqs = seqs.size();
+    const auto num_reactions = max(num_names, num_seqs);
+
+    for (int i=0; i<num_reactions; ++i) {
+        const string name = (i<num_names) ? names[i] : "";
+        const string seq  = (i<num_seqs) ? seqs[i] : "";
+        reaction_info.push_back(make_pair(name, seq));
+    }
+}
+
+
+void CSourceModParser::x_AddPCRPrimers(CAutoInitRef<CPCRReactionSet>& pcr_reaction_set)
+{
+    using TNameSeqPair = pair<string, string>;
+
+    const SMod* pNameMod = nullptr;
+    const SMod* pSeqMod = nullptr;
+
+    pNameMod = FindMod(s_Mod_fwd_primer_name, s_Mod_fwd_pcr_primer_name);
+    pSeqMod = FindMod(s_Mod_fwd_primer_seq, s_Mod_fwd_pcr_primer_seq);
+    vector<TNameSeqPair> fwd_primer_info; 
+    s_GetPrimerInfo(pNameMod, pSeqMod, fwd_primer_info);
+
+
+    pNameMod = FindMod(s_Mod_rev_primer_name, s_Mod_rev_pcr_primer_name);
+    pSeqMod = FindMod(s_Mod_rev_primer_seq, s_Mod_rev_pcr_primer_seq);
+    vector<TNameSeqPair> rev_primer_info; 
+    s_GetPrimerInfo(pNameMod, pSeqMod, rev_primer_info);
+
+    if (fwd_primer_info.empty() &&
+        rev_primer_info.empty()) {
+        return;
+    }
+
+    auto num_fwd_primer_info = fwd_primer_info.size();
+    auto num_rev_primer_info = rev_primer_info.size();
+
+    if (num_fwd_primer_info == num_rev_primer_info) {
+    
+        for (auto i=0; i<num_fwd_primer_info; ++i) {
+            CRef<CPCRReaction> pcr_reaction(new CPCRReaction());
+            s_AddPrimers(fwd_primer_info[i], pcr_reaction->SetForward());
+            s_AddPrimers(rev_primer_info[i], pcr_reaction->SetReverse());
+            pcr_reaction_set->Set().push_back(pcr_reaction);
+        }
+    } 
+    else 
+    if (num_fwd_primer_info > num_rev_primer_info) {
+        auto diff = num_fwd_primer_info - num_rev_primer_info;
+        for (int i=0; i<diff; ++i) {
+            CRef<CPCRReaction> pcr_reaction(new CPCRReaction());
+            s_AddPrimers(fwd_primer_info[i], pcr_reaction->SetForward());
+            pcr_reaction_set->Set().push_back(pcr_reaction);
+        }
+
+        for (int i=diff; i<num_fwd_primer_info; ++i) {
+            CRef<CPCRReaction> pcr_reaction(new CPCRReaction());
+            s_AddPrimers(fwd_primer_info[i], pcr_reaction->SetForward());
+            s_AddPrimers(rev_primer_info[i-diff], pcr_reaction->SetReverse());
+            pcr_reaction_set->Set().push_back(pcr_reaction);
+        }
+    }
+    else 
+    if (num_fwd_primer_info < num_rev_primer_info) {
+        for (int i=0; i<num_fwd_primer_info; ++i) {
+            CRef<CPCRReaction> pcr_reaction(new CPCRReaction());
+            s_AddPrimers(fwd_primer_info[i], pcr_reaction->SetForward());
+            s_AddPrimers(rev_primer_info[i], pcr_reaction->SetReverse());
+            pcr_reaction_set->Set().push_back(pcr_reaction);
+        }
+
+        for (int i=num_fwd_primer_info; i<num_rev_primer_info; ++i) {
+            CRef<CPCRReaction> pcr_reaction(new CPCRReaction());
+            s_AddPrimers(rev_primer_info[i], pcr_reaction->SetReverse());
+            pcr_reaction_set->Set().push_back(pcr_reaction);
+        }
+    } 
+}
+
+
 void CSourceModParser::x_ApplyMods(CAutoInitDesc<CBioSource>& bsrc,
                                    CTempString organism)
 {
@@ -705,11 +962,11 @@ void CSourceModParser::x_ApplyMods(CAutoInitDesc<CBioSource>& bsrc,
     // org[anism]
     if (organism.empty())
     {
-        if ((mod = FindMod(s_Mod_organism.Get(), s_Mod_org.Get())) != NULL) {
+        if ((mod = FindMod(s_Mod_organism, s_Mod_org)) != NULL) {
             organism = mod->value;
         }
         else
-        if ((mod = FindMod(s_Mod_taxname.Get())) != NULL) {
+        if ((mod = FindMod(s_Mod_taxname)) != NULL) {
             organism = mod->value;
         }
     }
@@ -721,7 +978,7 @@ void CSourceModParser::x_ApplyMods(CAutoInitDesc<CBioSource>& bsrc,
             if (bsrc->GetOrg().IsSetTaxname())
             {
                 bsrc->ResetOrg();
-                bsrc->ResetSubtype();
+//                bsrc->ResetSubtype();
             }
             bsrc->SetOrg().SetTaxname(organism);
             reset_taxid = true;
@@ -729,7 +986,7 @@ void CSourceModParser::x_ApplyMods(CAutoInitDesc<CBioSource>& bsrc,
     }
 
     // location
-    if ((mod = FindMod(s_Mod_location.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_location)) != NULL) {
         if (NStr::EqualNocase(mod->value, "mitochondrial")) {
             bsrc->SetGenome(CBioSource::eGenome_mitochondrion);
         } else if (NStr::EqualNocase(mod->value, "provirus")) {
@@ -749,7 +1006,7 @@ void CSourceModParser::x_ApplyMods(CAutoInitDesc<CBioSource>& bsrc,
     }
 
     // origin
-    if ((mod = FindMod(s_Mod_origin.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_origin)) != NULL) {
         try {
             // also check for special cases that don't match the enum name
             if( NStr::EqualNocase(mod->value, "natural mutant") ) {
@@ -769,7 +1026,7 @@ void CSourceModParser::x_ApplyMods(CAutoInitDesc<CBioSource>& bsrc,
     for(const auto & smod_orgsubtype : kSModOrgSubtypeMap.Get()) {
         const SMod & smod = smod_orgsubtype.first;
         const COrgMod::ESubtype e_subtype = smod_orgsubtype.second;
-        if ((mod = FindMod(smod)) != NULL) {
+        if ((mod = FindMod(smod.key)) != NULL) {
             CRef<COrgMod> org_mod(new COrgMod);
             org_mod->SetSubtype(e_subtype);
             org_mod->SetSubname(mod->value);
@@ -782,7 +1039,7 @@ void CSourceModParser::x_ApplyMods(CAutoInitDesc<CBioSource>& bsrc,
     for( const auto & smod_subsrcsubtype : kSModSubSrcSubtypeMap.Get() ) {
         const SMod & smod = smod_subsrcsubtype.first;
         const CSubSource::ESubtype e_subtype = smod_subsrcsubtype.second;
-        if ((mod = FindMod(smod)) != NULL) {
+        if ((mod = FindMod(smod.key)) != NULL) {
             auto& subtype = bsrc->SetSubtype();
             CRef<CSubSource> subsource(new CSubSource);
             subsource->SetSubtype(e_subtype);
@@ -811,49 +1068,23 @@ void CSourceModParser::x_ApplyMods(CAutoInitDesc<CBioSource>& bsrc,
 
     // handle PCR Primers
     {{
-        CAutoInitRef<CPCRReaction> pcr_reaction;
-        
-        CAutoInitRef<CPCRPrimer> fwd_primer;
-        CAutoInitRef<CPCRPrimer> rev_primer;
+        CAutoInitRef<CPCRReactionSet> pcr_reaction_set;
+        x_AddPCRPrimers(pcr_reaction_set);
+        if (pcr_reaction_set.IsInitialized()) {
+            if (!bsrc->IsSetPcr_primers()) {
+                bsrc->SetPcr_primers(*pcr_reaction_set);
+            }
+            else {
+                bsrc->SetPcr_primers().Set().splice(
+                        bsrc->SetPcr_primers().Set().end(),
+                        pcr_reaction_set->Set());
+            }
+        }
+     }}   
 
-        bool used_fwd = false;
-        bool used_rev = false;
-
-        if ((mod = FindMod(s_Mod_fwd_primer_name.Get())) != NULL) {
-            fwd_primer->SetName().Set( mod->value );
-            used_fwd = true;
-        }
-        if ((mod = FindMod(s_Mod_fwd_primer_seq.Get())) != NULL) {
-            fwd_primer->SetSeq().Set( mod->value );
-            NStr::ToLower( fwd_primer->SetSeq().Set() );
-            used_fwd = true;
-        }
-        if ((mod = FindMod(s_Mod_rev_primer_name.Get())) != NULL) {
-            rev_primer->SetName().Set( mod->value );
-            used_rev = true;
-        }
-        if ((mod = FindMod(s_Mod_rev_primer_seq.Get())) != NULL) {
-            rev_primer->SetSeq().Set( mod->value );
-            NStr::ToLower( rev_primer->SetSeq().Set() );
-            used_rev = true;
-        }
-
-        if( used_fwd ) {
-            pcr_reaction->SetForward().Set().push_back(
-                CRef<CPCRPrimer>(&*fwd_primer) );
-        }
-        if( used_rev ) {
-            pcr_reaction->SetReverse().Set().push_back(
-                CRef<CPCRPrimer>(&*rev_primer) );
-        }
-        if( used_fwd || used_rev ) {
-            bsrc->SetPcr_primers().Set().push_back(
-                CRef<CPCRReaction>(&*pcr_reaction) );
-        }
-    }}
 
     // db_xref
-    TModsRange db_xref_mods_range = FindAllMods( s_Mod_db_xref.Get(), s_Mod_dbxref.Get() );
+    TModsRange db_xref_mods_range = FindAllMods( s_Mod_db_xref, s_Mod_dbxref );
     for( TModsCI db_xref_iter = db_xref_mods_range.first; 
             db_xref_iter != db_xref_mods_range.second; 
             ++db_xref_iter ) {
@@ -879,34 +1110,34 @@ void CSourceModParser::x_ApplyMods(CAutoInitDesc<CBioSource>& bsrc,
     }
 
     // div[ision]
-    if ((mod = FindMod(s_Mod_division.Get(), s_Mod_div.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_division, s_Mod_div)) != NULL) {
         bsrc->SetOrg().SetOrgname().SetDiv( mod->value );
     }
     
     // lineage
-    if ((mod = FindMod(s_Mod_lineage.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_lineage)) != NULL) {
         bsrc->SetOrg().SetOrgname().SetLineage( mod->value );
     }
     
     // gcode
-    if ((mod = FindMod(s_Mod_gcode.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_gcode)) != NULL) {
         bsrc->SetOrg().SetOrgname().SetGcode( NStr::StringToInt(mod->value, NStr::fConvErr_NoThrow) );
     }
 
     // mgcode
-    if ((mod = FindMod(s_Mod_mgcode.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_mgcode)) != NULL) {
         bsrc->SetOrg().SetOrgname().SetMgcode( NStr::StringToInt(mod->value, NStr::fConvErr_NoThrow) );
     }
 
     // pgcode
-    if ((mod = FindMod(s_Mod_pgcode.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_pgcode)) != NULL) {
         bsrc->SetOrg().SetOrgname().SetPgcode( NStr::StringToInt(mod->value, NStr::fConvErr_NoThrow) );
     }
 
     // note[s]
     TModsRange mods[2];
-    mods[0] = FindAllMods(s_Mod_note.Get());
-    mods[1] = FindAllMods(s_Mod_notes.Get());
+    mods[0] = FindAllMods(s_Mod_note);
+    mods[1] = FindAllMods(s_Mod_notes);
     for (size_t i = 0; i < 2; i++)
     {
         for (TModsCI it = mods[i].first; it != mods[i].second; it++)
@@ -919,14 +1150,14 @@ void CSourceModParser::x_ApplyMods(CAutoInitDesc<CBioSource>& bsrc,
     }
 
     // focus
-    if ((mod = FindMod(s_Mod_focus.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_focus)) != NULL) {
         if( NStr::EqualNocase( mod->value, "TRUE" ) ) {
             bsrc->SetIs_focus();
         }
     }
 
 
-    if ((mod = FindMod(s_Mod_taxid.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_taxid)) != NULL) {
         bsrc->SetOrg().SetTaxId( NStr::StringToInt(mod->value, NStr::fConvErr_NoThrow) );
     }
     else 
@@ -986,7 +1217,7 @@ void CSourceModParser::x_ApplyMods(CAutoInitDesc<CMolInfo>& mi)
     const SMod* mod = NULL;
 
     // mol[-]type
-    if ((mod = FindMod(s_Mod_moltype.Get(), s_Mod_mol_type.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_moltype, s_Mod_mol_type)) != NULL) {
         TBiomolMap::const_iterator it = sc_BiomolMap.find(mod->value.c_str());
         if (it == sc_BiomolMap.end()) {
             // construct the possible bad values by hand
@@ -998,7 +1229,7 @@ void CSourceModParser::x_ApplyMods(CAutoInitDesc<CMolInfo>& mi)
     }
 
     // tech
-    if ((mod = FindMod(s_Mod_tech.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_tech)) != NULL) {
         TTechMap::const_iterator it = sc_TechMap.find(mod->value.c_str());
         if (it == sc_TechMap.end()) {
             x_HandleBadModValue(*mod);
@@ -1008,7 +1239,7 @@ void CSourceModParser::x_ApplyMods(CAutoInitDesc<CMolInfo>& mi)
     }
 
     // complete[d]ness
-    if ((mod = FindMod(s_Mod_completeness.Get(), s_Mod_completedness.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_completeness, s_Mod_completedness)) != NULL) {
         TTechMap::const_iterator it = sc_CompletenessMap.find(mod->value.c_str());
         if (it == sc_CompletenessMap.end()) {
             x_HandleBadModValue(*mod);
@@ -1023,22 +1254,22 @@ void CSourceModParser::x_ApplyMods(CAutoInitRef<CGene_ref>& gene)
     const SMod* mod = NULL;
 
     // gene
-    if ((mod = FindMod(s_Mod_gene.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_gene)) != NULL) {
         gene->SetLocus(mod->value);
     }
 
     // allele
-    if ((mod = FindMod(s_Mod_allele.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_allele)) != NULL) {
         gene->SetAllele( mod->value );
     }
 
     // gene_syn[onym]
-    if ((mod = FindMod(s_Mod_gene_syn.Get(), s_Mod_gene_synonym.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_gene_syn, s_Mod_gene_synonym)) != NULL) {
         gene->SetSyn().push_back( mod->value );
     }
     
     // locus_tag
-    if ((mod = FindMod(s_Mod_locus_tag.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_locus_tag)) != NULL) {
         gene->SetLocus_tag( mod->value );
     }
 }
@@ -1049,22 +1280,22 @@ void CSourceModParser::x_ApplyMods(CAutoInitRef<CProt_ref>& prot)
     const SMod* mod = NULL;
 
     // prot[ein]
-    if ((mod = FindMod(s_Mod_protein.Get(), s_Mod_prot.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_protein, s_Mod_prot)) != NULL) {
         prot->SetName().push_back(mod->value);
     }
 
     // prot[ein]_desc
-    if ((mod = FindMod(s_Mod_prot_desc.Get(), s_Mod_protein_desc.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_prot_desc, s_Mod_protein_desc)) != NULL) {
         prot->SetDesc( mod->value );
     }
     
     // EC_number 
-    if ((mod = FindMod(s_Mod_EC_number.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_EC_number)) != NULL) {
         prot->SetEc().push_back( mod->value );
     }
 
     // activity/function
-    if ((mod = FindMod(s_Mod_activity.Get(), s_Mod_function.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_activity, s_Mod_function)) != NULL) {
         prot->SetActivity().push_back( mod->value );
     }
 }
@@ -1075,8 +1306,8 @@ void CSourceModParser::x_ApplyMods(CAutoInitDesc<CGB_block>& gbb)
     const SMod* mod = NULL;
 
     // secondary-accession[s]
-    if ((mod = FindMod(s_Mod_secondary_accession.Get(),
-                       s_Mod_secondary_accessions.Get())) != NULL)
+    if ((mod = FindMod(s_Mod_secondary_accession,
+                       s_Mod_secondary_accessions)) != NULL)
     {
         list<CTempString> ranges;
         NStr::Split(mod->value, ",", ranges, NStr::fSplit_MergeDelimiters);
@@ -1094,7 +1325,7 @@ void CSourceModParser::x_ApplyMods(CAutoInitDesc<CGB_block>& gbb)
     }
 
     // keyword[s]
-    if ((mod = FindMod(s_Mod_keyword.Get(), s_Mod_keywords.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_keyword, s_Mod_keywords)) != NULL) {
         list<string> keywordList;
         NStr::Split(mod->value, ",;", keywordList, NStr::fSplit_MergeDelimiters);
         // trim every string and push it into the real keyword list
@@ -1111,8 +1342,8 @@ void CSourceModParser::x_ApplyMods(CAutoInitRef<CSeq_hist>& hist)
     const SMod* mod = NULL;
 
     // secondary-accession[s]
-    if ((mod = FindMod(s_Mod_secondary_accession.Get(),
-                       s_Mod_secondary_accessions.Get())) != NULL)
+    if ((mod = FindMod(s_Mod_secondary_accession,
+                       s_Mod_secondary_accessions)) != NULL)
     {
         list<CTempString> ranges;
         NStr::Split(mod->value, ",", ranges, NStr::fSplit_MergeDelimiters);
@@ -1190,7 +1421,7 @@ void CSourceModParser::x_ApplyTPAMods(CAutoInitRef<CUser_object>& tpa)
     const SMod* mod = NULL;
 
     // primary[-accessions]
-    if ((mod = FindMod(s_Mod_primary.Get(), s_Mod_primary_accessions.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_primary, s_Mod_primary_accessions)) != NULL) {
         CUser_object::TData data;
         list<CTempString> accns;
         NStr::Split(mod->value, ",", accns, NStr::fSplit_MergeDelimiters);
@@ -1210,13 +1441,115 @@ void CSourceModParser::x_ApplyTPAMods(CAutoInitRef<CUser_object>& tpa)
 }
 
 
+static CRef<CSeqdesc> s_SetDBLinkDesc(CBioseq& bioseq) 
+{
+    CConstRef<CBioseq_set> pParentSet = bioseq.GetParentSet();
+    CSeq_descr& descriptors = (pParentSet &&
+                               pParentSet->GetClass() == CBioseq_set::eClass_nuc_prot) ?
+
+        (const_cast<CBioseq_set&>(*pParentSet)).SetDescr() :
+        bioseq.SetDescr();
+
+
+    for (auto pDesc : descriptors.Set()) {
+        if (pDesc->IsUser() && pDesc->GetUser().IsDBLink()) {
+            return pDesc;
+        }
+    }
+
+    auto pDBLinkDesc = Ref(new CSeqdesc());
+    pDBLinkDesc->SetUser().SetObjectType(CUser_object::eObjectType_DBLink);
+    descriptors.Set().push_back(pDBLinkDesc);
+    return pDBLinkDesc;
+}
+
+
+static void s_SetDBLinkFieldVals(const string& label, 
+                                const list<CTempString>& vals,
+                                CSeqdesc& dblink_desc)
+{
+    if (vals.empty()) {
+        return;
+    }
+
+    auto& user_obj = dblink_desc.SetUser();
+    CRef<CUser_field> pField;
+    if (user_obj.IsSetData()) {
+        for (auto pUserField : user_obj.SetData()) {
+            if (pUserField->IsSetLabel() &&
+                pUserField->GetLabel().IsStr() &&
+                NStr::EqualNocase(pUserField->GetLabel().GetStr(), label)) {
+                pField = pUserField;
+                break;
+            }
+        }
+    }
+
+    if (!pField) {
+        pField = Ref(new CUser_field());
+        pField->SetLabel().SetStr() = label;
+        user_obj.SetData().push_back(pField);
+    }
+
+    pField->SetData().SetStrs().clear(); // RW-518 - clear any preexisting entries
+    for (const auto& val : vals) {
+        pField->SetData().SetStrs().push_back(val);
+    }
+    pField->SetNum(pField->GetData().GetStrs().size());
+}
+
+
+static void s_SetDBLinkField(const string& label,
+                             const string& vals,
+                             CRef<CSeqdesc>& pDBLinkDesc,
+                             CBioseq& bioseq) 
+{
+    list<CTempString> value_list;
+    NStr::Split(vals, ",", value_list, NStr::fSplit_MergeDelimiters);
+    for (auto& val : value_list) {
+        val = NStr::TruncateSpaces_Unsafe(val);
+    }
+    value_list.remove_if([](const CTempString& val){ return val.empty(); });
+    if (value_list.empty()) { // nothing to do
+        return;
+    }
+
+    if (!pDBLinkDesc) {
+        pDBLinkDesc =  s_SetDBLinkDesc(bioseq);
+    }
+
+    s_SetDBLinkFieldVals(label,
+                         value_list,
+                         *pDBLinkDesc);
+}
+
+
+void CSourceModParser::x_ApplyDBLinkMods(CBioseq& bioseq) 
+{
+    CRef<CSeqdesc> pDBLinkDesc;
+    const SMod* mod = NULL;
+    if ((mod = FindMod(s_Mod_SRA)) != NULL) {
+        s_SetDBLinkField("Sequence Read Archive", mod->value, pDBLinkDesc, bioseq);
+    }
+
+    if ((mod = FindMod(s_Mod_bioproject)) != NULL) {
+        s_SetDBLinkField("BioProject", mod->value, pDBLinkDesc, bioseq);
+    }
+
+    if ((mod = FindMod(s_Mod_biosample)) != NULL) {
+        s_SetDBLinkField("BioSample", mod->value, pDBLinkDesc, bioseq);
+    }
+}
+
+
+
 void
 CSourceModParser::x_ApplyGenomeProjectsDBMods(CAutoInitRef<CUser_object>& gpdb)
 {
     const SMod* mod = NULL;
 
     // project[s]
-    if ((mod = FindMod(s_Mod_project.Get(), s_Mod_projects.Get())) != NULL) {
+    if ((mod = FindMod(s_Mod_project, s_Mod_projects)) != NULL) {
         CUser_object::TData data;
         list<CTempString> ids;
         NStr::Split(mod->value, ",;", ids, NStr::fSplit_MergeDelimiters);
@@ -1262,8 +1595,8 @@ void s_ApplyPubMods(CBioseq& bioseq, const CSourceModParser::TModsRange& range)
 void CSourceModParser::ApplyPubMods(CBioseq& seq)
 {
     // find PubMed IDs
-    s_ApplyPubMods(seq, FindAllMods(s_Mod_PubMed.Get()));
-    s_ApplyPubMods(seq, FindAllMods(s_Mod_PMID.Get()));
+    s_ApplyPubMods(seq, FindAllMods(s_Mod_PubMed));
+    s_ApplyPubMods(seq, FindAllMods(s_Mod_PMID));
 }
 
 CSourceModParser::CBadModError::CBadModError( 
@@ -1287,6 +1620,23 @@ string CSourceModParser::CBadModError::x_CalculateErrorString(
     return str_strm.str();
 }
 
+CSourceModParser::CUnkModError::CUnkModError(
+    const SMod& unkMod )
+    : runtime_error(x_CalculateErrorString(unkMod)), m_UnkMod(unkMod)
+{
+}
+
+string CSourceModParser::CUnkModError::x_CalculateErrorString(
+    const SMod& unkMod)
+{
+    stringstream str_strm;
+    str_strm << "Bad modifier key at seqid '" 
+        << ( unkMod.seqid ? unkMod.seqid->AsFastaString() : "UNKNOWN")
+        << "'. '" << unkMod.key << "' is not a recognized modifier key";
+    return str_strm.str();
+}
+
+
 CSourceModParser::TMods CSourceModParser::GetMods(TWhichMods which) const
 {
     if (which == fAllMods) {
@@ -1307,23 +1657,27 @@ CSourceModParser::TMods CSourceModParser::GetMods(TWhichMods which) const
 }
 
 const CSourceModParser::SMod* CSourceModParser::FindMod(
-    const SMod & smod, const SMod & alt_smod)
+    //const SMod & smod, const SMod & alt_smod)
+    const CTempString& key, const CTempString& alt_key)
 {
     // check against m_pModFilter, if any
     if( m_pModFilter ) {
-        if( ! (*m_pModFilter)(smod.key) || ! (*m_pModFilter)(alt_smod.key) ) {
+        if( ! (*m_pModFilter)(key) || ! (*m_pModFilter)(alt_key) ) {
             return NULL;
         }
     }
 
+    SMod mod;
+
     for (int tries = 0;  tries < 2;  ++tries) {
-        const SMod & mod = ( tries == 0 ? smod : alt_smod );
-        if( mod.key.empty() ) {
+        const CTempString & modkey = ( tries == 0 ? key : alt_key );
+        if( modkey.empty() ) {
             continue;
         }
+        mod.key = modkey;
 
         TModsCI it = m_Mods.lower_bound(mod);
-        if (it != m_Mods.end()  &&  EqualKeys(it->key, mod.key)) {
+        if (it != m_Mods.end()  &&  EqualKeys(it->key, modkey)) {
             // set iterators are const since changing an object could affect
             // its order in the set.  However, in this case we know that
             // changing the `used` field won't affect the order so we know
@@ -1344,6 +1698,14 @@ CSourceModParser::FindAllMods(const CTempString& key)
     return FindAllMods(smod);
 }
 
+CSourceModParser::TModsRange
+CSourceModParser::FindAllMods(const CTempString& key, const CTempString& alt_key)
+{
+    SMod smod(key);
+    SMod alt_smod(alt_key);
+    return FindAllMods(smod, alt_smod);
+}
+
 CSourceModParser::TModsRange
 CSourceModParser::FindAllMods(const SMod & smod, const SMod & alt_smod)
 {
@@ -1502,11 +1864,70 @@ void CSourceModParser::x_HandleBadModValue(
     case eHandleBadMod_PrintToCerr:
         cerr << badModError.what() << endl;
         break;
+    case eHandleBadMod_ErrorListener: {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+                eDiag_Warning,
+                m_LineNumber,
+                badModError.what(),
+                ILineError::eProblem_GeneralParsingError) );
+        x_ProcessError(*pErr);
+        break;
+    }
     default:
         _TROUBLE;
     }
 }
 
+void CSourceModParser::x_HandleUnkModValue(
+    const SMod& mod)
+{
+    if (m_HandleBadMod == eHandleBadMod_Ignore) {
+        return;
+    }
+    if (m_pModFilter  &&  !m_pModFilter->operator()(mod.key)) {
+        return;
+    }
+    CUnkModError unkModError(mod);
+
+    switch( m_HandleBadMod ) {
+    case eHandleBadMod_Throw:
+        throw unkModError;
+    case eHandleBadMod_PrintToCerr:
+        cerr << unkModError.what() << endl;
+        break;
+    case eHandleBadMod_ErrorListener: {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+                eDiag_Warning,
+                m_LineNumber,
+                unkModError.what(),
+                ILineError::eProblem_GeneralParsingError) );
+        x_ProcessError(*pErr);
+        break;
+    }
+    default:
+        _TROUBLE;
+    }
+}
+
+void CSourceModParser::x_ProcessError(
+    CObjReaderLineException& err)
+{
+    if (!m_pErrorListener) {
+        err.Throw();
+    }
+    if (!m_pErrorListener->PutError(err)) {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+                eDiag_Critical,
+                0,
+                "Error allowance exceeded",
+                ILineError::eProblem_GeneralParsingError) );
+        pErr->Throw();
+    }
+}
+
 void CSourceModParser::ApplyMods(CBioSource& bsrc, CTempString organism)
 {
     CAutoInitDesc<CBioSource> ref(bsrc);
@@ -1541,8 +1962,8 @@ void CSourceModParser::SetAllUnused()
 
 void CSourceModParser::AddMods(const CTempString& name, const CTempString& value)
 {
-    SMod newmod(name);
-    newmod.value = value;
+    SMod newmod(NStr::TruncateSpaces_Unsafe(name));
+    newmod.value = NStr::TruncateSpaces_Unsafe(value);
     newmod.used = false;
 
     m_Mods.insert(newmod);
