@@ -1,4 +1,4 @@
-/*  $Id: ucscregion_reader.cpp 472138 2015-07-07 16:07:55Z grichenk $
+/*  $Id: ucscregion_reader.cpp 535710 2017-05-11 11:59:45Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -88,8 +88,6 @@
 #include <objtools/readers/ucscregion_reader.hpp>
 #include <objtools/error_codes.hpp>
 
-#include <objmgr/util/feature.hpp>
-
 #include <algorithm>
 
 
@@ -127,10 +125,10 @@ CRef<objects::CSeq_feat> CUCSCRegionReader::xParseFeatureUCSCFormat(const string
             string SubLine = Line.substr(IdSubI );
             //cerr << IdStr << endl;
             //cerr << SubLine << endl;
-            NStr::Tokenize(NStr::TruncateSpaces(SubLine), Delims, Tokens, NStr::eMergeDelims);
+            NStr::Split(NStr::TruncateSpaces(SubLine), Delims, Tokens, NStr::fSplit_Tokenize);
             Tokens.insert(Tokens.begin(), IdStr);
         } else {
-            NStr::Tokenize(NStr::TruncateSpaces(Line), Delims, Tokens, NStr::eMergeDelims);
+            NStr::Split(NStr::TruncateSpaces(Line), Delims, Tokens, NStr::fSplit_Tokenize);
         }
     }}
 
@@ -181,7 +179,7 @@ CRef<objects::CSeq_feat> CUCSCRegionReader::xParseFeatureUCSCFormat(const string
 //  ----------------------------------------------------------------------------
 void CUCSCRegionReader::xSmartFieldSplit(vector<string>& fields, CTempString line)
 {
-    NStr::Tokenize(line, " \t.-:", fields, NStr::eMergeDelims);
+    NStr::Split(line, " \t.-:", fields, NStr::fSplit_Tokenize);
     if (line[line.length()-1] == '-')
         fields.push_back("-");
     while (fields.size() > 3)
@@ -195,6 +193,8 @@ void CUCSCRegionReader::xSmartFieldSplit(vector<string>& fields, CTempString lin
             fields[0] += line[len];
             fields[0] += fields[1];
             fields.erase(fields.begin()+1);
+        } else {
+            break;
         }
     }
 }
