@@ -1,4 +1,4 @@
-/*  $Id: vcf_reader.cpp 539451 2017-06-22 14:57:32Z ivanov $
+/*  $Id: vcf_reader.cpp 561221 2018-04-04 13:42:04Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -161,9 +161,16 @@ ESpecNumber SpecNumber(
         return ESpecNumber( NStr::StringToInt( specnumber ) );
     }
     catch( ... ) {
-        throw "Unexpected --- ##: bad number specifier!";
-        return ESpecNumber( 0 );
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+                eDiag_Warning,
+                0,
+                "CVcfReader::xProcessMetaLineInfo: Unrecognized SpecNumber type in FORAMT directive. "
+                "Recognized settings are \'A\', \'G\', \'.\', or numeric.",
+                ILineError::eProblem_GeneralParsingError) );
+        pErr->Throw();
     }    
+    return ESpecNumber( 0 );
 };
 
 //  ----------------------------------------------------------------------------
@@ -844,12 +851,12 @@ CVcfReader::xParseData(
 
         data.m_strChrom = columns[0];
         data.m_iPos = NStr::StringToInt( columns[1] );
-        NStr::Split( columns[2], ";", data.m_Ids, NStr::eNoMergeDelims );
+        NStr::Split( columns[2], ";", data.m_Ids );
         if ( (data.m_Ids.size() == 1)  &&  (data.m_Ids[0] == ".") ) {
             data.m_Ids.clear();
         }
         data.m_strRef = columns[3];
-        NStr::Split( columns[4], ",", data.m_Alt, NStr::eNoMergeDelims );
+        NStr::Split( columns[4], ",", data.m_Alt );
         if ( columns[5] != "." ) {
             data.m_pdQual = new double( NStr::StringToDouble( columns[5] ) );
         }
