@@ -1,4 +1,4 @@
-/*  $Id: choice.cpp 497217 2016-04-05 11:37:33Z ivanov $
+/*  $Id: choice.cpp 556054 2018-01-25 17:43:38Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -132,6 +132,8 @@ void CChoiceTypeInfo::InitChoiceTypeInfoFunctions(void)
     SetSkipFunction(&TFunc::SkipChoiceDefault);
     m_SelectDelayFunction = 0;
     m_AllowEmpty = false;
+    m_WhichOffset = 0;
+    m_WhichSize = 0;
 }
 
 void CChoiceTypeInfo::AdjustChoiceTypeInfoFunctions(void)
@@ -345,6 +347,29 @@ void CChoiceTypeInfo::SetSelectDelay(TSelectDelayFunction func)
     m_SelectDelayFunction = func;
 }
 
+void CChoiceTypeInfo::SetSelectorInfo(const void* offset, size_t sz)
+{
+    m_WhichOffset = TPointerOffsetType(offset);
+    m_WhichSize = sz;
+}
+
+TMemberIndex CChoiceTypeInfo::Which(const void* object) const
+{
+    _ASSERT(m_WhichSize != 0);
+    const void* p = CRawPointer::Add(object, m_WhichOffset);
+    switch (m_WhichSize) {
+    case 1:
+        return *CTypeConverter<Uint1>::SafeCast(p);
+    case 2:
+        return *CTypeConverter<Uint2>::SafeCast(p);
+    case 4:
+        return *CTypeConverter<Uint4>::SafeCast(p);
+    default:
+        break;
+    }
+    return *CTypeConverter<Uint8>::SafeCast(p);
+}
+
 void CChoiceTypeInfo::SetDelayIndex(TObjectPtr objectPtr,
                                     TMemberIndex index) const
 {
@@ -522,7 +547,7 @@ void CChoiceTypeInfo::SetGlobalHook(const CTempString& variants,
     }
     else {
         vector<CTempString> tokens;
-        NStr::Split(variants, ",", tokens, NStr::fSplit_NoMergeDelims);
+        NStr::Split(variants, ",", tokens);
         ITERATE ( vector<CTempString>, it, tokens ) {
             const_cast<CVariantInfo*>(GetVariantInfo(*it))->
                 SetGlobalReadHook(hook);
