@@ -1,4 +1,4 @@
-/*  $Id: classinfo.cpp 532474 2017-04-05 15:51:46Z gouriano $
+/*  $Id: classinfo.cpp 548810 2017-10-18 13:38:41Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -303,12 +303,14 @@ void CClassTypeInfo::ReadImplicitMember(CObjectIStream& in,
     in.ReadNamedType(classType,
                      memberInfo->GetTypeInfo(),
                      memberInfo->GetItemPtr(objectPtr));
-    if (memberInfo->HaveSetFlag() && in.GetSpecialCaseUsed() == CObjectIStream::eReadAsNil) {
-        memberInfo->UpdateSetFlagNo(objectPtr);
-    }
-    if (memberInfo->Nillable()) {
-        in.SetMemberDefault(0);
+    if (memberInfo->HaveSetFlag()) {
+        if (in.GetSpecialCaseUsed() == CObjectIStream::eReadAsNil) {
+            memberInfo->UpdateSetFlagNo(objectPtr);
+        } else if (in.GetVerifyData() == eSerialVerifyData_Yes) {
+            memberInfo->Validate(objectPtr, in);
+        }
     }
+    in.UnsetMemberNillable();
 }
 
 void CClassTypeInfo::WriteClassRandom(CObjectOStream& out,
@@ -364,6 +366,9 @@ void CClassTypeInfo::WriteImplicitMember(CObjectOStream& out,
             }
         } 
     }
+    if (memberInfo->HaveSetFlag() && out.GetVerifyData() == eSerialVerifyData_Yes) {
+        memberInfo->Validate(objectPtr, out);
+    }
 do_write:
     out.WriteNamedType(classType,
                        memberInfo->GetTypeInfo(),
@@ -399,7 +404,7 @@ void CClassTypeInfo::CopyImplicitMember(CObjectStreamCopier& copier,
         copier.In().SetMemberNillable();
     }
     copier.CopyNamedType(classType, memberInfo->GetTypeInfo());
-    copier.In().SetMemberDefault(0);
+    copier.In().UnsetMemberNillable();
 }
 
 void CClassTypeInfo::SkipClassRandom(CObjectIStream& in,
@@ -431,7 +436,7 @@ void CClassTypeInfo::SkipImplicitMember(CObjectIStream& in,
         in.SetMemberNillable();
     }
     in.SkipNamedType(classType, memberInfo->GetTypeInfo());
-    in.SetMemberDefault(0);
+    in.UnsetMemberNillable();
 }
 
 bool CClassTypeInfo::IsDefault(TConstObjectPtr /*object*/) const
@@ -550,7 +555,7 @@ void CClassTypeInfo::SetGlobalHook(const CTempString& members,
     }
     else {
         vector<CTempString> tokens;
-        NStr::Split(members, ",", tokens, NStr::fSplit_NoMergeDelims);
+        NStr::Split(members, ",", tokens);
         ITERATE ( vector<CTempString>, it, tokens ) {
             const_cast<CMemberInfo*>(GetMemberInfo(*it))->
                 SetGlobalReadHook(hook);
