@@ -1,4 +1,4 @@
-/*  $Id: classinfob.cpp 412224 2013-09-05 15:30:00Z gouriano $
+/*  $Id: classinfob.cpp 554977 2018-01-11 14:18:53Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -123,6 +123,12 @@ void CClassTypeInfoBase::AssignItemsTags(void)
     GetItems().AssignItemsTags(GetTagType());
 }
 
+void CClassTypeInfoBase::DataSpec(EDataSpec spec)
+{
+    CParent::DataSpec(spec);
+    GetItems().DataSpec(spec);
+}
+
 void CClassTypeInfoBase::InitClassTypeInfoBase(const type_info& id)
 {
     m_Id = &id;
@@ -157,7 +163,7 @@ CClassTypeInfoBase::TClassesById& CClassTypeInfoBase::ClassesById(void)
         classes = sm_ClassesById;
         if ( !classes ) {
             const TClasses& cc = Classes();
-            auto_ptr<TClassesById> keep(classes = new TClassesById);
+            unique_ptr<TClassesById> keep(classes = new TClassesById);
             ITERATE ( TClasses, i , cc ) {
                 const CClassTypeInfoBase* info = *i;
                 if ( info->GetId() != typeid(bool) ) {
@@ -184,7 +190,7 @@ CClassTypeInfoBase::TClassesByName& CClassTypeInfoBase::ClassesByName(void)
         CMutexGuard GUARD(s_ClassInfoMutex);
         classes = sm_ClassesByName;
         if ( !classes ) {
-            auto_ptr<TClassesByName> keep(classes = new TClassesByName);
+            unique_ptr<TClassesByName> keep(classes = new TClassesByName);
             const TClasses& cc = Classes();
             ITERATE ( TClasses, i, cc ) {
                 const CClassTypeInfoBase* info = *i;
@@ -365,7 +371,7 @@ public:
 
     CPreReadHook(TPreReadFunction func);
 
-    void ReadObject(CObjectIStream& in, const CObjectInfo& object);
+    virtual void ReadObject(CObjectIStream& in, const CObjectInfo& object) override;
 
 private:
     TPreReadFunction m_PreRead;
@@ -391,7 +397,7 @@ public:
 
     CPostReadHook(TPostReadFunction func);
 
-    void ReadObject(CObjectIStream& in, const CObjectInfo& object);
+    virtual void ReadObject(CObjectIStream& in, const CObjectInfo& object) override;
 
 private:
     TPostReadFunction m_PostRead;
@@ -417,7 +423,7 @@ public:
 
     CPreWriteHook(TPreWriteFunction func);
 
-    void WriteObject(CObjectOStream& out, const CConstObjectInfo& object);
+    virtual void WriteObject(CObjectOStream& out, const CConstObjectInfo& object) override;
 
 private:
     TPreWriteFunction m_PreWrite;
@@ -443,7 +449,7 @@ public:
 
     CPostWriteHook(TPostWriteFunction func);
 
-    void WriteObject(CObjectOStream& out, const CConstObjectInfo& object);
+    virtual void WriteObject(CObjectOStream& out, const CConstObjectInfo& object) override;
 
 private:
     TPostWriteFunction m_PostWrite;
