@@ -1,4 +1,4 @@
-/*  $Id: blocktype.cpp 485707 2015-11-25 13:58:19Z gouriano $
+/*  $Id: blocktype.cpp 569882 2018-08-30 16:58:39Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -115,6 +115,134 @@ void CDataMemberContainerType::PrintSpecDumpExtra(CNcbiOstream& out, int indent)
     m_LastComments.PrintASN(out, indent, CComments::eNoEOL);
 }
 
+void CDataMemberContainerType::PrintJSONSchema(CNcbiOstream& out, int indent, list<string>& required, bool contents_only) const
+{
+    const CDataMember* data = GetDataMember();
+    bool hasNotag = data && data->Notag();
+    list<string> this_req;
+    list<string>& req = (hasNotag || contents_only) ? required : this_req;
+    bool hasAttlist = find_if(m_Members.begin(), m_Members.end(),
+        [](const TMembers::value_type& e) { return e->Attlist();}) != m_Members.end();
+    bool first = true;
+
+    if (!contents_only && m_Members.size()==1 && 
+            (m_Members.front()->GetType()->IsContainer() || m_Members.front()->GetType()->IsUniSeq())) {
+        m_Members.front()->PrintJSONSchema(out, indent, required);
+        return;
+    }
+
+#if 0
+// allOf is kind of... wrong...
+// it is impossible to restrict additionalProperties
+// so, instance validation either fails or allows undefined elements
+// I do not know...  
+// I leave this code for future, in a hope that JSON schema will be improved.
+
+    bool hasUnnamedContainer = find_if(m_Members.begin(), m_Members.end(),
+        [](const TMembers::value_type& e) { return e->Notag() && e->GetType()->IsContainer();}) != m_Members.end();
+    if (hasUnnamedContainer) {
+        first = true;
+        PrintASNNewLine(out, indent++) << "\"definitions\": {";
+        ITERATE ( TMembers, i, m_Members ) {
+            const CDataMember& member = **i;
+            if (member.Notag() && member.GetType()->IsContainer()) {
+                if (!first) {
+                    out << ",";
+                } else {
+                    first = false;
+                }
+                PrintASNNewLine(out, indent++) << "\"" << "$" << member.GetName() << "\": {";
+                member.PrintJSONSchema(out, indent, req, true);
+                PrintASNNewLine(out, --indent) << "}";
+            }
+        }
+        PrintASNNewLine(out, --indent) << "},";
+        PrintASNNewLine(out, indent++) << "\"allOf\": [";
+        first = true;
+        ITERATE ( TMembers, i, m_Members ) {
+            const CDataMember& member = **i;
+            if (member.Notag() && member.GetType()->IsContainer()) {
+                if (!first) {
+                    out << ",";
+                } else {
+                    first = false;
+                }
+                PrintASNNewLine(out, indent) << "{\"$ref\": \"#/definitions/" << GetMemberName() << "/definitions/$" << member.GetName() << "\"}";
+            }
+        }
+        out << ",";
+        PrintASNNewLine(out, indent++) << "{";
+    }
+#endif
+
+    if (!contents_only) {
+        PrintASNNewLine(out, indent) << "\"type\": \"object\",";
+        PrintASNNewLine(out, indent++) << "\"properties\": {";
+    }
+
+    first = true;
+    ITERATE ( TMembers, i, m_Members ) {
+        const CDataMember& member = **i;
+        if (hasAttlist && member.Notag() && dynamic_cast<const CNullDataType*>(member.GetType()) != nullptr) {
+// special case: null with attributes
+            continue;
+        }
+#if 0
+        if (member.Notag() && member.GetType()->IsContainer()) {
+            continue;
+        }
+#endif
+        if (!first) {
+            out << ",";
+        } else {
+            first = false;
+        }
+        if (member.Attlist()) {
+            member.PrintJSONSchema(out, indent, req, true);
+        }
+        else if (member.Notag()) {
+            if (member.GetType()->IsContainer()) {
+                member.PrintJSONSchema(out, indent, req, true);
+            } else {
+                PrintASNNewLine(out, indent++) << "\"" << "#" << member.GetName() << "\": {";
+                member.PrintJSONSchema(out, indent, req);
+                PrintASNNewLine(out, --indent) << "}";
+            }
+        }
+        else { //if (!member.Notag() && !member.Attlist()) {
+            PrintASNNewLine(out, indent++) << "\"" << member.GetName() << "\": {";
+            member.PrintJSONSchema(out, indent, req);
+            PrintASNNewLine(out, --indent) << "}";
+        }
+    }
+
+    if (!contents_only) {
+        PrintASNNewLine(out, --indent) << "}";
+        if (!req.empty()) {
+            out << ',';
+            PrintASNNewLine(out, indent) << "\"required\": [";
+            transform(req.begin(), req.end(), Dt_ostream_iterator<string>(out, ", "),
+                [](const string& e) {
+                    return string("\"").append(e).append("\"");
+                });
+            req.clear();
+            out << "]";
+        }
+        out << ",";
+        PrintASNNewLine(out, indent) << "\"additionalProperties\": false";
+    }
+
+#if 0
+    if (hasUnnamedContainer) {
+        PrintASNNewLine(out, --indent) << "}";
+        PrintASNNewLine(out, --indent) << "]";
+    }
+#endif
+    if (data && !data->Notag() && !data->Optional() && !data->Attlist()) {
+        required.push_back(data->GetName());
+    }
+}
+
 // XML schema generator submitted by
 // Marc Dumontier, Blueprint initiative, dumontier@mshri.on.ca
 // modified by Andrei Gourianov, gouriano@ncbi
@@ -129,6 +257,8 @@ void CDataMemberContainerType::PrintXMLSchema(CNcbiOstream& out,
     bool isSimple= false, isSimpleSeq= false, isSeq= false, isMixed=false;
     bool isSimpleContainer= false, parent_isSeq= false;
     bool defineAsType = false;
+    bool isGlobalType  = GetGlobalType() == CDataType::eType;
+    bool isGlobalGroup = GetGlobalType() == CDataType::eGroup;
     string simpleType;
     list<string> opentag, closetag1, closetag2;
     CNcbiOstream* os = &out;
@@ -244,73 +374,81 @@ void CDataMemberContainerType::PrintXMLSchema(CNcbiOstream& out,
     if (!isAttlist && !parent_isSeq) {
         if (!hasNotag) {
             if (!contents_only) {
-                tmp = "<xs:element name=\"" + tag + "\"";
-                if (isOptionalMember) {
-                    tmp += " minOccurs=\"0\"";
-                }
-                if (IsNillable()) {
-                    tmp += " nillable=\"true\"";
-                }
                 string tname;
-                if (defineAsType) {
-                    const CDataType* par = GetParentType();
-                    while ( par->GetParentType() ) {
-                        par = par->GetParentType();
-                    }
-                    tname = par->GetMemberName() + tag + "_Type";
-                    tmp += " type=\"" + tname + "\"/>";
-                    PrintASNNewLine(out, indent) << tmp;
+                if (isGlobalType || isGlobalGroup) {
+                    tname = tag;
                 } else {
-#if _DATATOOL_USE_SCHEMA_STYLE_COMMENTS
-                    PrintASNNewLine(out, indent) << tmp;
-                    if (!Comments().PrintSchemaComments(out, indent+1)) {
-                        out << '>';
+                    tmp = "<xs:element name=\"" + tag + "\"";
+                    if (isOptionalMember) {
+                        tmp += " minOccurs=\"0\"";
                     }
-                    ++indent;
+                    if (IsNillable()) {
+                        tmp += " nillable=\"true\"";
+                    }
+                    if (defineAsType) {
+                        const CDataType* par = GetParentType();
+                        while ( par->GetParentType() ) {
+                            par = par->GetParentType();
+                        }
+                        tname = par->GetMemberName() + tag + "_Type";
+                        tmp += " type=\"" + tname + "\"/>";
+                        PrintASNNewLine(out, indent) << tmp;
+                    } else {
+#if _DATATOOL_USE_SCHEMA_STYLE_COMMENTS
+                        PrintASNNewLine(out, indent) << tmp;
+                        if (!Comments().PrintSchemaComments(out, indent+1)) {
+                            out << '>';
+                        }
+                        ++indent;
 #else
-                    tmp += ">";
-                    opentag.push_back(tmp);
+                        tmp += ">";
+                        opentag.push_back(tmp);
 #endif
-                    closetag2.push_front("</xs:element>");
+                        closetag2.push_front("</xs:element>");
+                    }
                 }
 
-                tmp = "<xs:complexType";
+                tmp = isGlobalGroup ? "<xs:group" : "<xs:complexType";
                 if (isMixed) {
                     tmp += " mixed=\"true\"";
                 }
-                if (defineAsType) {
+                if (defineAsType || isGlobalType || isGlobalGroup) {
                     tmp += " name=\"" + tname + "\"";
-                    os = &otype;
+                    if (!isGlobalType && !isGlobalGroup) {
+                        os = &otype;
+                    }
                     indent = 0;
                 }
                 opentag.push_back(tmp + ">");
-                closetag2.push_front("</xs:complexType>");
+                closetag2.push_front(isGlobalGroup ? "</xs:group" : "</xs:complexType>");
             }
         }
-        if (!isSimple && !isSimpleContainer) {
-            if (!contents_only || hasNotag) {
-                if(NStr::CompareCase(asnk,"CHOICE")==0) {
-                    xsdk = "choice";
-                } else if(NStr::CompareCase(asnk,"SEQUENCE")==0) {
-                    xsdk = "sequence";
-                } else if(NStr::CompareCase(asnk,"SET")==0) {
-                    xsdk = "all";
+        if (!isGlobalGroup) {
+            if (!isSimple && !isSimpleContainer) {
+                if (!contents_only || hasNotag) {
+                    if(NStr::CompareCase(asnk,"CHOICE")==0) {
+                        xsdk = "choice";
+                    } else if(NStr::CompareCase(asnk,"SEQUENCE")==0) {
+                        xsdk = "sequence";
+                    } else if(NStr::CompareCase(asnk,"SET")==0) {
+                        xsdk = "all";
+                    }
+                    tmp = "<xs:" + xsdk;
+                    if (isOptionalContent || (hasNotag && isOptionalMember)) {
+                        tmp += " minOccurs=\"0\"";
+                    }
+                    if (isSeq) {
+                        tmp += " maxOccurs=\"unbounded\"";
+                    }
+                    opentag.push_back(tmp + ">");
+                    closetag1.push_front("</xs:" + xsdk + ">");
                 }
-                tmp = "<xs:" + xsdk;
-                if (isOptionalContent || (hasNotag && isOptionalMember)) {
-                    tmp += " minOccurs=\"0\"";
-                }
-                if (isSeq) {
-                    tmp += " maxOccurs=\"unbounded\"";
-                }
-                opentag.push_back(tmp + ">");
-                closetag1.push_front("</xs:" + xsdk + ">");
+            } else if (!simpleType.empty()) {
+                opentag.push_back("<xs:simpleContent>");
+                closetag2.push_front("</xs:simpleContent>");
+                opentag.push_back("<xs:extension base=\"" + simpleType + "\">");
+                closetag2.push_front("</xs:extension>");
             }
-        } else if (!simpleType.empty()) {
-            opentag.push_back("<xs:simpleContent>");
-            closetag2.push_front("</xs:simpleContent>");
-            opentag.push_back("<xs:extension base=\"" + simpleType + "\">");
-            closetag2.push_front("</xs:extension>");
         }
     }
     ITERATE ( list<string>, s, opentag ) {
@@ -624,8 +762,8 @@ CClassTypeInfo* CDataContainerType::CreateClassInfo(void)
         if ( mem->Optional() || mem->GetDefault() || mem->Nillable() )
             ++itemCount;
     }
-    auto_ptr<CAnyTypeClassInfo> typeInfo(new CAnyTypeClassInfo(GlobalName(),
-                                                               itemCount));
+    unique_ptr<CAnyTypeClassInfo> typeInfo(new CAnyTypeClassInfo(
+        HasExternalName() ? GlobalName() : kEmptyStr, itemCount));
     size_t index = 0;
     for ( TMembers::const_iterator i = GetMembers().begin();
           i != GetMembers().end(); ++i ) {
@@ -645,7 +783,7 @@ CClassTypeInfo* CDataContainerType::CreateClassInfo(void)
         if ( mem->Optional() || mem->GetDefault() || mem->Nillable() ) {
             memInfo->SetSetFlag(typeInfo->GetSetFlagPtr(index++));
         }
-        if (mem->NoPrefix()) {
+        if (!IsASNDataSpec()) {
             memInfo->SetNoPrefix();
         }
         if (mem->Attlist()) {
@@ -736,7 +874,7 @@ AutoPtr<CTypeStrings> CDataContainerType::AddMembers(
                         (*i)->GetType()->GetVar("_pointer"),
                         optional, defaultCode, delayed,
                         (*i)->GetType()->GetTag(),
-                        (*i)->NoPrefix(), (*i)->Attlist(), (*i)->Notag(),
+                        !IsASNDataSpec(), (*i)->Attlist(), (*i)->Notag(),
                         (*i)->SimpleType(),(*i)->GetType(),false,
                         (*i)->Comments());
         (*i)->GetType()->SetTypeStr(&(*code));
@@ -993,6 +1131,11 @@ void CDataMember::PrintSpecDump(CNcbiOstream& out, int indent, const char* tag)
     }
 }
 
+void CDataMember::PrintJSONSchema(CNcbiOstream& out, int indent, list<string>& required, bool contents_only) const
+{
+    GetType()->PrintJSONSchema(out, indent, required, contents_only);
+}
+
 void CDataMember::PrintXMLSchema(CNcbiOstream& out, int indent, bool contents_only) const
 {
 #if !_DATATOOL_USE_SCHEMA_STYLE_COMMENTS
@@ -1027,7 +1170,7 @@ void CDataMember::SetOptional(void)
 
 void CDataMember::SetNoPrefix(void)
 {
-    m_NoPrefix = true;
+//    m_NoPrefix = true;
 }
 
 void CDataMember::SetAttlist(void)
