@@ -1,4 +1,4 @@
-/*  $Id: datatool.cpp 532212 2017-04-03 13:24:13Z gouriano $
+/*  $Id: datatool.cpp 554977 2018-01-11 14:18:53Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -91,13 +91,14 @@ int CDataTool::Run(void)
 CDataTool::CDataTool(void)
 {
     SetVersion( CVersionInfo(DATATOOL_VERSION_MAJOR,DATATOOL_VERSION_MINOR,DATATOOL_VERSION_PATCH) );
+    m_codestyle = 0;
 }
 
 void CDataTool::Init(void)
 {
     SetDiagPostLevel(eDiag_Warning);
 
-    auto_ptr<CArgDescriptions> d(new CArgDescriptions);
+    unique_ptr<CArgDescriptions> d(new CArgDescriptions);
 
     d->SetUsageContext("datatool", "work with ASN.1/XML data");
 
@@ -121,6 +122,9 @@ void CDataTool::Init(void)
     d->AddOptionalKey("fxs", "XMLSchemaFile",
                       "write XML Schema file (\"-fxs m\" writes modular Schema file)",
                       CArgDescriptions::eOutputFile);
+    d->AddOptionalKey("fjs", "JSONSchemaFile",
+                      "write JSON Schema file",
+                      CArgDescriptions::eOutputFile);
     d->AddOptionalKey("fd", "SpecificationDump",
                       "write specification dump file (datatool internal format)",
                       CArgDescriptions::eOutputFile);
@@ -259,6 +263,30 @@ bool CDataTool::ProcessModules(void)
     list<string> modulesPath;
     string opt;
 
+    m_codestyle = 0;
+    string style = GetConfigValue("-", "CodeGenerationStyle");
+    if (!style.empty()) {
+        vector<string> values;
+        NStr::Split(style, " ,", values, NStr::fSplit_Tokenize);
+        for ( const string& v : values) {
+            if (v == "#") {
+                break;
+            } else if (NStr::CompareNocase(v,"no_globaltypes")==0) {
+                m_codestyle |= FCodeGenerationStyle(eNoGlobalTypeClasses);
+            } else if (NStr::CompareNocase(v,"no_globalgroups")==0) {
+                m_codestyle |= FCodeGenerationStyle(eNoGlobalGroupClasses);
+            } else if (NStr::CompareNocase(v,"keep_nestedelem")==0) {
+                m_codestyle |= FCodeGenerationStyle(ePreserveNestedElements);
+            } else if (NStr::CompareNocase(v,"xmlelem_enums")==0) {
+                m_codestyle |= FCodeGenerationStyle(eXmlElementEnums);
+            } else if (NStr::CompareNocase(v,"no_restrictions")==0) {
+                m_codestyle |= FCodeGenerationStyle(eNoRestrictions);
+            } else {
+                ERR_POST_X(1, Warning << "Unknown code generation value: " << v);
+            }
+        }
+    }
+
     if ( generator.GetOpt("oR", &opt) ) {
         // NCBI directory tree
         const string& rootDir = opt;
@@ -331,6 +359,11 @@ bool CDataTool::ProcessModules(void)
         }
     }
 
+    if ( const CArgValue& ax = args["fjs"] ) {
+        generator.GetMainModules().PrintJSONSchema(ax.AsOutputFile());
+        ax.CloseFile();
+    }
+
     if ( !generator.Check() ) {
         if ( !args["i"] ) { // ignored
             ERR_POST_X(1, "some types are unknown");
@@ -383,7 +416,7 @@ bool CDataTool::ProcessData(void)
     else // no input data
         return true;
 
-    auto_ptr<CObjectIStream>
+    unique_ptr<CObjectIStream>
         in(CObjectIStream::Open(inFormat, inFileName, eSerial_StdWhenAny));
     if (inFormat == eSerial_Xml) {
         CObjectIStreamXml *is = dynamic_cast<CObjectIStreamXml*>(in.get());
@@ -403,6 +436,9 @@ bool CDataTool::ProcessData(void)
     }
     else {
         typeName = in->ReadFileHeader();
+        if (typeName.empty() && CDataType::GetSourceDataSpec() == EDataSpec::eJSON) {
+            typeName = "JsonValue";
+        }
     }
 
     bool type_guessed = false;
@@ -484,7 +520,7 @@ bool CDataTool::ProcessData(void)
     bool readFull = bool(args["F"]);
     if ( outFormat != eSerial_None ) {
         // copy
-        auto_ptr<CObjectOStream>
+        unique_ptr<CObjectOStream>
             out(CObjectOStream::Open(outFormat, outFileName,
                                         eSerial_StdWhenAny));
         if ( outFormat == eSerial_Xml ) {
@@ -579,6 +615,7 @@ bool CDataTool::ProcessData(void)
 bool CDataTool::GenerateCode(bool undo)
 {
     string opt;
+
     //if ( const CArgValue& oD = args["oD"] )
     //    generator.AddConfigLine(oD.AsString());
 
