@@ -1,4 +1,4 @@
-/*  $Id: dtdparser.cpp 534303 2017-04-26 13:43:42Z gouriano $
+/*  $Id: dtdparser.cpp 550286 2017-11-01 15:49:17Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -340,6 +340,7 @@ void DTDParser::ParseElementContent(const string& name, bool embedded)
         ConsumeElementContent(node);
         if (embedded) {
             node.SetEmbedded();
+            node.SetNamed(false);
             return;
         }
         break;
@@ -579,6 +580,9 @@ string DTDParser::CreateEmbeddedName(const DTDElement& node, int depth) const
 
 void DTDParser::FixEmbeddedNames(DTDElement& node)
 {
+    if (node.GetType() == DTDElement::eEnum || node.GetType() == DTDElement::eIntEnum) {
+        return;
+    }
     const list<string>& refs = node.GetContent();
     list<string> fixed;
     for (list<string>::const_iterator i= refs.begin(); i != refs.end(); ++i) {
@@ -939,6 +943,7 @@ AutoPtr<CDataType> DTDParser::Type(
     AutoPtr<CDataType> type(x_Type(node, occ, fromInside, ignoreAttrib));
     if (m_SrcType != eDTD && !fromInside) {
         type->Comments() = node.GetComments();
+        type->SetEmptyExternalName( !node.IsNamed());
     }
     return type;
 }
@@ -1056,8 +1061,13 @@ CDataType* DTDParser::x_Type(
                     }
                     base = b->second.GetTypeName();
                 }
-                type = new CReferenceDataType(base);
-                type->SetIsAlias(true);
+                if (node.IsNillable()) {
+                    type = NillableBlock(node,ignoreAttrib);
+                } else {
+                    type = new CReferenceDataType(base);
+                    type->SetIsAlias(true);
+                    type->SetIsTypeAlias(true);
+                }
             }
             break;
         case DTDElement::eEnum:
@@ -1126,6 +1136,7 @@ CDataType* DTDParser::x_Type(
     type->SetSourceLine(node.GetSourceLine());
     type->SetNamespaceName( node.GetNamespaceName());
     type->SetNsQualified( node.IsQualified());
+    type->SetGlobalType( node.IsGlobalType() ? CDataType::eType : (node.IsGlobalGroup() ? CDataType::eGroup : CDataType::eElement));
     if (node.IsNillable()) {
         type->SetNillable();
     }
@@ -1148,7 +1159,9 @@ CDataType* DTDParser::x_Type(
         container->AddMember(member);
         type = container.release();
     }
-
+    if (!fromInside) {
+        type->SetRestrictions(node.GetRestrictions());
+    }
     if (!embtype.empty()) {
         m_ElementEmbTypes.pop_back();
     }
@@ -1178,6 +1191,8 @@ AutoPtr<CDataValue> DTDParser::x_Value(const DTDElement& node)
     switch (node.GetType()) {
     default:
         break;;
+    case DTDElement::eEnum:
+        return AutoPtr<CDataValue>(new CIdDataValue(node.GetDefault()));
     case DTDElement::eString:
         return AutoPtr<CDataValue>(new CStringDataValue(node.GetDefault()));
     case DTDElement::eOctetString:
@@ -1196,6 +1211,7 @@ AutoPtr<CDataValue> DTDParser::x_Value(const DTDElement& node)
             }
             return AutoPtr<CDataValue>(new CBoolDataValue(b));
         }
+    case DTDElement::eIntEnum:
     case DTDElement::eInteger:
     case DTDElement::eBigInt:
         return AutoPtr<CDataValue>(new CIntDataValue(
@@ -1308,7 +1324,7 @@ CDataType* DTDParser::TypesBlock(
             (occ == DTDElement::eZeroOrMore)) {
             member->SetOptional();
         }
-        if (refNode.IsEmbedded() && !refNode.IsNamed()) {
+        if (!refNode.IsNamed()) {
             member->SetNotag();
         }
         if (!refNode.GetDefault().empty() && !refNode.HasAttributes()) {
@@ -1321,6 +1337,9 @@ CDataType* DTDParser::TypesBlock(
         if (m_SrcType == eDTD || refNode.IsEmbedded()) {
             member->Comments() = refNode.GetComments();
         }
+        if (refNode.IsEmbedded()) {
+            member->SetRestrictions( refNode.GetRestrictions());
+        }
         container->AddMember(member);
     }
     if (m_SrcType == eDTD || node.IsEmbedded()) {
