@@ -1,4 +1,4 @@
-/*  $Id: generate.cpp 554977 2018-01-11 14:18:53Z gouriano $
+/*  $Id: generate.cpp 581625 2019-03-04 16:47:32Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -972,8 +972,9 @@ void CCodeGenerator::GenerateClientCode(const string& name, bool mandatory)
         return; // not configured
     }
     CFileCode code(this,Path(m_FileNamePrefix, name));
+    CClientPseudoDataType type(*this, name, class_name);
     code.UseQuotedForm(m_UseQuotedForm);
-    code.AddType(new CClientPseudoDataType(*this, name, class_name));
+    code.AddType(&type);
     code.GenerateCode();
     string filename;
     code.GenerateHPP(m_HPPDir, filename);
