@@ -1,4 +1,4 @@
-/*  $Id: jsdparser.cpp 569883 2018-08-30 16:59:22Z ivanov $
+/*  $Id: jsdparser.cpp 580649 2019-02-19 12:35:09Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -547,6 +547,8 @@ void JSDParser::SkipUnknown(TToken tokend)
             SkipUnknown(K_END_ARRAY);
         } else if (tok == K_BEGIN_OBJECT) {
             SkipUnknown(K_END_OBJECT);
+        } else if (tok == T_EOF) {
+            ParseError("Unexpected end-of-file", tokend == K_END_ARRAY ? "end-of-array" : "end-of-object");
         }
     }
 }
