@@ -1,4 +1,4 @@
-/*  $Id: parser.cpp 507796 2016-07-21 17:25:37Z gouriano $
+/*  $Id: parser.cpp 573070 2018-10-22 18:55:59Z fukanchi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -246,6 +246,9 @@ CDataType* ASNParser::x_Type(void)
     case K_BIT:
         Consume();
         Consume(K_STRING, "STRING");
+        if ( CheckSymbol('{') ) {
+            return new CBitStringDataType(EnumeratedBlock(new CIntEnumDataType()));
+        }
         return new CBitStringDataType();
     case K_OCTET:
         Consume();
