@@ -1,7 +1,7 @@
 #ifndef TYPESTR_HPP
 #define TYPESTR_HPP
 
-/*  $Id: typestr.hpp 507796 2016-07-21 17:25:37Z gouriano $
+/*  $Id: typestr.hpp 547690 2017-10-03 13:59:36Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -42,6 +42,7 @@ BEGIN_NCBI_SCOPE
 class CClassContext;
 class CNamespace;
 class CDataType;
+enum class ESerialFacet;
 
 class CTypeStrings {
 public:
@@ -144,7 +145,7 @@ public:
     void SetSpecialRef(const string& ref) {
         m_SpecialRef = ref;
     }
-
+    static const char* GetFacetString(ESerialFacet c);
 private:
     string m_ModuleName;
     string m_NamespaceName;
