@@ -1,4 +1,4 @@
-/*  $Id: unitype.cpp 532212 2017-04-03 13:24:13Z gouriano $
+/*  $Id: unitype.cpp 547959 2017-10-06 12:34:54Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -99,6 +99,73 @@ void CUniSequenceDataType::PrintSpecDumpExtra(CNcbiOstream& out, int indent) con
     GetElementType()->Comments().PrintASN(out, indent, CComments::eNoEOL);
 }
 
+void CUniSequenceDataType::PrintJSONSchema(CNcbiOstream& out, int indent, list<string>& required, bool) const
+{
+    PrintASNNewLine(out, indent) << "\"type\": \"array\",";
+    PrintASNNewLine(out, indent++) << "\"items\": {";
+    GetElementType()->PrintJSONSchema(out, indent, required);
+    const CDataMember* mem = GetDataMember();
+    if (mem) {
+        const list<CMemberFacet>& con = mem->GetRestrictions();
+        string pattern;
+        for (const CMemberFacet& f : con) {
+            string fname;
+            switch (f.GetType()) {
+            case ESerialFacet::eMinLength:
+                out << ",";  PrintASNNewLine(out, indent) << "\"minLength\": " << f.GetValue();
+                break;
+            case ESerialFacet::eMaxLength:
+                out << ",";  PrintASNNewLine(out, indent) << "\"maxLength\": " << f.GetValue();
+                break;
+            case ESerialFacet::eExclusiveMinimum:
+                out << ",";  PrintASNNewLine(out, indent) << "\"exclusiveMinimum\": " << f.GetValue();
+                break;
+            case ESerialFacet::eInclusiveMinimum:
+                out << ",";  PrintASNNewLine(out, indent) << "\"minimum\": " << f.GetValue();
+                break;
+            case ESerialFacet::eExclusiveMaximum:
+                out << ",";  PrintASNNewLine(out, indent) << "\"exclusiveMaximum\": " << f.GetValue();
+                break;
+            case ESerialFacet::eInclusiveMaximum:
+                out << ",";  PrintASNNewLine(out, indent) << "\"maximum\": " << f.GetValue();
+                break;
+            case ESerialFacet::eMultipleOf:
+                out << ",";  PrintASNNewLine(out, indent) << "\"multipleOf\": " << f.GetValue();
+                break;
+            case ESerialFacet::ePattern:
+                if (!pattern.empty()) {
+                    pattern.append("|");
+                }
+                pattern.append(f.GetValue());
+                break;
+            default: break;
+            }
+        }
+        if (!pattern.empty()) {
+            out << ",";  PrintASNNewLine(out, indent) << "\"pattern\": \"" << pattern << "\"";
+        }
+    }
+    PrintASNNewLine(out, --indent) << "}";
+    if (mem) {
+        const list<CMemberFacet>& con = mem->GetRestrictions();
+        for (const CMemberFacet& f : con) {
+            string fname;
+            switch (f.GetType()) {
+            case ESerialFacet::eMinItems:
+                out << ",";  PrintASNNewLine(out, indent) << "\"minItems\": " << f.GetValue();
+                break;
+            case ESerialFacet::eMaxItems:
+                out << ",";  PrintASNNewLine(out, indent) << "\"maxItems\": " << f.GetValue();
+                break;
+            case ESerialFacet::eUniqueItems:
+                out << ",";  PrintASNNewLine(out, indent) << "\"uniqueItems\": true";
+                break;
+            default: break;
+            }
+        }
+    }
+}
+
 // XML schema generator submitted by
 // Marc Dumontier, Blueprint initiative, dumontier@mshri.on.ca
 // modified by Andrei Gourianov, gouriano@ncbi
@@ -126,15 +193,18 @@ void CUniSequenceDataType::PrintXMLSchema(CNcbiOstream& out,
             }
         }
     }
+    bool hasFacets = !hasAttlist && GetParentType() && GetDataMember() && !GetDataMember()->GetRestrictions().empty();
 
     string tag(XmlTagName());
     string userType = typeRef ? typeRef->UserTypeXmlTagName() : typeElem->XmlTagName();
+    bool isTypeAlias = typeElem->IsTypeAlias();
+    bool isGlobalGroup = typeElem->GetGlobalType() == CDataType::eGroup;
 
     if ( GetEnforcedStdXml() && (typeStatic || (typeRef && tag == userType))) {
         string type(typeStatic ? typeStatic->GetSchemaTypeString() : typeElem->GetSchemaTypeString());
         bool any = dynamic_cast<const CAnyContentDataType*>(typeStatic) != 0;
         PrintASNNewLine(out, indent++);
-        string xsdk("element");
+        string xsdk(isGlobalGroup ? "group" : "element");
         if (any) {
             xsdk = "any";
             out << "<xs:any processContents=\"lax\"";
@@ -143,24 +213,40 @@ void CUniSequenceDataType::PrintXMLSchema(CNcbiOstream& out,
                 out << " namespace=\"" << ns << "\"";
             }
         } else {
-            out << "<xs:element ";
+            out << "<xs:" << xsdk;
             if (typeRef) {
-                out << "ref";
+                out << " ref";
             } else {
-                out << "name";
+                out << " name";
             }
             out << "=\"" << tag << "\"";
-            if (typeStatic && !type.empty() && !hasAttlist) {
+            if (typeStatic && !type.empty() && !hasAttlist && !hasFacets) {
                 out << " type=\"" << type << "\"";
             }
         }
         if (GetParentType()) {
             bool isOptional = GetDataMember() ?
                 GetDataMember()->Optional() : !IsNonEmpty();
-            if (isOptional) {
+            bool found_min = false, found_max = false;
+            if (hasFacets) {
+                const list<CMemberFacet>& con = GetDataMember()->GetRestrictions();
+                list<CMemberFacet>::const_iterator f = find_if(con.begin(), con.end(), [](const CMemberFacet& i) { return i.GetType() == ESerialFacet::eMinItems;});
+                if (f != con.end()) {
+                    out << " minOccurs=\"" << f->GetValue() << "\"";
+                    found_min = true;
+                }
+                f = find_if(con.begin(), con.end(), [](const CMemberFacet& i) { return i.GetType() == ESerialFacet::eMaxItems;});
+                if (f != con.end()) {
+                    out << " maxOccurs=\"" << f->GetValue() << "\"";
+                    found_max = true;
+                }
+            }
+            if (isOptional && !found_min) {
                 out << " minOccurs=\"0\"";
             }
-            out << " maxOccurs=\"unbounded\"";
+            if (!found_max) {
+                out << " maxOccurs=\"unbounded\"";
+            }
             if (typeStatic && typeStatic->GetDataMember() && typeStatic->GetDataMember()->Nillable()) {
                 out <<  " nillable=\"true\"";
             }
@@ -181,6 +267,32 @@ void CUniSequenceDataType::PrintXMLSchema(CNcbiOstream& out,
             PrintASNNewLine(out, --indent) << "</xs:simpleContent>";
             PrintASNNewLine(out, --indent) << "</xs:complexType>";
             PrintASNNewLine(out, --indent) << "</xs:element>";
+        } else if (hasFacets) {
+            out << ">";
+            PrintASNNewLine(out, indent++) << "<xs:simpleType>";
+            PrintASNNewLine(out, indent++) << "<xs:restriction base=\"" << type << "\">";
+            const list<CMemberFacet>& con = GetDataMember()->GetRestrictions();
+            for (const CMemberFacet& f : con) {
+                string fname;
+                switch (f.GetType()) {
+                case ESerialFacet::eMinLength:         fname = "minLength";    break;
+                case ESerialFacet::eMaxLength:         fname = "maxLength";    break;
+                case ESerialFacet::eLength:            fname = "length";       break;
+                case ESerialFacet::ePattern:           fname = "pattern";      break;
+                case ESerialFacet::eInclusiveMinimum:  fname = "minInclusive"; break;
+                case ESerialFacet::eExclusiveMinimum:  fname = "minExclusive"; break;
+                case ESerialFacet::eInclusiveMaximum:  fname = "maxInclusive"; break;
+                case ESerialFacet::eExclusiveMaximum:  fname = "maxExclusive"; break;
+                default: break;
+                }
+                if (!fname.empty()) {
+                    PrintASNNewLine(out, indent) << "<xs:" << fname << " value=\"" << f.GetValue() << "\"/>";
+                }
+            }
+            PrintASNNewLine(out, --indent) << "</xs:restriction>";
+            PrintASNNewLine(out, --indent) << "</xs:simpleType>";
+            PrintASNNewLine(out, --indent) << "</xs:" << xsdk << ">";
+            return;
         } else {
 #if _DATATOOL_USE_SCHEMA_STYLE_COMMENTS
             if (GetDataMember() && GetDataMember()->GetComments().PrintSchemaComments(out, indent)) {
@@ -257,6 +369,9 @@ void CUniSequenceDataType::PrintXMLSchema(CNcbiOstream& out,
                 xsdk = "all";
             }
             string tmp = "<xs:element name=\"" + tag + "\"";
+            if (isTypeAlias) {
+                tmp += " type=\"" + userType + "\"";
+            }
             if (GetDataMember()) {
                 if (GetDataMember()->Optional()) {
                     tmp += " minOccurs=\"0\"";
@@ -268,6 +383,10 @@ void CUniSequenceDataType::PrintXMLSchema(CNcbiOstream& out,
                     tmp += " nillable=\"true\"";
                 }
             }
+            if (isTypeAlias) {
+                PrintASNNewLine(out, indent) << tmp << "/>";
+                return;
+            }
             opentag.push_back(tmp + ">");
             closetag.push_front("</xs:element>");
 
@@ -307,6 +426,7 @@ void CUniSequenceDataType::PrintXMLSchema(CNcbiOstream& out,
                 opentag.push_back(tmp + ">");
                 closetag.push_front("</xs:" + xsdk + ">");
             }
+
             ITERATE ( list<string>, s, opentag ) {
                 PrintASNNewLine(out, indent++) << *s;
             }
