@@ -1,4 +1,4 @@
-/*  $Id: item.cpp 448671 2014-10-08 13:11:24Z gouriano $
+/*  $Id: item.cpp 547688 2017-10-03 13:21:50Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -38,33 +38,37 @@ BEGIN_NCBI_SCOPE
 CItemInfo::CItemInfo(const CMemberId& id,
                      TPointerOffsetType offset, TTypeInfo type)
     : m_Id(id), m_Index(kInvalidMember), m_Offset(offset), m_Type(type),
-      m_NonEmpty(false), m_Optional(false)
+      m_NonEmpty(false), m_Optional(false), m_Restrict(nullptr)
+
 {
 }
 
 CItemInfo::CItemInfo(const CMemberId& id,
                      TPointerOffsetType offset, const CTypeRef& type)
     : m_Id(id), m_Index(kInvalidMember), m_Offset(offset), m_Type(type),
-      m_NonEmpty(false), m_Optional(false)
+      m_NonEmpty(false), m_Optional(false), m_Restrict(nullptr)
 {
 }
 
 CItemInfo::CItemInfo(const char* id,
                      TPointerOffsetType offset, TTypeInfo type)
     : m_Id(id), m_Index(kInvalidMember), m_Offset(offset), m_Type(type),
-      m_NonEmpty(false), m_Optional(false)
+      m_NonEmpty(false), m_Optional(false), m_Restrict(nullptr)
 {
 }
 
 CItemInfo::CItemInfo(const char* id,
                      TPointerOffsetType offset, const CTypeRef& type)
     : m_Id(id), m_Index(kInvalidMember), m_Offset(offset), m_Type(type),
-      m_NonEmpty(false), m_Optional(false)
+      m_NonEmpty(false), m_Optional(false), m_Restrict(nullptr)
 {
 }
 
 CItemInfo::~CItemInfo(void)
 {
+    if (m_Restrict) {
+        delete m_Restrict;
+    }
 }
 
 CItemInfo* CItemInfo::SetNonEmpty(void)
