@@ -1,4 +1,4 @@
-/*  $Id: member.cpp 532692 2017-04-07 17:09:13Z gouriano $
+/*  $Id: member.cpp 547822 2017-10-04 15:45:27Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -721,13 +721,12 @@ void CMemberInfoFunctions::ReadSimpleMember(CObjectIStream& in,
 {
     _ASSERT(!memberInfo->CanBeDelayed());
     _ASSERT(!memberInfo->HaveSetFlag());
-    in.SetMemberDefault(0);
     if (memberInfo->Nillable()) {
         in.SetMemberNillable();
     }
     in.ReadObject(memberInfo->GetItemPtr(classPtr),
                   memberInfo->GetTypeInfo());
-    in.SetMemberDefault(0);
+    in.UnsetMemberNillable();
 }
 
 void CMemberInfoFunctions::ReadWithSetFlagMember(CObjectIStream& in,
@@ -740,6 +739,9 @@ void CMemberInfoFunctions::ReadWithSetFlagMember(CObjectIStream& in,
     try {
         in.ReadObject(memberInfo->GetItemPtr(classPtr),
                       memberInfo->GetTypeInfo());
+        if (in.GetVerifyData() == eSerialVerifyData_Yes) {
+            memberInfo->Validate(classPtr, in);
+        }
     }
     catch (CSerialException& e) {
         if (e.GetErrCode() == CSerialException::eNullValue) {
@@ -825,10 +827,13 @@ void CMemberInfoFunctions::ReadWithDefaultMemberX(CObjectIStream& in,
                 memberInfo->UpdateSetFlagMaybe(classPtr);
             }
         }
-        in.SetMemberDefault(0);
+        else if (in.GetVerifyData() == eSerialVerifyData_Yes) {
+            memberInfo->Validate(classPtr, in);
+        }
+        in.UnsetMemberSpecialCase();
     }
     catch (CSerialException& e) {
-        in.SetMemberDefault(0);
+        in.UnsetMemberSpecialCase();
         if (e.GetErrCode() == CSerialException::eNullValue) {
             if ( memberInfo->HaveSetFlag() && memberInfo->Nillable() ) {
                 memberInfo->UpdateSetFlagNo(classPtr);
@@ -1017,6 +1022,9 @@ void CMemberInfoFunctions::WriteWithDefaultMemberX(
             return;
         }
     }
+    if (out.GetVerifyData() == eSerialVerifyData_Yes) {
+        memberInfo->Validate(classPtr, out);
+    }
 do_write:
     out.WriteClassMember(memberInfo->GetId(), memberType, memberPtr);
 }
@@ -1081,6 +1089,9 @@ void CMemberInfoFunctions::WriteWithSetFlagMember(CObjectOStream& out,
         }
     }
 #endif
+    if (out.GetVerifyData() == eSerialVerifyData_Yes) {
+        memberInfo->Validate(classPtr, out);
+    }
 do_write:
     out.WriteClassMember(memberInfo->GetId(),
                          memberInfo->GetTypeInfo(),
@@ -1130,9 +1141,7 @@ void CMemberInfoFunctions::WriteLongMember(CObjectOStream& out,
 void CMemberInfoFunctions::CopySimpleMember(CObjectStreamCopier& copier,
                                             const CMemberInfo* memberInfo)
 {
-    copier.In().SetMemberDefault(0);
     copier.CopyObject(memberInfo->GetTypeInfo());
-    copier.In().SetMemberDefault(0);
 }
 
 void CMemberInfoFunctions::CopyWithDefaultMemberX(CObjectStreamCopier& copier,
@@ -1143,7 +1152,7 @@ void CMemberInfoFunctions::CopyWithDefaultMemberX(CObjectStreamCopier& copier,
         copier.In().SetMemberNillable();
     }
     copier.CopyObject(memberInfo->GetTypeInfo());
-    copier.In().SetMemberDefault(0);
+    copier.In().UnsetMemberSpecialCase();
 }
 
 void CMemberInfoFunctions::CopyMissingSimpleMember(CObjectStreamCopier& copier,
@@ -1162,9 +1171,7 @@ void CMemberInfoFunctions::CopyMissingOptionalMember(CObjectStreamCopier& /*copi
 void CMemberInfoFunctions::SkipSimpleMember(CObjectIStream& in,
                                             const CMemberInfo* memberInfo)
 {
-    in.SetMemberDefault(0);
     in.SkipObject(memberInfo->GetTypeInfo());
-    in.SetMemberDefault(0);
 }
 
 void CMemberInfoFunctions::SkipWithDefaultMemberX(CObjectIStream& in,
@@ -1175,7 +1182,7 @@ void CMemberInfoFunctions::SkipWithDefaultMemberX(CObjectIStream& in,
         in.SetMemberNillable();
     }
     in.SkipObject(memberInfo->GetTypeInfo());
-    in.SetMemberDefault(0);
+    in.UnsetMemberSpecialCase();
 }
 
 void CMemberInfoFunctions::SkipMissingSimpleMember(CObjectIStream& in,
