@@ -1,4 +1,4 @@
-/*  $Id: objistr.cpp 528362 2017-02-22 17:18:08Z gouriano $
+/*  $Id: objistr.cpp 551770 2017-11-22 15:12:22Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -81,6 +81,10 @@
 
 BEGIN_NCBI_SCOPE
 
+NCBI_PARAM_DECL(bool, SERIAL, READ_MMAPBYTESOURCE);
+NCBI_PARAM_DEF_EX(bool, SERIAL, READ_MMAPBYTESOURCE, false,
+                  eParam_NoThread, SERIAL_READ_MMAPBYTESOURCE);
+
 CRef<CByteSource> CObjectIStream::GetSource(ESerialDataFormat format,
                                             const string& fileName,
                                             TSerialOpenFlags openFlags)
@@ -114,8 +118,14 @@ CRef<CByteSource> CObjectIStream::GetSource(ESerialDataFormat format,
             return CRef<CByteSource>(new CFileByteSource(fileName, binary));
         }
         else {
-            // open file as stream
-            return CRef<CByteSource>(new CFStreamByteSource(fileName, binary));
+            static CSafeStatic<NCBI_PARAM_TYPE(SERIAL, READ_MMAPBYTESOURCE)> s_MmapSrc;
+            if (s_MmapSrc->Get()) {
+                // open file as file mapping
+                return CRef<CByteSource>(new CMMapByteSource(fileName));
+            } else {
+                // open file as stream
+                return CRef<CByteSource>(new CFStreamByteSource(fileName, binary));
+            }
         }
     }
 }
@@ -438,6 +448,7 @@ CObjectIStream::CObjectIStream(ESerialDataFormat format)
     : m_DiscardCurrObject(false),
       m_DataFormat(format),
       m_ParseDelayBuffers(eDelayBufferPolicyNotSet),
+      m_TypeAlias(nullptr),
       m_FixMethod(x_GetFixCharsMethodDefault()),
       m_VerifyData(x_GetVerifyDataDefault()),
       m_SkipUnknown(eSerialSkipUnknown_Default),
@@ -464,7 +475,7 @@ void CObjectIStream::ResetState(void)
 {
     CObjectStack::ResetState();
     m_DiscardCurrObject = false;
-    SetMemberDefault(0);
+    UnsetMemberSpecialCase();
 }
 
 void CObjectIStream::Open(CByteSourceReader& reader)
@@ -504,6 +515,7 @@ void CObjectIStream::Open(CNcbiIstream& inStream, EOwnership deleteInStream)
 void CObjectIStream::ResetLocalHooks(void)
 {
     CMutexGuard guard(GetTypeInfoMutex());
+    ResetPathHooks();
     m_ObjectHookKey.Clear();
     m_ClassMemberHookKey.Clear();
     m_ChoiceVariantHookKey.Clear();
@@ -1490,13 +1502,21 @@ void CObjectIStream::SkipChoiceSimple(const CChoiceTypeInfo* choiceType)
 void CObjectIStream::ReadAlias(const CAliasTypeInfo* aliasType,
                                TObjectPtr aliasPtr)
 {
+    if (aliasType->IsFullAlias()) {
+        m_TypeAlias = aliasType;
+    }
     ReadNamedType(aliasType, aliasType->GetPointedType(),
         aliasType->GetDataPtr(aliasPtr));
+    m_TypeAlias = nullptr;
 }
 
 void CObjectIStream::SkipAlias(const CAliasTypeInfo* aliasType)
 {
+    if (aliasType->IsFullAlias()) {
+        m_TypeAlias = aliasType;
+    }
     SkipNamedType(aliasType, aliasType->GetPointedType());
+    m_TypeAlias = nullptr;
 }
 
 ///////////////////////////////////////////////////////////////////////
