@@ -1,4 +1,4 @@
-/*  $Id: objistrasn.cpp 534119 2017-04-24 17:07:34Z gouriano $
+/*  $Id: objistrasn.cpp 564851 2018-06-04 13:13:16Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -528,41 +528,45 @@ void CObjectIStreamAsn::ReadAnyContentObject(CAnyContentObject& obj)
     obj.SetValue(CUtf8::AsUTF8(value,eEncoding_UTF8));
 }
 
-void CObjectIStreamAsn::SkipAnyContent(void)
-{
-    char to = GetChar(true);
-    if (to == '{') {
-        to = '}';
-    } else if (to == '\"') {
-    } else {
-        to = '\0';
-    }
-    for (char c = m_Input.PeekChar(); ; c = m_Input.PeekChar()) {
-        if (to != '\"') {
-            if (to != '}' && (c == '\n' || c == ',' || c == '}')) {
-                return;
-            } else if (c == '\"' || c == '{') {
-                SkipAnyContent();
-                continue;
-            }
+void CObjectIStreamAsn::SkipAnyContent(void) 
+{ 
+    bool ap = false;
+    char to = GetChar(true); 
+    if (to == '{') { 
+        to = '}'; 
+    } else if (to == '\"') { 
+    } else if (to == '\'') { 
+        ap = true;
+        to = '\0'; 
+    } else { 
+        to = '\0'; 
+    } 
+    for (char c = m_Input.PeekChar(); ; c = m_Input.PeekChar()) { 
+        if (!ap && to != '\"') { 
+            if (to != '}' && (c == '\n' || c == ',' || c == '}')) { 
+                return; 
+            } else if (c == '\"' || c == '{') { 
+                SkipAnyContent(); 
+                continue; 
+            } 
+        } 
+        if (c == to) { 
+            m_Input.SkipChar(); 
+            return; 
+        } 
+        if (c == '\"' || (c == '{' && to != '\"')) { 
+            SkipAnyContent(); 
+            continue; 
+        } 
+        if (c == '\'' && to != '\"') {
+            ap = !ap;
         }
-        if (c == to) {
-            m_Input.SkipChar();
-            if (c == '\n') {
-                SkipEndOfLine(c);
-            }
-            return;
-        }
-        if (c == '\"' || (c == '{' && to != '\"')) {
-            SkipAnyContent();
-            continue;
-        }
-        m_Input.SkipChar();
-        if (c == '\n') {
-            SkipEndOfLine(c);
-        }
-    }
-}
+        m_Input.SkipChar(); 
+        if (c == '\n') { 
+            SkipEndOfLine(c); 
+        } 
+    } 
+} 
 
 void CObjectIStreamAsn::SkipAnyContentObject(void)
 {
@@ -1312,7 +1316,10 @@ CObjectIStreamAsn::BeginClassMember(const CClassTypeInfo* classType,
     CTempString id = ReadMemberId(SkipWhiteSpace());
     TMemberIndex index = GetMemberIndex(classType, id, pos);
     if ( index == kInvalidMember ) {
-        if (CanSkipUnknownMembers()) {
+        if (m_TypeAlias && classType->GetMembers().GetItemInfo(pos)->GetId().HasNotag()) {
+            m_TypeAlias = nullptr;
+            return pos;
+        } else if (CanSkipUnknownMembers()) {
             SetFailFlags(fUnknownValue);
             SkipAnyContent();
             return BeginClassMember(classType, pos);
