@@ -1,4 +1,4 @@
-/*  $Id: objostr.cpp 507795 2016-07-21 17:24:14Z gouriano $
+/*  $Id: objostr.cpp 542167 2017-07-27 15:20:36Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -297,6 +297,7 @@ CObjectOStream::CObjectOStream(ESerialDataFormat format,
       m_WriteNamedIntegersByValue(false),
       m_FastWriteDouble(s_FastWriteDouble->Get()),
       m_EnforceWritingDefaults(false),
+      m_TypeAlias(nullptr),
       m_FixMethod(x_GetFixCharsMethodDefault()),
       m_VerifyData(x_GetVerifyDataDefault())
 {
@@ -332,6 +333,8 @@ void CObjectOStream::Close(void)
                 m_Objects->Clear();
             ClearStack();
             m_Fail = fNotOpen;
+            ResetState();
+            m_Output.Close();
         }
         catch (CException& exc) {
             if ( InGoodState() )
@@ -343,6 +346,7 @@ void CObjectOStream::Close(void)
 void CObjectOStream::ResetLocalHooks(void)
 {
     CMutexGuard guard(GetTypeInfoMutex());
+    ResetPathHooks();
     m_ObjectHookKey.Clear();
     m_ClassMemberHookKey.Clear();
     m_ChoiceVariantHookKey.Clear();
@@ -1044,6 +1048,7 @@ void CObjectOStream::CopyChoice(const CChoiceTypeInfo* choiceType,
             const CMemberInfo* memberInfo =
                 dynamic_cast<const CMemberInfo*>(
                     choiceType->GetVariants().GetItemInfo(index));
+            SetTopMemberId(memberInfo->GetId());
             BeginClassMember(memberInfo->GetId());
             memberInfo->CopyMember(copier);
             EndClassMember();
@@ -1082,15 +1087,23 @@ void CObjectOStream::CopyChoice(const CChoiceTypeInfo* choiceType,
 void CObjectOStream::WriteAlias(const CAliasTypeInfo* aliasType,
                                 TConstObjectPtr aliasPtr)
 {
+    if (aliasType->IsFullAlias()) {
+        m_TypeAlias = aliasType;
+    }
     WriteNamedType(aliasType, aliasType->GetPointedType(),
         aliasType->GetDataPtr(aliasPtr));
+    m_TypeAlias = nullptr;
 }
 
 void CObjectOStream::CopyAlias(const CAliasTypeInfo* aliasType,
                                 CObjectStreamCopier& copier)
 {
+    if (aliasType->IsFullAlias()) {
+        copier.In().m_TypeAlias = m_TypeAlias = aliasType;
+    }
     CopyNamedType(aliasType, aliasType->GetPointedType(),
         copier);
+    copier.In().m_TypeAlias = m_TypeAlias = nullptr;
 }
 
 void CObjectOStream::BeginBytes(const ByteBlock& )
