@@ -1,4 +1,4 @@
-/*  $Id: objostrjson.cpp 534119 2017-04-24 17:07:34Z gouriano $
+/*  $Id: objostrjson.cpp 545428 2017-09-06 16:50:07Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -503,7 +503,12 @@ void CObjectOStreamJson::BeginClassMember(const CMemberId& id)
             NextElement();
 #endif
         } else {
-            m_SkippedMemberId = id.GetName();
+            auto tn = [this]()->const string& {
+                const string& r(m_TypeAlias->GetName());
+                m_TypeAlias = nullptr;
+                return r;
+            };
+            m_SkippedMemberId = (m_TypeAlias && id.HasNotag()) ? tn() : id.GetName();
         }
         return;
     }
