@@ -1,4 +1,4 @@
-/*  $Id: objostrxml.cpp 534119 2017-04-24 17:07:34Z gouriano $
+/*  $Id: objostrxml.cpp 541701 2017-07-20 17:29:17Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -416,35 +416,34 @@ void CObjectOStreamXml::WriteEnum(const CEnumeratedTypeValues& values,
 {
     bool skipname = valueName.empty() ||
                   (m_WriteNamedIntegersByValue && values.IsInteger());
-    bool valueonly = GetRecentTypeInfo() && GetRecentTypeInfo()->GetDataSpec() == EDataSpec::eJSON;
-    if ( !m_SkipNextTag && !values.GetName().empty() ) {
-        // global enum
-        if (valueonly) {
-            if ( values.IsInteger() ) {
-                m_Output.PutInt4(value);
-            } else {
-                m_Output.PutString(valueName);
-            }
+    bool valueonly = m_StdXml;
+    if (valueonly) {
+        if ( values.IsInteger() ) {
+            m_Output.PutInt4(value);
         } else {
-            OpenTagStart();
+            m_Output.PutString(valueName);
+        }
+        return;
+    }
+    if ( !m_SkipNextTag && !values.GetName().empty() ) {
+        OpenTagStart();
+        m_Output.PutString(values.GetName());
+        if ( !skipname ) {
+            m_Output.PutString(" value=\"");
+            m_Output.PutString(valueName);
+            m_Output.PutChar('\"');
+        }
+        if ( values.IsInteger() ) {
+            OpenTagEnd();
+            m_Output.PutInt4(value);
+            CloseTagStart();
             m_Output.PutString(values.GetName());
-            if ( !skipname ) {
-                m_Output.PutString(" value=\"");
-                m_Output.PutString(valueName);
-                m_Output.PutChar('\"');
-            }
-            if ( values.IsInteger() ) {
-                OpenTagEnd();
-                m_Output.PutInt4(value);
-                CloseTagStart();
-                m_Output.PutString(values.GetName());
-                CloseTagEnd();
-            }
-            else {
-                _ASSERT(!valueName.empty());
-                SelfCloseTagEnd();
-                m_LastTagAction = eTagClose;
-            }
+            CloseTagEnd();
+        }
+        else {
+            _ASSERT(!valueName.empty());
+            SelfCloseTagEnd();
+            m_LastTagAction = eTagClose;
         }
     }
     else {
@@ -1211,6 +1210,11 @@ void CObjectOStreamXml::BeginArrayElement(TTypeInfo elementType)
             TopFrame().SetNotag();
             return;
         }
+        if (m_SkipNextTag && type.GetTypeFamily() == eTypeFamilyPrimitive) {
+            m_Output.PutChar(' ');
+            TopFrame().SetNotag();
+            return;
+        }
     }
     OpenStackTag(0);
 }
@@ -1291,16 +1295,17 @@ void CObjectOStreamXml::WriteContainerContents(const CContainerTypeInfo* cType,
 
 void CObjectOStreamXml::BeginNamedType(TTypeInfo namedTypeInfo)
 {
-    bool isclass = false;
-    if (m_SkipNextTag) {
+    CheckStdXml(namedTypeInfo);
+    if (m_SkipNextTag || namedTypeInfo->GetName().empty()) {
         TopFrame().SetNotag();
         m_SkipNextTag = false;
     } else {
-        const CClassTypeInfo* classType =
-            dynamic_cast<const CClassTypeInfo*>(namedTypeInfo);
-        if (classType) {
-            CheckStdXml(classType);
-            isclass = true;
+        TTypeInfo realtype = GetRealTypeInfo(namedTypeInfo);
+        if (realtype->GetTypeFamily() == eTypeFamilyPrimitive &&
+            GetStackDepth() > 2 && m_StdXml) {
+            TopFrame().SetNotag();
+            m_SkipNextTag = false;
+            return;
         }
         bool needNs = x_ProcessTypeNamespace(namedTypeInfo);
         OpenTag(namedTypeInfo);
@@ -1308,12 +1313,14 @@ void CObjectOStreamXml::BeginNamedType(TTypeInfo namedTypeInfo)
             x_WriteClassNamespace(namedTypeInfo);
         }
     }
-    if (!isclass) {
-        const CAliasTypeInfo* aliasType = 
-            dynamic_cast<const CAliasTypeInfo*>(namedTypeInfo);
-        if (aliasType) {
-            m_SkipNextTag = aliasType->IsFullAlias();
-        }
+    const CAliasTypeInfo* aliasType = 
+        dynamic_cast<const CAliasTypeInfo*>(namedTypeInfo);
+    if (aliasType) {
+        m_SkipNextTag = aliasType->IsFullAlias();
+    }
+    else if (m_StdXml) {
+        const CClassTypeInfo* classType = dynamic_cast<const CClassTypeInfo*>(namedTypeInfo);
+        m_SkipNextTag = (classType && classType->Implicit());
     }
 }
 
@@ -1357,20 +1364,28 @@ void CObjectOStreamXml::CopyNamedType(TTypeInfo namedTypeInfo,
 }
 #endif
 
-void CObjectOStreamXml::CheckStdXml(const CClassTypeInfoBase* classType)
+void CObjectOStreamXml::CheckStdXml(TTypeInfo typeinfo)
 {
-    TMemberIndex first = classType->GetItems().FirstIndex();
-    m_StdXml = classType->GetItems().GetItemInfo(first)->GetId().HaveNoPrefix();
+    if (typeinfo->GetCodeVersion() > 21600) {
+        m_StdXml = typeinfo->GetDataSpec() != EDataSpec::eASN;
+    } else {
+        const CClassTypeInfo* classType =
+            dynamic_cast<const CClassTypeInfo*>(typeinfo);
+        if (classType) {
+            TMemberIndex first = classType->GetItems().FirstIndex();
+            m_StdXml = classType->GetItems().GetItemInfo(first)->GetId().HaveNoPrefix();
+        }
+    }
 }
 
 void CObjectOStreamXml::BeginClass(const CClassTypeInfo* classInfo)
 {
+    CheckStdXml(classInfo);
     if (m_SkipNextTag) {
         TopFrame().SetNotag();
         m_SkipNextTag = false;
         return;
     }
-    CheckStdXml(classInfo);
     bool needNs = x_ProcessTypeNamespace(classInfo);
     OpenTagIfNamed(classInfo);
     if (needNs) {
@@ -1387,7 +1402,11 @@ void CObjectOStreamXml::EndClass(void)
     if (!m_Attlist && m_LastTagAction != eTagSelfClosed) {
         EolIfEmptyTag();
     }
-    CloseTagIfNamed(TopFrame().GetTypeInfo());
+    if (m_LastTagAction == eTagSelfClosed) {
+        m_LastTagAction = eTagClose;
+    } else {
+        CloseTagIfNamed(TopFrame().GetTypeInfo());
+    }
     x_EndTypeNamespace();
 }
 
@@ -1420,8 +1439,8 @@ void CObjectOStreamXml::BeginClassMember(TTypeInfo memberType,
                         elem_type->GetName() != mem_type->GetName());
                 }
             } else {
-                needTag = (type == eTypeFamilyPrimitive &&
-                    !id.HasNotag() && !id.HasAnyContent());
+                needTag = !id.HasNotag() && !id.HasAnyContent() && type != eTypeFamilyContainer;
+                m_SkipNextTag = type != eTypeFamilyPrimitive && type != eTypeFamilyContainer;
             }
             if (needTag) {
                 OpenStackTag(0);
@@ -1439,6 +1458,7 @@ void CObjectOStreamXml::BeginClassMember(TTypeInfo memberType,
 
 void CObjectOStreamXml::EndClassMember(void)
 {
+    m_SkipNextTag = false;
     if (TopFrame().GetNotag()) {
         TopFrame().SetNotag(false);
         m_Attlist = false;
@@ -1511,10 +1531,8 @@ void CObjectOStreamXml::WriteClassMemberSpecialCase(const CMemberId& memberId,
         return;
     }
     m_SpecialCaseWrite = how;
-    if (memberId.HasNotag()) {
+    if (memberId.HasNotag() || m_SkipNextTag) {
         x_SpecialCaseWrite();
-        TopFrame().SetNotag();
-        m_LastTagAction = eTagClose;
     } else {
         BEGIN_OBJECT_FRAME2(eFrameClassMember, memberId);
         OpenStackTag(0);
@@ -1529,12 +1547,12 @@ void CObjectOStreamXml::WriteClassMemberSpecialCase(const CMemberId& memberId,
 
 void CObjectOStreamXml::BeginChoice(const CChoiceTypeInfo* choiceType)
 {
+    CheckStdXml(choiceType);
     if (m_SkipNextTag) {
         TopFrame().SetNotag();
         m_SkipNextTag = false;
         return;
     }
-    CheckStdXml(choiceType);
     bool needNs = x_ProcessTypeNamespace(choiceType);
     OpenTagIfNamed(choiceType);
     if (needNs) {
@@ -1567,7 +1585,8 @@ void CObjectOStreamXml::BeginChoiceVariant(const CChoiceTypeInfo* choiceType,
                     elem_type->GetName() != var_type->GetName());
             }
         } else {
-            needTag = (type == eTypeFamilyPrimitive && !id.HasNotag());
+            needTag = !id.HasNotag() && !id.HasAnyContent() && type != eTypeFamilyContainer;
+            m_SkipNextTag = type != eTypeFamilyPrimitive && type != eTypeFamilyContainer;
         }
         if (needTag) {
             OpenStackTag(0);
@@ -1584,6 +1603,7 @@ void CObjectOStreamXml::BeginChoiceVariant(const CChoiceTypeInfo* choiceType,
 
 void CObjectOStreamXml::EndChoiceVariant(void)
 {
+    m_SkipNextTag = false;
     if (TopFrame().GetNotag()) {
         TopFrame().SetNotag(false);
     } else {
