@@ -1,4 +1,4 @@
-/*  $Id: pathhook.cpp 481847 2015-10-16 15:34:27Z gouriano $
+/*  $Id: pathhook.cpp 540952 2017-07-12 15:58:51Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -73,6 +73,9 @@ bool CPathHook::SetHook(CObjectStack* stk, const string& path, CObject* hook)
         insert(pair<CObjectStack* const, pair<string, CRef<CObject> > >(
             stk,pair<string, CRef<CObject> >(path,CRef<CObject>(hook))));
         state = !state;
+        if (stk) {
+            stk->RegisterPathHook(this);
+        }
     }
     bool wildcard = path.find(s_One)  != string::npos ||
                     path.find(s_Many) != string::npos;
