@@ -1,4 +1,4 @@
-/*  $Id: serial.cpp 403742 2013-06-18 15:26:09Z grichenk $
+/*  $Id: serial.cpp 529265 2017-03-02 16:00:11Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -73,9 +73,13 @@ void Read(CObjectIStream& in, TObjectPtr object, TTypeInfo type)
     in.Read(object, type);
 }
 
-void Serial_FilterSkip(CObjectIStream& in, CObjectTypeInfo& ctype)
+bool Serial_FilterSkip(CObjectIStream& in, const CObjectTypeInfo& ctype)
 {
-    in.Skip(ctype);
+    if (!in.EndOfData()) {
+        in.Skip(ctype);
+        return true;
+    }
+    return false;
 }
 
 DEFINE_STATIC_FAST_MUTEX(s_ModuleNameMutex);
