@@ -1,4 +1,4 @@
-/*  $Id: bytesrc.cpp 538188 2017-06-08 13:29:06Z ivanov $
+/*  $Id: bytesrc.cpp 551974 2017-11-27 19:36:41Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -266,6 +266,107 @@ CFStreamByteSource::~CFStreamByteSource(void)
     delete m_Stream;
 }
 
+CMMapByteSource::CMMapByteSource(const string& fileName, size_t num_blocks)
+    : m_FileMap(fileName), m_CBlocks(num_blocks)
+{
+}
+
+CMMapByteSource::~CMMapByteSource(void)
+{
+}
+
+CRef<CByteSourceReader> CMMapByteSource::Open(void)
+{
+    return CRef<CByteSourceReader>
+        (new CMMapByteSourceReader(this, &m_FileMap, m_CBlocks));
+}
+
+
+CMMapByteSourceReader::CMMapByteSourceReader(const CByteSource* source, CMemoryFileMap* fmap, size_t num_blocks)
+    : m_Source(source), m_Fmap(fmap), m_Ptr(nullptr),
+        m_UnitSize(GetVirtualMemoryAllocationGranularity()), m_DefaultSize(0), 
+        m_ChunkOffset(0),  m_CurOffset(0), m_NextOffset(0),
+        m_FileSize(fmap->GetFileSize())
+{
+    if (num_blocks == 0) {
+        num_blocks = 128;
+    } else if (num_blocks == 1) {
+        num_blocks = 2;
+    }
+    if (m_UnitSize == 0) {
+        m_UnitSize = 64 * 1024;
+    }
+    m_DefaultSize = num_blocks * m_UnitSize;
+}
+
+CMMapByteSourceReader::~CMMapByteSourceReader(void)
+{
+    if (m_Ptr) {
+        m_Fmap->Unmap(m_Ptr);
+    }
+}
+
+void CMMapByteSourceReader::x_GetNextChunkAt(size_t offset)
+{
+    if (m_Ptr) {
+        m_Fmap->Unmap(m_Ptr);
+        m_Ptr = nullptr;
+    }
+    if (offset < m_FileSize) {
+        m_CurOffset = offset;
+        m_ChunkOffset = (m_CurOffset / m_UnitSize) * m_UnitSize;
+        m_Ptr = (char*)m_Fmap->Map(m_ChunkOffset, min(m_FileSize - m_ChunkOffset, m_DefaultSize));
+        m_Fmap->MemMapAdvise(m_Ptr, CMemoryFileMap::eMMA_Sequential);
+        m_NextOffset = m_ChunkOffset + m_Fmap->GetSize(m_Ptr);
+    }
+}
+
+size_t CMMapByteSourceReader::GetNextPart(char** buffer, size_t copy_count)
+{
+    x_GetNextChunkAt(m_NextOffset - copy_count);
+    if (m_Ptr) {
+        *buffer = m_Ptr + (m_CurOffset - m_ChunkOffset);
+        size_t len = m_NextOffset - m_CurOffset;
+        m_CurOffset = m_NextOffset;
+        return len;
+    }
+    return 0;
+}
+
+size_t CMMapByteSourceReader::Read(char* buffer, size_t bufferLength)
+{
+    if (m_CurOffset == m_NextOffset) {
+        x_GetNextChunkAt(m_NextOffset);
+    }
+    size_t len = 0;
+    if (m_Ptr) {
+        len = min( m_NextOffset - m_CurOffset, bufferLength);
+        if (len != 0) {
+            memcpy(buffer, m_Ptr + (m_CurOffset - m_ChunkOffset), len);
+            m_CurOffset += len;
+        }
+    }
+    return len;
+}
+
+bool CMMapByteSourceReader::EndOfData(void) const
+{
+    return m_CurOffset >= m_FileSize;
+}
+
+bool CMMapByteSourceReader::Pushback(const char* data, size_t size)
+{
+    if (m_Ptr && (m_CurOffset >= m_ChunkOffset + size)) {
+        m_CurOffset -= size;
+        return true;
+    }
+    return false;
+}
+
+void CMMapByteSourceReader::Seekg(CNcbiStreampos pos)
+{
+    m_NextOffset = pos;
+}
 
 /////////////////////////////////////////////////////////////////////////////
 // CFileByteSource
