@@ -1,4 +1,4 @@
-/*  $Id: checksum.cpp 489095 2016-01-08 13:02:41Z ivanov $
+/*  $Id: checksum.cpp 573617 2018-10-30 14:36:09Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -23,9 +23,9 @@
  *
  * ===========================================================================
  *
- * Author:  Eugene Vasilchenko
+ * Author:  Eugene Vasilchenko, Vladimir Ivanov
  *
- * File Description:  Checksum (CRC32, Adler32 or MD5) calculation class
+ * File Description:  Checksum and hash calculation classes.
  *
  */
 
@@ -34,6 +34,16 @@
 #include <corelib/ncbifile.hpp>
 #include <util/checksum.hpp>
 
+// Use builtin versions of CityHash and FarmHash libraries,
+// that compiles as separate files to avoid name clashing.
+#include "checksum/cityhash/city.h"
+#include "checksum/farmhash/config.h" // need for farmhash.h
+#include "checksum/farmhash/farmhash.h"
+// And include MurmurHash directly
+#include "checksum/murmurhash/MurmurHash2.cxx"
+#include "checksum/murmurhash/MurmurHash3.cxx"
+
+
 #define USE_CRC32C_INTEL // try to use Intel CRC32C instructions
 
 #ifdef USE_CRC32C_INTEL
@@ -59,8 +69,10 @@
 # endif
 #endif
 
+
 BEGIN_NCBI_SCOPE
 
+
 static const size_t kCRC32Size = 256;
 typedef Uint4 TCRC32Table[kCRC32Size];
 
@@ -92,81 +104,86 @@ static const char sx_CharCount[] = "chars: ";
 #endif
 
 
-CChecksum::CChecksum(EMethod method)
+
+CChecksumBase::CChecksumBase(EMethodDef method)
     : m_Method(eNone)
 {
-    Reset(method);
+    x_Reset(method);
 }
 
 
-CChecksum::CChecksum(const CChecksum& cks)
-    : m_LineCount(cks.m_LineCount),
-      m_CharCount(cks.m_CharCount),
-      m_Method(cks.m_Method)
-{
-    if ( GetMethod() == eMD5 ) {
-        m_Checksum.m_MD5 = new CMD5(*cks.m_Checksum.m_MD5);
-    } else {
-        m_Checksum.m_CRC32 = cks.m_Checksum.m_CRC32;
-    }
-}
-
-
-CChecksum::~CChecksum()
+CChecksumBase::~CChecksumBase()
 {
     x_Free();
 }
 
 
-void CChecksum::x_Free(void)
+CChecksumBase::CChecksumBase(const CChecksumBase& other)
+    : m_Method(other.m_Method),
+      m_CharCount(other.m_CharCount)
 {
-    if ( GetMethod() == eMD5 ) {
-        delete m_Checksum.m_MD5;
-        m_Checksum.m_MD5 = NULL;
+    if ( m_Method == eMD5 ) {
+        m_Value.md5 = new CMD5(*other.m_Value.md5);
+    } else {
+        m_Value.v64 = other.m_Value.v64;
     }
 }
 
 
-CChecksum& CChecksum::operator= (const CChecksum& cks)
+CChecksumBase& CChecksumBase::operator= (const CChecksumBase& other)
 {
     x_Free();
 
-    m_LineCount = cks.m_LineCount;
-    m_CharCount = cks.m_CharCount;
-    m_Method    = cks.m_Method;
+    m_Method    = other.m_Method;
+    m_CharCount = other.m_CharCount;
 
-    if ( GetMethod() == eMD5 ) {
-        m_Checksum.m_MD5 = new CMD5(*cks.m_Checksum.m_MD5);
+    if ( m_Method == eMD5 ) {
+        m_Value.md5 = new CMD5(*other.m_Value.md5);
     } else {
-        m_Checksum.m_CRC32 = cks.m_Checksum.m_CRC32;
+        m_Value.v64 = other.m_Value.v64;
     }
     return *this;
 }
 
 
-void CChecksum::Reset(EMethod method)
+string CChecksumBase::GetResultHex(void) const
+{
+    switch (m_Method ) {
+    case eMD5:
+        return m_Value.md5->GetHexSum();
+    default:
+        if (GetBits() == 64) {
+            return NStr::NumericToString(GetResult64(), 0, 16);
+        }
+        if (GetBits() == 32) {
+            return NStr::NumericToString(GetResult32(), 0, 16);
+        }
+        _ASSERT(0);
+        return kEmptyStr;
+    }
+}
+
+
+void CChecksumBase::x_Reset(EMethodDef method)
 {
     x_Free();
 
-    m_LineCount = 0;
+    m_Method = method;
+    m_Value.v64 = 0;
     m_CharCount = 0;
-    if (method != eNone) {
-        m_Method = method;
-    }
 
-    switch ( GetMethod() ) {
+    switch ( method ) {
     case eCRC32:
     case eCRC32CKSUM:
-        m_Checksum.m_CRC32 = 0;
         s_InitTableCRC32Forward();
         break;
     case eCRC32ZIP:
     case eCRC32INSD:
-        m_Checksum.m_CRC32 = ~0;
+        m_Value.v32 = ~0;
         s_InitTableCRC32Reverse();
         break;
     case eCRC32C:
-        m_Checksum.m_CRC32 = ~0;
+        m_Value.v32 = ~0;
 #ifdef USE_CRC32C_INTEL
         if ( s_IsCRC32CIntelEnabled() ) {
             break;
@@ -174,63 +191,111 @@ void CChecksum::Reset(EMethod method)
 #endif
         s_InitTableCRC32CReverse();
         break;
+    case eAdler32:
+        m_Value.v32 = 1;
+        break;
     case eMD5:
-        m_Checksum.m_MD5 = new CMD5;
+        m_Value.md5 = new CMD5;
         break;
-    case eAdler32:
-        m_Checksum.m_CRC32 = 1;
+    case eCityHash32:
+    case eCityHash64:
+    case eFarmHash32:
+    case eFarmHash64:
+    case eMurmurHash2_32:
+    case eMurmurHash2_64:
+    case eMurmurHash3_32:
         break;
-    default:
-        break;
-    }
-}
-
-
-size_t CChecksum::GetChecksumSize(void) const
-{
-    switch ( GetMethod() ) {
-    case eMD5:  return 16;
-    case eNone: return 0;
-    default:    return 4;
-    }
-}
-
-
-Uint4 CChecksum::GetChecksum(void) const
-{
-    switch ( GetMethod() ) {
-    case eCRC32:
-        return m_Checksum.m_CRC32;
-    case eCRC32CKSUM:
-    {
-        // POSIX checksum includes length of data
-        char extra_bytes[8];
-        size_t extra_len = 0;
-        for ( size_t len = m_CharCount; len; len >>= 8 ) {
-            extra_bytes[extra_len++] = char(len & 0xff);
-        }
-        CChecksum extra(*this);
-        extra.AddChars(extra_bytes, extra_len);
-        return ~extra.m_Checksum.m_CRC32;
-    }
-    case eCRC32ZIP:
-        return ~m_Checksum.m_CRC32;
-    case eCRC32INSD:
-        return m_Checksum.m_CRC32;
-    case eCRC32C:
-        return ~m_Checksum.m_CRC32;
-    case eAdler32:
-        return m_Checksum.m_CRC32;
     default:
         _ASSERT(0);
-        return 0;
     }
 }
 
 
+CHash::CHash(EMethod method)
+    : CChecksumBase((EMethodDef)method)
+{
+}
+
+
+CHash::CHash(const CHash& other)
+    : CChecksumBase(other)
+{
+}
+
+
+CHash& CHash::operator= (const CHash& other)
+{
+    CChecksumBase::operator=(other);
+    return *this;
+}
+
+
+/// @sa CHash::SetSeed()
+Uint8 CChecksumBase::m_Seed = 0;
+
+
+void CHash::SetSeed(Uint8 seed)
+{
+    m_Seed = seed;
+}
+
+
+void CHash::Calculate(const CTempString str, EMethod method, Uint4& hash)
+{
+    CHash h(method);
+    h.Calculate(str);
+    hash = h.GetResult32();
+}
+
+
+void CHash::Calculate(const CTempString str, EMethod method, Uint8& hash)
+{
+    CHash h(method);
+    h.Calculate(str);
+    hash = h.GetResult64();
+}
+
+
+void CHash::Calculate(const char* str, size_t len, EMethod method, Uint4& hash)
+{
+    CHash h(method);
+    h.Calculate(str, len);
+    hash = h.GetResult32();
+}
+
+
+void CHash::Calculate(const char* str, size_t len, EMethod method, Uint8& hash)
+{
+    CHash h(method);
+    h.Calculate(str, len);
+    hash = h.GetResult64();
+}
+
+
+CChecksum::CChecksum(EMethod method)
+    : CChecksumBase((EMethodDef)method)
+{
+}
+
+
+CChecksum::CChecksum(const CChecksum& other)
+    : CChecksumBase(other),
+      m_LineCount(other.m_LineCount)
+{
+}
+
+
+CChecksum& CChecksum::operator= (const CChecksum& other)
+{
+    CChecksumBase::operator=(other);
+    m_LineCount = other.m_LineCount;
+    return *this;
+}
+
+
 CNcbiOstream& CChecksum::WriteChecksum(CNcbiOstream& out) const
 {
-    if (!Valid()   ||  !out.good()) {
+    if (!out.good()) {
         return out;
     }
     out << sx_Start
@@ -241,41 +306,26 @@ CNcbiOstream& CChecksum::WriteChecksum(CNcbiOstream& out) const
 }
 
 
-bool CChecksum::ValidChecksumLineLong(const char* line, size_t length) const
+bool CChecksum::ValidChecksumLineLong(const char* line, size_t len) const
 {
-    if ( !Valid() ) {
-        return false;
-    }
     CNcbiOstrstream buffer;
     WriteChecksum(buffer);
     string buffer_str = CNcbiOstrstreamToString(buffer);
-    if ( buffer_str.size() != length + 1 ) { // account for '\n'
+    if ( buffer_str.size() != len + 1 ) { // account for '\n'
         return false;
     }
-    return memcmp(line, buffer_str.data(), length) == 0;
-}
-
-
-string CChecksum::GetHexSum(void) const
-{
-    switch ( GetMethod() ) {
-    case eMD5:
-        return m_Checksum.m_MD5->GetHexSum();
-    case eNone:
-        return kEmptyStr;
-    default:
-        return NStr::NumericToString(GetChecksum(), 0, 16);
-    }
+    return memcmp(line, buffer_str.data(), len) == 0;
 }
 
 
 CNcbiOstream& CChecksum::WriteHexSum(CNcbiOstream& out) const
 {
     if ( GetMethod() == eMD5 ) {
-        out << m_Checksum.m_MD5->GetHexSum();
+        out << m_Value.md5->GetHexSum();
     } else {
         IOS_BASE::fmtflags flags = out.setf(IOS_BASE::hex, IOS_BASE::basefield);
-        out << setprecision(8) << GetChecksum();
+        out << setprecision(8);
+        out << GetChecksum();
         out.flags(flags);
     }
     return out;
@@ -285,26 +335,24 @@ CNcbiOstream& CChecksum::WriteHexSum(CNcbiOstream& out) const
 CNcbiOstream& CChecksum::WriteChecksumData(CNcbiOstream& out) const
 {
     switch ( GetMethod() ) {
+    case eMD5:
+        out << "MD5: ";
+        break;
+    case eAdler32:
+        out << "Adler32: ";
+        break;
     case eCRC32:
     case eCRC32ZIP:
     case eCRC32INSD:
     case eCRC32CKSUM:
     case eCRC32C:
         out << "CRC32: ";
-        WriteHexSum(out);
-        break;
-    case eMD5:
-        out << "MD5: ";
-        WriteHexSum(out);
-        break;
-    case eAdler32:
-        out << "Adler32: ";
-        WriteHexSum(out);
         break;
     default:
-        out << "none";
-        break;
+        _ASSERT(0);
+        return out;
     }
+    WriteHexSum(out);
     return out;
 }
 
@@ -333,8 +381,7 @@ void CChecksum::AddFile(const string& file_path)
     }
     catch (CFileException& e) {
         f.Close();
-        NCBI_RETHROW(e, CChecksumException, eFileIO,
-                     "Error add checksum for file: " + file_path);
+        NCBI_RETHROW(e, CChecksumException, eFileIO, "Error add checksum for file: " + file_path);
         throw;
     }
 }
@@ -346,8 +393,7 @@ void CChecksum::AddStream(CNcbiIstream& is)
         return;
     }
     if ( !is.good() ) {
-        NCBI_THROW(CChecksumException, eStreamIO,
-                   "Input stream is not good()");
+        NCBI_THROW(CChecksumException, eStreamIO, "Input stream is not good()");
         return;
     }
     CChecksum tmp(*this);
@@ -360,8 +406,7 @@ void CChecksum::AddStream(CNcbiIstream& is)
             tmp.AddChars(buf, n);
         } else {
             if (is.fail()  &&  !is.eof()) {
-                NCBI_THROW(CChecksumException, eStreamIO,
-                           "Error reading from input stream");
+                NCBI_THROW(CChecksumException, eStreamIO, "Error reading from input stream");
                 return;
             }
         }
@@ -392,8 +437,8 @@ CChecksum& ComputeFileChecksum_deprecated(const string& path, CChecksum& checksu
 // @deprecated
 CChecksum ComputeFileChecksum(const string& path, CChecksum::EMethod method)
 {
-    CChecksum cks(method);
-    return ComputeFileChecksum_deprecated(path, cks);
+    CChecksum checksum(method);
+    return ComputeFileChecksum_deprecated(path, checksum);
 }
 
 // @deprecated
@@ -405,12 +450,12 @@ CChecksum& ComputeFileChecksum(const string& path, CChecksum& checksum)
 // @deprecated
 Uint4 ComputeFileCRC32(const string& path)
 {
-    CChecksum cks(CChecksum::eCRC32);
-    return ComputeFileChecksum_deprecated(path, cks).GetChecksum();
+    CChecksum checksum(CChecksum::eCRC32);
+    return ComputeFileChecksum_deprecated(path, checksum).GetChecksum();
 }
 
 
-void CChecksum::InitTables(void)
+void CChecksumBase::InitTables(void)
 {
     s_InitTableCRC32Forward();
     s_InitTableCRC32Reverse();
@@ -1097,7 +1142,7 @@ Uint4 s_UpdateAdler32(Uint4 sum, const char* data, size_t len)
 }
 
 
-void CChecksum::PrintTables(CNcbiOstream& out)
+void CChecksumBase::PrintTables(CNcbiOstream& out)
 {
     InitTables();
     s_PrintTable(out, "s_CRC32TableForward", s_CRC32TableForward);
@@ -1106,42 +1151,163 @@ void CChecksum::PrintTables(CNcbiOstream& out)
 }
 
 
-void CChecksum::x_Update(const char* str, size_t count)
+void CChecksumBase::x_Update(const char* str, size_t count)
 {
-    switch ( GetMethod() ) {
+    switch ( m_Method ) {
     case eCRC32:
     case eCRC32CKSUM:
-        m_Checksum.m_CRC32 =
-            s_UpdateCRC32Forward(m_Checksum.m_CRC32, str, count, s_CRC32TableForward);
+        m_Value.v32 = s_UpdateCRC32Forward(m_Value.v32, str, count, s_CRC32TableForward);
         break;
     case eCRC32ZIP:
     case eCRC32INSD:
-        m_Checksum.m_CRC32 =
-            s_UpdateCRC32Reverse(m_Checksum.m_CRC32, str, count, s_CRC32TableReverse);
+        m_Value.v32 = s_UpdateCRC32Reverse(m_Value.v32, str, count, s_CRC32TableReverse);
         break;
     case eCRC32C:
 #ifdef USE_CRC32C_INTEL
         if ( s_IsCRC32CIntelEnabled() ) {
-            m_Checksum.m_CRC32 =
-                s_UpdateCRC32CIntel(m_Checksum.m_CRC32, str, count);
+            m_Value.v32 = s_UpdateCRC32CIntel(m_Value.v32, str, count);
             break;
         }
 #endif
-        m_Checksum.m_CRC32 =
-            s_UpdateCRC32Reverse(m_Checksum.m_CRC32, str, count, s_CRC32CTableReverse);
+        m_Value.v32 = s_UpdateCRC32Reverse(m_Value.v32, str, count, s_CRC32CTableReverse);
         break;
     case eAdler32:
-        m_Checksum.m_CRC32 = s_UpdateAdler32(m_Checksum.m_CRC32, str, count);
+        m_Value.v32 = s_UpdateAdler32(m_Value.v32, str, count);
         break;
     case eMD5:
-        m_Checksum.m_MD5->Update(str, count);
+        m_Value.md5->Update(str, count);
+        break;
+    case eCityHash32:
+        _ASSERT(!m_CharCount);
+        m_Value.v32 = CityHash32(str, count);
+        break;
+    case eCityHash64:
+        _ASSERT(!m_CharCount);
+        m_Value.v64 = CityHash64(str, count);
+        break;
+    case eFarmHash32:
+        _ASSERT(!m_CharCount);
+        m_Value.v32 = farmhash::Hash32(str, count);
+        break;
+    case eFarmHash64:
+        _ASSERT(!m_CharCount);
+        m_Value.v64 = farmhash::Hash64(str, count);
+        break;
+    case eMurmurHash2_32:
+        {{
+            _ASSERT(!m_CharCount);
+            int n = count > kMax_Int ? kMax_Int : (int)count;
+            m_Value.v32 = MurmurHash2(str, n, (uint32_t)m_Seed);
+        }}
+        break;
+    case eMurmurHash2_64:
+        {{
+            _ASSERT(!m_CharCount);
+            int n = count > kMax_Int ? kMax_Int : (int)count;
+            m_Value.v64 = MurmurHash64A(str, n, m_Seed);
+        }}
+        break;
+    case eMurmurHash3_32:
+        {{
+            _ASSERT(!m_CharCount);
+            int n = count > kMax_Int ? kMax_Int : (int)count;
+            MurmurHash3_x86_32(str, n, (uint32_t)m_Seed, &m_Value.v32);
+        }}
         break;
     default:
+        _ASSERT(0);
         break;
     }
 }
 
 
+//////////////////////////////////////////////////////////////////////////////
+//
+// NHash
+//
+
+Uint4 NHash::CityHash32(const CTempString str)
+{
+    return ::CityHash32(str.data(), str.length());
+}
+
+Uint4 NHash::CityHash32(const char* str, size_t len)
+{
+    return ::CityHash32(str, len);
+}
+
+Uint8 NHash::CityHash64(const CTempString str)
+{
+    return ::CityHash64(str.data(), str.length());
+}
+
+Uint8 NHash::CityHash64(const char* str, size_t len)
+{
+    return ::CityHash64(str, len);
+}
+
+Uint4 NHash::FarmHash32(const CTempString str)
+{
+    return farmhash::Hash32(str.data(), str.length());
+ 
+}
+
+Uint4 NHash::FarmHash32(const char* str, size_t len)
+{
+    return farmhash::Hash32(str, len);
+}
+
+Uint8 NHash::FarmHash64(const CTempString str)
+{
+    return farmhash::Hash64(str.data(), str.length());
+}
+
+Uint8 NHash::FarmHash64(const char* str, size_t len)
+{
+    return farmhash::Hash64(str, len);
+}
+
+Uint4 NHash::MurmurHash2(const CTempString str, Uint4 seed)
+{
+    _ASSERT(str.length() <= kMax_Int);
+    return ::MurmurHash2(str.data(), (int)str.length(), seed);
+}
+
+Uint4 NHash::MurmurHash2(const char* str, size_t len, Uint4 seed)
+{
+    _ASSERT(len <= kMax_Int);
+    return ::MurmurHash2(str, (int)len, seed);
+}
+
+Uint8 NHash::MurmurHash64A(const CTempString str, Uint8 seed)
+{
+    _ASSERT(str.length() <= kMax_Int);
+    return ::MurmurHash64A(str.data(), (int)str.length(), seed);
+}
+
+Uint8 NHash::MurmurHash64A(const char* str, size_t len, Uint8 seed)
+{
+    _ASSERT(len <= kMax_Int);
+    return ::MurmurHash64A(str, (int)len, seed);
+}
+
+Uint4 NHash::MurmurHash3_x86_32(const CTempString str, Uint4 seed)
+{
+    _ASSERT(str.length() <= kMax_Int);
+    Uint4 result;
+    ::MurmurHash3_x86_32(str.data(), (int)str.length(), seed, &result);
+    return result;
+}
+
+Uint4 NHash::MurmurHash3_x86_32(const char* str, size_t len, Uint4 seed)
+{
+    _ASSERT(len <= kMax_Int);
+    Uint4 result;
+    ::MurmurHash3_x86_32(str, (int)len, seed, &result);
+    return result;
+}
+
+
 
 //////////////////////////////////////////////////////////////////////////////
 //
