@@ -1,4 +1,4 @@
-/* $Id: archive_zip.cpp 430126 2014-03-24 12:25:47Z ivanov $
+/* $Id: archive_zip.cpp 500279 2016-05-03 17:12:04Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -220,6 +220,10 @@ size_t CArchiveZip::GetNumEntries(void)
     _ASSERT(m_Mode == eRead);
     ZIP_CHECK;
     mz_uint n = mz_zip_reader_get_num_files(ZIP_HANDLE);
+    // TODO: remove with zip64 support added to miniz
+    if (n == 65535) {
+        ZIP_THROW(eUnsupported, "Too many files in the archive");
+    }
     return n;
 }
 
@@ -259,7 +263,7 @@ void CArchiveZip::GetEntryInfo(size_t index, CArchiveEntryInfo* info)
     // is very OS- and creation software dependent. 
     // Try to analyze some common cases for Unix-type attributes:
 
-    char ver = fs.m_version_made_by >> 8;
+    char ver = (char)(fs.m_version_made_by >> 8);
     mode_t mode = (fs.m_external_attr >> 16) & 0xFFFF;
 
     switch (ver) {
@@ -447,6 +451,14 @@ void CArchiveZip::TestEntry(const CArchiveEntryInfo& info)
 void CArchiveZip::AddEntryFromFileSystem(const CArchiveEntryInfo& info, 
                                          const string& src_path, ELevel level)
 {
+    // TODO: remove with zip64 support added to miniz
+    {{
+        mz_uint n = mz_zip_reader_get_num_files(ZIP_HANDLE);
+        if (n >= 65534) {
+            ZIP_THROW(eAppend, "Too many files in the archive");
+        }
+    }}
+
     const string& comment = info.m_Comment;
     mz_uint16 comment_size = (mz_uint16)comment.size();
     mz_bool status;
@@ -471,6 +483,14 @@ void CArchiveZip::AddEntryFromFileSystem(const CArchiveEntryInfo& info,
 void CArchiveZip::AddEntryFromMemory(const CArchiveEntryInfo& info,
                                      void* buf, size_t size, ELevel level)
 {
+    // TODO: remove with zip64 support added to miniz
+    {{
+        mz_uint n = mz_zip_reader_get_num_files(ZIP_HANDLE);
+        if (n >= 65534) {
+            ZIP_THROW(eAppend, "Too many files in the archive");
+        }
+    }}
+
     const string& comment = info.m_Comment;
     mz_uint16 comment_size = (mz_uint16)comment.size();
     mz_bool status;
