@@ -1,4 +1,4 @@
-/*  $Id: smalldns.cpp 489095 2016-01-08 13:02:41Z ivanov $
+/*  $Id: smalldns.cpp 548806 2017-10-18 12:19:30Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -91,7 +91,7 @@ bool CSmallDNS::IsValidIP(const string& ip)
 {
     list<string> dig;
     
-    NStr::Split(ip, ".", dig, NStr::fSplit_NoMergeDelims);
+    NStr::Split(ip, ".", dig);
     if (dig.size() != 4) {
         return false;
     }
