@@ -1,4 +1,4 @@
-/*  $Id: thread_pool.cpp 497641 2016-04-08 15:17:34Z ucko $
+/*  $Id: thread_pool.cpp 535328 2017-05-08 15:21:08Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -1662,7 +1662,7 @@ CThreadPool_Impl::AddTask(CThreadPool_Task* task, const CTimeSpan* timeout)
     }
 
     CThreadPool_Guard guard(this, false);
-    auto_ptr<CTimeSpan> adjusted_timeout;
+    unique_ptr<CTimeSpan> adjusted_timeout;
 
     if (!m_IsQueueAllowed) {
         guard.Guard();
