@@ -1,4 +1,4 @@
-/*  $Id: transmissionrw.cpp 477239 2015-08-27 15:35:49Z sadyrovr $
+/*  $Id: transmissionrw.cpp 538909 2017-06-15 15:11:18Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -40,6 +40,7 @@ BEGIN_NCBI_SCOPE
 
 static const Uint4 sStartWord = 0x01020304;
 static const Uint4 sEndPacket = 0xFFFFFFFF;
+const size_t kLengthSize = sizeof(Uint4);
 
 CTransmissionWriter::CTransmissionWriter(IWriter* wrt, 
                                          EOwnership own_writer,
@@ -52,8 +53,8 @@ CTransmissionWriter::CTransmissionWriter(IWriter* wrt,
     _ASSERT(wrt);
 
     size_t written;
-    ERW_Result res = m_Wrt->Write(&sStartWord, sizeof(sStartWord), &written);
-    if (res != eRW_Success || written != sizeof(sStartWord)) {
+    ERW_Result res = m_Wrt->Write(&sStartWord, kLengthSize, &written);
+    if (res != eRW_Success || written != kLengthSize) {
         NCBI_THROW(CIOException, eWrite,  "Cannot write the byte order");
     }
 }
@@ -88,10 +89,10 @@ ERW_Result CTransmissionWriter::Write(const void* buf,
     if (!m_PacketBytesToWrite) {
         Uint4 cnt = (Uint4) count;
         size_t written = 0;
-        ERW_Result res = m_Wrt->Write(&cnt, sizeof(cnt), &written);
+        ERW_Result res = m_Wrt->Write(&cnt, kLengthSize, &written);
         if (res != eRW_Success) 
             return res;
-        if (written != sizeof(cnt))
+        if (written != kLengthSize)
             return eRW_Error;
         m_PacketBytesToWrite = cnt;
     }
@@ -117,7 +118,7 @@ ERW_Result CTransmissionWriter::Close(void)
 
     m_SendEof = eDontSendEofPacket;
 
-    return m_Wrt->Write(&sEndPacket, sizeof(sEndPacket));
+    return m_Wrt->Write(&sEndPacket, kLengthSize);
 }
 
 CTransmissionWriter::~CTransmissionWriter()
@@ -128,6 +129,40 @@ CTransmissionWriter::~CTransmissionWriter()
         delete m_Wrt;
 }
 
+template <typename TType>
+inline const TType* CTransmissionReader::SReadData::Data() const
+{
+    return reinterpret_cast<const TType*>(data() + m_Start);
+}
+
+inline size_t CTransmissionReader::SReadData::DataSize() const
+{
+    return m_End - m_Start;
+}
+
+inline char* CTransmissionReader::SReadData::Space()
+{
+    return data() + m_End;
+}
+
+inline size_t CTransmissionReader::SReadData::SpaceSize() const
+{
+    return size() - m_End;
+}
+
+inline void CTransmissionReader::SReadData::Add(size_t count)
+{
+    m_End += count;
+    _ASSERT(m_End <= size());
+}
+
+inline void CTransmissionReader::SReadData::Remove(size_t count)
+{
+    m_Start += count;
+    if (DataSize() == 0) m_Start = m_End = 0;
+    _ASSERT(m_Start < size());
+}
+
 CTransmissionReader::CTransmissionReader(IReader* rdr, EOwnership own_reader)
     : m_Rdr(rdr),
       m_OwnRdr(own_reader),
@@ -135,7 +170,6 @@ CTransmissionReader::CTransmissionReader(IReader* rdr, EOwnership own_reader)
       m_ByteSwap(false),
       m_StartRead(false)
 {
-    return;
 }
 
 
@@ -147,43 +181,63 @@ CTransmissionReader::~CTransmissionReader()
 }
 
 
+ERW_Result CTransmissionReader::ReadData()
+{
+    size_t read;
+
+    auto res = m_Rdr->Read(m_ReadData.Space(), m_ReadData.SpaceSize(), &read);
+    if (res == eRW_Success) m_ReadData.Add(read);
+
+    return res;
+}
+
+
 ERW_Result CTransmissionReader::Read(void*    buf,
                                      size_t   count,
                                      size_t*  bytes_read)
 {
-    ERW_Result res;
+    const size_t kMinReadSize = 32 * 1024;
+
     CIOBytesCountGuard read(bytes_read);
 
     if (!m_StartRead) {
-        res = x_ReadStart();
-        if (res != eRW_Success) {
-            return res;
-        }
+        auto res = x_ReadStart();
+        if (res != eRW_Success) return res;
     }
 
     // read packet header
     while (m_PacketBytesToRead == 0) {
-        Uint4 cnt;
-        res = x_ReadRepeated(&cnt, sizeof(cnt));
-        if (res != eRW_Success) { 
-            return res;
-        }
-        if (m_ByteSwap) {
-            m_PacketBytesToRead = CByteSwap::GetInt4((unsigned char*)&cnt);
-        } else {
-            m_PacketBytesToRead = cnt;
-        }
+        auto res = ReadLength(m_PacketBytesToRead);
+        if (res != eRW_Success) return res;
     }
 
-    if (m_PacketBytesToRead == sEndPacket) 
-        return  eRW_Eof;
+    if (m_PacketBytesToRead == sEndPacket) return eRW_Eof;
 
-    size_t to_read = min(count, m_PacketBytesToRead);
+    size_t to_read = count < m_PacketBytesToRead ? count : m_PacketBytesToRead;
+    size_t already_read = m_ReadData.DataSize();
 
-    res = m_Rdr->Read(buf, to_read, &read.count);
-    m_PacketBytesToRead -= read.count;
+    if (!already_read) {
+        // If chunk to read is big enough, read directly in the buf
+        if (to_read >= kMinReadSize) {
+            auto res = m_Rdr->Read(buf, to_read, &read.count);
+            if (res == eRW_Success) m_PacketBytesToRead -= static_cast<Uint4>(read.count);
+            return res;
+        }
 
-    return res;
+        auto res = ReadData();
+        if (res != eRW_Success) return res;
+
+        already_read = m_ReadData.DataSize();
+    }
+
+    if (already_read) {
+        read.count = min(to_read, already_read);
+        copy_n(m_ReadData.Data(), read.count, static_cast<char*>(buf));
+        m_ReadData.Remove(read.count);
+    }
+
+    m_PacketBytesToRead -= static_cast<Uint4>(read.count);
+    return eRW_Success;
 }
 
 
@@ -199,41 +253,38 @@ ERW_Result CTransmissionReader::x_ReadStart()
 
     m_StartRead = true;
 
-    ERW_Result res;
-    unsigned start_word_coming;
+    Uint4 start_word_coming;
+
+    auto res = ReadLength(start_word_coming);
+    if (res != eRW_Success) return res;
 
-    res = x_ReadRepeated(&start_word_coming, sizeof(start_word_coming));
-    if (res != eRW_Success) {
-        return res;
-    }
     m_ByteSwap = (start_word_coming != sStartWord);
 
-    //    _ASSERT(start_word_coming == 0x01020304 || 
-    //            start_word_coming == 0x04030201);
-
     if (start_word_coming != sStartWord &&
         start_word_coming != 0x04030201)
         NCBI_THROW(CUtilException, eWrongData,
                    "Cannot determine the byte order. Got: " 
                    + NStr::UIntToString(start_word_coming, 0, 16));
 
-    return res;
+    return eRW_Success;
 }
 
 
-ERW_Result CTransmissionReader::x_ReadRepeated(void* buf, size_t count)
+ERW_Result CTransmissionReader::ReadLength(Uint4& length)
 {
-    ERW_Result res = eRW_Success;
-    char* ptr = (char*)buf;
-    size_t read;
-    
-    for( ; count > 0; count -= read, ptr += read) {
-        res = m_Rdr->Read(ptr, count, &read);
-        if (res != eRW_Success) {
-            return res;
-       }
+    while (m_ReadData.DataSize() < kLengthSize) {
+        auto res = ReadData();
+        if (res != eRW_Success) return res;
     }
-    return res;
+
+    if (m_ByteSwap) {
+        length = CByteSwap::GetInt4(m_ReadData.Data<unsigned char>());
+    } else {
+        length = *m_ReadData.Data<Uint4>();
+    }
+
+    m_ReadData.Remove(kLengthSize);
+    return eRW_Success;
 }
 
 
