@@ -1,4 +1,4 @@
-/*  $Id: unicode.cpp 459667 2015-02-20 16:49:22Z gouriano $
+/*  $Id: unicode.cpp 565977 2018-06-22 13:20:14Z gouriano $
  * ==========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -358,7 +358,7 @@ UnicodeToAscii(TUnicode character, const TUnicodeTable* table,
 TUnicode UTF8ToUnicode( const char* theUTF )
 {
     const char *p = theUTF;
-    char counter = *p++;
+    Int1 counter = *p++;
 
     if ( ((*theUTF) & 0xC0) != 0xC0 ) {
         TUnicode RC = 0;
@@ -368,7 +368,7 @@ TUnicode UTF8ToUnicode( const char* theUTF )
 
     TUnicode acc = counter & 037;
 
-    while ((counter <<= 1) < 0) {
+    while ((counter = Int1(counter << 1)) < 0) {
         unsigned char c = *p++;
         if ((c & ~077) != 0200) { // Broken UTF-8 chain
             return 0;
@@ -383,7 +383,7 @@ TUnicode UTF8ToUnicode( const char* theUTF )
 size_t UTF8ToUnicode( const char* theUTF, TUnicode* theUnicode )
 {
     const char *p = theUTF;
-    char counter = *p++;
+    Int1 counter = *p++;
 
     if ( (unsigned char)theUTF[0] < 0x80 ) {
         // This is one character UTF8. I.e. regular character.
@@ -391,8 +391,8 @@ size_t UTF8ToUnicode( const char* theUTF, TUnicode* theUnicode )
         return 1;
     }
 
-    if ( ((*theUTF) & 0xC0) != 0xC0 ) {
-        // This is not a unicode
+    if ( ((*theUTF) & 0xC0) != 0xC0  || ((*theUTF) & 0xFE) == 0xC0) {
+        // This is not UTF8
         return 0;
     }
 
@@ -401,7 +401,7 @@ size_t UTF8ToUnicode( const char* theUTF, TUnicode* theUnicode )
         acc = counter & 07;
     }
 
-    while ((counter <<= 1) < 0) {
+    while ((counter = Int1(counter << 1)) < 0) {
         unsigned char c = *p++;
         if ((c & ~077) != 0200) { // Broken UTF-8 chain
             return 0;
